#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <signal.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/resource.h>
#include <time.h>
#include <net/if.h>
#include <upnp/upnp.h>
#include "globals.h"
#include "config.h"
#include "gatedevice.h"
#include "util.h"
#include "pmlist.h"
#ifdef UPNP_LISTEN_API
#include <pthread.h>
#endif


// Global variables
#define TEAMF1_CHANGES
int adPeriod ;
int adTTL;
struct GLOBALS g_vars;
#ifdef UPNP_LISTEN_API
extern int api_thread_exit;
extern void *api_start_server(void *);
#endif

#ifdef TEAMF1_CHANGES
#define XML_FILE "/var/linuxigd/gatedesc.xml"
#define presentationLen (50)
#endif /* TEAMF1_CHANGES  */

void advertise ()
{
		
		int ret;
		if ( (ret = UpnpSendAdvertisement(deviceHandle, 1800 , adTTL ) != UPNP_E_SUCCESS ))
		{
			syslog(LOG_ERR, "Error Sending Advertisements.  Exiting ...");
			UpnpFinish();
			exit(1);
		}
		trace(2, "Advertisements Sent.  Listening for requests ... ");
		alarm(adPeriod);

}
#ifdef TEAMF1_CHANGES

/*******************************************************************************
 * 
 * SetXMLFile 
 *      
 * This routine changes /var/linuxigd/gatedesc.xml file whenever lan ip
 * changes, which is required in upnp
 *  
 */

void SetXMLFile
(
  char *IPAddr   
  )
  {
    char *buf,str[presentationLen],*tmp;
    int bytes_count=0;
    FILE * fp;
    memset(str,'\0',sizeof(str));
    
    fp = fopen(XML_FILE,"r+");
    if(fp==NULL){
        syslog(LOG_ERR,"[%s:%d]: Failed to open /var/gatedesc.xml file",__func__,__LINE__);
        return;
    }
    fseek(fp,0,SEEK_END);
    bytes_count=ftell(fp); /* Getting the file size */
    buf=(char*)malloc(bytes_count*sizeof(char)+10);   
    if(buf==NULL) {
        syslog(LOG_ERR,"[%s:%d]: Failed to allocate memory",__func__,__LINE__);
        return;
    }

    memset(buf,'\0',sizeof(buf));
    
    fseek(fp,0,SEEK_SET);
    
    bytes_count = fread(buf,sizeof(char),bytes_count-1,fp);
    tmp=strstr(buf,"presentationURL>");
    if(tmp==NULL){
        syslog(LOG_ERR,"[%s:%d]: Failed to update /var/gatedesc.xml file",__func__,__LINE__);
        return;
    } else {
        sprintf(str,"presentationURL>http://%-16s<",IPAddr);
        tmp=memccpy(tmp,str,'<',sizeof(str)-1);
    }
    fseek(fp,0,SEEK_SET);  
    fwrite(buf,sizeof(char),bytes_count,fp);
    fclose(fp);
    return;
  }
#endif /* TEAMF1_CHANGES */

int main (int argc, char** argv)
{
	char descDocUrl[7+15+1+5+1+sizeof(g_vars.descDocName)+1]; // http://ipaddr:port/docName<null>
	char intIpAddress[16];     // Server internal ip address
	sigset_t sigsToCatch;
	int ret, signum, arg = 1, foreground = 0;
#ifdef UPNP_LISTEN_API
	mode_t oldmask;
	pthread_t api_thr;
#endif

	if (argc < 5 || argc > 6) {
	  printf("Usage: upnpd [-f] <external ifname> <internal ifname> <advertising frequency ><advertising TTL>\n");
	  printf("  -f\tdon't daemonize\n");
	  printf("Example: upnpd ppp0 eth0 600 4\n");
	  exit(0);
	}
	
	
	parseConfigFile(&g_vars);

	// check for '-f' option
	if (strcmp(argv[arg], "-f") == 0) {
		foreground = 1;
		arg++;
	}

	// Save interface names for later use
	strncpy(g_vars.extInterfaceName, argv[arg++], IFNAMSIZ);
	strncpy(g_vars.intInterfaceName, argv[arg++], IFNAMSIZ);

	adPeriod = strtol(argv[arg++],NULL,10);
	adTTL = strtol(argv[arg],NULL,10);
	// Get the internal ip address to start the daemon on
	if (GetIpAddressStr(intIpAddress, g_vars.intInterfaceName) == 0) {
		fprintf(stderr, "Invalid internal interface name '%s'\n", g_vars.intInterfaceName);
		exit(EXIT_FAILURE);
	}
#ifdef TEAMF1_CHANGES
        SetXMLFile(intIpAddress);
#endif /* TEAMF1_CHANGES */

	if (!foreground) {
		struct rlimit resourceLimit = { 0, 0 };
		pid_t pid, sid;
		unsigned int i;

		// Put igd in the background as a daemon process.
		pid = fork();
		if (pid < 0)
		{
			perror("Error forking a new process.");
			exit(EXIT_FAILURE);
		}
		if (pid > 0)
			exit(EXIT_SUCCESS);

		// become session leader
		if ((sid = setsid()) < 0)
		{
			perror("Error running setsid");
			exit(EXIT_FAILURE);
		}

		// close all file handles
		resourceLimit.rlim_max = 0;
		ret = getrlimit(RLIMIT_NOFILE, &resourceLimit);
		if (ret == -1) /* shouldn't happen */
		{
		    perror("error in getrlimit()");
		    exit(EXIT_FAILURE);
		}
		if (0 == resourceLimit.rlim_max)
		{
		    fprintf(stderr, "Max number of open file descriptors is 0!!\n");
		    exit(EXIT_FAILURE);
		}	
		for (i = 0; i < resourceLimit.rlim_max; i++)
		    close(i);
	
		// fork again so child can never acquire a controlling terminal
		pid = fork();
		if (pid < 0)
		{
			perror("Error forking a new process.");
			exit(EXIT_FAILURE);
		}
		if (pid > 0)
			exit(EXIT_SUCCESS);
	
		if ((chdir("/")) < 0)
		{
			perror("Error setting root directory");
			exit(EXIT_FAILURE);
		}
	}

#ifdef UPNP_LISTEN_API
	oldmask = 
#endif	
	umask(0);

// End Daemon initialization

	openlog("upnpd", LOG_CONS | LOG_NDELAY | LOG_PID | (foreground ? LOG_PERROR : 0), LOG_LOCAL6);

	// Initialize UPnP SDK on the internal Interface
	trace(3, "Initializing UPnP SDK ... ");
	if ( (ret = UpnpInit(intIpAddress,0) ) != UPNP_E_SUCCESS)
	{
		syslog (LOG_ERR, "Error Initializing UPnP SDK on IP %s ",intIpAddress);
		syslog (LOG_ERR, "  UpnpInit returned %d", ret);
		UpnpFinish();
		exit(1);
	}
	trace(2, "UPnP SDK Successfully Initialized.");
#ifdef TEAMF1_CHANGES
	sleep(5);
#endif /* TEAMF1_CHANGES  */
	// Set the Device Web Server Base Directory
	trace(3, "Setting the Web Server Root Directory to %s",g_vars.xmlPath);
	if ( (ret = UpnpSetWebServerRootDir(g_vars.xmlPath)) != UPNP_E_SUCCESS )
	{
		syslog (LOG_ERR, "Error Setting Web Server Root Directory to: %s", g_vars.xmlPath);
		syslog (LOG_ERR, "  UpnpSetWebServerRootDir returned %d", ret); 
		UpnpFinish();
		exit(1);
	}
	trace(2, "Succesfully set the Web Server Root Directory.");

	//initialize the timer thread for expiration of mappings
	if (ExpirationTimerThreadInit()!=0) {
	  syslog(LOG_ERR,"ExpirationTimerInit failed");
	  UpnpFinish();
	  exit(1);
	}

	// Form the Description Doc URL to pass to RegisterRootDevice
	sprintf(descDocUrl, "http://%s:%d/%s", UpnpGetServerIpAddress(),
				UpnpGetServerPort(), g_vars.descDocName);

	// Register our IGD as a valid UPnP Root device
	trace(3, "Registering the root device with descDocUrl %s", descDocUrl);
	if ( (ret = UpnpRegisterRootDevice(descDocUrl, EventHandler, &deviceHandle,
					   &deviceHandle)) != UPNP_E_SUCCESS )
	{
		syslog(LOG_ERR, "Error registering the root device with descDocUrl: %s", descDocUrl);
		syslog(LOG_ERR, "  UpnpRegisterRootDevice returned %d", ret);
		UpnpFinish();
		exit(1);
	}

	trace(2, "IGD root device successfully registered.");
	
	// Initialize the state variable table.
	StateTableInit(descDocUrl);
	
	// Record the startup time, for uptime
	startup_time = time(NULL);
	
	// Send out initial advertisements of our device's services with timeouts of 30 minutes
	if ( (ret = UpnpSendAdvertisement(deviceHandle, 1800 ,adTTL) != UPNP_E_SUCCESS ))
	{
		syslog(LOG_ERR, "Error Sending Advertisements.  Exiting ...");
		UpnpFinish();
		exit(1);
	}
	trace(2, "Advertisements Sent.  Listening for requests ... ");
	alarm(adPeriod);

#ifdef UPNP_LISTEN_API
	if (pthread_create(&api_thr, NULL, api_start_server, NULL)) {
		syslog(LOG_ERR, "Error Starting API server. Exiting ...");
		UpnpFinish();
		exit(1);
	}
#endif
	
	// Loop until program exit signals received
	do {
	  sigemptyset(&sigsToCatch);
	  sigaddset(&sigsToCatch, SIGINT);
	  sigaddset(&sigsToCatch, SIGTERM);
	  sigaddset(&sigsToCatch, SIGUSR1);
	  sigaddset(&sigsToCatch,SIGALRM);	
	  pthread_sigmask(SIG_SETMASK, &sigsToCatch, NULL);
	  sigwait(&sigsToCatch, &signum);
	  trace(3, "Caught signal %d...\n", signum);
	  switch (signum) {
	  case SIGUSR1:
	    DeleteAllPortMappings();
	    break;
	  case SIGALRM:
		advertise();
		break;	
	  default:
	    break;
	  }
	} while (signum!=SIGTERM && signum!=SIGINT);

	trace(2, "Shutting down on signal %d...\n", signum);

	#ifdef UPNP_LISTEN_API
	api_thread_exit = 1;
	pthread_join(api_thr, NULL);
	#endif

	// Cleanup UPnP SDK and free memory
	DeleteAllPortMappings();
	ExpirationTimerThreadShutdown();

	UpnpUnRegisterRootDevice(deviceHandle);
	UpnpFinish();

	// Exit normally
	return (0);
}
