/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 2005-2007 Cavium Networks
 */
/*
 * File version info: $Id: executive-config.h 25756 2007-07-02 23:25:50Z kreese $
 */
#ifndef __EXECUTIVE_CONFIG_H__
#define __EXECUTIVE_CONFIG_H__
/* Define to enable IPfwd/NAT driver offload */
#define CVM_IPFWD_DRIVER_OFFLOAD

/* Define to enable dynamic short with IPfwd/NAT driver offload */
//#define CVM_IPFWD_DYNAMIC_SHORT

/* Define to enable the use of simple executive DFA functions */
//#define CVMX_ENABLE_DFA_FUNCTIONS

/* Define to enable the use of simple executive packet output functions.
** For packet I/O setup enable the helper functions below.
*/
#define CVMX_ENABLE_PKO_FUNCTIONS

/* Define to enable the use of simple executive timer bucket functions.
** Refer to cvmx-tim.[ch] for more information
*/
//#define CVMX_ENABLE_TIMER_FUNCTIONS

/* Define to enable the use of simple executive helper functions. These
** include many harware setup functions.  See cvmx-helper.[ch] for
** details.
*/
#define CVMX_ENABLE_HELPER_FUNCTIONS

/* CVMX_HELPER_FIRST_MBUFF_SKIP is the number of bytes to reserve before
** the beginning of the packet. If necessary, override the default
** here.  See the IPD section of the hardware manual for MBUFF SKIP
** details.*/
#define CVMX_HELPER_FIRST_MBUFF_SKIP 184

/* CVMX_HELPER_NOT_FIRST_MBUFF_SKIP is the number of bytes to reserve in each
** chained packet element. If necessary, override the default here */
#define CVMX_HELPER_NOT_FIRST_MBUFF_SKIP 0

/* CVMX_HELPER_ENABLE_BACK_PRESSURE controls whether back pressure is enabled
** for all input ports. This controls if IPD sends backpressure to all ports if
** Octeon's FPA pools don't have enough packet or work queue entries. Even when
** this is off, it is still possible to get backpressure from individual
** hardware ports. When configuring backpressure, also check
** CVMX_HELPER_DISABLE_*_BACKPRESSURE below. If necessary, override the default
** here */
#define CVMX_HELPER_ENABLE_BACK_PRESSURE 1

/* CVMX_HELPER_ENABLE_IPD controls if the IPD is enabled in the helper
**  function. Once it is enabled the hardware starts accepting packets. You
**  might want to skip the IPD enable if configuration changes are need
**  from the default helper setup. If necessary, override the default here */
#define CVMX_HELPER_ENABLE_IPD 0

/* CVMX_HELPER_INPUT_TAG_TYPE selects the type of tag that the IPD assigns
** to incoming packets. */
#define CVMX_HELPER_INPUT_TAG_TYPE CVMX_POW_TAG_TYPE_ORDERED

/* The following select which fields are used by the PIP to generate
** the tag on INPUT
** 0: don't include
** 1: include */
#define CVMX_HELPER_INPUT_TAG_IPV6_SRC_IP	    0
#define CVMX_HELPER_INPUT_TAG_IPV6_DST_IP   	0
#define CVMX_HELPER_INPUT_TAG_IPV6_SRC_PORT 	0
#define CVMX_HELPER_INPUT_TAG_IPV6_DST_PORT 	0
#define CVMX_HELPER_INPUT_TAG_IPV6_NEXT_HEADER 	0
#define CVMX_HELPER_INPUT_TAG_IPV4_SRC_IP	    1
#define CVMX_HELPER_INPUT_TAG_IPV4_DST_IP   	1
#define CVMX_HELPER_INPUT_TAG_IPV4_SRC_PORT 	1
#define CVMX_HELPER_INPUT_TAG_IPV4_DST_PORT 	1
#define CVMX_HELPER_INPUT_TAG_IPV4_PROTOCOL	    1
#define CVMX_HELPER_INPUT_TAG_INPUT_PORT	    0

/* Select skip mode for input ports */
#define CVMX_HELPER_INPUT_PORT_SKIP_MODE	CVMX_PIP_PORT_CFG_MODE_SKIPL2

/* Define the number of queues per output port */
#define CVMX_HELPER_PKO_QUEUES_PER_PORT_INTERFACE0	1
#define CVMX_HELPER_PKO_QUEUES_PER_PORT_INTERFACE1	1

/* Force backpressure to be disabled.  This overrides all other
** backpressure configuration */
#define CVMX_HELPER_DISABLE_RGMII_BACKPRESSURE 0

/* Disable the SPI4000's processing of backpressure packets and backpressure
** generation. When this is 1, the SPI4000 will not stop sending packets when
** receiving backpressure. It will also not generate backpressure packets when
** its internal FIFOs are full. */
#define CVMX_HELPER_DISABLE_SPI4000_BACKPRESSURE 0

/* Select the number of low latency memory ports (interfaces) that
** will be configured.  Valid values are 1 and 2.
*/
#define CVMX_LLM_CONFIG_NUM_PORTS 1

/* Enable the fix for PKI-100 errata ("Size field is 8 too large in WQE and next
** pointers"). If CVMX_ENABLE_LEN_M8_FIX is not enabled the fix for this errats will 
** not be enabled. 
** 0: Fix is not enabled
** 1: Fix is enabled, if supported by hardware
*/
#define CVMX_ENABLE_LEN_M8_FIX  1

#if defined(CVMX_ENABLE_HELPER_FUNCTIONS) && !defined(CVMX_ENABLE_PKO_FUNCTIONS)
#define CVMX_ENABLE_PKO_FUNCTIONS
#endif

#ifndef CVM_IPFWD_DRIVER_OFFLOAD
/* Executive resource descriptions provided in cvmx-resources.config */
#include "cvmx-resources.config"
#else /* CVM_IPFWD_DRIVER_OFFLOAD */

#ifdef CVM_IPFWD_DYNAMIC_SHORT
#undef CVMX_ENABLE_HELPER_FUNCTIONS
#endif /* CVM_IPFWD_DYNAMIC_SHORT */

/* Executive resource descriptions provided in cvmx-resources.config */
#include "cvmx-resources.config"

#ifdef CVM_IPFWD_DYNAMIC_SHORT
#define CVMX_ENABLE_HELPER_FUNCTIONS
/* Define CVMX_ENABLE_HELPER_FUNCTIONS to allocate resources for the
 * helper functions */
#ifdef CAVIUM_COMPONENT_REQUIREMENT
cvmxconfig
{
    fpa CVMX_FPA_WQE_POOL
        pool        = 0
        size        = 16
        priority    = 1
        description = "Work queue entrys";
}
#endif /* CAVIUM_COMPONENT_REQUIREMENT */

#endif /* CVM_IPFWD_DYNAMIC_SHORT */

#ifdef CAVIUM_COMPONENT_REQUIREMENT
cvmxconfig
{
    fpa CVMX_FPA_FLOW_INFO_POOL
        size        = 1
        protected   = 1
        description = "Pool for flow info buffers";
}
#endif /* CAVIUM_COMPONENT_REQUIREMENT */
#endif /* CVM_IPFWD_DRIVER_OFFLOAD */
#endif /* __EXECUTIVE_CONFIG_H__ */
