/*************************************************************************
* Cavium Octeon Ethernet Driver
*
* Author: Cavium Networks info@caviumnetworks.com
*
* Copyright (c) 2003-2007  Cavium Networks (support@cavium.com). All rights
* reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are
* met:
*
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*
*     * Redistributions in binary form must reproduce the above
*       copyright notice, this list of conditions and the following
*       disclaimer in the documentation and/or other materials provided
*       with the distribution.
*
*     * Neither the name of Cavium Networks nor the names of
*       its contributors may be used to endorse or promote products
*       derived from this software without specific prior written
*       permission.
*
* This Software, including technical data, may be subject to U.S.  export
* control laws, including the U.S.  Export Administration Act and its
* associated regulations, and may be subject to export or import regulations
* in other countries.  You warrant that You will comply strictly in all
* respects with all such regulations and acknowledge that you have the
* responsibility to obtain licenses to export, re-export or import the
* Software.
*
* TO THE MAXIMUM EXTENT PERMITTED BY LAW, THE SOFTWARE IS PROVIDED "AS IS"
* AND WITH ALL FAULTS AND CAVIUM NETWORKS MAKES NO PROMISES, REPRESENTATIONS
* OR WARRANTIES, EITHER EXPRESS, IMPLIED, STATUTORY, OR OTHERWISE, WITH
* RESPECT TO THE SOFTWARE, INCLUDING ITS CONDITION, ITS CONFORMITY TO ANY
* REPRESENTATION OR DESCRIPTION, OR THE EXISTENCE OF ANY LATENT OR PATENT
* DEFECTS, AND CAVIUM SPECIFICALLY DISCLAIMS ALL IMPLIED (IF ANY) WARRANTIES
* OF TITLE, MERCHANTABILITY, NONINFRINGEMENT, FITNESS FOR A PARTICULAR
* PURPOSE, LACK OF VIRUSES, ACCURACY OR COMPLETENESS, QUIET ENJOYMENT, QUIET
* POSSESSION OR CORRESPONDENCE TO DESCRIPTION.  THE ENTIRE RISK ARISING OUT
* OF USE OR PERFORMANCE OF THE SOFTWARE LIES WITH YOU.
*************************************************************************/
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/netdevice.h>
#include <linux/init.h>
#include <linux/etherdevice.h>
#include <linux/ip.h>
#include <linux/string.h>
#include <linux/ethtool.h>
#include <linux/mii.h>
#include <linux/seq_file.h>
#include <linux/proc_fs.h>
#include <net/dst.h>
#ifdef CONFIG_XFRM
#include <linux/xfrm.h>
#include <net/xfrm.h>
#endif  /* CONFIG_XFRM */

#include "wrapper-cvmx-includes.h"
#include "ethernet-headers.h"
#ifdef CONFIG_TF1_HW_OFFLOAD
#include "ethernet-offload.h"
#endif /*CONFIG_TF1_HW_OFFLOAD*/

extern int pow_receive_group;
extern struct net_device *cvm_oct_device[];
static struct tasklet_struct cvm_oct_tasklet[NR_CPUS];
#ifdef CONFIG_TF1_HW_OFFLOAD
extern int (*tf1_wlb_rate_ext_hook)(struct sk_buff *, struct net_device * in,
                                    int inVlanId, struct net_device * out, int outVlanId);

extern int (*tf1_ipt_account_ext_hook)(struct sk_buff *, struct net_device * in,
                                     struct net_device * out, 
                                     unsigned int proto, unsigned int port);
extern int ipfwd_driver_offload;
extern int cvm_ipfwd_cache_flow(struct sk_buff *skb, struct net_device *dev, cvm_packet_info_t *info);
extern void cvm_ipfwd_setup_flow_info(cvm_ipfwd_flow_info_t *info, struct sk_buff *skb, struct net_device *dev);
extern void cvm_ipfwd_setup_entry(struct iphdr *ip, cvm_ipfwd_flow_entry_t *entry);
extern int cvm_ipfwd_get_free_entry(cvm_ipfwd_flow_bucket_t *bucket, cvm_ipfwd_flow_entry_t *entry);
extern void cvm_ipfwd_mark_packet(struct sk_buff *skb, cvmx_wqe_t *work, cvm_ipfwd_flow_bucket_t *bucket);
extern int cvm_ipfwd_check_bucket_timestamp(cvm_ipfwd_flow_bucket_t *bucket);
extern int cvm_ipfwd_find_entry(cvm_ipfwd_flow_bucket_t *bucket,	 /* ptr to hash bucket (input) */
                                struct iphdr* ip,				 /* ptr to IP header (input) */
#ifdef CONFIG_TF1_HW_OFFLOAD
#ifdef CONFIG_VLAN_8021Q
                                uint16_t vlan_TCI, 
#endif
#endif /*CONFIG_TF1_HW_OFFLOAD*/
                                cvm_ipfwd_flow_entry_t *entry);
extern int cvm_ipfwd_check_timestamp(uint64_t created);
extern void flushFlowInfo();
extern int isMarkable(struct iphdr *ip);
extern void *cvm_ipfwd_ptr_alloc(void);
/* Shared pointer to the forwarding cache */
extern cvm_ipfwd_flow_bucket_t *cvm_gbl_ipfwd_cache;

#endif /*CONFIG_TF1_HW_OFFLOAD*/

/**
 * Interrupt handler. The interrupt occurs whenever the POW
 * transitions from 0->1 packets in our group.
 *
 * @param cpl
 * @param dev_id
 * @param regs
 * @return
 */
irqreturn_t cvm_oct_do_interrupt(int cpl, void *dev_id)
{
    /* Acknowledge the interrupt */
    if (INTERRUPT_LIMIT)
        cvmx_write_csr(CVMX_POW_WQ_INT, 1<<pow_receive_group);
    else
        cvmx_write_csr(CVMX_POW_WQ_INT, 0x10001<<pow_receive_group);
    preempt_disable();
    tasklet_schedule(cvm_oct_tasklet + smp_processor_id());
    preempt_enable();
    return IRQ_HANDLED;
}


#ifdef CONFIG_NET_POLL_CONTROLLER
/**
 * This is called when the kernel needs to manually poll the
 * device. For Octeon, this is simply calling the interrupt
 * handler. We actually poll all the devices, not just the
 * one supplied.
 *
 * @param dev    Device to poll. Unused
 */
void cvm_oct_poll_controller(struct net_device *dev)
{
    preempt_disable();
    tasklet_schedule(cvm_oct_tasklet + smp_processor_id());
    preempt_enable();
}
#endif


/**
 * Tasklet function that is scheduled on a core when an interrupt occurs.
 *
 * @param unused
 */
void cvm_oct_tasklet_rx(unsigned long unused)
{
    const int           coreid = cvmx_get_core_num();
    uint64_t            old_group_mask;
    uint64_t            old_scratch;
    int                 rx_count = 0;
    int                 number_to_free;
    int                 packet_not_copied;
	int number_real_free;
	int oom_flag;
#ifdef CONFIG_TF1_HW_OFFLOAD
#ifdef CVM_IPFWD_DRIVER_OFFLOAD
	cvm_ipfwd_eh_t *eh;
	cvm_ipfwd_flow_entry_t entry;
	cvm_ipfwd_flow_bucket_t *bucket = NULL;
#ifdef CONFIG_VLAN_8021Q
    struct vlan_ethhdr *vhdr;
#endif /* CONFIG_VLAN_8021Q */
	struct iphdr *ip;
	struct tcphdr *th;
	cvm_ipfwd_flow_info_t *info;
	int idx, status;
	int skip_fastpath;
#ifdef CONFIG_VLAN_8021Q
    int vlan_present=0;
#endif /* CONFIG_VLAN_8021Q */
#endif /* CVM_IPFWD_DRIVER_OFFLOAD */
    int inVlanId = 0;
    int outVlanId = 0;
    int vlan_TCI = 0;	
#endif /* CONFIG_TF1_HW_OFFLOAD */

    /* Prefetch cvm_oct_device since we know we need it soon */
    CVMX_PREFETCH(cvm_oct_device, 0);

    if (USE_ASYNC_IOBDMA)
    {
        /* Save scratch in case userspace is using it */
        CVMX_SYNCIOBDMA;
        old_scratch = cvmx_scratch_read64(CVMX_SCR_SCRATCH);
    }

    /* Only allow work for our group (and preserve priorities) */
    old_group_mask = cvmx_read_csr(CVMX_POW_PP_GRP_MSKX(coreid));
    cvmx_write_csr(CVMX_POW_PP_GRP_MSKX(coreid),
        (old_group_mask & ~0xFFFFull) | 1<<pow_receive_group);

    if (USE_ASYNC_IOBDMA)
        cvmx_pow_work_request_async(CVMX_SCR_SCRATCH, CVMX_POW_NO_WAIT);

	// we should first do this step in rx when oom

	oom_flag = 0;
    if (USE_SKBUFFS_IN_HW)
    {
        /* Refill the packet buffer pool */
        number_to_free = cvmx_fau_fetch_and_add32(FAU_NUM_PACKET_BUFFERS_TO_FREE, 0);
        if (number_to_free>0)
        {
            number_real_free = cvm_oct_mem_fill_fpa_cameo(CVMX_FPA_PACKET_POOL, CVMX_FPA_PACKET_POOL_SIZE, number_to_free);
		cvmx_fau_atomic_add32(FAU_NUM_PACKET_BUFFERS_TO_FREE, -number_real_free);
		if (number_to_free > number_real_free)
		{
			oom_flag = 1;
		}
		else
		{
			oom_flag = 0;
		}
		}
	else
	{
		oom_flag = 0;
        }
    }

    while (1)
    {
        void *start_of_buffer;
        struct sk_buff *skb;
        cvm_oct_callback_result_t callback_result;
        cvmx_wqe_t *work;
        if (USE_ASYNC_IOBDMA)
        {
            work = cvmx_pow_work_response_async(CVMX_SCR_SCRATCH);
        }
        else
        {
            if ((INTERRUPT_LIMIT == 0) || likely(rx_count < 60))
                work = cvmx_pow_work_request_sync(CVMX_POW_NO_WAIT);
            else
                work = NULL;
        }
        if (work == NULL)
            break;

        /* Limit each core to processing 60 packets without a break. This way
            the RX can't starve the TX task. */
        if (USE_ASYNC_IOBDMA)
        {
            if ((INTERRUPT_LIMIT == 0) || likely(rx_count < 60))
                cvmx_pow_work_request_async_nocheck(CVMX_SCR_SCRATCH, CVMX_POW_NO_WAIT);
            else
            {
                cvmx_scratch_write64(CVMX_SCR_SCRATCH, 0x8000000000000000ull);
                cvmx_pow_tag_sw_null_nocheck();
            }
        }
        rx_count++;
#ifdef CONFIG_TF1_HW_OFFLOAD
#ifdef CVM_IPFWD_DRIVER_OFFLOAD
		if(ipfwd_driver_offload)
		{
			bucket = &cvm_gbl_ipfwd_cache[work->tag & CVM_IPFWD_LUT_MASK];
			CVMX_PREFETCH0(bucket);
		}
#ifdef CVM_IPFWD_DYNAMIC_SHORT
		if(work->word2.s.bufs == 0)
		{
			/* Dynamic short pre-adjustment */
			work->packet_ptr.u64 = 0;
			work->packet_ptr.s.size = CVMX_FPA_WQE_POOL_SIZE;
			work->packet_ptr.s.pool = CVMX_FPA_WQE_POOL;
			if(work->word2.s.not_IP)
			{
				/* Non-IP */
				work->packet_ptr.s.addr = (uint64_t)(&work->packet_data);
			} else
			{
				if(!work->word2.s.is_v6)
				{
					/* IPv4 (shifted four bytes) */
					work->packet_ptr.s.addr = (uint64_t)(&work->packet_data) + CVM_IPFWD_IP_OFFSET + 4 - work->word2.s.ip_offset;
				} else
					/* IPv6 */
					work->packet_ptr.s.addr	= (uint64_t)(&work->packet_data) + CVM_IPFWD_IP_OFFSET - work->word2.s.ip_offset;
			}
		}
#endif /* CVM_IPFWD_DYNAMIC_SHORT */
#endif /* CVM_IPFWD_DRIVER_OFFLOAD */
#endif /* CONFIG_TF1_HW_OFFLOAD */


        CVMX_PREFETCH(cvm_oct_device[work->ipprt], 0);
        start_of_buffer = cvm_oct_get_buffer_ptr(work->packet_ptr);
        CVMX_PREFETCH(start_of_buffer, -8); /* Should be -sizeof(void*), but you can't use that with this macro. */

        /* Immediately throw away all packets when oom add by Joe */
	if ( oom_flag )
	{
              cvm_oct_free_work(work);
              continue;
	}
        /* Immediately throw away all packets with receive errors */
        if (unlikely(work->word2.snoip.rcv_error))
        {
            if ((work->word2.snoip.err_code == 10) && (work->len <= 64))
            {
                /* Ignore length errors on min size packets. Some equipment
                    incorrectly pads packets to 64+4FCS instead of 60+4FCS.
                    Note these packets still get counted as frame errors. */
            }
            else if (USE_10MBPS_PREAMBLE_WORKAROUND && ((work->word2.snoip.err_code == 5) || (work->word2.snoip.err_code == 7)))
            {
                /* We received a packet with either an alignment error or a
                    FCS error. This may be signalling that we are running
                    10Mbps with GMXX_RXX_FRM_CTL[PRE_CHK} off. If this is the
                    case we need to parse the packet to determine if we can
                    remove a non spec preamble and generate a correct packet */
                int interface = cvmx_helper_get_interface_num(work->ipprt);
                int index = cvmx_helper_get_interface_index_num(work->ipprt);
                cvmx_gmxx_rxx_frm_ctl_t gmxx_rxx_frm_ctl;
                gmxx_rxx_frm_ctl.u64 = cvmx_read_csr(CVMX_GMXX_RXX_FRM_CTL(index, interface));
                if (gmxx_rxx_frm_ctl.s.pre_chk == 0)
                {
                    uint8_t *ptr = cvmx_phys_to_ptr(work->packet_ptr.s.addr);
                    int i = 0;
                    while (i<work->len-1)
                    {
                        if (*ptr != 0x55)
                            break;
                        ptr++;
                        i++;
                    }
                    if (*ptr == 0xd5)
                    {
                        //DEBUGPRINT("Port %d received 0xd5 preamble\n", work->ipprt);
                        work->packet_ptr.s.addr += i+1;
                        work->len -= i+5;
                    }
                    else if ((*ptr & 0xf) == 0xd)
                    {
                        //DEBUGPRINT("Port %d received 0x?d preamble\n", work->ipprt);
                        work->packet_ptr.s.addr += i;
                        work->len -= i+4;
                        for (i=0; i<work->len; i++)
                        {
                            *ptr = ((*ptr&0xf0)>>4) | ((*(ptr+1)&0xf)<<4);
                            ptr++;
                        }
                    }
                    else
                    {
                        DEBUGPRINT("Port %d unknown preamble, packet dropped\n", work->ipprt);
                        //cvmx_helper_dump_packet(work);
                        cvm_oct_free_work(work);
                        continue;
                    }
                }
            }
            else
            {
                DEBUGPRINT("Port %d receive error code %d, packet dropped\n", work->ipprt, work->word2.snoip.err_code);
                cvm_oct_free_work(work);
                continue;
            }
        }

        /* We can only use the zero copy path if skbuffs are in the FPA pool
            and the packet fit in a single buffer */
#if (defined (CONFIG_TF1_HW_OFFLOAD) &&  defined(CVM_IPFWD_DRIVER_OFFLOAD) && defined(CVM_IPFWD_DYNAMIC_SHORT)) 
        if (USE_SKBUFFS_IN_HW && likely(work->word2.s.bufs <= 1))
#else /* CONFIG_TF1_HW_OFFLOAD */
        if (USE_SKBUFFS_IN_HW && likely(work->word2.s.bufs == 1))
#endif /* CONFIG_TF1_HW_OFFLOAD */
        {
            skb = *(struct sk_buff **)(start_of_buffer - sizeof(void*));
            /* This calculation was changed in case the skb header is using a
                different address aliasing type than the buffer. It doesn't make
                any differnece now, but the new one is more correct */
            skb->data = skb->head + work->packet_ptr.s.addr - cvmx_ptr_to_phys(skb->head);
            CVMX_PREFETCH(skb->data, 0);
            skb->len = work->len;
            skb->tail = skb->data + skb->len;
#ifdef CONFIG_TF1_HW_OFFLOAD
#ifdef CVM_IPFWD_DRIVER_OFFLOAD
			/* 
			 * Fast path: Do a flow-based IPFWD lookup 
			 *    Packet requirements: 
			 *       1) IPv4
			 *       2) TCP/UDP (if TCP, no SYN, FIN, or RESET)
			 *       3) non-fragment (at least for now)
			 */

			skip_fastpath = 0;
#ifdef CONFIG_TF1_HW_OFFLOAD
#ifdef CONFIG_VLAN_8021Q
            vhdr = (struct vlan_ethhdr*) (skb->data);
            vlan_present = ((vhdr->h_vlan_proto == htons(ETH_P_8021Q))?1:0);
#endif /* CONFIG_VLAN_8021Q */
#endif /*CONFIG_TF1_HW_OFFLOAD*/
			ip = (struct iphdr*) (skb->data + work->word2.s.ip_offset); 
			th = (struct tcphdr *) ((uint32_t*)ip + ip->ihl);
			if((ip->protocol == IP_PROTOCOL_TCP) && ((th->syn) || (th->fin) || (th->rst)))
				skip_fastpath = 1;

			if((ipfwd_driver_offload) &&
			   ((!work->word2.s.is_v6) &&
				(work->word2.s.tcp_or_udp) &&
				(!work->word2.s.is_frag) &&
				(!skip_fastpath)))
			{
#if 0
				{
					cvm_packet_info_t *pinfo = (cvm_packet_info_t *) ((((uint64_t) skb->end +
																		sizeof(struct skb_shared_info)) + 7) & ~7ull);
					if(pinfo->cookie == CVM_IPFWD_COOKIE_VALUE)
						printf("ERROR: rx packet has a valid cookie!\n");
				}
#endif

				//printk("%s: cvm_ipfwd_find_entry(bucket = %p)\n", __func__, bucket);
#ifdef CONFIG_TF1_HW_OFFLOAD
#ifdef CONFIG_VLAN_8021Q
                if(vlan_present == 1)
                idx = cvm_ipfwd_find_entry(bucket, ip,
                                           vhdr->h_vlan_TCI, &entry);
                else
                idx = cvm_ipfwd_find_entry(bucket, ip, NULL, &entry);
#else /* CONFIG_VLAN_8021Q */
				idx = cvm_ipfwd_find_entry(bucket, ip, &entry);
#endif  /* CONFIG_VLAN_8021Q */
#else /*CONFIG_TF1_HW_OFFLOAD*/
				idx = cvm_ipfwd_find_entry(bucket, ip, &entry);
#endif /*CONFIG_TF1_HW_OFFLOAD */

				if(idx == CVM_IPFWD_NOT_FOUND)
					goto not_found;

				/* Match found */

				/* Check its timestamp */
				status = cvm_ipfwd_check_timestamp(entry.s.timestamp);

				if(status == CVM_IPFWD_EXPIRED)
				{
#if 0
					ipfwd_slow_path_packets[cvmx_get_core_num()]++;
#endif
					goto not_found;
				}

				if((status != CVM_IPFWD_NEW) && (cvm_ipfwd_random_select()))
				{
					/* Old match found and this packet was selected to go through
					 * the Linux stack.  Make sure there is not already someone
					 * in this bucket doing this. */
					if(cvm_ipfwd_check_bucket_timestamp(bucket))
					{
#if 0
						ipfwd_slow_path_refresh[cvmx_get_core_num()]++;
#endif
						goto not_found;
					}
				}

				/* If we get here, entry is not expired */
				if(entry.s.flow_info)
				{
					/* match found -- format header */
					info = (cvm_ipfwd_flow_info_t*) cvmx_phys_to_ptr((uint64_t)entry.s.flow_info << 7);
					eh = (cvm_ipfwd_eh_t *) skb->data;
					eh->enet_dst_hi = info->enet_dst_hi;
					eh->enet_dst_lo = info->enet_dst_lo;
					eh->enet_src_hi = info->enet_src_hi;
					eh->enet_src_lo = info->enet_src_lo;
#ifdef CONFIG_TF1_HW_OFFLOAD
#ifdef CONFIG_VLAN_8021Q
                    if((vlan_present == 1) && (info->vlan_present == 1))
                    {
                        vlan_TCI = ntohs(vhdr->h_vlan_TCI);
                        inVlanId = (vlan_TCI & VLAN_VID_MASK);

                        vhdr->h_vlan_TCI = info->vlan_info;

                        vlan_TCI = ntohs(vhdr->h_vlan_TCI);
                        outVlanId = (vlan_TCI & VLAN_VID_MASK);
                    }
                    /* TODO: Need to optimize this portion of code
                     Untagging can be done by moving the data pointer
                     up instead of moving the entire packet data.But it
                     Involves change in work structure entries.This can be 
                     included in the next release*/
                    // Untagging the packet
                    else if( (vlan_present == 1) && (info->vlan_present == 0) )
                        {
                        vlan_TCI = ntohs(vhdr->h_vlan_TCI);
                        inVlanId = (vlan_TCI & VLAN_VID_MASK);
                        outVlanId = 0;

                        memmove (skb->data + VLAN_HLEN, skb->data, VLAN_ETH_ALEN * 2);
                        skb_pull (skb, VLAN_HLEN);
                        work->word2.s.ip_offset = 14;
                        work->packet_ptr.s.addr = cvmx_ptr_to_phys(skb->data);
                        work->len -= 4;
                        }
                    // Tagging the packet
                    else if( (vlan_present == 0) && (info->vlan_present == 1) )
                        {
                        if (skb_headroom(skb) >= VLAN_HLEN)
                            {
                            skb_push (skb, VLAN_HLEN);
                            memmove (skb->data, skb->data + VLAN_HLEN, VLAN_ETH_ALEN * 2);
                            vhdr = (struct vlan_ethhdr*) (skb->data);
                            }
                        else
                            {
                            memmove(skb->data+16,skb->data+12,skb->len-12);
                            }
                        vhdr->h_vlan_proto = htons(ETH_P_8021Q);
            		    vhdr->h_vlan_TCI = info->vlan_info;
		                work->word2.s.ip_offset = 18;
            		    work->packet_ptr.s.addr = cvmx_ptr_to_phys(skb->data);
            		    work->len += 4;
            		    inVlanId = 0;
            		    vlan_TCI = ntohs(vhdr->h_vlan_TCI);
            		    outVlanId = (vlan_TCI & VLAN_VID_MASK);
		                }
#endif
#endif /* !CONFIG_TF1_HW_OFFLOAD */
					ip = (struct iphdr*) (skb->data + work->word2.s.ip_offset);
					ip->ttl--;
					ip->saddr = info->ip_saddr;
					ip->daddr = info->ip_daddr;
					ip->protocol = info->ip_proto;
					*((uint32_t*) ip + ip->ihl) = info->ports;

					/* compute IP checksum */
					ip->check = cvm_ipfwd_calculate_ip_header_checksum((uint16_t*) ip);


					/* Use the cvm_oct_transmit() to send the packet */
					cvm_oct_transmit_qos(info->dev, work, 1,0);

#ifdef CONFIG_TF1_HW_OFFLOAD
					/* TeamF1: wlb changes - Update rate estimator */
					info->lastused = jiffies;
                    /* Update stats in ipt account*/
                    if (tf1_ipt_account_ext_hook != NULL) {
                        (*tf1_ipt_account_ext_hook)(skb, cvm_oct_device[work->ipprt], info->dev, info->ip_proto,info->ports);
                    }

#endif /* CONFIG_TF1_HW_OFFLOAD */


					/* done */
					continue;
				}

				not_found:
				/* 
				 * No match found (or old match found) -- 
				 * mark the packet (buffer) so that the flow can be
				 * cached on the way out.  
				 */
				cvm_ipfwd_mark_packet(skb, work, bucket);
			}
#endif /* CVM_IPFWD_DRIVER_OFFLOAD */
#endif /* CONFIG_TF1_HW_OFFLOAD */
            packet_not_copied = 1;
        }
        else
        {
            /* We have to copy the packet. First allocate an skbuff for it */
            skb = dev_alloc_skb(work->len);
            if (!skb)
            {
                DEBUGPRINT("Port %d failed to allocate skbuff, packet dropped\n", work->ipprt);
                cvm_oct_free_work(work);
                continue;
            }

            /* Check if we've received a packet that was entirely stored the
                work entry. This is untested */
            if (unlikely(work->word2.s.bufs == 0))
            {
                uint8_t *ptr = work->packet_data;
                if (cvmx_likely(!work->word2.s.not_IP))
                {
                    /* The beginning of the packet moves for IP packets */
                    if (work->word2.s.is_v6)
                        ptr += 2;
                    else
                        ptr += 6;
                }
                memcpy(skb_put(skb, work->len), ptr, work->len);
                /* No packet buffers to free */
            }
            else
            {
                int segments = work->word2.s.bufs;
                cvmx_buf_ptr_t segment_ptr = work->packet_ptr;
                int len = work->len;
                while (segments--)
                {
                    cvmx_buf_ptr_t next_ptr = *(cvmx_buf_ptr_t*)cvmx_phys_to_ptr(segment_ptr.s.addr-8);
                    /* Octeon Errata PKI-100: The segment size is wrong. Until it
                        is fixed, calculate the segment size based on the packet
                        pool buffer size. When it is fixed, the following line should
                        be replaced with this one:
                            int segment_size = segment_ptr.s.size; */
                    int segment_size = CVMX_FPA_PACKET_POOL_SIZE - (segment_ptr.s.addr - (((segment_ptr.s.addr >> 7) - segment_ptr.s.back) << 7));
                    /* Don't copy more than what is left in the packet */
                    if (segment_size > len)
                        segment_size = len;
                    /* Copy the data into the packet */
                    memcpy(skb_put(skb, segment_size), cvmx_phys_to_ptr(segment_ptr.s.addr), segment_size);
                    /* Reduce the amount of bytes left to copy */
                    len -= segment_size;
                    segment_ptr = next_ptr;
                }
            }
            packet_not_copied = 0;
        }

        if (likely((work->ipprt < TOTAL_NUMBER_OF_PORTS) && cvm_oct_device[work->ipprt]))
        {
            struct net_device *dev = cvm_oct_device[work->ipprt];
            cvm_oct_private_t* priv = (cvm_oct_private_t*)netdev_priv(dev);

            /* Only accept packets for devices that are currently up */
            if (likely(dev->flags & IFF_UP))
            {
                skb->protocol = eth_type_trans(skb, dev);
                skb->dev = dev;

                if (unlikely(work->word2.s.not_IP || work->word2.s.IP_exc || work->word2.s.L4_error))
                    skb->ip_summed = CHECKSUM_NONE;
                else
                    skb->ip_summed = CHECKSUM_UNNECESSARY;
                /* Increment RX stats for virtual ports */
                if (work->ipprt >= CVMX_PIP_NUM_INPUT_PORTS)
                {
#ifdef CONFIG_64BIT
                    cvmx_atomic_add64_nosync(&priv->stats.rx_packets, 1);
                    cvmx_atomic_add64_nosync(&priv->stats.rx_bytes, skb->len);
#else
                    cvmx_atomic_add32_nosync((int32_t*)&priv->stats.rx_packets, 1);
                    cvmx_atomic_add32_nosync((int32_t*)&priv->stats.rx_bytes, skb->len);
#endif
                }

                if (priv->intercept_cb)
                {
                    callback_result = priv->intercept_cb(dev, work, skb);
                    switch (callback_result)
                    {
                        case CVM_OCT_PASS:
                            netif_receive_skb(skb);
                            break;
                        case CVM_OCT_DROP:
                            dev_kfree_skb_irq(skb);
#ifdef CONFIG_64BIT
                            cvmx_atomic_add64_nosync(&priv->stats.rx_dropped, 1);
#else
                            cvmx_atomic_add32_nosync((int32_t*)&priv->stats.rx_dropped, 1);
#endif
                            break;
                        case CVM_OCT_TAKE_OWNERSHIP_WORK:
                            /* Interceptor stole our work, but we need to free
                                the skbuff */
                            if (USE_SKBUFFS_IN_HW && likely(packet_not_copied))
                            {
                                /* We can't free the skbuff since its data is
                                    the same as the work. In this case we don't
                                    do anything */
                            }
                            else
                                dev_kfree_skb_irq(skb);
                            break;
                        case CVM_OCT_TAKE_OWNERSHIP_SKB:
                            /* Interceptor stole our packet */
                            break;
                    }
                }
                else
                {
                    netif_receive_skb(skb);
                    callback_result = CVM_OCT_PASS;
                }
            }
            else
            {
                /* Drop any packet received for a device that isn't up */
                //DEBUGPRINT("%s: Device not up, packet dropped\n", dev->name);
#ifdef CONFIG_64BIT
                cvmx_atomic_add64_nosync(&priv->stats.rx_dropped, 1);
#else
                cvmx_atomic_add32_nosync((int32_t*)&priv->stats.rx_dropped, 1);
#endif
                dev_kfree_skb_irq(skb);
                callback_result = CVM_OCT_DROP;
            }
        }
        else
        {
            /* Drop any packet received for a device that doesn't exist */
            DEBUGPRINT("Port %d not controlled by Linux, packet dropped\n", work->ipprt);
            dev_kfree_skb_irq(skb);
            callback_result = CVM_OCT_DROP;
        }

        /* We only need to free the work if the interceptor didn't take over
            ownership of it */
        if (callback_result != CVM_OCT_TAKE_OWNERSHIP_WORK)
        {
            /* Check to see if the skbuff and work share the same packet buffer */
            if (USE_SKBUFFS_IN_HW && likely(packet_not_copied))
            {
                /* This buffer needs to be replaced, increment the number of buffers we need to free by one */
                cvmx_fau_atomic_add32(FAU_NUM_PACKET_BUFFERS_TO_FREE, 1);
                cvmx_fpa_free(work, CVMX_FPA_WQE_POOL, DONT_WRITEBACK(1));
            }
            else
                cvm_oct_free_work(work);
        }
    }

    /* Restore the original POW group mask */
    cvmx_write_csr(CVMX_POW_PP_GRP_MSKX(coreid), old_group_mask);
    if (USE_ASYNC_IOBDMA)
    {
        /* Restore the scratch area */
        cvmx_scratch_write64(CVMX_SCR_SCRATCH, old_scratch);
    }

    if (USE_SKBUFFS_IN_HW)
    {
        /* Refill the packet buffer pool */
        number_to_free = cvmx_fau_fetch_and_add32(FAU_NUM_PACKET_BUFFERS_TO_FREE, 0);
        if (number_to_free>0)
        {
        	// cavium original code
        	/* 
            cvmx_fau_atomic_add32(FAU_NUM_PACKET_BUFFERS_TO_FREE, -number_to_free);
            cvm_oct_mem_fill_fpa(CVMX_FPA_PACKET_POOL, CVMX_FPA_PACKET_POOL_SIZE, number_to_free);
		*/
			// Joe modify		
			
            number_real_free = cvm_oct_mem_fill_fpa_cameo(CVMX_FPA_PACKET_POOL, CVMX_FPA_PACKET_POOL_SIZE, number_to_free);
		cvmx_fau_atomic_add32(FAU_NUM_PACKET_BUFFERS_TO_FREE, -number_real_free);
        }
    }
}



void cvm_oct_rx_initialize(void)
{
    int i;
    /* Initialize all of the tasklets */
    for (i=0; i<NR_CPUS; i++)
        tasklet_init(cvm_oct_tasklet + i, cvm_oct_tasklet_rx, 0);
}

void cvm_oct_rx_shutdown(void)
{
    int i;
    /* Shutdown all of the tasklets */
    for (i=0; i<NR_CPUS; i++)
        tasklet_kill(cvm_oct_tasklet + i);
}

