/* ipt_BWPROF.h - bandwidth profile module header file */

/* Copyright (c) 2007, TeamF1, Inc. */

/*
modification history
--------------------
01b,26dec07,suh ported for 32-bit userland on a 64-bit kernel
01a,28Feb07,rnp created
*/
#ifndef _IPT_BWPROF_H_target
#define _IPT_BWPROF_H_target

#include <linux/config.h>

#ifndef __KERNEL__
/* must be same as IP_CT_DIR_XXX ip_conntrack.h */
#define IP_CT_DIR_ORIGINAL 0
#define IP_CT_DIR_REPLY    1
#define IP_CT_DIR_MAX      2
#endif

/* possible directions */
#define BWPROF_DIR_ORIGINAL IP_CT_DIR_ORIGINAL
#define BWPROF_DIR_REPLY    IP_CT_DIR_REPLY
#define BWPROF_DIR_BOTH     2
#define BWPROF_DIR_NONE     3
#define BWPROF_DIR_UNKNOWN  ((unsigned long)-1)

/* the same profile can be set on upto 8 interfaces */
#define BWPROF_MAX_IF  8
/* 16 chars per if name */
#define BWPROF_MAX_IF_NAME_SZ  (16)

/* possible types */
#define BWPROF_TYPE_ALL        0
#define BWPROF_TYPE_PER_SRC_IP 1
#define BWPROF_TYPE_PER_DST_IP 2

#define BWPROF_NO_TC_INDEX      0
#define BWPROF_INVALID_TC_INDEX ((uint32_t)-1)

/* this should probably be same as prefix in struct ipt_log_info */
#define BWPROF_MAX_LOG_PREFIX_SZ    (30)

/* this should probably be same as format in struct ipt_log_info */
#define BWPROF_MAX_LOG_FORMAT_SZ    (256)

/* possible actions */
#define BWPROF_ACTION_ACCEPT           0
#define BWPROF_ACTION_BLOCK            1
#define BWPROF_ACTION_LOG_ACCEPT       2
#define BWPROF_ACTION_LOG_BLOCK        3
#define BWPROF_ACTION_SCHED_ACCEPT     4
#define BWPROF_ACTION_SCHED_BLOCK      5
#define BWPROF_ACTION_SCHED_LOG_ACCEPT 6
#define BWPROF_ACTION_SCHED_LOG_BLOCK  7



#define IPT_BWPROF_OPT_LOG          0x1e
#define IPT_BWPROF_OPT_LOG_PREFIX   0x07
#define IPT_BWPROF_OPT_LOG_FORMAT   0x08
#define FLAG(opt)   (1<<opt)
    

#ifdef CONFIG_COMPAT
struct ipt_bwprof_target_info
    {
    /* profile parameters */
    uint64_t      id;
    uint64_t      dir;
    uint64_t      minbw;
    uint64_t      maxbw;
    uint64_t      type;
    uint64_t      ifindex[BWPROF_MAX_IF];
    uint32_t      ct_tc_index;
    int           maxInstances;  
    int32_t       flags;
    int32_t       action;  
    uint16_t      timeStart;  /* 0 < time_start < 23*60+59 = 1439 */
    uint16_t      timeStop;   /* 0:0 < time_stat < 23:59 */
    uint8_t       timeDays;   /* 1 bit per day. -SMTWTFS */
    unsigned char logLevel;
    char          logPrefix[BWPROF_MAX_LOG_PREFIX_SZ];
    char          logFormat[BWPROF_MAX_LOG_FORMAT_SZ];
    char          ifName[BWPROF_MAX_IF][BWPROF_MAX_IF_NAME_SZ];
    };
#else /* CONFIG_COMPAT */
struct ipt_bwprof_target_info
    {
    /* profile parameters */
    unsigned long id;
    unsigned long dir;
    unsigned long minbw;
    unsigned long maxbw;
    unsigned long type;
    unsigned long ct_tc_index;
    int           maxInstances;  
    int           flags;
    int           action;  
    unsigned long ifindex[BWPROF_MAX_IF];
    u_int16_t     timeStart;  /* 0 < time_start < 23*60+59 = 1439 */
    u_int16_t     timeStop;   /* 0:0 < time_stat < 23:59 */
    u_int8_t      timeDays;   /* 1 bit per day. -SMTWTFS */
    unsigned char logLevel;
    char          logPrefix[BWPROF_MAX_LOG_PREFIX_SZ];
    char          logFormat[BWPROF_MAX_LOG_FORMAT_SZ];
    char          ifName[BWPROF_MAX_IF][BWPROF_MAX_IF_NAME_SZ];
    };
#endif /* CONFIG_COMPAT */

#ifdef __KERNEL__
#define BWPROF_INVALID_IFINDEX ((unsigned long)-1)
typedef struct ipt_bwprof_target_info BW_PROF_INFO;

typedef struct bwMgmt
    {
    /* 
     * add/delete a class of <classid> on given <ifindex> with <minbw>, <maxbw>
     * properties 
     */
    int (*classCtl)(uint32_t classid, uint32_t cmd, uint32_t ifindex,
                    uint32_t minbw, uint32_t maxbw);

    int (*qdiscCtl)(uint32_t cmd, uint32_t parent, 
                    uint32_t ifindex);
                    
    } BW_MGMT;

int bwprofClassPut (uint32_t, BW_PROF_INFO *);
#endif /* __KERNEL__ */

#endif /*_IPT_BWPROF_H_target*/
