/* ipt_ANTISPOOF.c - ANTI SPOOF Target */

/* Copyright (c) 2009, TeamF1, Inc. */

/*
 * modification history
 * --------------------
 *  01a,06Feb09,hk created
 *  */

/* This is a target which is used for droping the spoofed packets. */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * */

#include <linux/module.h>
#include <linux/skbuff.h>
#include <linux/ip.h>
#include <net/checksum.h>
#include <net/ip_fib.h>

#include <linux/netfilter/x_tables.h>
#include <linux/netlink.h>
#include <linux/netfilter_ipv4.h>
#include <linux/netfilter_ipv4/ipt_ANTISPOOF.h>
#include <linux/netfilter_ipv4/ipt_LOG.h>

MODULE_DESCRIPTION("IP tables ANTISPOOF module");

#ifdef CONFIG_IP_NF_TARGET_ANTISPOOF
int fib_antiSpoof_target(__be32 src, __be32 dst, u8 tos,
			const struct net_device *dev,  struct sk_buff  *pskb)
{
    int oif =0;
	struct flowi fl = { .nl_u = { .ip4_u =
				      { .daddr = src,
					.saddr = dst,
					.tos = tos } },
			    .iif = oif };
	struct fib_result res;
	int ret=0;
   

   /* Attempt to look up the source address in the FIB. A return code of NULL 
    * indicates success, and on success, res points to a filled in results 
    * structure. On failure it is necessary to visit the last_resort.
    * */
    if (fib_lookup(&fl, &res))
        goto last_resort;

    if (res.type == RTN_LOCAL) {
        fib_res_put(&res);
        return 1;
    }

    /* broadcast and multicast source addresses are considered martian(commonly
     * used to refer to an IP address that appears to be defective or spoofed).
     * Therefore, since a source address is being validated, only unicast route
     * types are legitimate */
    if (res.type != RTN_UNICAST)
        goto e_inval_res;

    /*The FIB_RES_DEV(res) macro, returns the  device that should be used for 
     * the next outgoing hop associated with # define FIB_RES_DEV(res) 
     * (FIB_RE thSi_s NFHIB( reentsry).. n h _dev) */

    /*Since the key that was used to obtain the res pointer used the remote
     * source address of this packet,the device on which the packet arrived 
     * should be the next hop device for a transmission back to the source. */

#ifdef CONFIG_IP_ROUTE_MULTIPATH
    /*If multipath routing is enabled and there are multiple possible next hops,
     * (res.fi.>fib_nhs > 1), the code does not bother to ensure that the 
     * device upon which the  packet was received is included in the possible 
     * outgoing next hops. */
	if (FIB_RES_DEV(res) == dev || res.fi->fib_nhs > 1)
#else
	if (FIB_RES_DEV(res) == dev)
#endif
	{
    /*If the device upon which the packet was received is consistent with the
     * perceived next hop, and the scope of the FIB entry is RT_SCOPE_HOST 
     * release the FIB entry return success now */
     
		ret = FIB_RES_NH(res).nh_scope >= RT_SCOPE_HOST;
		fib_res_put(&res);
		return ret;
	}

	fib_res_put(&res);

    /* return invalid value */
    goto e_inval;

last_resort:
		goto e_inval;
e_inval_res:
    /* fib_res_put(which decrements r_clntref) should be called on after every 
     * succesful fib lookup (which decrements reference count) */
	fib_res_put(&res);
e_inval:
	return -EINVAL;
}
#endif /* CONFIG_IP_NF_TARGET_ANTISPOOF */

static unsigned int
ipt_antiSpoof_target(struct sk_buff **pskb,
	       const struct net_device *in, const struct net_device *out,
	       unsigned int hooknum, const struct xt_target *target,
	       const void *targinfo)
{
    int result=1;
    const u_int8_t logStatus = 
            ((struct ipt_antispoof_target_info *)targinfo)->logStatus;

    /* call antispoof routine with source and input interfaces to determine
     * whether route exists back to the sender of a  received packet 
     */
    result = fib_antiSpoof_target((*pskb)->nh.iph->saddr,
                                  (*pskb)->nh.iph->daddr,
                                  (*pskb)->nh.iph->tos,in,*pskb);

    /*  if result is less than zero (i.e spoofed packet) then packet should be
     *  dropped */
    if (result < 0) {
        /* log the packet if it is enabled from the rule * else don't drop */
        if (logStatus == IPANTISPOOF_LOG_ENABLE) {
            struct ipt_log_info  loginfo;

            memset (&loginfo, 0x00, sizeof(loginfo));
            snprintf (loginfo.prefix, sizeof(loginfo.prefix),
                      "[SPOOFED][DROP]");
            dump_packet_wrapper(&loginfo, (*pskb),0);
        }        
        return NF_DROP;
    }

    /* if not a spoofed packet ,then continue */
    return XT_CONTINUE;
}

/* paramter sanity check */
static int ipt_antiSpoof_checkentry(const char *tablename,
		const void *e,
		const struct xt_target *target,
		void *targinfo,
		unsigned int hook_mask)
{
  const u_int8_t logStatus = 
          ((struct ipt_antispoof_target_info *)targinfo)->logStatus;

  /* return if log status is other than enable /disable */ 
  if (logStatus != IPANTISPOOF_LOG_ENABLE && 
      logStatus != IPANTISPOOF_LOG_DISABLE) {
      return 0;
  }

	return 1;
}

static struct xt_target ipt_ANTISPOOF = {
	.name 		= "ANTISPOOF",
	.family		= AF_INET,
	.target 	= ipt_antiSpoof_target,
	.targetsize	= sizeof(struct ipt_antispoof_target_info),
	.table		= "filter",
	.checkentry 	= ipt_antiSpoof_checkentry,
	.me 		= THIS_MODULE,
};

static int __init ipt_antiSpoof_init(void)
{
	return xt_register_target(&ipt_ANTISPOOF);
}

static void __exit ipt_antiSpoof_fini(void)
{
	xt_unregister_target(&ipt_ANTISPOOF);
}

module_init(ipt_antiSpoof_init);
module_exit(ipt_antiSpoof_fini);
