/*
 * This is a module which is used for rejecting packets.
 * Added support for customized reject packets (Jozsef Kadlecsik).
 * Added support for ICMP type-3-code-13 (Maciej Soltysiak). [RFC 1812]
 */

/* (C) 1999-2001 Paul `Rusty' Russell
 * (C) 2002-2004 Netfilter Core Team <coreteam@netfilter.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include <linux/module.h>
#include <linux/skbuff.h>
#ifdef TEAMF1_CHANGES
#include <linux/netdevice.h>
#include <linux/inetdevice.h>
#endif /* TEAMF1_CHANGES */
#include <linux/ip.h>
#include <linux/udp.h>
#include <linux/icmp.h>
#include <net/icmp.h>
#include <net/ip.h>
#include <net/tcp.h>
#include <net/route.h>
#include <net/dst.h>
#include <linux/netfilter/x_tables.h>
#include <linux/netfilter_ipv4/ip_tables.h>
#include <linux/netfilter_ipv4/ipt_REJECT.h>
#ifdef CONFIG_BRIDGE_NETFILTER
#include <linux/netfilter_bridge.h>
#endif
#ifdef TEAMF1_CHANGES
#include <linux/netfilter_ipv4/ip_conntrack.h>

/* NOTE:- This is only for SSLVPN */
#ifdef CONFIG_CPU_CAVIUM_OCTEON
#define CONFIG_SSLVPN
#endif
/* NOTE:- SSLVPN is not present in RV120W so redirection to https is not required. */
#ifdef CONFIG_OCTEON_RV100W
#undef CONFIG_SSLVPN
#endif
/* NOTE:- SSLVPN is not present in RV220W for now so redirection to https is not required. */
#ifdef CONFIG_OCTEON_RV200W
#undef CONFIG_SSLVPN
#endif
#endif /* TEAMF1_CHANGES */

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Netfilter Core Team <coreteam@netfilter.org>");
MODULE_DESCRIPTION("iptables REJECT target module");

#if 0
#define DEBUGP printk
#else
#define DEBUGP(format, args...)
#endif

#ifdef TEAMF1_CHANGES
#define httpHdrCat(cmdBuf, length, fmt...)                          \
	do {                                                        \
		result = snprintf(cmdBuf+bufOffSet,                 \
				  length-bufOffSet, fmt);           \
		if (result == 0)                                    \
			return -1;                                  \
		bufOffSet += result;                                \
	} while(0)



static unsigned long getIfAddr(char * ifname)
{
	struct in_ifaddr *ia;
	struct net_device *dev;

	if (!ifname)
		return 0;
    
	read_lock(&dev_base_lock);  
	for (dev = dev_base; dev != NULL; dev = dev->next) {
		struct in_device *in_dev = in_dev_get(dev);
		if (in_dev==NULL || (ia = in_dev->ifa_list) == NULL)
			continue;
		for( ; ia; ia = ia->ifa_next) {
			if (!strcmp(ifname, ia->ifa_label)) {
				read_unlock(&dev_base_lock);
				return ia->ifa_address;
			}
		}
	}
	read_unlock(&dev_base_lock);
	return 0;
}

int httpRedirectGen (char * ipAddrStr, char * buff, int len)
{
	int     bufOffSet=0;
	int     result=0;    
	char    htmlPage[128];
	int     dataLen;

	memset (htmlPage, '\0', sizeof(htmlPage));
	dataLen = snprintf (htmlPage, 128, "<html><head>.<META HTTP-EQUIV=\"Refresh\" Content= \"0; URL=%s/blocked.htm\">.</head>\r\n", ipAddrStr); 
	httpHdrCat(buff, len,"HTTP/1.1 307 Temporary redirect\r\n");
#ifdef CONFIG_SSLVPN    
	httpHdrCat(buff, len,"Location:https://%s/scgi-bin/platform.cgi?page=blocked.htm\r\n", ipAddrStr);
#else
	httpHdrCat(buff, len,"Location:http://%s/blocked.htm\r\n", ipAddrStr);
#endif    
	httpHdrCat(buff, len,"Content-Length:%d\r\n", dataLen);
	httpHdrCat(buff, len,"Content-type: text/html\r\n");
	httpHdrCat(buff, len,"\r\n");
	httpHdrCat(buff, len, htmlPage);
	return bufOffSet;
}


/* Send http redirect back */
static void send_http_redirect(struct sk_buff *oldskb, int hook)
{
	struct sk_buff *nskb;
	struct sk_buff *nskb2;
	struct tcphdr _otcph, *oth, *tcph;
	u_int16_t tmp_port;
	u_int32_t tmp_addr;
	char buff[1024];
	char * pData = NULL;
	int  packetLength = 0;
    	char ipAddrStr[32];
	unsigned int addr_type;
	enum ip_conntrack_info ctinfo;
	struct ip_conntrack *ct;

	/* IP header checks: fragment. */
	if (oldskb->nh.iph->frag_off & htons(IP_OFFSET))
		return;

	oth = skb_header_pointer(oldskb, oldskb->nh.iph->ihl * 4,
				 sizeof(_otcph), &_otcph);
	if (oth == NULL)
		return;

	/* No RST for RST. */
	if (oth->rst)
		return;

	/* Check checksum */
	if (nf_ip_checksum(oldskb, hook, oldskb->nh.iph->ihl * 4, IPPROTO_TCP))
		return;

	/* We need a linear, writeable skb.  We also need to expand
	   headroom in case hh_len of incoming interface < hh_len of
	   outgoing interface */
	nskb = skb_copy_expand(oldskb, LL_MAX_HEADER, skb_tailroom(oldskb)+1024,
			       GFP_ATOMIC);
	if (!nskb)
		return;

	tmp_addr = getIfAddr(oldskb->dev->name);
	snprintf (ipAddrStr, sizeof(ipAddrStr), "%u.%u.%u.%u", NIPQUAD(tmp_addr));
	packetLength = httpRedirectGen(ipAddrStr, buff, sizeof(buff));

	/* This packet will not be the same as the other: clear nf fields */
	nf_reset(nskb);
	nskb->mark = 0;
	skb_init_secmark(nskb);

	skb_shinfo(nskb)->gso_size = 0;
	skb_shinfo(nskb)->gso_segs = 0;
	skb_shinfo(nskb)->gso_type = 0;

	tcph = (struct tcphdr *)((u_int32_t*)nskb->nh.iph + nskb->nh.iph->ihl);

	/* Swap source and dest */
	tmp_addr = nskb->nh.iph->saddr;
	nskb->nh.iph->saddr = nskb->nh.iph->daddr;
	nskb->nh.iph->daddr = tmp_addr;
	tmp_port = tcph->source;
	tcph->source = tcph->dest;
	tcph->dest = tmp_port;

	/* Truncate to length (no data) */
	tcph->doff = sizeof(struct tcphdr)/4;
	skb_trim(nskb, nskb->nh.iph->ihl*4 + sizeof(struct tcphdr));
	nskb->nh.iph->tot_len = htons(nskb->len);

	pData = nskb->data + nskb->nh.iph->ihl*4 + sizeof(struct tcphdr);
	memcpy(pData, buff, packetLength);
	skb_put(nskb, packetLength);

	nskb->nh.iph->tot_len = htons(nskb->len);

	tcph->seq = oth->ack_seq;
		tcph->ack_seq = htonl(ntohl(oth->seq) + oth->syn + oth->fin
				      + oldskb->len - oldskb->nh.iph->ihl*4
				      - (oth->doff<<2));
	/* Reset flags */
	((u_int8_t *)tcph)[13] = 0;
	tcph->psh = 1;
	tcph->ack = 1;
	tcph->fin = 1;

	tcph->window = 0;
	tcph->urg_ptr = 0;

	/* Adjust TCP checksum */
	tcph->check = 0;
	tcph->check = tcp_v4_check(sizeof(struct tcphdr) + packetLength,
				   nskb->nh.iph->saddr,
				   nskb->nh.iph->daddr,
				   csum_partial((char *)tcph,
						sizeof(struct tcphdr) + packetLength, 0));

	/* Set DF, id = 0 */
	nskb->nh.iph->frag_off = htons(IP_DF);
	nskb->nh.iph->id = 0;

	addr_type = RTN_UNSPEC;
	if (hook != NF_IP_FORWARD
#ifdef CONFIG_BRIDGE_NETFILTER
	    || (nskb->nf_bridge && nskb->nf_bridge->mask & BRNF_BRIDGED)
#endif
	   )
		addr_type = RTN_LOCAL;

	if (ip_route_me_harder(&nskb, addr_type))
		goto free_nskb;

	nskb->ip_summed = CHECKSUM_NONE;

	/* Adjust IP TTL */
	nskb->nh.iph->ttl = MAXTTL;

	/* Adjust IP checksum */
	nskb->nh.iph->check = 0;
	nskb->nh.iph->check = ip_fast_csum((unsigned char *)nskb->nh.iph,
					   nskb->nh.iph->ihl);

	/* "Never happens" */
	if (nskb->len > dst_mtu(nskb->dst->path))
		goto free_nskb;

	nf_ct_attach(nskb, oldskb);

	/* Reset the mark for redirect packet*/
	ct = ip_conntrack_get(nskb, &ctinfo);
	ct->mark = 0;
	nskb->mark = 0;

	NF_HOOK(PF_INET, NF_IP_LOCAL_OUT, nskb, NULL, nskb->dst->dev,
		dst_output);

	/* send a TCP reset to the server */

	/* Check checksum */
	if (nf_ip_checksum(oldskb, hook, oldskb->nh.iph->ihl * 4, IPPROTO_TCP))
		return;

	/* We need a linear, writeable skb.  We also need to expand
	   headroom in case hh_len of incoming interface < hh_len of
	   outgoing interface */
	nskb2 = skb_copy_expand(oldskb, LL_MAX_HEADER, skb_tailroom(oldskb),
			       GFP_ATOMIC);
	if (!nskb2)
		return;

	/* This packet will not be the same as the other: clear nf fields */
	nf_reset(nskb2);
	nskb2->mark = 0;
	skb_init_secmark(nskb2);

	skb_shinfo(nskb2)->gso_size = 0;
	skb_shinfo(nskb2)->gso_segs = 0;
	skb_shinfo(nskb2)->gso_type = 0;

	tcph = (struct tcphdr *)((u_int32_t*)nskb2->nh.iph + nskb2->nh.iph->ihl);

	/* Truncate to length (no data) */
	tcph->doff = sizeof(struct tcphdr)/4;
	skb_trim(nskb2, nskb2->nh.iph->ihl*4 + sizeof(struct tcphdr));
	nskb2->nh.iph->tot_len = htons(nskb2->len);

	tcph->seq = oth->seq;
	tcph->ack_seq = oth->ack_seq;    

	/* Reset flags */
	((u_int8_t *)tcph)[13] = 0;
	tcph->rst = 1;
	tcph->ack = 1;

	tcph->window = 0;
	tcph->urg_ptr = 0;

	/* Adjust TCP checksum */
	tcph->check = 0;
	tcph->check = tcp_v4_check(sizeof(struct tcphdr),
				   nskb2->nh.iph->saddr,
				   nskb2->nh.iph->daddr,
				   csum_partial((char *)tcph,
						sizeof(struct tcphdr), 0));

	/* Set DF, id = 0 */
	nskb2->nh.iph->frag_off = htons(IP_DF);
	nskb2->nh.iph->id = 0;

	addr_type = RTN_UNSPEC;
	if (hook != NF_IP_FORWARD
#ifdef CONFIG_BRIDGE_NETFILTER
	    || (nskb2->nf_bridge && nskb2->nf_bridge->mask & BRNF_BRIDGED)
#endif
	   )
		addr_type = RTN_LOCAL;

	if (ip_route_me_harder(&nskb2, addr_type))
		goto free_nskb2;

	nskb2->ip_summed = CHECKSUM_NONE;

	/* Adjust IP TTL */
	nskb2->nh.iph->ttl = MAXTTL;

	/* Adjust IP checksum */
	nskb2->nh.iph->check = 0;
	nskb2->nh.iph->check = ip_fast_csum((unsigned char *)nskb2->nh.iph,
					   nskb2->nh.iph->ihl);

	/* "Never happens" */
	if (nskb2->len > dst_mtu(nskb2->dst->path))
		goto free_nskb2;

	nf_ct_attach(nskb2, oldskb);

	NF_HOOK(PF_INET, NF_IP_LOCAL_OUT, nskb2, NULL, nskb2->dst->dev,
		dst_output);
	return;

 free_nskb2:
	kfree_skb(nskb2);
    return;

 free_nskb:
	kfree_skb(nskb);
    return;
}
#endif /* TEAMF1_CHANGES */

/* Send RST reply */
static void send_reset(struct sk_buff *oldskb, int hook)
{
	struct sk_buff *nskb;
	struct iphdr *iph = oldskb->nh.iph;
	struct tcphdr _otcph, *oth, *tcph;
	__be16 tmp_port;
	__be32 tmp_addr;
	int needs_ack;
	unsigned int addr_type;

	/* IP header checks: fragment. */
	if (oldskb->nh.iph->frag_off & htons(IP_OFFSET))
		return;

	oth = skb_header_pointer(oldskb, oldskb->nh.iph->ihl * 4,
				 sizeof(_otcph), &_otcph);
	if (oth == NULL)
		return;

	/* No RST for RST. */
	if (oth->rst)
		return;

	/* Check checksum */
	if (nf_ip_checksum(oldskb, hook, iph->ihl * 4, IPPROTO_TCP))
		return;

	/* We need a linear, writeable skb.  We also need to expand
	   headroom in case hh_len of incoming interface < hh_len of
	   outgoing interface */
	nskb = skb_copy_expand(oldskb, LL_MAX_HEADER, skb_tailroom(oldskb),
			       GFP_ATOMIC);
	if (!nskb)
		return;

	/* This packet will not be the same as the other: clear nf fields */
	nf_reset(nskb);
	nskb->mark = 0;
	skb_init_secmark(nskb);

	skb_shinfo(nskb)->gso_size = 0;
	skb_shinfo(nskb)->gso_segs = 0;
	skb_shinfo(nskb)->gso_type = 0;

	tcph = (struct tcphdr *)((u_int32_t*)nskb->nh.iph + nskb->nh.iph->ihl);

	/* Swap source and dest */
	tmp_addr = nskb->nh.iph->saddr;
	nskb->nh.iph->saddr = nskb->nh.iph->daddr;
	nskb->nh.iph->daddr = tmp_addr;
	tmp_port = tcph->source;
	tcph->source = tcph->dest;
	tcph->dest = tmp_port;

	/* Truncate to length (no data) */
	tcph->doff = sizeof(struct tcphdr)/4;
	skb_trim(nskb, nskb->nh.iph->ihl*4 + sizeof(struct tcphdr));
	nskb->nh.iph->tot_len = htons(nskb->len);

	if (tcph->ack) {
		needs_ack = 0;
		tcph->seq = oth->ack_seq;
		tcph->ack_seq = 0;
	} else {
		needs_ack = 1;
		tcph->ack_seq = htonl(ntohl(oth->seq) + oth->syn + oth->fin
				      + oldskb->len - oldskb->nh.iph->ihl*4
				      - (oth->doff<<2));
		tcph->seq = 0;
	}

	/* Reset flags */
	((u_int8_t *)tcph)[13] = 0;
	tcph->rst = 1;
	tcph->ack = needs_ack;

	tcph->window = 0;
	tcph->urg_ptr = 0;

	/* Adjust TCP checksum */
	tcph->check = 0;
	tcph->check = tcp_v4_check(sizeof(struct tcphdr),
				   nskb->nh.iph->saddr,
				   nskb->nh.iph->daddr,
				   csum_partial((char *)tcph,
						sizeof(struct tcphdr), 0));

	/* Set DF, id = 0 */
	nskb->nh.iph->frag_off = htons(IP_DF);
	nskb->nh.iph->id = 0;

	addr_type = RTN_UNSPEC;
	if (hook != NF_IP_FORWARD
#ifdef CONFIG_BRIDGE_NETFILTER
	    || (nskb->nf_bridge && nskb->nf_bridge->mask & BRNF_BRIDGED)
#endif
	   )
		addr_type = RTN_LOCAL;

	if (ip_route_me_harder(&nskb, addr_type))
		goto free_nskb;

	nskb->ip_summed = CHECKSUM_NONE;

	/* Adjust IP TTL */
	nskb->nh.iph->ttl = dst_metric(nskb->dst, RTAX_HOPLIMIT);

	/* Adjust IP checksum */
	nskb->nh.iph->check = 0;
	nskb->nh.iph->check = ip_fast_csum((unsigned char *)nskb->nh.iph,
					   nskb->nh.iph->ihl);

	/* "Never happens" */
	if (nskb->len > dst_mtu(nskb->dst))
		goto free_nskb;

	nf_ct_attach(nskb, oldskb);

	NF_HOOK(PF_INET, NF_IP_LOCAL_OUT, nskb, NULL, nskb->dst->dev,
		dst_output);
	return;

 free_nskb:
	kfree_skb(nskb);
}

static inline void send_unreach(struct sk_buff *skb_in, int code)
{
	icmp_send(skb_in, ICMP_DEST_UNREACH, code, 0);
}

static unsigned int reject(struct sk_buff **pskb,
			   const struct net_device *in,
			   const struct net_device *out,
			   unsigned int hooknum,
			   const struct xt_target *target,
			   const void *targinfo)
{
	const struct ipt_reject_info *reject = targinfo;

	/* Our naive response construction doesn't deal with IP
	   options, and probably shouldn't try. */
	if ((*pskb)->nh.iph->ihl<<2 != sizeof(struct iphdr))
		return NF_DROP;

	/* WARNING: This code causes reentry within iptables.
	   This means that the iptables jump stack is now crap.  We
	   must return an absolute verdict. --RR */
	switch (reject->with) {
	case IPT_ICMP_NET_UNREACHABLE:
		send_unreach(*pskb, ICMP_NET_UNREACH);
		break;
	case IPT_ICMP_HOST_UNREACHABLE:
		send_unreach(*pskb, ICMP_HOST_UNREACH);
		break;
	case IPT_ICMP_PROT_UNREACHABLE:
		send_unreach(*pskb, ICMP_PROT_UNREACH);
		break;
	case IPT_ICMP_PORT_UNREACHABLE:
		send_unreach(*pskb, ICMP_PORT_UNREACH);
		break;
	case IPT_ICMP_NET_PROHIBITED:
		send_unreach(*pskb, ICMP_NET_ANO);
		break;
	case IPT_ICMP_HOST_PROHIBITED:
		send_unreach(*pskb, ICMP_HOST_ANO);
		break;
	case IPT_ICMP_ADMIN_PROHIBITED:
		send_unreach(*pskb, ICMP_PKT_FILTERED);
		break;
	case IPT_TCP_RESET:
		send_reset(*pskb, hooknum);
	case IPT_ICMP_ECHOREPLY:
		/* Doesn't happen. */
		break;
#ifdef TEAMF1_CHANGES
	case IPT_HTTP_REDIRECT:
    		send_http_redirect(*pskb, hooknum);
		break;    
#endif /* TEAMF1_CHANGES */
	}

	return NF_DROP;
}

static int check(const char *tablename,
		 const void *e_void,
		 const struct xt_target *target,
		 void *targinfo,
		 unsigned int hook_mask)
{
	const struct ipt_reject_info *rejinfo = targinfo;
	const struct ipt_entry *e = e_void;

	if (rejinfo->with == IPT_ICMP_ECHOREPLY) {
		printk("REJECT: ECHOREPLY no longer supported.\n");
		return 0;
#ifdef TEAMF1_CHANGES
	} else if (rejinfo->with == IPT_TCP_RESET
	           || (rejinfo->with == IPT_HTTP_REDIRECT)) {
#else /* TEAMF1_CHANGES */
	} else if (rejinfo->with == IPT_TCP_RESET) {
#endif /* TEAMF1_CHANGES */
		/* Must specify that it's a TCP packet */
		if (e->ip.proto != IPPROTO_TCP
		    || (e->ip.invflags & XT_INV_PROTO)) {
			DEBUGP("REJECT: TCP_RESET invalid for non-tcp\n");
			return 0;
		}
	}
	return 1;
}

static struct xt_target ipt_reject_reg = {
	.name		= "REJECT",
	.family		= AF_INET,
	.target		= reject,
	.targetsize	= sizeof(struct ipt_reject_info),
	.table		= "filter",
	.hooks		= (1 << NF_IP_LOCAL_IN) | (1 << NF_IP_FORWARD) |
			  (1 << NF_IP_LOCAL_OUT),
	.checkentry	= check,
	.me		= THIS_MODULE,
};

static int __init ipt_reject_init(void)
{
	return xt_register_target(&ipt_reject_reg);
}

static void __exit ipt_reject_fini(void)
{
	xt_unregister_target(&ipt_reject_reg);
}

module_init(ipt_reject_init);
module_exit(ipt_reject_fini);
