/* 
	  This is a module which is used for time matching 
	  It is using some modified code from dietlibc (localtime() function) 
	  that you can find at http://www.fefe.de/dietlibc/ 
	  This file is distributed under the terms of the GNU General Public 
	  License (GPL). Copies of the GPL can be obtained from: ftp://prep.ai.mit.edu/pub/gnu/GPL 
	  2001-05-04 Fabrice MARIE <fabrice@netfilter.org> : initial development. 
	  2001-21-05 Fabrice MARIE <fabrice@netfilter.org> : bug fix in the match code, 
	     thanks to "Zeng Yu" <zengy@capitel.com.cn> for bug report. 
	  2001-26-09 Fabrice MARIE <fabrice@netfilter.org> : force the match to be in LOCAL_IN or PRE_ROUTING only. 
	  2001-30-11 Fabrice : added the possibility to use the match in FORWARD/OUTPUT with a little hack, 
	     added Nguyen Dang Phuoc Dong <dongnd@tlnet.com.vn> patch to support timezones. 
	  2004-05-02 Fabrice : added support for date matching, from an idea of Fabien COELHO. 
	*/ 
	 
	#include <linux/module.h> 
	#include <linux/skbuff.h> 
	#include <linux/netfilter_ipv4/ip_tables.h> 
	#include <linux/netfilter_ipv4/ipt_time.h> 
	#include <linux/time.h> 
	 
	MODULE_AUTHOR("Fabrice MARIE <fabrice@netfilter.org>"); 
	MODULE_DESCRIPTION("Match arrival timestamp/date"); 
	MODULE_LICENSE("GPL"); 
	int tf1_tz_diff  ;   /* This variable stores the difference in timezone */



#if 0
#define DEBUGP(format, args...) printk(KERN_INFO "ipt_TIME: " format, ## args)
#else
#define DEBUGP(format, args...)
#endif

static u_int8_t prevOrNextDayGet( u_int8_t indexOfCurrentDay ,
                                  u_int8_t nextOrPrevDayValue );

static u_int8_t checkIfPacketIsScheduled (u_int16_t  startTime,
                                          u_int16_t  endTime,
                                          u_int16_t  currTime,
                                          u_int8_t   dayInPacket,
                                          u_int8_t   daysMatch,
                                          u_int8_t   newDaysMatch);
static u_int8_t newDaysMatchGet (u_int8_t daysMatch,u_int16_t startTime,
                                 u_int16_t endTime);
 
	struct tm 
	{ 
	        int tm_sec;                   /* Seconds.     [0-60] (1 leap second) */ 
	        int tm_min;                   /* Minutes.     [0-59] */ 
	        int tm_hour;                  /* Hours.       [0-23] */ 
	        int tm_mday;                  /* Day.         [1-31] */ 
	        int tm_mon;                   /* Month.       [0-11] */ 
	        int tm_year;                  /* Year - 1900.  */ 
	        int tm_wday;                  /* Day of week. [0-6] */ 
	        int tm_yday;                  /* Days in year.[0-365] */ 
	        int tm_isdst;                 /* DST.         [-1/0/1]*/ 
	 
	        long int tm_gmtoff;           /* we don't care, we count from GMT */ 
	        const char *tm_zone;          /* we don't care, we count from GMT */ 
	}; 
	 
	void 
	localtime(const u32 time, struct tm *r); 
	 
	static int 
	match(const struct sk_buff *skb, 
	      const struct net_device *in, 
	      const struct net_device *out, 
	      const struct xt_match *match, 
	      const void *matchinfo, 
	      int offset, 
	      unsigned int protoff, 
	      int *hotdrop) 
	{ 
	        const struct ipt_time_info *info = matchinfo;   /* match info for rule */ 
	        struct tm currenttime;                          /* time human readable */ 
                u_int8_t days_of_week[7] = {
                                IPT_SUNDAY, IPT_MONDAY, IPT_TUESDAY,
                                IPT_WEDNESDAY, IPT_THRUSDAY, IPT_FRIDAY, 
                                IPT_SATURDAY
                                };

	        u_int16_t packet_time; 
	        
                 /* variable to store convertd local time from UTC time in minutes */
        	int localTimeInMin ;
        	/* variable to store the prev or next day as in days_of_week[] */
        	u_int8_t  localCurrentDay ;
        	/*  variable to store the bits of days including the next day bit */
        	u_int8_t  newDaysMatch = 0 ;
              

	        /* We might not have a timestamp, get one */ 
	        if (skb->tstamp.off_sec == 0) 
	                __net_timestamp((struct sk_buff *)skb); 
#if 0	 
	        /* First we make sure we are in the date start-stop boundaries */ 
	        if ((skb->tstamp.off_sec < info->date_start) || (skb->tstamp.off_sec > info->date_stop)) 
	                return 0; /* We are outside the date boundaries */ 
#endif
	 
	        /* Transform the timestamp of the packet, in a human readable form */ 
	        localtime(skb->tstamp.off_sec, &currenttime); 

                /*commented below condition as we are not checking with updated days,
                 because of this it is returning from here.we are checking whether 
                 the day's are matching or not with updated days in the following code(line#156)*/
#if 0	 
	        /* check if we match this timestamp, we start by the days... */ 
	        if ((days_of_week[currenttime.tm_wday] & info->days_match) != days_of_week[currenttime.tm_wday]) 
	                return 0; /* the day doesn't match */ 
#endif
	 
	        /* ... check the time now */ 
	        packet_time = (currenttime.tm_hour * 60) + currenttime.tm_min; 
	       /* if ((packet_time < info->time_start) || (packet_time > info->time_stop)) 
	                return 0;*/
               /* Will store the local time in minutes */
     		localTimeInMin = packet_time - tf1_tz_diff ;

     		/* Check if local time is greater than 1440 */
     		if (localTimeInMin >  FW_MINUTES_IN_DAY)
        	{
        	localCurrentDay = prevOrNextDayGet(currenttime.tm_wday,IPT_NEXT_DAY );
        	if (!localCurrentDay)
                 	return 0;

          	/* get the modified time after changing UTC time to local time*/
          	localTimeInMin = localTimeInMin - FW_MINUTES_IN_DAY;
        	}
    		else if (localTimeInMin < 0)
           	{
            	localCurrentDay = prevOrNextDayGet(currenttime.tm_wday,
                                                IPT_PREV_DAY );
             	if (!localCurrentDay)
                	return 0;

             	/* get the modified time after changing UTC time to local time*/
             	localTimeInMin = FW_MINUTES_IN_DAY + localTimeInMin;
           	}
        	else
           	{
                 /* 
               * current day will not change and localCurrentDay will have
               * the same index as mentioned in packet. 
                */
              	localCurrentDay = days_of_week[currenttime.tm_wday] ;
           	}

    		/* conversion of newDaysMatch <- info->days_match */
    		newDaysMatch =  newDaysMatchGet(info->days_match,info->time_start,
                                    info->time_stop );

                 /* check if we match this timestamp, we start by the days... */
    		if ((localCurrentDay & newDaysMatch )
        		!= localCurrentDay)
       		{
         	DEBUGP("we are not in the days boundaries \n");
         	return 0; /* the day doesn't match */
       		}

    		/* start and end time has not been specified */
    		if ((!info->time_stop) && (!info->time_start))
        		return 1;

    		/* end time is 00:00 */
    		if (!info->time_stop) {
        		if (localTimeInMin < info->time_start)
            			return 0;
        		else
            			return 1;
    			}

    		/* start time is 00:00 */
    		if (!info->time_start) {
        		if (localTimeInMin > info->time_stop)
            			return 0;
        		else
            			return 1;
    		}

 
            	DEBUGP ("packettime: %d, time start: %d, time stop: %d \n",
            	localTimeInMin, info->time_start, info->time_stop);

    		if (info->time_start > info->time_stop) {

         		if (checkIfPacketIsScheduled (info->time_start,
                                       info->time_stop,localTimeInMin,
                                       localCurrentDay, info->days_match,
                                       newDaysMatch) == 0)
                    		return 0;

        		} else
             		{
        		/* start and end time are in the same day */
        		if ((localTimeInMin < info->time_start) ||
            		(localTimeInMin > info->time_stop)) {
            		DEBUGP("we are not in the time boundaries \n");
            		return 0;
        		}
    		}	
 
	 
	        /* here we match ! */ 
	        return 1; 
	} 
	 
	static int 
	checkentry(const char *tablename, 
	           const void *ip, 
	           const struct xt_match *match, 
	           void *matchinfo, 
	           unsigned int hook_mask) 
	{ 
	        struct ipt_time_info *info = matchinfo;   /* match info for rule */ 
	 
	        /* First, check that we are in the correct hooks */ 
	        if (hook_mask 
	            & ~((1 << NF_IP_PRE_ROUTING) | (1 << NF_IP_LOCAL_IN) | (1 << NF_IP_FORWARD) | (1 << NF_IP_LOCAL_OUT))) 
	        { 
	                printk("ipt_time: error, only valid for PRE_ROUTING, LOCAL_IN, FORWARD and OUTPUT)\n"); 
	                return 0; 
	        } 
	 
	        /* Now check the coherence of the data ... */ 
	        if ((info->time_start > 1439) ||        /* 23*60+59 = 1439*/ 
	            (info->time_stop  > 1439)) 
	        { 
	                printk(KERN_WARNING "ipt_time: invalid argument\n"); 
	                return 0; 
	        } 
	 
	        return 1; 
	} 
	 
	static struct ipt_match time_match = { 
	        .name = "time", 
		.family = AF_INET,
	        .match = &match, 
	        .matchsize = sizeof(struct ipt_time_info), 
	        .checkentry = &checkentry, 
	        .me = THIS_MODULE 
	}; 
	 
	static int __init init(void) 
	{ 
	        printk("ipt_time loading\n"); 
	        return xt_register_match(&time_match); 
	} 
	 
	static void __exit fini(void) 
	{ 
	        xt_unregister_match(&time_match); 
	        printk("ipt_time unloaded\n"); 
	} 
	 
	module_init(init); 
	module_exit(fini); 
	 
	 
	/* The part below is borowed and modified from dietlibc */ 
	 
	/* seconds per day */ 
	#define SPD 24*60*60 
	 
	void 
	localtime(const u32 time, struct tm *r) { 
	        u32 i, timep; 
	        extern struct timezone sys_tz; 
	        const unsigned int __spm[12] = 
	                { 0, 
	                  (31), 
	                  (31+28), 
	                  (31+28+31), 
	                  (31+28+31+30), 
	                  (31+28+31+30+31), 
	                  (31+28+31+30+31+30), 
	                  (31+28+31+30+31+30+31), 
	                  (31+28+31+30+31+30+31+31), 
	                  (31+28+31+30+31+30+31+31+30), 
	                  (31+28+31+30+31+30+31+31+30+31), 
	                  (31+28+31+30+31+30+31+31+30+31+30), 
	                }; 
	        register u32 work; 
	 
	        timep = time - (sys_tz.tz_minuteswest * 60); 
	        work=timep%(SPD); 
	        r->tm_sec=work%60; work/=60; 
	        r->tm_min=work%60; r->tm_hour=work/60; 
	        work=timep/(SPD); 
	        r->tm_wday=(4+work)%7; 
	        for (i=1970; ; ++i) { 
	                register time_t k= (!(i%4) && ((i%100) || !(i%400)))?366:365; 
	                if (work>k) 
	                        work-=k; 
	                else 
	                        break; 
	        } 
	        r->tm_year=i-1900; 
	        for (i=11; i && __spm[i]>work; --i) ; 
	        r->tm_mon=i; 
	        r->tm_mday=work-__spm[i]+1; 
	} 
/*******************************************************************************
*  prevOrNextDayGet - get the index of prev or next day
* 
*  This function takes index of current day in the array days_of_week[]. Will
*  get the index of next or previous day by checking the value of
*  nextOrPrevDayValue varaible
*
*  RETURNS: index of next or prev day in days_of_week[] otherwise 0
* 
*/

static u_int8_t prevOrNextDayGet 
    (
    u_int8_t indexOfCurrentDay , /* Index value of current day in
                                    days_of_week[] */ 
    u_int8_t nextOrPrevDayValue   /* Can be  0 or 1 value  */
    ) 
    {
    u_int8_t days_of_week[7] = {
        IPT_SUNDAY, IPT_MONDAY, IPT_TUESDAY,
        IPT_WEDNESDAY, IPT_THRUSDAY, IPT_FRIDAY, IPT_SATURDAY
    };

    u_int8_t otherDay = 0;

    switch (nextOrPrevDayValue)
        {
        case IPT_NEXT_DAY:
            {
                if ( days_of_week[indexOfCurrentDay ] == IPT_SATURDAY)
                    otherDay = IPT_SUNDAY;
                else
                    otherDay = days_of_week[indexOfCurrentDay ]>> 1;
                break; 
            }
        case IPT_PREV_DAY:
            {
                if ( days_of_week[indexOfCurrentDay ] == IPT_SUNDAY)
                    otherDay = IPT_SATURDAY;
                else
                    otherDay = days_of_week[indexOfCurrentDay ] << 1;
                break;
            } 
        default:
            {
                printk(KERN_WARNING "ipt_time: IPT_DAY didn't matched\n");
                return 0;
            }
        } /* end of switch */ 

    return otherDay;  
    } /* end of  prevOrNextDayGet */ 



/*******************************************************************************
* checkIfPacketIsScheduled - check if the packet is scheduled.
* 
* This function checks if the <currTime> of the packet falls in the schedule
* which is defined by the <startTime>, <endTime> and <daysMatch>. This 
* function gives consideration to overnight schedules, 
*
* For Example: 
* startTime is 9 PM (Tuesday) and endTime is 3 AM (wednesday).
* 
* RETURNS: 1 - OK, the value of current and other day are set.
*          0 - ERROR
*/
static u_int8_t checkIfPacketIsScheduled
    (
    u_int16_t        startTime,      /* start time of schedule in the day */ 
    u_int16_t        endTime,        /* end time of schedule in the day */  
    u_int16_t        currTime,       /* current time */ 
    u_int8_t         dayInPacket,    /* value of day in packet */
    u_int8_t         daysMatch,      /* days given from user space*/
    u_int8_t         newDaysMatch    /* modified days acc to timezone*/
    ) 
    { 
    u_int8_t days_of_week[7] = {
        IPT_SUNDAY, IPT_MONDAY, IPT_TUESDAY,
        IPT_WEDNESDAY, IPT_THRUSDAY, IPT_FRIDAY, IPT_SATURDAY
    };
    u_int8_t       count = 0;
    u_int8_t       yesterday=0;
    u_int8_t       tommorrow=0;
    u_int8_t       schedOverLap=0;    
    u_int8_t       today=0;   
    u_int8_t       weekday=0;
   
    while (count < 7)
        {
        weekday = days_of_week[count];

        /* check if the weekday is today */
        if (weekday != dayInPacket)
	  {
//    printk(KERN_INFO "SAN [%d:%s] \n",__LINE__,__FILE__);
            count ++;
            continue;
          }

        today = weekday;

        /* check if the schedule is valid on this day of the week */ 
        if (!(today & newDaysMatch))
           {
    //printk(KERN_INFO "SAN [%d:%s] \n",__LINE__,__FILE__);
            count ++ ;
            continue;
           }
    
        /*
         * The schedule is valid today. In case
         * of an overnight schedule, the schdule could cross day boundaries
         * (i.e The start time of the schedule is today and the end time
         * is tomorrow).check if tomorrow has also been configured by the
         * user. For that we need the calculate what day is tomorrow.
         *
         * A overnight schedule is said to be overlapped if in a day
         * a previous day schedule ends and a new schedule starts.
         * (i.e endTime < startTime)
         */

        /* calculate what day is it tommorrow */
         if (today == IPT_SATURDAY)
             tommorrow = IPT_SUNDAY;
         else        
             tommorrow = days_of_week[count] >> 1;

         /* calculate what day was it yesterday */
         if (today == IPT_SUNDAY)
             yesterday = IPT_SATURDAY;
         else
             yesterday = days_of_week[count] << 1;

         if ((today & daysMatch) && (yesterday & daysMatch))
             schedOverLap = 1;   
         
        if (schedOverLap)
            {
	      if ((currTime <= endTime) || (currTime >= startTime))    
                 return 1;
            }
        else
            {
            /* This is not a case of overlapped schedule */

            /* check if the schedule has started today */     
            if (today & daysMatch)
                {
                if (currTime >= startTime)    
                    return 1;
                }    
            else
                {
                if (currTime <= endTime)
                    return 1;
                }    
            }

        count ++ ;
        } /* End While*/

    return 0;
    }


/*******************************************************************************
* newDaysMatchGet - get the newDaysMatch value.
* 
* This function get the value for newDaysMatch. This variable contains the days
* given by user and there corresponding next days in week.
*
* RETURNS: value in newDaysMatch 
*
*/
static u_int8_t newDaysMatchGet
     (
      u_int8_t       daysMatch,
      u_int16_t      startTime,
      u_int16_t      endTime
     )
     { 
     u_int8_t days_of_week[7] = {
        IPT_SUNDAY, IPT_MONDAY, IPT_TUESDAY,
        IPT_WEDNESDAY, IPT_THRUSDAY, IPT_FRIDAY, IPT_SATURDAY
        };
     u_int8_t      count = 0;
     u_int8_t      weekday=0;
     u_int8_t      newDaysMatch =0 ;

     /* when it is normal schedule */
     if ( startTime < endTime ) 
        return daysMatch ;
       
     /* when it is overnight schedule */
     while (count < 7)
       { 
         weekday = days_of_week[count];
	 if (!( weekday & daysMatch ))
	    {
	      count ++;
              continue ; 
            }

         if (weekday == IPT_SATURDAY)
	   newDaysMatch |= (daysMatch | IPT_SUNDAY);
         else        
	   newDaysMatch |= (daysMatch | (weekday >> 1));

         count ++;
       } /* end of while loop */

       return newDaysMatch;
     } /* end of newDaysMatch*/ 
     
