/****************************************************************************/
/*                                                                          */
/*                                                                          */
/*                        Copyright 2006 - 2010                             */
/*                    One Convergence Devices Pvt Ltd                       */
/*                         ALL RIGHTS RESERVED                              */
/*                                                                          */
/* Permission is hereby granted to licensees of One Convergence Devices Pvt */
/* Ltd' products to use or abstract this computer program for the sole      */
/* purpose of implementing a product based on One Convergence Devices Pvt   */
/* Ltd products. No other rights to reproduce, use, or disseminate this     */
/* computer program, whether in part or in whole, are granted.              */
/****************************************************************************/

/****************************************************************************
*  File: sip_ct_helpers.c
*
*  Description: File contains sipalg module definition and declarations
*
*  Author: sriphaneesh.n@oneconvergence.com
*
*  Revision History:
*
*  01a,04Aug09,rra added support for SIP over VPN by adding IPSEC mark.
*
*****************************************************************************/

#ifdef DEBUG
#define DEBUGP(format, args...) printk(KERN_WARNING format, ## args)
#else
#define DEBUGP(format, args...)
#endif

#define SIPALG_CFG

#define SUCCESS                0
#define FAILURE               -1
#define TRUE                   1
#define FALSE                  0

#define SIP_CT_PORT            5060

#define SIP_CT_OUTBOUND        0
#define SIP_CT_INBOUND         1

#define SIP_CT_EXP_PERM        0x1
#define SIP_CT_EXP_SIP         0x2
#define SIP_CT_EXP_RTP         0x4
#define SIP_CT_EXP_RTCP        0x8

#define SIP_PORTMAP_VIA        0
#define SIP_PORTMAP_CONTACT    1
#define SIP_PORTMAP_RTP        2

#define SIP_REQUEST            1
#define SIP_STATUS             2
#define SIP_KEEPALIVE          3

#define SIP_CT_MAX_HDRS       20

//#define SIP_CT_MAX_PORT_MAPS   3
//#define SIP_CT_MAX_EXPS       10
//#define SIP_CT_MASTER_TIMEOUT   180
//#define SIP_CT_CLOSE_TIMEOUT     31

#define SIP_CT_TIMER_TIMEOUT     30
#define SIP_CT_DESTROY_DELAY      (1+SIP_CT_TIMER_TIMEOUT)
#define SIP_CT_FORCED_DEATH       2
#define SIP_CT_DEFAULT_TIMEOUT   91
#define SIP_CT_INCALL_TIMEOUT   181
#define SIP_CT_HELD_TIMEOUT       (2*SIP_CT_INCALL_TIMEOUT)

#define FW_IPSEC_MARK 0xf1dead

/* The following list of headers are our primary concern with respect to
 * conntrack and nat helpers.
 */

enum sip_headers_e
{
    SIP_REQUEST_STATUS=1,
    SIP_VIA_HDR,
    SIP_CONTACT_HDR,
    SIP_FROM_HDR,
    SIP_TO_HDR,
    SIP_RROUTE_HDR,
    SIP_ROUTE_HDR,
    SIP_CALLID_HDR,
    SIP_CSEQ_HDR,
    SIP_CONTENT_LEN,
    SIP_CONTENT_TYPE,
    SIP_EXPIRE_TIME,
    SIP_SDP_CONTENT,
    SIP_SDP_MEDIADESCR,
    SIP_SDP_CONNINFO,
    SIP_SDP_ORIGININFO,
    SIP_UNKNOWN_HDR
};


/* The following is the list of sip requests we are interested
 */
/* TF1: Added OPTIONS message type support */
enum sip_reuest_e
{
    SIP_INVITE = 1,
    SIP_ACK,
    SIP_BYE,
    SIP_REGISTER,
    SIP_SUBSCRIBE,
    SIP_NOTIFY,
    SIP_OPTIONS,
    SIP_CANCEL,
    SIP_IGNORE
};

enum sip_ct_states_e
{
    SIP_CT_STATE_NONE=0,
    SIP_CT_STATE_REGISTERING,
    SIP_CT_STATE_INVITING,
    SIP_CT_STATE_REGISTERED,
    SIP_CT_STATE_INCALL,
    SIP_CT_STATE_CLOSING,
    SIP_CT_STATE_CLOSED,
    SIP_CT_STATE_HOLDING,
    SIP_CT_STATE_HELD
};

enum sip_ct_events_e
{
    SIP_CT_EVENT_IGNORE=0,
    SIP_CT_EVENT_REGISTER,
    SIP_CT_EVENT_UNREGISTER,
    SIP_CT_EVENT_INVITE,
    SIP_CT_EVENT_INCALL,
    SIP_CT_EVENT_OK,
    SIP_CT_EVENT_ACK,
    SIP_CT_EVENT_HOLD,
    SIP_CT_EVENT_CLOSE,
    SIP_CT_EVENT_REDIRECT,
    SIP_CT_EVENT_FAILURE,
    SIP_CT_EVENT_REQUEST
};


#define SET_BIT(ibit, ibitmap)    ibitmap = (ibitmap | (1 << (ibit-1)))
#define CLEAR_BIT(ibit, ibitmap)  ibitmap = (ibitmap & ~(1 << (ibit-1)))
#define CHECK_BIT(ibit, ibitmap)  (ibitmap & (1 << (ibit-1)))


#define SIP_CT_CALLID_LEN     64
#define SIP_CT_MAX_CALLS      50 
#define SIP_CT_EXPS_PERCALL    6
#define SIP_CT_MAX_EXPS        (SIP_CT_MAX_CALLS * SIP_CT_EXPS_PERCALL)


typedef struct
{
    char    callid[SIP_CT_CALLID_LEN+1];
    int     cidlen;
    int     method;
    int     state;
    int     lan2lan;
    int     itimeout;
    int     keepalive;
    int     rtpactive;
} sip_call_info_t;

typedef struct
{
    int    flags;
    int    usedby;
    struct ip_conntrack_expect *exp1;
    struct ip_conntrack_tuple  origtuple;
    u_int16_t                  natport;
} sip_exp_info_t;


/* This structure holds state information with respect to call/session/client.
 * This is going to be linked with conntrack enty.
 */

typedef struct
{
    int               keepalive;
    struct timer_list timeout;
    u_int32_t         natip;
    u_int32_t         origip;
    u_int8_t          proto;
    int               initdir;
    int               initmethod;
    int               rtpactive;

    sip_exp_info_t    explist[SIP_CT_MAX_EXPS];
    sip_call_info_t   calllist[SIP_CT_MAX_CALLS];
} sip_ct_info_t;


/* Structure to store list of headers we are interested to process in any sip
 * message.
 */

typedef struct
{
        int      hdrtype;
        int      hdrlen;
        char    *hdrptr;
} sip_hdr_list_t;


/* We keep lot of useful sip message info in this struct, which is gathered in
 * various functions and will be used by other functions while processing msg.
 */

typedef struct 
{
    int8_t    msgtype;        //Request or Status?
    int8_t    method;         //Register or Invite or etc?
    int8_t    hdrcount;       //Number of headers in our hdrlist
    int8_t    conttype;       //SDP or etc?
    int8_t    revent;         //Notify special event to fsm
    int8_t    rrflag;         //Message with record-route? 
    int8_t    msgdir;         //Outbound or Inbound?
    int8_t    callid;         //Callid within alginfo
    int8_t    proto;          //Message transport proto
    int8_t    sdplendiff;     //SDP len diff after mangle
    int8_t    msglendiff;     //Message len diff after mangle

    enum ip_conntrack_info ctinfo;    //Message dir w.r.t to Netfilter
    struct ip_conntrack    *ct;       //Conntrack of this message 
    struct ip_conntrack    *master;   //Master conntrack for this message
    struct sk_buff         **pskb;    //Packet skbuffer
    struct iphdr           *iph;      //IP header pointer
    struct udphdr          *udph;     //UDP header pointer
    struct tcphdr          *tcph;     //TCP header pointer
    char                   *msgptr;   //Pointer to the sip message
    char                   *sdpptr;   //Pointer to the sdp content

    int16_t    msglen;
    int16_t    sdplen;
    int16_t    expires;
    int16_t    status;         //Status code
    u_int16_t  rtcpnatport;

    sip_hdr_list_t hdrlist[SIP_CT_MAX_HDRS];
} sip_msg_info_t;


/* Structure to store required info while parsing different headers.
 *
 */

typedef struct
{
    char        *ip1begin;
    char        *ip1end;
    char        *ip2begin;
    char        *ip2end;
    char        *port1begin;
    char        *port1end;
    u_int32_t    ipaddr1;
    u_int32_t    ipaddr2;
    u_int16_t    port1;
    u_int16_t    expires;
    u_int8_t     proto1;
} sip_hdr_info_t;


static inline int sip_ct_tuple_cmp(const struct ip_conntrack_tuple *tuple1,
                                   const struct ip_conntrack_tuple *tuple2)
{
    return ((tuple1->src.ip       == tuple2->src.ip) &&
            (tuple1->src.u.all    == tuple2->src.u.all) &&
            (tuple1->dst.ip       == tuple2->dst.ip) &&
            (tuple1->dst.u.all    == tuple2->dst.u.all) &&
            (tuple1->dst.protonum == tuple2->dst.protonum));
}


/* These functions are defines in sip_ct_parser.c */

char *my_strtok(char *, char *, char *, char **, char **, char **);

int sip_ct_parse_msg(char *, int , sip_msg_info_t *);
int sip_ct_parse_sdp(sip_msg_info_t *, sip_hdr_info_t *);
int sip_ct_parse_callid(sip_msg_info_t *, char **);
int sip_ct_parse_request_line(char *, sip_hdr_info_t *);
int sip_ct_parse_via_hdr(char *, sip_hdr_info_t *);
int sip_ct_parse_contact_hdr(char *, sip_hdr_info_t *);
int sip_ct_parse_rroute_hdr(char *, sip_hdr_info_t *);

/* These functions are defines in sip_nat_helpers.c */

int sip_nat_mangle_hdr(int,sip_hdr_info_t *,sip_msg_info_t *,sip_ct_info_t *);
int sip_nat_mangle_sdp(sip_hdr_info_t *, sip_msg_info_t *, sip_ct_info_t *,
                       struct ip_conntrack_expect *);
int sip_nat_mangle_content_len(sip_msg_info_t *, int);
void sip_nat_update_portmap(sip_ct_info_t *,struct ip_conntrack_expect *,int);
int sip_nat_create_expect(int, struct ip_conntrack_expect *,
                                      sip_exp_info_t *, sip_msg_info_t *,
                                      sip_ct_info_t *);

/* These functions are defines in sip_ct_expect.c */

sip_exp_info_t *sip_ct_create_expect(int, int, struct ip_conntrack_tuple *,
                                     struct ip_conntrack_tuple *,
                                     sip_msg_info_t *, sip_ct_info_t *);
int sip_ct_refresh_expect(struct ip_conntrack_expect *);
int sip_ct_put_expect(struct ip_conntrack_expect *, sip_ct_info_t *);
sip_exp_info_t *sip_ct_get_expinfo(struct ip_conntrack_expect *exp1,
                                   sip_ct_info_t *alginfo);

/* These functions are defines in sip_ct_callfsm.c */

void sip_ct_idletimer(unsigned long ct);
int sip_ct_validate_call(sip_msg_info_t *msginfo, sip_ct_info_t *alginfo);
void sip_ct_update_callstate(sip_msg_info_t *msginfo, sip_ct_info_t *alginfo);

/* These functions are defined in linux kernel */

extern int ip_nat_mangle_packet(struct sk_buff **, struct ip_conntrack *,
                                enum ip_conntrack_info, unsigned int,
                                unsigned int, const char *, unsigned int);
extern void ip_nat_mangle_tcp_sequence(struct sk_buff **, struct ip_conntrack *,
                                       enum ip_conntrack_info, int);

extern void ip_conntrack_unexpect_related2(struct ip_conntrack_expect *exp);

int sip_ct_tcp_helper(struct sk_buff **pskb,
                      struct ip_conntrack *ct,
                      enum ip_conntrack_info ctinfo);
int sip_ct_udp_helper(struct sk_buff **pskb,
                      struct ip_conntrack *ct,
                      enum ip_conntrack_info ctinfo);
