/* Copyright (c) 2008, TeamF1, Inc. */

/*
modification history
--------------------
* 01b,12oct09,grj Added sip support to VLANs
* 01a,10sep08,sat Added Hardware offload support
*/

/****************************************************************************/
/*                                                                          */
/*                                                                          */
/*                        Copyright 2006 - 2010                             */
/*                    One Convergence Devices Pvt Ltd                       */
/*                         ALL RIGHTS RESERVED                              */
/*                                                                          */
/* Permission is hereby granted to licensees of One Convergence Devices Pvt */
/* Ltd' products to use or abstract this computer program for the sole      */
/* purpose of implementing a product based on One Convergence Devices Pvt   */
/* Ltd products. No other rights to reproduce, use, or disseminate this     */
/* computer program, whether in part or in whole, are granted.              */
/****************************************************************************/

/****************************************************************************
*  File: sipalg_cfg.c
*
*  Description: File contains sipalg module functions for Linux Netfilter
*
*  Author: sriphaneesh.n@oneconvergence.com
*
*  Revision History:
*
*****************************************************************************/


#include <linux/config.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/ip.h>
#include <linux/proc_fs.h>
#include <linux/seq_file.h>
#include <net/tcp.h>
#include <net/udp.h>

#include <linux/netfilter.h>
#include <linux/netfilter_ipv4/ip_conntrack.h>
#include <linux/netfilter_ipv4/ip_conntrack_core.h>
#include <linux/netfilter_ipv4/ip_conntrack_helper.h>
#include <linux/netfilter_ipv4/ip_nat_helper.h>

//#define DEBUG
#include "sip_ct_helpers.h"
#include "sipalg_cfg.h"

#ifdef SIPALG_CFG

extern u_int8_t   rmmod;

extern atomic_t sip_session_count;

extern struct ip_conntrack_helper *psip_ct_tcp_helpers;
extern struct ip_conntrack_helper *psip_ct_udp_helpers;
extern struct ip_conntrack_helper sip_ct_rtp_helpers;
extern int    lan_netaddr, lan_netmask;
extern unsigned short int    sig_dscp, rtp_dscp;

static sipalg_cfg_t sipalg_cfg;
static struct ip_conntrack_helper sip_ct_helpers[SIPALG_MAX_PORTS*2];
static int reg_ports;

static struct proc_dir_entry * pSipCfgProcDir = NULL;
static struct proc_dir_entry * pSipCfgProcFile = NULL;

/* major number of device */
static int major;

DEFINE_RWLOCK (sipAlgCfgEntryTblLock);
static SIP_NODE_ENTRY_HEAD * pSipNodeEntryTbl;

/*---------------------------------------------------------------------------*/


static inline int
port_check(struct ip_conntrack *i, void *port)
{
	int ret = 0;
    SIP_READ_LOCK (&sipAlgCfgEntryTblLock);

	if((ntohs(i->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.u.udp.port) == 
            (int)(short)port) ||
       (ntohs(i->tuplehash[IP_CT_DIR_REPLY].tuple.src.u.udp.port) == 
            (int)(short)port))        
        ret = 1;

    SIP_READ_UNLOCK(&sipAlgCfgEntryTblLock);	

	return ret;
}


static void sipalg_stop(void)
{
    int i;

    psip_ct_tcp_helpers = NULL;
    psip_ct_udp_helpers = NULL;

    rmmod = TRUE;
    
    do
    {
        schedule_timeout(SIP_CT_DESTROY_DELAY*HZ);
    } while (atomic_read(&sip_session_count));

    
    for( i = 0; i < reg_ports; i++ )
    {
#ifdef CONFIG_TF1_HW_OFFLOAD
        if(sip_ct_helpers[i].tuple.dst.protonum == IPPROTO_TCP)
        hwOffloadUnregister(sip_ct_helpers[i].tuple.src.u.tcp.port,0);
        else if(sip_ct_helpers[i].tuple.dst.protonum == IPPROTO_UDP)
        hwOffloadUnregister(sip_ct_helpers[i].tuple.src.u.udp.port,0);
#endif /*CONFIG_TF1_HW_OFFLOAD*/
        ip_conntrack_helper_unregister(&sip_ct_helpers[i]);
    }

    /* RTP helper is not registered but media connections are pointing to this
       helper. Work-around to remove RTP helper pointers from media cts */

    if (!ip_conntrack_helper_register(&sip_ct_rtp_helpers))
        ip_conntrack_helper_unregister(&sip_ct_rtp_helpers);
    
    /*Delete all stale conntrack entries on sip port(if any)*/
    ip_ct_iterate_cleanup(port_check, SIP_CT_PORT); 

    reg_ports = 0;

    return;
}


static int register_helpers( unsigned short *ports,
                             int num_ports,
                             int proto )
{
    int i;
    int ret;


    for( i = 0; i < num_ports; i++, reg_ports++ )
    {
        memset( &sip_ct_helpers[reg_ports], 0,
                sizeof( struct ip_conntrack_helper ) );

        sip_ct_helpers[reg_ports].name         = "SIPALG";
        sip_ct_helpers[reg_ports].me           = THIS_MODULE;
        sip_ct_helpers[reg_ports].max_expected = SIP_CT_MAX_EXPS;
        sip_ct_helpers[reg_ports].timeout      = SIP_CT_DEFAULT_TIMEOUT;

        sip_ct_helpers[reg_ports].tuple.dst.protonum = proto;
        sip_ct_helpers[reg_ports].mask.dst.protonum  = 0xff;

        if (proto == IPPROTO_TCP)
        {
            sip_ct_helpers[reg_ports].tuple.src.u.tcp.port =
                __constant_htons(ports[i]);
            sip_ct_helpers[reg_ports].mask.src.u.tcp.port = 0xffff;
            sip_ct_helpers[reg_ports].help  = sip_ct_tcp_helper;
#ifdef CONFIG_TF1_HW_OFFLOAD
            hwOffloadRegister(htons(ports[i]),0);
#endif /* CONFIG_TF1_HW_OFFLOAD */

            if (!psip_ct_tcp_helpers)
                psip_ct_tcp_helpers = &sip_ct_helpers[reg_ports]; 
        }
        else
        {
            sip_ct_helpers[reg_ports].tuple.src.u.udp.port =
                __constant_htons(ports[i]);
            sip_ct_helpers[reg_ports].mask.src.u.udp.port = 0xffff;
            sip_ct_helpers[reg_ports].help  = sip_ct_udp_helper;

            if (!psip_ct_udp_helpers)
                psip_ct_udp_helpers = &sip_ct_helpers[reg_ports];
        }

        ret = ip_conntrack_helper_register(&sip_ct_helpers[reg_ports]);

        if( ret )
        {
            DEBUGP("SIPALG: Failed to register helper\n");
#ifdef CONFIG_TF1_HW_OFFLOAD
            hwOffloadUnregister(htons(ports[i]),0);
#endif /* CONFIG_TF1_HW_OFFLOAD */
            break;
        }
    }

    return( ret );
}


static int sipalg_start(void)
{
    int    ret;

   if ( reg_ports )
    {
        DEBUGP("SIPALG: Already running\n");
        return( 0 );
    }

    ret = register_helpers( sipalg_cfg.udp_ports, sipalg_cfg.num_udpports,
                            IPPROTO_UDP );
    if( ret )
    {
        goto lerror;
    }

    ret = register_helpers( sipalg_cfg.tcp_ports, sipalg_cfg.num_tcpports,
                            IPPROTO_TCP );
    if( ret )
    {
        goto lerror;
    }

    rmmod = FALSE;

    return( 0 );

lerror:
    sipalg_stop();
    return( ret );
}


int sipalg_ioctl( struct inode *inode,
                  struct file *pfile,
                  unsigned int cmd,
                  unsigned long arg )
{
    int rval = 0;

    sipalg_cfg_t *pcfg;


    switch( cmd )
    {
    case SIPALG_SET_CFG:

        pcfg = (sipalg_cfg_t*) arg;        

        if (((!pcfg->num_tcpports) && (!pcfg->num_udpports)) ||
            (!pcfg->lan_netaddr) || (!pcfg->lan_netaddr))
        {
            DEBUGP("SIPALG: Mandatory arguments are missing\n");
            rval = -1;
            break;
        }

        memcpy( &sipalg_cfg, ( void * )arg, sizeof( sipalg_cfg ) );
        
        lan_netaddr = sipalg_cfg.lan_netaddr;
        lan_netmask = sipalg_cfg.lan_netmask;

        DEBUGP (KERN_CRIT "type of operation %d zone ID %d \n", 
                sipalg_cfg.operation, sipalg_cfg.zone);

        /* check operation and add/delete accordingly */
        if (sipalg_cfg.operation == SIP_CONF_ADD)
        {
            sipAlgCfgEntryAdd (sipalg_cfg.lan_netaddr, sipalg_cfg.lan_netmask, 
                    sipalg_cfg.zone);
        }
        else if (sipalg_cfg.operation == SIP_CONF_DELETE)
        {
            sipAlgCfgEntryDelete (sipalg_cfg.lan_netaddr, 
                    sipalg_cfg.lan_netmask, sipalg_cfg.zone);
        }

        sig_dscp = sipalg_cfg.sig_dscp;
        rtp_dscp = sipalg_cfg.rtp_dscp;
        
    break;

    case SIPALG_GET_CFG:

        memcpy( ( void * )arg, &sipalg_cfg, sizeof( sipalg_cfg ) );

    break;

    case SIPALG_START:

        rval = sipalg_start();

        if( rval )
        {
            rval = -1;
        }

    break;

    case SIPALG_STOP:

        sipalg_stop();

    break;
    }

    return( rval );
}


#if defined (CONFIG_COMPAT)

/*******************************************************************************
*
* sipalgCompat_Ioctl - 32 bit userspace compatible IOCTLs for 64 bit kernels
*
* RETURNS: >=0 on success
*           <0 on failure 
*/
long sipalgCompat_Ioctl
    (
    struct file   * pFile,
    unsigned int    cmd,
    unsigned long   arg
    )
    {
    return sipalg_ioctl (pFile->f_dentry->d_inode, pFile, cmd, arg);
    }

#endif /* CONFIG_COMPAT */

/***************************************************************************/

int sipalg_open(struct inode *inode, struct file *file)
{
    return( 0 );
}


int sipalg_release(struct inode *inode, struct file *file)
{
    return( 0 );
}


static struct file_operations sipalg_fops =
{
    .owner   = THIS_MODULE,
    .poll    = NULL,
    .ioctl   = sipalg_ioctl,
#if defined (CONFIG_COMPAT)
    .compat_ioctl = sipalgCompat_Ioctl,
#endif /* CONFIG_COMPAT */
    .mmap    = NULL,
    .open    = sipalg_open,
    .release = sipalg_release,
};


static void __exit fini(void)
{
    SIP_NODE_LIST * pNode       = NULL;
    SIP_NODE_LIST * pTempNode   = NULL;

    DEBUGP("SIPALG: Uninitializing\n");

    /* lock */
    SIP_WRITE_LOCK (&sipAlgCfgEntryTblLock);

    /* delete all entries of the list */
    SIP_ALG_CFG_LIST_FOR_EACH_ENTRY_SAFE (pNode, pTempNode, 
                                          &pSipNodeEntryTbl->list, list)
            {
            SIP_ALG_CFG_LIST_DELETE (&pNode->list);
            kfree (pNode);
            }

    if (pSipNodeEntryTbl)
        kfree (pSipNodeEntryTbl);
    
    /*Remove the proc file */
    if(sipCfgProcDeInit() != 0)
        printk (KERN_ERR "proc entry delete failed\n");

    /* unlock */
    SIP_WRITE_UNLOCK (&sipAlgCfgEntryTblLock);

    sipalg_stop();

    unregister_chrdev(major, "sipalg");

    return;
}

int sipAlgCfgEntryAdd (int ipaddr, int subnet, int zone)
{
    SIP_NODE_LIST * pSipListNode = NULL;
          
    DEBUGP ("sipAlgCfgEntryAdd: Adding to the list ipaddr[%d] subnet[%d] "
            "zone[%d]\n", ipaddr, subnet, zone);

    pSipListNode = kmalloc (sizeof (SIP_NODE_LIST), GFP_ATOMIC);
    if (!pSipListNode)
    {
        printk ("Failed to allocate memory for pSipListNode\n");
        goto sipAlgCfgEntryAddErr;
    }

    SIP_ALG_CFG_INIT_LIST_HEAD (&pSipListNode->list);
    pSipListNode->sip_cfg_node.netaddr = ipaddr;
    pSipListNode->sip_cfg_node.netmask = subnet;
    pSipListNode->sip_cfg_node.zone = zone;

    /* lock */
    SIP_WRITE_LOCK (&sipAlgCfgEntryTblLock);

    SIP_ALG_CFG_LIST_ADD (&pSipListNode->list, &pSipNodeEntryTbl->list);

    /* unlock*/
    SIP_WRITE_UNLOCK (&sipAlgCfgEntryTblLock);

    return (0);

sipAlgCfgEntryAddErr:

    return (-1);
}

void sipAlgCfgEntryDelete (int ipaddr, int subnet, int zone)
{
    SIP_NODE_LIST * pSipListNode = NULL;

    pSipListNode = sipNodeEntryFind (ipaddr, subnet, zone);

    if (!pSipListNode)
        return;

    /* lock */
    SIP_WRITE_LOCK (&sipAlgCfgEntryTblLock);

    /* delete from the list */
    SIP_ALG_CFG_LIST_DELETE (&pSipListNode->list);

    /* free the node */
    kfree (pSipListNode);

    /* unlock */
    SIP_WRITE_UNLOCK (&sipAlgCfgEntryTblLock);

}

SIP_NODE_LIST * sipNodeEntryFind (int ipaddr, int subnet, int zone)
{
    SIP_NODE_LIST * pSipListNode = NULL;

    if (SIP_ALG_CFG_IS_LIST_EMPTY (&pSipNodeEntryTbl->list))
    {
        goto sipNodeEntryFindOut;
    }

    SIP_ALG_CFG_LIST_FOR_EACH_ENTRY (pSipListNode, &pSipNodeEntryTbl->list, list)
    {
        if (pSipListNode)
        {
            if (pSipListNode->sip_cfg_node.netaddr   == ipaddr    && 
                    pSipListNode->sip_cfg_node.netmask   == subnet    && 
                    pSipListNode->sip_cfg_node.zone   == zone )
            {
                goto sipNodeEntryFindOut;
            }
        }
    }
    return NULL ;

sipNodeEntryFindOut:
    return (pSipListNode);
}

SIP_NODE_LIST * sipSubnetEntryFind (int ipaddr, int subnet)
{
    SIP_NODE_LIST * pSipListNode = NULL;

    if (SIP_ALG_CFG_IS_LIST_EMPTY (&pSipNodeEntryTbl->list))
    {
        goto sipSubnetEntryFindOut;
    }

    SIP_ALG_CFG_LIST_FOR_EACH_ENTRY (pSipListNode, &pSipNodeEntryTbl->list, list)
    {
        if (pSipListNode)
        {
            if (((pSipListNode->sip_cfg_node.netaddr) & 
                (pSipListNode->sip_cfg_node.netmask)) ==
                ((ipaddr) & (subnet)))
            {
                goto sipSubnetEntryFindOut;
            }
        }
    }
   return NULL ;

sipSubnetEntryFindOut:
    return (pSipListNode);
}

EXPORT_SYMBOL(sipSubnetEntryFind);
/*******************************************************************************
*
* seqSingleStart - Callback for Seq file start.
*
* This routine is a callback routine for seq file start of operation.
* RETURNS: Pointer to the start of file.
*
* ERRNO: N/A
*
* SEE_ALSO: N/A
*/
static void *seqSingleStart
    (
    struct seq_file *p,  /* file pointer */
    loff_t *pos          /* file pointer offset */
    )
    {
    return NULL + (*pos == 0);
    }

/*******************************************************************************
*
* seqSingleNext - Advances pointer for next record.
*
* This routine is a callback routine advancing file pointer by 1 record.
* RETURNS: Pointer to the next record.
*
* ERRNO: N/A
*
* SEE_ALSO: N/A
*/
static void *seqSingleNext
    (
    struct seq_file *p,    /* file pointer */
    void *v, loff_t *pos   /* file pointer offset */
    )
    {
        ++*pos;
        return NULL;
    }

/*******************************************************************************
*
* seqSingleStop - End of file operation.
*
* This routine is a callback routine when file operation ends.
* RETURNS:
*
* ERRNO: N/A
*
* SEE_ALSO: N/A
*/
static void seqSingleStop
    (
    struct seq_file *p,  /* file pointer  */
    void *v              /* private data  */
    )
    {
    }

/*******************************************************************************
*
* seqSingleOpen - Open a seq file.
*
* This routine is a callback routine when file is opened.
* RETURNS: File Descriptor
*
* ERRNO: N/A
*
* SEE_ALSO: N/A
*/

int seqSingleOpen
    (
    struct file *file,                      /* file pointer         */ 
    int (*show)(struct seq_file *, void *), /* Callback for display */
    void *data                              /* private data         */
    )
    {
    struct seq_operations *op = kmalloc(sizeof(*op), GFP_KERNEL);
    int res = -ENOMEM;

    if (op) 
        {
        op->start = seqSingleStart;
        op->next = seqSingleNext;
        op->stop = seqSingleStop;
        op->show = show;
        res = seq_open(file, op);

        if (!res)
            ((struct seq_file *)file->private_data)->private = data;
        else
            kfree(op);
        }
    return res;
    }


/*******************************************************************************
*
* procSipCfgEntriesShow - Show SIP cfg Table
*
* This routine shows SIP cfg Table Entries.
* RETURNS:
*
* ERRNO: N/A
*
* SEE_ALSO: N/A
*/
static int procSipCfgEntriesShow 
    (
    struct seq_file *s, /* sequential file pointer */
    void *v             /* private data            */
    )
{
    SIP_NODE_LIST * pNode = NULL, * pTmpNode;    

    /* lock */
    SIP_READ_LOCK (&sipAlgCfgEntryTblLock);

    /* Display header */
    seq_printf(s,
            "    lanAddr         lanSubnet       zone\n");

    SIP_ALG_CFG_LIST_FOR_EACH_ENTRY_SAFE (pNode, pTmpNode, 
            &pSipNodeEntryTbl->list, list)
    {
        seq_printf(s, "  %u.%u.%u.%u       %u.%u.%u.%u       %d\n", 
                NIPQUAD(pNode->sip_cfg_node.netaddr), 
                NIPQUAD(pNode->sip_cfg_node.netmask),
                pNode->sip_cfg_node.zone); 
    }

    /* unlock */
    SIP_READ_UNLOCK(&sipAlgCfgEntryTblLock);

    return 0;
}

/*******************************************************************************
*
* procSipCfgEntriesOpen - Open a proc file.
*
* This routine is a callback routine when proc file is opened.
* RETURNS: File Descriptor
*
* ERRNO: N/A
*
* SEE_ALSO: N/A
*/

static int procSipCfgEntriesOpen 
    (
    struct inode *inode,
    struct file *file
    )
    {
    int  ret=0;
    ret  = seqSingleOpen(file, procSipCfgEntriesShow, NULL);
    if (!ret) seq_lseek(file, 0, 0);    
    return (ret);
    }

/*******************************************************************************
*
* procSipCfgEntriesRelease - Release a proc file.
*
* This routine is a callback routine when proc file is closed.
* RETURNS:
*
* ERRNO: N/A
*
* SEE_ALSO: N/A
*/

static int procSipCfgEntriesRelease (struct inode *inode, struct file *file)
    {
    return (seq_release(inode, file));
    }

static struct file_operations proc_sip_cfg_status_file_ops = {
  .owner = THIS_MODULE,
  .open = procSipCfgEntriesOpen,
  .read = seq_read,
  .llseek = seq_lseek,
  .release = procSipCfgEntriesRelease
};

/*******************************************************************************
*
* sipCfgProcInit - create proc file
*
* This routine  create proc file.
*
* RETURNS:
*
* ERRNO: N/A
*
* SEE_ALSO: N/A
*/

inline int sipCfgProcInit (void)
    {

    DEBUGP (KERN_INFO "initializing sip cfg proc entries\n");

    /* Create proc dir /proc/net/sip */
    pSipCfgProcDir = proc_mkdir(SIP_CFG_PROC_DIR, proc_net);
    if (!pSipCfgProcDir) 
        {       
        printk(KERN_INFO "failed to create procfs entry .\n");
        return (-1);
        }
    pSipCfgProcDir->owner = THIS_MODULE;

    /* Create /proc/net/sip/sipCfgEntries */
    pSipCfgProcFile = create_proc_entry (SIP_CFG_PROC_FILE, 
                                          SIP_CFG_PROC_FILE_MODE, 
                                          pSipCfgProcDir);

    if (pSipCfgProcFile)
        {
        pSipCfgProcFile->proc_fops = &proc_sip_cfg_status_file_ops;
        pSipCfgProcFile->owner = THIS_MODULE;
        }

    return (0);
    }

/*******************************************************************************
*
* sipCfgProcDeInit - cleans up proc file
*
* This routine  cleans proc file.
*
* RETURNS:
*
* ERRNO: N/A
*
* SEE_ALSO: N/A
*/

int sipCfgProcDeInit (void)
    {
    DEBUGP (KERN_INFO "De-initializing sip proc entries\n");
	remove_proc_entry (SIP_CFG_PROC_FILE, pSipCfgProcDir);
	remove_proc_entry (SIP_CFG_PROC_DIR, proc_net);

    return (0);
    }


static int __init init(void)
{

    DEBUGP("SIPALG: Initializing\n");

    if((major=register_chrdev(0, "sipalg", &sipalg_fops))<0)
    {
        DEBUGP("SIPALG: Unable to register character device\n");
        return(-EIO);
    }

    DEBUGP("SIPALG: LAN Network Address: 0x%x\n", lan_netaddr);
    DEBUGP("SIPALG: LAN Network Mask: 0x%x\n", lan_netmask);

    memset((void*) &sipalg_cfg, 0, sizeof(sipalg_cfg));

    sipalg_cfg.udp_ports[0] = SIP_CT_PORT;
    sipalg_cfg.num_udpports = 1;

    sipalg_cfg.lan_netaddr = lan_netaddr;
    sipalg_cfg.lan_netmask = lan_netmask;
    sipalg_cfg.sig_dscp = sig_dscp;

    reg_ports = 0;
    psip_ct_udp_helpers = NULL;
    psip_ct_tcp_helpers = NULL;

    /* initialize sip cfg table head */
    pSipNodeEntryTbl = kmalloc (sizeof (SIP_NODE_ENTRY_HEAD), GFP_KERNEL);
    
    if (!pSipNodeEntryTbl)
    {
        printk("SIPALG: Memeory allocation failed for pSipNodeEntryTbl\n");
        return(-1);
    }

    SIP_ALG_CFG_INIT_LIST_HEAD (&pSipNodeEntryTbl->list);

    /* proc entry initialization */
    if (sipCfgProcInit () != 0)
        printk (KERN_ERR "proc entry initialization failed\n");

    sipalg_start();

    return(0);
}


module_init(init);
module_exit(fini);

#endif /*SIPALG_CFG*/
