/****************************************************************************/
/*                                                                          */
/*                                                                          */
/*                        Copyright 2006 - 2010                             */
/*                    One Convergence Devices Pvt Ltd                       */
/*                         ALL RIGHTS RESERVED                              */
/*                                                                          */
/* Permission is hereby granted to licensees of One Convergence Devices Pvt */
/* Ltd' products to use or abstract this computer program for the sole      */
/* purpose of implementing a product based on One Convergence Devices Pvt   */
/* Ltd products. No other rights to reproduce, use, or disseminate this     */
/* computer program, whether in part or in whole, are granted.              */
/****************************************************************************/
/*
modification history
--------------------
01a,12oct09,grj Added sip support to VLANs
*/

#ifndef _SIPALG_CFG_H
#define _SIPALG_CFG_H

#define SIPALG_SET_CFG          (('s' << 16) | 0)
#define SIPALG_GET_CFG          (('s' << 16) | 1)
#define SIPALG_START            (('s' << 16) | 2)
#define SIPALG_STOP             (('s' << 16) | 3)
#define SIPALG_MAX_PORTS        8

typedef struct _sipalg_cfg
{
    unsigned short udp_ports[SIPALG_MAX_PORTS];        /* Listening ports */
    int num_udpports;                   /* Number of UDP listening ports */

    unsigned short tcp_ports[SIPALG_MAX_PORTS];        /* Listening ports */
    int num_tcpports;                   /* Number of TCP listening ports */
#if 0
    int asymmetric_resp;                /* Support asymmetric responses */
    int domain_rewrite;                 /* rewrite domain names */
#endif
    unsigned int lan_netaddr;           /* Used to identify internal and */
    unsigned int lan_netmask;           /* external networks             */
    u_int8_t sig_dscp;                  /* DSCP value for sip signalling */
    u_int8_t rtp_dscp;                  /* DSCP value for sip RTP flows  */
    u_int8_t operation;                 /* type of operation 1-insert,2-delete*/
    u_int8_t zone;                      /* zone <1-SECURE,2-INSECURE,4-PUBLIC */
} sipalg_cfg_t;

/* sip alg cfg list related */
#define SIP_ALG_CFG_LIST_HEAD_INIT(x)              LIST_HEAD_INIT(x)
#define SIP_ALG_CFG_LISTHEAD(x)                    LIST_HEAD(x)
#define SIP_ALG_CFG_INIT_LIST_HEAD(x)              INIT_LIST_HEAD(x)
#define SIP_ALG_CFG_LIST_ADD(new, head)            list_add(new, head)
#define SIP_ALG_CFG_LIST_ADD_TAIL(new, head)       list_add_tail(new, head)
#define SIP_ALG_CFG_LIST_DELETE(entry)             list_del(entry)
#define SIP_ALG_CFG_LIST_DEL_INIT(entry)           list_del_init(entry)
#define SIP_ALG_CFG_LIST_MOVE(list, head)          list_move(list, head)
#define SIP_ALG_CFG_LIST_MOVE_TAIL(list, head)     list_move_tail(list, head)
#define SIP_ALG_CFG_IS_LIST_EMPTY(head)            list_empty(head)
#define SIP_ALG_CFG_LIST_SPLICE(list, head)        list_splice(list, head)
#define SIP_ALG_CFG_LIST_SPLICE_INIT(list, head)   list_splice_init(list, head)
#define SIP_ALG_CFG_LIST_ENTRY(ptr, type, member)  list_entry(ptr, type, member)
#define SIP_ALG_CFG_LIST_FOR_EACH(pos, head)       list_for_each(pos, head)
#define SIP_ALG_CFG_LIST_FOR_EACH_PREV(pos, head)  list_for_each_prev(pos, head)
#define SIP_ALG_CFG_LIST_FOR_EACH_SAFE(pos, n, head)  \
                                  list_for_each_safe(pos, n, head)
#define SIP_ALG_CFG_LIST_FOR_EACH_ENTRY(pos, head, member)  \
                                  list_for_each_entry(pos, head, member)
#define SIP_ALG_CFG_LIST_FOR_EACH_ENTRY_SAFE(pos, n, head, member)  \
                                  list_for_each_entry_safe(pos, n, head, member)

#define SIP_CFG_PROC_DIR               "sip"
#define SIP_CFG_PROC_FILE              "sipCfgEntries"
#define SIP_CFG_PROC_FILE_MODE         0644
#define	PROC_FILE_SIZE				   sizeof (int)
#define SIP_CONF_ADD                   1
#define SIP_CONF_DELETE                2

#define SIP_WRITE_LOCK(x)          write_lock_bh(x)
#define SIP_WRITE_UNLOCK(x)        write_unlock_bh(x)
#define SIP_READ_LOCK(x)           read_lock_bh(x)
#define SIP_READ_UNLOCK(x)         read_unlock_bh(x)

/* typedefs */

typedef struct list_head    SIP_ALG_CFG_LIST_HEAD;

typedef struct sip_node_entry_head {
    SIP_ALG_CFG_LIST_HEAD list;
} SIP_NODE_ENTRY_HEAD;

typedef struct sip_node {
    int netaddr;
    int netmask;
    int zone;
} SIP_NODE;

typedef struct sip_node_list {
    SIP_ALG_CFG_LIST_HEAD list;
    SIP_NODE sip_cfg_node;
} SIP_NODE_LIST;

/* sip node list related routines */
int sipAlgCfgEntryAdd (int ipaddr, int subnet, int zone);
void sipAlgCfgEntryDelete (int ipaddr, int subnet, int zone);
SIP_NODE_LIST * sipNodeEntryFind (int ipaddr, int subnet, int zone);
SIP_NODE_LIST * sipSubnetEntryFind (int ipaddr, int subnet);
#endif /* _SIPALG_CFG_H */
