#ifndef __INCLUDE_PMAP_H__
#define __INCLUDE_PMAP_H__

#include "sf_types.h"
#include "pcrm.h"

#ifndef DETECTION_OPTION_TREE
#error "PMAP requires DETECTION_OPTION_TREE"
#endif

#define PP_PMAP             254
#define MPSE_PMAP           254
#define PMAP_ENQUEUED       1
#define PMAP_DEQUEUED       2
#define PMAP_SEARCH_DONE    4
#define PMAP_PCRE_RULE_ID_BASE  10000


struct pmap_pattern_group;

typedef struct _inline_vals
{
    int drop;
    int reject;
    int replace;
    int proto;
} IV;

typedef struct 
    {
    u_int32_t   dequeue_interval;
    u_int32_t   max_results;
    u_int32_t   max_packet_memory_size;
    u_int32_t   max_packet_size;
    u_int32_t   ipq_read_timeout;
    char *      bin_rule_file;

    void *      pHwCtx;
    u_int8_t *  s5_pkt_data;
 
    /* packet functions */
    void * (*pmapPktGet) ();
    void (*pmapPktFree) (void *);
    void (*pmapInput) ( void * p, void * context);
    void (*pmapOutput) ();

    /* pattern functions */
    struct pmap_pattern_group * (*pmapPGNew) ( void (*userfree)(void *p),
                                    void (*optiontreefree)(void **p));
    void (*pmapPGFree)( struct pmap_pattern_group  * pg );

    int (*pmapPGAddPattern)( struct pmap_pattern_group * pg,
                            unsigned char * pat, int patlen, int nocase,
                            int offset, int depth, void * userdata);

    int (*pmapPGCompile)( struct pmap_pattern_group * pg,
                         int (*build_tree)(void * id, void **existing_tree));

    unsigned int (*pmapPGSearch)( struct pmap_pattern_group * pg, 
                    unsigned char * t, int tlen,
                    int (*match)(void * id, void *tree, int index, void *data),
                    void * sdata );

    int      (*pmapPGSetupFlow) (struct pmap_pattern_group *, void * flow, 
                                 unsigned short * groupId);

    void    (*pmapRulesDone) (void);

    /* session function */
    void (*newSession) (void * session, void * packet);
    void (*deleteSession) (void * session);

    /* snort support functions & variables */
    PORT_RULE_MAP * prmTcpRTNX;
    PORT_RULE_MAP * prmUdpRTNX;
    int *           do_detect_content_ptr;
    int *           do_detect_ptr;

    int  (*mpseSearch)( void *pv, const unsigned char * T, int n, 
                    int ( *action )(void* id, void * tree, int index, void *data), 
                    void * data, int* current_state ); 
    int (*mpseSetupFlow)( void *pvoid, void *flow, unsigned short * groupId);
 
    int (*rule_tree_match)( void * id, void *tree, int index, void * data);

    int (*prmFindRuleGroup)( PORT_RULE_MAP * p, int dport, int sport, PORT_GROUP ** src, PORT_GROUP **dst , PORT_GROUP ** gen);
    void * (*initOmd) (void);
    IV  ** ppIV;

    void (*handlePkt)(void *m);
    UINT64  spPcreHWMatch;
    UINT64  spPcreSWMatch;
    }PMAP_CTRL;

extern PMAP_CTRL *  pPmapCtrl;
extern PMAP_CTRL    pmapCtrl;
void *              pGlobalPkt;

typedef struct pmapMatchInfo
    {
    struct pmapMatchInfo *  next;
    unsigned short          groupId;
    unsigned short          ruleId;
    unsigned short          offset;
    }PMAP_MATCH_INFO;

struct pmap_job;

typedef struct 
    {
    void *              next;
    void *              mem;
    void *              m;
    int                 flags;
    u_int8_t            refcount;
    u_int8_t            uri_count;
    IV                  iv;
    struct pmap_job *   jobs;
    PMAP_MATCH_INFO *   pcreResults;
    }
    PMAP_PACKET;

typedef struct
    {
    void *          flow;
    unsigned short  groupId1;
    unsigned short  groupId2;
    PORT_GROUP *    pg1;
    PORT_GROUP *    pg2;
    }PMAP_FLOW;

#define PMAP_SESSION_DELETE 1
#define PMAP_SESSION_DELETED 2

typedef struct
    {
    PMAP_FLOW   clientFlow;
    PMAP_FLOW   serverFlow;
    PMAP_FLOW   clientFlowURI;
    PMAP_FLOW   serverFlowURI;
    int         flags;
    int         (*deleteSessionFunc)(void *sessionCache, void *ssn);
    void *      deleteSessionArg;
    int         refcount;
    }
    PMAP_SESSION;

#endif
