#!/pfrm2.0/bin/lua 

--[[
#### File: configUpdate.lua
#### Description: restore functions

#### Revisions:
01a,28Nov'07, nsr written
]]--


--************* Requires *************
require "teamf1lualib/util"
require "teamf1lualib/db"

--************* Functions *************

function filterTable (t, f)
	assert (type (t) == "table")
	assert (type (f) == "function")
	local k, v
    local kk, vv
	local result = {}
	for k, v in pairs (t) do
        result[k]={}
	    for kk, vv in pairs (v) do
    	    if f (kk) then result [k][kk] = vv end
        end
  	end 
  	return result
end 

function tableAppend (table1, table2)
    if (table1 == nil or table2 == nil) then return table1 end
    local k,v
    local kk, vv
    for k, v in pairs (table2) do
        for kk,vv in pairs (v) do
            -- create new row if required
            if (not table1[k]) then
                table1[k]= {}
            end
            if (vv == "") then
                table1[k][kk] = nil;
            else
                table1[k][kk] = vv;
            end
        end -- for
    end -- for

    return table1
end


function tableSubtract (table1, table2, tableName)
	local result = {}

    result = filterTable (table1, 
    function (k)
        local key, value
        local kk, vv
		for key, value in pairs(table2) do
		    for kk, vv in pairs(value) do
                if kk == k then 
                    return true 
                end
            end
        end
      	return false
    end
    )
    return result;
end

--************* Initial Code *************

local oldTable = {}
local table = {}

--************* Logic *************

db.connect (arg[1])

local SETTINGS_FILE = db.getAttribute("environment", "name", "TEAMF1_CFG_ASCII", "value")
local saveTables = db.getTable("saveTables2", false)

dofile (SETTINGS_FILE) -- read old configuration

for k,v in pairs(saveTables) do
    local tableName = v ["TableName"]

    -- record the old table values
    oldTable[tableName] = util.getLuaVariable(tableName)
    
    -- clear glob. variables
    _G[tableName] = nil
end


dofile (arg[2]) -- read new configuration

for k,v in pairs(saveTables) do

    local tableName = v ["TableName"]
    local table1 = util.getLuaVariable(tableName)
    local table2 = oldTable[tableName]

    if (table1 == nil) then
        
        print ("configUpdate : no LUA variable : " .. tableName)
    else

        -- TODO : Handle systemConfig table properly
        if (table2 and tableName ~= "systemConfig") then

            -- make new table compatible for old values
            local allKeys = db.getColNames(tableName);

            -- First make the defaults table have all the columns defined for that table in the schema
            for kk,vv in pairs(allKeys) do
                    for kkk,vvv in pairs(table1) do
                        -- if column required
                    if (not table1[kkk][vv]) then 
                            table1[kkk][vv] = "";
                        end
                    end -- for
            end -- for

            -- There are 3 cases 
            -- a. The table we are trying to merge has less number of columns than the current table
            -- b. The table we are trying to merge has more number of columns than the current table
            -- c. The table we are trying to merge has exactly the same columns as the current table
            --
            --
            -- a. To deal with this case we first add the missing columns to the table we are trying to merge.
            for kk,vv in pairs(allKeys) do
                for kkk,vvv in pairs(table2) do
                    -- if column required
                    if (not table2[kkk][vv]) then 
                        table2[kkk][vv] = "";
                end
            end -- for
            end -- for
       
            -- b. To deal with this case we now delete the additional columns from the table we are trying 
            --     to merge.
            if (#table1 ~= 0) then
                table2 = tableSubtract (table2, table1) 
            end

            -- c. We should now be in this situation. So we merge the tables.
            table1 = tableAppend (table1, table2)
        end

        -- Check table integrity 
        for kk,vv in pairs (table1) do
            -- add columns for NOT NULL kind of fields
            for key,val in pairs (table1[1]) do
                -- if field is missing from a row
                if (not table1[kk][key]) then
                    -- use value from first row
                    table1[kk][key] = table1[1][key]
                end
            end
            -- delete rows from table if not needed
            if (table2 and not table2[kk]) then
                table1[kk] = nil
            end
        end
    
        -- Save new table 
        if (#table1 >= 0) then
            table[tableName] = table1;
        end
    end
end -- for


-- save configuration to file
 
local file = io.open(SETTINGS_FILE .. "", "ab")

for k,v in pairs(saveTables) do

    local tableName = v ["TableName"]
    db.saveTable(file, tableName, table[tableName], false, 1, 1)
end

db.close() -- close db handle
