-- advancedCLI.lua
-- Ravindranath
-- TeamF1
-- 02/03/10
-- CLISH Security Checks,IGMP proxy and IPS LUA routines
--
-- Required Libraries
require "teamf1lualib/attackChecks"
require "teamf1lualib/ips"
require "teamf1lualib/igmp"

-- This function displays the current Security Checks configuration
function securityChecksInfoGet (args)
    printLabel ("Attack Checks")
    row = db.getRow("AttackChecks", "AttackChecks._ROWID_",1)
--  The AttackChecks table should have only one entry
     
    print ("WAN Security Checks:","")
    if (row["AttackChecks.StealthMode"] == "0") then
       print( " Enable Stealth mode               : No")
    else 
       print( " Enable Stealth mode               : Yes")    
    end
    if (row["AttackChecks.TcpFlood"] == "0") then
       print( " Block TCP Flood                   : No")      
    else 
       print( " Block TCP Flood                   : Yes")    
    end
    print ("LAN Security Checks:","")
    if (row["AttackChecks.UdpFlood"] == "0") then
       print( " Block UDP Flood                   : No")      
    else 
       print( " Block UDP Flood                   : Yes")    
    end

    row = db.getRow("IcsaSettings", "IcsaSettings._ROWID_",1)
    print ("ICSA Settings:","")
  --The IcsaSettings table should have only one entry
      
    if (row["IcsaSettings.BlockICMPNotification"] == "0") then
       print( " Block ICMP Notification           : No")      
    else 
       print( " Block ICMP Notification           : Yes")    
    end
    if (row["IcsaSettings.BlockFragPkts"] == "0") then
       print( " Block Multicast Packets           : No")      
    else 
       print( " Block Multicast Packets           : Yes")    
    end
    if (row["IcsaSettings.BlockMulticastPkts"] == "0") then
       print( " Block Multicast Packets           : No")      
    else 
       print( " Block Multicast Packets           : Yes")    
    end

    row = db.getRow("FwDosAttackSettings", "FwDosAttackSettings._ROWID_",1)
--  The FwDosAttackSettings table should have only one entry
    
    print ("DoS Attacks:","")
    print ("Syn Flood Detect Rate[max/sec]        :",row["FwDosAttackSettings.SynFloodDetectRate"])
    print ("Echo Storm Flood Rate[ping pkts/sec]  :",row["FwDosAttackSettings.EchoStormFloodRate"])
    print ("ICMP Flood Rate[ICMP pkts/sec]        :",row["FwDosAttackSettings.IcmpFloodRate"])
end
function igmpInfoGet()
configRow = db.getRow("Igmp", "_ROWID_", "1")
printLabel("IGMP Configuration")
	if (configRow["Igmp.IgmpEnable"] == "1")then
		print("Igmp Proxy: Enabled")
	else
		print("Igmp Proxy: Disabled")
	end
end
function ipsInfoGet()
	local status=db.getRow("IPSStatus","_ROWID_","1")
	ipsconfig=db.getRow("IPSConfig","_ROWID_","1")
	ipscheckconfiglan=db.getRow("IPSNetworkSegments","_ROWID_","1")	
	ipscheckconfigdmz=db.getRow("IPSNetworkSegments","_ROWID_","2")	
	printLabel("Intrusion Detection/Prevention Enable")
	if (ipsconfig["IPSConfig.idsEnable"] == "1")then
		print("Intrusion Detection: Enabled")
	else
		print("Intrusion Detection: Disabled")
	end
	if (ipsconfig["IPSConfig.ipsEnable"] == "1")then
		print("Intrusion Prevention: Enabled")
	else
		print("Intrusion Prevention: Disabled")
	end
	printLabel("IPS Checks Active Between")
	if (ipscheckconfiglan["IPSNetworkSegments.ipsEnable"] == "1")then
		print("LAN and WAN :Yes")
	else
		print("LAN and WAN :No")
	end	
	if (ipscheckconfigdmz["IPSNetworkSegments.ipsEnable"] == "1")then
		print("DMZ and WAN :Yes")
	else
		print("DMZ and WAN :No")
	end
	printLabel("IPS Status")
	
	print("Number of Signatures Loaded: " ..status["IPSStatus.numRules"])
end

-- This function saves the Modified Security Checks information
function securityChecksCfgSave(configRow)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""
   
    errorFlag, statusCode = attackChecks.config (configRow, "1", "edit")

    -- save db if no error
    if (errorFlag == "OK") then	db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage
end

-- This function reads the current SecurityChecks information into a configuration object
function securityChecksCfgInit(args)
    local rowId = 1
    local configRow = attackChecks.configGet()

    if configRow == nil then
        return rowId, {}
    end
    return rowId, configRow
end
function igmpCfgInit(args)
configRow = db.getRow("Igmp", "_ROWID_", "1")
return 1,configRow
end
function igmpCfgSave(configRow)
	local errorFlag="ERROR"
	local statusCode=""
	local statusMessage=""
	errorFlag, statusCode = igmp.config(configRow, "1", "edit")
	-- save db if no error
	if (errorFlag == "OK") then db.save() end
	statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or statusCode
	return errorFlag,statusMessage
end

function ipsCfgInit(args)
local configRow={}
configRow = ips.configGet()
return 1,configRow
end

function ipsCfgSave(configRow)
    local errorFlag="ERROR"
    local statusMessage=""
    local statusCode=""
    local bwRow = {}
    bwRow = db.getRow("BandWidthProfileStatus", "_ROWID_", "1")
   	errorFlag, statusCode = ips.config (configRow, "1", "edit")

	-- save db if no error
    if (errorFlag == "OK") then 
        db.save()
        local procEntry = db.getAttribute("environment", "name", "HW_OFFLOAD_PROC_ENTRY", "value")
        if (procEntry ~= nil) then
            if (configRow["IPSConfig.idsEnable"] == "1") then
                util.appendDebugOut("Exec: " .. os.execute("echo 0 >" .. procEntry))
            elseif (configRow["IPSConfig.ipsEnable"] == "1") then
                util.appendDebugOut("Exec: " .. os.execute("echo 0 >" .. procEntry))
            else
                if (bwRow["BandWidthProfileStatus.Status"] == "1") then
                    util.appendDebugOut("Exec: " .. os.execute("echo 0 >" .. procEntry))                
                else
                    util.appendDebugOut("Exec: " .. os.execute("echo 1 >" .. procEntry))
                end 
            end
        end
    end
        statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or statusCode
return errorFlag,statusMessage
end
