--[[
#### Abhinendra Babu. T
#### TeamF1
#### www.TeamF1.com
#### June 07, 2008
#### File: bwLimit.lua
#### Description: Bandwidth Profiles and Traffic Selector Functions
### Revisions:
]]--

--************* Requires *************

--************* Initial Code *************

--package bwProfiles
bwProfiles = {}
trafficSelector = {}

--************* Functions *************
-- Bandwidth Profiles Configuration
function bwProfiles.config (inputTable, rowid, operation)
    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            return db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            return db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
            return db.delete(DBTable, inputTable)
        end
    end
    return false
end

-- Bandwidth Profiles Configuration
function bwProfiles.profileConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false
    
    -- For Add Operation
    if (operation == "add") then
    	local id = db.existsRow("DscpClassList", "Status", "0")
    	if (id) then
	        local classId = db.getAttribute("DscpClassList", "_ROWID_", id, "ClassId")
	        local dscpMark = db.getAttribute("DscpClassList", "_ROWID_", id, "DscpMark")
    	    inputTable["BandWidthProfile.DscpMark"] = dscpMark
	        inputTable["BandWidthProfile.ClassId"] = classId
    	    local dscpTable = {}
	        dscpTable["DscpClassList.Status"] = 1
	        valid = db.update("DscpClassList", dscpTable, id)
    	    if (valid) then
	            valid = bwProfiles.config(inputTable, rowid, "add")
	        end
    	else
	        return "ERROR", "BWPROFILE_DSCP_CONFIG_FAILED"
    	end
    end
    
    -- For Edit Operation
    if (operation == "edit") then
        if (DBTable == "BandWidthProfile") then
            local bwProfileName = db.getAttribute("BandWidthProfile", "_ROWID_", rowid, "ProfileName")
        	local rows = db.getRows("TrafficSelector", "ProfileName", bwProfileName)
            if (#rows == 0) then
                valid = bwProfiles.config(inputTable, rowid, operation)
            else
            	db.rollback()
                return "ERROR", "BWPROFILE_INUSE_EDIT_FAILED"
            end
        else
            valid = bwProfiles.config(inputTable, rowid, operation)            
        end
    end
    
     -- return
    if (valid) then
    	db.commitTransaction(true)
        if (operation == "edit" and DBTable == "BandWidthProfile") then
            db.setAttributeWhere ("BandWidthProfileStatus", "rowId != 0" , "Status", 0);
            os.execute ("sleep 1")
            db.setAttributeWhere ("BandWidthProfileStatus", "rowId != 0" , "Status", 1);
        end
    	return "OK", "STATUS_OK"
    else
    	db.rollback()
	    if(DBTable == "BandWidthProfile") then
	         return "ERROR", "BWPROFILE_CONFIG_FAILED"
    	else
	         return "ERROR", "BWPROFILESTATUS_CONFIG_FAILED"
    	end
    end
end

-- Bandwidth Profiles Deletion
function bwProfiles.deleteProfiles (rowIds)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false
    local case = ""
    
    -- delete routes
    for k,v in pairs(rowIds) do
        local inTable = {}
        inTable["BandWidthProfile._ROWID_"] = v;
    	local dscpMark = db.getAttribute("BandWidthProfile", "_ROWID_", v, "DscpMark")
	    local id = db.existsRow("DscpClassList", "DscpMark", dscpMark)
        local bwProfileName = db.getAttribute("BandWidthProfile", "_ROWID_", v, "ProfileName")
    	local rows = db.getRows("TrafficSelector", "ProfileName", bwProfileName)
    	local dscpTable = {}
	    dscpTable["DscpClassList.Status"] = 0
        if (#rows ~= 0) then
	        valid = false
	        case = "1"
    	else 
	        valid = true
    	end
	    if (valid) then
	        valid = db.update("DscpClassList", dscpTable, id)	    
    	end
	    if (valid) then
	        valid = db.delete("BandWidthProfile", inTable)
    	end
	    if (not valid) then
	        break
    	end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        if (case == "1") then
   	    return "ERROR", "BWPROFILE_USE_DELETE_FAILED"
	else
	    return "ERROR", "BWPROFILE_DELETE_FAILED"
	end
    end
end

-- Traffic Selector Configuration
function trafficSelector.config (inputTable, rowid, operation)
    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            return db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            return db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
            return db.delete(DBTable, inputTable)
        end
    end
    return false
end

-- Traffic Selector Configuration
function trafficSelector.profileConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false
    
    valid = trafficSelector.config(inputTable, rowid, operation)
       	
    -- return
    if (valid) then
    	db.commitTransaction(true)
    	return "OK", "STATUS_OK"
    else
    	db.rollback()
        if (operation == "delete") then
            return "ERROR", "TRAF_SELECTOR_DELETE_FAILED"            
        else
        	return "ERROR", "TRAF_SELECTOR_CONFIG_FAILED"            
        end
    end
end
