-- bwLimitCLI.lua
-- Samer Sayeed
-- TeamF1
-- www.TeamF1.com
--
-- Modification History
-- 04mar10,db  added input validations and match type support in traffice selectors
-- 05jan10,db  added wan interface support and fixed validation routines
-- 29jan09,ss  added input validation routines and fixed init routines 
-- 29nov08,ss  written 
--
-- Description
-- CLI interface set and get routines

require "teamf1lualib/bwLimit"

function bwProfileStatInit(args)
    local bwProfileStatus = args[1]
    local rowId = -1

    local configRow = db.getRow("BandWidthProfileStatus","Status", bwProfileStatus)
    if (configRow == nil) then
        configRow = {}
        configRow["BandWidthProfileStatus.Status"] = bwProfileStatus
    else
        rowId = configRow["BandWidthProfileStatus._ROWID_"]
    end
    return rowId, configRow
end

function bwProfileStatSave(configRow)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = nil
    DBTable = "BandWidthProfileStatus"

    errorFlag, statusCode = bwProfiles.profileConfig (configRow, 1, "edit")

   -- save db if no error
   if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)

    return errorFlag, statusMessage
end

function bwProfileStatInputVal(configRow)
    return true
end

function bwProfileCfgInit(args)
    local row = args[1]

    if (row == nil) then
        configRow = {}
        configRow["BandWidthProfile._ROWID_"] = "-1"
    else
        configRow = {}
        configRow["BandWidthProfile._ROWID_"] = row
        configRow = db.getRow("BandWidthProfile", "_ROWID_", row)
        if (configRow == nil) then
           print("Row does not exist")          
           return -1, {}
        end
    end
    return configRow["BandWidthProfile._ROWID_"], configRow
end

function bwProfileCfgSave(configRow)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""
    DBTable = "BandWidthProfile"
    
    local RowId = configRow["BandWidthProfile._ROWID_"]
    
    if (RowId == "-1") then
        errorFlag, statusCode = bwProfiles.profileConfig (configRow, "-1", "add")
    else
        errorFlag, statusCode = bwProfiles.profileConfig (configRow, RowId, "edit")
    end

    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)

    return errorFlag, statusMessage
end

function bwProfileCfgDel(row)
    local errorFlag = "OK"
    local statusCode =""
    local statusMessage =""
    DBTable = "BandWidthProfile"
    if (row ~= nil) then
        configRow = {}
        configRow["BandWidthProfile._ROWID_"] = row
        configRow = db.getRow("BandWidthProfile", "_ROWID_", row)
        if (configRow == nil) then
           print("Row does not exist")          
           return -1, {}
        end
    else
        print("Please enter ROW ID")
    end

    local rows = {_ROWID_ = configRow["BandWidthProfile._ROWID_"]}
    errorFlag , statusCode =  bwProfiles.deleteProfiles (rows)
   
   -- save db if no error
    if (errorFlag == "OK") then 
        db.save()
        print ("Row Deleted")
    end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)

    return errorFlag, statusMessage
end

function bwProfileCfgInputVal(configRow)
    local bwProfileStatusTable = db.getRow ("BandWidthProfileStatus","_ROWID_", "1")
    
        if(bwProfileStatusTable ["BandWidthProfileStatus.Status"] ~= nil and bwProfileStatusTable ["BandWidthProfileStatus.Status"] == "0") then
            printCLIError ("Bandwidth Profile Configuration failed, band width profile status is Disabled")
            return false
        end
        if (configRow["BandWidthProfile._ROWID_"] ~= "-1") then
        local bwProfileName = db.getAttribute("BandWidthProfile", "_ROWID_", configRow["BandWidthProfile._ROWID_"], "ProfileName")
    	local rows = db.getRows("TrafficSelector", "ProfileName", bwProfileName)
        if (#rows ~= 0) then
            printCLIError ("Bandwidth Profile configuration failed as profile name is in use by traffic Selector.")
	    return false
        end
        end
        if(configRow["BandWidthProfile.ProfileName"] == nil or configRow["BandWidthProfile.ProfileName"] == "") then
            printCLIError ("Enter valid profile name\n")
            return false
        end
        if(configRow["BandWidthProfile.ProfileType"] == nil or configRow["BandWidthProfile.ProfileType"] == "") then
            printCLIError ("Select valid profile type Priority/Bandwidth Rate\n")
            return false
        end
        if(configRow["BandWidthProfile.ProfileType"] == "0") then
            if (configRow["BandWidthProfile.Priority"] == "" or configRow["BandWidthProfile.Priority"] == nil) then
            printCLIError ("Enter valid Priority Low/Medium/High\n")
            return false
            end
        end
        if(configRow["BandWidthProfile.ProfileType"] == "1") then
            if (configRow["BandWidthProfile.MinimumRate"] == "" or configRow["BandWidthProfile.MaximumRate"] == "" or configRow["BandWidthProfile.MinimumRate"] == nil or configRow["BandWidthProfile.MaximumRate"] == nil) then
            printCLIError ("Enter valid bandwidth rate range\n")
            return false
            end
        if(configRow["BandWidthProfile.MinimumRate"] ~= nil and configRow["BandWidthProfile.MaximumRate"] ~= nil) then
        local bwMaxRate =  tonumber (configRow["BandWidthProfile.MaximumRate"])
        local bwMinRate =  tonumber (configRow["BandWidthProfile.MinimumRate"])
        if (bwMaxRate < bwMinRate) then
            printCLIError ("Invalid bandwidth rate range, Minimum bandwidth rate cannot be greater than Maximum bandwidth rate\n")
            return false
        end
        end
        end
        if(configRow["BandWidthProfile.WANID"] == nil) then
            printCLIError ("Enter valid WAN Interface type \n")
            return false
        end

    return true
end

function trafficSelectorCfgInit(args)
    local rowId = args[1]

    if (rowId == nil) then
        configRow = {}
        configRow["TrafficSelector._ROWID_"] = "-1"
        configRow["TrafficSelector.OID"] = "10"
    else
        configRow = db.getRow("TrafficSelector","_ROWID_", rowId)
        if (configRow == nil) then
           print("Row does not exist")          
           return -1, {}
        end
    end
    return configRow["TrafficSelector._ROWID_"], configRow
end

function trafficSelectorCfgSave(configRow)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""
    DBTable = "TrafficSelector"

    if (configRow ["TrafficSelector._ROWID_"] == "-1") then
        errorFlag, statusCode = trafficSelector.profileConfig(configRow, "-1", "add")
    else
        errorFlag, statusCode = trafficSelector.profileConfig(configRow, configRow["TrafficSelector._ROWID_"], "edit")
    end

   -- save db if no error
   if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)

    return errorFlag, statusMessage
end

function trafficSelectorCfgDel(row_id)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""
    DBTable = "TrafficSelector"

    if (row_id ~= nil) then
        configRow = db.getRow("TrafficSelector","_ROWID_", row_id)
        if (configRow == nil) then
           print("Row does not exist")          
           return -1, {}
        end
    else
        print ("Please enter the row id")
    end

    local rows = {_ROWID_ = configRow["TrafficSelector._ROWID_"]}
    errorFlag, statusCode = trafficSelector.profileConfig(rows, "-1", "delete")
    
    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)

    return errorFlag, statusMessage
end

function trafficSelectorCfgInputVal(configRow)
    local bwProfileStatusTable = db.getRow ("BandWidthProfileStatus","_ROWID_", "1")
    
    if(bwProfileStatusTable ["BandWidthProfileStatus.Status"] ~= nil and bwProfileStatusTable ["BandWidthProfileStatus.Status"] == "0") then
        printCLIError ("Bandwidth Profile Status is Disabled")
        return false
    else
    if (configRow["TrafficSelector.ProfileName"] == nil or configRow["TrafficSelector.ProfileName"] == "") then 
        printCLIError ("Enter valid bandwidth profile name\n")
        return false
    end
    local row = db.getRow ("BandWidthProfile", "ProfileName", configRow["TrafficSelector.ProfileName"]) 
    if (row == nil) then
        printCLIError ("Invalid bandwidth profile name\n")
        return false
    end
    if (configRow["TrafficSelector.ServiceName"] == nil or configRow["TrafficSelector.ServiceName"] == "") then 
        printCLIError ("Enter valid service name\n")
        return false
    end
    local serviceRow = db.getRow ("Services", "ServiceName", configRow["TrafficSelector.ServiceName"]) 
    if (serviceRow == nil) then
        printCLIError ("Invalid Service name, please enter valid service name\n")
        return false
    end
    if (configRow["TrafficSelector.MatchType"] == nil or configRow["TrafficSelector.MatchType"] == "") then 
        printCLIError ("Select valid match type IP/MAC/Port/Interface Address\n")
        return false
    end
    if (configRow["TrafficSelector.MatchType"] == "1" and (configRow["TrafficSelector.IPAddress"] == "" or configRow["TrafficSelector.IPAddress"] == nil )) then 
        printCLIError ("Enter valid IP Address\n")
        return false
    end
    if (configRow["TrafficSelector.MatchType"] == "2" and configRow["TrafficSelector.MACAddress"] == nil or configRow["TrafficSelector.MACAddress"] == "") then 
        printCLIError ("Enter valid MAC Address\n")
        return false
    end
    if (configRow["TrafficSelector.MatchType"] == "3") then
    if (configRow["TrafficSelector.PortName"] == nil or configRow["TrafficSelector.PortName"] == "") then 
        printCLIError ("Enter valid port name\n")
        return false
    end
    local row = db.getRow ("PortMgmt", "PortName", configRow["TrafficSelector.PortName"]) 
    if (row == nil) then
        printCLIError ("Invalid port name\n")
        return false
    end
    end
    if (configRow["TrafficSelector.MatchType"] == "4") then
    if (configRow["TrafficSelector.VlanId"] == nil or configRow["TrafficSelector.VlanId"] == "") then 
        printCLIError ("Enter valid Vlan Id\n")
        return false
    end
    local row = db.getRow ("vlan", "portName = 'dummy' and vlanId", configRow["TrafficSelector.VlanId"]) 
    if (row == nil) then
        printCLIError ("Invalid Vlan Id\n")
        return false
    end
    end

    return true
    end 
end

function bwProfileGet()
    local bwProfileStatusTable =  db.getRow ("BandWidthProfileStatus","_ROWID_", "1")
    local bwProfileTable = db.getTable ("BandWidthProfile")
    local resultTab= {} 
    local resultTab1= {} 
    local bwProfileRow = nil
    local rowId = 0
    
    if(bwProfileStatusTable ["BandWidthProfileStatus.Status"] ~= nil and bwProfileStatusTable ["BandWidthProfileStatus.Status"] == "0") then
      resTab.insertField (resultTab, "BandWidth Profile Status is", "Disabled")
    else
      resTab.insertField (resultTab, "BandWidth Profile Status is ", "Enabled")
    end -- if
    printLabel ("List of Available Bandwidth Profiles")
    for col, val in pairs(bwProfileTable) do
      rowId = rowId+1
      local bwProfileRow = bwProfileTable[rowId]
        if (rowId ~= nil) then 
          resTab.insertField (resultTab1, "ROW ID", bwProfileRow["BandWidthProfile._ROWID_"] or "")
          resTab.insertField (resultTab1, "Name", bwProfileRow["BandWidthProfile.ProfileName"] or "")
          if(bwProfileRow["BandWidthProfile.ProfileType"] == "1") then
          local rate = bwProfileRow["BandWidthProfile.MinimumRate"] .. "-" .. bwProfileRow["BandWidthProfile.MaximumRate"]
          resTab.insertField (resultTab1, "Bandwidth Range/Priority", rate)
          elseif(bwProfileRow["BandWidthProfile.ProfileType"] == "0")then
          if(bwProfileRow["BandWidthProfile.Priority"] == "0") then
          resTab.insertField (resultTab1, "Bandwidth Range/Priority", "Low")
          elseif(bwProfileRow["BandWidthProfile.Priority"] == "1")then
          resTab.insertField (resultTab1, "Bandwidth Range/Priority", "Medium")
          elseif(bwProfileRow["BandWidthProfile.Priority"] == "2")then
          resTab.insertField (resultTab1, "Bandwidth Range/Priority", "High")
          end 
          end
        end --if
    end -- for
    resTab.print (resultTab, 0)
    resTab.print (resultTab1, 0)
end

function trafficSelectorGet()
    local trafficSelectorTable = db.getTable ("TrafficSelector")
    local resultTab= {} 
    local trafficSelectorRow = nil
    local rowId = 0

    printLabel ("List of Available Traffic Selectors")
    for col, val in pairs(trafficSelectorTable) do
        rowId = rowId+1
        local trafficSelectorRow = trafficSelectorTable[rowId]
        if (rowId ~= nil) then 
        resTab.insertField (resultTab, "ROW ID", 
			      trafficSelectorRow["TrafficSelector._ROWID_"] or "")
        resTab.insertField (resultTab, "Traffic Selector Details", 
			      trafficSelectorRow["TrafficSelector.ServiceName"] or "")
        resTab.insertField (resultTab, "Bandwidth Profile Name", 
			      trafficSelectorRow["TrafficSelector.ProfileName"] or "")
        if(trafficSelectorRow["TrafficSelector.MatchType"]=="2") then
        resTab.insertField (resultTab, "Traffic Selector", "MAC Address")
        elseif(trafficSelectorRow["TrafficSelector.MatchType"]=="1") then
        resTab.insertField (resultTab, "Traffic Selector", "IP Address")
        elseif(trafficSelectorRow["TrafficSelector.MatchType"]=="3") then
        resTab.insertField (resultTab, "Traffic Selector", "Port Name")
        elseif(trafficSelectorRow["TrafficSelector.MatchType"]=="4") then
        resTab.insertField (resultTab, "Traffic Selector", "Interface")
        end
        end
    end
    resTab.print (resultTab, 0)
end 

