#!/pfrm2.0/bin/lua 

--[[
  #### File: config.lua
  #### Description: configuration library

  #### Revisions:
  01a,19Dec07,nsr written
  ]]--

--************* Requires *************
require "teamf1lualib/util"

-- package config
config = {}

-- generate md5 sum for file
function config.getChecksum (fileName)

    local pfile = io.popen ("cat " .. fileName .. " | grep -v 'checksum' | md5sum  | cut -d' ' -f1")
    local checksum = pfile:read ("*a")
    pfile:close()

    -- remove new line character
    return string.gsub (checksum, "\n", "")
end

-- verify checksum on file 
function config.verifyChecksum (fileName)

    -- populate table entires from file
    if (pcall (loadfile (fileName))) then
        dofile (fileName)
    else
        return nil
    end

    -- get checksum from systemConfig tbl
    local systemTbl = util.getLuaVariable ("systemConfig")
    if(systemTbl ~= nil) then
        local checksum_file = systemTbl[1]["checksum"]
        -- generate md5sum for file
        local checksum_md5sum = config.getChecksum (fileName)

        -- verify checksum is correct or 0
        if (checksum_file == checksum_md5sum or checksum_file == "0") then
            return "ok"
        else
            return nil
        end
    else
        return nil
    end
end

-- update checksum in file
function config.updateChecksum (db, fileName)

    local saveTables = db.getTable("saveTables", false)
    local checksum = config.getChecksum (fileName)
    local file = io.open(fileName .. "", "wb")

    for k,v in pairs(saveTables) do
        local tableName = v ["tableName"]
        local table = db.getTable (tableName, false)
        
        -- if table is systemConfig, update csum
        if (tableName == "systemConfig") then
            table[1]["checksum"] = checksum
        end
        -- require to pass following extra arguments to handle the configuration
        -- which needs to be saved or not saved using _metadata.
        db.saveTable(file, tableName, table, false, 1, 1)
    end
end
