-- customServicesCLI.lua
-- Samer Sayeed  
-- TeamF1
-- www.TeamF1.com

-- Modification History
-- 27dec10,rac Modified all functions to support present changes 
-- 26dec08,ss  written
--
-- Description
-- CLI interface set and get routines

require "teamf1lualib/fwView"

-- this routine reads in a current access control rule configuration into a 
-- configuration object. Or wii just initialize the defaults in case a new
-- rule is getting added.
function customServicesCfgInit(args)
    local rowId = args[1]
    if (rowId == nil) then
	configRow = db.getDefaults(true, "Services")
    else
        configRow= db.getRow("Services","_ROWID_",rowId)
        if (configRow == nil) then
           print("Row does not exist")          
           return -1, {}
        end
    end
    return configRow["Services._ROWID_"], configRow
end

-- The routine saves the modified/ newly added access control rule
function customServicesCfgSave(configRow)
    local errorFlag = "OK"
    local statusCode = ""
    local statusMessage = ""
    DBTable = "Services"

    if (configRow["Services._ROWID_"] == nil) then
	configRow["Services.OID"]=nil
	if(configRow["Services.Protocol"] == "1" or configRow["Services.Protocol"] == "58")then
	configRow["Services.DestinationPortEnd"]=nil
	end
	configRow["Services.IsDefault"]="0"
        errorFlag, statusCode = fwView.serviceConfig(configRow, "-1", "add")
    else
        errorFlag, statusCode = fwView.serviceConfig(configRow, configRow["Services._ROWID_"], "edit")
    end
    -- save db if no error
    if (errorFlag == "OK") then db.save() end
        statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage
end

-- The routine saves the modified/ newly added access control rule
function customServicesCfgDel(args)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""
    DBTable = "Services"
   configRow= db.getRow("Services","_ROWID_",args[1])
        if (configRow == nil) then
		printCLIError("Row doesnt exist")
		return errorFlag, statusMessage
        end
    local rows = {}
    rows["rowid"] = configRow["Services._ROWID_"]
    errorFlag, statusCode = fwView.deleteServices(rows)
    
    -- save db if no error
    if (errorFlag == "OK") then db.save() end
        statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage
end

function customServicesCfgInputVal(configRow)
    if(configRow["Services.IsDefault"] == "1") then
        printCLIError ("Invalid Row Id\n")
        return false
    end
    if (configRow["Services.ServiceName"] == "" or configRow["Services.ServiceName"] == nil) then
        printCLIError ("Enter unique service name\n")
        return false
    end
    if (configRow["Services.Protocol"] == nil) then
        printCLIError ("Enter valid protocol name\n")
        return false
    end
    if ((configRow["Services.Protocol"] == "1" or configRow["Services.Protocol"] == "58") and configRow["Services.DestinationPortStart"] == nil) then
        printCLIError ("Enter valid ICMP Port\n")
        return false
    elseif (configRow["Services.Protocol"] == "1" and configRow["Services.DestinationPortStart"] ~= nil) then
        if (tonumber(configRow["Services.DestinationPortStart"]) < 0 or tonumber(configRow["Services.DestinationPortStart"]) > 40) then
        printCLIError ("Enter valid ICMP Port between 0-40\n")
        return false
        end
    elseif (configRow["Services.Protocol"] == "58" and configRow["Services.DestinationPortStart"] ~= nil) then
        if (tonumber(configRow["Services.DestinationPortStart"]) < 1 or tonumber(configRow["Services.DestinationPortStart"]) > 255) then
        printCLIError ("Enter valid ICMP Port between 1-255\n")
        return false
        end
    end
    if (configRow["Services.Protocol"] == "6" or configRow["Services.Protocol"] == "17") then 
        if(configRow["Services.DestinationPortStart"] == nil or configRow["Services.DestinationPortEnd"] == nil) then
            printCLIError ("Enter valid Port Range\n")
            return false
        end
    end
    return true
end

function customServicesCfgGet()
    local serviceTable = db.getTable ("Services")
    local resultTab = {} -- table to store DHCP leased clients list
    local serviceRow = nil
    local rowId = "0"

    printLabel ("List of Available Custom Services")    
    for col, val in pairs(serviceTable) do
        rowId = rowId+1
        local serviceRow = serviceTable[rowId]
	if (rowId ~= nil) then
            if(serviceRow["Services.IsDefault"] == "0") then
	    resTab.insertField (resultTab, "ROW ID", 
			      serviceRow["Services._ROWID_"] or "")
	    resTab.insertField (resultTab, "Name", 
			      serviceRow["Services.ServiceName"] or "")
            if(serviceRow["Services.Protocol"] == "1")then
            resTab.insertField (resultTab, "Type", "ICMP")
            elseif(serviceRow["Services.Protocol"] == "6")then
            resTab.insertField (resultTab, "Type", "TCP")
            elseif(serviceRow["Services.Protocol"] == "17")then
            resTab.insertField (resultTab, "Type", "UDP")
            elseif(serviceRow["Services.Protocol"] == "58")then
            resTab.insertField (resultTab, "Type", "ICMPv6")
            end -- if Type
            if(serviceRow["Services.Protocol"] == "1" or serviceRow["Services.Protocol"] == "58")then
            resTab.insertField (resultTab, "ICMP Type / Port Range", serviceRow["Services.DestinationPortStart"] or "")
            else
            local portRange = serviceRow["Services.DestinationPortStart"] .. "-" .. serviceRow["Services.DestinationPortEnd"]
            resTab.insertField (resultTab, "ICMP Type / Port Range", portRange or "")
            end -- if Type
            end -- if Default
        end -- if rowid nil
    end -- for 
    resTab.print (resultTab, 0)
end
