--[[
#### Hussain Vali.
#### TeamF1
#### www.TeamF1.com
#### Feb 06, 2008

#### File: ddns.lua
#### Description: DDNS functions

#### Revisions:
01b, 15Jul08, shv Added Wrapper function to support WAN1 and WAN2 DDNS.
]]--


--************* Requires *************

--************* Initial Code *************
--package ddns
ddns = {}
--************* Functions *************
-- ddns update Configuration
function ddns.updateConfig (inputTable, rowid, operation)
    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            return false
        elseif (operation == "edit") then
            return db.update(DBTable,inputTable,rowid)
        elseif (operation == "delete") then
            return false
        end
    end
end

-- ddns config
function ddns.config (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    
    db.beginTransaction() --begin transaction
    local valid = false

    valid = ddns.updateConfig (inputTable, rowid, operation)

    -- return
    if (valid) then
        db.commitTransaction()
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "DDNS_CONFIG_FAILED"
    end
end

-- Wrapper function to configure WAN1 and WAN2 DDNS.
function ddns.configWarp (inputTable, rowid1, rowid2)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    
    db.beginTransaction() --begin transaction
    local valid = false

    -- updateting WAN1 DDNS Info
    valid = ddns.updateConfig (inputTable, rowid1, "edit")

    -- updateting WAN2 DDNS Info    
    if (valid and inputTable["ddns.ddnsService2"] ~= nil) then
        local ddnsInfo = {}
        ddnsInfo ["ddns.ddnsService"] = inputTable["ddns.ddnsService2"]
        if (inputTable["ddns.ddnsService2"] ~= "0") then
            ddnsInfo ["ddns.hostname"] = inputTable["ddns.hostname2"]
            ddnsInfo ["ddns.username"] = inputTable["ddns.username2"]
            ddnsInfo ["ddns.password"] = inputTable["ddns.password2"]
            ddnsInfo ["ddns.wildflag"] = inputTable["ddns.wildflag2"]
            ddnsInfo ["ddns.timePeriod"] = inputTable["ddns.timePeriod2"]            
        end
        valid = ddns.updateConfig (ddnsInfo, rowid2, "edit")
    end

    -- return
    if (valid) then
        db.commitTransaction()
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "DDNS_CONFIG_FAILED"
    end    
end

-- function to get ddns information.
function ddns.configGet (intName)
    return db.getRow ("ddns","interfaceName",intName)
end
