--[[
#### Hussain Vali
#### TeamF1
#### www.TeamF1.com
#### Mar 28, 2008

#### File: dhcp.lua
#### Description: dhcp functions

#### Revisions:
01a,19Jul08,shv added DHCP Relay Gateway configuration function.
]]--


--************* Requires *************

--************* Initial Code *************
--package dhcp
dhcp = {}
--************* Functions *************
-- dhcp config
function dhcp.config (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
            return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false

    if (operation == "add") then
        return nil
    elseif (operation == "edit") then
        valid = dhcp.updateConfig(inputTable, rowid)
    elseif (operation == "delete") then
        return nil
    end

    -- return
    if (valid) then
        db.commitTransaction()
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "DHCP_CONFIG_FAILED"
    end
end

-- Updating Configurations
function dhcp.updateConfig (inputTable, rowid)
    -- updating DHCP Configurations
    local inTable = {}
    inTable["dhcp.dhcpEnabled"] = inputTable["dhcp.dhcpEnabled"]
    if (inputTable["dhcp.dhcpEnabled"] ~= 0) then
        inTable["dhcp.domainName"] = inputTable["dhcp.domainName"]
        inTable["dhcp.startAddress"] = inputTable["dhcp.startAddress"]
        inTable["dhcp.endAddress"] = inputTable["dhcp.endAddress"]
        inTable["dhcp.primaryDnsServer"] = inputTable["dhcp.primaryDnsServer"]
        inTable["dhcp.secondaryDnsServer"] = inputTable["dhcp.secondaryDnsServer"]
        --inTable["dhcp.primaryTftpServer"] = inputTable["dhcp.primaryTftpServer"]
        --inTable["dhcp.secondaryTftpServer"] = inputTable["dhcp.secondaryTftpServer"]
        inTable["dhcp.winsServer"] = inputTable["dhcp.winsServer"]
        inTable["dhcp.leaseTime"] = inputTable["dhcp.leaseTime"]
        inTable["dhcp.enableDnsProxy"] = inputTable["dhcp.enableDnsProxy"]
    end

    -- validate
    if (dhcp.inputvalidate(inTable)) then
        return db.update("dhcp",inTable, rowid)
    end
end

-- dhcp inputvalidate
function dhcp.inputvalidate (inputTable)
    if (true) then
        return db.typeAndRangeValidate(inputTable)
    end
    return false
end

function dhcp.fixedIpConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
            return "ACCESS_DENIED", "ADMIN_REQD"
    end

    if(operation == "add") then
    	if(db.existsRow(DBTable,"IpAddr",inputTable["DhcpfixedIpAddress.IpAddr"])) then
    		return "ERROR", "DUPLICATE_IPADDR"
    	end
    	if(db.existsRow(DBTable,"MacAddr",inputTable["DhcpfixedIpAddress.MacAddr"])) then
    		return "ERROR", "DUPLICATE_MACADDR"
    	end
    elseif(operation == "edit") then
    	local dhcpFixedRowId = db.getAttribute("DhcpfixedIpAddress","IpAddr",inputTable["DhcpfixedIpAddress.IpAddr"],"_ROWID_")
    	if(dhcpFixedRowId ~= nil and dhcpFixedRowId ~= rowid) then
    		return "ERROR", "DUPLICATE_IPADDR"
    	end
    	dhcpFixedRowId = db.getAttribute("DhcpfixedIpAddress","MacAddr",inputTable["DhcpfixedIpAddress.MacAddr"],"_ROWID_")
    	if(dhcpFixedRowId ~= nil and dhcpFixedRowId ~= rowid) then
    		return "ERROR", "DUPLICATE_MACADDR"
    	end
    end

    db.beginTransaction() --begin transaction
    local valid = false

    if (operation == "add") then
        valid = db.insert(DBTable, inputTable)
    elseif (operation == "edit") then
        valid = db.update(DBTable, inputTable, rowid)
    elseif (operation == "delete") then
        valid = db.delete(DBTable, inputTable)
    end

    -- return
    if (valid) then
        db.commitTransaction()
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "DHCP_CONFIG_FAILED"
    end
end

function dhcp.dhcpConfigWrap (inputTable, logIfName)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local dhcpRelayUpdate = false
    local valid = false
    local where = "LogicalIfName = '" .. logIfName .."'"
    local dhcpsConfig = db.getRowWhere ("dhcp",where)
    local dhcpRelayConfig = db.getRowWhere ("DhcpRelay",where)

    local dhcpRelayInTable = {}
    if (inputTable["dhcp.dhcpEnabled"] == "3") then
        inputTable["dhcp.dhcpEnabled"] = "0"
        dhcpRelayInTable["DhcpRelay.DhcpRelayStatus"] = "1"
        dhcpRelayInTable["DhcpRelay.RelayGateway"] = inputTable["DhcpRelay.RelayGateway"]
    else
        dhcpRelayInTable["DhcpRelay.DhcpRelayStatus"] = "0"        
    end

    local rowid = nil
    if (dhcpsConfig ~= nil) then
        rowid = dhcpsConfig["dhcp._ROWID_"]
    end
    if (rowid ~= nil) then
                    valid = dhcp.updateConfig (inputTable, rowid)
        end
    
    if (valid and dhcpRelayConfig ~= nil) then
        rowid = dhcpRelayConfig["DhcpRelay._ROWID_"]
            if (valid) then
                valid = db.update("DhcpRelay", dhcpRelayInTable, rowid)
            end
        end
    
    -- return
    if (valid) then
        db.commitTransaction()
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "DHCP_CONFIG_FAILED"
    end
end
