-- dhcpCLI.lua
-- Samer Sayeed
-- TeamF1
-- www.TeamF1.com
--
-- Modification History
-- 04mar10,db modified dhcpReservedIPsCfgDel routine
-- 19sep08,ss written 
--
-- Description
-- CLI interface set and get routines

require "teamf1lualib/dhcp"

local trafficTypeTab= {
"Best Effort",
"Background",
"Spare",
"Excellent Effort",
"Controlled Load",
"Video",
"Voice",
"Network Control"
}

---------------------------------------------------------------
-- This routine implements the CLI command to show DHCP Servers
-- configuration
---------------------------------------------------------------

function dhcpSetupGet ()
    local resultTab = {} -- table to store DHCP Server setup information
    local dhcpTable = nil
    local dhcpRelayTable = nil

    printLabel ("DHCP Setup Configuration")    
    dhcpTable = db.getRow ("dhcp","_ROWID_", 1)
    dhcpRelayTable = db.getRow ("DhcpRelay","_ROWID_", 1)
    
    if (dhcpTable ~= nil and dhcpRelayTable ~= nil) then 
        if (dhcpTable ["dhcp.dhcpEnabled"] == "0" and dhcpRelayTable ["DhcpRelay.DhcpRelayStatus"] == "1") then
               resTab.insertField (resultTab, "DHCP Relay ", "Enabled")
        end
        if (dhcpTable ["dhcp.dhcpEnabled"] == "1" and dhcpRelayTable ["DhcpRelay.DhcpRelayStatus"] == "0") then
               resTab.insertField (resultTab, "DHCP Server ", "Enabled")
        end
        if (dhcpTable ["dhcp.dhcpEnabled"] == "0" and dhcpRelayTable ["DhcpRelay.DhcpRelayStatus"] == "0") then
               resTab.insertField (resultTab, "DHCP Server and Relay ", "Disabled")
        end
        if (dhcpTable ["dhcp.dhcpEnabled"] == "1") then 
           resTab.insertField (resultTab, "Domain Name", 
	                      dhcpTable["dhcp.domainName"] or "")
           resTab.insertField (resultTab, "Starting IP Address", 
	                      dhcpTable["dhcp.startAddress"] or "")
           resTab.insertField (resultTab, "Ending IP Address", 
			      dhcpTable["dhcp.endAddress"] or "")
           resTab.insertField (resultTab, "Primary DNS Server", 
			      dhcpTable["dhcp.primaryDnsServer"] or "")
           resTab.insertField (resultTab, "Secondary DNS Server", 
			      dhcpTable["dhcp.secondaryDnsServer"] or "")
           resTab.insertField (resultTab, "WINS Server", 
			      dhcpTable["dhcp.winsServer"] or "")
           resTab.insertField (resultTab, "Lease Time in hrs ", 
			      dhcpTable["dhcp.leaseTime"] or "")
           if (dhcpTable["dhcp.enableDnsProxy"] == "1") then 
               resTab.insertField (resultTab, "DNS Proxy", " Enabled")
           else 
               resTab.insertField (resultTab, "DNS Proxy", " Disabled")
           end
        elseif (dhcpRelayTable ["DhcpRelay.DhcpRelayStatus"] == "1") then
            resTab.insertField (resultTab, "Relay Gateway", 
			      dhcpRelayTable["DhcpRelay.RelayGateway"] or "")
        end
    end 
    resTab.print (resultTab, 0)
end

--------------------------------------------------
-- This routine implements the CLI command to show 
-- DHCP Reserved IPs entries
--------------------------------------------------

function dhcpReservedIPSetupGet ()
    local dhcpReservedIpTable = db.getTable ("DhcpfixedIpAddress")
    local resultTab= {} 
    local dhcpReservedIpRow = nil
    local rowId = 0

    printLabel ("List of DHCP Reserved Addresses")
    for col, val in pairs(dhcpReservedIpTable) do
        rowId = rowId+1
        local dhcpReservedIpRow = dhcpReservedIpTable[rowId]
        if (rowId ~= nil) then 
        resTab.insertField (resultTab, "IP Address", 
			      dhcpReservedIpRow["DhcpfixedIpAddress.IpAddr"] or "")
        resTab.insertField (resultTab, "MAC Address", 
			      dhcpReservedIpRow["DhcpfixedIpAddress.MacAddr"] or "")
        end
    end
    resTab.print (resultTab, 0)
end

--------------------------------------------------
-- This routine implements the CLI command to show 
-- DHCP Mac Filtering entries
--------------------------------------------------

function dhcpMacFilteringSetupGet ()
    local dhcpMacFilteringTable = db.getTable ("DhcpMacExclusion")
    local resultTab= {} 
    local dhcpMacFilteringRow = nil
    local rowId = 0

    printLabel ("List of DHCP MAC Filtering identifiers")
    for col, val in pairs(dhcpMacFilteringTable) do
        rowId = rowId+1
        local dhcpMacFilteringRow = dhcpMacFilteringTable[rowId]
        if (rowId ~= nil) then 
        resTab.insertField (resultTab, "MAC Address", 
			      dhcpMacFilteringRow["DhcpMacExclusion.MacAddress"] or "")
            if (dhcpMacFilteringRow["DhcpMacExclusion.AllowFlag"] == "0") then 
                resTab.insertField (resultTab, "Status", "Deny")
            else
                resTab.insertField (resultTab, "Status", "Allow")
            end
        end
    end
    resTab.print (resultTab, 0)
end

---------------------------------------------------
-- This routine implements the CLI command to show 
-- DHCP Option 60 entries
---------------------------------------------------

function dhcpOption60SetupGet ()
    local dhcpOption60Table = db.getTable ("Option60")
    local resultTab= {} 
    local dhcpOption60Row = nil
    local rowId = "0"
    local idx = "1"

    printLabel ("List of DHCP Option 60 Identifiers")
    for col, val in pairs(dhcpOption60Table) do
        rowId = rowId+1
        local dhcpOption60Row = dhcpOption60Table[rowId]
        if (rowId ~= nil) then 
        resTab.insertField (resultTab, "Vendor Class Identifier", 
			      dhcpOption60Row["Option60.VendorId"] or "")
        resTab.insertField (resultTab, "Start IP Address", 
			      dhcpOption60Row["Option60.StartIp"] or "")
        resTab.insertField (resultTab, "End IP Address", 
			      dhcpOption60Row["Option60.EndIp"] or "")
        idx = tonumber (dhcpOption60Row["Option60.PriorityLevel"]) 
        idx = idx + 1
        resTab.insertField (resultTab, "Traffic Type", trafficTypeTab[idx]) 
        end
    end
    resTab.print (resultTab, 0)
end

--------------------------------------------------
-- This routine implements the CLI command to show 
-- DHCP Leased clients list
--------------------------------------------------

function dhcpLeasedListGet ()
    local dhcpViewTable = db.getTable ("DhcpLeasedClients")
    local resultTab = {} -- table to store DHCP leased clients list
    local dhcpViewRow = nil
    local rowId = "0"

    printLabel ("List of Available DHCP Leased Clients")    
    for col, val in pairs(dhcpViewTable) do
        rowId = rowId+1
        local dhcpViewRow = dhcpViewTable[rowId]
	if (rowId ~= nil) then
	    --resTab.insertField (resultTab, "Device Name", 
	--		      dhcpViewRow["DhcpView.DevName"] or "")
            resTab.insertField (resultTab, "IP Address", 
			      dhcpViewRow["DhcpLeasedClients.IPAddr"] or "")
            resTab.insertField (resultTab, "Mac Address", 
			      dhcpViewRow["DhcpLeasedClients.MacAddr"] or "")
        end
    end
    resTab.print (resultTab, 0)
end 

-------------------------------------------------------------------------------
-- This routine reads in the current DHCP Server configuration information
-- into a configuration object.
function dhcpServerCfgInit(args)
    local rowId = -1

    local configRow = db.getRow("dhcp", "_ROWID_", "1")
    if configRow == nil then
       return rowId, {}
    end
    
    rowId = configRow["dhcp._ROWID_"]   
    return rowId, configRow
end

--
-- This routine saves the modified DHCP Server configuration information.
function dhcpServerCfgSave(configRow)
    local errorFlag = nil
    local statusCode = ""
    local statusMessage = nil

    if (db.existsRow("dhcp","_ROWID_","1")) then
        errorFlag, statusCode = dhcp.config(configRow, "1","edit")
    else
        errorFlag, statusCode = dhcp.config(configRow, "-1","add")
    end
    
    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage           	 
end

--
-- This routine validates the DHCP Server configuration information.
function dhcpServerCfgInputVal(configRow)
    local configRelayRow = db.getRow ("DhcpRelay", "DhcpRelay._ROWID_", 1)
  
    if (configRow["dhcp.dhcpEnabled"] == "1" and configRelayRow["DhcpRelay.DhcpRelayStatus"] == "1") then 
        printCLIError ("Disable DHCP Relay before enabling DHCP Server")
        return false
    end
    if (configRow["dhcp.dhcpEnabled"] == "1" and configRow["dhcp.startAddress"] == nil) then
        printCLIError ("Invalid Start IP address")
        return false
    end
    if (configRow["dhcp.dhcpEnabled"] == "1" and configRow["dhcp.endAddress"] == nil) then
        printCLIError ("Invalid End IP address")
        return false
    end
    if (configRow["dhcp.dhcpEnabled"] == "1" and configRow["dhcp.leaseTime"] == nil) then
        printCLIError ("Invalid lease Time")
        return false
    end
    if (configRow["dhcp.dhcpEnabled"] == "1" and configRow["dhcp.enableDnsProxy"] == nil) then
        printCLIError ("Invalid DNS Proxy status")
        return false
    end
    return true
end

--
-- This routine reads in the current Dhcp Relay configuration information
-- into a configuration object.
function dhcpRelayCfgInit(args)
    local rowId = -1

    local configRow = db.getRow("DhcpRelay", "_ROWID_", "1")
    if configRow == nil then
       return rowId, {}
    end
    
    rowId = configRow["DhcpRelay._ROWID_"]   
    return rowId, configRow
end

--
-- This routine saves the modified Dhcp Relay configuration information.
function dhcpRelayCfgSave(configRow)
    local errorFlag = nil
    local statusCode = ""
    local statusMessage = nil

    if (db.existsRow("DhcpRelay","_ROWID_",1)) then
        errorFlag, statusCode = dhcpRelay.config(configRow, "1", "edit")
    end
    
    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage           	 
end

--
-- This routine validates the Dhcp Relay configuration information.
function dhcpRelayCfgInputVal(configRow)
    local configServerRow = db.getRow ("dhcp", "dhcp._ROWID_", 1)
  
    if (configServerRow["dhcp.dhcpEnabled"] == "1" and configRow["DhcpRelay.DhcpRelayStatus"] == "1") then 
        printCLIError ("Disable DHCP Server before enabling DHCP Relay")
        return false
    end
    if (configRow["DhcpRelay.DhcpRelayStatus"] == "1" and 
       (configRow["DhcpRelay.RelayGateway"] == nil or configRow["DhcpRelay.RelayGateway"] == "0.0.0.0")) then 
        printCLIError ("Invalid relay gateway")
        return false
    end
    return true
end

--
-- This routine reads in the current Dhcp Reserved IPs configuration information
-- into a configuration object.
function dhcpReservedIPsCfgInit (args)
    local macAddr = args[1]
    local rowId = -1

    local configRow = db.getRow("DhcpfixedIpAddress","MacAddr", macAddr)
    if (configRow == nil) then
        configRow = {}
        configRow["DhcpfixedIpAddress.MacAddr"] = macAddr
        configRow["DhcpfixedIpAddress.LogicalIfName"] = "LAN"
        configRow["DhcpfixedIpAddress._ROWID_"] = "-1"
    else
        rowId = configRow["DhcpfixedIpAddress._ROWID_"]
    end
    return rowId, configRow
end

--
-- This routine saves the modified Dhcp Reserved IPs configuration information.
function dhcpReservedIPsCfgSave (configRow)
    local errorFlag = nil
    local statusCode = ""
    local statusMessage = nil
    DBTable = "DhcpfixedIpAddress"

    if (configRow ["DhcpfixedIpAddress._ROWID_"] == "-1") then
            errorFlag, statusCode = dhcp.fixedIpConfig(configRow, configRow ["DhcpfixedIpAddress._ROWID_"], "add")
    else
            errorFlag, statusCode = dhcp.fixedIpConfig(configRow, configRow ["DhcpfixedIpAddress._ROWID_"], "edit")
    end

    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
   
    return errorFlag, statusMessage
end

--
-- This routine delete the modified Dhcp Reserved IPs configuration information.
function dhcpReservedIPsCfgDel (macAddr)
    local errorFlag = nil
    local statusCode = ""
    local statusMessage = nil
    DBTable = "DhcpfixedIpAddress"
    local configRow = db.getRow("DhcpfixedIpAddress","MacAddr", macAddr)

    if (configRow == nil or configRow == {}) then
        print ("Macaddress entry doesn't exist")
        return false
    else
    errorFlag, statusCode = dhcp.fixedIpConfig(configRow, configRow ["DhcpfixedIpAddress._ROWID_"], "delete")
    end
    -- save db if no error
    if (errorFlag == "OK") then 
        db.save()
        print ("Row Deleted")
    end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
   
    return errorFlag, statusMessage
end

--
-- This routine validates the Dhcp Reserved IPs configuration information.
function dhcpReservedIPsCfgInputVal (configRow)
    --local dhcpMacFilteringRows = db.getRowsWhere ("DhcpMacExclusion", "MacAddress = '" .. configRow["DhcpfixedIpAddress.MacAddr"] .. "'")
    --if (dhcpMacFilteringRows ~= nil and #dhcpMacFilteringRows ~= 0) then
--	printCLIError ("MAC conflict with dhcp mac filter entry") 
  --      return false 
    --end
    if (configRow["DhcpfixedIpAddress.IpAddr"] == nil) then
        printCLIError ("Invalid IP Address for the fixed mac address entry.")
        return false
    end
    return true
end

--
-- This routine reads in the current Dhcp Mac Filtering configuration information
-- into a configuration object.
function dhcpMacFilteringCfgInit (args)
    local macAddr = args[1]
    local rowId = -1

    local configRow = db.getRow("DhcpMacExclusion","MacAddress", macAddr)
    if (configRow == nil) then
        configRow = {}
        configRow["DhcpMacExclusion.MacAddress"] = macAddr
        configRow["DhcpMacExclusion.LogicalIfName"] = "LAN"
    else
        rowId = configRow["DhcpMacExclusion._ROWID_"]
    end
    return rowId, configRow 
end

--
-- This routine saves the modified Dhcp Mac Filtering configuration information.
function dhcpMacFilteringCfgSave (configRow)
    local errorFlag = nil
    local statusCode = ""
    local statusMessage = nil
    DBTable = "DhcpMacExclusion"

    if (configRow ["DhcpMacExclusion._ROWID_"] == nil) then
        errorFlag , statusMessage = dhcp.macFilteringConfig (configRow, "-1", "add")
    else
        errorFlag , statusMessage = dhcp.macFilteringConfig (configRow, configRow ["DhcpMacExclusion._ROWID_"], "edit")
    end

    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
   
    return errorFlag, statusMessage
end

--
-- This routine deletes the modified Dhcp mac filtering configuration information.
function dhcpMacFilteringCfgDel (configRow)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = nil
    DBTable = "DhcpMacExclusion"

    errorFlag , statusMessage = dhcp.macFilteringConfig (configRow, configRow ["DhcpMacExclusion._ROWID_"], "delete")
    
    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)

    return errorFlag, statusMessage 
end

--
-- This routine validates the Dhcp Mac Filtering configuration information.
function dhcpMacFilteringCfgInputVal (configRow)
    local dhcpReservedRows = db.getRowsWhere("DhcpfixedIpAddress", "MacAddr = '" .. configRow["DhcpMacExclusion.MacAddress"] .. "'")
    if (dhcpReservedRows ~= nil and #dhcpReservedRows ~= 0) then
	printCLIError ("MAC conflict with dhcp reserved ips entry") 
        return false 
    end
    if (configRow["DhcpMacExclusion.AllowFlag"] == nil) then
        printCLIError ("Invalid Allow/Deny Flag.")
        return false
    end
    return true
end

--
-- This routine reads in the current Dhcp Option 60 configuration information
-- into a configuration object.
function dhcpOption60CfgInit (args)
    local vendorId = args[1]
    local rowId = -1

    local configRow = db.getRow("Option60","VendorId", vendorId)
    if (configRow == nil) then
        configRow = {}
        configRow["Option60.VendorId"] = vendorId
    else
        rowId = configRow["Option60._ROWID_"]
    end
    return rowId, configRow    
end

--
-- This routine saves the modified Dhcp Option 60 configuration information.
function dhcpOption60CfgSave (configRow)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = nil
    DBTable = "Option60"

    if (configRow ["Option60._ROWID_"] == nil) then
        errorFlag , statusMessage = dhcp.option60Config (configRow, "-1", "add")
    else
        errorFlag , statusMessage = dhcp.option60Config (configRow, configRow ["Option60._ROWID_"], "edit")
    end

   -- save db if no error
   if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)

    return errorFlag, statusMessage 
end

--
-- This routine deletes the modified Dhcp option 60 configuration information.
function dhcpOption60CfgDel (configRow)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = nil
    DBTable = "Option60"

    errorFlag , statusMessage = dhcp.option60Config (configRow, configRow ["Option60._ROWID_"], "delete")
    
    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)

    return errorFlag, statusMessage 
end

--
-- This routine validates the Dhcp Option 60 configuration information.
function dhcpOption60CfgInputVal (configRow)
    if (configRow["Option60.StartIp"] == nil) then
        printCLIError ("Invalid Dhcp start ip address of dhcp option 60 pool.")
        return false
    end
    if (configRow["Option60.EndIp"] == nil) then
        printCLIError ("Invalid Dhcp end ip address of dhcp option 60 pool.")
        return false
    end
    if (configRow["Option60.PriorityLevel"] == nil or
        tonumber(configRow["Option60.PriorityLevel"]) < 0 or
        tonumber(configRow["Option60.PriorityLevel"]) > 7 ) then
        printCLIError ("Invalid priority level please enter between 0 and 7 only.")
        return false
    end
    return true
end
