-- dmzCLI.lua
-- Ravindranath
-- TeamF1
-- www.TeamF1.com
-- Description
-- CLISH dmz LUA routines
require "teamf1lualib/lan"
require "teamf1lualib/dhcp"

--
--This routine initializes the DMZ configuration
function dmzCfgInit()
    local where = "LogicalIfName = 'DMZ' AND AddressFamily = 2"
	local rowid = db.existsRowWhere ("ifStatic", where)
    configRow = db.getRow("ifStatic", "_ROWID_", rowid)	
    local dhcpConfigRow = db.getRow ("dhcp", "logicalIfName", "DMZ")
    if (dhcpConfigRow ~= nil) then
	    configRow ["dhcp.dhcpEnabled"] = dhcpConfigRow["dhcp.dhcpEnabled"]
		if (configRow ["dhcp.dhcpEnabled"] ~= 0) then			
			configRow ["dhcp.startAddress"] = dhcpConfigRow ["dhcp.startAddress"]
			configRow ["dhcp.endAddress"] = dhcpConfigRow ["dhcp.endAddress"]
			configRow ["dhcp.primaryDnsServer"] = dhcpConfigRow ["dhcp.primaryDnsServer"]
			configRow ["dhcp.secondaryDnsServer"] = dhcpConfigRow ["dhcp.secondaryDnsServer"]			
			configRow ["dhcp.winsServer"] = dhcpConfigRow ["dhcp.winsServer"]
			configRow ["dhcp.leaseTime"] = dhcpConfigRow ["dhcp.leaseTime"]
			configRow ["dhcp.enableDnsProxy"] = dhcpConfigRow ["dhcp.enableDnsProxy"]
		end
	end
	
    local dhcpRelayConfig = db.getRow("DhcpRelay", "LogicalIfName", "DMZ")
    if (dhcpRelayConfig ~= nil) then
    	if (dhcpRelayConfig["DhcpRelay.DhcpRelayStatus"] == "1") then
    		configRow ["dhcp.dhcpEnabled"] = "3"
    	end
    	configRow ["DhcpRelay.RelayGateway"] = dhcpRelayConfig["DhcpRelay.RelayGateway"]
    end	 
	return -1,configRow

end

--
--This routine validates the DMZ configuration
function dmzCfgInputVal(configRow)
local oldPort = db.getAttribute ("ConfigPort", "_ROWID_", "1","LogicalIfName")
print(oldPort)
if (oldPort ~= "DMZ")then
	printCLIError("Please set the Configurable port to DMZ")
	return false
end
    if (configRow["dhcp.dhcpEnabled"] == "1" and 
        (configRow["dhcp.startAddress"] == nil or 
         configRow["dhcp.startAddress"] == "" or 
         configRow["dhcp.startAddress"] == "0.0.0.0")) then
        printCLIError ("Invalid Start IP address")
        return false
    end
    if (configRow["dhcp.dhcpEnabled"] == "1" and 
        (configRow["dhcp.endAddress"] == nil or 
         configRow["dhcp.endAddress"] == "" or 
         configRow["dhcp.endAddress"] == "0.0.0.0")) then
        printCLIError ("Invalid End IP address")
        return false
    end
    if (configRow["dhcp.dhcpEnabled"] == "1" and 
        (configRow["dhcp.leaseTime"] == "" or
         configRow["dhcp.leaseTime"] == nil)) then
        printCLIError ("Invalid lease Time")
        return false
    end

    if (configRow["dhcp.dhcpEnabled"] == "3" and 
       (configRow["DhcpRelay.RelayGateway"] == nil or 
        configRow["DhcpRelay.RelayGateway"] == "" or 
        configRow["DhcpRelay.RelayGateway"] == "0.0.0.0")) then
        printCLIError ("Invalid relay gateway")
        return false
    end
    return true
end

--
--This routine saves the DMZ configuration
function dmzCfgSave(configRow)
	local errorFlag="OK"
	local statusCode=""
	local statusMessage=""
	local dmzConfig = db.getRowWhere ("ifStatic", "LogicalIfName = 'DMZ' AND AddressFamily = 2")
	if (dmzConfig ~= nil and dmzConfig["ifStatic.StaticIp"] == configRow["ifStatic.StaticIp"]) then
		errorFlag, statusCode = "ERROR", "IP_CONFLICTING_WITH_DMZ_IPADDR"
	else
		local where = "LogicalIfName = 'DMZ' AND AddressFamily = 2"
		local rowid = db.existsRowWhere ("ifStatic", where)
		errorFlag, statusCode = lan.ipv4LanConfig(configRow, rowid, "edit")
		if (errorFlag == "OK") then
			db.save()
			errorFlag, statusCode = dhcp.dhcpConfigWrap (configRow, "DMZ")
		end
	end

	-- save db if no error
	if (errorFlag == "OK") then db.save() end
	statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, DMZGUAGE) or statusCode
	return errorFlag,statusMessage

end

--
--This routine gets the DMZ configuration
function dmzSetupGet()
    local resultTab = {} -- table to store DMZ setup information
    local ifdevTable = nil
    local where = "LogicalIfName = 'DMZ' AND AddressFamily = 2"
    local rowid = db.existsRowWhere ("ifStatic", where)
    local ipv4LanRow = db.getRow("ifStatic", "_ROWID_", rowid )

    printLabel ("DMZ Setup")
    if(ipv4LanRow ~= nil) then
        resTab.insertField (resultTab, "IPv4 Address", ipv4LanRow["ifStatic.StaticIp"] or "")
        resTab.insertField (resultTab, "Subnet Mask", ipv4LanRow["ifStatic.NetMask"]or "")   
    end

    resTab.insertField (resultTab, "DHCP Setup Configuration", "")    
    dhcpTable = db.getRow ("dhcp","_ROWID_", 1)
    dhcpRelayTable = db.getRow ("DhcpRelay","_ROWID_", 1)
    
    if (dhcpTable ~= nil and dhcpRelayTable ~= nil) then 
        if (dhcpTable ["dhcp.dhcpEnabled"] == "0" and dhcpRelayTable ["DhcpRelay.DhcpRelayStatus"] == "1") then
               resTab.insertField (resultTab, "DHCP Mode", "DHCP Relay")
        end
        if (dhcpTable ["dhcp.dhcpEnabled"] == "1" and dhcpRelayTable ["DhcpRelay.DhcpRelayStatus"] == "0") then
               resTab.insertField (resultTab, "DHCP Mode", "DHCP Server")
        end
        if (dhcpTable ["dhcp.dhcpEnabled"] == "0" and dhcpRelayTable ["DhcpRelay.DhcpRelayStatus"] == "0") then
               resTab.insertField (resultTab, "DHCP Mode", "None")
        end
        if (dhcpTable ["dhcp.dhcpEnabled"] == "1") then 
           resTab.insertField (resultTab, "Domain Name", 
	                      dhcpTable["dhcp.domainName"] or "")
           resTab.insertField (resultTab, "Starting IP Address", 
	                      dhcpTable["dhcp.startAddress"] or "")
           resTab.insertField (resultTab, "Ending IP Address", 
			      dhcpTable["dhcp.endAddress"] or "")
           resTab.insertField (resultTab, "Primary DNS Server", 
			      dhcpTable["dhcp.primaryDnsServer"] or "")
           resTab.insertField (resultTab, "Secondary DNS Server", 
			      dhcpTable["dhcp.secondaryDnsServer"] or "")
           resTab.insertField (resultTab, "WINS Server", 
			      dhcpTable["dhcp.winsServer"] or "")
           resTab.insertField (resultTab, "Lease Time in hrs ", 
			      dhcpTable["dhcp.leaseTime"] or "")
        elseif (dhcpRelayTable ["DhcpRelay.DhcpRelayStatus"] == "1") then
            resTab.insertField (resultTab, "Relay Gateway", 
			      dhcpRelayTable["DhcpRelay.RelayGateway"] or "")
        end
    end		               
    IgmpTable = db.getRow ("Igmp","_ROWID_", 1)
    resTab.insertField (resultTab, "DMZ Proxies", "")
    if (dhcpTable["dhcp.enableDnsProxy"] == "1") then 
        resTab.insertField (resultTab, "DNS Proxy", " Enabled")
    else 
        resTab.insertField (resultTab, "DNS Proxy", " Disabled")
    end
    resTab.print (resultTab, 0)
end
