--[[
#### Adarsh Uppula
#### TeamF1
#### www.TeamF1.com
#### June 29, 2007

#### File: dot11.lua
#### Description: dot11 functions

#### Revisions:
01b,23dec07,sdk modified set BSSID secret function.
01a,07nov07,sdk added functions for IAPP.
]]--


--table packages
dot11Interface = {}
dot11Card = {}
dot11ACL = {}
dot11AuthorizedAP = {}
dot11RogueAP = {}
dot11Radio = {}
dot11Profile = {}
dot11VAP = {}
dot11IAPP = {}
dot11IAPPRemoteAP = {}

--************* Functions *************

-- dot11Interface config
function dot11Interface.config (inputTable, rowid, operation)
    -- validate
    if (dot11Interface.inputvalidate(inputTable, operation)) then
        if (operation == "add") then
            return nil
        elseif (operation == "edit") then
            return db.setAttribute("dot11Interface", "_ROWID_", rowid, "vapName", inputTable["dot11Interface.vapName"])
        elseif (operation == "delete") then
            return nil
        end
    end
    return false
end

-- dot11Interface inputvalidate
function dot11Interface.inputvalidate (inputTable, operation)
    if (true) then
        return db.typeAndRangeValidate(inputTable)
    end
    return false
end

-- dot11Profile config
function dot11Profile.config (inputTable, rowid, operation)
    -- validate
    local statFlag = false
    local StatCd = ""
    statFlag, StatCd = dot11Profile.inputvalidate(inputTable, operation)  
    if (statFlag) then
        if (operation == "add") then
            return db.insert("dot11Profile", inputTable)
        elseif (operation == "edit") then
            return db.update("dot11Profile", inputTable, rowid)
        elseif (operation == "delete") then
            for k,v in pairs(inputTable) do
                valid = db.deleteRow ("dot11Profile", "_ROWID_", v)
                if (not valid) then return false end
            end
            return true
        end
    end
    return false, StatCd
end

-- dot11Profile inputvalidate
function dot11Profile.inputvalidate (inputTable, operation)

    -- check wep key length ... (hex length)
    if (inputTable["dot11Profile.security"] == "WEP") then
        local wepKeyLength = inputTable["dot11Profile.groupCipher"]
        local hexLen = (wepKeyLength - 24)/4;
        local asciiLen = hexLen/2;
        if (inputTable["dot11Profile.wepkey0"] and inputTable["dot11Profile.wepkey0"] ~= '') then
            if(string.len (inputTable["dot11Profile.wepkey0"]) ~= asciiLen and string.len (inputTable["dot11Profile.wepkey0"]) ~= hexLen) then
                return false,"WEP_LENGTH_FAILED"
            end                        
            if(dot11VAP.hasTypeValidate(inputTable["dot11Profile.wepkey0"],"%X") == 1 and string.len (inputTable["dot11Profile.wepkey0"]) ~= asciiLen) then            	
            	return false,"WEP_ASCII_LENGTH_FAILED"
            elseif(string.len (inputTable["dot11Profile.wepkey0"]) ~= asciiLen and string.len (inputTable["dot11Profile.wepkey0"]) ~= hexLen) then
            	return false,"WEP_LENGTH_FAILED"            	
            end            
        end               
        if (inputTable["dot11Profile.wepkey1"] and inputTable["dot11Profile.wepkey1"] ~= '') then
            if(string.len (inputTable["dot11Profile.wepkey1"]) ~= asciiLen and string.len (inputTable["dot11Profile.wepkey1"]) ~= hexLen) then
                return false,"WEP_LENGTH_FAILED"
            end
            if(dot11VAP.hasTypeValidate(inputTable["dot11Profile.wepkey1"],"%X") == 1 and string.len (inputTable["dot11Profile.wepkey1"]) ~= asciiLen) then            	
            	return false,"WEP_ASCII_LENGTH_FAILED"
            elseif(string.len (inputTable["dot11Profile.wepkey1"]) ~= asciiLen and string.len (inputTable["dot11Profile.wepkey1"]) ~= hexLen) then
            	return false,"WEP_LENGTH_FAILED"            	
            end
        end
        if (inputTable["dot11Profile.wepkey2"] and inputTable["dot11Profile.wepkey2"] ~= '') then
            if(string.len (inputTable["dot11Profile.wepkey2"]) ~= asciiLen and string.len (inputTable["dot11Profile.wepkey2"]) ~= hexLen) then
                return false,"WEP_LENGTH_FAILED"
            end
            if(dot11VAP.hasTypeValidate(inputTable["dot11Profile.wepkey2"],"%X") == 1 and string.len (inputTable["dot11Profile.wepkey2"]) ~= asciiLen) then            	
            	return false,"WEP_ASCII_LENGTH_FAILED"
            elseif(string.len (inputTable["dot11Profile.wepkey2"]) ~= asciiLen and string.len (inputTable["dot11Profile.wepkey2"]) ~= hexLen) then
            	return false,"WEP_LENGTH_FAILED"            	
            end
        end
        if (inputTable["dot11Profile.wepkey3"] and inputTable["dot11Profile.wepkey3"] ~= '') then
            if(string.len (inputTable["dot11Profile.wepkey3"]) ~= asciiLen and string.len (inputTable["dot11Profile.wepkey3"]) ~= hexLen) then
                return false,"WEP_LENGTH_FAILED"
            end
            if(dot11VAP.hasTypeValidate(inputTable["dot11Profile.wepkey3"],"%X") == 1 and string.len (inputTable["dot11Profile.wepkey3"]) ~= asciiLen) then            	
            	return false,"WEP_ASCII_LENGTH_FAILED"
            elseif(string.len (inputTable["dot11Profile.wepkey3"]) ~= asciiLen and string.len (inputTable["dot11Profile.wepkey3"]) ~= hexLen) then
            	return false,"WEP_LENGTH_FAILED"            	
            end
        end
    elseif((inputTable["dot11Profile.security"] == "WPA") or 
          (inputTable["dot11Profile.security"] == "WPA2") or 
          (inputTable["dot11Profile.security"] == "WPA+WPA2")) and 
          inputTable["dot11Profile.pskPassAscii"] and
          (( string.len (inputTable["dot11Profile.pskPassAscii"]) < 8) or
           ( string.len (inputTable["dot11Profile.pskPassAscii"]) > 63)) then
           return false, "WPA_LENGTH_FAILED"
    end

    if (true) then
        return db.typeAndRangeValidate(inputTable)
    end

    return false
end

-- dot11Radio config
function dot11Radio.config (inputTable, rowid, operation)
    -- validate
    if (dot11Radio.inputvalidate(inputTable, operation)) then
        if (operation == "add") then
            return db.insert("dot11Radio", inputTable)
        elseif (operation == "edit") then
            return db.update("dot11Radio", inputTable, rowid)
        elseif (operation == "delete") then
            return db.delete("dot11Radio", inputTable)
        end
    end
    return false
end

-- dot11Radio inputvalidate
function dot11Radio.inputvalidate (inputTable, operation)
    if (true) then
        return db.typeAndRangeValidate(inputTable)
    end
    return false
end

-- dot11Card config
function dot11Card.config (inputTable, rowid, operation)
    -- validate
    if (dot11Card.inputvalidate(inputTable, operation)) then
        if (operation == "add") then
            return db.insert("dot11Card", inputTable)
        elseif (operation == "edit") then
            return db.update("dot11Card", inputTable, rowid)
        elseif (operation == "delete") then
            return db.delete("dot11Card", inputTable)
        end
    end
    return false
end

-- dot11Card inputvalidate
function dot11Card.inputvalidate (inputTable, operation)
    if (true) then
        return db.typeAndRangeValidate(inputTable)
    end
    return false
end

-- dot11ACL config
function dot11ACL.config (inputTable, rowid, operation)
    -- validate
    if (dot11ACL.inputvalidate(inputTable, operation)) then
        if (operation == "add") then
            return db.insert("dot11ACL", inputTable)
        elseif (operation == "edit") then
            return db.update("dot11ACL", inputTable, rowid)
        elseif (operation == "delete") then
            return db.delete("dot11ACL", inputTable)
        end
    end
    return false
end

-- dot11ACL inputvalidate
function dot11ACL.inputvalidate (inputTable, operation)
    if (true) then
        return db.typeAndRangeValidate(inputTable)
    end
    return false
end

-- dot11AuthorizedAP config
function dot11AuthorizedAP.config (inputTable, rowid, operation)
    -- validate
    if (dot11AuthorizedAP.inputvalidate(inputTable, operation)) then
        if (operation == "add") then
            return db.insert("dot11AuthorizedAP", inputTable)
        elseif (operation == "edit") then
            return db.update("dot11AuthorizedAP", inputTable, rowid)
        elseif (operation == "delete") then
            return db.delete("dot11AuthorizedAP", inputTable)
        end
    end
    return false
end

-- dot11AuthorizedAP inputvalidate
function dot11AuthorizedAP.inputvalidate (inputTable, operation)
    if (true) then
        return db.typeAndRangeValidate(inputTable)
    end
    return false
end

-- dot11RogueAP config
function dot11RogueAP.config (inputTable, rowid, operation)
    -- validate
    if (dot11RogueAP.inputvalidate(inputTable, operation)) then
        if (operation == "delete") then
            return db.delete("dot11RogueAP", inputTable)
        end
    end
    return false
end

-- dot11RogueAP inputvalidate
function dot11RogueAP.inputvalidate (inputTable, operation)
    if (true) then
        return db.typeAndRangeValidate(inputTable)
    end
    return false
end

-- dot11VAP config
function dot11VAP.config (inputTable, rowid, operation)
    -- validate
    if (dot11VAP.inputvalidate(inputTable, operation)) then
        if (operation == "add") then
            return db.insert("dot11VAP", inputTable)
        elseif (operation == "edit") then
            return db.update("dot11VAP", inputTable, rowid)
        elseif (operation == "delete") then
            for k,v in pairs(inputTable) do
                valid = db.deleteRow("dot11VAP", "_ROWID_", v)
                if (not valid) then  return false end
            end
            return true
        end
    end
    return false
end

-- dot11VAP inputvalidate
function dot11VAP.inputvalidate (inputTable, operation)
    local valid = db.typeAndRangeValidate(util.tableSplit(inputTable, ".", "dot11VAP"))

    if (valid == false) then 
        return false
    end

    -- if the VAP is configured as a WDS repeater then make sure that the
    -- profile is configured in Open/WEP mode and that no other VAP on the
    -- box is configured in the WDS repeater mode
    if (inputTable["dot11VAP.dot11Mode"] == "WDS Repeater") then
        -- first the profile check
        local profileRow = db.getRowWhere("dot11Profile", "profileName=='" .. inputTable["dot11VAP.profileName"] .. "'")

        util.appendOut("Profile Security is " .. profileRow["dot11Profile.security"] .. "<br>")
        if(profileRow["dot11Profile.security"] ~= "OPEN" and profileRow["dot11Profile.security"] ~= "WEP") then
            return false
        end

        -- now the WDS repeater exclusivity check
        local table = db.getTable("dot11VAP")
        for k,v in pairs(table) do
            util.appendOut("vapName " .. v["dot11VAP.vapName"] .. ", Mode " .. v["dot11VAP.dot11Mode"] .. "<br>") 
            if(v["dot11VAP.dot11Mode"] == "WDS Repeater" and v["dot11VAP.vapName"] ~= inputTable["dot11VAP.vapName"]) then
                return false
            end
        end
    end


    -- Check for the max number of associated clients. This number should not
    -- exceed a certain value across all AP's combined. Also make sure that the
    -- maxClients are greater than 0
    local maxClientsCount = 0;
    local vapRadios = util.tableSplit(inputTable, ".", "vapRadio")

    for k,v in pairs(vapRadios) do
        local splitparts = util.split(k, ".")
        local radioNo = splitparts[2]
        local assocRows = db.getRows("dot11Interface", "radioNo", radioNo);

        for kk,vv in pairs(assocRows) do
            local vapName = vv["dot11Interface.vapName"];
            local maxClients = db.getAttribute("dot11VAP", "vapName", vapName, "maxClients") or 0;
        
            -- update with the latest value for the vap edited ..
            if (vapName == inputTable["dot11VAP.vapName"]) then
                maxClients = inputTable["dot11VAP.maxClients"];
            end
            
            if (tonumber(maxClients) == 0) then
                return false;
            end

            maxClientsCount = maxClientsCount + maxClients;
            
            -- break if there are any 'unused' slot entries in dot11Interface tbl ... 
            if (vapName == 'unused') then 
                break
            end
        end
    end  

    local maxAllowableClients = tonumber(db.getAttribute("environment", "name", "DOT11_MAX_ASSOC_CLIENTS", "value"))

    if (maxClientsCount > maxAllowableClients) then
        return false; 
    end

    return true

end

-- dot11IAPP enable
function dot11IAPP.enable (rowids, enable)
    local valid = true
    -- enable IAPP on each AP checked
    for k,v in pairs(rowids) do
        local enStr = ""
        if (enable) then 
            enStr = "1" 
        else 
            enStr = "0" 
        end
        -- set iappEnabled
        valid = db.setAttribute("dot11VAP", "_ROWID_", v, "iappEnabled", enStr)

        if(valid == false) then
            return valid
        end
    end

    return valid
end

-- dot11IAPP set bssid secret
function dot11IAPP.setBssidSecret (secret)
    local valid = true

    valid = db.setAttribute("dot11GlobalConfig", "_ROWID_", "1", "iappBssidSecret", secret) 

    return valid
end

-- dot11IAPPRemoteAP config
function dot11IAPPRemoteAP.config (inputTable, rowid, operation)
    -- validate
    if (dot11IAPPRemoteAP.inputvalidate(inputTable, operation)) then
        if (operation == "add") then
            return db.insert("dot11IAPPRemoteAP", inputTable)
        elseif (operation == "edit") then
            return db.update("dot11IAPPRemoteAP", inputTable, rowid)
        elseif (operation == "delete") then
            return db.delete("dot11IAPPRemoteAP", inputTable)
        end
    end
    return false
end

-- dot11IAPPRemoteAP inputvalidate
function dot11IAPPRemoteAP.inputvalidate (inputTable, operation)
    return db.typeAndRangeValidate(inputTable)
end

-- dot11Radio 11n config
function dot11Radio.config11n (inputTable, rowid, operation)
    if (operation == "add") then
       return db.insert("dot11Radio11n", inputTable)
    elseif (operation == "edit") then
       return db.update("dot11Radio11n", inputTable, rowid)
    elseif (operation == "delete") then
       return db.delete("dot11Radio11n", inputTable)
    end
    return false
end

-- Pattern Matching with type
-- args : value , type of matchig (eg: %s = space, %a = alphabets, %p = special characters)
-- except = non matching the special character
-- count =  number of times type should be presents
-- returns error flag = 0  (no error), 1 (error)
function dot11VAP.hasTypeValidate(value,regexp,except,count)
    local num=0
    local flag = 0
    if (value == nil) then
       return 1, "ERR_EMPTY_VALUE_ENTERED"
    end
    if (except ~= nil) then
       for k,v in string.gfind(value,"["..regexp.."]") do
          if(k~=except) then
            flag = 1
          else
            num = num +1
          end
       end
       if (num ~= count and count ~= nil) then
          flag = 1
       end
    else
       if(string.find(value,"["..regexp.."]")) then
          flag = 1
       else
          flag = 0
       end
    end
    return flag
end

