-- dot11CofigCLI.lua
-- Gaurav Mathur
-- TeamF1
-- www.TeamF1.com

-- Modification History
-- 03mar10,rac modified dot11RadioInputVal() function to improve the checks
-- 24feb10,rac modified/added to support CLI supoort for radio,radius,wps and radio advanced configuration
-- 19feb09,ss  added dot11RadioAdvInputVal() routine. 
--             Modified get radio routines to fix spr#10845.
-- 05feb09,ss  modified access point, radio routines, added radio advanced set routine. 
-- 28jan08,gnm modified error message text in input validation routines.
-- 24dec07,gnm modified dot11RadioInputVal() to fetch correct country
--	       code, to display current channel from configuredChannel
--	       field; added dot11RadioCountryCfgSave() 
-- 23dec07,gnm added dot11IAPPInputVal, dot11IAPPCfgSave, dot11IAPPCfgInit
-- 06nov07 gnm written
--
-- Description
-- CLI DOT11 config routines

require "teamf1lualib/dot11View"
require "ifDevLib"
require "teamf1lualib/wps"

--
-- dot11 AP input validation function
function dot11APInputVal(configRow)
    local selProfile = db.getRow ("dot11Profile", "profileName",configRow["dot11VAP.profileName"])
    local vapRadios = util.tableSplit(configRow, ".", "vapRadio")

    if (selProfile == nil) then
	  printCLIError ("Selected profile does not exist.")
     	  return false
 	end

    if (db.tableSize("dot11Radio") == 0) then
       		printCLIError ("No radios configured.")
       		return false
    end
    if (configRow["crontab.unit"] == "1") then
		if (configRow["crontab.startHour"] == "" or configRow["crontab.startHour"] == nil)then
			printCLIError("Enter Valid start hour")
			return false
		end
		if (configRow["crontab.stopHour"] == "" or configRow["crontab.stopHour"] == nil)then
			printCLIError("Enter Valid stop hour")
			return false
		end
		if (configRow["crontab.startMinute"] == "" or configRow["crontab.startMinute"] == nil)then
			printCLIError("Enter Valid start minute")
			return false
		end
		if (configRow["crontab.stopMinute"] == "" or configRow["crontab.stopMinute"] == nil)then
			printCLIError("Enter Valid stop minute")
			return false
		end
		if (configRow["crontab.startMeridian"] == "" or configRow["crontab.startMeridian"] == nil)then
			printCLIError("Enter Valid start meridian")
			return false
		end
		if (configRow["crontab.stopMeridian"] == "" or configRow["crontab.stopMeridian"] == nil)then
			printCLIError("Enter Valid stop meridian")
			return false
		end
		if (configRow["crontab.startMeridian"] == "1" and configRow["crontab.stopMeridian"] == "0")then
				printCLIError("End time hours cannot extend to next day")
				return false
		end
		if (configRow["crontab.startMeridian"] == configRow["crontab.stopMeridian"])then
			if (configRow["crontab.startHour"] >  configRow["crontab.stopHour"])then
				printCLIError("End hour cannot be less then start hour")
				return false
			end	
			if (configRow["crontab.startHour"] == configRow["crontab.stopHour"])then
				if (configRow["crontab.startMinute"] > configRow["crontab.stopMinute"])then
					printCLIError("Please check minutes")
					return false
				end
			end 
		end

    end
    if (configRow["dot11VAP.maxClients"] == nil or
       configRow["dot11VAP.maxClients"] == "") then
        printCLIError ("Invalid maximum associated clients value.")
	return false
    end

    return true
end

--
--Dot11 accesspoint delete function
function dot11APCfgDel(args)
    local idrow = {}
    local errorFlag = "ERROR"
    local statusCode = "DOT11_AP_CONFIG_FAILED"
    local statusMessage = ""
 	configRow=db.getRow("dot11VAP","vapName",args[1])
	if (configRow == nil)then
		printCLIError("Row doesnt exist")
		return "ERROR","Row doesnt exist"
	else
		table.insert (idrow, configRow["dot11VAP._ROWID_"])
		errorFlag, statusCode = dot11.VAP_delete(idrow)
		if (errorFlag == "OK") then db.save() end
	end

    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage
end

--
--Dot11 accesspoint Disable function
function dot11APCfgDisable (args)
    local idrow = {}
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""
 	configRow=db.getRow("dot11VAP","vapName",args[1])
	if (configRow == nil)then
		printCLIError("Row doesnt exist")
		return "ERROR","Row doesnt exist"
    	else
		table.insert (idrow, configRow["dot11VAP._ROWID_"])
		errorFlag, statusCode = dot11.VAP_enable(idrow, false)
    	end
	-- save db if no error
	if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)

    return errorFlag, statusMessage
end

--
--Dot11 accesspoint Enable function
function dot11APCfgEnable (args)
    local idrow = {}
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""

 	configRow=db.getRow("dot11VAP","vapName",args[1])
	if (configRow == nil)then
		printCLIError("Row doesnt exist")
		return "ERROR","Row doesnt exist"
    	else
	table.insert (idrow, configRow["dot11VAP._ROWID_"])    
	errorFlag, statusCode = dot11.VAP_enable(idrow, true)
	-- save db if no error
	if (errorFlag == "OK") then db.save() end
    end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)

    return errorFlag, statusMessage
end

--
--Dot11 accesspoint configuration save function
function dot11APCfgSave (configRow)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""
      -- Check if the row id field exists. If it does not then we  adding a
    -- new row.
    if(configRow["crontab.unit"] == "1")then
	configRow["crontab.unit"]="2"
    end
    local testRow=db.getTable("dot11VAP")
    if (#testRow >= 4)then
		return "ERROR","max accesspoints limit reached"
    end 
    if (configRow["dot11VAP._ROWID_"] == nil) then
        local radioConfig = db.getRow ("dot11Radio", "_ROWID_", "1")
                configRow["dot11VAP.radioList"] = "1"
                configRow["dot11VAP.beaconInterval"] = radioConfig ["dot11Radio.beaconInterval"]
                configRow["dot11VAP.dtimInterval"] = radioConfig ["dot11Radio.dtimInterval"]
                configRow["dot11VAP.rtsThreshold"] = radioConfig ["dot11Radio.rtsThreshold"]
                configRow["dot11VAP.fragThreshold"] = radioConfig ["dot11Radio.fragThreshold"]
                configRow["dot11VAP.preambleMode"] = radioConfig ["dot11Radio.preambleMode"]
                configRow["dot11VAP.rtsCtsProtect"] = radioConfig ["dot11Radio.rtsCtsProtect"]
                configRow["dot11VAP.uapsd"] = radioConfig ["dot11Radio.uapsd"]
                configRow["dot11VAP.txPower"] = radioConfig ["dot11Radio.txPower"]
                configRow["dot11VAP.shortRetry"] = radioConfig ["dot11Radio.shortRetry"]
                configRow["dot11VAP.longRetry"] = radioConfig ["dot11Radio.longRetry"]
                configRow["dot11VAP.apIsolation"] = "0"

        errorFlag, statusCode = dot11.VAP_config(configRow, "-1", "add")
        -- Update the RowId
        --configRow["dot11VAP._ROWID_"] = 
          --  db.getAttribute("dot11VAP", "dot11VAP.vapName", 
            --    configRow["dot11VAP.vapName"], "_ROWID_")
    else
        errorFlag, statusCode = dot11.VAP_config (configRow,
                   configRow["dot11VAP._ROWID_"], "edit")
        --if (errorFlag ~= "ERROR") then
        --errorFlag, statusCode = dot11.VAP_advConfig (configRow, 
        --         configRow["dot11VAP._ROWID_"], "edit")
        --end
    end
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
 
		return  errorFlag, statusMessage 
end

--
-- Dot11 accesspoint initialization function
function dot11APCfgInit(args)
    local primKey = args[1] -- Profile name
    local timeConfig = {}

    local configRow = db.getRow("dot11VAP", "dot11VAP.vapName", primKey)
    if (configRow == nil) then
       configRow = db.getDefaults(true, "dot11VAP")
       configRow["dot11VAP.vapName"] = primKey
    else

		configRow = db.getRow ("dot11VAP", "_ROWID_",configRow["dot11VAP._ROWID_"])
    end
    return -1, configRow
end

-- 
-- This routine checks the WEP key entered so that it satisfies the
-- following constraints
-- a. The length of the key should be 5 ASCII characters (or 10 hex
-- characters) for 64-bit WEP
-- b. The length of the key should be 13 ASCII characters (or 26 hex
-- characters) for 128-bit WEP
local function dot11PfrWEPKeyInputVal(configRow)
    -- ToDo: Implement
    return true
end

--
--dot11 profile input validation function
local function dot11PfrAdvInputVal(configRow)
    local retVal = false
    local tempVal = 0
    local err

    err, tempVal = util.atoi (configRow["dot11Profile.assocTimeout"])
    if  tempVal > 3600 or tempVal < 1 or tempVal == nil then
       printCLIError ("Invalid association time interval value. Valid values between 1 and 999 seconds")
       return retVal
    end
    err, tempVal = util.atoi (configRow["dot11Profile.authTimeout"])
    if  tempVal > 3600 or tempVal < 1 or tempVal == nil then
       printCLIError ("Invalid autentication time interval value. Valid values between 1 and 999 seconds")
       return retVal
    end
    err, tempVal = util.atoi (configRow["dot11Profile.groupKeyUpdateInterval"])
    if  tempVal > 3600 or tempVal < 1 or tempVal == nil then
       printCLIError ("Invalid group key update time interval value. Valid values between 1 and 999 seconds")
       return retVal
    end
    err, tempVal = util.atoi (configRow["dot11Profile.pmksaLifetime"])
    if  tempVal > 3600 or tempVal < 1 or tempVal == nil then
       printCLIError ("Invalid PMKSA Lifetime value. Valid values between 1 and 999 seconds")
       return retVal
    end
    err, tempVal = util.atoi (configRow["dot11Profile.dot1xReauthInterval"])
    if  tempVal > 3600 or tempVal < 1 or tempVal == nil then
       printCLIError ("Invalid 802.1X re-auth time interval value. Valid values between 1 and 999 seconds")
       return retVal
    end

    retVal = true
    return retVal
end

--
--dot11profile input validation function
local function dot11PfrQoSInputVal(configRow)
    local retVal = false
    local idx = 0
    local fld = nil

    for idx = 0, 63 do
        fld = "dscpCosMap" .. idx
        if configRow[fld] == nil then
           configRow[fld] = "Default"
        end
    end

    retVal = true
    return retVal
end

--
-- dot11 profile input validation function
function dot11PfrInputVal(configRow)
    local retVal = false

    -- Check#1
    -- Check if required fields are not nil
    if configRow["dot11Profile.profileName"] == nil then
       printCLIError ("Invalid profile name.")
       return retVal
    end
    if configRow["dot11Profile.ssid"] == nil then
       printCLIError ("Invalid SSID for the access point.")
       return retVal
    end

    -- Check#2
    -- If WEP is chosen, then WEP requried fields should not be nil
    if configRow["dot11Profile.security"] == "WEP" and
       configRow["dot11Profile.pskPassHex"] == nil and 
       configRow["dot11Profile.wepkey0"] == nil and
       configRow["dot11Profile.wepkey1"] == nil and
       configRow["dot11Profile.wepkey2"] == nil and
       configRow["dot11Profile.wepkey3"] == nil then
       printCLIError ("Invalid WEP key.")
       return retVal
    end
 if (configRow["dot11Profile.security"] == "WPA" and configRow["dot11Profile.pairwiseCiphers"] == "CCMP")then
		printCLIError("Invalid encryption type for the selected security type")
		return false
 end
 if (configRow["dot11Profile.security"] == "WPA2" and configRow["dot11Profile.pairwiseCiphers"] == "TKIP")then
		printCLIError("Invalid encryption type for the selected security type")
		return false
 end
 if (configRow["dot11Profile.security"] == "WPA+WPA2" and  configRow["dot11Profile.pairwiseCiphers"] ~= "TKIP+CCMP")then
	printCLIError("Invalid encryption type for the selected security type")
		return false
 end

    -- Check#2
    -- If WPA is chosen, then WPA requried fields should not be nil
    if configRow["dot11Profile.security"] == "WPA" or
       configRow["dot11Profile.security"] == "WPA2" or
       configRow["dot11Profile.security"] == "WPA+WPA2" then
        if configRow["dot11Profile.authMethods"] == nil then
	   printCLIError ("WPA authentication type not specified.")
	   return retVal
	end
        if configRow["dot11Profile.pairwiseCiphers"] == nil then
	   printCLIError ("WPA encryption type not specified.")
	   return retVal
        end
    end
    
    -- Check #3
    -- If WPA authentication is PSK, then WPA password needs
    -- to be specified
    if configRow["dot11Profile.security"] == "WPA" or
       configRow["dot11Profile.security"] == "WPA2" or
       configRow["dot11Profile.security"] == "WPA+WPA2" then
        if configRow["dot11Profile.authMethods"] == "PSK" and 
	   configRow["dot11Profile.pskPassAscii"] == nil then
	   printCLIError ("WPA password not specified.")
	   return retVal
       end
    end

    -- Check #4
    -- If WPA authentication is RADIUS, then clear the WPA password field
    if configRow["dot11Profile.authMethods"] == "RADIUS" and
       configRow["dot11Profile.pskPassAscii"] == nil then
    end

    -- Check #5: If the WEP passphrase was supplied and the WEP key
    -- was not, use the passphrase to generate the WEP key
    if configRow["dot11Profile.security"] == "WEP" then
	-- Check #5a
	if (configRow["dot11Profile.groupCipher"] == nil) then
	    printCLIError ("Invalid WEP Encryption.")	
	    return retVal
	end	   
        -- Check #5b
	if (false == dot11PfrWEPKeyInputVal (configRow)) then
	    printCLIError ("Invalid WEP Key.")	
	    return retVal
	end
    end

    -- Check #6
    -- Check advanced profile configuration information
    if (false == dot11PfrAdvInputVal(configRow)) then
       return retVal
    end
    
    -- Check #7
    -- Set default class of service parameter fields
    if (false == dot11PfrQoSInputVal(configRow)) then
       return retVal
    end
    retVal = true
    return retVal
end

--
-- This routine is used to delete a profile from the system
function dot11PfrCfgDel (args)
    local idrow = {}
    local errorFlag = "ERROR"
    local statusCode = "DOT11_PROFILE_CONFIG_FAILED"
    local statusMessage = ""
    local configRow=db.getRow("dot11Profile","profileName",args[1])
    if (configRow == nil)then
	printCLIError("Row doesnt exist")
	return "ERROR","Row doesnt exist"
    else
	table.insert (idrow, configRow["dot11Profile._ROWID_"])
	errorFlag, statusCode = dot11.profile_config(idrow, "-1", "delete")
	if (errorFlag == "OK") then db.save() end
    end

    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage
end

--
-- This routine will save/add a modified/new profile to the system
function dot11PfrCfgSave (configRow)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""

    -- Check if the row id field exists. If it does not then we  adding a
    -- new row.
    if (configRow["dot11Profile._ROWID_"] == nil) then
        errorFlag, statusCode = dot11.profile_config (configRow, "-1", "add")
	-- Update the RowId
	configRow["dot11Profile._ROWID_"] = 
	    db.getAttribute("dot11Profile", "dot11Profile.profileName", 
	        configRow["dot11Profile.profileName"], "_ROWID_")
    else
        errorFlag, statusCode = dot11.profile_config (configRow, 
		   configRow["dot11Profile._ROWID_"], "edit")
    end

    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)	
    return errorFlag, statusMessage
end

-- 
-- This routine sets the default values for a new 802.11 profile
local function dot11PfrDefaultsSet(configRow)
    configRow["dot11Profile.security"] = "OPEN" -- default
    configRow["dot11Profile.broadcastSSID"] = "0"
    configRow["dot11Profile.assocTimeout"] =
        db.getDef("dot11Profile", "assocTimeout")
    configRow["dot11Profile.authTimeout"] =
        db.getDef("dot11Profile", "authTimeout")
    configRow["dot11Profile.groupKeyUpdateInterval"] =
        db.getDef("dot11Profile", "groupKeyUpdateInterval")
    configRow["dot11Profile.pmksaLifetime"] =
        db.getDef("dot11Profile", "pmksaLifetime")
    configRow["dot11Profile.dot1xReauthInterval"] =
        db.getDef("dot11Profile", "dot1xReauthInterval")
end

--
--This routine will initialize the profile configuration
function dot11PfrCfgInit(args)
    local primKey = args[1] -- Profile name

    local configRow = db.getRow("dot11Profile", "dot11Profile.profileName", 
		    primKey)
    if configRow == nil then
       configRow = {} -- Create an empty table
       configRow["dot11Profile.profileName"] = primKey
       dot11PfrDefaultsSet (configRow)
    end

    return rowId, configRow
end

--
--This routine validates the radio input values
function dot11RadioInputVal (configRow)
	if (configRow["dot11Radio.band"] == "a")then
		if(configRow["dot11Radio.opMode"] == "212992" or configRow["dot11Radio.opMode"] == "196608" or configRow["dot11Radio.opMode"] == "475136")then
		printCLIError("Invalid mode selected")
		return false
		end
	end
	if (configRow["dot11Radio.band"] == "b")then
		if(configRow["dot11Radio.opMode"] == "65536" or configRow["dot11Radio.opMode"] == "393216")then
		printCLIError("Invalid mode selected")
		return false
		end
	end	
	if ((configRow["dot11Radio.opMode"] == "212992" and configRow["dot11Radio11n.chanWidth"] == "2040") or (configRow["dot11Radio.opMode"] == "196608" and configRow["dot11Radio11n.chanWidth"] == "2040")or (configRow["dot11Radio.opMode"] == "65536" and configRow["dot11Radio11n.chanWidth"] == "2040"))then
		printCLIError("Invalid channel spacing")
		return false
	end
	if (configRow["dot11Radio.band"] =="b" and configRow["dot11Radio11n.chanWidth"] == "20")then
		if (tonumber(configRow["dot11Radio.configuredChannel"]) >= 0 and tonumber(configRow["dot11Radio.configuredChannel"]) <= 11)then
		else
			printCLIError("Invalid channel")
			return false
		end
	end
	if (configRow["dot11Radio.band"] =="b" and configRow["dot11Radio11n.chanWidth"] == "2040" and configRow["dot11Radio11n.ctlSideBand"] == "1" and configRow["dot11Radio.opMode"] == "475136")then
		if (configRow["dot11Radio.configuredChannel"] == "0" or (tonumber(configRow["dot11Radio.configuredChannel"]) >=5 and tonumber(configRow["dot11Radio.configuredChannel"]) <= 11))then 
		else
			printCLIError("Invalid channel")
			return false
		end
	end	
		if (configRow["dot11Radio.band"] =="b" and configRow["dot11Radio11n.chanWidth"] == "2040" and configRow["dot11Radio11n.ctlSideBand"] == "0" and configRow["dot11Radio.opMode"] == "475136")then
	if (tonumber(configRow["dot11Radio.configuredChannel"]) >= 0 and tonumber(configRow["dot11Radio.configuredChannel"]) <= 7)then 
		else
			printCLIError("Invalid channel")
			return false
		end
	end	

	if (configRow["dot11Radio.band"] =="b" and configRow["dot11Radio11n.chanWidth"] == "2040" and configRow["dot11Radio11n.ctlSideBand"] == "1" and configRow["dot11Radio.opMode"] == "131072")then
		if (configRow["dot11Radio.configuredChannel"] == "0" or (tonumber(configRow["dot11Radio.configuredChannel"]) >=5 and tonumber(configRow["dot11Radio.configuredChannel"]) <= 11))then 
		else
			printCLIError("Invalid channel")
			return false
		end
	end	
	if (configRow["dot11Radio.band"] =="b" and configRow["dot11Radio11n.chanWidth"] == "2040" and configRow["dot11Radio11n.ctlSideBand"] == "0" and configRow["dot11Radio.opMode"] == "131072")then
	if (tonumber(configRow["dot11Radio.configuredChannel"]) >= 0 and tonumber(configRow["dot11Radio.configuredChannel"]) <= 7)then 
		else
			printCLIError("Invalid channel")
			return false
		end
	end	
	if (configRow["dot11Radio.band"] =="a" and configRow["dot11Radio11n.chanWidth"] == "20")then
		if (configRow["dot11Radio.configuredChannel"] == "0" or (tonumber(configRow["dot11Radio.configuredChannel"]) >= 36 and tonumber(configRow["dot11Radio.configuredChannel"]) <= 165))then
		else
			printCLIError("Invalid channel")
			return false
		end
	end	
	if (configRow["dot11Radio.band"] =="a" and configRow["dot11Radio11n.chanWidth"] == "2040" and configRow["dot11Radio11n.ctlSideBand"] == "1" and configRow["dot11Radio.opMode"] == "393216")then
		if (configRow["dot11Radio.configuredChannel"] == "40" or configRow["dot11Radio.configuredChannel"] == "48" or configRow["dot11Radio.configuredChannel"] == "0" or configRow["dot11Radio.configuredChannel"] == "48" or configRow["dot11Radio.configuredChannel"] == "153" or configRow["dot11Radio.configuredChannel"] == "161" )then
		else
			printCLIError("Invalid channel")
			return false
		end
	end	
	if (configRow["dot11Radio.band"] =="a" and configRow["dot11Radio11n.chanWidth"] == "2040" and configRow["dot11Radio11n.ctlSideBand"] == "1" and configRow["dot11Radio.opMode"] == "131072")then
		if (configRow["dot11Radio.configuredChannel"] == "40" or configRow["dot11Radio.configuredChannel"] == "48" or configRow["dot11Radio.configuredChannel"] == "0" or configRow["dot11Radio.configuredChannel"] == "48" or configRow["dot11Radio.configuredChannel"] == "153" or configRow["dot11Radio.configuredChannel"] == "161" )then
		else
			printCLIError("Invalid channel")
			return false
		end
	end	
	if (configRow["dot11Radio.band"] =="a" and configRow["dot11Radio11n.chanWidth"] == "2040" and configRow["dot11Radio11n.ctlSideBand"] == "0" and configRow["dot11Radio.opMode"] == "393216")then
		if (configRow["dot11Radio.configuredChannel"] == "36" or configRow["dot11Radio.configuredChannel"] == "44" or configRow["dot11Radio.configuredChannel"] == "0" or configRow["dot11Radio.configuredChannel"] == "149" or configRow["dot11Radio.configuredChannel"] == "157")then
		else
			printCLIError("Invalid channel")
			return false
		end
	end	
	if (configRow["dot11Radio.band"] =="a" and configRow["dot11Radio11n.chanWidth"] == "2040" and configRow["dot11Radio11n.ctlSideBand"] == "0" and configRow["dot11Radio.opMode"] == "131072")then
		if (configRow["dot11Radio.configuredChannel"] == "36" or configRow["dot11Radio.configuredChannel"] == "44" or configRow["dot11Radio.configuredChannel"] == "0" or configRow["dot11Radio.configuredChannel"] == "149" or configRow["dot11Radio.configuredChannel"] == "157")then
		else
			printCLIError("Invalid channel")
			return false
		end
	end
return true
end

--
--This routine saves the Radio configuration
function dot11RadioCfgSave (configRow)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""

		configRow["dot11GlobalConfig.country"] = db.getAttribute ("dot11Countries", "country", configRow["dot11GlobalConfig.country"], "countrycode") or configRow["dot11GlobalConfig.country"]
	    local chWidth=""; local ctlSBand="";
	    -- determine the chwidth for mode setting
	    if (configRow["dot11Radio11n.chanWidth"] ~= "20") then chWidth="40"; end
	    -- determine the ctlSBand
	    if (configRow["dot11Radio11n.ctlSideBand"] == "1") then ctlSBand="-"; end
	    if (configRow["dot11Radio11n.ctlSideBand"] == "0") then ctlSBand="+"; end
	
	    configRow["dot11Radio11n.puren"]="0"
	    configRow["dot11Radio11n.mimoPreamble"]="0"
	    -- set the correct opMode to be used ..
	    if (configRow["dot11Radio.opMode"] == "212992") then 
	        configRow["dot11Radio.opMode"]= "b and g"; 
	    elseif (configRow["dot11Radio.opMode"] == "196608") then 
	        configRow["dot11Radio.opMode"]= "g only" 
	    elseif (configRow["dot11Radio.opMode"] == "65536") then 
	        configRow["dot11Radio.opMode"]= "a only"
	    else
	        if (configRow["dot11Radio.opMode"] == "475136") then 
	            configRow["dot11Radio.opMode"]= "ng"
	        elseif (configRow["dot11Radio.opMode"] == "393216") then 
	            configRow["dot11Radio.opMode"]= "na"
	        elseif (configRow["dot11Radio.opMode"] == "131072") then 
	            if (configRow["dot11Radio.band"] == "b") then
	                configRow["dot11Radio.opMode"]= "ng"
	            else
	                configRow["dot11Radio.opMode"]= "na"
	            end
	            -- set puren mode and greenfield mode
	            configRow["dot11Radio11n.puren"]="1"
	            configRow["dot11Radio11n.mimoPreamble"]="0" -- GF override is not needed
	        end 
	        -- add chanwidth, side band info
	        if (configRow["dot11Radio11n.chanWidth"] ~= "20")then
	        configRow["dot11Radio.opMode"]= configRow["dot11Radio.opMode"] .. chWidth .. ctlSBand
		end	
		
	    end
		errorFlag, statusCode = dot11.radio_config(configRow, "1", "edit")
		-- save db if no error
		if (errorFlag == "OK") then db.save() end
	    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)

    return errorFlag, statusMessage
end

--
--This routine initializes the radio configuration
function dot11RadioCfgInit(args)
    local txDefaultsRow = db.getRowWhere ("tableDefaults", "tableName = 'dot11VAP' and columnName='txPower'")
    local txPowerMax = 31
    if (txDefaultsRow ~= nil) then
        txPowerMax = txDefaultsRow["tableDefaults.maxValue"]
    end
	local cardConfig = db.getRow ("dot11Card", "_ROWID_", "1")
	if (cardConfig ~= nil) then
		configRow = db.getRow ("dot11Radio", "cardNo", cardConfig["dot11Card.cardNo"])
		configRow["dot11GlobalConfig.country"] = db.getAttribute ("dot11GlobalConfig", "_ROWID_", "1", "country") or ''
		configRow["dot11GlobalConfig.country"] = db.getAttribute ("dot11Countries", "countrycode", configRow["dot11GlobalConfig.country"], "country") or configRow["dot11GlobalConfig.country"]
		configRow["dot11Radio11n.chanWidth"] = db.getAttribute ("dot11Radio11n", "radioNo", configRow["dot11Radio.radioNo"], "chanWidth") or ''
		configRow["dot11Radio11n.puren"] = db.getAttribute ("dot11Radio11n", "radioNo", configRow["dot11Radio.radioNo"], "puren") or ''	
   	    configRow["dot11Radio11n.mimoPreamble"] = db.getAttribute ("dot11Radio11n", "radioNo", configRow["dot11Radio.radioNo"], "mimoPreamble") or ''
	configRow["dot11Radio11n.ctlSideBand"] = "1"
        -- update mode, ctl side band setting .. 
       if (string.find(configRow["dot11Radio.opMode"], "b and g")) then 
            configRow ["dot11Radio.opMode"] = "212992"; 
       end
       if (string.find(configRow["dot11Radio.opMode"], "g only")) then 
            configRow ["dot11Radio.opMode"] = "196608"; 
       end
       if (string.find(configRow["dot11Radio.opMode"], "a only")) then 
            configRow ["dot11Radio.opMode"] = "65536"; 
       end
       if (string.find(configRow["dot11Radio.opMode"], "ng")) then 
            configRow ["dot11Radio.opMode"] = "475136"; 
       end
       if (string.find(configRow["dot11Radio.opMode"], "na")) then 
            configRow ["dot11Radio.opMode"] = "393216"; 
       end
       if (configRow["dot11Radio11n.puren"] == "1") then
            configRow ["dot11Radio.opMode"] = "131072"; 
       end
	else
		cardConfig = {}
		configRow = {}
	end
	
	-- calling database update program to update current channel using
	-- in auto channel mode.
	if (configRow["dot11Radio.configuredChannel"] == "0") then
		local prog = db.getAttribute("environment", "name", "CHANNEL_DBUPDATE_PROGRAM", "value")
		local dot11IntVapName = db.getAttribute("dot11Interface", "_ROWID_", "1", "interfaceName")
	
		if (prog ~= nil and dot11IntVapName ~= nil) then
			util.appendDebugOut("Exec = " .. os.execute(prog .. " " .. DB_FILE_NAME .. " " .. dot11IntVapName))
 		    configRow["dot11Radio.currentChannel"] = db.getAttribute("dot11Radio",  "cardNo", cardConfig["dot11Card.cardNo"], "currentChannel")
	    end
	end
    -- get the countrycode from unitinfo table
    configuredCountry = db.getAttribute ("unitInfo", "rowid", 1, "regionInfo");

    local txrate = db.getAttribute ('dot11VAP', 'vapEnabled', '1', "txRate")
    configRow["dot11Radio.txRate"] = txrate;
    return 1,configRow
end

--
--This routine initializes the radius confguration
function dot11radiusCfgInit(args)

	configRow = db.getRow("radiusClient", "_ROWID_", "1")
	configRow["radiusClient.authserver2"] = db.getAttribute("radiusClient", "_ROWID_", "2", "authserver")
	if (configRow["radiusClient.authsecret"] ~= nil) then
		configRow["radiusClient.authsecret"] = util.mask (configRow["radiusClient.authsecret"])
	end
	return 1,configRow

end

--
--This routine saves radius confguration
function dot11radiusCfgSave(configRow)
	    if (configRow["radiusClient.authsecret"] ~= nil) then
			if (util.isAllMasked (configRow["radiusClient.authsecret"])) then
			configRow["radiusClient.authsecret"] = db.getAttribute ("radiusClient","_ROWID_","1","authsecret") or configRow["radiusClient.authsecret"]
			end
		end
		
		local tempInputTable = {}
		tempInputTable["radiusClient.authserver"] = configRow["radiusClient.authserver2"]
		configRow["radiusClient.authserver2"] = nil
                tempInputTable["radiusClient.authport"]=configRow["radiusClient.authport"]
		tempInputTable["radiusClient.authtimeout"]=configRow["radiusClient.authtimeout"]
		tempInputTable["radiusClient.authretries"]=configRow["radiusClient.authretries"]
		errorFlag, statusCode = radius.radius_config(configRow, "1", "edit")

		if (errorFlag == "OK") then
			errorFlag, statusCode = radius.radius_config(tempInputTable, "2", "edit")
		end
	
		-- save db if no error
		if (errorFlag == "OK") then db.save() end
	    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
	return errorFlag,statusMessage
	
end

--
--This routine validates the radius confguration
function dot11radiusInputVal(configRow)
	if (configRow["radiusClient.authsecret"] == "" or configRow["radiusClient.authsecret"] == nil) then
		printCLIError("Invalid secret")
		return false
	end
return true
end

--
--This routine initializes the advanced radio confguration
function dot11RadioAdvCfgInit(args)
	local cardConfig = db.getRow ("dot11Card", "_ROWID_", "1")
	if (cardConfig ~= nil) then
		configRow = db.getRow ("dot11Radio", "cardNo", cardConfig["dot11Card.cardNo"])
	else
		configRow = {}
	end
	return 1,configRow
end

--
--This routine saves the advanced radio confguration
function dot11RadioAdvCfgSave(configRow)
		local errorFlag="ERROR"
		local statusCode=""
		local statusMessage=""
		errorFlag, statusCode = dot11.radio_advanced_config(configRow, "1", "edit")
		-- save db if no error
		if (errorFlag == "OK") then db.save() end
	    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
		return errorFlag,statusMessage
end

--
--This routine validates the advanced radio confguration
function dot11RadioAdvInputVal(configRow)
return true
end

--
--This routine initializes the wps confguration
function dot11WPSCfgInit(args)
	configRow = db.getRow ("dot11WPS", "_ROWID_", "1")
	if (configRow == nil) then
		return -1,{}
	else
		configRow["dot11WPS.pin"]="0"
		configRow["dot11WPS.pbc"]="0"
		return 1,configRow
	end
end

--
--This routine save the wps confguration
function dot11WPSSave(configRow)
	local errorFlag="ERROR"
	local statusMessage=""
	local statusCode="" 
	local wpsConfig = db.getRow ("dot11WPS", "_ROWID_", "1")
	if (wpsConfig == nil) then
			errorFlag, statusCode = wps.wpsConfig(configRow, "-1", "add")
	else
			errorFlag, statusCode = wps.wpsConfig(configRow, "1", "edit")
	end
	-- save db if no error
	if (errorFlag == "OK") then db.save() end
    		statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or statusCode

	local status = ""
	if (configRow["dot11WPS.pin"] == "1" and configRow["dot11WPS.pbc"] == "0" and configRow["dot11WPS.wpsEnabled"] == "1") then
        	errorFlag, statusCode = wps.validatePin(configRow["dot11WPS.stationPin"])
	        if (errorFlag == "OK") then
        	    local prog = db.getAttribute("environment", "name", "WPS_PIN_PROGRAM", "value")
			    if (prog ~= nil) then
		    		status = os.execute (prog .. " " .. configRow["dot11WPS.stationPin"])
		 	    end
        	end
        	statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or ''
		return errorFlag,statusMessage 
	end
       if (configRow["dot11WPS.pin"] == "0" and configRow["dot11WPS.pbc"] == "1" and configRow["dot11WPS.wpsEnabled"] == "1") then
		local prog = db.getAttribute("environment", "name", "WPS_PBC_PROGRAM", "value")
		if (prog ~= nil) then
			status = os.execute (prog)
		end
	    	statusMessage = db.getAttribute("stringsMap", "stringId", "STATUS_OK", LANGUAGE) or ''
		return errorFlag,statusMessage	
       end 	
     return errorFlag,statusMessage
end

--
--This routine validates wps confguration
function dot11WPSInputVal(configRow)
	local testTable= db.getRow ("dot11Profile","security","WPA+WPA2")
	if (configRow["dot11WPS.vapName"] == nil or configRow["dot11WPS.vapName"] == "")then
		printCLIError("Please enter the access point name")
		return false
	end 
	local existTable=db.getRow("dot11VAP","vapName",configRow["dot11WPS.vapName"])
	if(testTable == nil)then
		printCLIError("No acess points are confiured with WPA+WPA2 security");
		return false
	end
    if (testTable["dot11Profile.broadcastSSID"] == "0")then
        printCLIError("No acess points are active with WPA+WPA2 security");
		return false

     end
	if(existTable==nil) then
		printCLIError("Acess point doesn't exist")
		return false
	end
return true
end

--
--This function initializes the ACL configuration
function dot11ACLCfgInit(args)
	configRow = db.getRow("dot11VAP","vapName",args[1])
	if (configRow == nil)then
		configRow = {}
	end
	return -1,configRow
end

--
--This function saves the ACL configuration
function dot11ACLSave(configRow)
	local errorFlag="OK"
	local statusCode=""
	local statusMessage=""
	local aclRow={}
	errorFlag, statusCode = dot11.VAP_aclConfig(configRow,configRow["dot11VAP._ROWID_"])
	if (errorFlag == "OK") then db.save() end
	aclRow["dot11ACL.vapName"]=configRow["dot11VAP.vapName"]
	aclRow["dot11ACL.macAddress"]=configRow["dot11ACL.macAddress"]
	if (aclRow["dot11ACL.macAddress"] ~= nil)then
	errorFlag, statusCode = dot11.ACL_config(aclRow, "-1", "add")
	end
	if (errorFlag == "OK") then db.save() end
	statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or statusCode
	return errorFlag,statusMessage
end

--
--This function validates the ACL configuration
function dot11ACLInputVal(configRow)
	if (configRow["dot11VAP._ROWID_"] == nil)then
		printCLIError("accesspoint doesnt exist")
		return false
	end
	return true
end
