-- dot11ShowCLI.lua
-- Gaurav Mathur
-- TeamF1
-- www.TeamF1.com

-- Modification History
-- 03mar10,rac  cleaned up the unnecessary functions and added/modified fucntions to support show for wireless devices
-- 24feb10,rac  added dot11wpsGet() dot11radiusGet() functions and modified dot11RadioCountryGet(),dot11RadioSingleGet() and dot11RadioAdvGet()
-- 05feb09,ss  modified access point, radio routines, added radio advanced get routine.
-- 24dec07,gnm Added dot11RadioCountryGet(); Modified
--	       dot11RadioAllGet() and dot11RadioSingleGet() to display
--	       current channel from the configuredChannel field;
--	       Modified dot11RadioGet() -- added call to dot11RadioCountryGet()
-- 23dec07,gnm Modified dot11ProfileQoSConfigGet and dot11ProfileDSCPMapGet to add
--	       checks for non-existant profiles; Modified dot11IAPPGet
--	       to read IAPP secret from the correct table; Modified
--             dot11StatsGet() to update system stats after command is
--             executed.
-- 06nov07 gnm written
--
-- Description
-- CLI DOT11 get and set routines

require "teamf1lualib/dot11View"

-- get IP TOS/DiffServ mapping for a profile
function dot11ProfileDSCPMapGet (profileName)
    local resultTab = {}
    local fieldNamePrefix = "dscpCosMap" 
    local fieldNameSuffix = 0 
    local fieldNameSuffixMax = 63
    local fieldName = "" 
    local val = ""

    row = db.getRow ("dot11Profile", "dot11Profile.profileName", profileName)
    if (row == nil) then --sanity check
       resTab.insertField (resultTab, "", "No such profile")
       return resultTab
    end
    
    while (fieldNameSuffix <= fieldNameSuffixMax) do
	fieldName = fieldNamePrefix .. fieldNameSuffix
	val = row["dot11Profile." .. fieldName]
	if (string.lower(val) ~= "default") then
	   resTab.insertField (resultTab, "IP DSCP/TOS", fieldNameSuffix)
	   resTab.insertField (resultTab, "Class Of Service", val)	   
        end
	fieldNameSuffix = fieldNameSuffix + 1
    end
    return resultTab
end

--get QoS configuration details for a profile
function dot11ProfileQoSConfigGet (profileName)
    local resultTab = {}    

    row = db.getRow("dot11Profile", "dot11Profile.profileName", profileName)
    if (row == nil) then --sanity check
       resTab.insertField (resultTab, "", "No such profile")
       return resultTab
    end

    resTab.insertYNField (resultTab, row["dot11Profile.qosEnable"] == "1", 
			 "QoS Enable")
    resTab.insertField (resultTab, "Default Class Of Service", 
		       row["dot11Profile.defaultCos"])
    return resultTab
end
      
-- get the field values from the dbase tables
function dot11ProfileSingleGet (profileName, displayQoS)
    local resultTab = {}

    if displayQoS ~= "Y" and displayQoS ~= "N" then --sanity check
       displayQoS = "N"
    end   
    row = db.getRow ("dot11Profile", "dot11Profile.profileName",
		     profileName)
    if (row == nil) then --sanity check
       resTab.insertField (resultTab, "", "No such profile")
       return resultTab
    end
		     
    resTab.insertField (resultTab, "Profile Name", row["dot11Profile.profileName"])
    resTab.insertField (resultTab, "SSID", row["dot11Profile.ssid"])
    if (row["dot11Profile.broadcastSSID"] == "1") then
       resTab.insertField (resultTab, "Broadcast SSID", "Enabled")
    else
        resTab.insertField (resultTab, "Broadcast SSID", "Disabled")
    end
    security = row["dot11Profile.security"]
    resTab.insertField (resultTab, "Security", security)
    if (security == "WEP") then 
	resTab.insertField (resultTab, "Authentication", 
			   row["dot11Profile.wepAuth"])
        resTab.insertField (resultTab, "Encryption",
			   row["dot11Profile.groupCipher"])
        local idx = row["dot11Profile.defWepkeyIdx"]
	resTab.insertField (resultTab, "WEP Key " .. (idx+1),
			       row["dot11Profile.wepkey" .. idx])
    elseif (security == "WPA") or (security == "WPA2") or 
	   (security == "WPA+WPA2") then
	resTab.insertField (resultTab, "Authentication", 
			   row["dot11Profile.authMethods"])
        resTab.insertField (resultTab, "Encryption",
			   row["dot11Profile.pairwiseCiphers"])
	resTab.insertField (resultTab, "WPA Password", 
			   util.mask(row["dot11Profile.pskPassAscii"]))
    end
              

    resTab.insertField (resultTab, "Profile Advanced Configuration","")
    resTab.insertField (resultTab, 
		       "Association Timeout Interval (in Seconds)", 
		       row["dot11Profile.assocTimeout"])
    resTab.insertField (resultTab, 
		       "Authentication Timeout Interval (in Seconds)", 
		       row["dot11Profile.authTimeout"])
    resTab.insertField (resultTab, 
		       "Group Key Refresh Interval (in Seconds)", 
		       row["dot11Profile.groupKeyUpdateInterval"])
    resTab.insertField (resultTab, 
		       "Group Key Refresh Interval (in Seconds)", 
		       row["dot11Profile.groupKeyUpdateInterval"])
    resTab.insertField (resultTab, "PMKSA LifeTime (in Seconds)", 		       
		       row["dot11Profile.pmksaLifetime"])
    resTab.insertField (resultTab, 
		       "802.1X Re-authentication Interval (in Seconds)",
		       row["dot11Profile.dot1xReauthInterval"])
    return resultTab
     
end

-- get the field values from the dbase tables
function dot11ProfileAllGet ()
    local i = 0
    local resultTab = {}
    local profileTab = db.getTable("dot11Profile")

    for k,v in pairs(profileTab) do
	i = i + 1
	local row = profileTab[i]

	resTab.insertField (resultTab, "Profile Name", row["dot11Profile.profileName"])
	resTab.insertField (resultTab, "SSID", row["dot11Profile.ssid"])
	resTab.insertField (resultTab, "Security", row["dot11Profile.security"])

	local encryption = row["dot11Profile.pairwiseCiphers"] or ""
	local authentication = row["dot11Profile.authMethods"] or ""

	if (row["dot11Profile.security"] == "OPEN")  then 
	    encryption = "NONE"
	    authentication = "NONE"
	elseif (row["dot11Profile.security"] == "WEP") then
	       encryption = row["dot11Profile.groupCipher"] or ""
	       authentication = row["dot11Profile.wepAuth"] or ""
	end
	resTab.insertField (resultTab, "Encryption", encryption)
	resTab.insertField (resultTab, "Authentication", authentication)
    end
    return resultTab
end

-- get the field values from the dbase tables
function dot11RadioSingleGet (radioNum)
    local i = 0
    local resultTab = {}

    --fetch radio information from the database
    local radioTable = db.getRow ("dot11Radio", "dot11Radio.radioNo", radioNum)
    local channelSpac = db.getAttribute ("dot11Radio11n", "radioNo", "1", "chanWidth")
    if (radioTable == nil) then  --sanity check
	resTab.insertField (resultTab, "", "No such radio")
	return resultTab
    end 

    --resTab.insertField (resultTab, "Radio", radioTable["dot11Radio.radioNo"])
    if (radioTable["dot11Radio.band"] == "a") then
        band = "5 GHz"
    elseif (radioTable["dot11Radio.band"] == "b") then
        band = "2.4 GHz"
    end
    resTab.insertField (resultTab, "Operating Frequency", band or "") 		 
    resTab.insertField (resultTab, "Mode", radioTable["dot11Radio.opMode"] or "") 		 
    resTab.insertField (resultTab, "Channel Spacing", channelSpac or "") 		 
    if (radioTable["dot11Radio.configuredChannel"] == "0") then
        resTab.insertField (resultTab, "Channel", "Auto")
    else
        resTab.insertField (resultTab, "Channel", 
			   radioTable["dot11Radio.configuredChannel"])
    end
    -- ToDo: This will be changed to the field 'currentChannel' once
    -- the backend implementation is fixed to update that field in
    -- the database with the value of the current channel.
    resTab.insertField (resultTab, "Current Channel", 
			   radioTable["dot11Radio.configuredChannel"])
    if(radioTable["dot11Radio.txPower"] == "31") then
        txpower = "100 %"
    elseif(radioTable["dot11Radio.txPower"] == "23") then
        txpower = "75 %"
    elseif(radioTable["dot11Radio.txPower"] == "17") then
        txpower = "50 %"
    elseif(radioTable["dot11Radio.txPower"] == "8") then
        txpower = "25 %"
    elseif(radioTable["dot11Radio.txPower"] == "0") then
        txpower = "0 %"
    end
    resTab.insertField (resultTab, "Default Transmit Power",  txpower or "")
    --resTab.insertField (resultTab, "Transmit Rate",radioTable["dot11Radio.txRate"])
    return (resultTab)
end

-- get the details for all access points configued
function dot11APAllGet ()
    local i = 0
    local resultTab = {}
    
    local apTable = db.getTableWithJoin({"dot11VAP:dot11Profile:profileName"})
    if (apTable == nil) then --sanity check
       resTab.insertField (resultTab, "", "No APs Configured")
       return resultTab
    end
    for k,v in pairs(apTable) do
	local tempTable = db.getRows("dot11Interface", "dot11Interface.vapName", v["dot11VAP.vapName"])
	local radios = ""
	for kk,vv in pairs(tempTable) do
	    radios = radios .. vv["dot11Interface.radioNo"] .. ", "
	end
	radios = radios:sub(1, radios:len()-2) --shave last comma
	i = i + 1
	local row = apTable[i]
	local timeTab = dot11.VAP_ConfigGet (row["dot11VAP._ROWID_"])	
	resTab.insertField (resultTab, "APName", row["dot11VAP.vapName"])
	if (row["dot11VAP.vapEnabled"] == "0") then
	   resTab.insertField (resultTab, "Enabled", "N")
	else
	   resTab.insertField (resultTab, "Enabled", "Y")
	end
	resTab.insertField (resultTab, "SSID", row["dot11Profile.ssid"])
	if (row["dot11Profile.broadcastSSID"] == "0") then
	   resTab.insertField (resultTab, "Broadcast", "N")
	else
	   resTab.insertField (resultTab, "Broadcast", "Y")
	end

	local profileProps = ""
	if (row["dot11Profile.security"] == "OPEN" or 
	   row["dot11Profile.security"] == "WEP") then
	   profileProps = row["dot11Profile.security"]
	else
	   profileProps = row["dot11Profile.security"] .. " / "
			  .. row["dot11Profile.pairwiseCiphers"] .. " / "
			  .. row["dot11Profile.authMethods"]
	end
	tmpStr = row["dot11VAP.profileName"] .. "(" .. profileProps .. ")"
	resTab.insertField (resultTab, "Profile", tmpStr)
	--resTab.insertField (resultTab, "Radios", radios)
	--resTab.insertField (resultTab, "VLAN_ID",
	--		   row["dot11VAP.vlanId"])
	local activetime="No"
	local starttime="-"
	local stoptime="-"
        if (timeTab["crontab.unit"] == "2") then
            --resTab.insertField (resultTab, "Active Time", "Yes")
           activetime="Yes" 
        	if (timeTab["crontab.startMeridian"] == "1") then 
            		startMeridiem = "PM"
        		elseif (timeTab["crontab.startMeridian"] == "0") then
            startMeridiem = "AM"
        end
        	if (timeTab["crontab.stopMeridian"] == "1") then 
            		stopMeridiem = "PM"
        		elseif (timeTab["crontab.stopMeridian"] == "0") then
            		stopMeridiem = "AM"
        	end
	starttime=timeTab["crontab.startHour"] .. ":" .. timeTab["crontab.startMinute"] .. " " .. startMeridiem
	stoptime=timeTab["crontab.stopHour"] .. ":" .. timeTab["crontab.stopMinute"] .. " " .. stopMeridiem
	end
resTab.insertField (resultTab, "Active Time",activetime) 
resTab.insertField (resultTab, "Start Time",starttime) 
resTab.insertField (resultTab, "Stop Time",stoptime) 
    end
resTab.print (resultTab,0) 
end

-- get ACL policy details for an AP
local function dot11APACLget (apName)
    local idx = 1
    local resultTab = {} --table to store ACL info in

    local vapTab = db.getRow ("dot11VAP", "dot11VAP.vapName", apName)
    local aclTab = db.getRows ("dot11ACL", "dot11ACL.vapName", apName)

    if (vapTab ~= nil) then		    
    resTab.insertField (resultTab, "ACL Policy Status", vapTab["dot11VAP.defACLPolicy"])
    end

    for k,v in pairs(aclTab) do
	resTab.insertField (resultTab, "Host"..idx, v["dot11ACL.macAddress"])
	idx = idx + 1
    end
    return resultTab
end

--  
function dot11RadioAdvGet (radioNum)
    local i = 0
    local resultTab = {} --table to store Radio info in
    local radios = ""

    local apTable = db.getRow("dot11Radio", "dot11Radio.radioNo", radioNum)
    		  
    resTab.insertField (resultTab, "Beacon Interval", apTable["dot11Radio.beaconInterval"])
    resTab.insertField (resultTab, "DTIM Interval", apTable["dot11Radio.dtimInterval"])
    resTab.insertField (resultTab, "RTS Threshold", apTable["dot11Radio.rtsThreshold"])
    resTab.insertField (resultTab, "Frag Threshold", apTable["dot11Radio.fragThreshold"])
    resTab.insertField (resultTab, "Preamble Mode", apTable["dot11Radio.preambleMode"])
    
    if (apTable["dot11Radio.rtsCtsProtect"] == "1") then
        resTab.insertField (resultTab, "Protection Mode:", "CTS-t-Self Protection")
    elseif (apTable["dot11Radio.rtsCtsProtect"] == "0") then
        resTab.insertField (resultTab, "Protection Mode", "None")
    end
    if (apTable["dot11Radio.uapsd"] == "1")then
         resTab.insertField (resultTab, "Power save enable", "Y")
    else
	resTab.insertField (resultTab, "Power save enable", "N")
    end
    resTab.insertField (resultTab, "Short Retry", apTable["dot11Radio.shortRetry"])
    resTab.insertField (resultTab, "Long Retry", apTable["dot11Radio.longRetry"])

    return resultTab
end

-- get for the access point specified by the argument
local function dot11APSingleGet (apName)
    local i = 0
    local resultTab = {} --table to store AP info in
    local radios = ""
    local timeTab = {}

    local apTable = db.getRowWithJoin ({"dot11VAP:dot11Profile:profileName"}, 
		  "dot11VAP.vapName", apName)
    if (apTable == nil) then --sanity check
       resTab.insertField (resultTab, "", "No such AP")
       return resultTab
    end
    	
    timeTab = dot11.VAP_ConfigGet (apTable["dot11VAP._ROWID_"])	  
    resTab.insertField (resultTab, "APName", apTable["dot11VAP.vapName"])
    if (apTable["dot11VAP.vapEnabled"] == "0") then
        resTab.insertField (resultTab, "Enabled", "N")
    else
        resTab.insertField (resultTab, "Enabled", "Y")
    end
    resTab.insertField (resultTab, "Role", apTable["dot11VAP.dot11Mode"])
    resTab.insertField (resultTab, "Profile", apTable["dot11VAP.profileName"])
    
    local tempTable = db.getRows("dot11Interface", "dot11Interface.vapName",
		      apTable["dot11VAP.vapName"])
    local radios = ""	
    for kk,vv in pairs(tempTable) do
	radios = radios .. vv["dot11Interface.radioNo"] .. ", "
    end
    radios = radios:sub(1, radios:len()-2) --shave last comma
    resTab.insertField (resultTab, "Radios", radios)
    --resTab.insertField (resultTab, "Mode", apTable["dot11VAP.opMode"])
    --resTab.insertField (resultTab, "VLAN Enabled", apTable["dot11VAP.vlanEnabled"])
    --resTab.insertField (resultTab, "VLAN ID", db.getDefStr("vlan", "vlanId"))
    resTab.insertField (resultTab, "SSID", apTable["dot11Profile.ssid"])
    resTab.insertField (resultTab, "Max Clients", apTable["dot11VAP.maxClients"])
    --resTab.insertField (resultTab, "Beacon Interval", apTable["dot11VAP.beaconInterval"])
    --resTab.insertField (resultTab, "DTIM Interval", apTable["dot11VAP.dtimInterval"])
    --resTab.insertField (resultTab, "RTS Threshold", apTable["dot11VAP.rtsThreshold"])
    --resTab.insertField (resultTab, "Frag Threshold", apTable["dot11VAP.fragThreshold"])
    --resTab.insertField (resultTab, "Preamble Mode", apTable["dot11VAP.preambleMode"])

    --if (apTable["dot11VAP.rtsCtsProtect"] == "0") then
    --    resTab.insertField (resultTab, "RTS/CTS Protection", "Y")
    --else
    --    resTab.insertField (resultTab, "RTS/CTS Protection", "N")
    --end

    --resTab.insertField (resultTab, "TX Power", apTable["dot11VAP.txPower"])
    --resTab.insertField (resultTab, "Short Retry", apTable["dot11VAP.shortRetry"])
    --resTab.insertField (resultTab, "Long Retry", apTable["dot11VAP.longRetry"])

    local rateStr = ""
	if (apTable["dot11VAP.rate1Mbps"] == "1") then
	   rateStr = rateStr .. " " .. "1"
    end
	if (apTable["dot11VAP.rate2Mbps"] == "1") then
	   rateStr = rateStr .. ", " .. "2"
    end
	if (apTable["dot11VAP.rate5_5Mbps"] == "1") then
	   rateStr = rateStr .. ", " .. "5.5"
    end
	if (apTable["dot11VAP.rate6Mbps"] == "1") then
	   rateStr = rateStr .. ", " .. "6"
    end
	if (apTable["dot11VAP.rate9Mbps"] == "1") then
	   rateStr = rateStr .. ", " .. "9"
    end
	if (apTable["dot11VAP.rate11Mbps"] == "1") then
	   rateStr = rateStr .. ", " .. "11"
    end
	if (apTable["dot11VAP.rate12Mbps"] == "1") then
	   rateStr = rateStr .. ", " .. "12"
    end
	if (apTable["dot11VAP.rate18Mbps"] == "1") then
	   rateStr = rateStr .. ", " .. "18"
    end
	if (apTable["dot11VAP.rate24Mbps"] == "1") then
	   rateStr = rateStr .. ", " .. "24"
    end
	if (apTable["dot11VAP.rate36Mbps"] == "1") then
	   rateStr = rateStr .. ", " .. "36"
    end
	if (apTable["dot11VAP.rate48Mbps"] == "1") then
	   rateStr = rateStr .. ", " .. "48"
    end
	if (apTable["dot11VAP.rate54Mbps"] == "1") then
	   rateStr = rateStr .. ", " .. "54"
    end
    --resTab.insertField (resultTab, "Supported Rate", rateStr)
    if (timeTab ~= nil) then
        if (timeTab["crontab.unit"] == "2") then
            resTab.insertField (resultTab, "Active Time", "Yes")
        if (timeTab["crontab.startMeridian"] == "1") then 
            startMeridiem = "PM"
        elseif (timeTab["crontab.startMeridian"] == "0") then
            startMeridiem = "AM"
        end
        if (timeTab["crontab.stopMeridian"] == "1") then 
            stopMeridiem = "PM"
        elseif (timeTab["crontab.stopMeridian"] == "0") then
            stopMeridiem = "AM"
        end
        resTab.insertField (resultTab, "Start Time", timeTab["crontab.startHour"] .. ":" .. timeTab["crontab.startMinute"] .. " " .. startMeridiem)
        resTab.insertField (resultTab, "Stop Time", timeTab["crontab.stopHour"] .. ":" .. timeTab["crontab.stopMinute"] .. " " .. stopMeridiem)
        else
            resTab.insertField (resultTab, "Active Time", "No")
        end
    end
    return resultTab
end

--

-- This routine gets the statistics for a VAP defined by an Access
-- point identifier and the radio number. If both the access point
-- identifier and the radio number are not specified, this routine
-- will display statistics for all the VAPs

function dot11StatsGet (args)
    local apName = args[1]
    local radio = args[2]
    
    if (apName ~= "all" and radio ~= "-1") then
        resultTab = dot11APDetailedStatsGet (apName, radio)
	printLabel ("AP Detailed Statistics")
	resTab.print (resultTab, 0)
    else
        printLabel ("Radio Statistics")
	resultTab = dot11RadioStatsGet ()
        resTab.print (resultTab, 0)
	printLabel ("AP Statistics")
	resultTab = dot11APStatsGet ()
	resTab.print (resultTab, 0)
    end
    -- run program to read the stats and update DB
    local prog = db.getAttribute("environment", "name", "IFDEVSTATS_PROGRAM", "value")
    os.execute(prog .. " " .. DB_FILE_NAME)
end   

--
-- main function for dot11 rogueap get
-- 
function dot11RogueAPGet (args)
    printLabel ("Authorized AP Mac Addresses")
    resultTab = dot11RAPAllGet ()
    resTab.print (resultTab, 0)
    printLabel ("Detected RogueAPs")
    resultTab = dot11RAPDetAllGet ()	
    resTab.print (resultTab, 0)
end   

--
-- main function for dot11 get
--
function dot11APGet (args)
   local apName = args[1]

   -- We build different types of tables depending on whether
   -- the AP name is given or not
   if (apName ~= "all") then
	printLabel ("AP Configuration")
	resultTab = dot11APSingleGet (apName)
	resTab.print (resultTab, 8)
	printLabel ("ACL Policy and MAC Addresses")
	resultTab = dot11APACLget (apName)
	resTab.print (resultTab, 8)
   else
	resultTab = dot11APAllGet ()
	--resTab.print (resultTab, 8)
   end
   -- Now print the result table   
end

--
-- main function for dot11 radio get
-- 
function dot11RadioGet (args)
    local radioNum = 1
   
	printLabel ("Radio Configuration")
	resultTab = dot11RadioSingleGet (radioNum)
	resTab.print (resultTab, 0)
	printLabel ("Radio Advanced Configuration")
        resultTab = dot11RadioAdvGet (radioNum)
	resTab.print (resultTab, 0)
end   

--
-- main function for dot11 profile get
-- 
function dot11ProfileGet (args)
    local profileName = args[1]
    local displayQoS = args[2]   
    
    if (profileName ~= "all") then
   	resultTab = dot11ProfileSingleGet (profileName, displayQoS)
	printLabel ("Profile Configuration")
	resTab.print (resultTab, 0)
	if (displayQoS == "Y") then
	   printLabel ("Profile QoS Configuration")
	   resultTab = dot11ProfileQoSConfigGet (profileName)
	   resTab.print (resultTab, 0)
	   resultTab = dot11ProfileDSCPMapGet (profileName)
	   printLabel ("IP TOS/DiffServ Mapping (non default values)")
	   resTab.print (resultTab, 0)
        end
    else
	resultTab = dot11ProfileAllGet ()
        resTab.print (resultTab, 0)
    end
end   

function dot11radiusGet(args)
   local resultTab={}
   local PrimaryRadiusTable = db.getRow("radiusClient","_ROWID_","1")
   local SecondaryRadiusTable = db.getRow("radiusClient","_ROWID_","2")
   resTab.insertField (resultTab,"Authentication_Server_IP_Address(Primary)",PrimaryRadiusTable["radiusClient.authserver"])
   resTab.insertField (resultTab,"Authentication_Server_IP_Address(Secondary)",SecondaryRadiusTable["radiusClient.authserver"])
   resTab.insertField (resultTab,"Authentication Port",PrimaryRadiusTable["radiusClient.authport"])
   resTab.insertField (resultTab,"Secret",PrimaryRadiusTable["radiusClient.authsecret"])
   resTab.insertField (resultTab,"Timeout",PrimaryRadiusTable["radiusClient.authtimeout"])
   resTab.insertField (resultTab,"Retries",PrimaryRadiusTable["radiusClient.authretries"])
   printLabel ("Radius Configuration")
   resTab.print (resultTab, 0)    			 
   
end
function dot11wpsGet(args)
	local resultTab={}
	local wpsTable=db.getRow("dot11WPS","_ROWID_","1")
	if (wpsTable == nil)then
		printCLIError("WPS not configured")		
	else
		resTab.insertField (resultTab,"Access Point Name",wpsTable["dot11WPS.vapName"])
		if (wpsTable["dot11WPS.wpsEnabled"] == "1") then
			resTab.insertField (resultTab,"WPS Enabled","Y")
		else
			resTab.insertField (resultTab,"WPS Enabled","N")
		end
		resTab.print (resultTab, 0)
	end
	
end
--
-- 
function dot11ACLGet (args)
    local apName = args[1]
    local label = "" 
    local resultTab = {}   
    local where = "vapName = '" .. apName .. "'"

    printLabel ("Default ACL Policy")    
    local defPol = db.getAttribute("dot11VAP", "dot11VAP.vapName", apName, 
	  "defACLPolicy")
    resTab.insertField (resultTab, "ACL Policy Status", defPol)    	  
    resTab.print (resultTab, defPol)
    
    resultTab = {}
    local table = db.getRowsWhere("dot11ACL", where)
    for _, row in pairs(table) do
        resTab.insertField (resultTab, "", row["dot11ACL.macAddress"])
    end

    printLabel ("List of MAC Address")    
    resTab.print (resultTab, 0)
end
--
--
function dot11WirelessGet(args)
local resultTab={}
printLabel("Wireless Statistics")
              local enabledVAPs = db.getRowsWhere ("dot11VAP", "vapEnabled = 1")

                    for m,n in pairs (enabledVAPs) do

                        local vapName = n["dot11VAP.vapName"] or ''
						local table = db.getRowsWithJoin({"dot11VAP:dot11Interface:vapName"}, "dot11VAP.vapName", vapName)
						for k,v in pairs(table) do
				   			local table1 = db.getRowsWhere("dot11STA", "interfaceName = '" .. v["dot11Interface.interfaceName"] .. "'")
							for i,row in pairs(table1) do
				           		row["dot11VAP.vapName"] = v["dot11VAP.vapName"]
				           		row["dot11Interface.radioNo"] = v["dot11Interface.radioNo"]
				           		local timeElapsed = dot11.STA_connectedTime(row["dot11STA.timeConnected"])
			resTab.insertField (resultTab,"AP Name",row["dot11VAP.vapName"] or '')
			resTab.insertField (resultTab,"MAC Address",row["dot11STA.macAddress"]or '')
			resTab.insertField (resultTab,"Radio",row["dot11Interface.radioNo"] or '')
			resTab.insertField (resultTab,"Security",row["dot11STA.security"] or '')
			resTab.insertField (resultTab,"Authentication",row["dot11STA.authentication"] or '')			
			resTab.insertField (resultTab,"Time Connected",timeElapsed)
							end
                        end
                    end
			resTab.print (resultTab, 0)
	
end
function dot11accesspointGet(args)
local resultTab={}
local otherresultTab={}
	local table = db.getRowsWithJoin({"dot11VAP:dot11Interface:vapName"}, "dot11VAP.vapName",args[1])
				    for k,v in pairs(table) do
					local row = db.getRow("interfaceStats", "interfaceStats.interfaceName", v["dot11Interface.interfaceName"])
						row["dot11VAP.vapName"] = args[1]
						row["dot11Interface.radioNo"] = v["dot11Interface.radioNo"]
						row["dot11Interface.rowid"] = v["dot11Interface._ROWID_"]
						resTab.insertField (resultTab,"AP Name",row["dot11VAP.vapName"] or '')
						resTab.insertField (resultTab,"Radio",row["dot11Interface.radioNo"] or '')
						resTab.insertField (resultTab,"PktRx",row["interfaceStats.rx_packets"] or '')
						resTab.insertField (resultTab,"PktTx",row["interfaceStats.tx_packets"] or '')
						resTab.insertField (resultTab,"ByteRx",row["interfaceStats.rx_bytes"] or '')
						resTab.insertField (resultTab,"ByteTx",row["interfaceStats.tx_bytes"] or '')
						resTab.insertField (resultTab,"ErrRx",row["interfaceStats.rx_errors"] or '')
						resTab.insertField (resultTab,"ErrTx",row["interfaceStats.tx_errors"] or '')						   
						resTab.insertField (resultTab,"DropRx",row["interfaceStats.rx_dropped"] or '')
						resTab.insertField (resultTab,"DropTx",row["interfaceStats.tx_dropped"] or '')
						resTab.insertField (resultTab,"MCast",row["interfaceStats.multicast"] or '')
						resTab.insertField (resultTab,"#Coll",row["interfaceStats.collisions"] or '' )
					end	
					printLabel("Access Point Status")	
						resTab.print (resultTab, 0)
							printLabel("Connected Clients")
	local testtable = db.getRowsWithJoin({"dot11VAP:dot11Interface:vapName"}, "dot11VAP.vapName",args[1])
						for k,v in pairs(testtable) do
				   			local table1 = db.getRowsWhere("dot11STA", "interfaceName = '" .. v["dot11Interface.interfaceName"] .. "'")
							for i,row in pairs(table1) do
				           		row["dot11Interface.radioNo"] = v["dot11Interface.radioNo"]
				           		local timeElapsed = dot11.STA_connectedTime(row["dot11STA.timeConnected"])
							resTab.insertField (otherresultTab,"MAC Address",row["dot11STA.macAddress"])
							resTab.insertField (otherresultTab,"Radio",row["dot11Interface.radioNo"])
							resTab.insertField (otherresultTab,"Security",row["dot11STA.security"])
							resTab.insertField (otherresultTab,"Encryption",row["dot11STA.cipher"])
							resTab.insertField (otherresultTab,"Authentication",row["dot11STA.authentication"])
							resTab.insertField (otherresultTab,"Time Connected",timeElapsed)
							printLabel(" ")
							end
                            end

							resTab.print (otherresultTab,0)	
						
end
function dot11accesspointallGet(args)
local accesspoints=db.getTable("dot11VAP","vapName",nil)
local i=0
	printLabel("Wireless Statistics")
local resultTab={}
for k,v in pairs(accesspoints) do
	i=i+1
	local test=accesspoints[i]
	local table = db.getRowsWithJoin({"dot11VAP:dot11Interface:vapName"}, "dot11VAP.vapName",test["dot11VAP.vapName"])
				    for k,v in pairs(table) do
					local row = db.getRow("interfaceStats", "interfaceStats.interfaceName", v["dot11Interface.interfaceName"])
						row["dot11VAP.vapName"] = test["dot11VAP.vapName"]
						row["dot11Interface.radioNo"] = v["dot11Interface.radioNo"]
						row["dot11Interface.rowid"] = v["dot11Interface._ROWID_"]
						resTab.insertField (resultTab,"AP Name",row["dot11VAP.vapName"] or '')
						resTab.insertField (resultTab,"Radio",row["dot11Interface.radioNo"] or '')
						resTab.insertField (resultTab,"PktRx",row["interfaceStats.rx_packets"] or '')
						resTab.insertField (resultTab,"PktTx",row["interfaceStats.tx_packets"] or '')
						resTab.insertField (resultTab,"ByteRx",row["interfaceStats.rx_bytes"] or '')
						resTab.insertField (resultTab,"ByteTx",row["interfaceStats.tx_bytes"] or '')
						resTab.insertField (resultTab,"ErrRx",row["interfaceStats.rx_errors"] or '')
						resTab.insertField (resultTab,"ErrTx",row["interfaceStats.tx_errors"] or '')						   
						resTab.insertField (resultTab,"DropRx",row["interfaceStats.rx_dropped"] or '')
						resTab.insertField (resultTab,"DropTx",row["interfaceStats.tx_dropped"] or '')
						resTab.insertField (resultTab,"MCast",row["interfaceStats.multicast"] or '')
						resTab.insertField (resultTab,"#coll",row["interfaceStats.collisions"] or '' )
					end	
end
						resTab.print (resultTab, 0)
end
