--[[
#### Adarsh Uppula
#### TeamF1
#### www.TeamF1.com
#### June 29, 2007

#### File: dot11.lua
#### Description: dot11 functions

#### Revisions:
01a,07nov07,sdk added functions for IAPP.
01b,11Sep08,shv changes with respect to radio configuration.
]]--


--************* Requires *************

require "teamf1lualib/ifDev"
require "teamf1lualib/dot11"
require "teamf1lualib/vlan"
require "teamf1lualib/crontab"
require "teamf1lualib/bridgeLib"

--************* Initial Code *************

--package dot11
dot11 = {}

--************* Functions *************

-- dot11VAP config
function dot11.VAP_config (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	
	db.beginTransaction() --begin transaction
	local valid = false

	--Converting the time into 24 hours format.
    local startTimeConfig = {}
    if (inputTable["crontab.startMeridian"] == "1") then
        if (tonumber(inputTable["crontab.startHour"]) == 12) then
            startTimeConfig["crontab.hour"] = "12"
        else
            startTimeConfig["crontab.hour"] = inputTable["crontab.startHour"] + 12
        end
        startTimeConfig["crontab.minute"] = inputTable["crontab.startMinute"]
    else
        if (tonumber(inputTable["crontab.startHour"]) == 12) then
            startTimeConfig["crontab.hour"] = "00"
        else
            startTimeConfig["crontab.hour"] = inputTable["crontab.startHour"]
        end
        startTimeConfig["crontab.minute"] = inputTable["crontab.startMinute"]
    end

    local stopTimeConfig = {}
    if (inputTable["crontab.stopMeridian"] == "1") then
        if (tonumber(inputTable["crontab.stopHour"]) == 12) then
            stopTimeConfig["crontab.hour"] = "12"
        else
            stopTimeConfig["crontab.hour"] = inputTable["crontab.stopHour"] + 12
        end
        stopTimeConfig["crontab.minute"] = inputTable["crontab.stopMinute"]
    else
        if (tonumber(inputTable["crontab.stopHour"]) == 12) then
            stopTimeConfig["crontab.hour"] = "00"
        else
            stopTimeConfig["crontab.hour"] = inputTable["crontab.stopHour"]
        end
        stopTimeConfig["crontab.minute"] = inputTable["crontab.stopMinute"]
    end

    -- Updating crontab configuration.
    startTimeConfig["crontab.unit"] = inputTable["crontab.unit"]
    stopTimeConfig["crontab.unit"] = inputTable["crontab.unit"]
    
    --Enbale or disable VAP based on current time falling within active time schedule
    if(inputTable["crontab.unit"]=="2") then
    	
    	local curHr = tonumber(os.date("%H"))
    	local curMin = tonumber(os.date("%M"))
    	setStrHr = tonumber(startTimeConfig["crontab.hour"])
    	setStrMin = tonumber(startTimeConfig["crontab.minute"])
    	setStpHr = tonumber(stopTimeConfig["crontab.hour"])
    	setStpMin = tonumber(stopTimeConfig["crontab.minute"])
    	setStrMins = (setStrHr*60) + setStrMin
    	setStpMins = (setStpHr*60) + setStpMin
    	curMins = (curHr*60) + curMin
    	if ( (curMins > setStrMins) and (curMins < setStpMins)) then
    		inputTable["dot11VAP.vapEnabled"] = "1"
      else
    		inputTable["dot11VAP.vapEnabled"] = "0"
    	end
    else
	    if(rowid == "-1") then
	    	inputTable["dot11VAP.vapEnabled"] = "1"
	    end   	

    end

	local vapName = ""
    local cmd = "/pfrm2.0/bin/stopStartVapInstance.sh %d %s";

	if(operation == "add") then
    	vapName = inputTable["dot11VAP.vapName"]
    else
    	vapName = db.getAttribute("dot11VAP", "dot11VAP._ROWID_", rowid, "vapName")
        inputTable["dot11VAP.vapName"] = vapName
    end
    local vapStartScheduleCmd = string.format (cmd, 1, vapName)
    local vapEndScheduleCmd   = string.format (cmd, 0, vapName)
    startTimeConfig["crontab.command"] = vapStartScheduleCmd
    stopTimeConfig["crontab.command"] = vapEndScheduleCmd

    -- remove existing cron entries for this vap
    db.deleteRow("crontab", "command", vapStartScheduleCmd)
    db.deleteRow("crontab", "command", vapEndScheduleCmd)
	 -- enter time entries into crontab table if time interval is set
    if(inputTable["crontab.unit"]=="2") then 
    	valid = db.insert("crontab", startTimeConfig)
    	if (valid) then
        valid = db.insert("crontab", stopTimeConfig)
    	end
    else
    	valid=true
    end

    local freeRow = db.existsRowWhere("dot11Interface", "vapName = 'unused'")
    -- changing vapName in dot11Interface table from unused to vapName
    if(operation == "add") then
        local inTable = {}
        local apIfaceRows = db.getRowsWhere ("dot11Interface", "vapName = '" .. inputTable["dot11VAP.vapName"] .. "'")
        if (apIfaceRows and #apIfaceRows == 0) then
            inTable["dot11Interface.vapName"] = inputTable["dot11VAP.vapName"]
            valid = dot11Interface.config(inTable, freeRow, "edit")
        else
            valid = false
        end
    end

    -- update new ssid value in vlanPortMgmt table
    if (operation == "edit") then
        
        local newSSID = db.getAttribute ("dot11Profile", "profileName", inputTable["dot11VAP.profileName"], "ssid")
    	local rowsToChange = db.getRowsWithJoin({"dot11VAP:dot11Interface:vapName"}, "dot11VAP.vapName",vapName)

	    for k,v in pairs(rowsToChange) do
		    if (valid) then
                valid = db.setAttributeWhere ("VlanPortMgmt", "PhyInterfaceName='"..v["dot11Interface.interfaceName"].."'", "PortName", newSSID)
    		end
    	end
    -- get the vap enabled state
    --inputTable ["dot11VAP.vapEnabled"] = db.getAttribute ("dot11VAP", "vapName", vapName, "vapEnabled")
    end

    -- get the radio level vap properties if any. This concept won't work in
    -- view of multi radio setup so we assume a single radio for now.
    if ((operation == "add") and valid) then
        local radio = db.getRow ("dot11Radio", "rowid", 1)
        local radio11n = db.getRow ("dot11Radio11n", "rowid", 1)

        if (not inputTable["dot11VAP.txPower"]) then
            inputTable["dot11VAP.txPower"] = radio["dot11Radio.txPower"]
        end

        if (not inputTable["dot11VAP.txRate"]) then
            inputTable["dot11VAP.txRate"] = radio["dot11Radio.txRate"]
        end

        if (not inputTable["dot11VAP.puren"]) then
            inputTable["dot11VAP.puren"] = radio11n["dot11Radio11n.puren"]
        end
    end
	 
    --Updating dot11VAP configuration.
    if (valid) then
        valid = dot11VAP.config (inputTable, rowid, operation)
    end

	-- config bridgePorts
	if (valid) then
		valid = dot11.VAP_portEnable (inputTable["dot11VAP.vapName"], true)
	end
	-- update new ssid value in vlanPortMgmt table
    if (operation == "add") then
        local newSSID = db.getAttribute ("dot11Profile", "profileName", inputTable["dot11VAP.profileName"], "ssid")
    	local rowsToChange = db.getRowsWithJoin({"dot11VAP:dot11Interface:vapName"}, "dot11VAP.vapName", inputTable["dot11VAP.vapName"])

	    for k,v in pairs(rowsToChange) do
		    if (valid) then
                valid = db.setAttributeWhere ("VlanPortMgmt", "PhyInterfaceName='"..v["dot11Interface.interfaceName"].."'", "PortName", newSSID)
    		end
    	end
    end
    
    -- update vlan-per-SSID entry
    if (operation == "add" or inputTable["dot11VAP.vapEnabled"] == "1") then
        local ssid = db.getAttribute ("dot11Profile", "profileName", inputTable["dot11VAP.profileName"], "ssid");
        valid = dot11.VAP_vlanPerSSID_config (inputTable["dot11VAP.vapName"], ssid, "BRIDGE")
    end

    -- return
	if (valid) then
		db.commitTransaction()
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "DOT11_AP_CONFIG_FAILED"
	end
end

function dot11.VAP_ConfigGet (rowid)
    local vapName = db.getAttribute ("dot11VAP", "_ROWID_", rowid, "vapName") or ''
    local cmd = "/pfrm2.0/bin/stopStartVapInstance.sh %d %s";

    local vapStartScheduleCmd = string.format (cmd, 1, vapName)
    local vapEndScheduleCmd   = string.format (cmd, 0, vapName)

    local startTimeConfig = db.getRow ("crontab", "command", vapStartScheduleCmd)
    local stopTimeConfig = db.getRow ("crontab", "command", vapEndScheduleCmd)
    local crontabConfig = {}

    if (startTimeConfig == nil) then
        crontabConfig["crontab.startHour"] = ""
        crontabConfig["crontab.startMinute"] = ""
        crontabConfig["crontab.startMeridian"] = ""
        crontabConfig["crontab.unit"] = "0"
    else
        crontabConfig["crontab.unit"] = startTimeConfig["crontab.unit"]
        if (tonumber (startTimeConfig["crontab.hour"]) == 0) then
            crontabConfig["crontab.startMeridian"] = "0"
            crontabConfig["crontab.startHour"] = "12"
        elseif (tonumber (startTimeConfig["crontab.hour"]) > 12) then
            crontabConfig["crontab.startMeridian"] = "1"
            crontabConfig["crontab.startHour"] = tonumber (startTimeConfig["crontab.hour"]) - 12
        elseif (tonumber (startTimeConfig["crontab.hour"]) == 12) then
           crontabConfig["crontab.startMeridian"] = "1"
           crontabConfig["crontab.startHour"] = "12"
        else
            crontabConfig["crontab.startHour"] = startTimeConfig["crontab.hour"]
            crontabConfig["crontab.startMeridian"] = "0"
        end
        crontabConfig["crontab.startMinute"] = startTimeConfig["crontab.minute"]
    end

    if (stopTimeConfig == nil ) then
        crontabConfig["crontab.stopHour"] = ""
        crontabConfig["crontab.stopMinute"] = ""
        crontabConfig["crontab.stopMeridian"] = ""
        crontabConfig["crontab.unit"] = "0"
    else
        crontabConfig["crontab.unit"] = stopTimeConfig["crontab.unit"]
        if (tonumber (stopTimeConfig["crontab.hour"]) == 0) then
            crontabConfig["crontab.stopMeridian"] = "0"
            crontabConfig["crontab.stopHour"] = "12"
        elseif (tonumber (stopTimeConfig["crontab.hour"]) > 12) then
            crontabConfig["crontab.stopMeridian"] = "1"
            crontabConfig["crontab.stopHour"] = tonumber (stopTimeConfig["crontab.hour"]) - 12
        elseif (tonumber (stopTimeConfig["crontab.hour"]) == 12) then
        	crontabConfig["crontab.stopMeridian"] = "1"
        	crontabConfig["crontab.stopHour"] = "12"
        else
            crontabConfig["crontab.stopHour"] = stopTimeConfig["crontab.hour"]
            crontabConfig["crontab.stopMeridian"] = "0"
        end
        crontabConfig["crontab.stopMinute"] = stopTimeConfig["crontab.minute"]
    end

    return crontabConfig
end

-- dot11VAP adv config
function dot11.VAP_advConfig (inputTable, rowid)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	
	db.beginTransaction() --begin transaction
	local valid = true
	local inTable = {}
	local vapName = db.getAttribute ("dot11VAP", "rowid", rowid, "vapName");

	-- de-config bridgePorts
	if (valid) then
		valid = dot11.VAP_portEnable (vapName, false)
	end

	-- config dot11VAP
	inTable = util.tableSplit(inputTable, ".", "dot11VAP")
	valid = dot11VAP.config(inTable, rowid, "edit")
    
	-- config bridgePorts
	if (valid) then
		valid = dot11.VAP_portEnable (vapName, true)
	end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "DOT11_ADV_AP_CONFIG_FAILED"
	end
end

-- dot11VAP delete
function dot11.VAP_delete (rowids)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = true
	local vlanDels = {}
	-- for each ap
	for k,v in pairs(rowids) do
		local row = db.getRow("dot11VAP", "dot11VAP._ROWID_", v)

		-- dont't allow delete if wps is using this Ap
        local dot11VapName =  row["dot11VAP.vapName"]
        local wpsRow = db.getRow("dot11WPS","vapName",dot11VapName)
        if (wpsRow ~= nil) then
            if (wpsRow["dot11WPS.wpsEnabled"] == "1") then
                db.rollback()
                return "ERROR", "DELETE_FROM_WPS"
            end
        end
        
        -- delete ACL entires for the ap
        local aclRows = db.getRows ("dot11ACL", "vapName", row["dot11VAP.vapName"])
        for kk,vv in pairs (aclRows) do
            local inTable = {}
            local rowid = db.getAttribute ("dot11ACL", "macAddress", vv["dot11ACL.macAddress"], "rowid")
            inTable ["dot11ACL._ROWID_"] = rowid
            valid = dot11ACL.config (inTable, "-1", "delete")
            if (not valid) then break end
        end

		-- if no row, break
		if (not row) then return "nonexistant delete" end
		
        -- update vlan-per-SSID entry
        local ssid = db.getAttribute ("dot11Profile", "profileName", row["dot11VAP.profileName"], "ssid");
        valid = dot11.VAP_vlanPerSSID_config (row["dot11VAP.vapName"], ssid, "VAP-DISABLED")

        -- disable bridgeport entry of the vap
    	if (valid) then
    		valid = dot11.VAP_portEnable (row["dot11VAP.vapName"], false)
    	end

		-- edit dot11Interface rows 
		local inTable = {}
		local freeRow = db.existsRowWhere("dot11Interface", "vapName = '" .. row["dot11VAP.vapName"] .. "'")
		inTable["dot11Interface.vapName"] = "unused"
		dot11Interface.config(inTable, freeRow, "edit")

		-- add to delete vlan
		table.insert(vlanDels, row["dot11VAP.vlanId"])
	end
	
	-- delete actual APs
	if (valid) then	
		for k,v in pairs(rowids) do
            local inTable = {}
            inTable["dot11VAP._ROWID_"] = v;
		    valid = dot11VAP.config(inTable, "-1", "delete")
            if (not valid) then break end
        end
	end

	-- delete vlan if possible
	if (valid) then
		for k,v in pairs(vlanDels) do
			local inTable = {}
			inTable["vlan.vlanId"] = v
			valid = vlan.config(inTable, "-1", "delete")
            if (not valid) then break end
		end
	end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "DOT11_AP_DEL_FAILED"
	end
end

-- dot11VAP enable
function dot11.VAP_enable (rowids, enable)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false
	
	-- enable/disable each AP checked
	for k,v in pairs(rowids) do
		local enStr = ""
		if (enable) then enStr = "1" else enStr = "0" end

		local row = db.getRow("dot11VAP", "dot11VAP._ROWID_", v)

        -- if vap is being disabled
        if (not enable) then
            
             
           -- checking if we are disabling access points which are being used by wps.If yes then don't disable
         --[[   local dot11VapName =  row["dot11VAP.vapName"]
            local wpsRow = db.getRow("dot11WPS","vapName",dot11VapName)
            if ( wpsRow ~= nil) then
                if ( wpsRow["dot11WPS.wpsEnabled"] == "1" ) then
                    db.rollback()
                    return "ERROR", "DISABLE_FROM_WPS"
                end
            end  
            --]]
            
            -- update vlan-per-SSID entry
            local ssid = db.getAttribute ("dot11Profile", "profileName", row["dot11VAP.profileName"], "ssid");
            valid = dot11.VAP_vlanPerSSID_config (row["dot11VAP.vapName"], ssid, "VAP-DISABLED")

            -- disable bridgeport entry of the vap
        	if (valid) then
        		valid = dot11.VAP_portEnable (row["dot11VAP.vapName"], false)
        	end

		    -- set vapEnabled to 0
            if (valid) then
        		valid = db.setAttribute("dot11VAP", "_ROWID_", v, "vapEnabled", enStr)	
            end
        end

        -- if vap is being enabled
        if (enable) then
		    -- set vapEnabled to 1
      		valid = db.setAttribute("dot11VAP", "_ROWID_", v, "vapEnabled", enStr)	

            -- enable bridgeport entry of the vap
        	if (valid) then
        		valid = dot11.VAP_portEnable (row["dot11VAP.vapName"], false)
        	end

            -- update vlan-per-SSID entry
            local ssid = db.getAttribute ("dot11Profile", "profileName", row["dot11VAP.profileName"], "ssid");
            if (valid) then
                valid = dot11.VAP_vlanPerSSID_config (row["dot11VAP.vapName"], ssid, "BRIDGE")
            end
        end
	end
	
	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
                if (enable) then
                    return "ERROR", "DOT11_AP_ENABLE_FAILED"
                else
                    return "ERROR", "DOT11_AP_DISABLE_FAILED"
                end

	end
end

-- dot11VAP enable helper
function dot11.VAP_portEnable (vapname, enable)
	local valid = true
	if (enable) then enStr = "1" else enStr = "0" end
	local rowsToChange = db.getRowsWithJoin({"dot11VAP:dot11Interface:vapName"}, "dot11VAP.vapName", vapname)

	-- set portEnabled
	for k,v in pairs(rowsToChange) do
		if (valid) then
			valid = db.setAttribute("bridgePorts", "interfaceName", v["dot11Interface.interfaceName"], "portEnabled", enStr)
		end
	end
	return valid
end

-- dot11VAP vlanPerSSID helper
function dot11.VAP_vlanPerSSID_config (vapname, ssid, portType)
	local valid = true
	if (enable) then enStr = "1" else enStr = "0" end
	local rowsToChange = db.getRowsWithJoin({"dot11VAP:dot11Interface:vapName"}, "dot11VAP.vapName", vapname)

    -- update vlan-per-SSID for interfaces using this vap
	for k,v in pairs(rowsToChange) do
		if (valid) then
            valid = db.setAttributeWhere ("VlanPortMgmt", "portName='"..ssid.."' AND PhyInterfaceName='"..v["dot11Interface.interfaceName"].."'", "PortType", portType)
		end
	end
	return valid
end

-- dot11Profile helper
function dot11.VapsOnProfile_portEnable (profileName, enable)
    local vapEntries = db.getRowsWhere("dot11VAP", "vapEnabled = '1' and profileName = '" .. profileName .. "'")
    local rows = {}
    for k,v in pairs (vapEntries) do
        dot11.VAP_portEnable (v["dot11VAP.vapName"], enable)
    end
end

-- dot11Profile config
function dot11.profile_config (inputTable, rowid, operation)
	local profileUseFlag = "0"
    local profileName = db.getAttribute("dot11Profile", "dot11Profile._ROWID_", rowid, "profileName")

	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	
	db.beginTransaction() --begin transaction
	local valid = false
    local oldSSID
    
    --checks for WPS enabled profie having security as WPA/WPA2 trying to chage to OPEN or WEP
    if (operation == "edit") then
    	proSec = db.getAttribute("dot11Profile", "dot11Profile._ROWID_", rowid, "security")
    	if (proSec == "WPA" or proSec == "WPA2" or proSec == "WPA+WPA2") then
			apRows = db.getRows ("dot11VAP", "profileName", profileName)
			if (#(apRows) ~= 0) then
				for i = 1,#(apRows) do
					row = apRows[i]
					disCheck = db.getAttribute("dot11WPS", "vapName", row["dot11VAP.vapName"], "wpsEnabled")
					if (disCheck ~=nil and disCheck == "1") then
						if(inputTable["dot11Profile.security"] == "OPEN" or inputTable["dot11Profile.security"] == "WEP") then
							return "ERROR", "WPA_VAP_USE_ERROR"
						end
					end
				end
			end
		end
	end
	
    -- disable vaps on bridgePorts
	if (operation == "edit") then
        dot11.VapsOnProfile_portEnable (profileName, 0)

       -- get the current ssid value
       oldSSID = db.getAttribute("dot11Profile", "dot11Profile._ROWID_", rowid, "ssid")
   	end

    -- Checking if any profile is being used by Ap. If yes, then don't allow delete.
    if(operation == "delete") then
        for k,v in pairs(inputTable) do
            local profilename = db.getAttribute("dot11Profile","_ROWID_",v,"profileName")
            local vapRow = db.getRow("dot11VAP","profileName",profilename)
            if(vapRow ~= nil) then
                db.rollback()
                return "ERROR", "AP_USING_PROFILE"
            end                 
        end
    end	

    -- config dot11Profile     	
    valid, statCd = dot11Profile.config(inputTable, rowid, operation)

    -- enable vaps on bridgePorts
    if (valid and operation == "edit") then
        dot11.VapsOnProfile_portEnable (profileName, 1)

        -- config vlanPortMgmt table only if the profile is being used on any AP
        vapTable = db.getTable("dot11VAP")
        local i = 0 
        for k,v in pairs(vapTable) do
            i = i+1
            row  = vapTable[i]
            if (row["dot11VAP.profileName"] == profileName) then
                profileUseFlag = "1"
            end
        end

        if (profileUseFlag == "1") then
            --obtain those interfaces for which the profile is being used
            local table = db.getRows ("dot11VAP", "profileName", profileName)
            for i = 1,#(table) do
                row = table[i]
                intName = db.getAttribute("dot11Interface","vapName",row["dot11VAP.vapName"],"interfaceName")
                valid = db.setAttribute("VlanPortMgmt","PhyInterfaceName",intName,"portName",inputTable["dot11Profile.ssid"])
            end
        end
    end

    -- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()		
		if(statCd ~= "") then
			return "ERROR", statCd
		else
			return "ERROR", "DOT11_PROFILE_CONFIG_FAILED"
		end
	end
end

-- dot11Profile adv config
function dot11.profile_advConfig (inputTable, rowid)
	local profileUseFlag = "0"
	local profileName = db.getAttribute("dot11Profile", "dot11Profile._ROWID_", rowid, "profileName")
	local ssid = db.getAttribute("dot11Profile", "dot11Profile._ROWID_", rowid, "ssid")	

	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	
	-- disable vaps on bridgePorts
    dot11.VapsOnProfile_portEnable (profileName, 0)
	
	db.beginTransaction() --begin transaction
	local valid = false
	
	-- config dot11Profile
	valid = dot11Profile.config(inputTable, rowid, "edit")

    -- enable vaps on bridgePorts
    dot11.VapsOnProfile_portEnable (profileName, 1)
    
    -- config vlanPortMgmt table only if the profile is being used on any AP
    vapTable = db.getTable("dot11VAP")
    local i = 0 
    for k,v in pairs(vapTable) do
        i = i+1
        row  = vapTable[i]
        if (row["dot11VAP.profileName"] == profileName) then
            profileUseFlag = "1"
        end
    end

    if (profileUseFlag == "1") then
        --obtain those interfaces for which the profile is being used
        table = db.getRows ("dot11VAP", "profileName", profileName)
        for i = 1,#(table) do
            row = table[i]
            intName = db.getAttribute("dot11Interface","vapName",row["dot11VAP.vapName"],"interfaceName")
            valid = db.setAttribute("VlanPortMgmt","PhyInterfaceName",intName,"portName",ssid)
        end
    end    
	
	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "DOT11_ADV_PROFILE_CONFIG_FAILED"
	end
end

-- dot11Profile QoS config
function dot11.profile_qosConfig (inputTable, rowid)
	local profileUseFlag = "0"
	local profileName = db.getAttribute("dot11Profile", "dot11Profile._ROWID_", rowid, "profileName")
	local ssid = db.getAttribute("dot11Profile", "dot11Profile._ROWID_", rowid, "ssid")
	
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	
	db.beginTransaction() --begin transaction
	local valid = false
    
    -- disable vaps on bridgePorts
    dot11.VapsOnProfile_portEnable (profileName, 0)
	
	-- config dot11Profile
	valid = dot11Profile.config(inputTable, rowid, "edit")
    
    -- enable vaps on bridgePorts
    dot11.VapsOnProfile_portEnable (profileName, 1)
	
    -- config vlanPortMgmt table only if the profile is being used on any AP
    vapTable = db.getTable("dot11VAP")
    local i = 0 
    for k,v in pairs(vapTable) do
        i = i+1
        row  = vapTable[i]
        if (row["dot11VAP.profileName"] == profileName) then
            profileUseFlag = "1"
        end
    end
	if (profileUseFlag == "1") then
        --obtain those interfaces for which the profile is being used
        table = db.getRows ("dot11VAP", "profileName", profileName)
        for i = 1,#(table) do
            row = table[i]
            intName = db.getAttribute("dot11Interface","vapName",row["dot11VAP.vapName"],"interfaceName")
            valid = db.setAttribute("VlanPortMgmt","PhyInterfaceName",intName,"portName",ssid)
        end
    end
	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "DOT11_QOS_CONFIG_FAILED"
	end
end

-- dot11Card config
function dot11.card_config (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false
	
	-- config dot11Card
	valid = dot11Card.config(inputTable, rowid, operation)
	
	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "DOT11_CARD_CONFIG_FAILED"
	end
end

-- dot11AuthorizedAP config
function dot11.authorizedap_config (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false

    -- make sure BSSID is unique in dot11RogueAP dot11AuthorizedAP lists
    local rogueTbl = db.getRows ("dot11RogueAP", "macAddress", inputTable["dot11AuthorizedAP.macAddress"] or "")

    if ((not rougeTbl) and (rogueTbl and #rogueTbl == 0)) then
        valid = true;
    end
    
	-- config dot11AuthorizedAP
    if (valid) then
  	    valid = dot11AuthorizedAP.config(inputTable, rowid, operation)
    end
	
	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "DOT11_AUTH_AP_CONFIG_FAILED"
	end
end

-- dot11RogueAP move
function dot11.rogueap_move (rowids)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false
	
    -- add to dot11AuthorizedAP first

	for k,v in pairs(rowids) do
        local row = db.getRow("dot11RogueAP", "dot11RogueAP._ROWID_", v)
		if (row) then 
            local inputTable = {}
            inputTable ["dot11AuthorizedAP.macAddress"] = row["dot11RogueAP.macAddress"]
            inputTable ["dot11AuthorizedAP.SSID"] = row["dot11RogueAP.SSID"]
            inputTable ["dot11AuthorizedAP.security"] = row["dot11RogueAP.security"]
            inputTable ["dot11AuthorizedAP.pairwiseCiphers"] = row["dot11RogueAP.pairwiseCiphers"]
            inputTable ["dot11AuthorizedAP.authMethods"] = row["dot11RogueAP.authMethods"]

            valid = dot11AuthorizedAP.config(inputTable, "-1", "add")
            if (not valid) then
                break;
            end
        end
    end

	-- delete from dot11RogueAP
    if (valid) then
	    valid = dot11RogueAP.config(rowids, "-1", "delete")
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "DOT11_ROGUE_AP_MOVE_FAILED"
	end
end

-- dot11ACL config
function dot11.ACL_config (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false
	
	-- config dot11ACL
	valid = dot11ACL.config(inputTable, rowid, operation)
	
	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "DOT11_ACL_CONFIG_FAILED"
	end
end

-- WPS config
function dot11.WPS_config (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false
	
	-- config dot11ACL
	valid = dot11WPS.config(inputTable, rowid, operation)
	
	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "DOT11_WPS_CONFIG_FAILED"
	end
end

-- WPS_Station config
function dot11.WPS_Station_config (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false
	
	-- config dot11ACL
	valid = dot11WPSStations.config(inputTable, rowid, operation)
	
	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "DOT11_WPS_CONFIG_FAILED"
	end
end

-- dot11Radio config
function dot11.radio_config (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

    db.beginTransaction() --begin transaction
	local valid = false
	
	-- update VAP entries for txPower
	local table1 = db.getTable ("dot11VAP")
    if (#table1 == 0) then
        valid = true
    else
        for k,v in pairs (table1) do
            v["dot11VAP.txPower"] = inputTable["dot11Radio.txPower"]
            v["dot11VAP.txRate"] = inputTable["dot11Radio.txRate"]
            v["dot11VAP.puren"] = inputTable["dot11Radio11n.puren"]

            -- also, set the crontab related info since we are doing an edit
            local timeConfig = dot11.VAP_ConfigGet (v["dot11VAP._ROWID_"])
            v["crontab.unit"] = timeConfig ["crontab.unit"]
            v["crontab.startHour"] = timeConfig ["crontab.startHour"]
            v["crontab.startMinute"] = timeConfig ["crontab.startMinute"]
            v["crontab.startMeridian"] = timeConfig ["crontab.startMeridian"]
            v["crontab.stopHour"] = timeConfig ["crontab.stopHour"]
            v["crontab.stopMinute"] = timeConfig ["crontab.stopMinute"]
            v["crontab.stopMeridian"] = timeConfig ["crontab.stopMeridian"]

            valid = dot11.VAP_config(v, v["dot11VAP._ROWID_"], "edit")
            if (valid == false) then
                break
            end
        end
    end

    -- Updating Country configuration.
    local countryConfig = db.getRow ("dot11GlobalConfig", "_ROWID_", rowid)
    if (valid and countryConfig ~= nil) then
        countryConfig["dot11GlobalConfig.country"] = inputTable["dot11GlobalConfig.country"]
        valid = db.update ("dot11GlobalConfig", countryConfig, rowid)
    else
        valid = false
    end

    -- Updating dot11 Radio configuration.
    if (valid) then
        local cardConfig = db.getRow ("dot11Card", "_ROWID_", "1")
        local cardNo = cardConfig["dot11Card.cardNo"]
        if (cardNo ~= nil) then
            local radioConfig = db.getRow ("dot11Radio", "cardNo", cardNo)
            if (radioConfig ~= nil) then
                if (valid) then
                    radioConfig["dot11Radio.opMode"] = inputTable["dot11Radio.opMode"]
                    radioConfig["dot11Radio.configuredChannel"] = inputTable["dot11Radio.configuredChannel"]
                    radioConfig["dot11Radio.txPower"] = inputTable["dot11Radio.txPower"]
                    radioConfig["dot11Radio.txRate"] = inputTable["dot11Radio.txRate"]
                    radioConfig["dot11Radio.band"] = inputTable["dot11Radio.band"]
                    if (inputTable["dot11Radio.dothEnabled"] ~= nil) then
                        radioConfig["dot11Radio.dothEnabled"] = inputTable["dot11Radio.dothEnabled"]
                    end
                    valid = dot11Radio.config (radioConfig, radioConfig["dot11Radio._ROWID_"], "edit")
                end
                local radio11nConfig = db.getRow ("dot11Radio11n", "radioNo", radioConfig["dot11Radio.radioNo"])
                if (radio11nConfig ~= nil) then
                    radio11nConfig["dot11Radio11n.chanWidth"] = inputTable["dot11Radio11n.chanWidth"]
                    radio11nConfig["dot11Radio11n.puren"] = inputTable["dot11Radio11n.puren"]
                    radio11nConfig["dot11Radio11n.mimoPreamble"] = inputTable["dot11Radio11n.mimoPreamble"]
                    valid = dot11Radio.config11n (radio11nConfig, radio11nConfig["dot11Radio11n._ROWID_"], "edit")
                end
            end
        end
    end
    
	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "DOT11_RADIO_CONFIG_FAILED"
	end
end

-- dot11Radio Advanced config
function dot11.radio_advanced_config (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false

    -- Seeting the advanced radio parameters
    valid = dot11Radio.config (inputTable, rowid, operation)

    if (valid) then
        -- setting the advanced parameters in all AP configuration
        local table = db.getTable ("dot11VAP")
        if (#table == 0) then
            valid = true
        else
            for k,v in pairs (table) do
                v["dot11VAP.beaconInterval"] = inputTable["dot11Radio.beaconInterval"]
                v["dot11VAP.dtimInterval"] = inputTable["dot11Radio.dtimInterval"]
                v["dot11VAP.rtsThreshold"] = inputTable["dot11Radio.rtsThreshold"]
                v["dot11VAP.fragThreshold"] = inputTable["dot11Radio.fragThreshold"]
                v["dot11VAP.preambleMode"] = inputTable["dot11Radio.preambleMode"]
                v["dot11VAP.rtsCtsProtect"] = inputTable["dot11Radio.rtsCtsProtect"]
                v["dot11VAP.uapsd"] = inputTable["dot11Radio.uapsd"]
                v["dot11VAP.shortRetry"] = inputTable["dot11Radio.shortRetry"]
                v["dot11VAP.longRetry"] = inputTable["dot11Radio.longRetry"]

            	-- de-config bridgePorts
                if (valid and v["dot11VAP.vapEnabled"] == "1") then
             		valid = dot11.VAP_portEnable (v["dot11VAP.vapName"], false)
            	end

	            -- config dot11VAP
                valid = dot11VAP.config(v, v["dot11VAP._ROWID_"], "edit")
                if (valid == false) then
                    break
                end

            	-- config bridgePorts
            	if (valid and v["dot11VAP.vapEnabled"] == "1") then
            		valid = dot11.VAP_portEnable (v["dot11VAP.vapName"], true)
            	end

                -- configure VLAN-perSSID
                local ssid = db.getAttribute ("dot11Profile", "profileName", v["dot11VAP.profileName"], "ssid");
            	if (valid and v["dot11VAP.vapEnabled"] == "1") then
                    valid = dot11.VAP_vlanPerSSID_config (v["dot11VAP.vapName"], ssid, "BRIDGE")
                end
            end
        end
    end
	
	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "DOT11_RADIO_CONFIG_FAILED"
	end
end

-- dot11countries config
function dot11.country_config (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false
	
	-- config dot11 country
    valid = db.update ("dot11GlobalConfig", inputTable, 1)
	
	-- return
	if (valid) then
        rebootFlag=1
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "DOT11_RADIO_CONFIG_FAILED"
	end
end


-- IAPP enable
function dot11.IAPP_enable (rowids, enable)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction

    local valid = false
    local action = "disable" 

    if(enable) then
        action = "enable"
    end

    -- config dot11Radio
    valid = dot11IAPP.enable(rowids, enable)

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        if (enable) then 
            return "ERROR", "DOT11_IAPP_ENABLE_FAILED" 
        else 
            return "ERROR", "DOT11_IAPP_DISABLE_FAILED" 
        end 
    end
end

-- dot11IAPPRemoteAP config
function dot11.IAPPRemoteAP_config (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false

    -- config dot11Profile
    valid = dot11IAPPRemoteAP.config(inputTable, rowid, operation)

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "DOT11_IAPP_CONFIG_FAILED_REMOTE"
    end
end

-- Set IAPP Bssid Secret 
function dot11.IAPP_setBssidSecret (secret)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false

    -- config dot11Profile
    valid = dot11IAPP.setBssidSecret(secret)

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "DOT11_IAPP_CONFIG_FAILED_BSSID"
    end
end

-- dot11Radio 11n config
function dot11.radio11n_config (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false
	
	-- config dot11Radio
	valid = dot11Radio.config11n(inputTable, rowid, operation)
	
	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "DOT11_RADIO_CONFIG_FAILED"
	end
end

-- dot11VAP ACL mode config
function dot11.VAP_aclConfig (inputTable, rowid)

	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	
	db.beginTransaction() --begin transaction
	local valid = true
	local inTable = {}
	local vapName = db.getAttribute ("dot11VAP", "rowid", rowid, "vapName");

    -- dont't allow edit if wps is using this Ap
  --[[
    local wpsRow = db.getRow("dot11WPS","vapName",vapName)
    if (wpsRow ~= nil) then
        if (wpsRow["dot11WPS.wpsEnabled"] == "1") then
            db.rollback()
            return "ERROR", "CAN_NOT_EDIT_ACCESS_CONTROL"        
        end
    end    
    --]]

	-- de-config bridgePorts
	if (valid) then
		valid = dot11.VAP_portEnable (vapName, false)
	end

	-- config dot11VAP (default acl policy)
	valid = db.setAttribute("dot11VAP", "vapName", vapName, "defACLPolicy", inputTable["dot11VAP.defACLPolicy"])

	-- config bridgePorts
	if (valid) then
		valid = dot11.VAP_portEnable (vapName, true)
	end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "DOT11_ADV_AP_CONFIG_FAILED"
	end
end

-- dot11STA connected time
function dot11.STA_connectedTime (timeConnected)
    -- if join time is zero then return '-'
	if (timeConnected == 0 or timeConnected == nil) then
		return "-"
	end

    local status, uptime = timeLib.uptime()

    local timeElapsed = uptime - timeConnected

    local mins = math.floor(timeElapsed / 60)
    local sec = timeElapsed % 60
    local hours = math.floor (mins / 60)
    mins = mins % 60
    local days = math.floor (hours / 24)
    hours = hours % 24

    timeElapsed = days .. " days, " .. hours .. " hours, " .. mins .. " minutes, " .. sec .. " seconds"

    return timeElapsed
end
--  wirelessSetupWizard config
function dot11.wirelessWizardconfig(inputTable)
    
        db.beginTransaction() --begin transaction
	    local valid = false
	    
        local wpsTable = {}
        local broadcastSSID = 1
        local security = "WPA+WPA2"
        local pairwiseCiphers = "TKIP+CCMP"
        local authMethods = "PSK"
        local preAuthStatus = 0
                
        wpsTable["dot11Profile.pskPassAscii"] = inputTable["dot11Profile.pskPassAscii"]
        
        wpsTable["dot11Profile.ssid"] =  inputTable["dot11Profile.ssid"]
        wpsTable["dot11Profile.security"] =  security
        wpsTable["dot11Profile.broadcastSSID"] =  broadcastSSID
        wpsTable["dot11Profile.pairwiseCiphers"] = pairwiseCiphers 
        wpsTable["dot11Profile.authMethods"] = authMethods 
        wpsTable["dot11Profile.preAuthStatus"] = preAuthStatus        
        valid = dot11.profile_config(wpsTable,"1","edit")
        if (valid) then
		    db.commitTransaction()
		    return "OK", "STATUS_OK"
	    else
		    db.rollback()
		    return "ERROR", "WIRELESS_WIZARD_FAILED"
	    end         
 end

	
