-- firewall-ruleCLI.lua
-- Neelima
-- TeamF1
-- 10/10/08
-- 
-- Modification History
-- 27feb10,rac  added functons to support ALGs and modified all other functions to support the new changes 
-- 17feb09,ss   added validation checks to fix spr#10808 
-- 31jan09,ss   modified accessControlRuleInfoGet() to fix spr#10544 
-- 10dec08,ss   did modifications for rmi 
-- 07oct08,nlm, written
--
-- Description
-- CLISH Firewall rule LUA routines
--
-- Required Libraries

require "teamf1lualib/fwView"
require "teamf1lualib/fwRules"
require "teamf1lualib/algs"

function accessControlRuleInfoGet (args)
    local firewallTable = db.getTable ("FirewallRules")
    local resultTab = {} -- table to store firewall rules
    local firewallRow = nil
    local rowId = "0"
    local status = nil
    local fromzone = nil
    local tozone = nil
    local destusrs = nil
    local srcusrs = nil
    local action = nil
    local status = nil

    printLabel ("Default Outbound Policy")
    defRow = db.getRow("FirewallRules", "RuleType", "DEFAULT_POLICY")
    if (defRow["FirewallRules.Action"] == "ACCEPT") then 
        print ("For IPv4 : Allow Always\n")
    elseif(defRow["FirewallRules.Action"] == "DROP") then
        print ("For IPv4 : Block Always")
    end
    printLabel ("List of Available IPv4 Firewall Rules")    
    for col, val in pairs(firewallTable) do
        rowId = rowId+1
        local firewallRow = firewallTable[rowId]
	if (rowId ~= nil) then
            if (firewallRow["FirewallRules._metadata"] ~= "edit=0" and firewallRow["FirewallRules.RuleType"] ~= "DEFAULT_POLICY") then
                if(firewallRow["FirewallRules.Status"] == "0") then
                    status =  "Disabled"
                elseif(firewallRow["FirewallRules.Status"] == "1") then
                    status =  "Enabled"
                end
                if(firewallRow["FirewallRules.FromZoneType"] == "SECURE") then
                    fromzone = "WAN"
		elseif (firewallRow["FirewallRules.FromZoneType"] == "INSECURE")then
		    fromzone = "LAN"
		else
		    fromzone = "PUBLIC"
                end
	  	if(firewallRow["FirewallRules.ToZoneType"] == "SECURE") then
                    tozone = "WAN"
		elseif (firewallRow["FirewallRules.ToZoneType"] == "INSECURE")then
		    tozone = "LAN"
		else
		    tozone = "PUBLIC"
                end
	
                if (firewallRow["FirewallRules.Action"] == "ACCEPT") then
                    action = "ALLOW Always"
                elseif (firewallRow["FirewallRules.Action"] == "DROP") then
                    action = "BLOCK Always"
                elseif (firewallRow["FirewallRules.Action"] == "DROP_BY_SCHEDULE") then
                    action = "BLOCK by schedule, otherwise allow"
                elseif (firewallRow["FirewallRules.Action"] == "ACCEPT_BY_SCHEDULE") then
                    action = "ALLOW by schedule,otherwise block"
                end
                if(firewallRow["FirewallRules.SourceAddressType"] == "0") then
                    srcusrs = "Any"
                elseif(firewallRow["FirewallRules.SourceAddressType"] == "1") then
                    srcusrs = firewallRow["FirewallRules.SourceAddressStart"]
                elseif(firewallRow["FirewallRules.SourceAddressType"] == "2") then
                    srcusrs = firewallRow["FirewallRules.SourceAddressStart"] .. " - " .. firewallRow["FirewallRules.SourceAddressEnd"]
                end
                if(firewallRow["FirewallRules.DestinationAddressType"] == "0") then
                    destusrs = "Any"
                elseif(firewallRow["FirewallRules.DestinationAddressType"] == "1") then
                    destusrs = firewallRow["FirewallRules.DestinationAddressStart"]
                elseif(firewallRow["FirewallRules.DestinationAddressType"] == "2") then
                    destusrs = firewallRow["FirewallRules.DestinationAddressStart"] .. " - " .. firewallRow["FirewallRules.DestinationAddressEnd"]
                end
                if (firewallRow["FirewallRules.LogLevel"] == "1") then
                    logLevel = "Never" 
                elseif (firewallRow["FirewallRules.LogLevel"] == "2") then
                    logLevel = "Always"
                end 
                resTab.insertField (resultTab, "ROW ID", firewallRow["FirewallRules._ROWID_"] or "")
                resTab.insertField (resultTab, "Status",  status or "")
                resTab.insertField (resultTab, "From Zone",fromzone)
                resTab.insertField (resultTab, "To Zone",tozone)
                resTab.insertField (resultTab, "Service", firewallRow["FirewallRules.ServiceName"] or "")
                resTab.insertField (resultTab, "Action", action or "")
                resTab.insertField (resultTab, "Source Hosts", srcusrs or "")
                resTab.insertField (resultTab, "Destination Hosts", destusrs or "")
                resTab.insertField (resultTab, "Local Server", firewallRow["FirewallRules.DNATAddress"] or "")
		resTab.insertField (resultTab, "Internet Dest", firewallRow["FirewallRules.DestinationPublicInterface"] or "")
                resTab.insertField (resultTab, "Log",  logLevel or "")
            end -- if _metadata
        end -- if rowid nil
    end -- for 
    resTab.print (resultTab, 0)
end

-- This function enables/disables/deletes one rule at a time.
function accessControlRuleModify(args)
    local errorFlag = "Error"
    local statusCode =""
    local statusMessage =""
    DBTable= "FirewallRules"     
    local rowIds={}
    rowIds["rowId"] = args[1]
    local operation = args[2]
    errorFlag, statusMessage = fwView.modifyRules(rowIds, operation)
    if (errorFlag == "OK") then
        db.save()
    end
    return errorFlag, statusMessage
end    

-- this routine reads in a current access control rule configuration into a 
-- configuration object. Or wii just initialize the defaults in case a new
-- rule is getting added.
function accessControlRuleCfgInit(args)
    local rowId = args[1]
    local configRow={}
    if (rowId == nil) then
        -- setting default values
        --configRow = db.getDefaults(true, "FirewallRules")
        configRow["FirewallRules.Status"]="1"
    else
        configRow= db.getRow("FirewallRules","_ROWID_",rowId)
        if (configRow == nil) then
           print("Row does not exist")          
           return -1
        end
    end
    return -1,configRow
end

-- The routine saves the modified/ newly added access control rule
function accessControlRuleCfgSave(configRow)
    local errorFlag = "Error"
    local statusCode = ""
    local statusMessage = ""
    DBTable = "FirewallRules"
    if (configRow["FirewallRules._ROWID_"] == nil) then
	configRow["FirewallRules.DNATPortEnable"]="0"
	configRow["FirewallRules.RuleType"]=nil
        errorFlag, statusMessage = fwView.firewallRulesConfig(configRow, "-1", "add")
    else
        errorFlag, statusMessage = fwView.firewallRulesConfig(configRow, configRow["FirewallRules._ROWID_"], "edit")
    end
    if (errorFlag == "OK") then
        db.save()
	local cacheResetProgram = db.getAttribute("environment", "name", "HW_CACHE_RESET", "value")
	util.appendDebugOut("Exec: " .. os.execute(cacheResetProgram));
    end
    return errorFlag, statusMessage
end    

function accessControlRuleCfgInputVal (configRow)

	if (configRow["FirewallRules.FromZoneType"] == nil or configRow["FirewallRules.FromZoneType"] == "")then
		printCLIError("Invalid from_zone")
		return false
	end	
	if (configRow["FirewallRules.ToZoneType"] == nil or configRow["FirewallRules.ToZoneType"] == "")then
		printCLIError("Invalid to_zone")
		return false
	end 
	if (configRow["FirewallRules.ToZoneType"] == configRow["FirewallRules.FromZoneType"])then
		printCLIError("from_zone and to_zone are expected to be different values")
		return false
	end
	if (configRow["FirewallRules.ServiceName"] == nil or configRow["FirewallRules.ServiceName"] == "")then
		printCLIError("Invalid service_name")
		return false
	end
	if (configRow["FirewallRules.Action"] == nil or configRow["FirewallRules.Action"] == "")then
		printCLIError("Invalid action value")
		return false
	end
	if (configRow["FirewallRules.SourceAddressType"] == nil or configRow["FirewallRules.SourceAddressType"] == "")then
		printCLIError("Invalid source_address_type")
		return false
	end 
	if (configRow["FirewallRules.FromZoneType"] == "SECURE")then
		if(configRow["FirewallRules.DestinationAddressType"] == nil or configRow["FirewallRules.SourceAddressType"] == "")then
			printCLIError("Invalid destination_address_type")
			return false
		end
	end	
	if (configRow["FirewallRules.SourceAddressType"] == "1" and (configRow["FirewallRules.SourceAddressStart"] == nil or configRow["FirewallRules.SourceAddressStart"] == "" or configRow["FirewallRules.SourceAddressStart"] == "0.0.0.0") )then
		printCLIError("Invalid source_address_start")
		return false
	end
	if (configRow["FirewallRules.SourceAddressType"] == "2")then
		if (configRow["FirewallRules.SourceAddressStart"] == nil or configRow["FirewallRules.SourceAddressStart"] == "" or configRow["FirewallRules.SourceAddressStart"] == "0.0.0.0")then
		printCLIError("Invalid source_address_start")
		return false
		end
		
		if (configRow["FirewallRules.SourceAddressEnd"] == nil or configRow["FirewallRules.SourceAddressEnd"] == "" or configRow["FirewallRules.SourceAddressEnd"] == "0.0.0.0")then
		printCLIError("Invalid source_address_end")
		return false
		end
	end
	if (configRow["FirewallRules.DestinationAddressType"] == "1" and (configRow["FirewallRules.DestinationAddressStart"] == nil or configRow["FirewallRules.DestinationAddressStart"] == "" or configRow["FirewallRules.DestinationAddressStart"] == "0.0.0.0") )then
		printCLIError("Invalid destination_address_start")
		return false
	end

	if (configRow["FirewallRules.DestinationAddressType"] == "2")then
		if (configRow["FirewallRules.DestinationAddressStart"] == nil or configRow["FirewallRules.DestinationAddressStart"] == "" or configRow["FirewallRules.DestinationAddressStart"] == "0.0.0.0")then
		printCLIError("Invalid destination_address_start")
		return false
		end	
		if (configRow["FirewallRules.DestinationAddressEnd"] == nil or configRow["FirewallRules.DestinationAddressEnd"] == "" or configRow["FirewallRules.DestinationAddressEnd"] == "0.0.0.0")then
		printCLIError("Invalid destination_address_end")
		return false
		end
	end
	if (configRow["FirewallRules.FromZoneType"] == "SECURE" and configRow["FirewallRules.ToZoneType"] == "INSECURE" and (configRow["FirewallRules.TypeOfService"] == "" or configRow["FirewallRules.TypeOfService"] == nil))then
		printCLIError("Invalid qos_priority")
		return false
	end
	if (configRow["FirewallRules.FromZoneType"] == "SECURE" and configRow["FirewallRules.ToZoneType"] == "INSECURE" and configRow["FirewallRules.Action"] ~= "DROP")then
		if (configRow["FirewallRules.SNATAddressType"] == nil or configRow["FirewallRules.SNATAddressType"] == "")then
		printCLIError("Invalid snat_address_type")
		return false
		end
	end
	if (configRow["FirewallRules.SNATAddressType"] == "7" and (configRow["FirewallRules.SNATInterface"] == nil or configRow["FirewallRules.SNATInterface"] =="")) then
		printCLIError("Invalid  wan_interface")
		return false
	end
	if (configRow["FirewallRules.SNATAddressType"] == "1" and (configRow["FirewallRules.SNATAddress"] == nil or configRow["FirewallRules.SNATAddress"] == "" or configRow["FirewallRules.SNATAddress"] == "0.0.0.0"))then
		printCLIError("Invalid snat_address")
		return false
	end
	if (configRow["FirewallRules.Action"] == "DROP_BY_SCHED_MATCH" or configRow["FirewallRules.Action"] == "ACCEPT_BY_SCHED_MATCH")then
		if (configRow["FirewallRules.ScheduleName"] == nil or configRow["FirewallRules.ScheduleName"] == "")then
		printCLIError("Invalid schedule")
		return false
		end	
	end 	
	if (configRow["FirewallRules.FromZoneType"] == "INSECURE" and (configRow["FirewallRules.DestinationPublicInterface"] ==nil or configRow["FirewallRules.DestinationPublicInterface"] == ""))then
		printCLIError("Invalid dnat_ipaddress")
		return false
	end
	if (configRow["FirewallRules.FromZoneType"] == "INSECURE" and configRow["FirewallRules.Action"] ~= "DROP" and (configRow["FirewallRules.DNATAddress"] == nil or configRow["FirewallRules.DNATAddress"] == "" or configRow["FirewallRules.DNATAddress"] == "0.0.0.0"))then
		printCLIError("Invalid internal_ip_address")
		return false
	end
	if (configRow["FirewallRules.LogLevel"] == nil or configRow["FirewallRules.LogLevel"] == "")then
		printCLIError("Inavlid log value")
		return false
	end
    return true
end

--
--
function FirewalALGsInit(args)
	configRow = db.getRow("AlgConf", "_ROWID_","1")
	if (configRow == nil) then
		configRow = {}
	end
	return -1,configRow
end

--
--
function FirewalALGsSave(configRow)
local errorFlag="ERROR"
local statusCode=""
local statusMessage=""
DBTable="AlgConf"
	if(configRow ~= {})then
		errorFlag, statusCode = algs.alg_config(configRow,"1", "edit")
	end
	-- save db if no error
	if (errorFlag == "OK") then db.save() end
	statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
	return errorFlag,statusMessage

end

--
--
function FirewalALGsInputVal(configRow)
end

--
--
function FirewalALGsGet(args)
local resultTab={}
      local configRow=db.getRow("AlgConf","_ROWID_","1")
      printLabel("ALGs")
      if (configRow ~= nil)then
		if (configRow["AlgConf.Pptp"] == "1")then
			resTab.insertField (resultTab,"PPTP","Enabled")
		else
			resTab.insertField (resultTab,"PPTP","Disabled")
		end
		if (configRow["AlgConf.Ipsec"] == "1")then
			resTab.insertField (resultTab,"Ipsec","Enabled")
		else
			resTab.insertField (resultTab,"Ipsec","Disabled")
		end
		if (configRow["AlgConf.Rtsp"] == "1")then
			resTab.insertField (resultTab,"Rtsp","Enabled")
		else
			resTab.insertField (resultTab,"Rtsp","Disabled")
		end
		if (configRow["AlgConf.Sip"] == "1")then
			resTab.insertField (resultTab,"Sip","Enabled")
		else
			resTab.insertField (resultTab,"Sip","Disabled")
		end

		if (configRow["AlgConf.H323"] == "1")then
			resTab.insertField (resultTab,"H323","Enabled")
		else
			resTab.insertField (resultTab,"H323","Disabled")
		end

		if (configRow["AlgConf.Smtp"] == "1")then
			resTab.insertField (resultTab,"Smtp","Enabled")
		else
			resTab.insertField (resultTab,"Smtp","Disabled")
		end

		if (configRow["AlgConf.Dns"] == "1")then
			resTab.insertField (resultTab,"Dns","Enabled")
		else
			resTab.insertField (resultTab,"Dns","Disabled")
		end

		if (configRow["AlgConf.Tftp"] == "1")then
			resTab.insertField (resultTab,"Tftp","Enabled")
		else
			resTab.insertField (resultTab,"Tftp","Disabled")
		end
resTab.print (resultTab, 0)
      end
end
