-- firewall6-ruleCLI.lua
-- Samer Sayeed
-- TeamF1
-- 10/10/08
-- 
-- Modification History
-- 17feb09,ss   added validation checks to fix spr#10808
-- 31jan09,ss  modified firewallRule6InfoGet() routine to make output more readable 
-- 10dec09,ss  written
--
-- Description
-- CLISH Firewall rule LUA routines
--
-- Required Libraries

require "teamf1lualib/fwView"
require "teamf1lualib/fwRules"

-- This function displays the current NAT configuration
function firewallRule6InfoGet (args)
    local firewallTable = db.getTable ("FirewallRules6")
    local resultTab = {} -- table to store firewall rules
    local firewallRow = nil
    local rowId = "0"
    local status = nil
    local ruleType = nil
    local destusrs = nil
    local srcusrs = nil
    local action = nil
    local status = nil

    printLabel ("Default Outbound Policy")
    defRow = db.getRow("FirewallRules6", "RuleType", "DEFAULT_POLICY")
    if (defRow["FirewallRules6.Action"] == "ACCEPT") then 
        print ("For IPv6 : Allow Always\n")
    elseif(defRow["FirewallRules6.Action"] == "DROP") then
        print ("For IPv6 : Block Always")
    end
    printLabel ("List of Available IPv4 Firewall Rules")    
    for col, val in pairs(firewallTable) do
        rowId = rowId+1
        local firewallRow = firewallTable[rowId]
	if (rowId ~= nil) then
            if (firewallRow["FirewallRules6._metadata"] ~= "edit=0" and firewallRow["FirewallRules6.RuleType"] ~= "DEFAULT_POLICY") then
                if(firewallRow["FirewallRules6.Status"] == "0") then
                    status =  "Disabled"
                elseif(firewallRow["FirewallRules6.Status"] == "1") then
                    status =  "Enabled"
                end
                if(firewallRow["FirewallRules6.RuleType"] == "SECURE_INSECURE") then
                    ruleType = "LAN To WAN"
                elseif(firewallRow["FirewallRules6.RuleType"] == "INSECURE_SECURE") then
                    ruleType = "WAN To LAN"
                end
                if (firewallRow["FirewallRules6.Action"] == "ACCEPT") then
                    action = "ALLOW Always"
                elseif (firewallRow["FirewallRules6.Action"] == "DROP") then
                    action = "BLOCK Always"
                elseif (firewallRow["FirewallRules6.Action"] == "DROP_BY_SCHEDULE") then
                    action = "BLOCK by schedule, otherwise allow"
                elseif (firewallRow["FirewallRules6.Action"] == "ACCEPT_BY_SCHEDULE") then
                    action = "ALLOW by schedule,otherwise block"
                end
                if(firewallRow["FirewallRules6.SourceAddressType"] == "0") then
                    srcusrs = "Any"
                elseif(firewallRow["FirewallRules6.SourceAddressType"] == "1") then
                    srcusrs = firewallRow["FirewallRules6.SourceAddressStart"]
                elseif(firewallRow["FirewallRules6.SourceAddressType"] == "2") then
                    srcusrs = firewallRow["FirewallRules6.SourceAddressStart"] .. " - " .. firewallRow["FirewallRules6.SourceAddressEnd"]
                end
                if(firewallRow["FirewallRules6.DestinationAddressType"] == "0") then
                    destusrs = "Any"
                elseif(firewallRow["FirewallRules6.DestinationAddressType"] == "1") then
                    destusrs = firewallRow["FirewallRules6.DestinationAddressStart"]
                elseif(firewallRow["FirewallRules6.DestinationAddressType"] == "2") then
                    destusrs = firewallRow["FirewallRules6.DestinationAddressStart"] .. " - " .. firewallRow["FirewallRules6.DestinationAddressEnd"]
                end
                if (firewallRow["FirewallRules6.LogLevel"] == "1") then
                    logLevel = "Never" 
                elseif (firewallRow["FirewallRules6.LogLevel"] == "2") then
                    logLevel = "Always"
                end 
                resTab.insertField (resultTab, "ROW ID", firewallRow["FirewallRules6._ROWID_"] or "")
                resTab.insertField (resultTab, "Status",  status or "")
                resTab.insertField (resultTab, "Rule Type",  ruleType or "")
                resTab.insertField (resultTab, "Service", firewallRow["FirewallRules6.ServiceName"] or "")
                resTab.insertField (resultTab, "Action", action or "")
                resTab.insertField (resultTab, "Source Users", srcusrs or "")
                resTab.insertField (resultTab, "Destination Users", destusrs or "")
                resTab.insertField (resultTab, "Log",  logLevel or "")
            end -- if _metadata
        end -- if rowid nil
    end -- for 
    resTab.print (resultTab, 0)
end

-- This function enables/disables/deletes one rule at a time.
function accessControlRule6Modify(args)
    local errorFlag = "Error"
    local statusCode =""
    local statusMessage =""
    DBTable= "FirewallRules6"     

    local rowIds={}
    rowIds["rowId"] = args[1]
    local operation = args[2]
    errorFlag, statusMessage = fwView.modifyIPv6Rules(rowIds, operation)
    if (errorFlag == "OK") then
        db.save()
    end
    firewallRule6InfoGet()
    return errorFlag, statusMessage
end    

-- this routine reads in a current access control rule configuration into a 
-- configuration object. Or wii just initialize the defaults in case a new
-- rule is getting added.
function accessControlRule6CfgInit(args)
    local rowId = args[1]
    local configRow={}
    if (rowId == nil) then
        configRow["FirewallRules6._ROWID_"] = "-1"
        configRow["FirewallRules6.ServiceName"] = "ANY"
        configRow["FirewallRules6.SourceAddressType"] = "0"
        configRow["FirewallRules6.DestinationAddressType"] = "0"
        configRow["FirewallRules6.LogLevel"] = "1"
        configRow["FirewallRules6.Action"] ="DROP"
    else
        configRow= db.getRow("FirewallRules6","_ROWID_",rowId)
        if (configRow == nil) then
           print("Row does not exist")          
           return -1
        end
    end
    --configRow["FirewallRules6.Call"] = "1"
    return configRow["FirewallRules6._ROWID_"], configRow
end

-- The routine saves the modified/ newly added access control rule
function accessControlRule6CfgSave(configRow)
    local errorFlag = "Error"
    local statusCode = ""
    local statusMessage = ""
    DBTable = "FirewallRules6"

    if (configRow["FirewallRules6._ROWID_"] == "-1") then
        if (configRow["FirewallRules6.RuleType"] == "INSECURE_SECURE") then 
        configRow["FirewallRules6.FromZoneType"] = "INSECURE"
        configRow["FirewallRules6.ToZoneType"] = "SECURE"
        elseif (configRow["FirewallRules6.RuleType"] == "SECURE_INSECURE") then 
        configRow["FirewallRules6.FromZoneType"] = "SECURE"
        configRow["FirewallRules6.ToZoneType"] = "INSECURE"
        end 
        errorFlag, statusMessage = fwView.ipv6FirewallRulesConfig(configRow, "-1", "add")
    else
        errorFlag, statusMessage = fwView.ipv6FirewallRulesConfig(configRow, configRow["FirewallRules6._ROWID_"], "edit")
    end
    if (errorFlag == "OK") then
        db.save()
    end
    return errorFlag, statusMessage
end   

-- this routine reads in a current access control rule configuration into a 
-- configuration object. Or wii just initialize the defaults in case a new
-- rule is getting added.
function firewallSettings4CfgInit(args)
    local action = args[1]
    local configRow={}
    
    configRow["FirewallRules.Action"] = db.getAttribute("FirewallRules", "RuleType", "DEFAULT_POLICY", "Action")
    if(action == "1") then
      configRow["FirewallRules.Action"] = "ACCEPT"
    else
      configRow["FirewallRules.Action"] = "DROP"
    end
    return configRow["FirewallRules._ROWID_"], configRow
end

-- this routine reads in a current access control rule configuration into a 
-- configuration object. Or wii just initialize the defaults in case a new
-- rule is getting added.
function firewallSettings6CfgInit(args)
    local action = args[1]
    local configRow={}
    
    configRow["FirewallRules6.Action"] = db.getAttribute("FirewallRules6", "RuleType", "DEFAULT_POLICY", "Action")
    if(action == "1") then
      configRow["FirewallRules6.Action"] = "ACCEPT"
    else
      configRow["FirewallRules6.Action"] = "DROP"
    end
    return configRow["FirewallRules6._ROWID_"], configRow
end

-- The routine saves the modified/ newly added access control rule
function firewallSettings4CfgSave(configRow)
    local errorFlag = "Error"
    local statusCode = ""
    local statusMessage = ""
	
    local rowid = db.getAttribute("FirewallRules", "RuleType", "DEFAULT_POLICY", "_ROWID_")
    if (rowid ~= nil) then
    DBTable = "FirewallRules"
    errorFlag, statusCode = fwView.defaultPolicyConfig(configRow, rowid, "edit")
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    else
    errorFlag = "ERROR"
    statusMessage = "Default outbound rule not found for IPv4 Firewall Rules."
    end
	
    -- save db if no error
    if (errorFlag == "OK") then db.save() end    
    return errorFlag, statusMessage
end  

-- The routine saves the modified/ newly added access control rule
function firewallSettings6CfgSave(configRow)
    local errorFlag = "Error"
    local statusCode = ""
    local statusMessage = ""
	
    local rowid = db.getAttribute("FirewallRules6", "RuleType", "DEFAULT_POLICY", "_ROWID_")
    if (rowid ~= nil) then
    DBTable = "FirewallRules6"
    errorFlag, statusCode = fwView.defaultPolicyConfig(configRow, rowid, "edit")
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    else
    errorFlag = "ERROR"
    statusMessage = "Default outbound rule not found for IPv6 Firewall Rules."
    end
	
    -- save db if no error
    if (errorFlag == "OK") then db.save() end    
    return errorFlag, statusMessage
end

function accessControlRule6CfgInputVal(configRow)
    if (configRow["FirewallRules6.RuleType"] == nil or 
        configRow["FirewallRules6.RuleType"] == "") then
        printCLIError ("Enter valid Rule Type\n")
        return false
    end
    if (configRow["FirewallRules6.SourceAddressType"] == "" or 
        configRow["FirewallRules6.SourceAddressType"] == nil) then 
        printCLIError("Enter valid source address type\n")
        return false
    end
    if (configRow["FirewallRules6.DestinationAddressType"] == "" or 
        configRow["FirewallRules6.DestinationAddressType"] == nil) then 
        printCLIError("Enter valid destination address type\n")
        return false
    end
    if (configRow["FirewallRules6.SourceAddressType"] == "1") then
        if (configRow["FirewallRules6.SourceAddressStart"] == "" or 
         configRow["FirewallRules6.SourceAddressStart"] == nil) then 
        printCLIError("Enter valid source address\n")
        return false
        end
        if (configRow["FirewallRules6.SourceAddressPrefixLength"] == "" or 
          configRow["FirewallRules6.SourceAddressPrefixLength"] == nil) then
        printCLIError("Enter valid source prefix length\n")
        return false
        end
    end
    if (configRow["FirewallRules6.SourceAddressType"] == "2") then
        if ((configRow["FirewallRules6.SourceAddressStart"] == "" or 
             configRow["FirewallRules6.SourceAddressStart"] == nil) or 
            (configRow["FirewallRules6.SourceAddressEnd"] == "" or 
             configRow["FirewallRules6.SourceAddressEnd"] == nil)) then
            printCLIError("Enter valid source address range\n")
            return false
        end
        if (configRow["FirewallRules6.SourceAddressPrefixLength"] == "" or 
          configRow["FirewallRules6.SourceAddressPrefixLength"] == nil) then
        printCLIError("Enter valid source prefix length\n")
        return false
        end
    end
    if (configRow["FirewallRules6.DestinationAddressType"] == "1") then 
        if (configRow["FirewallRules6.DestinationAddressStart"] == "" or 
         configRow["FirewallRules6.DestinationAddressStart"] == nil) then
       	printCLIError("Enter valid destination address\n")
       	return false
        end
        if (configRow["FirewallRules6.DestinationAddressPrefixLength"] == "" or 
          configRow["FirewallRules6.DestinationAddressPrefixLength"] == nil) then
        printCLIError("Enter valid destination prefix length\n")
        return false
        end
    end
    if (configRow["FirewallRules6.DestinationAddressType"] == "2") then
        if((configRow["FirewallRules6.DestinationAddressStart"] == "" or 
          configRow["FirewallRules6.DestinationAddressStart"] == nil) or 
          (configRow["FirewallRules6.DestinationAddressEnd"] == "" or 
          configRow["FirewallRules6.DestinationAddressEnd"] == nil)) then
            printCLIError("Enter valid destination address range\n")
            return false
        end
        if (configRow["FirewallRules6.DestinationAddressPrefixLength"] == "" or 
          configRow["FirewallRules6.DestinationAddressPrefixLength"] == nil) then
        printCLIError("Enter valid destination prefix length\n")
        return false
        end
    end
    return true
end
