--[[
#### Hussain Vali.
#### TeamF1
#### www.TeamF1.com
#### Feb 29, 2008
#### File: fwView.lua
#### Description: Firewall functions
#### Revisions:
]]--

--************* Requires *************

require "teamf1lualib/fwRules"

--************* Initial Code *************

--package wlNw
fwView = {}

--************* Functions *************
-- Services Configuration
function fwView.serviceConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false

    if (operation == "edit") then
        local ruleInUse = 0 
        local ruleInUse6 = 0
        local service = inputTable["Services.ServiceName"]
        local rows = db.existsRowWhere("FirewallRules", "ServiceName = '" .. service .. "'")      
        local configRow = db.getRows("FirewallRules", "ServiceName", service)
        local rows6 = db.existsRowWhere("FirewallRules6", "ServiceName = '" .. service .. "'")      
        local configRow6 = db.getRows("FirewallRules6", "ServiceName", service)
    	for a,b in pairs(configRow) do
             ruleInUse=0;
	         if (b["FirewallRules.Status"] == "1") then
                ruleInUse=1;
                break;       
             end         
        end
        for a,b in pairs(configRow6) do
             ruleInUse6=0;
	         if (b["FirewallRules6.Status"] == "1") then
                ruleInUse6=1;
                break;       
             end         
        end
        if (rows or rows6) and ((ruleInUse == 1) or (ruleInUse6 == 1)) then
            db.rollback()
    		return "ERROR", "SERVICE_IN_USE"
        end
    end

	-- config Service
	valid = fw.config(inputTable, rowid, operation)
	
	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "SERVICE_CONFIG_FAILED"
	end
end

-- Services Delete
function fwView.deleteServices (rowIds)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"

	-- Check for Using Services
    for k,v in pairs(rowIds) do
        local service = db.getAttribute("Services", "_ROWID_", v, "ServiceName")
        local rows = db.existsRowWhere("FirewallRules", "ServiceName = '" .. service .. "'")
        local rows6 = db.existsRowWhere("FirewallRules6", "ServiceName = '" .. service .. "'")
        if (rows or rows6) then
            valid = false
            errStr = "SERVICE_IN_USE_DEL_RULE"
            break
        end
	end

    if (valid) then
        for k,v in pairs(rowIds) do
            local inTable = {}
            inTable["Services._ROWID_"] = v;
            valid = fw.config(inTable, "-1", "delete")
            if (valid == false) then
                errStr = "SERVICE_DELETE_FAILED"
                break
            end
	    end
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "SERVICE_DELETE_OK"
	else
		db.rollback()
		return "ERROR", errStr
	end

end

-- Schedules Configuration Get
function fwView.scheduleConfigGet (inputTable)

    local inTable = {}
    inTable["Schedules.ScheduleName"] = inputTable["Schedules.ScheduleName"]
    inTable["Schedules._ROWID_"] = inputTable["Schedules._ROWID_"]
    inTable["Schedules.AllDay"] = inputTable["Schedules.AllDay"]
    local days = tonumber(inputTable["Schedules.Days"])

    if (days ~= nil) then
        if (days == 127 or days == 128) then
           inTable["Schedules.allDays"] = "1"
           days = days - days
        else
           inTable["Schedules.allDays"] = "0"
        end
    
        if (days >= 64) then
            inTable["Schedules.sunday"] = "1"
            days = days - 64
        else
            inTable["Schedules.sunday"] = "0"
        end

        if (days >= 32) then
            inTable["Schedules.saturday"] = "1"
            days = days - 32
        else
            inTable["Schedules.saturday"] = "0"
        end

        if (days >= 16) then
            inTable["Schedules.friday"] = "1"
            days = days - 16
        else
            inTable["Schedules.friday"] = "0"
        end

        if (days >= 8) then
            inTable["Schedules.thursday"] = "1"
            days = days - 8
        else
            inTable["Schedules.thursday"] = "0"
        end

        if (days >= 4) then
            inTable["Schedules.wednesday"] = "1"
            days = days - 4
        else
            inTable["Schedules.wednesday"] = "0"
        end

        if (days >= 2) then
            inTable["Schedules.tuesday"] = "1"
            days = days - 2
        else
            inTable["Schedules.tuesday"] = "0"
        end

        if (days >= 1) then
            inTable["Schedules.monday"] = "1"
            days = days - 1
        else
            inTable["Schedules.monday"] = "0"
        end
    end

    local hrs = tonumber(inputTable["Schedules.StartTimeHours"])
    if (hrs ~= nil) then
        if (inputTable["Schedules.StartTimeHours"] == "0" and inputTable["Schedules.StartTimeMins"] == "0") then
            inTable["Schedules.startTimeHour"] = "12"
            inTable["Schedules.startTimeMin"] = "00"
            inTable["Schedules.startTimeMeridian"] = "AM"
        elseif (inputTable["Schedules.StartTimeHours"] == "0") then
            inTable["Schedules.startTimeHour"] = "12"
            inTable["Schedules.startTimeMin"] = inputTable["Schedules.StartTimeMins"]
            inTable["Schedules.startTimeMeridian"] = "AM"
        elseif (hrs > 12) then
            inTable["Schedules.startTimeHour"] = hrs - 12
            inTable["Schedules.startTimeMin"] = inputTable["Schedules.StartTimeMins"]
            inTable["Schedules.startTimeMeridian"] = "PM"
        elseif (hrs == 12) then
            inTable["Schedules.startTimeHour"] = inputTable["Schedules.StartTimeHours"]
            inTable["Schedules.startTimeMin"] = inputTable["Schedules.StartTimeMins"]
            inTable["Schedules.startTimeMeridian"] = "PM"
        else
            inTable["Schedules.startTimeHour"] = inputTable["Schedules.StartTimeHours"]
            inTable["Schedules.startTimeMin"] = inputTable["Schedules.StartTimeMins"]
            inTable["Schedules.startTimeMeridian"] = "AM"
        end
    end

    hrs = tonumber(inputTable["Schedules.EndTimeHours"])
    if (hrs ~= nil) then
        if (inputTable["Schedules.EndTimeHours"] == "0" and inputTable["Schedules.EndTimeMins"] == "0") then
            inTable["Schedules.endTimeHour"] = "12"
            inTable["Schedules.endTimeMin"] = "00"
            inTable["Schedules.endTimeMeridian"] = "AM"
        elseif (inputTable["Schedules.EndTimeHours"] == "0") then
            inTable["Schedules.endTimeHour"] = "12"
            inTable["Schedules.endTimeMin"] = inputTable["Schedules.EndTimeMins"]
            inTable["Schedules.endTimeMeridian"] = "AM"
        elseif (hrs > 12) then
            inTable["Schedules.endTimeHour"] = hrs - 12
            inTable["Schedules.endTimeMin"] = inputTable["Schedules.EndTimeMins"]
            inTable["Schedules.endTimeMeridian"] = "PM"
        elseif (hrs == 12) then
            inTable["Schedules.endTimeHour"] = inputTable["Schedules.EndTimeHours"]
            inTable["Schedules.endTimeMin"] = inputTable["Schedules.EndTimeMins"]
            inTable["Schedules.endTimeMeridian"] = "PM"
        else
            inTable["Schedules.endTimeHour"] = inputTable["Schedules.EndTimeHours"]
            inTable["Schedules.endTimeMin"] = inputTable["Schedules.EndTimeMins"]
            inTable["Schedules.endTimeMeridian"] = "AM"
        end
    end
   
    local startHH = inTable["Schedules.startTimeHour"]
    local startMM = inTable["Schedules.startTimeMin"]
    local endHH = inTable["Schedules.endTimeHour"]
    local endMM = inTable["Schedules.endTimeMin"]
    
    if (startHH ~= nil and string.len(startHH) == 1) then
        inTable["Schedules.startTimeHour"] = string.format ("%d%d",0,startHH)
    end
    if (startMM ~= nil and string.len(startMM) == 1) then
        inTable["Schedules.startTimeMin"] = string.format ("%d%d",0,startMM)
    end
    if (endHH ~= nil and string.len(endHH) == 1) then
        inTable["Schedules.endTimeHour"] = string.format ("%d%d",0,endHH)
    end
    if (endMM ~= nil and string.len(endMM) == 1) then
        inTable["Schedules.endTimeMin"] = string.format ("%d%d",0,endMM)
    end

    return inTable
end

-- Schedules Configuration
function fwView.schedulesConfig (inputTable, rowid, operation)
        -- if not allowed to edit
        if (ACCESS_LEVEL ~= 0) then
                return "ACCESS_DENIED", "ADMIN_REQD"
        end

        db.beginTransaction() --begin transaction

        local valid = false
        local inTable = {}
        local days = 0
        if (inputTable["Schedules.monday"] == "1") then
            days = days + 1
        end
        if (inputTable["Schedules.tuesday"] == "1") then
            days = days + 2
        end
        if (inputTable["Schedules.wednesday"] == "1") then
            days = days + 4
        end
        if (inputTable["Schedules.thursday"] == "1") then
            days = days + 8
        end        
        if (inputTable["Schedules.friday"] == "1") then
            days = days + 16
        end
        if (inputTable["Schedules.saturday"] == "1") then
            days = days + 32
        end
        if (inputTable["Schedules.sunday"] == "1") then
            days = days + 64
        end
        if (inputTable["Schedules.allDays"] == "1") then
            days = days + 127
        end
      
        inTable["Schedules.ScheduleName"] = inputTable["Schedules.ScheduleName"]
        inTable["Schedules.Days"] = days
        inTable["Schedules.AllDay"] = inputTable["Schedules.AllDay"]        
        if (inputTable["Schedules.AllDay"] == "1") then
            inTable["Schedules.StartTimeHours"] = "0"
            inTable["Schedules.StartTimeMins"] = "0"
            inTable["Schedules.EndTimeHours"] = "23"
            inTable["Schedules.EndTimeMins"] = "59"
        else
            if (inputTable["Schedules.startTimeMeridian"] == "AM") then
                if (inputTable["Schedules.startTimeHour"] == "12") then
                    inTable["Schedules.StartTimeHours"] = "0"
                else
                    inTable["Schedules.StartTimeHours"] = inputTable["Schedules.startTimeHour"]
                end
            else
                if (inputTable["Schedules.startTimeHour"] ~= "12") then
                    inTable["Schedules.StartTimeHours"] = inputTable["Schedules.startTimeHour"] + 12
                else
                    inTable["Schedules.StartTimeHours"] = inputTable["Schedules.startTimeHour"]
                end
            end

            if (inputTable["Schedules.endTimeMeridian"] == "AM") then
                if (inputTable["Schedules.endTimeHour"] == "12") then
                    inTable["Schedules.EndTimeHours"] = "0"
                else
                    inTable["Schedules.EndTimeHours"] = inputTable["Schedules.endTimeHour"]
                end
            else
                if (inputTable["Schedules.endTimeHour"] ~= "12") then
                    inTable["Schedules.EndTimeHours"] = inputTable["Schedules.endTimeHour"] + 12
                else
                    inTable["Schedules.EndTimeHours"] = inputTable["Schedules.endTimeHour"]
                end
            end
            inTable["Schedules.StartTimeMins"] = inputTable["Schedules.startTimeMin"]
            inTable["Schedules.EndTimeMins"] = inputTable["Schedules.endTimeMin"]
        end
      
        valid = fw.config (inTable,  rowid, operation)

        -- return
        if (valid) then
                db.commitTransaction()
                return "OK", "STATUS_OK"
        else
                db.rollback()
                return "ERROR", "SCHEDULE_CONFIG_FAILED"
        end
end

-- Schedules Delete
function fwView.deleteSchedules (rowIds)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"

	-- Check for Using Schedules 
    for k,v in pairs(rowIds) do
        local schedule = db.getAttribute("Schedules", "_ROWID_", v, "ScheduleName")
        local rows = db.existsRowWhere("FirewallRules", "ScheduleName = '" .. schedule .. "'")
        local rows6 = db.existsRowWhere("FirewallRules6", "ScheduleName = '" .. schedule .. "'")
        if (rows or rows6) then
            valid = false
            errStr = "SCHEDULE_IN_USE_DEL_RULE"
            break       
        end      
	end

    if (valid) then
        for k,v in pairs(rowIds) do
            local inTable = {}
            inTable["Schedules._ROWID_"] = v;
            valid = fw.config(inTable, "-1", "delete")
            if (valid == false) then
                errStr = "SCHEDULE_DELETE_FAILED"
                break
            end
	    end
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "SCHEDULE_DELETE_OK"
	else
		db.rollback()
		return "ERROR", errStr
	end

end

-- Setting Default Outbound Policy Configuration
function fwView.defaultPolicyConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false

	-- config Service
	valid = fw.config(inputTable, rowid, operation)
	
	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "FW_DEFAULT_CONFIG_FAILED"
	end
end

-- Firewall Inbound Rules Configuration
function fwView.lwInConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false

	-- config Lan Wan In bound Rule
    -- If router is in Classical Routing then Set Lan Users.
    -- Other wise we should use send to lan Server Option.
    if (inputTable["FirewallRules.DestinationAddressType"] ~= nil) then
        if (inputTable["FirewallRules.DestinationAddressType"] == "1") then
            inputTable["FirewallRules.DestinationAddressStart"] = util.joinFields(inputTable,"FirewallRules","destStartIpAddrbyte", ".",4)
        elseif (inputTable["FirewallRules.DestinationAddressType"] == "2") then
            inputTable["FirewallRules.DestinationAddressStart"] = util.joinFields(inputTable,"FirewallRules","destStartIpAddrbyte", ".",4)
            inputTable["FirewallRules.DestinationAddressEnd"] = util.joinFields(inputTable,"FirewallRules","destEndIpAddrbyte", ".",4)            
        end
    else
        if (inputTable["FirewallRules.Action"] ~= "DROP") then
            inputTable["FirewallRules.DNATAddress"] = util.joinFields(inputTable,"FirewallRules","sendToSvrIpAddrbyte", ".",4)
        end
    end

    -- Setting WAN Destination IP Address
    if (inputTable["FirewallRules.DestinationPublicInterface"] ~= nil) then
        if (inputTable["FirewallRules.DestinationPublicInterface"] ~= "WAN") then
            inputTable["FirewallRules.DestinationPublicAddress"] = util.joinFields(inputTable,"FirewallRules","wanDestIpAddrbyte", ".",4)
        end
    end

    -- Setting WAN Users
    if (inputTable["FirewallRules.SourceAddressType"] == "1") then
        inputTable["FirewallRules.SourceAddressStart"] = util.joinFields(inputTable,"FirewallRules","sourceStartIpAddrbyte", ".",4)
    elseif (inputTable["FirewallRules.SourceAddressType"] == "2") then
        inputTable["FirewallRules.SourceAddressStart"] = util.joinFields(inputTable,"FirewallRules","sourceStartIpAddrbyte", ".",4)
        inputTable["FirewallRules.SourceAddressEnd"] = util.joinFields(inputTable,"FirewallRules","sourceEndIpAddrbyte", ".",4)            
    end

    -- check if schedule Name is selected.
    if (inputTable["FirewallRules.Action"] == "DROP_BY_SCHEDULE" or 
        inputTable["FirewallRules.Action"] == "ACCEPT_BY_SCHEDULE") then
        if (inputTable["FirewallRules.ScheduleName"] == nil) then
		    return "ERROR", "FW_SCHED_REQUIRE"
        end
    end
    
    valid = fw.config(inputTable, rowid, operation)
	
	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "FW_IN_RULE_CONFIG_FAILED"
	end
end

-- Firewall Outbound Rules Configuration
function fwView.lwOutConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false

	-- config Lan Wan Outbound Rule
    if (inputTable["FirewallRules.SourceAddressType"] == "1") then
        inputTable["FirewallRules.SourceAddressStart"] = util.joinFields(inputTable,"FirewallRules","sourceStartIpAddrbyte", ".",4)
    elseif (inputTable["FirewallRules.SourceAddressType"] == "2") then
        inputTable["FirewallRules.SourceAddressStart"] = util.joinFields(inputTable,"FirewallRules","sourceStartIpAddrbyte", ".",4)
        inputTable["FirewallRules.SourceAddressEnd"] = util.joinFields(inputTable,"FirewallRules","sourceEndIpAddrbyte", ".",4) 
    end

    -- Setting WAN Users
    if (inputTable["FirewallRules.DestinationAddressType"] == "1") then
        inputTable["FirewallRules.DestinationAddressStart"] = util.joinFields(inputTable,"FirewallRules","destStartIpAddrbyte", ".",4)
    elseif (inputTable["FirewallRules.DestinationAddressType"] == "2") then
        inputTable["FirewallRules.DestinationAddressStart"] = util.joinFields(inputTable,"FirewallRules","destStartIpAddrbyte", ".",4)
        inputTable["FirewallRules.DestinationAddressEnd"] = util.joinFields(inputTable,"FirewallRules","destEndIpAddrbyte", ".",4)            
    end

    -- Setting NAT IP
    if (inputTable["FirewallRules.SNATAddressType"] ~= nil) then
        if (inputTable["FirewallRules.SNATAddressType"] == "1") then
            inputTable["FirewallRules.SNATAddress"] = util.joinFields(inputTable,"FirewallRules","natIpAddrbyte", ".",4)
        end
        inputTable["FirewallRules.SNATInterface"] = "WAN1"
    end    

    -- check if schedule Name is selected.
    if (inputTable["FirewallRules.Action"] == "DROP_BY_SCHEDULE" or 
        inputTable["FirewallRules.Action"] == "ACCEPT_BY_SCHEDULE") then
        if (inputTable["FirewallRules.ScheduleName"] == nil) then
		    return "ERROR", "FW_SCHED_REQUIRE"
        end
    end
            
    valid = fw.config(inputTable, rowid, operation)
	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "FW_OUT_RULE_CONFIG_FAILED"
	end
end

-- Modify Firewall Rules (Enable/Disable/Delete)
function fwView.modifyRules (rowIds, operation)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"

    if (operation == "enable") then
        for k,v in pairs(rowIds) do
            local inTable = {}
            inTable["FirewallRules.Status"] = "1";
            valid = fw.config(inTable, v, "edit")
            if (valid == false) then
                errStr = "FW_RULES_ENABLED_FAILED"
                break
            end
	    end
        if (valid ~= false) then
            errStr = "FW_RULES_ENABLED"        
        end
    elseif (operation == "disable") then
        for k,v in pairs(rowIds) do
            local inTable = {}
            inTable["FirewallRules.Status"] = "0";
            valid = fw.config(inTable, v, "edit")
            if (valid == false) then
                errStr = "FW_RULES_DISABLED_FAILED"
                break
            end
	    end
        if (valid ~= false) then
            errStr = "FW_RULES_DISABLED"
        end
    else
        local priIds = {}
        local idx = 0
        for k,v in pairs(rowIds) do
            idx = idx + 1
            priIds[idx] = db.getAttribute ("FirewallRules", "_ROWID_", v, "PriorityId") or ''
            local inTable = {}
            inTable["FirewallRules._ROWID_"] = v;
            valid = fw.config(inTable, "-1", "delete")
            if (valid == false) then
                errStr = "FW_RULES_DELETE_FAILED"
                break
            end
        end
        if (valid ~= false) then
            errStr = "FW_RULES_DELETED"
        end

        if (valid ~= false) then
            -- Re-arrange Rules priority
            local rulesPriority = ""
            local rulesPriConfig = db.getRow ("fwPriorityMap", "RuleType", "ALL")
            if (rulesPriConfig ~= nil) then
                rulesPriority = rulesPriConfig["fwPriorityMap.PriorityStr"] or ''
                if (rulesPriority ~= nil and rulesPriority ~= "") then
                    local ruleIds = util.split (rulesPriority, ",")
                    if (ruleIds ~= false) then
                        rulesPriority = ""
                        for k,v in pairs (ruleIds) do
                            local notInDeleteList = true
                            for m,n in pairs (priIds) do
                                if (v == n) then
                                    notInDeleteList = false
                                    break
                                end
                            end

                            if (notInDeleteList) then
                                rulesPriority = rulesPriority .. v .. ","
                            end
                        end

                       	rulesPriority = string.sub(rulesPriority, 1, string.len(rulesPriority) - 1) --shave last comma

                        rulesPriConfig["fwPriorityMap.PriorityStr"] = rulesPriority
                        DBTable = "fwPriorityMap"
                        valid = fw.config(rulesPriConfig, rulesPriConfig["fwPriorityMap._ROWID_"], "edit")

                        if (valid) then
                        else
                            errStr = "FW_RULES_PRIORITY_CONFIG_FAILED"
                        end
                    else
                        valid = false
                        errStr = "FW_RULES_PRIORITY_CONFIG_FAILED"
                    end
                else
                    valid = false
                    errStr = "FW_RULES_PRIORITY_CONFIG_FAILED"
                end
            else
                valid = false
                errStr = "FW_RULES_PRIORITY_CONFIG_FAILED"
            end
        end
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", errStr
	else
		db.rollback()
		return "ERROR", errStr
	end
end

-- Move Firewall rules
function fwView.moveRules (rowIds, position)
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

    -- Collect priority Is
    local ruleIds = {}
    if (#rowIds == 0) then
        ruleIds["1"] = db.getAttribute ("FirewallRules", "_ROWID_", rowIds["1"], "PriorityId") or ''
    else
        local idx = 0
        for k,v in pairs(rowIds) do
            idx = idx + 1
            ruleIds[idx] = db.getAttribute ("FirewallRules", "_ROWID_", v, "PriorityId") or ''
        end
    end

	db.beginTransaction() --begin transactions
    local valid = false

    local rulesPriority = nil
    local rulesToMove = ""
    local rulesPriConfig = db.getRow ("fwPriorityMap", "RuleType", "ALL")
    if (rulesPriConfig ~= nil) then
        rulesPriority = rulesPriConfig["fwPriorityMap.PriorityStr"] or ''
    end

    if (rulesPriority ~= nil and rulesPriority ~= "") then
        local curRuleIds = util.split (rulesPriority, ",")
        if (curRuleIds ~= false) then
            -- Collect rules to be move
            if (#ruleIds == 0) then
                rulesToMove = ruleIds["1"] .. ","
            else
                for k,v in pairs(ruleIds) do
                    rulesToMove = rulesToMove .. v .. ","
                end
            end

            --util.appendDebugOut("[SHV]Rules to Move: " .. rulesToMove .. "<br>")

            -- Rearrange priorities.
            rulesPriority = ""
            local skipInsert = false
            for k,v in pairs (curRuleIds) do
                local notInMoveList = true
                for m,n in pairs (ruleIds) do
                    if (v == n) then
                        notInMoveList = false
                        break
                    end
                end

                if (notInMoveList) then
                    rulesPriority = rulesPriority .. v .. ","
                end
            end

            -- Insert selected rules in teh position.
            if (rulesPriority == "") then
                rulesPriority = rulesToMove
            elseif (tonumber(position) == 1) then -- Move to First
                rulesPriority = rulesToMove .. "," .. rulesPriority
            elseif (tonumber(position) == #curRuleIds) then -- Move to Last
                rulesPriority = rulesPriority .. "," .. rulesToMove
            else
                local newRulesList = util.split (rulesPriority, ",")
                if (newRulesList ~= false) then
                    if (tonumber(position) >= #newRulesList) then
                        rulesPriority = rulesPriority .. "," .. rulesToMove
                    else
                        rulesPriority = ""
                        local idx = 0
                        for k,v in pairs (newRulesList) do
                            idx = idx + 1
                            if (idx == tonumber(position)) then
                                rulesPriority = rulesPriority .. "," .. rulesToMove
                            end
                            rulesPriority = rulesPriority .. v .. ","
                        end
                    end
                end
            end

          	rulesPriority = string.gsub(rulesPriority, ",,", ",") --shave double comma
           	rulesPriority = string.sub(rulesPriority, 1, string.len(rulesPriority) - 1) --shave last comma
        end
    end

    -- util.appendDebugOut("[SHV]Final List: " .. rulesPriority .. "<br>")

    rulesPriConfig["fwPriorityMap.PriorityStr"] = rulesPriority
    DBTable = "fwPriorityMap"
    valid = fw.config(rulesPriConfig, rulesPriConfig["fwPriorityMap._ROWID_"], "edit")

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "MOVE_OPERATION_FAILED"
    end
end

-- Modify IPv6 Firewall Rules (Enable/Disable/Delete)
function fwView.modifyIPv6Rules (rowIds, operation)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"

    if (operation == "enable") then
        for k,v in pairs(rowIds) do
            local inTable = {}
            inTable["FirewallRules6.Status"] = "1";
            valid = fw.config(inTable, v, "edit")
            if (valid == false) then
                errStr = "FW_RULES_ENABLED_FAILED"
                break
            end
	    end
        if (valid ~= false) then
            errStr = "FW_RULES_ENABLED"        
        end
    elseif (operation == "disable") then
        for k,v in pairs(rowIds) do
            local inTable = {}
            inTable["FirewallRules6.Status"] = "0";
            valid = fw.config(inTable, v, "edit")
            if (valid == false) then
                errStr = "FW_RULES_DISABLED_FAILED"
                break
            end
	    end
        if (valid ~= false) then
            errStr = "FW_RULES_DISABLED"
        end
    else
        for k,v in pairs(rowIds) do
            local inTable = {}
            inTable["FirewallRules6._ROWID_"] = v;
            valid = fw.config(inTable, "-1", "delete")
            if (valid == false) then
                errStr = "FW_RULES_DELETE_FAILED"
                break
            end
        end
        if (valid ~= false) then
            errStr = "FW_RULES_DELETED"
        end
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", errStr
	else
		db.rollback()
		return "ERROR", errStr
	end
end

-- Port Triggering Configuration
function fwView.portTrgConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false

	-- config Service
	valid = fw.config(inputTable, rowid, operation)
	
	-- return
	if (valid) then
		db.commitTransaction(true)
	    return "OK", "STATUS_OK"            
	else
		db.rollback()
		return "ERROR", "FW_PORT_TRG_CONFIG_FAILED"
	end
end

-- Delete Port Triggering Rules
function fwView.deleteRules (rowIds)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"

    for k,v in pairs(rowIds) do
        local inTable = {}
        inTable["PortTriggering._ROWID_"] = v;
        valid = fw.config(inTable, "-1", "delete")
        if (valid == false) then
            errStr = "FW_PORT_TRG_RULES_DELETE_FAILED"
            break
        end
    end

    if (valid ~= false) then
        errStr = "FW_PORT_TRG_RULES_DELETED"
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", errStr
	else
		db.rollback()
		return "ERROR", errStr
	end
end

-- attack Checks Configuration
function fwView.attackChecksConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false

   	valid = fw.config(inputTable, rowid, operation)
	
	-- return
	if (valid) then
		db.commitTransaction(true)
	    return "OK", "STATUS_OK"            
	else
		db.rollback()
		return "ERROR", "FW_AC_CONFIG_FAILED"
	end
end

-- Content Filtering Configuration
function fwView.contentFileteringConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = true

    if (inputTable["ContentFiltering.Status"] == "1") then
        DBTable = "WebComponents"
       	valid = fw.config(inputTable, rowid, operation)
    end
    
    if (valid) then
        DBTable = "ContentFiltering"        
       	valid = fw.config(inputTable, rowid, operation)
    end
	
	-- return
	if (valid) then
		db.commitTransaction(true)
	    return "OK", "STATUS_OK"            
	else
		db.rollback()
		return "ERROR", "FW_CF_CONFIG_FAILED"
	end
end

-- Blocked Keywords Configuration
function fwView.blockedKeywordConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false

    if (operation == "add") then
        if (inputTable["BlockSites.KeywordBlockingTo"] == "ALL") then
        	local table = db.getTable ("LanGroup")
        	local i = 0
        	for k,v in pairs(table) do
		        i = i + 1
        		local row = table[i]
                local inTable = {}
                inTable["BlockSites.KeywordToBlock"] = inputTable["BlockSites.KeywordToBlock"]
                inTable["BlockSites.KeywordBlockingTo"] = row["LanGroup.GroupName"]
                inTable["BlockSites.Status"] = inputTable["BlockSites.Status"]
                valid = fw.config(inTable, rowid, operation)
                if (valid) then
                else
            		db.rollback()
            		return "ERROR", "FW_KEYWORD_CONFIG_FAILED"
                end
            end
        else
            valid = fw.config(inputTable, rowid, operation)
        end
    elseif (operation == "edit") then
        if (inputTable["BlockSites.KeywordBlockingTo"] == "ALL") then
        	local table = db.getTable ("LanGroup")
        	local i = 0
        	for k,v in pairs(table) do
		        i = i + 1
        		local row = table[i]
                local inTable = {}
                inTable["BlockSites.KeywordToBlock"] = inputTable["BlockSites.KeywordToBlock"]
                inTable["BlockSites.KeywordBlockingTo"] = row["LanGroup.GroupName"]
                inTable["BlockSites.Status"] = "1"
                local existsRows = db.existsRowWhere("BlockSites", "KeywordToBlock = '" .. inputTable["BlockSites.KeywordToBlock"] .. "' and KeywordBlockingTo = '" .. row["LanGroup.GroupName"] .. "'")      
                if (existsRows) then
                    valid = true
                else
                    valid = fw.config(inTable, rowid, "add")
                end
                if (valid) then
                else
            		db.rollback()
            		return "ERROR", "FW_KEYWORD_CONFIG_FAILED"
                end
            end
        else
            valid = fw.config(inputTable, rowid, operation)
        end
    end
	
	-- return
	if (valid) then
		db.commitTransaction(true)
	    return "OK", "STATUS_OK"            
	else
		db.rollback()
		return "ERROR", "FW_KEYWORD_CONFIG_FAILED"
	end
end

-- Trusted Domains Configuration
function fwView.trustedDomainConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false

   	valid = fw.config(inputTable, rowid, operation)
	
	-- return
	if (valid) then
		db.commitTransaction(true)
	    return "OK", "STATUS_OK"            
	else
		db.rollback()
		return "ERROR", "FW_DOMAINS_CONFIG_FAILED"
	end
end

--  Mac Address filter Configuration
function fwView.macFilteringConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false

   	valid = fw.config(inputTable, rowid, operation)
	
	-- return
	if (valid) then
		db.commitTransaction(true)
	    return "OK", "STATUS_OK"            
	else
		db.rollback()
		return "ERROR", "FW_MACFILTERING_CONFIG_FAILED"
	end
end

--  ip MAcBinding Configuration
function fwView.ipMacConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false

   	valid = fw.config(inputTable, rowid, operation)
	
	-- return
	if (valid) then
		db.commitTransaction(true)
	    return "OK", "STATUS_OK"            
	else
		db.rollback()
		return "ERROR", "FW_IPMACBINDING_CONFIG_FAILED"
	end
end

-- Delete Blocked Keywords
function fwView.deleteKeywords (rowIds)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"

    for k,v in pairs(rowIds) do
        local inTable = {}
        inTable["BlockSites._ROWID_"] = v;
        valid = fw.config(inTable, "-1", "delete")
        if (valid == false) then
            errStr = "FW_BLOCKED_KEYWORDS_DELETE_FAILED"
            break
        end
    end

    if (valid ~= false) then
        errStr = "FW_BLOCKED_KEYWORDS_DELETED"
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", errStr
	else
		db.rollback()
		return "ERROR", errStr
	end
end

-- Enable / Disable Blocked Keywords
function fwView.modifyKeywordStatus (rowIds, operation)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"
    local enableFlag = "1"
    if (operation == "enable") then
          enableFlag = "1"
    else
          enableFlag = "0"
    end

    for k,v in pairs(rowIds) do
        local inTable = {}
        inTable["BlockSites.Status"] = enableFlag;
        valid = fw.config(inTable, v, "edit")
        if (valid == false) then
            errStr = "FW_KEYWORD_CONFIG_FAILED"
            break
        end
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", errStr
	else
		db.rollback()
		return "ERROR", errStr
	end
end

-- Delete Trusted Domains
function fwView.deleteDomains (rowIds)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"

    for k,v in pairs(rowIds) do
        local inTable = {}
        inTable["TrustedDomains._ROWID_"] = v;
        valid = fw.config(inTable, "-1", "delete")
        if (valid == false) then
            errStr = "FW_TRUSTED_DOMAINS_DELETE_FAILED"
            break
        end
    end

    if (valid ~= false) then
        errStr = "FW_TRUSTED_DOMAINS_DELETED"
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", errStr
	else
		db.rollback()
		return "ERROR", errStr
	end
end

-- Delete Source MacFilterAddress 
function fwView.deleteMac (rowIds)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"

    for k,v in pairs(rowIds) do
        local inTable = {}
        inTable["SourceMACAddress._ROWID_"] = v;
        valid = fw.config(inTable, "-1", "delete")
        if (valid == false) then
            errStr = "FW_MAC_DELETE_FAILED"
            break
        end
    end

    if (valid ~= false) then
        errStr = "FW_MAC_DELETED"
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", errStr
	else
		db.rollback()
		return "ERROR", errStr
	end
end

-- Delete IpMacBindings 
function fwView.deleteIpMac (rowIds)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"

    for k,v in pairs(rowIds) do
        local inTable = {}
        inTable["IPMACBinding._ROWID_"] = v;
        valid = fw.config(inTable, "-1", "delete")
        if (valid == false) then
            errStr = "FW_IPMAC_DELETE_FAILED"
            break
        end
    end

    if (valid ~= false) then
        errStr = "FW_IPMAC_DELETED"
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", errStr
	else
		db.rollback()
		return "ERROR", errStr
	end
end

function fwView.getMaxPriority ()
    -- Get All Firewall Rules to find the newly added rule.
    local ruleType = "RuleType = 'SECURE_INSECURE'"
    ruleType = ruleType .. " or RuleType = 'INSECURE_SECURE'"
    ruleType = ruleType .. " or RuleType = 'PUBLIC_SECURE'"
    ruleType = ruleType .. " or RuleType = 'SECURE_PUBLIC'"
    ruleType = ruleType .. " or RuleType = 'INSECURE_PUBLIC'"
    ruleType = ruleType .. " or RuleType = 'PUBLIC_INSECURE'"

    local cur = db.execute(string.format([[SELECT MAX(PriorityId) PriorityId FROM FirewallRules WHERE %s]], ruleType))

 	-- check for rows
	local result = nil
	if (cur) then
		local row = cur:fetch ({}, "a")
		cur:close()
		if (row) then result = row["PriorityId"] or '' end
	end
	return result
end

-- Firewall Rules of Configuration
function fwView.firewallRulesConfig(inputTable, rowid, operation)
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transactions
    local valid = true

    if (inputTable["FirewallRules.RuleType"] == nil) then
        inputTable["FirewallRules.RuleType"] = inputTable["FirewallRules.FromZoneType"] .. "_" .. inputTable["FirewallRules.ToZoneType"]
    else
        local pos = string.find(inputTable["FirewallRules.RuleType"], "_")
        inputTable["FirewallRules.FromZoneType"] = string.sub(inputTable["FirewallRules.RuleType"], 0, pos-1)
        inputTable["FirewallRules.ToZoneType"] = string.sub(inputTable["FirewallRules.RuleType"], pos+1)
    end
    
    local newPriority = 0
    if (valid and operation == "add") then
        local maxPriorityId = fwView.getMaxPriority ()
        if (maxPriorityId == nil or
            maxPriorityId == "" or
            maxPriorityId == "0") then
            newPriority = 1
        else
            newPriority = tonumber(maxPriorityId) + 1
        end

	if (newPriority == 4294967295) then
	    newPriority = 1	
            while (true) do 
            	local priorityRow = db.getRow ("FirewallRules", "PriorityId", newPriority)
            	if (priorityRow == nil or priorityRow == "") then
                    break
            	else
                    newPriority = newPriority + 1
            	end
            end
        end
        inputTable["FirewallRules.PriorityId"] = newPriority
    end    

    valid = fw.config(inputTable, rowid, operation)


    -- Add the added rule into priority list
    if (valid and operation == "add") then
        local rulesPriority = db.getAttribute ("fwPriorityMap", "RuleType", "ALL", "PriorityStr") or ''
        if (rulesPriority == "") then
            -- First Firewall Rule Added
            rulesPriority = newPriority
        else
            rulesPriority = rulesPriority .. "," .. newPriority
        end

        --util.appendDebugOut("[SHV]Priority Order: " .. rulesPriority .. "<br>")

        local rulesPriConfig = db.getRow ("fwPriorityMap", "RuleType", "ALL")
        if (rulesPriConfig ~= nil) then
            rulesPriConfig["fwPriorityMap.PriorityStr"] = rulesPriority
            DBTable = "fwPriorityMap"
            valid = fw.config(rulesPriConfig, rulesPriConfig["fwPriorityMap._ROWID_"], "edit")
        else
            valid = false
        end
    end
    
    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "FW_CONFIG_FAILED"
    end
end

-- IPv6 Firewall Rules of Configuration
function fwView.ipv6FirewallRulesConfig(inputTable, rowid, operation)
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transactions
    local valid = true
    if (inputTable["FirewallRules6.RuleType"] == nil) then
        inputTable["FirewallRules6.RuleType"] = inputTable["FirewallRules6.FromZoneType"] .. "_" .. inputTable["FirewallRules6.ToZoneType"]
    else
        local pos = string.find(inputTable["FirewallRules6.RuleType"], "_")
        inputTable["FirewallRules6.FromZoneType"] = string.sub(inputTable["FirewallRules6.RuleType"], 0, pos-1)
        inputTable["FirewallRules6.ToZoneType"] = string.sub(inputTable["FirewallRules6.RuleType"], pos+1)
    end
    
    valid = fw.config(inputTable, rowid, operation)
    
    -- return
    if (valid) then
        db.commitTransaction(true)
    	return "OK", "STATUS_OK"
    else
	db.rollback()
	return "ERROR", "FW_CONFIG_FAILED"
    end
end

-- Routing Mode (NAT/Classical Routing) Configuration
function fwView.nwModeConfig (inputTable, rowid, operation)
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transactions
    local valid = true   
    DBTable = "NatTable"
    valid = fw.config(inputTable, rowid, operation)
    
    -- return
    if (valid) then
        db.commitTransaction(true)
    	return "OK", "STATUS_OK"
    else
	    db.rollback()
	    return "ERROR", "ROUTING_MODE_CONFIG_FAILED"
    end
end

-- Firewall Sessions Configurations
function fwView.fwSessionconfig (inputTable, rowid, operation)
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transactions
    local valid = true
    
    valid = fw.config(inputTable, rowid, operation)
    
    -- return
    if (valid) then
        db.commitTransaction(true)
    	return "OK", "STATUS_OK"
    else
	    db.rollback()
	    return "ERROR", "FW_SESSIONS_CONFIG_FAILED"
    end
end

-- Firewall Traffic Meter Configuration
function fwView.trafficMeterConfig (inputTable, rowid, operation)
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transactions
    local valid = true
    
    valid = fw.config(inputTable, rowid, operation)
    
    -- return
    if (valid) then
        db.commitTransaction(true)
    	return "OK", "STATUS_OK"
    else
	    db.rollback()
	    return "ERROR", "FW_TM_CONFIG_FAILED"
    end
end

-- NATServices Delete
function fwView.deleteNatRules (rowIds)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions
    local valid = true

        for k,v in pairs(rowIds) do
            local inTable = {}
            inTable["OneToOneNat._ROWID_"] = v;
            valid = fw.config(inTable, "-1", "delete")
	end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "CONFIG_FAILED"
	end

end

function fwView.natSeviceConfig (inputTable, rowid, operation)
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transactions
    local valid = true
    
    valid = fw.config(inputTable, rowid, operation)
    
    -- return
    if (valid) then
        db.commitTransaction(true)
    	return "OK", "STATUS_OK"
    else
	    db.rollback()
	    return "ERROR", "CONFIG_FAILED"
    end
end

function fwView.deleteNatServices (rowIds)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions
    local valid = true

        for k,v in pairs(rowIds) do
            local inTable = {}
            inTable["OneToOneNatService._ROWID_"] = v;
            valid = fw.config(inTable, "-1", "delete")
	end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "CONFIG_FAILED"
	end

end
