-- idsCLI.lua
-- Samer Sayeed
-- TeamF1
-- www.TeamF1.com

-- Modification History
-- 31jan09,ss  fixed cli related changes for spr 10442
-- 31jan09,ss  modified idsCfgSave routine to reboot device only when ids is enabled.
-- 26dec08,ss  written
--
-- Description
-- CLI interface set and get routines of ids and session settings

require "teamf1lualib/ids"
require "teamf1lualib/fwView"

function idsCfgInit(args)
    local configRow = {}
    configRow = db.getRow("IdsConfiguration","_ROWID_", "1")

    return 1, configRow
end

function idsCfgSave(configRow)
    errorFlag = "ERROR"
    statusCode = ""
    statusMessage = ""
    local triesMax = 5 -- hard-coded
    local triesCur = 0
    local confirm = ""
    local reboot = "0"
    local message = "Enabling IDS reboots the device.\n"
    DBTable = "IdsConfiguration"
   
    local idsStatus = db.getAttribute ("IdsConfiguration", "IdsConfiguration._ROWID_", "1", "IdsStatus")
    if(configRow["IdsConfiguration.IdsStatus"] == "1" and idsStatus == "0") then
        printCLIError (message)
        while (confirm ~= "Y" and confirm ~= "N" and triesCur < triesMax) do
	    printCLIError ("Are you sure you want to proceed [Y/N]?")
	    confirm = io.stdin:read'*l'
	    triesCur = triesCur + 1
        end
        if (confirm == "N" or (triesCur == triesMax)) then
            errorFlag = "ERROR"
            statusMessage = "Action Cancelled"
            return errorFlag, statusMessage
        elseif(confirm == "Y") then reboot = "1" end
    end
    
    errorFlag, statusCode = ids.config(configRow, "1", "edit")
    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    if (reboot == "1") then utilCLIReboot() end
    return errorFlag, statusMessage
end

function idsCfgVal(configRow)
    return true
end

function idsInfoGet()
    printLabel ("IDS")
    row = db.getRow("IdsConfiguration", "IdsConfiguration._ROWID_",1)
--  The IdsConfiguration table should have only one entry
     
    print ("IDS Settings")
    if (row["IdsConfiguration.IdsStatus"] == "0") then
       print( " Intrusion Detection  : Disabled")
    elseif(row["IdsConfiguration.IdsStatus"] == "1") then 
       print( " Intrusion Detection  : Enabled")
       --if (row["IdsConfiguration.BlackListingStatus"] == "0") then
       --    print( " Black Listing        : Disabled")
       --elseif (row["IdsConfiguration.BlackListingStatus"] == "1") then 
       --    print( " Black Listing        : Enabled")    
       --end
       --if (row["IdsConfiguration.BlockLogStatus"] == "0") then
       --    print( " Blocking Log         : Disabled")      
       --elseif (row["IdsConfiguration.BlockLogStatus"] == "1") then 
       --    print( " Blocking Log         : Enabled")    
       --end
       if (row["IdsConfiguration.IntrusionLogStatus"] == "0") then
           print( " Intrusion Log        : Disabled")      
       elseif(row["IdsConfiguration.IntrusionLogStatus"] == "1") then 
           print( " Intrusion Log        : Enabled")    
       end
    end
end

function sessionSettingsCfgSave(configRow)
    errorFlag = "ERROR"
    statusCode = ""
    statusMessage = "" 
    DBTable = "GlobalSettings"
    
    if (db.existsRow("GlobalSettings","_ROWID_","1")) then
        errorFlag, statusCode = fwView.fwSessionconfig (configRow, "1", "edit")
    else
	errorFlag, statusCode = fwView.fwSessionconfig (configRow, "-1", "add")	
    end

    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or statusCode

    return errorFlag, statusMessage
end

function sessionSettingsCfgInit(args)
    local configRow = {}
    configRow = db.getRow("GlobalSettings","_ROWID_", "1")
    if (configRow == nil) then
    	configRow = {}
        configRow["GlobalSettings._ROWID_"] = "-1"
    end 
    return configRow["GlobalSettings._ROWID_"], configRow
end

function sessionSettingsInfoGet()
    printLabel ("Session Settings")
    row = db.getRow("GlobalSettings", "GlobalSettings._ROWID_",1)

    --  The GlobalSettings table should have only one entry
     
    print ("Maximum Unidentified Sessions:" .. row["GlobalSettings.MaxUnidentifiedSessions"])
    print ("Maximum Half Open Sessions:" .. row["GlobalSettings.MaxHalfOpenSessions"])
    print ("TCP Session Timeout Duration:" .. row["GlobalSettings.TcpSessionTimoutDuration"] .. "(Secs)")
    print ("UDP Session Timeout Duration:" .. row["GlobalSettings.UdpSessionTimoutDuration"] .. "(Secs)")
    print ("Other Session Timeout Duration:" .. row["GlobalSettings.OtherSessionTimeoutDuration"] .. "(Secs)")
    print ("TCP Session Cleanup Latency:" .. row["GlobalSettings.TcpSessionCleanupLatency"] .. "(Secs)")
end
