-- ifDevCLI.lua
-- Gaurav Mathur
-- TeamF1
-- www.TeamF1.com

-- Modification History
-- 04mar10,db  modified pool related routines to fix issues.
-- 17feb09,ss  added check in lanCfgInputVal() to fix spr#10746, 
-- 23jan09,ss  added ipv6 lan setup routines, 
--             modified ipv4 lan setup, display routines. 
-- 25dec07,gnm Modified bdgCLIStatsGet() to use ifStatsPrint() to
--             print fields to CLI; Added ifStatsPrint(); Added
--	       ifCLIStatsGet() as generic interface stats print
--	       routine.
-- 23dec07,gnm modified aliasipGet(); added lanSetupGet; added
--             bdgCLIStatsGet()
-- 09dev07 gnm written
--
-- Description
-- CLI interface set and get routines

require "teamf1lualib/ifDev"
require "teamf1lualib/lan"
require "teamf1lualib/dhcp"
require "teamf1lualib/igmp"
require "teamf1lualib/validations"
--
-- This routine validates the LAN configuration information.
function lanCfgInputVal(configRow)
    if (configRow["dhcp.dhcpEnabled"] == "1" and 
        (configRow["dhcp.startAddress"] == nil or 
         configRow["dhcp.startAddress"] == "" or 
         configRow["dhcp.startAddress"] == "0.0.0.0")) then
        printCLIError ("Invalid Start IP address")
        return false
    end
    if (configRow["dhcp.dhcpEnabled"] == "1" and 
        (configRow["dhcp.endAddress"] == nil or 
         configRow["dhcp.endAddress"] == "" or 
         configRow["dhcp.endAddress"] == "0.0.0.0")) then
        printCLIError ("Invalid End IP address")
        return false
    end
    if (configRow["dhcp.dhcpEnabled"] == "1" and 
        (configRow["dhcp.leaseTime"] == "" or
         configRow["dhcp.leaseTime"] == nil)) then
        printCLIError ("Invalid lease Time")
        return false
    end

    if (configRow["dhcp.dhcpEnabled"] == "3" and 
       (configRow["DhcpRelay.RelayGateway"] == nil or 
        configRow["DhcpRelay.RelayGateway"] == "" or 
        configRow["DhcpRelay.RelayGateway"] == "0.0.0.0")) then
        printCLIError ("Invalid relay gateway")
        return false
    end
    return true
end

--
-- This routine validates the LAN configuration information.
function lanIPv6CfgInputVal(configRow)
    local ipmode = db.getRow("networkInfo", "_ROWID_", "1")
    if(ipmode["networkInfo.netWorkMode"] == "1")then
    printCLIError("Please Set IP Mode to IPv4/IPv6 to configure IPv6 LAN Configurations.")
    return false
    end
    if ((configRow["ifStatic.PrefixLength"] == nil) or (tonumber(configRow["ifStatic.PrefixLength"]) < 0) or (tonumber(configRow["ifStatic.PrefixLength"]) > 128 ))then
        printCLIError ("Invalid Prefix. Please enter a value between 0 and 128.")
        return false
    end

    return true
end

--
-- This routine validates the LAN configuration information.
function lanIPv6CfgPoolInputVal(configRow)
    local ipmode = db.getRow("networkInfo", "_ROWID_", "1")
    if(ipmode["networkInfo.netWorkMode"] == "1")then
    printCLIError("Please Set IP Mode to IPv4/IPv6 to configure IPv6 LAN Configurations.")
    return false
    end
    if (configRow ["dhcpv6sLANAddrPool.startAddress"] == "" or configRow ["dhcpv6sLANAddrPool.startAddress"] == nil) then
         printCLIError("Please Enter valid IPv6 Start Address.")
         return false

    end
    if (configRow ["dhcpv6sLANAddrPool.endAddress"] == "" or configRow ["dhcpv6sLANAddrPool.endAddress"] == nil) then
         printCLIError("Please Enter valid IPv6 End Address.")
         return false
    end

    if ((configRow["dhcpv6sLANAddrPool.prefixLength"] == nil) or (tonumber(configRow["dhcpv6sLANAddrPool.prefixLength"]) < 0) or (tonumber(configRow["dhcpv6sLANAddrPool.prefixLength"]) > 128 ))then
        printCLIError ("Invalid Prefix. Please enter a value between 0 and 128.")
        return false
    end
    return true
end

--
-- This routine saves the modified LAN configuration information.
function lanCfgSave(configRow)
    local dmzConfig = db.getRowWhere ("ifStatic", "LogicalIfName = 'DMZ' AND AddressFamily = 2")
	if (dmzConfig ~= nil and dmzConfig["ifStatic.StaticIp"] == configRow["ifStatic.StaticIp"]) then
		errorFlag, statusCode = "ERROR", "IP_CONFLICTING_WITH_DMZ_IPADDR"
	else
		local where = "LogicalIfName = 'LAN' AND AddressFamily = 2"
		local rowid = db.existsRowWhere ("ifStatic", where)
		errorFlag, statusCode = lan.ipv4LanConfig(configRow, configRow["ifStatic._ROWID_"], "edit")
		if (errorFlag == "OK") then
			db.save()
			errorFlag, statusCode = dhcp.dhcpConfigWrap (configRow,"LAN")
			if (errorFlag == "OK") then
				db.save()
				end
			end
		end
	
	-- save db if no error
	if (errorFlag == "OK") then db.save() end
	statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
        return errorFlag, statusMessage
end

--
-- This routine saves the modified LAN configuration information.
function lanIPv6CfgSave(configRow)
    local errorFlag = nil
    local statusCode = ""
    local statusMessage = nil

    local where = "LogicalIfName = 'LAN' AND AddressFamily = '10'"
	local rowid = db.existsRowWhere ("ifStatic", where)
	local valid = lan.ipv6LanConfig(configRow, rowid, "edit")

	-- save db if no error
    if (valid) then 
        statusCode = "STATUS_OK"
	errorFlag = "OK"
        db.save()
    else
	errorFlag = "ERROR"
	statusCode = "LAN_CONFIG_FAILED"
    end

    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage           	 
end

--
-- This routine reads in the current IPv6 LAN configuration information
-- into a configuration object.
function lanIPv6CfgInit(args)
    configRow = db.getRow("dhcpv6s", "_ROWID_", "1")
	if (configRow == nil) then
	    configRow = db.getDefaults(true, "dhcpv6s")
	end
    local staticRow = {}
    local where = "LogicalIfName = 'LAN' AND AddressFamily = 10"
    local rowid = db.existsRowWhere ("ifStatic", where)
    staticRow = db.getRow("ifStatic", "_ROWID_", rowid )
    if(staticRow ~= nil) then
        configRow["ifStatic.StaticIp"] = staticRow["ifStatic.StaticIp"]
        configRow["ifStatic.PrefixLength"] = staticRow["ifStatic.PrefixLength"]
    end
    return rowid, configRow
end

--
-- This routine reads in the current IPv4 LAN configuration information
-- into a configuration object.
function lanCfgInit(args)
    local where = "LogicalIfName = 'LAN' AND AddressFamily = 2"
	local rowid = db.existsRowWhere ("ifStatic", where)
    configRow = db.getRow("ifStatic", "_ROWID_", rowid)	
    local dhcpConfigRow = db.getRow ("dhcp", "logicalIfName", "LAN")
    if (dhcpConfigRow ~= nil) then
    	configRow ["dhcp.dhcpEnabled"] = dhcpConfigRow["dhcp.dhcpEnabled"]
   		if (configRow ["dhcp.dhcpEnabled"] ~= 0) then
    		configRow ["dhcp.domainName"] = dhcpConfigRow ["dhcp.domainName"]
    		configRow ["dhcp.startAddress"] = dhcpConfigRow ["dhcp.startAddress"]
            configRow ["dhcp.endAddress"] = dhcpConfigRow ["dhcp.endAddress"]
            configRow ["dhcp.primaryDnsServer"] = "" 
            configRow ["dhcp.secondaryDnsServer"] = "" 
            configRow ["dhcp.winsServer"] = " "
            configRow ["dhcp.leaseTime"] = dhcpConfigRow ["dhcp.leaseTime"]
            configRow ["dhcp.enableDnsProxy"] = "1"
    	end
    end

    local dhcpRelayConfig = db.getRow("DhcpRelay", "LogicalIfName", "LAN")
    if (dhcpRelayConfig ~= nil) then
    	if (dhcpRelayConfig["DhcpRelay.DhcpRelayStatus"] == "1") then
    		configRow ["dhcp.dhcpEnabled"] = "3"
    	end
    	configRow ["DhcpRelay.RelayGateway"] = dhcpRelayConfig["DhcpRelay.RelayGateway"]
    end
  
    local igmpConfig = db.getRow("Igmp", "_ROWID_", "1")
    if (igmpConfig ~=nil) then
        configRow ["Igmp.IgmpEnable"] = igmpConfig["Igmp.IgmpEnable"]
    end
    return rowid, configRow
end

function lanIPv6SetupGet (args)
    local resultTab = {}
    local resultTabPool = {}
    local dhcpv6sRow = db.getRow("dhcpv6s", "_ROWID_", "1")
    local where = "LogicalIfName = 'LAN' AND AddressFamily = 10"
    local rowid = db.existsRowWhere ("ifStatic", where)
    local ipv6LanRow = db.getRow("ifStatic", "_ROWID_", rowid )
    local dhcpv6sPoolTable = db.getTable ("dhcpv6sLANAddrPool")
    local dhcpv6sPoolRow = nil
    local rowId = 0

    local ipmode = db.getRow("networkInfo", "_ROWID_", "1")
    if(ipmode["networkInfo.netWorkMode"] == "1")then
    printCLIError("Please Set IP Mode to IPv4/IPv6 to access IPv6 LAN Configurations.")
    return false
    end

    printLabel ("IPv6 LAN Configuration")
    resTab.insertField (resultTab, "LAN TCP/IP Setup","")
    if(ipv6LanRow ~= nil) then
        resTab.insertField (resultTab, "IPv6 Address", ipv6LanRow["ifStatic.StaticIp"] or "")
        resTab.insertField (resultTab, "IPv6 Prefix Length", ipv6LanRow["ifStatic.PrefixLength"]or "")   
    end

    resTab.insertField (resultTab, "DHCPv6", "")
    if(dhcpv6sRow ~= nil) then
       if(dhcpv6sRow["dhcpv6s.isEnabled"]=="1")then
           resTab.insertField (resultTab, "DHCP Status", "Enable DHCPv6 Server")
       else
           resTab.insertField (resultTab, "DHCP Status", "Disable DHCPv6 Server")
       end
       if(dhcpv6sRow["dhcpv6s.statelessMode"]=="1")then
           resTab.insertField (resultTab, "DHCP Mode", "Stateless")
       else
           resTab.insertField (resultTab, "DHCP Mode", "Stateful")
       end
       resTab.insertField (resultTab, "Domain Name", dhcpv6sRow["dhcpv6s.domainName"]or "")
       resTab.insertField (resultTab, "Server Preference", dhcpv6sRow["dhcpv6s.serverPreference"]or "")
       if(dhcpv6sRow["dhcpv6s.useDNSServersFrom"]=="1")then
           resTab.insertField (resultTab, "DNS Servers", "Use DNS Proxy")
       elseif(dhcpv6sRow["dhcpv6s.useDNSServersFrom"]=="2")then
           resTab.insertField (resultTab, "DNS Servers", "Use DNS from ISP")
       elseif(dhcpv6sRow["dhcpv6s.useDNSServersFrom"]=="3")then
           resTab.insertField (resultTab, "DNS Servers", "Use Below")
           resTab.insertField (resultTab, "Primary DNS Server", dhcpv6sRow["dhcpv6s.primaryDNSServer"]or "")
           resTab.insertField (resultTab, "Secondary DNS Server", dhcpv6sRow["dhcpv6s.secondaryDNSServer"]or "")
       end
       resTab.insertField (resultTab, "Lease/Rebind Time", dhcpv6sRow["dhcpv6s.leaseTime"]or "")
    end
   
    for col, val in pairs(dhcpv6sPoolTable) do
       rowId = rowId+1
       local dhcpv6sPoolRow = dhcpv6sPoolTable[rowId]
       if (rowId ~= nil) then 
       resTab.insertField (resultTabPool, "Start Address", 
			      dhcpv6sPoolRow["dhcpv6sLANAddrPool.startAddress"] or "")
       resTab.insertField (resultTabPool, "End Address", dhcpv6sPoolRow["dhcpv6sLANAddrPool.endAddress"] or "")
       end
    end
    resTab.print (resultTab, 0)
    printLabel ("List of IPv6 Address Pools")
    resTab.print (resultTabPool, 0)
end  

--
-- This routine implements the CLI command to show the LAN
-- configuration
function lanSetupGet (args)
    local resultTab = {} -- table to store LAN setup information
    local ifdevTable = nil
    local where = "LogicalIfName = 'LAN' AND AddressFamily = 2"
    local rowid = db.existsRowWhere ("ifStatic", where)
    local ipv4LanRow = db.getRow("ifStatic", "_ROWID_", rowid )

    printLabel ("LAN Setup")
    resTab.insertField (resultTab, "LAN TCP/IP Setup","")
    if(ipv4LanRow ~= nil) then
        resTab.insertField (resultTab, "IPv4 Address", ipv4LanRow["ifStatic.StaticIp"] or "")
        resTab.insertField (resultTab, "Subnet Mask", ipv4LanRow["ifStatic.NetMask"]or "")   
    end

    resTab.insertField (resultTab, "DHCP Setup Configuration", "")    
    dhcpTable = db.getRow ("dhcp","_ROWID_", 1)
    dhcpRelayTable = db.getRow ("DhcpRelay","_ROWID_", 1)
    
    if (dhcpTable ~= nil and dhcpRelayTable ~= nil) then 
        if (dhcpTable ["dhcp.dhcpEnabled"] == "0" and dhcpRelayTable ["DhcpRelay.DhcpRelayStatus"] == "1") then
               resTab.insertField (resultTab, "DHCP Mode", "DHCP Relay")
        end
        if (dhcpTable ["dhcp.dhcpEnabled"] == "1" and dhcpRelayTable ["DhcpRelay.DhcpRelayStatus"] == "0") then
               resTab.insertField (resultTab, "DHCP Mode", "DHCP Server")
        end
        if (dhcpTable ["dhcp.dhcpEnabled"] == "0" and dhcpRelayTable ["DhcpRelay.DhcpRelayStatus"] == "0") then
               resTab.insertField (resultTab, "DHCP Mode", "None")
        end
        if (dhcpTable ["dhcp.dhcpEnabled"] == "1") then 
           resTab.insertField (resultTab, "Domain Name", 
	                      dhcpTable["dhcp.domainName"] or "")
           resTab.insertField (resultTab, "Starting IP Address", 
	                      dhcpTable["dhcp.startAddress"] or "")
           resTab.insertField (resultTab, "Ending IP Address", 
			      dhcpTable["dhcp.endAddress"] or "")
           resTab.insertField (resultTab, "Primary DNS Server", 
			      dhcpTable["dhcp.primaryDnsServer"] or "")
           resTab.insertField (resultTab, "Secondary DNS Server", 
			      dhcpTable["dhcp.secondaryDnsServer"] or "")
           resTab.insertField (resultTab, "WINS Server", 
			      dhcpTable["dhcp.winsServer"] or "")
           resTab.insertField (resultTab, "Lease Time in hrs ", 
			      dhcpTable["dhcp.leaseTime"] or "")
        elseif (dhcpRelayTable ["DhcpRelay.DhcpRelayStatus"] == "1") then
            resTab.insertField (resultTab, "Relay Gateway", 
			      dhcpRelayTable["DhcpRelay.RelayGateway"] or "")
        end
    end		               
    IgmpTable = db.getRow ("Igmp","_ROWID_", 1)
    resTab.insertField (resultTab, "LAN Proxies", "")
    if (dhcpTable["dhcp.enableDnsProxy"] == "1") then 
        resTab.insertField (resultTab, "DNS Proxy", " Enabled")
    else 
        resTab.insertField (resultTab, "DNS Proxy", " Disabled")
    end
    resTab.print (resultTab, 0)
end

--
-- This routine reads in the current dhcpv6sLANAddrPool configuration information
-- into a configuration object.
function lanIPv6PoolInit (args)
    local startAddr = args[1]
    local rowId = -1

    local configRow = db.getRow("dhcpv6sLANAddrPool","startAddress", startAddr)
    if (configRow == nil) then
        configRow = {}
        configRow["dhcpv6sLANAddrPool.startAddress"] = startAddr
        configRow ["dhcpv6sLANAddrPool._ROWID_"] = "-1"
    else
        rowId = configRow["dhcpv6sLANAddrPool._ROWID_"]
    end
    return rowId, configRow
end

--
-- This routine saves the modified dhcpv6sLANAddrPool information.
function lanIPv6PoolSave (configRow)
    local errorFlag = nil
    local statusCode = ""
    local statusMessage = nil
    DBTable = "dhcpv6sLANAddrPool"

    if (configRow ["dhcpv6sLANAddrPool._ROWID_"] == "-1") then
        errorFlag, statusCode = lan.poolConfig(configRow, "-1", "add")
    else
        errorFlag, statusCode = lan.poolConfig(configRow, configRow ["dhcpv6sLANAddrPool._ROWID_"], "edit")
    end

    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
   
    return errorFlag, statusMessage
end

--
-- This routine delete the modified dhcpv6sLANAddrPool configuration information.
function lanIPv6PoolDel (startAddress)
    local errorFlag = nil
    local statusCode = ""
    local statusMessage = nil
    DBTable = "dhcpv6sLANAddrPool"

    local startAddr = startAddress
    local rowId = -1

    local configRow = db.getRow("dhcpv6sLANAddrPool","startAddress", startAddr)
    if (configRow == nil) then
        printCLIError ("Start Address doesn't match with any of the configuration")
    else
        rowId = configRow["dhcpv6sLANAddrPool._ROWID_"]
    end
    row = {}
    row["_ROWID_"] = configRow ["dhcpv6sLANAddrPool._ROWID_"]
    errorFlag, statusCode = lan.lanPoolsDelete(row)

    -- save db if no error
    if (errorFlag == "OK") then 
        db.save() 
        print ("Row deleted")
    end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
   
    return errorFlag, statusMessage
end
  
--
-- This routine takes in a table which has sets of interface
-- statistics and prints them
function ifStatsPrint (table)
    local resultTab = {}

    for _, row in pairs(table) do
	resTab.insertField (resultTab, "IFACE", row["interfaceStats.interfaceName"])
	resTab.insertField (resultTab, "PktRx", row["interfaceStats.rx_packets"])
	resTab.insertField (resultTab, "PktTx", row["interfaceStats.tx_packets"])
	resTab.insertField (resultTab, "ByteRx", row["interfaceStats.rx_bytes"])
	resTab.insertField (resultTab, "ByteTx", row["interfaceStats.tx_bytes"])
	resTab.insertField (resultTab, "ErrRx", row["interfaceStats.rx_errors"])
	resTab.insertField (resultTab, "ErrTx", row["interfaceStats.tx_errors"])
	resTab.insertField (resultTab, "DropRx", row["interfaceStats.rx_dropped"])
	resTab.insertField (resultTab, "DropTx", row["interfaceStats.tx_dropped"])
	resTab.insertField (resultTab, "Mcast", row["interfaceStats.multicast"])
	resTab.insertField (resultTab, "Coll", row["interfaceStats.collisions"])
    end
    printLabel ("Interface Statistics")
    resTab.print (resultTab, 0)
end

--
-- This routine implements the CLI command to display bridge interface
-- statistics.
function bdgCLIStatsGet (args)
    -- run program to read the stats and update DB
    local prog = db.getAttribute("environment", "name", "IFDEVSTATS_PROGRAM", "value")
    os.execute(prog .. " " .. DB_FILE_NAME)

    local bdgTab = db.getTable ("bridgeTable")
    local resultTab = {}
    local tableIfaceInfo = {}

    for _,v in pairs(bdgTab) do
        local row = db.getRow("interfaceStats", "interfaceStats.interfaceName", 
	      v["bridgeTable.interfaceName"])
	if (row ~= nil) then
	    table.insert (tableIfaceInfo, row)
        end
    end

    ifStatsPrint (tableIfaceInfo)
    resTab.print (resultTab, 0)
end   

--
-- This routine can be used to print statistics for a set of
-- one or more interfaces. The set of interfaces is passed to this
-- routine as a table with interface names are it's elements
function ifCLIStatsGet (args)
    local tableIfaceInfo = {}	 
    local row = nil

    -- run program to read the stats and update DB
    local prog = db.getAttribute("environment", "name", "IFDEVSTATS_PROGRAM", "value")
    os.execute(prog .. " " .. DB_FILE_NAME)
    for _, iface in pairs (args) do
	row = db.getRow ("interfaceStats", "interfaceStats.interfaceName", iface)
	if (row ~= nil) then
	    table.insert (tableIfaceInfo, row)
        end
    end
    ifStatsPrint (tableIfaceInfo)
end
   
