--[[
#### Vivek Dutt Sharma.
#### TeamF1
#### www.TeamF1.com
#### Jan 21, 2010
#### File: ipSec.lua
#### Description: ipSec functions
#### Revisions:
]]--

--************* Initial Code *************

--package ipsec
ipsec = {}

--************* Functions *************
-- ipsec config
--[[
/******************************************************************************
*
* ipSec.lua - Used to collect values related to vpnPolicy and ikePolicy. 
*  RETURNS: Returns table values for vpnPolicy and ikePolicy 
* 
*
*/
]]--


--  Collecting vpnPolicy values in a table and returning it.
function ipsec.getVpnPolicyPara (inputTable)
	local vpnTable = {}
	local DEF_NETWORK = "0"
	if(inputTable["IpsecVPNPolicy.VPNPolicyName"] ~= nil) then
	    vpnTable["IpsecVPNPolicy.VPNPolicyName"] = inputTable["IpsecVPNPolicy.VPNPolicyName"]
	end	
	-- Checking for Policy Type 
	if(inputTable["IpsecVPNPolicy.PolicyType"] ~= nil) then
		if(inputTable["IpsecVPNPolicy.PolicyType"] == "1") then  -- Auto Policy
			vpnTable["IpsecVPNPolicy.PolicyType"] = inputTable["IpsecVPNPolicy.PolicyType"]
			if(inputTable["IpsecVPNPolicy.VPNPolicyName"] ~= nil) then
	    		vpnTable["IpsecVPNPolicy.IKEPolicyName"] = inputTable["IpsecVPNPolicy.VPNPolicyName"]
			end				
			if(inputTable["IpsecVPNPolicy.SALifeTime"] ~= nil) then
        		vpnTable["IpsecVPNPolicy.SALifeTime"] = inputTable["IpsecVPNPolicy.SALifeTime"]
    		end
    		if(inputTable["IpsecVPNPolicy.SALifeTimeType"] ~= nil) then
        		vpnTable["IpsecVPNPolicy.SALifeTimeType"] = inputTable["IpsecVPNPolicy.SALifeTimeType"]
   			 end
   			if(inputTable["IpsecVPNPolicy.EncryptionAlgorithm"] ~= nil) then
        		vpnTable["IpsecVPNPolicy.EncryptionAlgorithm"] = inputTable["IpsecVPNPolicy.EncryptionAlgorithm"]
    		end
    		if(inputTable["IpsecVPNPolicy.KeyLength"] ~= nil) then
        		vpnTable["IpsecVPNPolicy.KeyLength"] = inputTable["IpsecVPNPolicy.KeyLength"]
    		end
    		if(inputTable["IpsecVPNPolicy.AuthAlgorithm"] ~= nil) then
        		vpnTable["IpsecVPNPolicy.AuthAlgorithm"] = inputTable["IpsecVPNPolicy.AuthAlgorithm"]
    		end
    		if(inputTable["IpsecVPNPolicy.PFSKeyGroup"] ~= nil) then
        		vpnTable["IpsecVPNPolicy.PFSKeyGroup"] = inputTable["IpsecVPNPolicy.PFSKeyGroup"]
    		end
    		if(inputTable["IpsecVPNPolicy.PFSKeyGroup1"] ~= nil) then
        		vpnTable["IpsecVPNPolicy.PFSKeyGroup1"] = inputTable["IpsecVPNPolicy.PFSKeyGroup1"]
   			end
		elseif(inputTable["IpsecVPNPolicy.PolicyType"] == "2") then  -- manual Policy
			vpnTable["IpsecVPNPolicy.PolicyType"] = inputTable["IpsecVPNPolicy.PolicyType"]
			if(inputTable["IpsecVPNPolicy.VPNPolicyName"] ~= nil) then
	    		vpnTable["IpsecVPNPolicy.IKEPolicyName"] = inputTable["IpsecVPNPolicy.VPNPolicyName"]
			end				
			if(inputTable["IpsecVPNPolicy.SPIIn"] ~= nil) then
		        vpnTable["IpsecVPNPolicy.SPIIn"] = inputTable["IpsecVPNPolicy.SPIIn"]
		    end
		    if(inputTable["IpsecVPNPolicy.SPIOut"] ~= nil) then
		        vpnTable["IpsecVPNPolicy.SPIOut"] = inputTable["IpsecVPNPolicy.SPIOut"]
		    end 
		    if(inputTable["IpsecVPNPolicy.EncryptionAlgorithm"] ~= nil) then
        		vpnTable["IpsecVPNPolicy.EncryptionAlgorithm"] = inputTable["IpsecVPNPolicy.EncryptionAlgorithm"]
    		end
    		if(inputTable["IpsecVPNPolicy.KeyLength"] ~= nil) then
        		vpnTable["IpsecVPNPolicy.KeyLength"] = inputTable["IpsecVPNPolicy.KeyLength"]
    		end		    
		    if(inputTable["IpsecVPNPolicy.EncryptionKeyIn"] ~= nil) then
		        vpnTable["IpsecVPNPolicy.EncryptionKeyIn"] = inputTable["IpsecVPNPolicy.EncryptionKeyIn"]
		    end
		    if(inputTable["IpsecVPNPolicy.EncryptionKeyOut"] ~= nil) then
		        vpnTable["IpsecVPNPolicy.EncryptionKeyOut"] = inputTable["IpsecVPNPolicy.EncryptionKeyOut"]
		    end
		    if(inputTable["IpsecVPNPolicy.AuthAlgorithm"] ~= nil) then
        		vpnTable["IpsecVPNPolicy.AuthAlgorithm"] = inputTable["IpsecVPNPolicy.AuthAlgorithm"]
    		end		    
		    if(inputTable["IpsecVPNPolicy.AuthKeyIn"] ~= nil) then
		        vpnTable["IpsecVPNPolicy.AuthKeyIn"] = inputTable["IpsecVPNPolicy.AuthKeyIn"]
		    end
		    if(inputTable["IpsecVPNPolicy.AuthKeyOut"] ~= nil) then
		        vpnTable["IpsecVPNPolicy.AuthKeyOut"] = inputTable["IpsecVPNPolicy.AuthKeyOut"]
		    end	    		
		end       
    end    
   
    if(inputTable["IpsecVPNPolicy.Netbios"] ~= nil) then
        vpnTable["IpsecVPNPolicy.Netbios"] = inputTable["IpsecVPNPolicy.Netbios"]
    end
    if(inputTable["IpsecVPNPolicy.EnableDhcpOverIPSec"] ~= nil) then
        vpnTable["IpsecVPNPolicy.EnableDhcpOverIPSec"] = inputTable["IpsecVPNPolicy.EnableDhcpOverIPSec"]
    end
    if(inputTable["IpsecVPNPolicy.Rollover"] ~= nil) then
        vpnTable["IpsecVPNPolicy.Rollover"] = inputTable["IpsecVPNPolicy.Rollover"]
    end
    
    -- Checking for local type and assigning values for fields
    if(inputTable["IpsecVPNPolicy.TunnelOrTransport"] ~= nil) then
        vpnTable["IpsecVPNPolicy.TunnelOrTransport"] = inputTable["IpsecVPNPolicy.TunnelOrTransport"]
    end
    -- Remote and local type depends on ipsecMode
    if(inputTable["IpsecVPNPolicy.TunnelOrTransport"] == "1" and vpnTable["IpsecVPNPolicy.EnableDhcpOverIPSec"] == "0") then
	    if(inputTable["IpsecVPNPolicy.LocalNetworkType"] ~= nil) then
	        vpnTable["IpsecVPNPolicy.LocalNetworkType"] = inputTable["IpsecVPNPolicy.LocalNetworkType"]
	    end
	    if(vpnTable["IpsecVPNPolicy.LocalNetworkType"] == "1") then
	    	if(inputTable["IpsecVPNPolicy.LocalStartAddress"] ~= nil) then
	        	vpnTable["IpsecVPNPolicy.LocalStartAddress"] = inputTable["IpsecVPNPolicy.LocalStartAddress"]
	    	end
	    elseif(vpnTable["IpsecVPNPolicy.LocalNetworkType"] == "2")then
	    		if(inputTable["IpsecVPNPolicy.LocalStartAddress"] ~= nil) then
	        		vpnTable["IpsecVPNPolicy.LocalStartAddress"] = inputTable["IpsecVPNPolicy.LocalStartAddress"]
	    		end
	    		if(inputTable["IpsecVPNPolicy.LocalEndAddress"] ~= nil) then
	        		vpnTable["IpsecVPNPolicy.LocalEndAddress"] = inputTable["IpsecVPNPolicy.LocalEndAddress"]
	    		end
	    elseif(vpnTable["IpsecVPNPolicy.LocalNetworkType"] == "3")then
				if(inputTable["IpsecVPNPolicy.LocalStartAddress"] ~= nil) then
	        		vpnTable["IpsecVPNPolicy.LocalStartAddress"] = inputTable["IpsecVPNPolicy.LocalStartAddress"]
	    		end
	    		if(inputTable["IpsecVPNPolicy.LocalSubnetMask"] ~= nil) then
	        		vpnTable["IpsecVPNPolicy.LocalSubnetMask"] = inputTable["IpsecVPNPolicy.LocalSubnetMask"]
	    		end
	    end	
	    
	    -- Checking for remote type and assigning values for fields
	    
	    if(inputTable["IpsecVPNPolicy.RemoteNetworkType"] ~= nil) then
	        vpnTable["IpsecVPNPolicy.RemoteNetworkType"] = inputTable["IpsecVPNPolicy.RemoteNetworkType"]
	    end
	    if(vpnTable["IpsecVPNPolicy.RemoteNetworkType"] == "1") then
	    	if(inputTable["IpsecVPNPolicy.RemoteStartAddress"] ~= nil) then
	        	vpnTable["IpsecVPNPolicy.RemoteStartAddress"] = inputTable["IpsecVPNPolicy.RemoteStartAddress"]
	    	end
	    elseif(vpnTable["IpsecVPNPolicy.RemoteNetworkType"] == "2")then
	    		if(inputTable["IpsecVPNPolicy.RemoteStartAddress"] ~= nil) then
	        		vpnTable["IpsecVPNPolicy.RemoteStartAddress"] = inputTable["IpsecVPNPolicy.RemoteStartAddress"]
	    		end
	    		if(inputTable["IpsecVPNPolicy.RemoteEndAddress"] ~= nil) then
	        		vpnTable["IpsecVPNPolicy.RemoteEndAddress"] = inputTable["IpsecVPNPolicy.RemoteEndAddress"]
	    		end
	    elseif(vpnTable["IpsecVPNPolicy.RemoteNetworkType"] == "3")then
				if(inputTable["IpsecVPNPolicy.RemoteStartAddress"] ~= nil) then
	        		vpnTable["IpsecVPNPolicy.RemoteStartAddress"] = inputTable["IpsecVPNPolicy.RemoteStartAddress"]
	    		end
	    		if(inputTable["IpsecVPNPolicy.RemoteSubnetMask"] ~= nil) then
	        		vpnTable["IpsecVPNPolicy.RemoteSubnetMask"] = inputTable["IpsecVPNPolicy.RemoteSubnetMask"]
	    		end
	    end
	elseif(inputTable["IpsecVPNPolicy.TunnelOrTransport"] == "1" and vpnTable["IpsecVPNPolicy.EnableDhcpOverIPSec"] == "1") then
		-- for Local IP
		vpnTable["IpsecVPNPolicy.LocalNetworkType"] = DEF_NETWORK	
		
		-- for remote Ip
		vpnTable["IpsecVPNPolicy.RemoteNetworkType"] = DEF_NETWORK		
	end	
	if(inputTable["IpsecVPNPolicy.LocalGateway"] ~= nil) then
        vpnTable["IpsecVPNPolicy.LocalGateway"] = inputTable["IpsecVPNPolicy.LocalGateway"]
    end
    if(inputTable["IpsecVPNPolicy.RemoteEndPointType"] ~= nil) then
        vpnTable["IpsecVPNPolicy.RemoteEndPointType"] = inputTable["IpsecVPNPolicy.RemoteEndPointType"]
    end
    if(inputTable["IpsecVPNPolicy.RemoteEndPoint"] ~= nil) then
        vpnTable["IpsecVPNPolicy.RemoteEndPoint"] = inputTable["IpsecVPNPolicy.RemoteEndPoint"]
    end    
    return 	vpnTable	
end

--  Collecting ikePolicy values in a table and returning it.

function ipsec.getIkePolicyPara (inputTable)
	local vpnTable = {}
	if(inputTable["IpsecVPNPolicy.VPNPolicyName"] ~= nil) then
	    vpnTable["IpsecIKEPolicy.IKEPolicyName"] = inputTable["IpsecVPNPolicy.VPNPolicyName"]
	end	
    if(inputTable["IpsecIKEPolicy.ExchangeMode"] ~= nil) then
        vpnTable["IpsecIKEPolicy.ExchangeMode"] = inputTable["IpsecIKEPolicy.ExchangeMode"]
    end
    if(inputTable["IpsecIKEPolicy.Direction"] ~= nil) then
        vpnTable["IpsecIKEPolicy.Direction"] = inputTable["IpsecIKEPolicy.Direction"]
    end
    if(inputTable["IpsecIKEPolicy.NatTraversal"] ~= nil) then
        vpnTable["IpsecIKEPolicy.NatTraversal"] = inputTable["IpsecIKEPolicy.NatTraversal"]       
    -- If Nat traversal is on     
	 	if(inputTable["IpsecIKEPolicy.NatTraversal"] == "1") then  
	    	if(inputTable["IpsecIKEPolicy.NatKeepAliveTime"] ~= nil) then
	        	vpnTable["IpsecIKEPolicy.NatKeepAliveTime"] = inputTable["IpsecIKEPolicy.NatKeepAliveTime"]
	    	end
	 	end
	end
	-- For Local identifier
    if(inputTable["IpsecIKEPolicy.LocalIdentifierType"] ~= nil) then
        vpnTable["IpsecIKEPolicy.LocalIdentifierType"] = inputTable["IpsecIKEPolicy.LocalIdentifierType"]
        if(vpnTable["IpsecIKEPolicy.LocalIdentifierType"]== "1" or vpnTable["IpsecIKEPolicy.LocalIdentifierType"]== "2" or vpnTable["IpsecIKEPolicy.LocalIdentifierType"]== "3") then       
	    	vpnTable["IpsecIKEPolicy.LocalIdentifier"] = inputTable["IpsecIKEPolicy.LocalIdentifier"]
    	end
    end
    
    -- For Remote Identifier
    
	if(inputTable["IpsecIKEPolicy.RemoteIdentifierType"] ~= nil) then
        vpnTable["IpsecIKEPolicy.RemoteIdentifierType"] = inputTable["IpsecIKEPolicy.RemoteIdentifierType"]
        if(vpnTable["IpsecIKEPolicy.RemoteIdentifierType"]== "1" or vpnTable["IpsecIKEPolicy.RemoteIdentifierType"]== "2" or vpnTable["IpsecIKEPolicy.RemoteIdentifierType"]== "3") then       
	    	vpnTable["IpsecIKEPolicy.LocalIdentifier"] = inputTable["IpsecIKEPolicy.LocalIdentifier"]
    	end
    end    
    	
    if(inputTable["IpsecIKEPolicy.EncryptionAlgorithm"] ~= nil) then
        vpnTable["IpsecIKEPolicy.EncryptionAlgorithm"] = inputTable["IpsecIKEPolicy.EncryptionAlgorithm"]
    end	
    if(inputTable["IpsecIKEPolicy.AuthAlgorithm"] ~= nil) then
        vpnTable["IpsecIKEPolicy.AuthAlgorithm"] = inputTable["IpsecIKEPolicy.AuthAlgorithm"]
    end	
    if(inputTable["IpsecIKEPolicy.AuthType"] ~= nil) then
        vpnTable["IpsecIKEPolicy.AuthType"] = inputTable["IpsecIKEPolicy.AuthType"]
        if(vpnTable["IpsecIKEPolicy.AuthType"] == "0") then
        	if(inputTable["IpsecIKEPolicy.Presharedkey"] ~= nil) then
        		vpnTable["IpsecIKEPolicy.Presharedkey"] = inputTable["IpsecIKEPolicy.Presharedkey"]
   			end
        end
    end    
    if(inputTable["IpsecIKEPolicy.DHGroup"] ~= nil) then
        vpnTable["IpsecIKEPolicy.DHGroup"] = inputTable["IpsecIKEPolicy.DHGroup"]
    end
    if(inputTable["IpsecIKEPolicy.SALifeTime"] ~= nil) then
        vpnTable["IpsecIKEPolicy.SALifeTime"] = inputTable["IpsecIKEPolicy.SALifeTime"]
    end
    if(inputTable["IpsecIKEPolicy.DPD"] ~= nil) then
        vpnTable["IpsecIKEPolicy.DPD"] = inputTable["IpsecIKEPolicy.DPD"]
        if(inputTable["IpsecIKEPolicy.DPD"] == "1") then -- If DPD is enabled
        	if(inputTable["IpsecIKEPolicy.DPDDetectionPeriod"] ~= nil) then
        		vpnTable["IpsecIKEPolicy.DPDDetectionPeriod"] = inputTable["IpsecIKEPolicy.DPDDetectionPeriod"]
    		end
    		if(inputTable["IpsecIKEPolicy.DPDFailureCount"] ~= nil) then
        		vpnTable["IpsecIKEPolicy.DPDFailureCount"] = inputTable["IpsecIKEPolicy.DPDFailureCount"]
    		end
        end        
    end    
    if(inputTable["IpsecIKEPolicy.XAUTHType"] ~= nil) then
        vpnTable["IpsecIKEPolicy.XAUTHType"] = inputTable["IpsecIKEPolicy.XAUTHType"]
        if(inputTable["IpsecIKEPolicy.XAUTHType"] == "2") then
        	vpnTable["IpsecIKEPolicy.XAUTHType1"] = inputTable["IpsecIKEPolicy.XAUTHType1"]
        end
        if(inputTable["IpsecIKEPolicy.XAUTHType"] == "1") then -- If XAUTH type is enabled.
        	if(inputTable["IpsecIKEPolicy.UserName"] ~= nil) then
        		vpnTable["IpsecIKEPolicy.UserName"] = inputTable["IpsecIKEPolicy.UserName"]
    		end
    		if(inputTable["IpsecIKEPolicy.Password"] ~= nil) then
        		vpnTable["IpsecIKEPolicy.Password"] = inputTable["IpsecIKEPolicy.Password"]
    		end    
        end
    end    
    return 	vpnTable	
end

--[[
/******************************************************************************
*
* ipsec.ipsecPolicyConfig () - Used to configure IKE and VPN Policies.
*
*  RETURNS: Returns "OK" on success or "ERROR" on failure
*/
]]--
--  Configure IPSec Policies
function ipsec.ipsecPolicyConfig (inputTableIkeVpn)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction()
    errorFlag = "OK"
    statusCode = "STATUS_OK"

    if(inputTableIkeVpn["IpsecVPNPolicy.PolicyType"] == "1") then   -- Auto Policy 
        local inputTable = ipsec.getIkePolicyPara (inputTableIkeVpn)
        -- For Ike Policy 
        --automation part begin
        local tableIndex = 1
        validationTable = {
            function() return validations.isEmpty(inputTable["IpsecIKEPolicy.IKEPolicyName"],"IKE Policy Name Error") end,
            function() return validations.checkboxValidate(inputTable["IpsecIKEPolicy.AuthType"],"0",1) end,
            function() return validations.preShrKeyChk(inputTable["IpsecIKEPolicy.Presharedkey"],"Pre Share Key Error") end,
            function() return validations.checkboxValidate(inputTable["IpsecIKEPolicy.DPD"],"1",2) end,
            function() return validations.detPeriod(inputTable["IpsecIKEPolicy.DPDDetectionPeriod"],"Detection Period Error") end,
            function() return validations.reConCount(inputTable["IpsecIKEPolicy.DPDFailureCount"],"Re-Con Count Error") end
                }

        while (tableIndex <= #(validationTable)) do
            validationFunc = validationTable[tableIndex]
            errorFlag,statusCode,field,skipValidations = validationFunc()
            if(errorFlag==1 or errorFlag == "ERROR") then
                statusCode = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or statusCode
                statusCode = field .. ": " .. statusMessage
                break
            end

            if(skipValidations ~= "") then
                tableIndex = tableIndex + skipValidations + 1
            else
                tableIndex = tableIndex + 1
            end
        end

        DBTable = "IpsecIKEPolicy"
        if(errorFlag == "OK" or errorFlag == 0) then	   
            if (RowId == "-1") then			 
                errorFlag, statusCode = vipSecure.ikeConfig(inputTable, "-1", "add")
            else
                local ikeRow = db.getRow("IpsecVPNPolicy","_ROWID_",RowId)
                if(ikeRow["IpsecVPNPolicy.PolicyType"] == "1") then
                	local ikeRowId = db.getAttribute("IpsecIKEPolicy","IKEPolicyName",ikeRow["IpsecVPNPolicy.IKEPolicyName"],"_ROWID_")			    					    	
                    errorFlag, statusCode = vipSecure.ikeConfig(inputTable,ikeRowId, "edit")
                else
                    errorFlag, statusCode = vipSecure.ikeConfig(inputTable, "-1", "add")
                end	        			        
            end
        end		    
        --end for Ike Policy 
		    
        if(errorFlag == "OK") then
            local inputTable = ipsec.getVpnPolicyPara (inputTableIkeVpn)
            inputTable["IpsecIKEPolicy.IKEPolicyName"] = inputTable["IpsecVPNPolicy.VPNPolicyName"]

            if(inputTable["IpsecVPNPolicy.Netbios"] == "1") then
                inputTable["IpsecVPNPolicy.RemoteNetworkType"] = "3"
            end

            if(inputTable["IpsecVPNPolicy.TunnelOrTransport"] == "0") then
                inputTable["IpsecVPNPolicy.RemoteNetworkType"] = "0"
                inputTable["IpsecVPNPolicy.LocalNetworkType"] = "0"
            end

            if(inputTable["IpsecVPNPolicy.PFSKeyGroup"] == "1") then
                inputTable["IpsecVPNPolicy.PFSKeyGroup"] = inputTable["IpsecVPNPolicy.PFSKeyGroup1"]
            end

            if(inputTable["IpsecVPNPolicy.KeyLength"] == nil) then
                inputTable["IpsecVPNPolicy.KeyLength"] =0
            end	

            --automation part begin
            DBTable = "IpsecVPNPolicy"
            tableIndex = 1
            validationTable = {
					function() return validations.checkboxValidate(inputTable["IpsecVPNPolicy.RemoteEndPointType"],"0",1) end,
					function() return validations.ipAddressValidate(inputTable["IpsecVPNPolicy.RemoteEndPoint"],"","Remote IP error") end,
					function() return validations.checkboxValidate(inputTable["IpsecVPNPolicy.LocalNetworkType"],"1",1) end,
					function() return validations.ipAddressValidate(inputTable["IpsecVPNPolicy.LocalStartAddress"],"","Local IP error")end,
					function() return validations.checkboxValidate(inputTable["IpsecVPNPolicy.LocalNetworkType"],"2",1) end,
					function() return validations.ipRangeValidate(inputTable["IpsecVPNPolicy.LocalStartAddress"],inputTable["IpsecVPNPolicy.LocalEndAddress"],"Local IP start/end error") end,
					function() return validations.checkboxValidate(inputTable["IpsecVPNPolicy.RemoteNetworkType"],"1",1) end,
					function() return validations.ipAddressValidate(inputTable["IpsecVPNPolicy.RemoteStartAddress"],"","Remote IP error") end,
					function() return validations.checkboxValidate(inputTable["IpsecVPNPolicy.RemoteNetworkType"],"2",1) end,
					function() return validations.ipRangeValidate(inputTable["IpsecVPNPolicy.RemoteStartAddress"],inputTable["IpsecVPNPolicy.RemoteEndAddress"],"Remote start/stop error") end,
					function() return validations.checkboxValidate(inputTable["IpsecVPNPolicy.SALifeTimeType"],"0",1) end,
					function() return validations.vpnSaLifeTime(300,inputTable["IpsecVPNPolicy.SALifeTime"],"SA lifetime error") end,
					function() return validations.checkboxValidate(inputTable["IpsecVPNPolicy.SALifeTimeType"],"1",1) end,
					function() return validations.vpnSaLifeTime(1920000,inputTable["IpsecVPNPolicy.SALifeTime"],"SA lifetime error") end
                        }

            while (tableIndex <= #(validationTable)) do
                validationFunc = validationTable[tableIndex]
                errorFlag,statusCode,field,skipValidations = validationFunc()
                if(errorFlag==1 or errorFlag == "ERROR") then
                    statusCode = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or statusCode
                    statusCode = field .. ": " .. statusMessage
                    break
                end

                if(skipValidations ~= "") then
                    tableIndex = tableIndex + skipValidations + 1
                else
                    tableIndex = tableIndex + 1
                end
            end
			
            if (errorFlag == "OK" or errorFlag == 0) then
                --if add
                if (RowId == "-1") then
                    errorFlag, statusCode = vipSecure.ipsecVPNPolicyConfig(inputTable, RowId, "add")
                --if edit
                else				        		        		
                    errorFlag, statusCode = vipSecure.ipsecVPNPolicyConfig(inputTable, RowId, "edit")
                end
            end
        end
    else --For Manual Policy
        local inputTable = ipsec.getVpnPolicyPara (inputTableIkeVpn)
        
        if(inputTable["IpsecVPNPolicy.Netbios"] == "1") then
            inputTable["IpsecVPNPolicy.RemoteNetworkType"] = "3"
        end

        if(inputTable["IpsecVPNPolicy.TunnelOrTransport"] == "0") then
            inputTable["IpsecVPNPolicy.RemoteNetworkType"] = "0"
            inputTable["IpsecVPNPolicy.LocalNetworkType"] = "0"
        end

        if(inputTable["IpsecVPNPolicy.PFSKeyGroup"] == "1") then
            inputTable["IpsecVPNPolicy.PFSKeyGroup"] = inputTable["IpsecVPNPolicy.PFSKeyGroup1"]
        end

        if(inputTable["IpsecVPNPolicy.KeyLength"] == nil) then
            inputTable["IpsecVPNPolicy.KeyLength"] = 0
        end	
		
        --automation part begin
        DBTable = "IpsecVPNPolicy"
        tableIndex = 1
        errorFlag = "OK"

        validationTable = {
					function() return validations.checkboxValidate(inputTable["IpsecVPNPolicy.RemoteEndPointType"],"0",1) end,
					function() return validations.ipAddressValidate(inputTable["IpsecVPNPolicy.RemoteEndPoint"],"","Remote IP error") end,
					function() return validations.checkboxValidate(inputTable["IpsecVPNPolicy.LocalNetworkType"],"1",1) end,
					function() return validations.ipAddressValidate(inputTable["IpsecVPNPolicy.LocalStartAddress"],"","Local IP error")end,
					function() return validations.checkboxValidate(inputTable["IpsecVPNPolicy.LocalNetworkType"],"2",1) end,
					function() return validations.ipRangeValidate(inputTable["IpsecVPNPolicy.LocalStartAddress"],inputTable["IpsecVPNPolicy.LocalEndAddress"],"Local IP start/end error") end,
					function() return validations.checkboxValidate(inputTable["IpsecVPNPolicy.RemoteNetworkType"],"1",1) end,
					function() return validations.ipAddressValidate(inputTable["IpsecVPNPolicy.RemoteStartAddress"],"","Remote IP error") end,
					function() return validations.checkboxValidate(inputTable["IpsecVPNPolicy.RemoteNetworkType"],"2",1) end,
					function() return validations.ipRangeValidate(inputTable["IpsecVPNPolicy.RemoteStartAddress"],inputTable["IpsecVPNPolicy.RemoteEndAddress"],"Remote start/stop error") end,
					function() return validations.checkboxValidate(inputTable["IpsecVPNPolicy.SALifeTimeType"],"0",1) end,
					function() return validations.vpnSaLifeTime(300,inputTable["IpsecVPNPolicy.SALifeTime"],"SA lifetime error") end,
					function() return validations.checkboxValidate(inputTable["IpsecVPNPolicy.SALifeTimeType"],"1",1) end,
					function() return validations.vpnSaLifeTime(1920000,inputTable["IpsecVPNPolicy.SALifeTime"],"SA lifetime error") end
                        }

        while (tableIndex <= #(validationTable)) do
            validationFunc = validationTable[tableIndex]
            errorFlag,statusCode,field,skipValidations = validationFunc()
            if(errorFlag==1 or errorFlag == "ERROR") then
                statusCode = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or statusCode
                statusCode = field .. ": " .. statusMessage
                break
            end

            if(skipValidations ~= "") then
                tableIndex = tableIndex + skipValidations + 1
            else
                tableIndex = tableIndex + 1
            end
        end

        if (errorFlag == "OK" or errorFlag == 0) then
            --if add
            if (RowId == "-1") then
                errorFlag, statusCode = vipSecure.ipsecVPNPolicyConfig(inputTable, RowId, "add")
            --if edit
            else	
                local ikeName = db.getAttribute("IpsecVPNPolicy","_ROWID_",RowId,"IKEPolicyName")
                errorFlag, statusCode = vipSecure.ipsecVPNPolicyConfig(inputTable, RowId, "edit")
                local ikeRow = db.getRow("IpsecIKEPolicy","IKEPolicyName",ikeName)
                if(ikeRow ~= nil) then
                    local table = {}
                    table["IpsecIKEPolicy._ROWID_"] = ikeRow["IpsecIKEPolicy._ROWID_"] 
                    local valid = db.delete ("IpsecIKEPolicy",table)
                    if (valid) then
                        errorFlag, statusCode = "OK", "STATUS_OK"
                    else
                        errorFlag, statusCode = "ERROR", "IKE_DELETE_FAILED"
                    end
                end
            end				        
        end		     
	end

    -- return
    if (errorFlag == "OK") then 
        db.commitTransaction(true)
        return errorFlag, statusCode
    else
        db.rollback()
        return errorFlag, statusCode
    end
end
