--[[
#### Hussain Vali.
#### TeamF1
#### www.TeamF1.com
#### Dec 19, 2007

#### File: iproute.lua
#### Description: routing functions

#### Revisions:
01b 18oct08, nrj Fixed SPR 8705
01a 19Jun08, sdv Added ipv6 routing configuration functions
]]--


--************* Requires *************

--************* Initial Code *************

--package iproute
iproute = {}

--************* Functions *************

-- routing config
function iproute.config (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "Administrator Privileges Are Required"
	end

	db.beginTransaction() --begin transaction
	local valid = false
    local statusMsg = ""
	
	-- config Routes
	if (iproute.inputvalidate(inputTable)) then
	    if (operation == "add") then
            local table = db.getRowsWhere ("route", "routeName = '" .. inputTable["route.routeName"] .. "'")
            if (#table == 0) then
                table = db.getTable ("route")
                local count = 0
                local rowids = {}
                for k,v in pairs (table) do
                    if (v["route.dstIpAddr"] == inputTable["route.dstIpAddr"] and
                        v["route.metric"] == inputTable["route.metric"]) then
                        count = count + 1
                        rowids[count] = v["route._ROWID_"]
                    end
                end
                if (#rowids == 0) then
                    valid = db.insert("route", inputTable)
    	    	    if (valid) then
	    	    	    statusMsg = "Route Added Successfully"
    	    	    else
	    	    	    statusMsg = "Route Adding Failed"
    		        end
                else
                    statusMsg = "Route Already Exists"
                    valid = false
                end   
            else
                statusMsg = "Route Name Already Exists"
                valid = false
            end
    	elseif (operation == "edit") then
            local table = db.getTable ("route")
            local count = 0
            local rowids = {}
            for k,v in pairs (table) do
                if (v["route.dstIpAddr"] == inputTable["route.dstIpAddr"] and
                    v["route.metric"] == inputTable["route.metric"]) then
                    count = count + 1
                    rowids[count] = v["route._ROWID_"]
                end
            end
            if ((#rowids == 0) or (#rowids == 1 and rowids[1] == rowid)) then
                valid = db.update("route", inputTable, rowid)
    	    	if (valid) then
	    	    	statusMsg = "Route Edited Successfully"
    	    	else
	    	    	statusMsg = "Route Edit Failed"
    		    end
            else
                statusMsg = "Route Already Exists"
                valid = false
            end
	    end
	end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", statusMsg
	else
		db.rollback()
		return "ERROR", statusMsg
	end
end

--Route(s) Delete
function iproute.deleteRoute (rowIds)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "Administrator Privileges Are Required"
	end

	db.beginTransaction() --begin transaction
	local valid = false

	-- delete routes
	for k,v in pairs(rowIds) do
        local inTable = {}
        inTable["route._ROWID_"] = v;
	    valid = db.delete("route", inTable)
	    if (not valid) then
	    	break
	    end
	end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "Route(s) Deleted Successfully"
	else
		db.rollback()
		return "ERROR", "Route Delete Failed"
	end
end

-- route inputvalidate
function iproute.inputvalidate (inputTable)
	local valid = db.typeAndRangeValidate(inputTable)
	if (valid) then
		return true
	else
		return false
	end
end

-- Ipv6 routing config
function iproute.ipv6Config (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "Administrator Privileges Are Required"
    end

    db.beginTransaction() --begin transaction
    local valid = false
    local statusMsg = ""
	
    -- config Routes
    if (iproute.inputvalidate(inputTable)) then
        if (operation == "add") then
            local table = db.getRowsWhere ("route6", "routeName = '" .. inputTable["route6.routeName"] .. "'")
            if (#table == 0) then
                local table = db.getTable ("route6")
                local count = 0
                local rowids = {}
                for k,v in pairs (table) do
                    if (v["route6.dstIpAddr"] == inputTable["route6.dstIpAddr"] and
                        v["route6.metric"] == inputTable["route6.metric"]) then
                        count = count + 1
                        rowids[count] = v["route6._ROWID_"]
                    end
                end
                if (#rowids == 0) then
                    valid = db.insert("route6", inputTable)
	                if (valid) then
            		    statusMsg = "Ipv6 Route Added Successfully"
        	        else
		                statusMsg = "Ipv6 Route Adding Failed"
                    end
                else
                     statusMsg = "Route Already Exists"
                     valid = false
                end
            else
                statusMsg = "Route Name Already Exists"
                valid = false
            end
    	elseif (operation == "edit") then
            local table = db.getTable ("route6")
            local count = 0
            local rowids = {}
            for k,v in pairs (table) do
                if (v["route6.dstIpAddr"] == inputTable["route6.dstIpAddr"] and
                    v["route6.metric"] == inputTable["route6.metric"]) then
                    count = count + 1
                    rowids[count] = v["route6._ROWID_"]
                end
            end
            if ((#rowids == 0) or (#rowids == 1 and rowids[1] == rowid)) then
        	    valid = db.update("route6", inputTable, rowid)
	            if (valid) then
            		statusMsg = "Ipv6 Route Edited Successfully"
                else
            		statusMsg = "Ipv6 Route Edit Failed"
        	    end
            else
                statusMsg = "Route Already Exists"
                valid = false
            end
    	end
    end

    -- return
    if (valid) then
	    db.commitTransaction(true)
    	return "OK", statusMsg
    else
	    db.rollback()
    	return "ERROR", statusMsg
    end
end

-- Ipv6 Route(s) Delete
function iproute.deleteIpv6Route (rowIds)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
	    return "ACCESS_DENIED", "Administrator Privileges Are Required"
    end

    db.beginTransaction() --begin transaction
    local valid = false

    -- delete routes
    for k,v in pairs(rowIds) do
        local inTable = {}
        inTable["route6._ROWID_"] = v;
    	valid = db.delete("route6", inTable)
	    if (not valid) then
	        break
    	end
    end

    -- return
    if (valid) then
	    db.commitTransaction(true)
    	return "OK", "Ipv6 Route(s) Deleted Successfully"
    else
	    db.rollback()
    	return "ERROR", "Ipv6 Route Delete Failed"
    end
end
