--[[
#### Hussain Vali
#### TeamF1
#### www.TeamF1.com
#### Mar 28, 2008

#### File: lan.lua
#### Description: lan functions

#### Revisions:
None
]]--


--************* Requires *************
require "teamf1lualib/dhcp"

--************* Initial Code *************

lan = {}

--************* Functions *************

-- lan config
function lan.config (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
            return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false

	-- validate
	if (lan.inputvalidate(inputTable, operation)) then
		if (operation == "add") then
			return nil
		elseif (operation == "edit") then
			valid = db.update("Lan", inputTable, rowid)
		elseif (operation == "delete") then
			return nil
		end
	end

    -- configure DHCP Server,
    -- if lan ipaddress not the present dhcp address subnet
    if (inputTable["dhcp.set"] ~= nil) then
        if (inputTable["dhcp.set"] == "1") then
            local inTableDhcp = {}
            inTableDhcp["dhcp.startAddress"] = inputTable["dhcp.startAddress"]
            inTableDhcp["dhcp.endAddress"] = inputTable["dhcp.endAddress"]
            if (valid) then
                DBTable = "dhcp"
                errorFlag, statusCode = dhcp.config(inTableDhcp, "1","edit")
                if (errorFlag == "OK") then
                    valid = true
                else
                    db.rollback()
                    return "ERROR", statusCode
                end
            end
        end
    end

    -- return
    if (valid) then
        db.commitTransaction()
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "LAN_CONFIG_FAILED"
    end
end

-- lan inputvalidate
function lan.inputvalidate (inputTable, operation)
	if (true) then
		return db.typeAndRangeValidate(inputTable)
	end
	return false
end

-- IPV4 Lan Configuration
function lan.ipv4LanConfig(inputTable, rowid, operation)
     -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
            return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false
    
    -- validate & ifStatic Config
    --if (lan.inputvalidate(inputTable, operation)) then
    if (true) then

	if (operation == "add") then
            return nil
	elseif (operation == "edit") then
	    valid = db.update("ifStatic", inputTable, rowid)
	elseif (operation == "delete") then
	    return nil
	end
    end

    -- return
    if (valid) then
        db.commitTransaction()
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "IPV4_LAN_CONFIG_FAILED"
    end
end




-- IPV6 Lan Configuration
function lan.ipv6LanConfig(inputTable, rowid, operation)
     -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
            return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false
    
    -- validate & ifStatic Config
    if (lan.inputvalidate(inputTable, operation)) then
	if (operation == "add") then
            return nil
	elseif (operation == "edit") then
	    valid = db.update("ifStatic", inputTable, rowid)
	elseif (operation == "delete") then
	    return nil
	end
    end

    -- Dhcpv6s Configuration
    if(valid) then
        local isExists = db.existsRow ("dhcpv6s", "_ROWID_", "1")
        if (isExists) then
            valid = db.update("dhcpv6s", inputTable, "1")
        else
            valid = db.insert("dhcpv6s", inputTable)        
        end
    end

    -- return
    if (valid) then
        db.commitTransaction()
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "IPV6_LAN_CONFIG_FAILED"
    end
end

-- IPV6 Lan Address Pool Deletion
function lan.lanPoolsDelete (rowIds)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
	return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false

    -- delete routes
    for k,v in pairs(rowIds) do
        local inTable = {}
        inTable["dhcpv6sLANAddrPool._ROWID_"] = v
	valid = db.delete("dhcpv6sLANAddrPool", inTable) 
	if(valid==false) then
	    break
	end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "IPV6_LAN_POOL_DELETE_SUCCESS"
    else
        db.rollback()
        return "ERROR", "IPV6_LAN_POOL_DELETE_FAILED"
    end
end

-- IPV6 Lan Pool Configuration
function lan.poolConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
    	return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false
   

    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            valid = db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            valid = db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
	    valid = db.delete(DBTable, inputTable)
        end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "IPV6_LAN_POOL_CONFIG_FAILED"
    end
end
