-- logConfigCLI.lua
-- Samer Sayeed
-- TeamF1
-- www.TeamF1.com

-- Modification History
-- 10feb09,ss  moved all ipv6 and firewall logs set and get routines from 
--             loggingCLI.lua.
-- 15jan09,ss  added ipv6 log routines.
log = {}

function logCfgSave(configRow)
    DBTable = "FirewallLogs"
    errorFlag, statusCode = logConfigure(configRow, "1", "edit")
    -- save db if no error
    if (errorFlag == "OK") then
        DBTable = "OtherSystemLogs"
        errorFlag, statusCode = logConfigure(configRow, "1", "edit")
        if (errorFlag == "OK") then db.save() end
    end
    statusMessage = db.getAttribute("stringsMap","stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage
end

function logCfgInit(args)
    local configRow = {}
    local firewallGet = db.getRow("FirewallLogs","_ROWID_", "1")
    configRow = util.tableAppend (configRow, firewallGet)
    local otherSysGet = db.getRow("OtherSystemLogs","_ROWID_", "1")
    configRow = util.tableAppend (configRow, otherSysGet)
   
    if (configRow == nil) then
        configRow = {}
    end
    return 1, configRow
end

function logCfgVal(configRow)
    return true
end

function sysEventLogsGet(args)
    printLabel("Logging Config")
    printLabel("Routing Logs")
    local firewallLogsRow = db.getRow("FirewallLogs", "_ROWID_", "1")
    print("LAN to WAN")
    if(firewallLogsRow["FirewallLogs.LANToWANAccept"] == "1")then 		 
        print("Accepted Packets: Enabled") 		 
    else
        print("Accepted Packets: Disabled") 		 
    end
    if(firewallLogsRow["FirewallLogs.LANToWANDrop"] == "1")then 		 
        print("Dropped Packets: Enabled") 		 
    else
        print("Dropped Packets: Disabled") 		 
    end
    print("WAN to LAN") 	  	 
    if(firewallLogsRow["FirewallLogs.WANToLANAccept"] == "1")then 		 
        print("Accepted Packets: Enabled") 		 
    else
        print("Accepted Packets: Disabled") 		 
    end
    if(firewallLogsRow["FirewallLogs.WANToLANDrop"] == "1")then 		 
        print("Dropped Packets: Enabled") 		 
    else
        print("Dropped Packets: Disabled") 		 
    end
    print("System Logs")
    if(firewallLogsRow["FirewallLogs.UnicastTraffic"] == "1")then 		 
        print("All Unicast Traffic: Enabled") 		 
    else
        print("All Unicast Traffic: Disabled") 		 
    end
    if(firewallLogsRow["FirewallLogs.BroadCastORMulticastTraffic"] == "1")then 		 
        print("All Broadcast/Multicast Traffic: Enabled") 		 
    else
        print("All Broadcast/Multicast Traffic: Disabled") 		 
    end
    print("Other Event Logs")
    if(firewallLogsRow["FirewallLogs.SourceMACFilter"] == "1")then 		 
        print("Source MAC Filter: Enabled") 		 
    else
        print("Source MAC Filter: Disabled") 		 
    end
    if(firewallLogsRow["FirewallLogs.BandwidthLimitLogs"] == "1")then 		 
        print("Bandwidth Limit: Enabled") 	
    else
        print("Bandwidth Limit: Disabled") 	
    end
end

function ipv6LogsCfgSave (configRow)
    DBTable = "FirewallLogs6"
    errorFlag, statusCode = logConfigure(configRow, "1", "edit")

    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap","stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage
end

function ipv6LogsCfgInit (args)
    configRow = db.getRow ("FirewallLogs6", "_ROWID_", "1")
    if (configRow == nil) then
        configRow = db.getDefaults(true, "FirewallLogs6")
    end    
    return 1, configRow
end

function ipv6LogsCfgInputVal (args)
    local ipMode = 
    db.getAttribute ("networkInfo", "_ROWID_", "1", "netWorkMode") or ''
    if (ipMode == "1") then
        printCLIError ("Please Set IP Mode to IPv4/IPv6 to configure IPv6 " .. 
        "Logging.\n")
        return false
    end
    return true
end

function ipv6LogsCfgGet (args)
    local ipv6LogRow = db.getRow ("FirewallLogs6", "_ROWID_", "1")
    printLabel ("IPV6 Logging")
    if (ipv6LogsCfgInputVal()) then
        printLabel ("Routing Logs")
        print ("LAN To WAN:\n")
        if (ipv6LogRow["FirewallLogs6.LANToWANAccept"] == "1") then
            print ("Accepted Packets:  Enabled\t")
        else
            print ("Accepted Packets:  Disabled\t")
        end
        if (ipv6LogRow["FirewallLogs6.LANToWANDrop"] == "1") then
            print ("Dropped Packets:  Enabled\t")
        else
            print ("Dropped Packets:  Disabled\t")
        end
        print ("\nWAN To LAN:\n")
        if (ipv6LogRow["FirewallLogs6.WANToLANAccept"] == "1") then
            print ("Accepted Packets:  Enabled\t")
        else
            print ("Accepted Packets:  Disabled\t")
        end
        if (ipv6LogRow["FirewallLogs6.WANToLANDrop"] == "1") then
            print ("Dropped Packets:  Enabled\t")
        else
            print ("Dropped Packets:  Disabled\t")
        end
    end
end

-- log config
function log.config (inputTable, rowid, operation)
    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            return false
        elseif (operation == "edit") then
            return db.update(DBTable,inputTable,rowid)
        elseif (operation == "delete") then
            return false
        end
    else
        return false
    end
end

function logConfigure (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
    	return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
	local valid = false

    valid = log.config(inputTable, rowid, operation)

    -- return
    if (valid) then
        db.commitTransaction()
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "LOGGING_CONFIG_FAILED"
    end
end

