--[[
#### Adarsh Uppula
#### TeamF1
#### www.TeamF1.com
#### June 29, 2007

#### File: logging.lua
#### Description: logging functions

#### Revisions:
None
]]--


--************* Requires *************

require "teamf1lualib/crontab"
--************* Initial Code *************

--package logging
logging = {}

--table packages
logConfig = {}
sysLogInfo = {}
emailLogs = {}
logEventConfig = {}
--************* Functions *************

-- logging config
function logging.logConfig_config (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false

    -- config logConfig
    valid = logConfig.config(inputTable, rowid, operation)

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "LOGGING_CONFIG_FAILED"
    end
end

-- logConfig config
function logConfig.config (inputTable, rowid, operation)
    -- validate
    if (logConfig.inputvalidate(inputTable, operation)) then
        if (operation == "add") then
            return db.insert("logConfig", inputTable)
        elseif (operation == "edit") then
            return db.update("logConfig", inputTable, rowid)
        elseif (operation == "delete") then
            return db.delete("logConfig", inputTable)
        end
    end
    return false
end

-- logConfig inputvalidate
function logConfig.inputvalidate (inputTable, operation)
    if (true) then
        return db.typeAndRangeValidate(inputTable)
    end
    return false
end

--emailLogs config

function logging.emailLogs_config(inputTable)

    local entryName = db.getAttribute ("emailLogs","_ROWID_",1,"entryName")
    emailLogsBinary = db.getAttribute ("environment","name","EMAILLOGS_PROGRAM","value") .. " " .. DB_FILE_NAME

    if(inputTable["emailLogs.mailLogs"] == "0") then
        valid = db.update ("emailLogs", inputTable, 1)
        if (valid == false) then 
            return "ERROR" ,"LOGGING_EMAILLOGS_CONFIG_FAILED"
        end
        
        valid = db.deleteRow("crontab","command",emailLogsBinary)
        if (valid == false) then 
            return "ERROR" ,"CRONTAB_CONFIG_FAILED"
        end

    else

        valid = db.update("emailLogs",inputTable,1)
        if(valid == false) then
            return "ERROR","LOGGING_EMAILLOGS_CONFIG_FAILED"
        end

        inputTable["smtpServer.logFile"] = db.getAttribute("environment","name","LOG_FILE","value")
        if (db.existsRow("smtpServer","entryName",entryName)) then
            local RowId = db.getAttribute ("smtpServer","entryName",entryName,"_ROWID_")
            valid = db.update ("smtpServer",inputTable, RowId)
        else
            inputTable["smtpServer.entryName"]=entryName
            valid = db.insert("smtpServer",inputTable)
        end
        if (valid == false) then
            return "ERROR","LOGGING_SMTP_CONFIG_FAILED"
        end
       
        local errorString = "OK"
        
        if (inputTable["crontab.unit"] == "0") then
            valid = db.deleteRow("crontab","command",emailLogsBinary)
        else
            valid = db.deleteRow("crontab","command",emailLogsBinary)
            errorString = crontab.config(inputTable,-1,"insert",emailLogsBinary)
        end

        if (errorString ~= "OK" or valid == false) then
                return "ERROR","LOGGING_CRONTAB_CONFIG_FAILED"
        end
    end

    -- Set hostname
    --if(inputTable["emailLogs.logId"] ~= nil) then
      --  os.execute("/bin/hostname "..inputTable["emailLogs.logId"])
    --end

    return "OK","STATUS_OK"
end

--emailLogs getn info() for the logs_config_emails.htm page

function logging.emailLogs_getInfo()
    configRow = {}
    local entryName=db.getAttribute("emailLogs","_ROWID_",1,"entryName")
    
    if (not db.existsRow("smtpServer","entryName",entryName)) then
        configRow['smtpServer.emailServer'] = ''
        configRow['smtpServer.fromAddr'] = ''
        configRow['smtpServer.toAddr'] = ''
        configRow['smtpServer.userName'] = ''
        configRow['smtpServer.passWord'] = ''
        configRow["crontab.unit"] = "0"
        
    else
        
        local currentInfo = db.getRow("smtpServer","entryName",entryName)
        for k,v in pairs(currentInfo) do
            configRow[k] = v
        end
        if (tonumber(configRow['smtpServer.auth']) == -1) then
            configRow['smtpServer.userName'] = ''
            configRow['smtpServer.passWord'] = ''
        end
    end
    
    emailLogsBinary = db.getAttribute("environment","name","EMAILLOGS_PROGRAM","value") .. " " .. DB_FILE_NAME
    configRow["emailLogs.mailLogs"] = db.getAttribute("emailLogs","_ROWID_",1,"mailLogs")
    configRow["emailLogs.logId"] = db.getAttribute("emailLogs","_ROWID_",1,"logId")
    
    if(not db.existsRow("crontab","command",emailLogsBinary)) then
        configRow["crontab.unit"] = "0"
    else
        local RowId = db.getAttribute("crontab", "command", emailLogsBinary,"_ROWID_")
        configRow=crontab.getSchedule(RowId,configRow)
    end
    return configRow
end

-- sysLogInfo_config(), for updating the sysLogInfo table.

function logging.sysLogInfo_config(inputTable, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    
    db.beginTransaction()
    local valid = false
    
    if (operation == "add") then
        valid =  db.insert("sysLogInfo", inputTable)
    elseif (operation == "edit") then
        valid = db.update("sysLogInfo", inputTable, 1)
    end
    
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "LOGGING_SYSLOG_CONFIG_FAILED"
    end
end

--logging events config
function logging.logEventsConfig_config(inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false

    -- config logEventEmailConfig
    valid = logEventConfig.config(inputTable, rowid, operation)

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "LOGGING_CONFIG_FAILED"
    end
end

-- logEventConfig config
function logEventConfig.config (inputTable, rowid, operation)
    -- validate
    if (logConfig.inputvalidate(inputTable, operation)) then
        if (operation == "add") then
            return db.insert("logEventEmailConfig", inputTable)
        elseif (operation == "edit") then
            return db.update("logEventEmailConfig", inputTable, rowid)
        elseif (operation == "delete") then
            return db.delete("logEventEmailConfig", inputTable)
        end
    end
    return false
end

-- email Log Config
function logging.emailLogsConfig(inputTable)
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
	
    local valid = false
    db.beginTransaction()

    errorFlag, statusCode = logging.emailLogs_config(inputTable)
    if(errorFlag == "OK") then
	local syslogInfo = db.getTable ("sysLogInfo")
        for k,v in pairs (syslogInfo) do
            if(inputTable["sysLogInfo.Enable".. v["sysLogInfo.serverId"]] ~= nil) then
                v["sysLogInfo.Enable"] = inputTable["sysLogInfo.Enable" .. v["sysLogInfo.serverId"]]
		if(v["sysLogInfo.Enable"] == "1") then
	                v["sysLogInfo.serverName"] = inputTable["sysLogInfo.serverName" .. v["sysLogInfo.serverId"]]
        	        v["sysLogInfo.facilityId"] = inputTable["sysLogInfo.facilityId" .. v["sysLogInfo.serverId"]]
                	v["sysLogInfo.severity"] = inputTable["sysLogInfo.severity" .. v["sysLogInfo.serverId"]]
		end
                valid = db.update("sysLogInfo", v, v["sysLogInfo._ROWID_"])
                if (valid == false) then
                    break
                end
            end
	end	
    end
	
    if (valid and errorFlag == "OK") then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "LOGGING_SYSLOG_CONFIG_FAILED"
    end
end

function logging.emailLogsConfigGet()
    local outTable = logging.emailLogs_getInfo()
    if (outTable == nil) then
        outTable = {}
    end

    local syslogInfo = db.getTable ("sysLogInfo")
    for k,v in pairs(syslogInfo) do
        outTable["sysLogInfo.Enable" .. v["sysLogInfo.serverId"]] = v["sysLogInfo.Enable"]
        outTable["sysLogInfo.serverName" .. v["sysLogInfo.serverId"]] = v["sysLogInfo.serverName"]
        outTable["sysLogInfo.facilityId" .. v["sysLogInfo.serverId"]] = v["sysLogInfo.facilityId"]
        outTable["sysLogInfo.severity" .. v["sysLogInfo.serverId"]] = v["sysLogInfo.severity"]
	end
    return outTable
end
