-- nimf6CLI.lua
-- Samer Sayeed 
-- TeamF1
-- www.TeamF1.com
--
-- Modification History
-- 04mar10,db  added Dual Wan Support
-- 31jan09,ss  added nimf6CfgInputVal, ipModeCfgInputVal routine
-- 23jan09,ss  written
--
-- Description
-- CLISH nimf LUA routines

require "teamf1lualib/nimfView"

function wanV6SetupGet(wanType)
    local resultTab = {}
    local where = "AddressFamily = 10 and LogicalIfName = '"..wanType.."'"
    local nimfTabel = db.getRowWhere ("NimfConf", where)
    if (ipModeCfgInputVal()) then
    if (nimfTabel ["NimfConf.ConnectionType6"] == "ifStatic6") then
        resTab.insertField (resultTab, "Static IPv6 Configuration","")
        setupRow = db.getRowWhere ("ifStatic", "LogicalIfName='"..wanType.."' and AddressFamily=10")
        resTab.insertField (resultTab, "IPv6 Address", setupRow["ifStatic.StaticIp"]or"")
        resTab.insertField (resultTab, "IPv6 Prefix Length", setupRow["ifStatic.PrefixLength"]or"")
        resTab.insertField (resultTab, "Default IPv6 Gateway", setupRow["ifStatic.Gateway"]or"")
        resTab.insertField (resultTab, "Primary DNS Server", setupRow["ifStatic.PrimaryDns"]or"")
        resTab.insertField (resultTab, "Secondary DNS Server", setupRow["ifStatic.SecondaryDns"]or"")
    end
       
    if (nimfTabel ["NimfConf.ConnectionType6"] == "dhcp6c") then
        resTab.insertField (resultTab, "Dynamic IPv6 (DHCP) Configuration","")
        setupRow = db.getRowWhere ("dhcpv6c", "LogicalIfName='"..wanType.."'")
        if(setupRow["dhcpv6c.statelessMode"] == "1") then
        resTab.insertField (resultTab, "Stateless Address Auto Configuration", "Enabled")
        elseif(setupRow["dhcpv6c.statelessMode"] == "0") then
        resTab.insertField (resultTab, "Stateful Address Auto Configuration", "Enabled")
        end
    end
    printLabel ("IPv6 "..wanType.." Setup")
    resTab.print (resultTab, 0)
    end

end

--
-- get the field values from the dbase table
function wanV6StatusGet(wanType)
    local resultTab = {}   
    local ConnectionType = ""
    local where = "AddressFamily = 10 and LogicalIfName = '"..wanType.."'"
    local nimfTabel = db.getRowWhere ("NimfConf", where)

    if (ipModeCfgInputVal()) then
    if (nimfTabel ["NimfConf.ConnectionType6"] == "ifStatic6") then
        ConnectionType = "Static IPv6"
    elseif (nimfTabel ["NimfConf.ConnectionType6"] == "dhcp6c") then
        ConnectionType = "Dynamic IPv6 (DHCP)"
    end
   
    if (wanType ~= "WAN2") then 
        stat6Row = db.getRowWhere ("NimfStatus", "LogicalIfName = '"..wanType.."' and AddressFamily=10")
        resTab.insertField (resultTab, "IPv6 Connection Type", ConnectionType)
        resTab.insertField (resultTab, "IPv6 Connection State", stat6Row["NimfStatus.Nimfstatus"]or"")
    end

    stat6Row = db.getRowWhere ("ipAddressTable", "LogicalIfName='"..wanType.."' and addressFamily=10")
    if(stat6Row ~= nil) then
    resTab.insertField (resultTab, "IPv6 Address", stat6Row["ipAddressTable.ipAddress"]or"")
    resTab.insertField (resultTab, "IPv6 Prefix Length", stat6Row["ipAddressTable.ipv6PrefixLen"]or"")
    stat6Row = db.getRowWhere ("defaultRouters", "LogicalIfName='WAN1' and addressFamily=10")
    if(stat6Row ~= nil) then
    resTab.insertField (resultTab, "Default IPv6 Gateway", stat6Row["defaultRouters.nextHopGateway"]or"")
    end
    stat6Row = db.getRowWhere ("resolverConfig", "LogicalIfName='"..wanType.."' and addressFamily=10")
    if(stat6Row ~= nil) then
    resTab.insertField (resultTab, "Primary DNS Server", stat6Row["resolverConfig.nameserver1"]or"")
    resTab.insertField (resultTab, "Secondary DNS Server", stat6Row["resolverConfig.nameserver2"]or"")
    end
    end
    printLabel ("IPv6 "..wanType.." Status")
    resTab.print (resultTab, 0)
    end
end

--
-- This routine save WAN configuration
function nimf6CfgSave (configRow)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""
   
    configRow["NimfConf.LogicalIfName"] = "WAN1"
    errorFlag, statusCode = nimfView.ipv6WanConfig(configRow, "1", "edit")
	
	-- save db if no error
    if (errorFlag == "OK") then db.save() end
      statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    
    return errorFlag, statusMessage 
end

-- This routine takes in one argument that is the value of the
-- primary key for the object table.
function nimf6CfgInit (args)
    local configRow = {}
    local rowId = -1 
    configRow = nimfView.ipv6WanConfigGet("WAN1")
    print (configRow ["NimfConf._ROWID_"], configRow["NimfConf.AddressFamily"], 
           configRow["NimfConf.ConnectionType6"]) 
    return configRow["NimfConf._ROWID_"], configRow
end

function ipModeCfgInputVal(configRow)
    local ipmode = db.getRow("networkInfo", "_ROWID_", "1")
    if(ipmode["networkInfo.netWorkMode"] == "1")then
        printCLIError("Please Set IP Mode to IPv4/IPv6 to configure IPv6 WAN Configurations.")
        return false
    end
    return true
end

function nimf6CfgInputVal(configRow)
    local ipmode = db.getRow("networkInfo", "_ROWID_", "1")
    if(ipmode["networkInfo.netWorkMode"] == "1")then
        printCLIError("Please Set IP Mode to IPv4/IPv6 to configure IPv6 WAN Configurations.")
        return false
    end
    if (configRow["NimfConf.ConnectionType6"] == "ifStatic6") then
        if (configRow["ifStatic.StaticIp"] == "" or configRow["ifStatic.StaticIp"] == nil) then
            printCLIError("Enter valid static ipv6 address")
            return false
        end
        if (configRow["ifStatic.PrefixLength"] == "" or configRow["ifStatic.PrefixLength"] == nil) then
            printCLIError("Enter valid prefix length")
            return false
        end
        if (configRow["ifStatic.Gateway"] == "" or configRow["ifStatic.Gateway"] == nil) then
            printCLIError("Enter valid gateway ipv6 address")
            return false
        end
        if (configRow["ifStatic.PrimaryDns"] == "" or configRow["ifStatic.PrimaryDns"] == nil) then
            printCLIError("Enter valid primary ipv6 dns\n")
            return false
        end
        if (configRow["ifStatic.SecondaryDns"] == "" or configRow["ifStatic.SecondaryDns"] == nil) then
            printCLIError("Enter valid secondary ipv6 dns")
            return false
        end
    elseif (configRow["NimfConf.ConnectionType6"] == "dhcp6c") then
        if (configRow["dhcpv6c.statelessMode"] == "" or configRow["dhcpv6c.statelessMode"] == nil) then
            printCLIError("Enter valid Configuration Mode for ipv6 Dhcp")
            return false
        end
    end
    return true
end
--
-- This routine save WAN2 configuration
function nimf6Wan2CfgSave (configRow)
    local errorFlag = "ERROR"
    local statusCode = ""
    local statusMessage = ""
   
    configRow["NimfConf.LogicalIfName"] = "WAN2"
    errorFlag, statusCode = nimfView.ipv6WanConfig(configRow, "1", "edit")
	
	-- save db if no error
    if (errorFlag == "OK") then db.save() end
      statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    
    return errorFlag, statusMessage 
end

-- This routine takes in one argument that is the value of the
-- primary key for the object table.
function nimf6Wan2CfgInit (args)
    local configRow = {}
    local rowId = -1 
    configRow = nimfView.ipv6WanConfigGet("WAN2")
    print (configRow ["NimfConf._ROWID_"], configRow["NimfConf.AddressFamily"], 
           configRow["NimfConf.ConnectionType6"]) 
    return configRow["NimfConf._ROWID_"], configRow
end

function nimf6CfgInputVal(configRow)
    local ipmode = db.getRow("networkInfo", "_ROWID_", "1")
    if(ipmode["networkInfo.netWorkMode"] == "1")then
        printCLIError("Please Set IP Mode to IPv4/IPv6 to configure IPv6 WAN Configurations.")
        return false
    end
    if (configRow["NimfConf.ConnectionType6"] == "ifStatic6") then
        if (configRow["ifStatic.StaticIp"] == "" or configRow["ifStatic.StaticIp"] == nil) then
            printCLIError("Enter valid static ipv6 address")
            return false
        end
        if (configRow["ifStatic.PrefixLength"] == "" or configRow["ifStatic.PrefixLength"] == nil) then
            printCLIError("Enter valid prefix length")
            return false
        end
        if (configRow["ifStatic.Gateway"] == "" or configRow["ifStatic.Gateway"] == nil) then
            printCLIError("Enter valid gateway ipv6 address")
            return false
        end
        if (configRow["ifStatic.PrimaryDns"] == "" or configRow["ifStatic.PrimaryDns"] == nil) then
            printCLIError("Enter valid primary ipv6 dns\n")
            return false
        end
        if (configRow["ifStatic.SecondaryDns"] == "" or configRow["ifStatic.SecondaryDns"] == nil) then
            printCLIError("Enter valid secondary ipv6 dns")
            return false
        end
    elseif (configRow["NimfConf.ConnectionType6"] == "dhcp6c") then
        if (configRow["dhcpv6c.statelessMode"] == "" or configRow["dhcpv6c.statelessMode"] == nil) then
            printCLIError("Enter valid Configuration Mode for ipv6 Dhcp")
            return false
        end
    end
    return true
end
