--[[
#### Hussain Vali.
#### TeamF1
#### www.TeamF1.com
#### Mar 11, 2008
#### File: nimfView.lua
#### Description: NIMF Setup functions
#### Revisions:
01b 09Feb10, ar  Added dummy update on ifStatic table.Fix for SPR#16513.
01a 19Jun08, sdv Added ipv6 configuration functions
]]--

--************* Requires *************

require "teamf1lualib/ifStatic"
require "teamf1lualib/dhcpc"
require "teamf1lualib/pppoe"
require "teamf1lualib/pptp"
require "teamf1lualib/l2tp"
require "teamf1lualib/nat"
require "teamf1lualib/igmp"
require "teamf1lualib/nimf"
require "teamf1lualib/ThreeGInternet"

--************* Initial Code *************

--package NIMF Configuration
nimfView = {}

--************* Global Variables *************
logIfName = "WAN1"
addressFamily = "2"

--************* Functions *************
-- Wan Configuration
function nimfView.config (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

    -- return error if wan Configuration not found
    local NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)

    -- Retrying for configuration in NimfConf.
    local retryIdx = 1
    while (NimfRow == nil and retryIdx <= 3) do
        NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)
        retryIdx = retryIdx + 1
    end

    if (NimfRow == nil) then    	
        return "ERROR", "NIMF_CONFIG_NOTFOUND"
    end

    rowid = NimfRow["NimfConf._ROWID_"]

	db.beginTransaction() --begin transaction
	local valid = false
    local NimfConfTable = {}
    local wanConfTable = {}

	-- config NIMF
    NimfConfTable["NimfConf.LogicalIfName"] = logIfName
    NimfConfTable["NimfConf.RequireLogin"] = inputTable["Wan.IspLoginRequired"]
    if (inputTable["Wan.IspLoginRequired"] == "0") then    -- dhcpc or ifStatic configuration
        if (inputTable["Wan.IsGetIpDyn"] == "1") then -- dhcpc
            NimfConfTable["NimfConf.ConnectionType"] = "dhcpc"
            wanConfTable["Dhcpc.LogicalIfName"] = logIfName
 			wanConfTable["Dhcpc.HostName"] = inputTable["Wan.HostName"]            wanConfTable["Dhcpc.GetDnsFromIsp"] = inputTable["Wan.IsDNSDyn"]
            if (inputTable["Wan.IsDNSDyn"] == "0") then
                wanConfTable["Dhcpc.PrimaryDns"] = util.joinFields(inputTable,"Wan","PriDnsIpAddrbyte", ".",4)
                wanConfTable["Dhcpc.SecondaryDns"] = util.joinFields(inputTable,"Wan","SecDnsIpAddrbyte", ".",4)
            end
            if (db.existsRowWhere("Dhcpc", "LogicalIfName = '".. logIfName .. "'")) then
                valid = dhcpc.config (wanConfTable, db.getAttribute ("Dhcpc","LogicalIfName",logIfName,"_ROWID_"), "edit")
            else
                valid = dhcpc.config (wanConfTable, "-1", "add")
            end
        else -- ifStatic
            NimfConfTable["NimfConf.ConnectionType"] = "ifStatic"
            wanConfTable["ifStatic.LogicalIfName"] = logIfName            
            wanConfTable["ifStatic.StaticIp"] = util.joinFields(inputTable,"Wan","IpAddrbyte", ".",4)
            wanConfTable["ifStatic.NetMask"] = util.joinFields(inputTable,"Wan","SnetMaskbyte", ".",4)
            wanConfTable["ifStatic.Gateway"] = util.joinFields(inputTable,"Wan","GwIpAddrbyte", ".",4)
            wanConfTable["ifStatic.PrimaryDns"] = util.joinFields(inputTable,"Wan","PriDnsIpAddrbyte", ".",4)
            wanConfTable["ifStatic.SecondaryDns"] = util.joinFields(inputTable,"Wan","SecDnsIpAddrbyte", ".",4)
            --- added for IPv6 support
            wanConfTable["ifStatic.AddressFamily"] = "2"
            if (db.existsRowWhere("ifStatic", "LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)) then
                valid = ifStatic.config (wanConfTable, db.getAttribute ("ifStatic","LogicalIfName",logIfName,"_ROWID_"), "edit")
            else
                valid = ifStatic.config (wanConfTable, "-1", "add")
            end            
        end
    
    else -- pppoe or pptp or l2tp configuration
        if (inputTable["Wan.IspType"] == "1") then
            NimfConfTable["NimfConf.ConnectionType"] = "pppoe"
            local table = db.getRowsWhere ("PppoeProfile", "LogicalIfName = '" .. logIfName .. "'")
            
            -- Disabling pppoeprofiles first before applying user configuring
            if (table ~= nil and #table ~= 0) then
            	for k,v in pairs (table) do
            		valid = true
            		if(v["PppoeProfile.Status"] == "1") then
            			v["PppoeProfile.Status"] = 0
            			valid = db.update ("PppoeProfile", v, v["PppoeProfile._ROWID_"])
            			if(not valid)then
            				break
            			end
            		end            		 
            	end
			end

            if (valid and table ~= nil and #table ~= 0) then
                for k,v in pairs (table) do
                    if (inputTable["Wan.PppoeProfileName_" .. v["PppoeProfile._ROWID_"]] ~= nil and
                        inputTable["Wan.PppoeProfileName_" .. v["PppoeProfile._ROWID_"]] == "1") then
                        v["PppoeProfile.Status"] = "1"
                    else
                        v["PppoeProfile.Status"] = "0"
                    end
                    valid = db.update ("PppoeProfile", v, v["PppoeProfile._ROWID_"])
                    if (not valid) then
                        break
                    end
                 end
             end

        elseif (inputTable["Wan.IspType"] == "2") then
            NimfConfTable["NimfConf.ConnectionType"] = "pptp"            
            wanConfTable["Pptp.LogicalIfName"] = logIfName            
            wanConfTable["Pptp.UserName"] = inputTable["Wan.LoginName"]
            if (util.isAllMasked (inputTable["Wan.LoginPassword"]) == false) then
                wanConfTable["Pptp.Password"] = inputTable["Wan.LoginPassword"]
            end
            wanConfTable["Pptp.MppeEncryptSupport"] = inputTable["Wan.MppeEncryptSupport"]
            wanConfTable["Pptp.SplitTunnel"] = inputTable["Wan.SplitTunnel"]
            if (inputTable["Wan.MyIpAddr"] ~= nil) then                
                wanConfTable["Pptp.MyIp"] = inputTable["Wan.MyIpAddr"]
            else
                wanConfTable["Pptp.MyIp"] = util.joinFields(inputTable,"Wan","MyIpAddrbyte", ".",4)            
            end
            if (inputTable["Wan.SvrIpAddr"] ~= nil) then            
                wanConfTable["Pptp.ServerIp"] = inputTable["Wan.SvrIpAddr"]
            else
                wanConfTable["Pptp.ServerIp"] = util.joinFields(inputTable,"Wan","SvrIpAddrbyte", ".",4)
            end
            wanConfTable["Pptp.GetIpFromIsp"] = inputTable["Wan.IsGetIpDyn"]
            wanConfTable["Pptp.GetDnsFromIsp"] = inputTable["Wan.IsDNSDyn"]
            if (inputTable["Wan.IsGetIpDyn"] == "0") then
                wanConfTable["Pptp.StaticIp"] = util.joinFields(inputTable,"Wan","IpAddrbyte", ".",4)           
                wanConfTable["Pptp.NetMask"] = util.joinFields(inputTable,"Wan","SnetMaskbyte", ".",4)
            end
            if (inputTable["Wan.IsDNSDyn"] == "0") then
                wanConfTable["Pptp.PrimaryDns"] = util.joinFields(inputTable,"Wan","PriDnsIpAddrbyte", ".",4)
                wanConfTable["Pptp.SecondaryDns"] = util.joinFields(inputTable,"Wan","SecDnsIpAddrbyte", ".",4)
            end
            wanConfTable["Pptp.IdleTimeOutFlag"] = inputTable["Wan.IdleTimeOutFlag"]
            if (inputTable["Wan.IdleTimeOutFlag"] == "1") then
                wanConfTable["Pptp.IdleTimeOutValue"] = inputTable["Wan.IdleTime"]
            end
            if (db.existsRowWhere("Pptp", "LogicalIfName = '".. logIfName .. "'")) then
                valid = pptp.config (wanConfTable, db.getAttribute ("Pptp","LogicalIfName",logIfName,"_ROWID_"), "edit")
            else
                valid = pptp.config (wanConfTable, "-1", "add")
            end
        elseif (inputTable["Wan.IspType"] == "3") then
            NimfConfTable["NimfConf.ConnectionType"] = "l2tp"            
            wanConfTable["L2tp.LogicalIfName"] = logIfName            
            wanConfTable["L2tp.UserName"] = inputTable["Wan.LoginName"]
            if (util.isAllMasked (inputTable["Wan.LoginPassword"]) == false) then            
                wanConfTable["L2tp.Password"] = inputTable["Wan.LoginPassword"]
            end
            if (inputTable["Wan.AccountName"] ~= nil) then            
                wanConfTable["L2tp.AccountName"] = inputTable["Wan.AccountName"]
            end
            if (inputTable["Wan.DomainName"] ~= nil) then            
                wanConfTable["L2tp.DomainName"] = inputTable["Wan.DomainName"]
            end
            if (inputTable["Wan.Secret"] ~= nil) then            
                if (util.isAllMasked (inputTable["Wan.Secret"]) == false) then            
                    wanConfTable["L2tp.Secret"] = inputTable["Wan.Secret"]
                end
            end
             if (inputTable["Wan.MyIpAddr"] ~= nil) then
                wanConfTable["L2tp.MyIp"] = inputTable["Wan.MyIpAddr"]
            else
                wanConfTable["L2tp.ServerIp"] = util.joinFields(inputTable,"Wan","SvrIpAddrbyte", ".",4)                
            end

            if (inputTable["Wan.SvrIpAddr"] ~= nil) then
                wanConfTable["L2tp.ServerIp"] = inputTable["Wan.SvrIpAddr"]
            else
                wanConfTable["L2tp.ServerIp"] = util.joinFields(inputTable,"Wan","SvrIpAddrbyte", ".",4)                
            end
 			wanConfTable["L2tp.SplitTunnel"] = inputTable["Wan.SplitTunnel"]            
 			wanConfTable["L2tp.GetIpFromIsp"] = inputTable["Wan.IsGetIpDyn"]
            wanConfTable["L2tp.GetDnsFromIsp"] = inputTable["Wan.IsDNSDyn"]
            if (inputTable["Wan.IsGetIpDyn"] == "0") then
                wanConfTable["L2tp.StaticIp"] = util.joinFields(inputTable,"Wan","IpAddrbyte", ".",4)          
                wanConfTable["L2tp.NetMask"] = util.joinFields(inputTable,"Wan","SnetMaskbyte", ".",4)
            end
            if (inputTable["Wan.IsDNSDyn"] == "0") then
                wanConfTable["L2tp.PrimaryDns"] = util.joinFields(inputTable,"Wan","PriDnsIpAddrbyte", ".",4)
                wanConfTable["L2tp.SecondaryDns"] = util.joinFields(inputTable,"Wan","SecDnsIpAddrbyte", ".",4)
            end
            wanConfTable["L2tp.IdleTimeOutFlag"] = inputTable["Wan.IdleTimeOutFlag"]
            if (inputTable["Wan.IdleTimeOutFlag"] == "1") then
                wanConfTable["L2tp.IdleTimeOutValue"] = inputTable["Wan.IdleTime"]
            end
            if (db.existsRowWhere("L2tp", "LogicalIfName = '".. logIfName .. "'")) then
                valid = l2tp.config (wanConfTable, db.getAttribute ("L2tp","LogicalIfName",logIfName,"_ROWID_"), "edit")
            else
                valid = l2tp.config (wanConfTable, "-1", "add")
            end        
        elseif (inputTable["Wan.IspType"] == "4") then
    	    NimfConfTable["NimfConf.ConnectionType"] = "threeg"
    	    wanConfTable["ThreeG.LogicalIfName"] = logIfName
   	    	wanConfTable["ThreeG.UserName"] = inputTable["Wan.UserName"]
   	    if (util.isAllMasked (inputTable["Wan.Password"]) == false) then
    	        wanConfTable["ThreeG.Password"] = inputTable["Wan.Password"]
            end
            wanConfTable["ThreeG.DialNumber"] = inputTable["Wan.DialNumber"]
            wanConfTable["ThreeG.AuthMethod"] = inputTable["Wan.AuthMethod"]
            wanConfTable["ThreeG.Apn"] = inputTable["Wan.APN"]
            wanConfTable["ThreeG.IdleTimeOutFlag"] = inputTable["Wan.IdleTimeOutFlag"]
            if(inputTable["Wan.IdleTimeOutFlag"] == "1") then	
                wanConfTable["ThreeG.IdleTimeOutValue"] = inputTable["Wan.IdleTime"]
            end
            wanConfTable["ThreeG.GetDnsFromIsp"] = inputTable["Wan.GetDnsFromIsp"]
            if (inputTable["Wan.GetDnsFromIsp"] == "0") then
                wanConfTable["ThreeG.PrimaryDns"] = inputTable["Wan.PrimaryDns"]
                wanConfTable["ThreeG.SecondaryDns"] = inputTable["Wan.SecondaryDns"]
            end
            if (db.existsRowWhere("ThreeG", "LogicalIfName = '" .. logIfName .. "'")) then
                valid = ThreeGInternet.config (wanConfTable, db.getAttribute ("ThreeG", "LogicalIfName", logIfName, "_ROWID_"), "edit")

            else
                valid = ThreeGInternet.config (wanConfTable, "-1", "add")
            end 
        end
    end

    if (valid) then    	
        -- Check the previous WAN ISP Type, If it is PPPoE then
        -- set the Status to '0' for the PPPoE Profile if WAN Supports
        -- PPPoE Prifile selection        
        local oldConnType = db.getAttribute ("NimfConf", "_ROWID_", rowid, "ConnectionType")        
        if (oldConnType == "pppoe" and NimfConfTable["NimfConf.ConnectionType"] ~= "pppoe") then
            local table = db.getRowsWhere ("PppoeProfile", "LogicalIfName = '" .. logIfName .. "'")
            if (table ~= nil and #table ~= 0) then
                for k,v in pairs (table) do
                    v["PppoeProfile.Status"] = "0"
                    valid = db.update ("PppoeProfile", v, v["PppoeProfile._ROWID_"])
                    if (not valid) then
                        break
                    end
                 end
             end
        end
    end

    -- Set the wan advanced configuration,
    -- if all configurations are in same page.
    if (inputTable["Wan.DefMtu"] ~= nil) then
        NimfConfTable["NimfConf.UseDefaultMtu"] = inputTable["Wan.DefMtu"]
       -- NimfConfTable["NimfConf.SetMacAddress"] = inputTable["Wan.UseMac"]
        NimfConfTable["NimfConf.PortSpeed"] = inputTable["Wan.PortSpeed"]        
        if (inputTable["Wan.DefMtu"] == "0") then
            NimfConfTable["NimfConf.MtuSize"] = inputTable["Wan.MtuSize"]
        else
            if (NimfConfTable["NimfConf.RequireLogin"] == "0") then
                NimfConfTable["NimfConf.MtuSize"] = "1500"
            else
                NimfConfTable["NimfConf.MtuSize"] = "1492"
            end
        end
    end
    
    if (inputTable["Wan.UseMac"] ~= nil) then
		 NimfConfTable["NimfConf.SetMacAddress"] = inputTable["Wan.UseMac"]        
		 	if (inputTable["Wan.UseMac"] == "2") then
            NimfConfTable["NimfConf.MacAddress"] = inputTable["Wan.MacAddr"]
        end
    end	
    if (valid) then
        valid = nimf.config (NimfConfTable, rowid, "edit")
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "NIMF_CONFIG_FAILED"
	end
end

-- Wan Configuration Get
function nimfView.wanConfigGet ()
    local outTable = {}
    local NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)
    if (NimfRow ~= nil) then
        outTable["Wan.IspLoginRequired"] =  NimfRow["NimfConf.RequireLogin"]
        if (NimfRow["NimfConf.ConnectionType"] == "dhcpc") then
            outTable["Wan.IsGetIpDyn"] = "1"
            local ispConfigRow = db.getRowWhere ("Dhcpc","LogicalIfName = '" .. logIfName .. "'")
            if (ispConfigRow ~= nil) then
				 outTable["Wan.HostName"] = ispConfigRow["Dhcpc.HostName"]                outTable["Wan.IsDNSDyn"] = ispConfigRow["Dhcpc.GetDnsFromIsp"]
                if (ispConfigRow["Dhcpc.GetDnsFromIsp"] == "0") then
                    outTable["Wan.PriDnsIpAddr"] = ispConfigRow["Dhcpc.PrimaryDns"]
                    outTable["Wan.SecDnsIpAddr"] = ispConfigRow["Dhcpc.SecondaryDns"]
                end
            end
        elseif (NimfRow["NimfConf.ConnectionType"] == "ifStatic") then
            outTable["Wan.IsGetIpDyn"] = "0"
            outTable["Wan.IsDNSDyn"] = "0"            
            local ispConfigRow = db.getRowWhere ("ifStatic","LogicalIfName = '" .. logIfName .. "'")
            if (ispConfigRow ~= nil) then
               outTable["Wan.IpAddr"] = ispConfigRow["ifStatic.StaticIp"]
               outTable["Wan.SnetMask"] = ispConfigRow["ifStatic.NetMask"]
               outTable["Wan.GwIpAddr"] = ispConfigRow["ifStatic.Gateway"]               
               outTable["Wan.PriDnsIpAddr"] = ispConfigRow["ifStatic.PrimaryDns"]
               outTable["Wan.SecDnsIpAddr"] = ispConfigRow["ifStatic.SecondaryDns"]
            end
        elseif (NimfRow["NimfConf.ConnectionType"] == "pppoe") then
            local ispConfigRow = db.getRowWhere ("Pppoe","LogicalIfName = '" .. logIfName .. "'")
            if (ispConfigRow ~= nil) then
               outTable["Wan.IspType"] = "1"
               outTable["Wan.PppoeProfileName"] = ispConfigRow["Pppoe.ProfileName"] or ''
               outTable["Wan.LoginName"] = ispConfigRow["Pppoe.UserName"]
               outTable["Wan.LoginPassword"] = util.mask(ispConfigRow["Pppoe.Password"])
               outTable["Wan.AccountName"] = ispConfigRow["Pppoe.AccountName"]
               outTable["Wan.DomainName"] = ispConfigRow["Pppoe.DomainName"]
               outTable["Wan.IsGetIpDyn"] = ispConfigRow["Pppoe.GetIpFromIsp"]
               outTable["Wan.IsDNSDyn"] = ispConfigRow["Pppoe.GetDnsFromIsp"]
               if (ispConfigRow["Pppoe.GetIpFromIsp"] == "0") then
                   outTable["Wan.IpAddr"] = ispConfigRow["Pppoe.StaticIp"]             
                   outTable["Wan.SnetMask"] = ispConfigRow["Pppoe.NetMask"]
               end
               if (ispConfigRow["Pppoe.GetDnsFromIsp"] == "0") then
                   outTable["Wan.PriDnsIpAddr"] = ispConfigRow["Pppoe.PrimaryDns"]                   
                   outTable["Wan.SecDnsIpAddr"] = ispConfigRow["Pppoe.SecondaryDns"]
               end
               outTable["Wan.IdleTimeOutFlag"] = ispConfigRow["Pppoe.IdleTimeOutFlag"]
               if (ispConfigRow["Pppoe.IdleTimeOutFlag"] == "1") then
                   outTable["Wan.IdleTime"] = ispConfigRow["Pppoe.IdleTimeOutValue"]                   
               end
            end
        elseif (NimfRow["NimfConf.ConnectionType"] == "pptp") then
            local ispConfigRow = db.getRowWhere ("Pptp","LogicalIfName = '" .. logIfName .. "'")
            if (ispConfigRow ~= nil) then
               outTable["Wan.IspType"] = "2"                
               outTable["Wan.LoginName"] = ispConfigRow["Pptp.UserName"]
               outTable["Wan.LoginPassword"] = util.mask(ispConfigRow["Pptp.Password"])
               outTable["Wan.MyIpAddr"] = ispConfigRow["Pptp.MyIp"]
               outTable["Wan.SvrIpAddr"] = ispConfigRow["Pptp.ServerIp"]
               outTable["Wan.IsGetIpDyn"] = ispConfigRow["Pptp.GetIpFromIsp"]
               outTable["Wan.IsDNSDyn"] = ispConfigRow["Pptp.GetDnsFromIsp"]
               outTable["Wan.MppeEncryptSupport"] = ispConfigRow["Pptp.MppeEncryptSupport"]
			    outTable["Wan.SplitTunnel"] = ispConfigRow["Pptp.SplitTunnel"]               if (ispConfigRow["Pptp.GetIpFromIsp"] == "0") then
                   outTable["Wan.IpAddr"] = ispConfigRow["Pptp.StaticIp"]             
                   outTable["Wan.SnetMask"] = ispConfigRow["Pptp.NetMask"]
               end
               if (ispConfigRow["Pptp.GetDnsFromIsp"] == "0") then
                   outTable["Wan.PriDnsIpAddr"] = ispConfigRow["Pptp.PrimaryDns"]                   
                   outTable["Wan.SecDnsIpAddr"] = ispConfigRow["Pptp.SecondaryDns"]
               end
               outTable["Wan.IdleTimeOutFlag"] = ispConfigRow["Pptp.IdleTimeOutFlag"]
               if (ispConfigRow["Pptp.IdleTimeOutFlag"] == "1") then
                   outTable["Wan.IdleTime"] = ispConfigRow["Pptp.IdleTimeOutValue"]                   
               end
            end
        elseif (NimfRow["NimfConf.ConnectionType"] == "l2tp") then
            local ispConfigRow = db.getRowWhere ("L2tp","LogicalIfName = '" .. logIfName .. "'")
            if (ispConfigRow ~= nil) then
               outTable["Wan.IspType"] = "3"                
               outTable["Wan.LoginName"] = ispConfigRow["L2tp.UserName"]
               outTable["Wan.LoginPassword"] = util.mask(ispConfigRow["L2tp.Password"])
               outTable["Wan.AccountName"] = ispConfigRow["L2tp.AccountName"]
               outTable["Wan.DomainName"] = ispConfigRow["L2tp.DomainName"]
               if (ispConfigRow["L2tp.Secret"] == nil) then
                   outTable["Wan.Secret"] = ""
               else
                   outTable["Wan.Secret"] = util.mask(ispConfigRow["L2tp.Secret"])
               end
               outTable["Wan.MyIpAddr"] = ispConfigRow["L2tp.MyIp"]
               outTable["Wan.SvrIpAddr"] = ispConfigRow["L2tp.ServerIp"]
 			   outTable["Wan.SplitTunnel"] = ispConfigRow["L2tp.SplitTunnel"]               outTable["Wan.IsGetIpDyn"] = ispConfigRow["L2tp.GetIpFromIsp"]
               outTable["Wan.IsDNSDyn"] = ispConfigRow["L2tp.GetDnsFromIsp"]
               if (ispConfigRow["L2tp.GetIpFromIsp"] == "0") then
                   outTable["Wan.IpAddr"] = ispConfigRow["L2tp.StaticIp"]             
                   outTable["Wan.SnetMask"] = ispConfigRow["L2tp.NetMask"]
               end
               if (ispConfigRow["L2tp.GetDnsFromIsp"] == "0") then
                   outTable["Wan.PriDnsIpAddr"] = ispConfigRow["L2tp.PrimaryDns"]                   
                   outTable["Wan.SecDnsIpAddr"] = ispConfigRow["L2tp.SecondaryDns"]
               end
               outTable["Wan.IdleTimeOutFlag"] = ispConfigRow["L2tp.IdleTimeOutFlag"]
               if (ispConfigRow["L2tp.IdleTimeOutFlag"] == "1") then
                   outTable["Wan.IdleTime"] = ispConfigRow["L2tp.IdleTimeOutValue"]                   
               end
             end
          elseif (NimfRow["NimfConf.ConnectionType"] == "threeg") then
            local ispConfigRow = db.getRowWhere ("ThreeG","LogicalIfName = '" .. logIfName .. "'")
            if (ispConfigRow ~= nil) then
               outTable["Wan.IspType"] = "4"                
               outTable["Wan.UserName"] = ispConfigRow["ThreeG.UserName"]
               outTable["Wan.Password"] = util.mask(ispConfigRow["ThreeG.Password"])
               outTable["Wan.DialNumber"] = ispConfigRow["ThreeG.DialNumber"]
               outTable["Wan.AuthMethod"] = ispConfigRow["ThreeG.AuthMethod"]
               outTable["Wan.APN"] = ispConfigRow["ThreeG.Apn"]
               outTable["Wan.IsGetIpDyn"] = ispConfigRow["ThreeG.GetIpFromIsp"]
               outTable["Wan.GetDnsFromIsp"] = ispConfigRow["ThreeG.GetDnsFromIsp"]
               if (ispConfigRow["ThreeG.GetDnsFromIsp"] == "0") then
                   outTable["Wan.PrimaryDns"] = ispConfigRow["ThreeG.PrimaryDns"]             
                   outTable["Wan.SecondaryDns"] = ispConfigRow["ThreeG.SecondaryDns"]
               end
               outTable["Wan.IdleTimeOutFlag"] = ispConfigRow["ThreeG.IdleTimeOutFlag"]
               if (ispConfigRow["ThreeG.IdleTimeOutFlag"] == "1") then
                   outTable["Wan.IdleTime"] = ispConfigRow["ThreeG.IdleTimeOutValue"]                   
               end
            end
        end       
    end

	return outTable
end

-- Wan Advanced Configuration
function nimfView.advConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

    -- return error if wan Configuration not found
    local NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)

    -- Retrying for configuration in NimfConf.
    local retryIdx = 1
    while (NimfRow == nil and retryIdx <= 3) do
        NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)
        retryIdx = retryIdx + 1
    end    

    if (NimfRow == nil) then
        return "ERROR", "NIMF_CONFIG_NOTFOUND"
    end

    rowid = NimfRow["NimfConf._ROWID_"]

	db.beginTransaction() --begin transaction
	local valid = false
    local wanConfTable = {}

	-- config Advanced NIMF Configurations
    local NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. logIfName .. "'")
    if (NimfRow ~= nil) then
        NimfRow["NimfConf.UseDefaultMtu"] = inputTable["Wan.DefMtu"]
        NimfRow["NimfConf.SetMacAddress"] = inputTable["Wan.UseMac"]
        NimfRow["NimfConf.PortSpeed"] = inputTable["Wan.PortSpeed"]        
        if (inputTable["Wan.DefMtu"] == "0") then
            NimfRow["NimfConf.MtuSize"] = inputTable["Wan.MtuSize"]
        else
            if (NimfRow["NimfConf.RequireLogin"] == "0") then
                NimfRow["NimfConf.MtuSize"] = "1500"
            else
                NimfRow["NimfConf.MtuSize"] = "1492"
            end
        end
        if (inputTable["Wan.UseMac"] == "2") then
            NimfRow["NimfConf.MacAddress"] = inputTable["Wan.MacAddr"]
        end
        NimfRow["NimfConf.AddressFamily"] = "2"
        valid = nimf.config (NimfRow, rowid, "edit")
        if (valid == false) then
    		db.rollback()
	    	return "ERROR", "NIMF_CONFIG_FAILED"            
        end
    end
	
    -- setting NIMF Mode / NAT
    if (inputTable["NatTable.Enable"] ~= nil) then
        if (valid) then
            local natTable = {}
            natTable["NatTable.Enable"] = inputTable["NatTable.Enable"]
            valid = nat.config (natTable, "1", "edit")
        end
    end

    -- setting IGMP Proxy Settings.
    if (inputTable["Igmp.IgmpEnable"] ~= nil) then
        if (valid) then
            local igmpTable = {}
            igmpTable ["Igmp.IgmpEnable"] = inputTable["Igmp.IgmpEnable"]
            valid = igmp.config (igmpTable, "1", "edit")
        end
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "NIMF_CONFIG_FAILED"
	end
end

-- Wan Configuration Get
function nimfView.wanAdvConfigGet ()
    local outTable = {}
    local ispConfigRow = nil
    local NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)
    if (NimfRow ~= nil) then
        outTable["Wan.DefMtu"] = NimfRow["NimfConf.UseDefaultMtu"] or ''
        outTable["Wan.MtuSize"] = NimfRow["NimfConf.MtuSize"] or ''
        outTable["Wan.UseMac"] = NimfRow["NimfConf.SetMacAddress"] or ''
        outTable["Wan.PortSpeed"] = NimfRow["NimfConf.PortSpeed"] or '' 
        outTable["Wan.MacAddr"] = NimfRow["NimfConf.MacAddress"] or ''
        if (NimfRow["NimfConf.RequireLogin"] == "0") then
            outTable["Wan.MtuSizeMin"] = "68"
            outTable["Wan.MtuSizeMax"] = "1500"
        else
            outTable["Wan.MtuSizeMin"] = "128"
            outTable["Wan.MtuSizeMax"] = "1492"
        end        
    end

    -- getting WAN Mode / NAT
    outTable["NatTable.Enable"] = db.getAttribute ("NatTable","_ROWID_","1","Enable") or ''
    outTable["Igmp.IgmpEnable"] = db.getAttribute ("Igmp","_ROWID_","1","IgmpEnable") or ''

	return outTable
end

-- Wan Status Get
function nimfView.wanStatusGet ()
    local outTable = {}
    local wanStatusRow = nil
    local NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)
    if (NimfRow ~= nil) then
        local connectionType = NimfRow["NimfConf.ConnectionType"]
        if (addressFamily == "10") then --ipv6 address
            connectionType = NimfRow["NimfConf.ConnectionType6"]
        end
        if (connectionType == "dhcpc") then
            outTable["Wan.ConnectionType"] = "Dynamic IP (DHCP)"
            local dhcpStatus = db.getRowWhere ("DhcpStatus","LogicalIfName = '" .. logIfName .. "'")
            if (dhcpStatus ~= nil) then
                outTable["Dhcpc.DhcpServerIp"] = dhcpStatus["DhcpStatus.DhcpServerIp"]
                outTable["Dhcpc.LeaseTime"] = dhcpStatus["DhcpStatus.LeaseTime"]
                outTable["Dhcpc.LeaseObtained"] = dhcpStatus["DhcpStatus.LeaseObtained"]                
            end
        elseif (connectionType == "dhcp6c") then
            outTable["Wan.ConnectionType"] = "Dynamic IP (DHCP)"
            outTable["Wan.IPVersion"] = "ipv6"
        elseif (connectionType == "ifStatic" or connectionType == "ifStatic6") then
            outTable["Wan.ConnectionType"] = "STATIC"
        elseif (connectionType == "pppoe") then
            outTable["Wan.ConnectionType"] = "PPPOE"
        elseif (connectionType == "pptp") then
            outTable["Wan.ConnectionType"] = "PPTP"
        elseif (connectionType == "l2tp") then
            outTable["Wan.ConnectionType"] = "L2TP"
        end

        if (NimfRow["NimfConf.IfStatus"] == "0") then
            outTable["Wan.operation"] = "Enable"
        elseif (NimfRow["NimfConf.IfStatus"] == "1") then
            outTable["Wan.operation"] = "Disable"            
        end
    end

    wanStatusRow = db.getRowWhere ("NimfStatus","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)
    if (wanStatusRow ~= nil) then
        outTable["Wan.ConnectionState"] = wanStatusRow["NimfStatus.Nimfstatus"] or ""
        outTable["Wan.ConnectionTime"] =  wanStatusRow["NimfStatus.WanUpTime"]  or ""
        outTable["Wan.LinkState"] = wanStatusRow["NimfStatus.Linkstatus"] or ""
    end

    if (addressFamily == "10") then
        local wan6Rows = db.getRowsWhere ("ipAddressTable","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)
        local gwRows = db.getRowsWhere ("defaultRouters","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)
        local nsRows = db.getRowsWhere ("resolverConfig","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)

        outTable["Wan.IpAddr"] = ""
        outTable["Wan.Gateway"] = ""
        outTable["Wan.DNS"] = ""
        local i = 0
        for k,v in pairs(wan6Rows) do
            i = i + 1
            local row = wan6Rows[i]
            outTable["Wan.IpAddr"] = outTable["Wan.IpAddr"] .. row["ipAddressTable.ipAddress"] .. "/" .. row["ipAddressTable.ipv6PrefixLen"]
            if(i ~= #wan6Rows) then
                outTable["Wan.IpAddr"] = outTable["Wan.IpAddr"] .. ", "
			end
        end

        i = 0
        for k,v in pairs(gwRows) do
            i = i + 1
            local row = gwRows[i]
            outTable["Wan.Gateway"] = outTable["Wan.Gateway"] .. row["defaultRouters.nextHopGateway"] or ''
            if(i ~= #gwRows) then
                outTable["Wan.Gateway"] = outTable["Wan.Gateway"] .. ", "
			end
        end

        i = 0
        for k,v in pairs(nsRows) do
            i = i + 1
            local row = nsRows[i]
            outTable["Wan.DNS"] = outTable["Wan.DNS"] .. row["resolverConfig.nameserver1"] or ''
            if(i ~= #nsRows) then
                outTable["Wan.DNS"] = outTable["Wan.DNS"] .. ", "
			end
        end
    else
        wanStatusRow = db.getRowWhere ("ipAddressTable","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)
        local gwStatusRow = db.getRowWhere ("defaultRouters","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)
        local dnsStatusRow = db.getRowWhere ("resolverConfig","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)
        if (wanStatusRow ~= nil) then
            outTable["Wan.IpAddr"] = wanStatusRow["ipAddressTable.ipAddress"]
            outTable["Wan.SnetMask"] = wanStatusRow["ipAddressTable.subnetMask"]
        end        
        if (gwStatusRow ~= nil) then
            outTable["Wan.Gateway"] = gwStatusRow["defaultRouters.nextHopGateway"]      
        end
        if (dnsStatusRow ~= nil) then        
            outTable["Wan.DNS"] = dnsStatusRow["resolverConfig.nameserver1"]
        end
    end

    local wanState = db.getAttribute ("networkInterface", "LogicalIfName", logIfName, "IfStatus") or ''
    if (wanState ~= '') then
        if (wanState == "1" or wanState == "3" or wanState == "2") then
            if (logIfName == 'WAN1' and wanState == "1") then
                if (addressFamily == "10") then
                    outTable["Wan.WanState"] = "DOWN"
                else
                    outTable["Wan.WanState"] = "UP"
                end
            elseif (logIfName == 'WAN1' and wanState == "2") then
                if (addressFamily == "2") then
                    outTable["Wan.WanState"] = "DOWN"
                else
                    outTable["Wan.WanState"] = "UP"
                end
            elseif (logIfName == 'WAN1' and wanState == "3") then
                outTable["Wan.WanState"] = "UP"
            else
                outTable["Wan.WanState"] = "UP"
            end
        elseif (wanState == "0") then
            outTable["Wan.WanState"] = "DOWN"
        end
    end

	return outTable
end

-- Wan Disable/Enable
function nimfView.wanStatusChange (operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

    local valid = false
    local NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)

    -- Retrying for configuration in NimfConf.
    local retryIdx = 1
    while (NimfRow == nil and retryIdx <= 3) do
        NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)
        retryIdx = retryIdx + 1
    end

    if (NimfRow == nil) then
        return "ERROR", "NIMF_CONFIG_NOTFOUND"
    end

    local rowid = NimfRow["NimfConf._ROWID_"]
    if (NimfRow ~= nil) then
        local nimfInTable = {}
        if (operation == "Enable") then
            nimfInTable["NimfConf.IfStatus"] = "1"
        elseif (operation == "Disable") then
            nimfInTable["NimfConf.IfStatus"] = "0"            
        end
        valid = nimf.config (nimfInTable, rowid, "edit")
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "NIMF_CONFIG_FAILED"
	end    
end

-- DHCP Release/Renew
function nimfView.dhcpStatusChange (operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

    local valid = false
    local NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)

    -- Retrying for configuration in NimfConf.
    local retryIdx = 1
    while (NimfRow == nil and retryIdx <= 3) do
        NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. logIfName .. "' and AddressFamily = " .. addressFamily)
        retryIdx = retryIdx + 1
    end

    if (NimfRow == nil) then
        return "ERROR", "NIMF_CONFIG_NOTFOUND"
    end

    local rowid = NimfRow["NimfConf._ROWID_"]
    if (addressFamily == "10") then
        valid = true;
    else
        if (db.existsRowWhere("Dhcpc", "LogicalIfName = '".. logIfName .. "'")) then
            local dhcpcInTable = {}
            dhcpcInTable["Dhcpc.ReleaseLease"] = "1"
            valid = dhcpc.config (dhcpcInTable, db.getAttribute ("Dhcpc","LogicalIfName",logIfName,"_ROWID_"), "edit")
        end
    end

    -- Dummy update the NimfConf Table to trigger the event handler.
    if (valid) then
        if (NimfRow ~= nil) then
            if (operation == "release") then
                local nimfInTable = {}
                nimfInTable["NimfConf.IfStatus"] = "0"
                valid = nimf.config (nimfInTable, rowid, "edit")
            elseif (operation == "renew") then
                local nimfInTable = {}
                nimfInTable["NimfConf.IfStatus"] = "1"
                valid = nimf.config (nimfInTable, rowid, "edit")
            end
        else
            valid = false
        end
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "NIMF_CONFIG_FAILED"
	end    
end

-- WAN Configuration Get Wrapper
function nimfView.wanConfigGetWrap (wanLogicalIfName)
    logIfName = wanLogicalIfName
    local wanConfig = nimfView.wanConfigGet ()
    local wanAdvConfig = nimfView.wanAdvConfigGet ()
    local outTable = {}
    if (wanConfig ~= nil and wanAdvConfig ~= nil) then
        outTable = util.tableAppend (wanConfig, wanAdvConfig)
    end
    return outTable
end

-- WAN Configuration Get Wrapper
function nimfView.wanConfigSetWrap (inputTable, rowid, operation)
   --[[ if ( inputTable["Wan.IspType"] == "1") then
        local idleTimeOutFlag = db.getAttribute("PppoeProfile","ProfileName",inputTable["Wan.PppoeProfileName"],"IdleTimeOutFlag")
        local rows = db.getRowWhere("WanMode","(DedicatedLogicalIfName = 'WAN1' AND  AddressFamily = 2) or (DedicatedLogicalIfName = 'WAN2' AND  AddressFamily = 2)")
        local wanMode = rows["WanMode.Wanmode"] or ''
        if (idleTimeOutFlag == "1" and wanMode == "1" ) then 
            return "ERROR", "LOAD_BALANCE_ERROR"
        end
    end
    --]]
    logIfName = inputTable["NimfConf.LogicalIfName"] or 'WAN1'
    if (inputTable["Wan.MyIpAddr"] ~= nil) then
        local ipaddr = util.split(inputTable["Wan.MyIpAddr"] or "",".")
        inputTable["Wan.MyIpAddrbyte1"] = ipaddr[1]
        inputTable["Wan.MyIpAddrbyte2"] = ipaddr[2]
        inputTable["Wan.MyIpAddrbyte3"] = ipaddr[3]
        inputTable["Wan.MyIpAddrbyte4"] = ipaddr[4]
    end    
    --[[if (inputTable["Wan.SvrIpAddr"] ~= nil) then
        local ipaddr = util.split(inputTable["Wan.SvrIpAddr"] or "",".")
        inputTable["Wan.SvrIpAddrbyte1"] = ipaddr[1]
        inputTable["Wan.SvrIpAddrbyte2"] = ipaddr[2]
        inputTable["Wan.SvrIpAddrbyte3"] = ipaddr[3]
        inputTable["Wan.SvrIpAddrbyte4"] = ipaddr[4]
    end     
	--]]    if (inputTable["Wan.IpAddr"] ~= nil) then
        local ipaddr = util.split(inputTable["Wan.IpAddr"] or "",".")
        inputTable["Wan.IpAddrbyte1"] = ipaddr[1]
        inputTable["Wan.IpAddrbyte2"] = ipaddr[2]
        inputTable["Wan.IpAddrbyte3"] = ipaddr[3]
        inputTable["Wan.IpAddrbyte4"] = ipaddr[4]
    end
    if (inputTable["Wan.SnetMask"] ~= nil) then
        local ipaddr = util.split(inputTable["Wan.SnetMask"] or "",".")
        inputTable["Wan.SnetMaskbyte1"] = ipaddr[1]
        inputTable["Wan.SnetMaskbyte2"] = ipaddr[2]
        inputTable["Wan.SnetMaskbyte3"] = ipaddr[3]
        inputTable["Wan.SnetMaskbyte4"] = ipaddr[4]
    end
    if (inputTable["Wan.GwIpAddr"] ~= nil) then
        local ipaddr = util.split(inputTable["Wan.GwIpAddr"] or "",".")
        inputTable["Wan.GwIpAddrbyte1"] = ipaddr[1]
        inputTable["Wan.GwIpAddrbyte2"] = ipaddr[2]
        inputTable["Wan.GwIpAddrbyte3"] = ipaddr[3]
        inputTable["Wan.GwIpAddrbyte4"] = ipaddr[4]
    end
    if (inputTable["Wan.PriDnsIpAddr"] ~= nil) then
        local ipaddr = util.split(inputTable["Wan.PriDnsIpAddr"] or "",".")
        inputTable["Wan.PriDnsIpAddrbyte1"] = ipaddr[1]
        inputTable["Wan.PriDnsIpAddrbyte2"] = ipaddr[2]
        inputTable["Wan.PriDnsIpAddrbyte3"] = ipaddr[3]
        inputTable["Wan.PriDnsIpAddrbyte4"] = ipaddr[4]
    end
    if (inputTable["Wan.SecDnsIpAddr"] ~= nil and inputTable["Wan.SecDnsIpAddr"] ~= "") then
        local ipaddr = util.split(inputTable["Wan.SecDnsIpAddr"] or "",".")
        inputTable["Wan.SecDnsIpAddrbyte1"] = ipaddr[1]
        inputTable["Wan.SecDnsIpAddrbyte2"] = ipaddr[2]
        inputTable["Wan.SecDnsIpAddrbyte3"] = ipaddr[3]
        inputTable["Wan.SecDnsIpAddrbyte4"] = ipaddr[4]
    elseif (inputTable["Wan.SecDnsIpAddr"] == "") then
        inputTable["Wan.SecDnsIpAddrbyte1"] = ""
        inputTable["Wan.SecDnsIpAddrbyte2"] = ""
        inputTable["Wan.SecDnsIpAddrbyte3"] = ""
        inputTable["Wan.SecDnsIpAddrbyte4"] = ""
    end
    if(inputTable["Wan.UseMac"] == "2" ) then
    	local macAddr = inputTable["Wan.MacAddr"]
    	local secondBit = string.sub(macAddr,2,2)
    	if(secondBit == "1" or secondBit == "3" or secondBit == "5" or secondBit == "7" or secondBit == "9" or secondBit == "b" or secondBit == "d" or secondBit == "f" or secondBit == "B" or secondBit == "D" or secondBit == "F") then
    		return "ERROR", "MULTICAST_MAC_ERROR"
    	end
    end
    return nimfView.config (inputTable, rowid, operation)
end

function nimfView.wanStatusChangeWrap (operation, wanLogicalIfName, wanAddressFamily)
    logIfName =  wanLogicalIfName or 'WAN1'
    addressFamily = wanAddressFamily or '2'
    return nimfView.wanStatusChange (operation)
end

function nimfView.dhcpStatusChangeWrap (operation, wanLogicalIfName, wanAddressFamily)
    logIfName =  wanLogicalIfName or 'WAN1'
    addressFamily = wanAddressFamily or '2'
    return nimfView.dhcpStatusChange (operation)
end

function nimfView.wanStatusGetWrap (wanLogicalIfName, wanAddressFamily)
    logIfName =  wanLogicalIfName or 'WAN1'
    addressFamily = wanAddressFamily or '2'    
    return nimfView.wanStatusGet ()
end

function nimfView.portModeConfig (inputTable, rowid, operation)
	-- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transaction
	local valid = false

    local boardName = db.getAttribute ("unitName","_ROWID_","1","unitName")
    if (boardName ~= nil and boardName ~= "WRVS5110") then
        boardName = true
    else
        boardName = false
    end

    if (boardName and (inputTable["ConfigPort.LogicalIfName"] ~= nil and inputTable["ConfigPort.LogicalIfName"] ~= "LAN")) then
        -- check that vlanId:4094(Reserved for WAN/DMZ) is there in vlan or not
        local isExistsVlan = db.existsRow ("vlan", "vlanId", "4094")
        if (isExistsVlan) then
		    db.rollback()
    		return "ERROR", "VLAN_4094_EXISTS"        
        end
        -- check that PVID:4094(Reserved for WAN/DMZ) is there in VlanPortMgmt or not
        local isRows = db.getRowsWhere ("VlanPortMgmt", "DefaultPVID = 4094 and PortName != 'OptionalPort'")
        if (#isRows ~= 0) then
		    db.rollback()
    		return "ERROR", "PORT_VLAN_PVID_4094_EXISTS"        
        end

        -- Enable vlan Status if config Port set to WAN/DMZ
        --local vlanState = {}
        --vlanState["VlanStatus.VlanState"] = "1"
        --valid = db.update("VlanStatus", vlanState, "1")
        valid = true
    else
        valid = true
    end

    --default config for optional port is wan.
    local optPortRowid = 3
    local IfgroupId = 2
    if (inputTable["ConfigPort.LogicalIfName"] == "DMZ")  then
       IfgroupId = 4
    elseif (inputTable["ConfigPort.LogicalIfName"] == "LAN" ) then
        IfgroupId = 1
    end
    if (valid) then
    	local isExists = db.existsRow ("ConfigPort", "_ROWID_", rowid)
        if (isExists) then
            local ifGrpInfo = {}
            ifGrpInfo["DefaultIfGrpInfo.LogicalIfName"] = inputTable["ConfigPort.LogicalIfName"]
            ifGrpInfo["DefaultIfGrpInfo.IfGroupId"] = IfgroupId
            valid = db.update("DefaultIfGrpInfo", ifGrpInfo, optPortRowid)
            if(valid) then
                valid = db.update("ConfigPort", inputTable, rowid)
            end
        else
            valid = db.insert("ConfigPort", inputTable)        
        
        end
    end

    -- Disable dhcp server on DMZ interface
    -- if configurable port not equal to DMZ
    local dmzDhcpConfig = db.getRow ("dhcp", "logicalIfName", "DMZ")
    if (valid and dmzDhcpConfig ~= nil) then
        if (inputTable["ConfigPort.LogicalIfName"] ~= "DMZ" and dmzDhcpConfig["dhcp.dhcpEnabled"] == "1") then
            dmzDhcpConfig["dhcp.oldState"] = "1"
            dmzDhcpConfig["dhcp.dhcpEnabled"] = "0"
        elseif (inputTable["ConfigPort.LogicalIfName"] == "DMZ" and dmzDhcpConfig["dhcp.oldState"] == "1") then
            local temp = db.getRowWhere ("ifStatic", "LogicalIfName = 'DMZ' and AddressFamily = 2")
            if (temp ~= nil) then
				db.setAttribute ("ifStatic", "_ROWID_", temp["ifStatic._ROWID_"], "LogicalIfName", temp["ifStatic.LogicalIfName"])
            end
            dmzDhcpConfig["dhcp.oldState"] = "0"
            dmzDhcpConfig["dhcp.dhcpEnabled"] = "1"
        end
        valid = db.update("dhcp", dmzDhcpConfig, dmzDhcpConfig["dhcp._ROWID_"])
    end

	-- return    
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "PORT_MODE_SET_FAILED"
	end    
end

function nimfView.wanModeConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
    	return "ACCESS_DENIED", "ADMIN_REQD"
    end

    if (inputTable["WanMode.Wanmode"] ~= nil and inputTable["WanMode.Wanmode"] == "1") then
        local type1=db.getRowWhere("NimfConf","LogicalIfName = 'WAN1' AND  AddressFamily = 2")
        local connection1 = type1["NimfConf.ConnectionType"]
        if((connection1 == "l2tp") or (connection1 == "pptp")) then 
            idle1=db.getAttribute(connection1,"LogicalIfName","WAN1","IdleTimeOutFlag")
        elseif((connection1 == "pppoe")) then
            profile1=db.getAttribute("pppoe","LogicalIfName","WAN1","ProfileName")
            idle1=db.getAttribute("PppoeProfile","ProfileName",profile1,"IdleTimeOutFlag")
        else
            idle1="0"
        end

        local type2=db.getRowWhere("NimfConf","LogicalIfName = 'WAN2' AND  AddressFamily = 2")
        local connection2 = type2["NimfConf.ConnectionType"]
   
        if((connection2 == "l2tp") or (connection2 == "pptp")) then
            idle2=db.getAttribute(connection2,"LogicalIfName","WAN2","IdleTimeOutFlag")
        elseif((connection2 == "pppoe")) then
            profile2=db.getAttribute("pppoe","LogicalIfName","WAN2","ProfileName")
            idle2=db.getAttribute("PppoeProfile","ProfileName",profile2,"IdleTimeOutFlag")
        else
            idle2="0"
        end
        
        if (idle1 == "1" or idle2 == "1") then
            return "ERROR", "IDLE_TIMEOUT_ERROR"
        end
    end

    db.beginTransaction() --begin transaction
    local valid = false

    if (inputTable["WanMode.Wanmode"] ~= nil and inputTable["WanMode.Wanmode"] == "2") then
        if (inputTable["WanMode.FailoverPriLogicalIfName"] ~= nil and inputTable["WanMode.FailoverPriLogicalIfName"] == "WAN1") then
            inputTable["WanMode.FailoverSecLogicalIfName"] = "WAN2"
        elseif (inputTable["WanMode.FailoverPriLogicalIfName"] ~= nil and inputTable["WanMode.FailoverPriLogicalIfName"] == "WAN2") then
            inputTable["WanMode.FailoverSecLogicalIfName"] = "WAN1"
        end
    end

    if(db.existsRow("WanMode", "_ROWID_", "1")) then
    	valid=db.update("WanMode",inputTable,rowid)
    else
    	valid=db.insert("WanMode",inputTable)
    end

    local failureTable = {}
    failureTable["WanFailure.LogicalIfName"]="WAN1"
    failureTable["WanFailure.AddressFamily"]="2"    
    failureTable["WanFailure.PingIp"]=inputTable["WanFailure.PingIp"]
    failureTable["WanFailure.DnsIp"]=inputTable["WanFailure.DnsIp"]

    if(valid) then
	    if(db.existsRow("WanFailure","LogicalIfName","WAN1")) then
            local rowId = db.getAttribute("WanFailure","LogicalIfName","WAN1","_ROWID_")
	        valid=db.update("WanFailure",failureTable,rowId)
    	else
	        valid=db.insert("WanFailure",failureTable)
    	end
    end

    failureTable = {}
    failureTable["WanFailure.LogicalIfName"]="WAN2"
    failureTable["WanFailure.AddressFamily"]="2"
    failureTable["WanFailure.PingIp"]=inputTable["WanFailure.PingIp1"]
    failureTable["WanFailure.DnsIp"]=inputTable["WanFailure.DnsIp1"]

    if(valid) then
    	if(db.existsRow("WanFailure","LogicalIfName","WAN2")) then
            local rowId = db.getAttribute("WanFailure","LogicalIfName","WAN2","_ROWID_")            
	        valid=db.update("WanFailure",failureTable,rowId)
    	else
	        valid=db.insert("WanFailure",failureTable)
    	end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "WM_CONFIG_FAILED"
    end
end

function nimfView.wanMode6Config (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
    	return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false
   
    if (inputTable["WanMode.Wanmode"] ~= nil and inputTable["WanMode.Wanmode"] == "2") then
        if (inputTable["WanMode.FailoverPriLogicalIfName"] ~= nil and inputTable["WanMode.FailoverPriLogicalIfName"] == "WAN1") then
            inputTable["WanMode.FailoverSecLogicalIfName"] = "WAN2"
        elseif (inputTable["WanMode.FailoverPriLogicalIfName"] ~= nil and inputTable["WanMode.FailoverPriLogicalIfName"] == "WAN2") then
            inputTable["WanMode.FailoverSecLogicalIfName"] = "WAN1"
        end
    end

    if(db.existsRow("WanMode", "_ROWID_", "2")) then
    	valid=db.update("WanMode",inputTable,rowid)
    else
    	valid=db.insert("WanMode",inputTable)
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "WM_CONFIG_FAILED"
    end
end

function nimfView.protocolBindingConfig (inputTable, rowid, operation)
end

-- Function for IPV6 WAN Config
function nimfView.ipv6WanConfig(inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    logIfName = inputTable["NimfConf.LogicalIfName"]
    -- checking for Wan Configuration
    local where = "LogicalIfName = '" .. logIfName .. "' AND AddressFamily = 10"
    local nimfConfig = db.getRowWhere ("NimfConf", where)
    if (nimfConfig == nil) then
        return "ERROR", "NIMF_CONFIG_NOTFOUND"
    end
	rowid = nimfConfig["NimfConf._ROWID_"]
    db.beginTransaction() --begin transaction
    local valid = false
    
    local NimfConfTable = {}
    
    NimfConfTable["NimfConf.LogicalIfName"] = logIfName
    NimfConfTable["NimfConf.AddressFamily"] = "10"

    if(inputTable["NimfConf.ConnectionType6"] == "dhcp6c") then
        NimfConfTable["NimfConf.ConnectionType6"] = "dhcp6c"
	    inputTable["dhcpv6c.LogicalIfName"] = logIfName
	    if (db.existsRowWhere("dhcpv6c", "LogicalIfName='" .. logIfName .. "'")) then
            valid = nimfView.dhcp6Config (inputTable, db.existsRowWhere ("dhcpv6c", "LogicalIfName='" .. logIfName .. "'"), "edit")
        else
            valid = nimfView.dhcp6Config (inputTable, "-1", "add")
	    end
    else
        NimfConfTable["NimfConf.ConnectionType6"] = "ifStatic6"
	    inputTable["ifStatic.LogicalIfName"] = logIfName
	    inputTable["ifStatic.AddressFamily"] = "10"
        if (db.existsRowWhere("ifStatic", where)) then
            valid = ifStatic.config (inputTable, db.existsRowWhere("ifStatic", where), "edit")
        else
            valid = ifStatic.config (inputTable, "-1", "add")
        end
    end
    
    if (valid) then
        valid = nimf.config (NimfConfTable, rowid, "edit")            
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "IPV6WAN_CONFIG_FAILED"
    end
end


-- Getting IPv6 Wan Configuration 
function nimfView.ipv6WanConfigGet(logIfNameVar)
	logIfName=logIfNameVar    local outTable = {}
    local NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. logIfName .. "' AND AddressFamily=10")
    if (NimfRow ~= nil) then
    	if(NimfRow["NimfConf.ConnectionType6"] == "dhcp6c") then
	        outTable["NimfConf.ConnectionType6"] = "dhcp6c"
	        local DhcpRow = db.getRowWhere ("dhcpv6c","LogicalIfName = '" .. logIfName .. "'")
    	    if(DhcpRow ~= nil) then
	            outTable["dhcpv6c.statelessMode"] = DhcpRow["dhcpv6c.statelessMode"]
    	    	outTable["dhcpv6c.requestPreferredAddress"] = DhcpRow["dhcpv6c.requestPreferredAddress"]
	    	    outTable["dhcpv6c.preferredAddress"] = DhcpRow["dhcpv6c.preferredAddress"]
    		    outTable["dhcpv6c.preferredAddressPrefixLength"] = DhcpRow["dhcpv6c.preferredAddressPrefixLength"]
	        end
    	elseif(NimfRow["NimfConf.ConnectionType6"] == "ifStatic6") then
	        outTable = db.getRowWhere ("ifStatic","LogicalIfName = '" .. logIfName .. "' AND AddressFamily=10")
	        outTable["NimfConf.ConnectionType6"] = "ifStatic6"
    	end
    end
    return outTable
end

--dhcp6c Configuration
function nimfView.dhcp6Config(inputTable, rowid, operation)
    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            return db.insert("dhcpv6c", inputTable)
        elseif (operation == "edit") then
            return db.update("dhcpv6c", inputTable, rowid)
        elseif (operation == "delete") then
            return nil
        end
    end
    return false
end

-- IP Mode Configuration
function nimfView.ipModeConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false
    local isExists = db.existsRow ("networkInfo", "_ROWID_", rowid)
    if (isExists) then
        valid = db.update("networkInfo", inputTable, rowid)
    else
        valid = db.insert("networkInfo", inputTable)        
    end

    -- return    
    if (valid) then
	db.commitTransaction(true)
	return "OK", "STATUS_OK"
    else
	db.rollback()
	return "ERROR", "IP_MODE_SET_FAILED"
    end    
end

-- 6to4 Tunnel Configuration
function nimfView.sixtoFourTunnelConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false
    local isExists = db.existsRow ("sixToFourTunnel", "_ROWID_", rowid)
    if (isExists) then
        valid = db.update("sixToFourTunnel", inputTable, rowid)
    else
        valid = db.insert("sixToFourTunnel", inputTable)        
    end

    -- return    
    if (valid) then
	db.commitTransaction(true)
	return "OK", "STATUS_OK"
    else
	db.rollback()
	return "ERROR", "6TO4_TUNNEL_CONFIG_FAILED"
    end    
end


-- ISATAP Tunnel Deletion
function nimfView.isatapTunnelDelete (rowIds)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
	return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false

    -- delete routes
    for k,v in pairs(rowIds) do
        local inTable = {}
        inTable["isatapTunnel._ROWID_"] = v
	valid = db.delete("isatapTunnel", inTable) 
	if(valid==false) then
	    break
	end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "ISATAP_TUNNEL_DELETE_SUCCESS"
    else
        db.rollback()
        return "ERROR", "ISATAP_TUNNEL_DELETE_FAILED"
    end
end


 -- ISATAP Tunnel Configuration
function nimfView.isatapTunnelConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
    	return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false
    
    inputTable["isatapTunnel.transportIfName"] = "LAN"

    if(inputTable["isatapTunnel.useLanAddress"] == "1") then
        inputTable["isatapTunnel.localIPv4Address"] = ""
    end

    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            valid = db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            valid = db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
	    valid = db.delete(DBTable, inputTable)
        end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "ISATAP_TUNNEL_CONFIG_FAILED"
    end
end

-- IPV6 MLD Configuration
function nimfView.ipv6MldConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false
    local isExists = db.existsRow ("mldproxy", "_ROWID_", rowid)
    if (isExists) then
        valid = db.update("mldproxy", inputTable, rowid)
    else
        valid = db.insert("mldproxy", inputTable)        
    end

    -- return    
    if (valid) then
	db.commitTransaction(true)
	return "OK", "STATUS_OK"
    else
	db.rollback()
	return "ERROR", "IPV6_MLD_CONFIG_FAILED"
    end    
end

function nimfView.configurablePortUsageCheck (inputTable)
    if (inputTable["ConfigPort.LogicalIfName"] ~= "LAN") then
        local usingRows = db.getRowsWhere ("TrafficSelector","PortName = 'OptionalPort'")
        if (usingRows ~= nil and #usingRows ~= 0) then
            return "ERROR", "CONFIG_LAN_IN_USE_BY_BW_TRAFFIC_SELECTOR"
        end
        usingRows = db.getRowsWhere ("PortMirroring", "MirroringEnable = 1 and CapturePort = 'OptionalPort'")
        if (usingRows ~= nil and #usingRows ~= 0) then
            return "ERROR", "CONFIG_LAN_IN_USE_BY_PORT_MIRRORING"
        end
    end
    return "OK", "STATUS_OK"
end

function nimfView.checkOptionWanTables (inputTable)
	local oldPort = db.getAttribute ("ConfigPort", "_ROWID_", "1","LogicalIfName")
	local profileTable = db.getRowsWhere ("BandWidthProfile","WANID = 'WAN2'")
	local fwTable = db.getRowsWhere ("FirewallRules",
	                                 "((DestinationPublicInterface = 'WAN2' AND RuleType = 'INSECURE_SECURE') OR " ..
					  "(DestinationPublicInterface = 'WAN2' AND RuleType = 'INSECURE_PUBLIC') OR " ..
					  "(SNATInterface = 'WAN2' AND RuleType ='SECURE_INSECURE') OR " ..
					  "(SNATInterface = 'WAN2' AND RuleType ='PUBLIC_INSECURE'))")
	local protocalTable = db.getRowsWhere ("ProtocolBinding","LocalGatewayType = 2")
	local routeTable = db.getRowsWhere ("route","interfaceName = 'WAN2'")
	local vpnTable = db.getRowsWhere ("IpsecVPNPolicy","LocalGateway = 1")			

	local alertFlag = 0
	if(((profileTable ~= nil) and (#profileTable ~= 0)) 
		or ((fwTable ~= nil) and (#fwTable ~= 0)) 
		or ((protocalTable ~= nil) and (#protocalTable ~= 0)) 
		or ((routeTable ~= nil) and (#routeTable ~= 0)) 
		or ((vpnTable ~= nil) and (#vpnTable ~= 0)))then
		alertFlag = 1
	end
	local len = 0
	local tblArr = {}
	if(#profileTable ~= 0) then
		len = len+1
		tblArr[len] = "BandwidthProfiles"
	end
	if(#fwTable ~= 0) then
		len = len+1
		tblArr[len] = "Firewall"
	end
	if(#protocalTable ~= 0) then
		len = len+1
		tblArr[len] = "ProtocalBindings"
	end
	if(#routeTable ~= 0) then
		len = len+1
		tblArr[len] = "StaticRoutes"
	end
	if(#vpnTable ~= 0) then
		len = len+1
		tblArr[len] = "VPNProfiles"
	end
	
	local tables = tblArr[1] or ''
	if(len ~= 1) then
		for i=2,len do
			if(i ~= len) then
				tables = tables .. ", " .. tblArr[i]
			else
				tables = tables .. " and " .. tblArr[i]
			end
		end
	end	

	
	if((oldPort == 'WAN2') and (inputTable["ConfigPort.LogicalIfName"] ~= 'WAN2') and (alertFlag == 1)) then
		return "ERROR", "Please remove Optional WAN Configurations from " .. tables
	else
		return "OK", "STATUS_OK"
	end
end

-- WAN Config Function
function nimfView.wanConfigSet (inputTable, rowid, operation)
    logIfName = inputTable["NimfConf.LogicalIfName"] or 'WAN1'
    if (inputTable ["Wan.IspType"] == "5") then -- 3G Internet
        inputTable ["Wan.IspLoginRequired"] = "1"
        inputTable["Wan.IspType"] = "4"
    elseif (inputTable ["Wan.IspType"] == "4") then -- L2TP
        inputTable ["Wan.IspLoginRequired"] = "1"
        inputTable["Wan.IspType"] = "3"
    elseif (inputTable ["Wan.IspType"] == "3") then --PPTP
        inputTable ["Wan.IspLoginRequired"] = "1"
        inputTable["Wan.IspType"] = "2"
    elseif (inputTable ["Wan.IspType"] == "2") then --PPPoE
        inputTable ["Wan.IspLoginRequired"] = "1"
        inputTable["Wan.IspType"] = "1"
    elseif (inputTable ["Wan.IspType"] == "1") then --Static
        inputTable ["Wan.IspLoginRequired"] = "0"
        inputTable ["Wan.IsGetIpDyn"] = "0"
        inputTable ["Wan.IspType"] = nil
    else -- DHCP (Default)
        inputTable ["Wan.IspLoginRequired"] = "0"
        inputTable ["Wan.IsGetIpDyn"] = "1"
        inputTable ["Wan.IspType"] = nil
    end
    return nimfView.wanConfigSetWrap (inputTable, rowid, operation)
end

-- WAN ConfigGet Function
function nimfView.wanConfigurationGet (wanIfName)
    logIfName = wanIfName or 'WAN1'
    local outTable = nimfView.wanConfigGetWrap (logIfName)
    if (outTable["Wan.IspType"] == "1") then -- PPPOE
        outTable["Wan.IspType"] = "2"
    elseif (outTable["Wan.IspType"] == "2") then -- PPTP
        outTable["Wan.IspType"] = "3"
    elseif (outTable["Wan.IspType"] == "3") then -- L2TP
        outTable["Wan.IspType"] = "4"
    elseif (outTable["Wan.IspType"] == "4") then -- 3G Internet
        outTable["Wan.IspType"] = "5"
    elseif (outTable["Wan.IspLoginRequired"] == "0" and outTable["Wan.IsGetIpDyn"] == "0") then -- Static
        outTable["Wan.IspType"] = "1"
    elseif (outTable["Wan.IspLoginRequired"] == "0" and outTable["Wan.IsGetIpDyn"] == "1") then -- DHCP
        outTable["Wan.IspType"] = "0"
    end
    return outTable
end




