--[[
#### Hussain Vali.
#### TeamF1
#### www.TeamF1.com
#### July 24, 2008
#### File: portMgmt.lua
#### Description: Port Management functions
#### Revisions:
]]--


--************* Initial Code *************
--table packages
portMgmt = {}

--************* Functions *************
-- Port Management config
function portMgmt.config (inputTable, rowid, operation)
    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            return db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            return db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
            return false
        end
    end
end


-- portMgmt config
function portMgmt.portMgmtConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false
	
	local table = db.getTable ("PortMgmt")
	local i = 0
	local allDisabled = true
	for k,v in pairs(table) do
		i = i + 1
		local row = table[i]
        row["PortMgmt.PortStatus"] = inputTable["PortMgmt.PortStatus_" .. row["PortMgmt._ROWID_"]]
        if (row["PortMgmt.PortStatus"] == "1") then
			allDisabled = false
            row["PortMgmt.AutoNegEnable"] = inputTable["PortMgmt.AutoNegEnable_" .. row["PortMgmt._ROWID_"]]
            if (row["PortMgmt.AutoNegEnable"] == "0") then
                row["PortMgmt.DuplexMode"] = inputTable["PortMgmt.DuplexMode_" .. row["PortMgmt._ROWID_"]]
                row["PortMgmt.PortSpeed"] = inputTable["PortMgmt.PortSpeed_" .. row["PortMgmt._ROWID_"]]
            end
        end
        valid = portMgmt.config (row, row["PortMgmt._ROWID_"], operation)
        if (not valid) then
            break
        end
    end
	
	if (allDisabled) then
		db.rollback()
		-- Operation Failed - Router cannot be accessible if all ports are disabled.
		return "ERROR", "ALL_PORTS_DISABLED"
	end
	
	-- return
	if (valid) then
		db.commitTransaction(true)    
        return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "PORT_MGMT_CONFIG_FAILED"
	end
end

-- port Mirroring config
function portMgmt.portMirroringConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false
	
	valid = portMgmt.config (inputTable, rowid, operation)

	-- return
	if (valid) then
		db.commitTransaction(true)    
        return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "PORT_MIRRORING_CONFIG_FAILED"
	end
end

-- port config
function portMgmt.portMgmtConfigWrap (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

    db.beginTransaction() --begin transaction
	local valid = false

    local portConfig = db.getRow ("PortMgmt", "_ROWID_", rowid)
    if (portConfig ~= nil) then
        -- updating Link Speed
        portConfig["PortMgmt.AutoNegEnable"] = "0"
        if (inputTable["PortMgmt.LinkSpeed"] == "2") then
            portConfig["PortMgmt.DuplexMode"] = "0"
            portConfig["PortMgmt.PortSpeed"] = "10"
        elseif (inputTable["PortMgmt.LinkSpeed"] == "3") then
            portConfig["PortMgmt.DuplexMode"] = "1"
            portConfig["PortMgmt.PortSpeed"] = "10"
        elseif (inputTable["PortMgmt.LinkSpeed"] == "4") then
            portConfig["PortMgmt.DuplexMode"] = "0"
            portConfig["PortMgmt.PortSpeed"] = "100"
        elseif (inputTable["PortMgmt.LinkSpeed"] == "5") then
            portConfig["PortMgmt.DuplexMode"] = "1"
            portConfig["PortMgmt.PortSpeed"] = "100"
        elseif (inputTable["PortMgmt.LinkSpeed"] == "6") then
            portConfig["PortMgmt.DuplexMode"] = "1"
            portConfig["PortMgmt.PortSpeed"] = "1000"
        else
            portConfig["PortMgmt.AutoNegEnable"] = "1"
        end

        -- If LAN Ports, configure MAC Addess
        if (portConfig["PortMgmt.PortPhyID"] == "1" or
            portConfig["PortMgmt.PortPhyID"] == "2" or
            portConfig["PortMgmt.PortPhyID"] == "3" or
            portConfig["PortMgmt.PortPhyID"] == "4") then
            portConfig["PortMgmt.Mtu"] = inputTable["PortMgmt.Mtu"]
            portConfig["PortMgmt.SetMacAddress"] = inputTable["PortMgmt.SetMacAddress"]
            if (inputTable["PortMgmt.SetMacAddress"] == "0") then
                portConfig["PortMgmt.MacAddress"] = portConfig["PortMgmt.DefaultMacAddress"]
            else
                portConfig["PortMgmt.MacAddress"] = inputTable["PortMgmt.MacAddress"]
            end
        end

        valid = portMgmt.config (portConfig, rowid, operation)

        if (valid and
            (portConfig["PortMgmt.PortPhyID"] == "1" or
             portConfig["PortMgmt.PortPhyID"] == "2" or
             portConfig["PortMgmt.PortPhyID"] == "3" or
             portConfig["PortMgmt.PortPhyID"] == "4")) then
            -- Give a dummy update to vlan table
            local vlanConfig = db.getRow ("vlan", "portName", portConfig["PortMgmt.PortName"])
            if (vlanConfig ~= nil) then
                valid = db.setAttribute ("vlan", "portName", portConfig["PortMgmt.PortName"], "portName", portConfig["PortMgmt.PortName"])
            end
        end

        -- Updating MTU Size and MAC Address
        if (valid) then
            if (portConfig["PortMgmt.PortPhyID"] == "6") then --WAN Port
                local wanConfig = db.getRow ("NimfConf", "LogicalIfName", "WAN1")
                if (wanConfig ~= nil) then
                    wanConfig["NimfConf.UseDefaultMtu"] = "0"
                    wanConfig["NimfConf.MtuSize"] = inputTable["PortMgmt.Mtu"]
                    wanConfig["NimfConf.SetMacAddress"] = inputTable["PortMgmt.SetMacAddress"]
                    wanConfig["NimfConf.MacAddress"] = inputTable["PortMgmt.MacAddress"]
                    DBTable = "NimfConf"
                    valid = portMgmt.config (wanConfig, wanConfig["NimfConf._ROWID_"], operation)
                else
                    valid = false
                end
            end
        end
    end

	-- return
	if (valid) then
		db.commitTransaction(true)    
        return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "PORT_CONFIG_FAILED"
	end
end

function portMgmt.portMgmtConfigGetWrap (portPhyID)
    local outTable = db.getRow ("PortMgmt", "PortPhyID", portPhyID)
    if (outTable ~= nil) then
        -- Finding Interface Name
        if (outTable["PortMgmt.PortPhyID"] == "1") then
            outTable["PortMgmt.interfaceName"] = "LAN1"
        elseif (outTable["PortMgmt.PortPhyID"] == "2") then
            outTable["PortMgmt.interfaceName"] = "LAN2"
        elseif (outTable["PortMgmt.PortPhyID"] == "3") then
            outTable["PortMgmt.interfaceName"] = "LAN3"
        elseif (outTable["PortMgmt.PortPhyID"] == "4") then
            outTable["PortMgmt.interfaceName"] = "LAN4"
        elseif (outTable["PortMgmt.PortPhyID"] == "6") then
            outTable["PortMgmt.interfaceName"] = "WAN"
        end

        -- Finding Link Speed
        if (outTable["PortMgmt.AutoNegEnable"] == "1") then
            outTable["PortMgmt.LinkSpeed"] = "1"
        elseif (outTable["PortMgmt.PortSpeed"] == "10") then
            if (outTable["PortMgmt.DuplexMode"] == "0") then
                outTable["PortMgmt.LinkSpeed"] = "2"
            else
                outTable["PortMgmt.LinkSpeed"] = "3"
            end
        elseif (outTable["PortMgmt.PortSpeed"] == "100") then
            if (outTable["PortMgmt.DuplexMode"] == "0") then
                outTable["PortMgmt.LinkSpeed"] = "4"
            else
                outTable["PortMgmt.LinkSpeed"] = "5"
            end
        elseif (outTable["PortMgmt.PortSpeed"] == "1000") then
            if (outTable["PortMgmt.DuplexMode"] == "1") then
                outTable["PortMgmt.LinkSpeed"] = "6"
            end
        end

        -- Filling MTU Size and MAC Address Information
        if (outTable["PortMgmt.PortPhyID"] == "6") then
            local ifName = db.getAttribute ("networkInterface", "LogicalIfName", "WAN1", "interfaceName") or ''
            if (ifName ~= nil) then
                outTable["PortMgmt.DefaultMacAddress"] = db.getAttribute ("ethernet", "interfaceName", ifName, "macAddress") or ''
            end
            local wanConfig = db.getRow ("NimfConf", "LogicalIfName", "WAN1")
            if (wanConfig ~= nil) then
                outTable["PortMgmt.Mtu"] = wanConfig["NimfConf.MtuSize"] or ''
                outTable["PortMgmt.SetMacAddress"] = wanConfig["NimfConf.SetMacAddress"] or ''
                outTable["PortMgmt.MacAddress"] = wanConfig["NimfConf.MacAddress"] or ''
            end
        end
    end
    return outTable
end
