--[[
#### Vivek Dutt Sharma.
#### TeamF1
#### www.TeamF1.com
#### Dec 15, 2009
#### File: pppoeProfile.lua
#### Description: PPPoE profile functions
#### Revisions:
]]--

--************* Requires *************

--************* Initial Code *************

--package pppoeProfile
pppoeProfile = {}

--[[
/******************************************************************************
*
* pppoeProfile.profileConfig - manipulates data for configuration operations
*  RETURNS: ACCESS_DENIED or OK or ERROR
* 
*
*/
]]--

function pppoeProfile.profileConfig (inputTable,rowid, operation)
    local PPPOE_PROFILE_MAX_LIMIT = 2
    
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    local wanRowCount = db.getAttribute("PppoeProfile",'LogicalIfName',inputTable["PppoeProfile.LogicalIfName"],"count(*)") or '0'
    if(operation == "add" and tonumber(wanRowCount) >= PPPOE_PROFILE_MAX_LIMIT) then
        return "ERROR", "PPPOE_PROFILES_LIMIT_ERROR"
    end

    if(operation == "add") then
        if(db.existsRow("PppoeProfile","ProfileName",inputTable["PppoeProfile.ProfileName"])) then
            return "ERROR", "PROFILE_NAME_ALREADY_EXIST"            
        end
    end

    db.beginTransaction() --begin transaction
    local valid = false

    -- pppoe configuration  
    local pppoeTable={}       
    if(operation == "add") then
        local conId = 1             
        while(conId <=8 ) do
            if(db.existsRow("NimfConf","ConnectionId",conId)) then
                conId = conId+1
            else
                break;
            end
        end 
       
        --Adding in pppoe table
        DBTable = "PppoeProfile"
        valid = pppoeProfile.config(inputTable,rowid,operation)

        if (valid) then
            pppoeTable["Pppoe.LogicalIfName"] = inputTable["PppoeProfile.LogicalIfName"]
            pppoeTable["Pppoe.UserName"] = inputTable["PppoeProfile.UserName"]
            pppoeTable["Pppoe.Password"] = inputTable["PppoeProfile.Password"]
            pppoeTable["Pppoe.GetIpFromIsp"] = inputTable["PppoeProfile.GetIpFromIsp"]
            if(inputTable["PppoeProfile.GetIpFromIsp"] == "0") then
                pppoeTable["Pppoe.StaticIp"] = inputTable["PppoeProfile.StaticIp"]
                pppoeTable["Pppoe.NetMask"] = inputTable["PppoeProfile.NetMask"]
            end
            pppoeTable["Pppoe.GetDnsFromIsp"] = inputTable["PppoeProfile.GetDnsFromIsp"]
            if(inputTable["PppoeProfile.GetDnsFromIsp"] == "0") then
                pppoeTable["Pppoe.PrimaryDns"] = inputTable["PppoeProfile.PrimaryDns"]
                pppoeTable["Pppoe.SecondaryDns"] = inputTable["PppoeProfile.SecondaryDns"]
            end           
            pppoeTable["Pppoe.IdleTimeOutFlag"] = inputTable["PppoeProfile.IdleTimeOutFlag"]
            pppoeTable["Pppoe.IdleTimeOutValue"] = inputTable["PppoeProfile.IdleTimeOutValue"]
            pppoeTable["Pppoe.ProfileName"] = inputTable["PppoeProfile.ProfileName"]      
            pppoeTable["Pppoe.ConnectionId"] = conId
       
            DBTable = "Pppoe"
            valid = pppoeProfile.config(pppoeTable,rowid,operation)
        end

        -- Adding for MultiPppoe table
        if (valid) then
            local multiPPoETable={}        
            multiPPoETable["MultiPppoe.LogicalIfName"] = inputTable["PppoeProfile.LogicalIfName"]
            multiPPoETable["MultiPppoe.ConnectionId"] = conId
            
            DBTable = "MultiPppoe"    
            valid = pppoeProfile.config(multiPPoETable,rowid,operation)
        end

        -- NimfConf configuration             
        if(valid) then
            local nimfTable={}       
            local addressFamily = 2
            local connectiotype = "pppoe"
            local ifStatus = 0
            local requireLogin = 0
            local mtuSize = 1500
            local portSpeed = 1
            local setMacAddr = 0
            local useDefaultMtu = 1

            nimfTable["NimfConf.LogicalIfName"] = inputTable["PppoeProfile.LogicalIfName"]
            nimfTable["NimfConf.AddressFamily"] = addressFamily
            nimfTable["NimfConf.ConnectionType"] = connectiotype
            nimfTable["NimfConf.IfStatus"] = ifStatus
            nimfTable["NimfConf.RequireLogin"] = requireLogin
            nimfTable["NimfConf.MtuSize"] = mtuSize
            nimfTable["NimfConf.PortSpeed"] = portSpeed
            nimfTable["NimfConf.SetMacAddress"] = setMacAddr
            nimfTable["NimfConf.UseDefaultMtu"] = useDefaultMtu
            nimfTable["NimfConf.ConnectionId"] = conId
            
            DBTable = "NimfConf"
            valid = pppoeProfile.config (nimfTable,rowid,operation)
        end
    else  -- edit  
        -- checking if there is any other row with editing name
        DBTable = "PppoeProfile"
        local profRowId = rowid          
        if(db.existsRow("PppoeProfile","ProfileName",inputTable["PppoeProfile.ProfileName"])) then
            profRowId = db.getAttribute("PppoeProfile","ProfileName",inputTable["PppoeProfile.ProfileName"],"_ROWID_")
        end

        if( profRowId == rowid) then    
            valid = pppoeProfile.config(inputTable,rowid,operation)  
            pppoeTable["Pppoe.LogicalIfName"] = inputTable["PppoeProfile.LogicalIfName"]
            pppoeTable["Pppoe.UserName"] = inputTable["PppoeProfile.UserName"]
            pppoeTable["Pppoe.Password"] = inputTable["PppoeProfile.Password"]
            pppoeTable["Pppoe.GetIpFromIsp"] = inputTable["PppoeProfile.GetIpFromIsp"]
            if(inputTable["PppoeProfile.GetIpFromIsp"] == "0") then
                pppoeTable["Pppoe.StaticIp"] = inputTable["PppoeProfile.StaticIp"]
                pppoeTable["Pppoe.NetMask"] = inputTable["PppoeProfile.NetMask"]
            end
            pppoeTable["Pppoe.GetDnsFromIsp"] = inputTable["PppoeProfile.GetDnsFromIsp"]
            if(inputTable["PppoeProfile.GetDnsFromIsp"] == "0") then
                pppoeTable["Pppoe.PrimaryDns"] = inputTable["PppoeProfile.PrimaryDns"]
                pppoeTable["Pppoe.SecondaryDns"] = inputTable["PppoeProfile.SecondaryDns"]
            end           
            pppoeTable["Pppoe.IdleTimeOutFlag"] = inputTable["PppoeProfile.IdleTimeOutFlag"]
            pppoeTable["Pppoe.IdleTimeOutValue"] = inputTable["PppoeProfile.IdleTimeOutValue"]
            pppoeTable["Pppoe.ProfileName"] = inputTable["PppoeProfile.ProfileName"]             

            DBTable="Pppoe"
            valid = pppoeProfile.config(pppoeTable,rowid,operation)
        else
            db.rollback()
            return "ERROR", "PROFILE_NAME_ALREADY_EXIST"
        end
     end
    
    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"            
    else
        db.rollback()
        return "ERROR", "PPPOE_PROFILE_CONFIG_FAILED"
    end
end

--[[
/******************************************************************************
*
*  pppoeProfile.config - handles all data base manipulation operations.
*
* RETURNS: true or false
*/
]]--

function pppoeProfile.config (inputTable, rowid, operation)
    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            return db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            return db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
            return db.delete(DBTable, inputTable)
        end
    end
    return false
end


--[[
/******************************************************************************
*
* pppoeProfile.modifyProfileStatus - updates configurations from pppoe profiles
*
* This routine updates data base for the following configurations.
* 1. Enables profiles
* 2. Disables profiles
* 3. Delete profiles
* 
* RETURNS: ACCESS_DENIED or OK or ERROR
*/
]]--

function pppoeProfile.modifyProfileStatus (rowIds, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"
    if (operation == "enable") then
        for k,v in pairs(rowIds) do
            local logicalName = db.getAttribute("PppoeProfile", "_ROWID_", v, "LogicalIfName") or ''
            local where = "LogicalIfName ='" .. logicalName .."' and AddressFamily = 2 and ConnectionId = 0"
            local conRow = db.getRowWhere("NimfConf",where)
            local conType =  conRow["NimfConf.ConnectionType"]
            if (conType ~= "pppoe") then
                db.rollback()
        		return "ERROR", "SELECT_PPPOE"
            end
            local pfTable = {}
            local nimfTable = {}
	        local enabled="1"
            pfTable["PppoeProfile.Status"] = enabled;
            nimfTable["NimfConf.IfStatus"] = enabled;
            DBTable="PppoeProfile"            
            valid = pppoeProfile.config(pfTable, v, "edit")
            
            --Getting profile Name
            local profileName = db.getAttribute("PppoeProfile","_ROWID_",v,"ProfileName")
            
            if (valid == false) then
                errStr = "ENABLE_OP_FAILED"
                break
            end
            if(valid) then
                local connectionId = db.getAttribute("Pppoe","ProfileName",profileName,"ConnectionId")
                local nimfRow = db.getAttribute("NimfConf","ConnectionId",connectionId,"_ROWID_") 
                DBTable="NimfConf"               
                valid = pppoeProfile.config (nimfTable,nimfRow,"edit")
               end
 
        end
    elseif (operation == "disable") then
        -- disabling profiles.
        for k,v in pairs(rowIds) do
            local pfTable = {}
            local nimfTable = {}
            local disabled = "0"
            pfTable["PppoeProfile.Status"] = disabled;
            nimfTable["NimfConf.IfStatus"] = disabled;
            DBTable="PppoeProfile" 
            valid = pppoeProfile.config(pfTable, v, "edit")
            local profileName = db.getAttribute("PppoeProfile","_ROWID_",v,"ProfileName")
            if (valid == false) then
                errStr = "DISABLE_OP_FAILED"
                break
            end
            if(valid) then
                local connectionId = db.getAttribute("Pppoe","ProfileName",profileName,"ConnectionId")
                local nimfRow = db.getAttribute("NimfConf","ConnectionId",connectionId,"_ROWID_") 
                DBTable = "NimfConf"                
                valid = pppoeProfile.config (nimfTable,nimfRow,"edit")
               end
            if (valid == false) then
                errStr = "DISABLE_OP_FAILED"
                break
            end
           
        end
    else 
        local profileName
        local pppoeRowId
        local enabled = "1"  
        for k,v in pairs(rowIds) do
            profileName = db.getAttribute("PppoeProfile","_ROWID_",v,"ProfileName")
            pppoeRowId = db.getAttribute("Pppoe","ProfileName",profileName,"_ROWID_")  
            local connectionId = db.getAttribute("Pppoe","ProfileName",profileName,"ConnectionId")
            local nimfRow = db.getAttribute("NimfConf","ConnectionId",connectionId,"_ROWID_")
            local nimfRowStatus = db.getAttribute("NimfConf","_ROWID_",nimfRow,"IfStatus")
            if( nimfRowStatus == enabled ) then
                db.rollback()
                return "ERROR", "DISABLE_SELECTED_PROFILE_BEFORE_DELETE"
            end
        end

        for k,v in pairs(rowIds) do                         
            local pfTable = {}
            local nimfTable = {}
            local pppoeTable = {}
            local multiPPPOETable = {}           
            
            profileName = db.getAttribute("PppoeProfile","_ROWID_",v,"ProfileName")
            pppoeRowId = db.getAttribute("Pppoe","ProfileName",profileName,"_ROWID_")           
            local connectionId = db.getAttribute("Pppoe","ProfileName",profileName,"ConnectionId")
            local nimfRow = db.getAttribute("NimfConf","ConnectionId",connectionId,"_ROWID_")
            local nimfRowStatus = db.getAttribute("NimfConf","_ROWID_",nimfRow,"IfStatus")
            local mutiPppoeRow = db.getAttribute("MultiPppoe","ConnectionId",connectionId,"_ROWID_")
                            
            -- Deleting profiles begins 
            local valid = true                       
            pfTable["PppoeProfile._ROWID_"] = v
            pppoeTable["Pppoe._ROWID_"] = pppoeRowId
            multiPPPOETable["MultiPppoe._ROWID_"] = mutiPppoeRow 
            nimfTable["NimfConf._ROWID_"] = nimfRow                
            DBTable = "Pppoe"
            valid = pppoeProfile.config (pppoeTable,"-1","delete")
            if(valid) then
                DBTable = "PppoeProfile"
                valid = pppoeProfile.config(pfTable,"-1","delete")
            end
            if(valid) then
                DBTable = "MultiPppoe"
                valid = pppoeProfile.config (multiPPPOETable,"-1","delete")
            end
            if(valid) then
                DBTable = "NimfConf" 
                valid = pppoeProfile.config (nimfTable,"-1","delete")
            end
            if (valid == false) then
                errStr = "PROF_DELETE_FAILED"
                break
            end              
        end        
    end
    --   Deleting profiles end
      
    -- return "OK", if succeed all updates
    -- or
    -- return "ERROR", if update failed and also Roolback the database.    
    if (valid) then
        db.commitTransaction(true)
        return "OK", errStr
    else
        db.rollback()
        return "ERROR", errStr
    end
end
