-- remoteMgmtCLI.lua
-- Bhanu R
-- TeamF1
-- www.TeamF1.com

-- Modification History
-- 27dec08,ss  did modifications for rmi
-- 27sep08,rbh written.
--
-- Description
-- CLI interface set and get routines

require "teamf1lualib/firewall"
require "teamf1lualib/sslvpn"

--
-- This routine validates the accessMgmt configuration information.
function accessMgmtInputVal(configRow)
    if (configRow["accessMgmt.accessMgmtEnable"] == "0" ) then
        return true
    end
    if (tonumber(configRow["accessMgmt.accessType"]) > 2 ) then
        printCLIError ("Please enter a choice 0, 1 or 2")
        return false
    end
    if (tonumber(configRow["accessMgmt.accessType"]) > 0 ) then
        if (configRow["accessMgmt.accessMgmtIP1"] == nil) then
            printCLIError ("Invalid accessMgmtIP1 address")
            return false
        end
    end
    if (configRow["accessMgmt.accessType"] == "1" ) then
        if (configRow["accessMgmt.accessMgmtIP2"] == nil) then
            printCLIError ("Invalid Ending IP address")
            return false
        end
    end
    if (configRow["accessMgmt.serviceType"] == "0" ) then
        if ((tonumber(configRow["accessMgmt.port"]) <= 1024 ) and 
            (configRow["accessMgmt.port"] ~= "443") ) then
            printCLIError ("Port already in use")
            return false
        end
    end
    return true
end


--
-- This routine reads in the current remote https configuration information
-- into a configuration object.
function accessMgmtHttpsInit(args)
    local rowId = -1

    local configRow = db.getRow("accessMgmt", "servicetype", "0")
    if (configRow == nil) then
       return rowId, {}
    end
    
    rowId = configRow["accessMgmt._ROWID_"]   
    return rowId, configRow
end

--
-- This routine reads in the current telnet https configuration information
-- into a configuration object.
function accessMgmtTelnetInit(args)
    local rowId = -1

    local configRow = db.getRow("accessMgmt", "serviceType", "1")
    if (configRow == nil) then
       return rowId, {}
    end
    
    rowId = configRow["accessMgmt._ROWID_"]   
    return rowId, configRow
end

--
-- This routine implements the CLI command to show remote mgmt
-- configuration
function  remoteMgmtGet()
    local accessMgmtTbl = nil
    local serviceType = {}
    serviceType ["https"] = 0
    --serviceType ["telnet"] = 1

    for k,v in pairs(serviceType) do

        printLabel ("Remote Mgmt Configuration for " .. k )    
        accessMgmtTbl = db.getRow ("accessMgmt","serviceType", serviceType[k])
        if (accessMgmtTbl ~= nil) then
            if (accessMgmtTbl["accessMgmt.accessMgmtEnable"] == "0") then
                print ("Remote access through " .. k .. " is currently disabled")
            else
                if (accessMgmtTbl["accessMgmt.accessType"] == "0" ) then
                    print ("Access granted to everyone")
                end

                if (accessMgmtTbl["accessMgmt.accessType"] == "1" ) then
                    print ("Access granted to a range of IPs from :" .. accessMgmtTbl["accessMgmt.accessMgmtIP1"] .. " to ".. accessMgmtTbl["accessMgmt.accessMgmtIP2"])
                end

                if (accessMgmtTbl["accessMgmt.accessType"] == "2" ) then
                    print ("Access granted to the IP " .. accessMgmtTbl["accessMgmt.accessMgmtIP1"])
                end
                print ("port being used : " .. accessMgmtTbl["accessMgmt.port"])
            end
        end
    end
end

--
-- This routine saves the modified remote management configuration information.
function accessMgmtCfgSave(configRow)
    local errorFlag = nil
    local statusCode = ""
    local statusMessage = ""

    --[[errorFlag, statusCode = remoteMgmt.config (configRow)

    -- save db if no error
    if (errorFlag == "OK") then 
        db.save()
    end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage]]--
    configRow["httpsMgmt.httpsMgmtPort"] = configRow["accessMgmt.port"]
    errorFlag, statusMessage = httpsMgmt.config(configRow, "1", "edit")
    if (errorFlag == "OK") then
        errorFlag, statusMessage = fwAccessMgmt.config(configRow, "0", "edit")
    end
    if (errorFlag == "OK") then
       local rmtSnmpConfig = db.getRow ("RemoteSnmp", "_ROWID_", "1")
       if (rmtSnmpConfig == nil) then
           errorFlag, statusMessage = fwAccessMgmt.remoteSNMPConfig (configRow, "-1", "add")
       else
           errorFlag, statusMessage = fwAccessMgmt.remoteSNMPConfig (configRow, "1", "edit")
       end
    end
    if (errorFlag == "OK") then db.save() end             
    statusMessagee= db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
end

