-- schedulesCLI.lua
-- Samer Sayeed
-- TeamF1
-- www.TeamF1.com
--
-- Modification History
-- 17feb09,ss  added check in validation routine for fixing SPR# 10807 
-- 07jan09,ss  written 
--
-- Description
-- CLI interface set and get routines

require "teamf1lualib/fwView"

DBTable = "Schedules"
-- 
function schedulesCfgInit(args)
    -- for add
    if (args[1] == nil) then
        configRow = db.getDefaults(true, "Schedules")
        configRow["Schedules._ROWID_"] = "-1"
    -- for edit
    else
        configRow = db.getRow("Schedules", "Schedules._ROWID_", args[1])
    end
    if(configRow == nil) then
        print("Row(s) doesnot exist\n")
        return -1, {}
    else
        configRow = fwView.scheduleConfigGet (configRow)
        rowid = configRow["Schedules._ROWID_"]
    end
    return rowid, configRow
end

--
function schedulesCfgSave(configRow)
    RowId = configRow["Schedules._ROWID_"]
    --if add
    if (RowId == "-1") then
        errorFlag, statusCode = fwView.schedulesConfig(configRow, RowId, "add")
    --if edit
    else
        errorFlag, statusCode = fwView.schedulesConfig(configRow, RowId, "edit")
    end

    print(RowId, errorFlag, statusCode)
    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, 
                                    LANGUAGE)
    return errorFlag, statusMessage
end

--
function schedulesCfgDel(configRow)
    rows = {}
    rows["rowid"] = configRow["Schedules._ROWID_"]
    -- if any rows to delete
    if (rows) then
        errorFlag, statusCode = fwView.deleteSchedules (rows)
    end
    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage
end

--
function schedulesCfgInputVal(configRow)
    if (configRow["Schedules.ScheduleName"] == "") then
        printCLIError ("Enter unique schedule name\n")
        return false
    end
    if (configRow["Schedules.allDays"] == "1") then 
        if ((configRow["Schedules.monday"] == "1") or
            (configRow["Schedules.tuesday"] == "1") or
            (configRow["Schedules.wednesday"] == "1") or
            (configRow["Schedules.thursday"] == "1") or
            (configRow["Schedules.friday"] == "1") or
            (configRow["Schedules.saturday"] == "1") or
            (configRow["Schedules.sunday"] == "1")) then
            printCLIError ("days all: Disable this field when specific" .. 
                           " day is selected\n")
            return false
        end
    end
    if ((configRow["Schedules.allDays"] == "0") and 
        (configRow["Schedules.monday"] == "0") and 
        (configRow["Schedules.tuesday"] == "0") and 
        (configRow["Schedules.wednesday"] == "0") and 
        (configRow["Schedules.thursday"] == "0") and 
        (configRow["Schedules.friday"] == "0") and 
        (configRow["Schedules.saturday"] == "0") and 
        (configRow["Schedules.sunday"] == "0")) then
            printCLIError ("Select valid day of the week\n")
            return false
    end
    if (configRow["Schedules.AllDay"] == "0") then
        if(configRow["Schedules.startTimeMin"] == nil or 
           configRow["Schedules.startTimeHour"] == nil or 
           configRow["Schedules.startTimeMeridian"] == nil) then
            printCLIError ("Enter valid start time (hours, mins, AM/PM)\n")
        return false
        end
        if (configRow["Schedules.endTimeMin"] == nil or 
            configRow["Schedules.endTimeHour"] == nil or 
            configRow["Schedules.endTimeMeridian"] == nil) then
        printCLIError ("Enter valid end time (hours, mins, AM/PM)\n")
        return false
        end
    end
    if (configRow["Schedules.StartTimeHours"] == "" and 
        configRow["Schedules.StartTimeMins"] == "") then
        printCLIError ("Enter valid start time \n")
        return false
    end
    if (configRow["Schedules.EndTimeHours"] == "" and 
        configRow["Schedules.EndTimeMins"] == "") then
        printCLIError ("Enter valid end time \n")
        return false
    end
    return true
end

function schedulesSetupGet(args)
    local resultTab = {}
    printLabel("Schedules")
    print("List of Available Schedules\n")
    local i = 0
    local table = db.getTable("Schedules")
    for k,v in pairs(table) do
        i = i + 1
        local row = fwView.scheduleConfigGet (table[i])
        local days = ""
        if (row["Schedules.allDays"] == "1") then
            days = "All Days"
        else	
        if (row["Schedules.sunday"] == "1") then
            days = days.."Sunday" .. ", "
        end
        if (row["Schedules.monday"] == "1") then
            days = days.."Monday" .. ", "
        end
        if (row["Schedules.tuesday"] == "1") then
            days = days.."Tuesday" .. ", "
        end
        if (row["Schedules.wednesday"] == "1") then
            days = days.."Wednesday" .. ", "
        end
        if (row["Schedules.thursday"] == "1") then
            days = days.."Thursday" .. ", "
        end
        if (row["Schedules.friday"] == "1") then
            days = days.."Friday" .. ", "
        end
        if (row["Schedules.saturday"] == "1") then
            days = days.."Saturday" .. ", "
        end
        days = days:sub(1, days:len()-2) --shave last comma
        end
    local startTime = row["Schedules.startTimeHour"] .. ":" .. 
                      row["Schedules.startTimeMin"] .. " " .. 
                      row["Schedules.startTimeMeridian"]
    local endTime = row["Schedules.endTimeHour"] .. ":" .. 
                    row["Schedules.endTimeMin"]  .. " " .. 
                    row["Schedules.endTimeMeridian"]
    resTab.insertField (resultTab, "ROW ID", row["Schedules._ROWID_"])
    resTab.insertField (resultTab, "Name", row["Schedules.ScheduleName"])
    resTab.insertField (resultTab, "Days", days)
    resTab.insertField (resultTab, "Start Time", startTime)
    resTab.insertField (resultTab, "End Time", endTime)
    end -- for
    resTab.print (resultTab, 0)
end
