-- snmpCLI.lua
-- Gaurav Mathur
-- TeamF1
-- www.TeamF1.com

-- Modification History
-- 05jan10,db  Added snmp Access control routines and
--         modified snmp trap routines
-- 28nov07,gnm written
--
-- Description
-- CLISH SNMP LUA routines

-- load required library
require "teamf1lualib/net-snmp"

--
-- SNMP input validation routine
function snmpSysConfigInputVal (args)
    return TRUE
end

--
-- This routine save user password configuration information
function snmpSysConfigSave (configRow)
    local errorFlag = "ERROR" -- default 
    local statusCode = ""
    local statusMessage = ""

    if (configRow["system.sysContact"] == nil) then
       statusMessage = "Please enter valid system contact value"
       return errorFlag, statusMessage
    elseif (configRow["system.name"] == nil) then	   
       statusMessage = "Please enter valid system name"
       return errorFlag, statusMessage
    elseif (configRow["system.sysLoc"] == nil) then
       statusMessage = "Please enter valid system location"
       return errorFlag, statusMessage
    end
	
    -- Write configuration changes to the database
    errorFlag, statusCode = snmp.system_config (configRow, "1", "edit")
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage
end

--
-- This routine takes in one argument that is the value of the
-- primary key for the object table.
function snmpSysConfigInit (args)
    local primKey = args[1] -- Agent server IP address in the snmpTrap table

    local configRow = db.getRow ("system","system._ROWID_", 1)
    if configRow == nil then
       configRow = {} -- Create an empty table
    end
           	      
    -- for k, v in pairs(configRow) do print (k, type(v), v) end
    return rowId, configRow
end

--
-- SNMP input validation routine
function snmpTrapConfigInputVal (args)
    return TRUE
end

--
-- This routine save user password configuration information
function snmpTrapConfigDel (configRow)
    local errorFlag = "ERROR" -- default 
    local statusCode = "SNMP_CONFIG_FAILED"
    local statusMessage = ""
    local rows = {}
    
    table.insert (rows, configRow["snmpTrap._ROWID_"])
    if (rows) then
        errorFlag, statusCode = snmp.snmpTrap_config(rows, "-1", "delete")
    end
    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage
end

--
-- This routine save user password configuration information
function snmpTrapConfigSave (configRow)
    local errorFlag = "ERROR" -- default 
    local statusCode = ""
    local statusMessage = ""

    -- Input validation
    if (configRow["snmpTrap.snmpVersion"] == nil) then
        statusMessage = "Invalid snmp version."
        return errorFlag, statusMessage
    elseif (configRow["snmpTrap.ipAddr"] == nil) then
        statusMessage = "Invalid IP address."
        return errorFlag, statusMessage
    elseif (configRow["snmpTrap.port"] == nil) then
	statusMessage = "Invalid port number."
        return errorFlag, statusMessage
    elseif (configRow["snmpTrap.commName"] == nil) then
        statusMessage = "Please enter a valid SNMP community string."
	return errorFlag, statusMessage
    end

    -- Check if the row id field exists. If it does not then we  adding a
    -- new row.
    if (configRow["snmpTrap._ROWID_"] == nil) then
        errorFlag, statusCode = snmp.snmpTrap_config (configRow, "-1", "add")
	-- Update the RowId
	configRow["snmpTrap._ROWID_"] = db.getAttribute("snmpTrap", 
				      "snmpTrap.ipAddr", configRow["snmpTrap.ipAddr"],
				       "_ROWID_")
    else
        errorFlag, statusCode = snmp.snmpTrap_config (configRow, 
		   configRow["snmpTrap._ROWID_"], "edit")
    end

    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)    

    return errorFlag, statusMessage
end

--
-- This routine takes in one argument that is the value of the
-- primary key for the object table.
function snmpTrapConfigInit (args)
    local primKey = args[1] -- Agent server IP address in the snmpTrap table

    local configRow = db.getRow("snmpTrap", "snmpTrap.ipAddr", primKey)
    if configRow == nil then
       configRow = {} -- Create an empty table
       configRow["snmpTrap.ipAddr"] = primKey
    end
           	      
    -- for k, v in pairs(configRow) do print (k, type(v), v) end
    return rowId, configRow
end

--
-- SNMP input validation routine access Control
function snmpACLConfigInputVal (args)
    return TRUE
end

--
-- This routine save user password configuration information
function snmpACLConfigDel (configRow)
    local errorFlag = "ERROR" -- default 
    local statusCode = "SNMP_CONFIG_FAILED"
    local statusMessage = ""
    local rows = {}
    
    table.insert (rows, configRow["snmpAccessControl._ROWID_"])
    if (rows) then
        errorFlag, statusCode = snmp.snmpAccessControl_config(rows, "-1", "delete")
    end
    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage
end

--
-- This routine save user password configuration information
function snmpACLConfigSave (configRow)
    local errorFlag = "ERROR" -- default 
    local statusCode = ""
    local statusMessage = ""

    -- Input validation
    if (configRow["snmpAccessControl.subnetMask"] == nil) then
        statusMessage = "Invalid subnet mask."
        return errorFlag, statusMessage
    elseif (configRow["snmpAccessControl.ipAddr"] == nil) then
        statusMessage = "Invalid IP address."
        return errorFlag, statusMessage
    elseif (configRow["snmpAccessControl.accessType"] == nil) then
	statusMessage = "Invalid port number."
        return errorFlag, statusMessage
    elseif (configRow["snmpAccessControl.commName"] == nil) then
        statusMessage = "Please enter a valid SNMP community string."
	return errorFlag, statusMessage
    end

    -- Check if the row id field exists. If it does not then we  adding a
    -- new row.
    if (configRow["snmpAccessControl._ROWID_"] == "-1") then
        errorFlag, statusCode = snmp.snmpAccessControl_config (configRow, "-1", "add")
    else
        errorFlag, statusCode = snmp.snmpAccessControl_config (configRow, 
		   configRow["snmpAccessControl._ROWID_"], "edit")
    end

    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)    

    return errorFlag, statusMessage
end

--
-- This routine takes in one argument that is the value of the
-- primary key for the object table.
function snmpACLConfigInit (args)
-- for add
    local configRow = {}
    if (args[1] == "-1") then
        configRow ["snmpAccessControl._ROWID_"] = args[1]
        -- for edit
    else
        configRow = db.getRow("snmpAccessControl", "snmpAccessControl._ROWID_", args[1])
    end
    return configRow ["snmpAccessControl._ROWID_"], configRow
end
