-- sourceMacFilteringCLI.lua
-- Samer Sayeed
-- TeamF1
-- www.TeamF1.com
--
-- Modification History
-- 09jan09,ss written 
--
-- Description
-- CLI interface set and get routines

require "teamf1lualib/fwView"

function sourceMacFilteringCfgInit (args)
    -- for add
    if (args[1] == nil) then
        configRow = db.getDefaults(true, "SourceMACAddress")
        configRow["SourceMACAddress._ROWID_"] = "-1"
        configRow["SourceMACAddress.OID"] = "10"
    -- for edit
    else
        configRow = db.getRow("SourceMACAddress", "SourceMACAddress._ROWID_", args[1])
    end
    if(configRow == nil)then
        print("Deleted row\n")
        return -1, {}
    end
    return configRow["SourceMACAddress._ROWID_"], configRow
end

function sourceMacFilteringCfgSave (configRow)
    DBTable = "SourceMACAddress"
    -- for add
    RowId = configRow ["SourceMACAddress._ROWID_"]
    if (RowId == "-1") then
    errorFlag, statusCode = fwView.macFilteringConfig(configRow, RowId, "add")
    else
    errorFlag, statusCode = fwView.macFilteringConfig(configRow, RowId, "edit")
    end
    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage
end

function sourceMacFilteringCfgDel (configRow)
    DBTable = "SourceMACAddress"
    rows = {}
    rows["rowid"] = configRow["SourceMACAddress._ROWID_"]
    if (rows) then
    errorFlag, statusCode = fwView.deleteMac (rows)
    end
    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage
end

function sourceMacFilteringCfgInputVal (configRow)
    if (configRow["SourceMACAddress.SourceMACAddress"] == "") then
        printCLIError ("Enter valid mac address\n")
        return false
    end
    return true
end

function sourceMacFilteringCfgGet (args)
    local resultTab = {}
    local resultTable = {}
    local row = db.getRow ("MACFiltering", "_ROWID_", "1")
    if (row["MACFiltering.MACFilterEnable"] == "1") then
        resTab.insertField (resultTab, "MAC Filtering", "Enabled")
    else
        resTab.insertField (resultTab, "MAC Filtering", "Disabled")
    end
    if (row["MACFiltering.MACAddressPolicy"] == "0") then
        resTab.insertField (resultTab, "Policy for MAC Addresses", "Block and Permit the rest")
    elseif (row["MACFiltering.MACAddressPolicy"] == "1") then
        resTab.insertField (resultTab, "Policy for MAC Addresses", "Permit and Block the rest")
    else
        resTab.insertField (resultTab, "Policy for MAC Addresses", "Unknown")
    end
    local table = db.getTable ("SourceMACAddress")
    local i = 0
    for col, val in pairs(table) do
    i = i+1
    local smRow = table[i]
    resTab.insertField (resultTable, "ROW ID", smRow["SourceMACAddress._ROWID_"] or "") 
    resTab.insertField (resultTable, "MAC Address", smRow["SourceMACAddress.SourceMACAddress"] or "") 
    end
    printLabel("Source MAC Filter\n")
    resTab.print (resultTab, 0)
    printLabel("List of Available MAC Addresses\n")
    resTab.print (resultTable, 0)
end

function macFilteringCfgInit (args)
    configRow = db.getRow("MACFiltering","_ROWID_", "1")
    return 1, configRow
end

function macFilteringCfgSave (configRow)
    DBTable = "MACFiltering"
    errorFlag, statusCode = fwView.macFilteringConfig(configRow, "1", "edit")
    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage
end

function ipMacBindingCfgInit (args)
    -- for Add
    if (args[1] == nil) then
    configRow = db.getDefaults(true, "IPMACBinding")
    configRow["IPMACBinding._ROWID_"] = "-1"
    configRow["IPMACBinding.OID"] = "10"
    -- for edit
    else
    configRow = db.getRow("IPMACBinding", "IPMACBinding._ROWID_", args[1])
    end
    if(configRow == nil)then
        return -1, {}
    end
    return configRow["IPMACBinding._ROWID_"], configRow
end

function ipMacBindingCfgSave (configRow)
    DBTable = "IPMACBinding"
    --configRow = db.getRow("IPMACEmailLogsConfig","_ROWID_", "1")
    if(configRow["IPMACBinding._ROWID_"] == "-1") then
        errorFlag, statusCode = fwView.ipMacConfig(configRow, "-1", "add")
    else
        errorFlag, statusCode = fwView.ipMacConfig(configRow, configRow["IPMACBinding._ROWID_"], "edit")
    end
	
    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage
end

function ipMacBindingCfgDel (configRow)
    DBTable = "IPMACBinding"
    rows = {}
    rows["rowid"] = configRow["IPMACBinding._ROWID_"]
    -- if any rows to delete
    if (rows) then
        errorFlag, statusCode = fwView.deleteIpMac (rows)
    end
    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage
end

function ipMacBindingCfgInputVal (configRow)
    if (configRow["IPMACBinding.Name"] == "") then
        printCLIError ("Enter valid ip/mac binding rule name\n")
        return false
    end
    if (configRow["IPMACBinding.SourceMACAddress"] == "") then
        printCLIError ("Enter valid ip/mac binding source mac address\n")
        return false
    end
    if (configRow["IPMACBinding.IPAddress"] == "") then
        printCLIError ("Enter valid ip/mac binding ip address\n")
        return false
    end
    if (configRow["IPMACBinding.LogStatus"] == "") then
        printCLIError ("Enter valid ip/mac binding logging status\n")
        return false
    end
    return true
end

function ipMacBindingCfgGet (args)
local resultTab = {}
printLabel ("IP/MAC Binding")
print ("List of Available IP/MAC Bindings\n")
local table = db.getTable ("IPMACBinding")
    local i = 0
    for col, val in pairs(table) do
    i = i+1
    local smRow = table[i]
    resTab.insertField (resultTab, "ROW ID", smRow["IPMACBinding._ROWID_"] or "") 
    resTab.insertField (resultTab, "Name", smRow["IPMACBinding.Name"] or "") 
    resTab.insertField (resultTab, "MAC Address", smRow["IPMACBinding.SourceMACAddress"] or "") 
    resTab.insertField (resultTab, "IP Address", smRow["IPMACBinding.IPAddress"] or "") 
    if (smRow["IPMACBinding.LogStatus"] == "1") then 
    resTab.insertField (resultTab,"Log Dropped Packets", "Enabled")
    else 
    resTab.insertField (resultTab,"Log Dropped Packets", "Disabled") 
    end
    end
    resTab.print (resultTab, 0)
end

