--[[
#### Hussain Vali.
#### TeamF1
#### www.TeamF1.com
#### June 10, 2008
#### File: sslvpn.lua
#### Description: SSL VPN functions
#### Revisions:
]]--

--************* Requires *************
require "teamf1lualib/db"
require "sslvpnLuaLib"

--************* Initial Code *************

--package Admin Configurations
sslvpn = {}
httpsMgmt = {}

--************* Functions *************
-- Portal Layout Configuration
function sslvpn.portalLayoutConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false

    -- if add , prepare Portal Url
    if (operation == "add") then
        local wanIp = db.getAttribute("networkInterface", "LogicalIfName", "WAN1", "ipaddr")
        inputTable["SSLVPNPortalLayout.PortalURL"] = "https://" .. wanIp .. "/portal/" .. inputTable["SSLVPNPortalLayout.PortalLayoutName"]
    end
    
    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            valid = db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            valid = db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
            if (inputTable["SSLVPNPortalLayout.UseCount"] > "0") then
                valid = false       
            else
                valid = db.delete(DBTable, inputTable)
            end
        end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "SSLVPN_PORTAL_LAYOUT_CONFIG_FAILED"
    end
end

-- Function for Deleting Portal Layout(s)  
function sslvpn.deleteLayouts (rowIds)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false

    -- delete routes
    for k,v in pairs(rowIds) do
        local inTable = {}
        inTable["SSLVPNPortalLayout._ROWID_"] = v;
        local row = db.getRow("SSLVPNPortalLayout", "_ROWID_", v)
        if (row["SSLVPNPortalLayout.UseCount"] > "0") then
            valid = false
        else
            valid = db.delete("SSLVPNPortalLayout", inTable) 
        end
        if(valid==false) then
            break;
        end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "SSLVPN_PORTAL_LAYOUT_DELETE_SUCCESS"
    else
        db.rollback()
        return "ERROR", "SSLVPN_PORTAL_LAYOUT_HAS_DOMAIN_DEL_FAILED"
    end
end

-- Function for setting default Layout
function sslvpn.setDefault (rowId)
     -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false
    
    -- checking for default row exists
    if( db.existsRow ("SSLVPNPortalLayout", "DefaultPortal", "2")) then
        local defRow = db.getRow("SSLVPNPortalLayout", "DefaultPortal", "2")
        defRow["SSLVPNPortalLayout.DefaultPortal"] = "0"
        valid = db.update("SSLVPNPortalLayout", defRow, defRow["SSLVPNPortalLayout._ROWID_"])
    end
    
    -- setting default Layout
    local row = db.getRow("SSLVPNPortalLayout", "_ROWID_", rowId)
    if(row["SSLVPNPortalLayout.UseCount"] == "0") then
        valid = false
    else
        if (row["SSLVPNPortalLayout.DefaultPortal"] ~= "1") then
            row["SSLVPNPortalLayout.DefaultPortal"] = "2"
        end
        valid = db.update("SSLVPNPortalLayout", row, rowId)
    end
    

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "SSLVPN_PORTAL_LAYOUT_NO_DOMAIN_SET_AS_DEFAULT_FAILED"
    end
end

-- Port Forwarding Config
function sslvpn.portForwardConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false

    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            valid = db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            valid = db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
            valid = db.delete(DBTable, inputTable)
        end
    end
    
    -- return
    if (valid) then
        db.commitTransaction(true)
            return "OK", "STATUS_OK"            
    else
        db.rollback()
        return "ERROR", "SSLVPN_PORTFORWARDING_CONFIG_FAILED"
    end
end

-- Resources Config
function sslvpn.resourceDeleteConfig (inputTable, rowid, operation)
    -- validate
    if (operation == "delete") then
            return db.delete(DBTable, inputTable)
    end    
    return false
end

function sslvpn.resourceConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false

    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            valid = db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            valid = db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
            valid = db.delete(DBTable, inputTable)
        end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"            
    else
        db.rollback()
        return "ERROR", "SSLVPN_RESOURCE_CONFIG_FAILED"
    end
end

-- resources Delete
function sslvpn.deleteResources (rowIds)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"

    -- Check for Using Services
    for k,v in pairs(rowIds) do
        local l = 0
        local name = db.getAttribute("SSLVPNResource", "_ROWID_", v, "ResourceName")
        local count = db.getAttribute("SSLVPNResource", "_ROWID_", v, "UseCount")
        local rows = db.getRows ("SSLVPNResourceObject", "ResourceName",name)
        if(tonumber(count) > 0) then
            rows = nil
            valid = false
            errStr = "SSLVPN_RESOURCE_IN_USE_DELETE_FAILED"
        end               
        if (rows) then
            for i,j in pairs(rows) do
                local inTable = {}  
                inTable["SSLVPNResourceObject._ROWID_"] = j["SSLVPNResourceObject._ROWID_"]
                DBTable = "SSLVPNResourceObject"
                valid = sslvpn.resourceDeleteConfig(inTable, "-1", "delete")
                if (valid == false) then
                    errStr = "RESOURCE_OBJECT_DELETE_FAILED"
                    break
                end
            end
        end
    end

    for k,v in pairs(rowIds) do
        if (valid) then
            local inTable1 = {}
            inTable1["SSLVPNResource._ROWID_"] = v;
            DBTable = "SSLVPNResource"
            valid = sslvpn.resourceDeleteConfig(inTable1, "-1", "delete")
            if (valid == false) then
                errStr = "RESOURCE_DELETE_FAILED"
                break
            end
        end
    end 

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "RESOURCE_DELETE_OK"
    else
        db.rollback()
        return "ERROR", errStr
    end
end

-- SSL VPN Client Configuration
function sslvpn.sslVpnClientConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false

    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            valid = db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            valid = db.update(DBTable, inputTable, rowid)
        end
    end 
    
    -- return
    if (valid) then
        db.commitTransaction(true)
            return "OK", "STATUS_OK"            
    else
        db.rollback()
        return "ERROR", "VPN_SSLVPNCLIENT_CONFIG_FAILED"
    end
end

-- Function to configure sslvpn client routes
function sslvpn.sslClientRouteConfig (inputTable, rowid, operation)
     -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false
    
    if (operation == "add") then
        valid = db.insert(DBTable, inputTable)
    elseif (operation == "edit") then
        valid = false
    elseif (operation == "delete") then
        valid = db.delete(DBTable, inputTable)
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "SSL_VPN_CLIENT_ROUTE_CONFIG_FAILED"
    end
end

-- Function for SSL VPN Policy Configuration
function sslvpn.policyConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false
    
    -- Setting Policy Owner Name
    if(inputTable["SSLVPNPolicy.PolicyType"] == "global") then
        inputTable["SSLVPNPolicy.PolicyOwnerName"] = "global"
    end

    -- Setting value for Destination Object 
    if(inputTable["SSLVPNPolicy.DestinationObjectType"] == "1") then
        inputTable["SSLVPNPolicy.DestinationObject"] =  inputTable["SSLVPNPolicy.ResourceName"]
    elseif(inputTable["SSLVPNPolicy.DestinationObjectType"] == "2") then
        inputTable["SSLVPNPolicy.DestinationObject"] =  inputTable["SSLVPNPolicy.Address"] .. ":" .. inputTable["SSLVPNPolicy.StartPort"] .. "-" .. inputTable["SSLVPNPolicy.EndPort"]
    elseif(inputTable["SSLVPNPolicy.DestinationObjectType"] == "3") then
        inputTable["SSLVPNPolicy.DestinationObject"] =  inputTable["SSLVPNPolicy.Address"] .. "/" .. inputTable["SSLVPNPolicy.MaskLength"] .. ":" .. inputTable["SSLVPNPolicy.StartPort"] .. "-" .. inputTable["SSLVPNPolicy.EndPort"]
    elseif(inputTable["SSLVPNPolicy.DestinationObjectType"] == "4") then
        inputTable["SSLVPNPolicy.DestinationObject"] =  "0.0.0.0/" .. inputTable["SSLVPNPolicy.StartPort"] .. "-" .. inputTable["SSLVPNPolicy.EndPort"]
    end
    

    if(operation == "edit") then
        if (inputTable["SSLVPNPolicy.DestinationObjectType"] == "1") then
            inputTable["SSLVPNPolicy.ServiceType"] = ""
        end
    end

    -- Checking the resource Objects count
    if ((operation == "add" or operation == "edit") and inputTable["SSLVPNPolicy.ResourceName"] ~= nil) then
        local resObjTable = db.getRowsWhere ("SSLVPNResourceObject", "ResourceName = '" .. inputTable["SSLVPNPolicy.ResourceName"] .. "'")
        if (resObjTable ~= nil and #resObjTable == 0) then
            db.rollback()
            return "ERROR", "NO_RESOURCE_OBJECTS_CONFIGURED"
        end
    end

    -- geting Old Resource Name
    local Rname = ""
    local dstObjType = ""
    if (operation == "edit") then
        dstObjType = db.getAttribute("SSLVPNPolicy", "_ROWID_", rowid, "DestinationObjectType")
        if(dstObjType == "1") then
            Rname = db.getAttribute("SSLVPNPolicy", "_ROWID_", rowid, "ResourceName")
        end
    end

    -- config Policy
    valid = sslvpn.config(inputTable, rowid, operation)

    -- Updating SSLVPNResource Usecount 
    if (valid and inputTable["SSLVPNPolicy.DestinationObjectType"] == "1") then
        if (operation == "add" ) then
            sslvpn.updateResource (inputTable["SSLVPNPolicy.ResourceName"], operation)
        end
    end

    -- decrementing Old Resource UseCount according to Policy
    if (valid and operation  == "edit") then
        if (dstObjType == "1" and inputTable["SSLVPNPolicy.DestinationObjectType"] == "1" ) then
            if (Rname ~= inputTable["SSLVPNPolicy.ResourceName"]) then
                valid = sslvpn.updateResource (Rname, "delete")
                if(valid) then
                    valid = sslvpn.updateResource (inputTable["SSLVPNPolicy.ResourceName"] , "edit")
                end
            end
        elseif(dstObjType == "1" and inputTable["SSLVPNPolicy.DestinationObjectType"] ~= "1" ) then
            valid = sslvpn.updateResource (Rname, "delete")
        elseif(dstObjType ~= "1" and inputTable["SSLVPNPolicy.DestinationObjectType"] == "1") then
            valid = sslvpn.updateResource (inputTable["SSLVPNPolicy.ResourceName"] , "edit")
        end
    end
   
    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "SSLVPN_POLICY_CONFIG_FAILED"
    end
end


-- SSL VPN Policy Config
function sslvpn.config (inputTable, rowid, operation)
 -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            return db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            return db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
            return db.delete(DBTable, inputTable)
        end
    end
    return false
end

-- Function for Deleting SSL VPN Policies
function sslvpn.deletePolicy (rowIds)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    
    db.beginTransaction() --begin transaction
    local valid = false

    -- delete routes
    for k,v in pairs(rowIds) do
        local inTable = {}
        inTable["SSLVPNPolicy._ROWID_"] = v;
    
        -- Updating Resource , if Destination Type is Network Resource
        local dstType = db.getAttribute("SSLVPNPolicy", "_ROWID_", inTable["SSLVPNPolicy._ROWID_"], "DestinationObjectType")
        if(dstType == "1") then
            local ResourceName = db.getAttribute("SSLVPNPolicy", "_ROWID_", inTable["SSLVPNPolicy._ROWID_"], "ResourceName")
            valid = sslvpn.updateResource(ResourceName, "delete");
        else
            valid = true
        end

        if(valid) then
            valid = db.delete("SSLVPNPolicy", inTable) 
        end
    
        if(valid==false) then
            break;
        end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "SSLVPN_POLICY_DELETE_FAILED"
    end
end

-- Function for Updating Resource UseCount
function sslvpn.updateResource (ResourceName, operation)
    local valid = false
    local tmpTable = {}
    local useCount = db.getAttribute("SSLVPNResource", "ResourceName", ResourceName, "UseCount")
    local rowID = db.getAttribute("SSLVPNResource", "ResourceName", ResourceName, "_ROWID_")

    if (operation == "add") then
        tmpTable["SSLVPNResource.UseCount"] = tonumber(useCount) + 1
        valid = db.update("SSLVPNResource", tmpTable, rowID)
    end
    if (operation == "edit") then
        tmpTable["SSLVPNResource.UseCount"] = tonumber(useCount) + 1
        valid = db.update("SSLVPNResource", tmpTable, rowID)
    end

    if (operation == "delete") then
        tmpTable["SSLVPNResource.UseCount"] = tonumber(useCount) - 1
        valid = db.update("SSLVPNResource", tmpTable, rowID)
    end

     -- return
    if (valid) then
        return true
    else
        return false
    end
end

-- Function for setting SSL VPN Active User State
function sslvpn.setUserState (rowid)
     -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false
    local inputTable = {}
    local userName = ""
    
    userName = db.getAttribute("SSLVPNActiveUsers", "_ROWID_", rowid, "UserName")
    local id = db.getAttribute("SSLVPNActiveUsers", "UserName", userName, "_ROWID_")
    inputTable["SSLVPNUserState.ActiveUser"] = "0"
    
    valid = db.update("SSLVPNUserState", inputTable, id)

      -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "SSL_VPN_USER_DISCONNECT_SUCCESS"
    else
        db.rollback()
        return "ERROR", "SSL_VPN_USER_DISCONNECT_FAILED"
    end
end

-- 
-- Funtion to update activeUsers Tbl with associated Tunnel's
-- Statistics
-- 
function sslvpn.sslvpnTunnelStatsGet()
    local status = sslvpnLuaLib.sslTunnelStatsGet()
    if (status == 0) then
        return 0
    else
        return -1
    end
end


-- httpsMgmt config
function httpsMgmt.config (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "Administrator Privileges Are Required"
    end
    db.beginTransaction() --begin transaction
    local valid = false

    -- validate
    if (httpsMgmt.inputvalidate(inputTable, operation)) then        
        if (operation == "add") then
            return nil
        elseif (operation == "edit") then
            if (inputTable["httpsMgmt.httpsMgmtPort"] == "0") then
                return "ERROR", "Invalid Port Configured."
            elseif (inputTable["httpsMgmt.httpsMgmtPort"] == "80") then
                return "ERROR", "This Port Already Reserved."
            end
            valid = db.update("httpsMgmt", inputTable, rowid)
        elseif (operation == "delete") then
            return nil
        end
    end
    
    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "Operation Succedded"
    else
        db.rollback()
        return "ERROR", "Configuration Update Failed"
    end
end

-- httpsMgmt inputvalidate
function httpsMgmt.inputvalidate (inputTable, operation)
    if (true) then
            return db.typeAndRangeValidate(inputTable)
        end
    return false
end

function  sslvpn.deleteDefinedResource (rowIds, rName)
	local resObjTable = db.getRowsWhere ("SSLVPNResourceObject", "ResourceName = '" .. rName .. "'")
	local policiesTable = db.getRowsWhere ("SSLVPNPolicy", "ResourceName= '" .. rName .. "'")
	if (policiesTable ~= nil and resObjTable ~= nil and #policiesTable ~= 0 and (#resObjTable == 1 or #resObjTable == #rowIds)) then
		return "ERROR", "CANNOT_DELETE_ONLY_RESOURCE"
	end
	for k,v in pairs(rowIds) do
		configRowId = v
		local rows = {configRowId}
		errorFlag, statusCode = sslvpn.resourceConfig (rows,"-1","delete")
		if(errorFlag ~= "OK") then
			return errorFlag, statusCode
		end	
	end
	return errorFlag, statusCode
end
