-- systemCLI.lua 
-- Gaurav Mathur
-- TeamF1
-- 10/23/07
--
-- Modification History
-- 02mar10,db  Added status related routines
-- 05jan10,db  Added SNMP Access control specific routines and modified the time
--         zoneTab by adding a zone for GMT-04:30
-- 25dec07,gnm Added check for nil firmwareVer and system name fields
--	       in sysInfoGet; Added alias interface name argument
--	       value to aliasipGet() call in sysStatusGet; Added
--	       ethCLISingleGet() calls to display eth1 and eth0
--	       information for show system status command
-- 24dec07,gnm modified sysLogFacilityInfoGet() to add check for
--	       invalid facility values
-- 28nov07 gnm written
--
-- Description
-- CLISH system LUA routines
--

local dayTab = {
"Sunday", 
"Monday",
"Tuesday",
"Wednesday",
"Thursday", 
"Friday", 
"Saturday",
}

local timezoneTab= {
"(GMT) Greenwich Mean Time : Edinburgh, London",
"(GMT-12:00) Eniwetok,Kwajalein",
"(GMT-11:00) Midway Island,Samoa",
"(GMT-10:00) Hawaii",
"(GMT-09:30) Marqesas Is.",
"(GMT-09:00) Alaska",
"(GMT-08:30) Pitciarn Is.",
"(GMT-08:00) Pacific Time(Canada); Tijuana",
"(GMT-08:00) Pacific Time(US); Tijuana",
"(GMT-07:00) Mountain Time, Arizona(Canada)",
"(GMT-07:00) Mountain Time, Arizona(US)",
"(GMT-06:00) Mexico City(US, Canada)",
"(GMT-06:00) CentralTime(USA)",
"(GMT-05:00) EasternTime",
"(GMT-05:00) EasternTime(USA),Lima,Indiana East",
"(GMT-04:30) Caracas",
"(GMT-04:00) Atlantic Time(Canada), Caracas",
"(GMT-03:30) Newfoundland",
"(GMT-03:00) Brasilia, Buenos Aires",
"(GMT-02:00) Mid-Atlantic",
"(GMT-01:00) Azores, Cape Verde Is.",
"(GMT+01:00) Europe",
"(GMT+02:00) Athens, Istanbul, Minsk, Cairo",
"(GMT+03:00) Baghdad, Kuwait, Moscow",
"(GMT+03:30) Tehran",
"(GMT+04:00) Abu Dhabi, Muscat, Baku",
"(GMT+04:30) Kabul",
"(GMT+05:00) Ekaterinburg, Islamabad, Karachi",
"(GMT+05:30) Bombay, Calcutta, Madras, Delhi",
"(GMT+06:00) Almaty, Dhaka, Colombo",
"(GMT+06:30) Burma",
"(GMT+07:00) Bangkok, Hanoi, Jakarta",
"(GMT+08:00) Beijing, Chongqing, Hong Kong",
"(GMT+09:00) Osaka, Sapporo, Tokyo, Seoul",
"(GMT+09:30) Adelaide, Darwin",
"(GMT+10:00) Brisbane, Guam, Port Moresby",
"(GMT+10:30) Lord Howe Is.",
"(GMT+11:00) Magadan, Solomon Is, New Caledonia",
"(GMT+11:30) Norfolk I.",
"(GMT+12:00) Auckland, Wellington, New Zealand, Fiji",
"(GMT+13:00) Tonga",
"(GMT+14:00) Kiribati,Western Samoa"
}

function tzInfoGet ()
    local resultTab = {}
    local idx = "1"
    row = db.getRow("ntp", "ntp._ROWID_", 1) -- the ntp table should
					--only have one entry
    resTab.insertField (resultTab, "Current Time", util.time())
    idx = tonumber(row["ntp.timezone"])
    idx = idx + 1
    resTab.insertField (resultTab, "Timezone", timezoneTab[idx])
    if (row["ntp.autoDaylight"] == "1") then 
       resTab.insertField (resultTab, "Automatically Adjust for Daylight " ..
       				      "Savings Time", "Yes")
    else
       resTab.insertField (resultTab, "Automatically Adjust for Daylight " ..
				      "Savings Time", "No")
    end				     
    if (row["ntp.useDefServers"] == "1") then
       resTab.insertField (resultTab, "Default NTP servers used ", "Yes")
    else
       resTab.insertField (resultTab, "Default NTP servers used ", "No")
       resTab.insertField (resultTab, "Server 1 Name / IP Address",
			  row["ntp.server1"])
       resTab.insertField (resultTab, "Server 2 Name / IP Address",
			  row["ntp.server2"])
    end

    return resultTab 	    
end

--
-- This routine returns the Identifier that's prepended to all log messages
-- in the system
--
function logIdentifierGet ()
    local ident = db.getAttribute("emailLogs","_ROWID_",1,"logId") or "EAP3000"
    return ident
end

--
-- This routine returns the Syslog server set on the system
--
function syslogServerGet ()
    return db.getAttribute("sysLogInfo","_ROWID_", 1, "serverName")
end

function sysLogFacilityInfoGet (facility)

    if (facility == nil) then --sanity check
       local resultTab = {}
       resTab.insertField (resultTab, "", "No such facility")
       return resultTab
    end

    local logTab = db.getRow ("logConfig", "logConfig.facility", facility)
    if (logTab == nil) then --sanity check
       local resultTab = {}
       resTab.insertField (resultTab, "", "No such facility")
       return resultTab
    end

    local resultTab = resTab.createRowLbl ("Emergency", "Alert", "Critical", 
	    "Error", "Warning",	"Notification", "Information", "Debugging")
    
    local emergencyChoices = util.integerToBits(logTab["logConfig.emergency"])
    local alertChoices = util.integerToBits(logTab["logConfig.alert"])
    local criticalChoices = util.integerToBits(logTab["logConfig.critical"])
    local errorChoices = util.integerToBits(logTab["logConfig.error"])
    local warningChoices = util.integerToBits(logTab["logConfig.warning"])
    local noticeChoices = util.integerToBits(logTab["logConfig.notice"])
    local informationChoices = util.integerToBits(logTab["logConfig.information"])
    local dbgChoices = util.integerToBits(logTab["logConfig.debug"])
    
    local idxTab = {1,3,4}
    for i, idx in ipairs (idxTab) do
	if (idx == 1) then colLbl = "Display in event log"
	elseif (idx == 3) then colLbl = "Send to syslog"
	elseif (idx == 4) then colLbl = "Display on telnet/SSH monitor"
	end
	
	resTab.insertYNField (resultTab, emergencyChoices[idx] == 1, colLbl)
        resTab.insertYNField (resultTab, alertChoices[idx] == 1, colLbl)
	resTab.insertYNField (resultTab, criticalChoices[idx] == 1, colLbl)
        resTab.insertYNField (resultTab, errorChoices[idx] == 1, colLbl)
	resTab.insertYNField (resultTab, warningChoices[idx] == 1, colLbl)
	resTab.insertYNField (resultTab, noticeChoices[idx] == 1, colLbl)
	resTab.insertYNField (resultTab, informationChoices[idx] == 1,colLbl)
	resTab.insertYNField (resultTab, dbgChoices[idx] == 1, colLbl)
    end

    return resultTab 	    
end

--
--
--
local function eth2Get ()
    local resultTab = {}
    local row = db.getRowWithJoin(
	  	  {"ethernet:networkInterface:interfaceName"}, 
		  "ethernet.interfaceName", "eth1")
    if (row["ethernet.vlanEnabled"] == "0") then
        row["ethernet.vlanId"]= "0";
    end
    resTab.insertField (resultTab, "MAC Address",
		       row["ethernet.macAddress"] or "")
    resTab.insertField (resultTab, "IP Address", 
		       row["networkInterface.ipaddr"])
    resTab.insertField (resultTab, "IP Subnet Mask", 
		       row["networkInterface.subnetmask"])
    resTab.insertField (resultTab, "Gateway IP Address", 
		       row["networkInterface.gateway"] or "")
    resTab.insertField (resultTab, "DNS Server", 
		       row["networkInterface.dns1"] or "")
    resTab.insertField (resultTab, "VLAN", vlanId or "")
    return resultTab    
end


--
--
--
local function eth1Get ()
    local resultTab = {}

    local rowBdg = db.getRowWithJoin(
		 {"bridgeTable:networkInterface:interfaceName"},
		 "bridgeTable.interfaceName", "bdg")
    local rowEth = db.getRowWithJoin({"ethernet:networkInterface:interfaceName"},
		 "ethernet.interfaceName", "eth0")
    local vlanId = "0";

    if (rowEth["ethernet.vlanEnabled"] == "1") then 
       vlanId = db.getAttribute("ethernet", "interfaceName", "eth0", "vlanId")
    end

    resTab.insertField (resultTab, "MAC Address",
		       rowBdg["bridgeTable.macAddress"] or "")
    resTab.insertField (resultTab, "IP Address", 
		       rowBdg["networkInterface.ipaddr"])
    resTab.insertField (resultTab, "IP Subnet Mask", 
		       rowBdg["networkInterface.subnetmask"])
    resTab.insertField (resultTab, "Gateway IP Address", 
		       rowBdg["networkInterface.gateway"] or "")
    resTab.insertField (resultTab, "DNS Server", 
		       rowBdg["networkInterface.dns1"] or "")
    resTab.insertField (resultTab, "VLAN", vlanId or "")
    return resultTab
end

--
-- build SNMP result table
-- 
local function buildSNMP (resultTab, row)
    if (row == nil) then return end --sanity check

    resTab.insertField (resultTab, "IP Address", row["snmpTrap.ipAddr"])
    resTab.insertField (resultTab, "Snmp Version", row["snmpTrap.snmpVersion"])
    resTab.insertField (resultTab, "Port", row["snmpTrap.port"])
    resTab.insertField (resultTab, "Community", row["snmpTrap.commName"])
end

--
-- build SNMP Access Control table
-- 
local function buildSNMPACL (resultTabACL, row)
    if (row == nil) then return end --sanity check

    resTab.insertField (resultTabACL, "IP Address", row["snmpAccessControl.ipAddr"])
    resTab.insertField (resultTabACL, "Subnet Mask", row["snmpAccessControl.subnetMask"])
    resTab.insertField (resultTabACL, "Access Type", row["snmpAccessControl.accessType"])
    resTab.insertField (resultTabACL, "Community", row["snmpAccessControl.commName"])
end

--
-- This routine gets the system SNMP configuration
-- 
function sysSNMPConfigGet ()
    local resultTab = {}

    row = db.getRow ("system","system._ROWID_", 1)
    if (row == nil) then
       return nil
    end
    resTab.insertField (resultTab, "SysContact", row["system.sysContact"] or "")
    resTab.insertField (resultTab, "SysLocation", row["system.sysLoc"] or "")
    resTab.insertField (resultTab, "SysName", row["system.name"])
    return resultTab
end

--
-- get general system information
-- 
local function sysInfoGet (args)
    local resultTab = {}
    local system = db.getRow("system", "_ROWID_", "1")

    resTab.insertField (resultTab, "System Up Time", util.uptime())
    resTab.insertField (resultTab, "System Name", 
		       system["system.name"] or "")
    resTab.insertField (resultTab, "Firmware Version", 
		       system["system.firmwareVer"] or "")
    local serNum = db.getAttribute("unitInfo", "_ROWID_", "1", "serialNumber")
    resTab.insertField (resultTab, "Serial Number", 
		       serNum or "")
    return resultTab
end

--
--
--
local function hourStringMake (field)
    local hour = "-1"

    if (field == nil) then return hour end --sanity check
    hour = tonumber (field)
    if (hour >= 0 and hour < 12) then         
	hour = hour .. ":00" .. " A.M."
    elseif (hour >= 12 and hour <= 23) then
	hour = hour - 12
	hour = hour .. ":00" .. " P.M."
    end
    return hour     
end


--
--
--
local function schedGet (resultTab, cmd)
    row = db.getRow ("cronTab", "command", cmd)
    if (row == nil) then
        resTab.insertField (resultTab, "Unit", "Never")
	resTab.insertField (resultTab, "Day", "-")
	resTab.insertField (resultTab, "Time", "-")
	return resultTab
    end
    if (row["cronTab.unit"] == "1") then
        resTab.insertField (resultTab, "Unit", "Hourly")
        resTab.insertField (resultTab, "Day", "-")
        resTab.insertField (resultTab, "Time", "-")
    elseif (row["cronTab.unit"] == "2") then                    
        resTab.insertField (resultTab, "Unit", "Daily")
        resTab.insertField (resultTab, "Day", "-")
        resTab.insertField (resultTab, "Time", hourStringMake(row["cronTab.hour"]))
    elseif (row["cronTab.unit"] == "3") then
	resTab.insertField (resultTab, "Unit", "Weekly")
	local day = row["cronTab.dayOfWeek"]
	day = tonumber(day) + 1
	resTab.insertField (resultTab, "Day", dayTab[day])
	resTab.insertField (resultTab, "Time", hourStringMake(row["cronTab.hour"]))
    end
end

--
--
--
function sysSchedGet ()
    local resultTab = resTab.create ()
        
    emailLogs = db.getAttribute ("emailLogs", "_ROWID_", 1, "mailLogs")
    if (emailLogs ~= "1") then
       return nil
    end

    schdCmd = db.getAttribute("environment","name","EMAILLOGS_PROGRAM","value") .. 
	    " " .. DB_FILE_NAME
    schedGet (resultTab, schdCmd)	
    return resultTab
end

--
--
--
function sysEmailLogGet ()
    local resultTab = resTab.create ()

    emailLogs = db.getAttribute ("emailLogs", "_ROWID_", 1, "mailLogs")
    if (emailLogs ~= "1") then
       return nil
    end

    local entryName = db.getAttribute("emailLogs","_ROWID_",1,"entryName")
    row = db.getRow ("smtpServer", "entryName", entryName)
    if (row == nil ) then
    else
        resTab.insertField (resultTab, "E-Mail Server Address", 
			   row['smtpServer.emailServer'])
        resTab.insertField (resultTab, "Return E-Mail Address", 
			   row['smtpServer.fromAddr'])
        resTab.insertField (resultTab, "Send to E-Mail Address", 
			   row['smtpServer.toAddr'])
        resTab.insertField (resultTab, "Send to E-Mail Address", 
			   row['smtpServer.toAddr'])

        local auth = row['smtpServer.auth']
	auth = tonumber (auth)
	local tmpStr = ""
	if (auth == -1) then tmpStr = "No Authentication" 
	elseif (auth == 0) then tmpStr = "Login Plain"
	elseif (auth == 1) then tmpStr = "CRAM-MD5"
	end
        resTab.insertField (resultTab, "Authentication", tmpStr)
	
	if (auth == 0 or auth == 1) then 
	   resTab.insertField (resultTab, "User Name", row['smtpServer.userName'])
	   resTab.insertField (resultTab, "Password", row['smtpServer.passWord'])
        end
	resTab.insertYNField (resultTab, row['smtpServer.respondIdentd'] == "1", 
			     "Respond to Identd from SMTP Server")
    end
    return resultTab
end

--
--
--
function sysEmailSchedGet (args)

    print ("\n" .. "Log Identifier: " .. logIdentifierGet())	 	 
    resultTab = sysEmailLogGet ()
    if (resultTab == nil) then
        print ("\n" .. "E-Mail logs are disabled" .. "\n")
    else	
        printLabel ("Enable E-Mail Logs")
        resTab.print (resultTab, 0)
    end
    resultTab = sysSchedGet()
    if (resultTab ~= nil) then
       printLabel ("Send E-mail logs by Schedule")
       resTab.print (resultTab, 0)
    end                       
end

--
-- main function for system get
-- 
function sysLoggingGet (args)
    local facility = args[1]
    local ident = logIdentifierGet() or "" 	  
    local server = syslogServerGet() or ""

    print ("\n" .. "Log Identifier: " .. ident)
    print ("Syslog Server: " .. server)

    printLabel ("Logs Facility and Level Configuration")
    if (facility == "all") then
       local logTab = db.getTable ("logConfig")
       for k, v in pairs (logTab) do
	   printLabel (v["logConfig.facility"])
	   resultTab = sysLogFacilityInfoGet(v["logConfig.facility"])
           resTab.print (resultTab)
       end
    else
       resultTab = sysLogFacilityInfoGet (facility)
       printLabel (facility)
       resTab.print (resultTab)
    end
end
--
-- main function for system get
-- 
function sysTimezoneGet (args)
    printLabel ("Time Zone & NTP Servers Configuration")    
    resultTab = tzInfoGet()
    resTab.print (resultTab, 0)    
end   

--
-- main function for system get
-- 
function sysUsersGet (args)
    printLabel ("User Selection")    
    resultTab = sysUserGet ()
    resTab.print (resultTab, 0)
    printLabel ("Available Access Points")
    resultTab = dot11APAllGet ()
    resTab.print (resultTab, 0)
    printLabel ("Ethernet Port 1")    
    resultTab = eth1Get()
    resTab.print (resultTab, 0)
    printLabel ("Ethernet Port 2")    
    resultTab = eth2Get()
    resTab.print (resultTab, 0)
end   



function statusGet (wanType)
    local resultTab = {}
    if (wanType == "WAN1" or wanType == "WAN2") then
    
    local where = "LogicalIfName = '"..wanType.."' and addressFamily = 2"
						local where6 = "LogicalIfName = '"..wanType.."' and addressFamily = 10"
					    local wanv4Config = db.getRowWhere ("NimfConf", where)
					    local wanv6Config = db.getRowWhere ("NimfConf", where6)
						local wanRows = db.getRowsWhere ("ipAddressTable", where)
						local wan6Rows = db.getRowsWhere ("ipAddressTable", where6)	
					    local nsRows = db.getRows("resolverConfig", "LogicalIfName", wanType)
					    local gwRows = db.getRows("defaultRouters", "LogicalIfName", wanType)
					    local wanIntName = db.getAttribute ("networkInterface", "LogicalIfName", wanType, "interfaceName")
					    local wanState = db.getAttribute ("networkInterface", "LogicalIfName", wanType, "IfStatus") or ''
					    local wanMacAddr = ""
					    local natEnabled = ""    
					    local connetionType4 = ""
					    local connectionType6 =""
					    local connetionStatus4 = ""	
			            local connetionStatus6 = ""
			            local linkStatus4 = ""
					
						-- Getting MAC Address of the WAN Interface
						if (wanIntName ~= nil) then	
							wanMacAddr = db.getAttribute ("ethernet", "interfaceName", wanIntName, "macAddress") or ''
						end
					
						-- Getting Connection Type of the WAN(ipv4) Interface
					    if (wanv4Config["NimfConf.ConnectionType"] ~= nil) then
					        if (wanv4Config["NimfConf.ConnectionType"] == "dhcpc") then
					            connetionType4 = "Dynamic IP (DHCP)"
					        elseif (wanv4Config["NimfConf.ConnectionType"] == "ifStatic") then
					            connetionType4 = "STATIC"
					        elseif (wanv4Config["NimfConf.ConnectionType"] == "pppoe") then
					            connetionType4 = "PPPOE"
					        elseif (wanv4Config["NimfConf.ConnectionType"] == "pptp") then
					            connetionType4 = "PPTP"
					        elseif (wanv4Config["NimfConf.ConnectionType"] == "l2tp") then
					            connetionType4 = "L2TP"
                            elseif (wanv4Config["NimfConf.ConnectionType"] == "threeg") then
					            connetionType4 = "ThreeG"
					        end
					    end
					    
						-- Getting Connection Type of the WAN(ipv6) Interface
					    if (db.getAttribute("networkInfo", "_ROWID_", "1", "netWorkMode") == "3") then
							if (wanv6Config["NimfConf.ConnectionType6"] ~= nil) then
								if (wanv6Config["NimfConf.ConnectionType6"] == "dhcp6c") then
					                connectionType6 = "Dynamic IP (DHCPv6)"
					            elseif (wanv6Config["NimfConf.ConnectionType6"] == "ifStatic6") then
						            connectionType6 = "Static IPv6"
								elseif (wanv6Config["NimfConf.ConnectionType6"] == "") then
						            connectionType6 = "Disabled"
								end
						    end
						else
							connectionType6 = "IPv6 is disabled" 
						end                                            
					
						-- Getting Connection State of the WAN(ipv4) Interface
					    local nimfStatusConfig = db.getRowWhere("NimfStatus", where)
					    if (nimfStatusConfig ~= nil) then
					    	connetionStatus4 = nimfStatusConfig["NimfStatus.Nimfstatus"] or ""
					    end
					
						-- Getting Connection State of the WAN(ipv6) Interface
						if (db.getAttribute("networkInfo", "_ROWID_", "1", "netWorkMode") == "3") then
							nimfStatusConfig = db.getRowWhere("NimfStatus", where6)
							if (nimfStatusConfig ~= nil) then
								connetionStatus6 = nimfStatusConfig["NimfStatus.Nimfstatus"] or ""
							end
						else
							connetionStatus6 = "IPv6 is disabled" 
			            end  
			
			            local nimfStatusConfig = db.getRowWhere("NimfStatus", where)
			            if (nimfStatusConfig ~= nil) then
					    	linkStatus4 = nimfStatusConfig["NimfStatus.Linkstatus"] or ""
					    end    
                        
                        if (wanv4Config["NimfConf.ConnectionType"] == "threeg") then
					        linkStatus4 = ""
					    end

                        -- Getting NAT Mode Status.
					    if (db.getAttribute("NatTable", "_ROWID_", "1", "Enable") == "1") then
					    	natEnabled = "Enabled"
					    else
					    	natEnabled = "Disabled"
					    end
     resTab.insertField (resultTab, "MAC Address:", wanMacAddr or '')
     local ipAddrStr = ""
	 local i = 0
     if(wanRows) then
	 for k,v in pairs(wanRows) do
	    i = i + 1
	    local row = wanRows[i]
	    ipAddrStr = ipAddrStr .. row["ipAddressTable.ipAddress"] .. " / " .. row["ipAddressTable.subnetMask"]
	    if(i ~= #wanRows) then
	      	ipAddrStr = ipAddrStr .. ", "
	    end
	end
	end
    resTab.insertField (resultTab, "IPv4 Address:", ipAddrStr or '')
    ipAddrStr = ""
	if (db.getAttribute("networkInfo", "_ROWID_", "1", "netWorkMode") == "3") then							
		i = 0
    	if(wan6Rows) then
	    	for k,v in pairs(wan6Rows) do
		    	i = i + 1
			    local row = wan6Rows[i]
				ipAddrStr = ipAddrStr .. row["ipAddressTable.ipAddress"] .. " / " .. row["ipAddressTable.ipv6PrefixLen"]
				if(i ~= #wan6Rows) then
				    ipAddrStr = ipAddrStr .. ", "
				end
			end
		end
	end
    resTab.insertField (resultTab, "IPv6 Address:", ipAddrStr or '')
    if (wanState == "1") then
		wanState = "UP"
    elseif (wanState == "2") then
		wanState = "UP(IPv6)"
	elseif (wanState == "3") then
		wanState = "UP(IPv4 and IPv6)"
	elseif (wanState == "0") then
		wanState = "DOWN"
	end
    resTab.insertField (resultTab, "Wan State:", wanState or '')
    resTab.insertField (resultTab, "NAT (IPv4 only):", natEnabled or '')
    resTab.insertField (resultTab, "IPv4 Connection Type:", connetionType4 or '')
    resTab.insertField (resultTab, "IPv6 Connection Type:", connectionType6 or '')
    resTab.insertField (resultTab, "IPv4 Connection State:", connetionStatus4 or '')
    resTab.insertField (resultTab, "IPv6 Connection State:", connetionStatus6 or '')
    resTab.insertField (resultTab, "Link State:", linkStatus4 or '')

    	local wanMode = ""
					local wmodeConfig = db.getRow("WanMode", "_ROWID_", "1")
					if (wmodeConfig ~= nil) then
						local wan = ""
						if (wmodeConfig["WanMode.Wanmode"] == "2") then
					    	wan = wmodeConfig["WanMode.FailoverPriLogicalIfName"]
					    else
					    	wan = wmodeConfig["WanMode.DedicatedLogicalIfName"]
					    end
					                                	
						if (wan ~= nil) then
							if (wan == "WAN1") then
								wan = ": Dedicated WAN"
							elseif (wan == "WAN2") then
								wan = ": Secondary WAN"
							end
						else
							wan = ""
						end
						
						if (wmodeConfig["WanMode.Wanmode"] == "2") then
							wanMode = "Auto-Rollover using WAN port" .. wan
						elseif (wmodeConfig["WanMode.Wanmode"] == "1") then
							wanMode = "Load Balancing - Round Robin"
						elseif (wmodeConfig["WanMode.Wanmode"] == "3") then
							wanMode = "Load Balancing - SpillOver"
						elseif (wmodeConfig["WanMode.Wanmode"] == "0") then
							wanMode = "Use only single WAN port" .. wan
						end
					end
    resTab.insertField (resultTab, "WAN Mode:", wanMode or '')
    local gwIps = ""
	    if(gwRows) then
		    local i = 0
		    for k,v in pairs(gwRows) do
			    i = i + 1        
			    local row = gwRows[i]
				if (row["defaultRouters.nextHopGateway"] ~= nil and row["defaultRouters.nextHopGateway"] ~= "") then
				   	if (gwIps == "") then
				   		gwIps = gwIps .. row["defaultRouters.nextHopGateway"]
				   	else
				   		gwIps = gwIps .. ", " .. row["defaultRouters.nextHopGateway"]
				   	end
				end               
			end
		end
    resTab.insertField (resultTab, "Gateway:", gwIps or '')

    local priDNSIps = ""
    if(nsRows) then
        local i = 0
		for k,v in pairs(nsRows) do
		  	i = i + 1        
		    local row = nsRows[i]
		    if (row["resolverConfig.nameserver1"] ~= nil and row["resolverConfig.nameserver1"] ~= "") then
		       	if (priDNSIps == "") then
		       		priDNSIps = priDNSIps .. row["resolverConfig.nameserver1"]
		       	else
		       		priDNSIps = priDNSIps .. ", " .. row["resolverConfig.nameserver1"]
		       	end
		    end              
		end
	end
    resTab.insertField (resultTab, "Primary DNS:", priDNSIps or '')

    local secDNSIps = ""
		if(nsRows) then
	    	local i = 0
		    for k,v in pairs(nsRows) do
		    	i = i + 1        
			    local row = nsRows[i]
			    if (row["resolverConfig.nameserver2"] ~= nil and row["resolverConfig.nameserver2"] ~= "") then
			       	if (secDNSIps == "") then
			       		secDNSIps = secDNSIps .. row["resolverConfig.nameserver2"]
			      	else
			       		secDNSIps = secDNSIps .. ", " .. row["resolverConfig.nameserver2"]
			     	end
			    end              
			end
		end
    resTab.insertField (resultTab, "Secondary DNS:", secDNSIps or '')

    elseif (wanType == "DMZ") then
    
        local where = "LogicalIfName = 'DMZ' AND addressFamily = '2'"
					local lanRows = db.getRowsWhere ("ipAddressTable", where)										
					local dhcpConf = db.getRow("dhcp", "dhcp._ROWID_", "2")    
					local dhcpStatus = ""
					if (dhcpConf ~= nil) then
						if (dhcpConf["dhcp.dhcpEnabled"] == "1") then
							dhcpStatus = "Enabled"
						elseif (dhcpConf["dhcp.dhcpEnabled"] == "0") then
							dhcpStatus = "Disabled"
						end
					end					
					local dhcpRelayConf = db.getRow("dhcpRelay", "dhcpRelay.LogicalIfName", "DMZ")    
					local dhcpRelayStatus = ""
					if (dhcpRelayConf ~= nil) then
						if (dhcpRelayConf["dhcpRelay.DhcpRelayStatus"] == "1") then
							dhcpRelayStatus = "Enabled"
						elseif (dhcpRelayConf["dhcpRelay.DhcpRelayStatus"] == "0") then
							dhcpRelayStatus = "Disabled"
						end
					end					
					local row = db.getRow("networkInterface", "LogicalIfName", "DMZ")
					local lanMacAddr = db.getAttribute("ethernet", "ethernet.interfaceName", row["networkInterface.interfaceName"], "macAddress")
          resTab.insertField (resultTab, "MAC Address:", lanMacAddr or '')

          local lanIPStr = ""
					local i = 0
					if(lanRows) then
					    for k,v in pairs(lanRows) do
						    i = i + 1
						    local row = lanRows[i]
						    lanIPStr = lanIPStr .. row["ipAddressTable.ipAddress"] .. " / " .. row["ipAddressTable.subnetMask"]
					        if(i ~= #lanRows) then
					        	lanIPStr = lanIPStr ..  ", "
					        end
					    end
					end
        resTab.insertField (resultTab, "IP Address:", lanIPStr or '')
        resTab.insertField (resultTab, "DHCP Server:", dhcpStatus or '')
        resTab.insertField (resultTab, "DHCP Relay:", dhcpRelayStatus or '')
    end

   return resultTab 
end

function lanStatusGet ()
    local resultTab = {} 
    local where = "LogicalIfName = 'LAN' AND addressFamily = '2'"
					local lanRows = db.getRowsWhere ("ipAddressTable", where)
					local where6 = "LogicalIfName = 'LAN' AND addressFamily = '10'"
					local lan6Rows = db.getRowsWhere ("ipAddressTable", where6)
					local dhcpConf = db.getRow("dhcp", "dhcp._ROWID_", "1")    
					local dhcpStatus = ""
					if (dhcpConf ~= nil) then
						if (dhcpConf["dhcp.dhcpEnabled"] == "1") then
							dhcpStatus = "Enabled"
						elseif (dhcpConf["dhcp.dhcpEnabled"] == "0") then
							dhcpStatus = "Disabled"
						end
					end
					
					local dhcpRelayConf = db.getRow("dhcpRelay", "dhcpRelay.LogicalIfName", "LAN")    
					local dhcpRelayStatus = ""
					if (dhcpRelayConf ~= nil) then
						if (dhcpRelayConf["dhcpRelay.DhcpRelayStatus"] == "1") then
							dhcpRelayStatus = "Enabled"
						elseif (dhcpRelayConf["dhcpRelay.DhcpRelayStatus"] == "0") then
							dhcpRelayStatus = "Disabled"
						end
					end
			
					local dhcp6Conf = db.getRow("dhcpv6s", "dhcpv6s._ROWID_", "1")    
					local dhcp6Status = ""
					if (db.getAttribute("networkInfo", "_ROWID_", "1", "netWorkMode") == "3") then
						if (dhcp6Conf ~= nil) then
							if (dhcp6Conf["dhcpv6s.isEnabled"] == "1") then
								dhcp6Status = "Enabled"
							elseif (dhcp6Conf["dhcpv6s.isEnabled"] == "0") then
								dhcp6Status = "Disabled"
							end
						end
					else
						dhcp6Status = "IPv6 is disabled"
					end
			
					local row = db.getRow("networkInterface", "LogicalIfName", "LAN")
					local lanMacAddr = db.getAttribute("ethernet", "ethernet.interfaceName", row["networkInterface.interfaceName"], "macAddress")
    resTab.insertField (resultTab, "MAC Address:", lanMacAddr or '')

    local lanIPStr = ""
					local i = 0
					if(lanRows) then
					    for k,v in pairs(lanRows) do
						    i = i + 1
						    local row = lanRows[i]
						    lanIPStr = lanIPStr .. row["ipAddressTable.ipAddress"] .. " / " .. row["ipAddressTable.subnetMask"]
					        if(i ~= #lanRows) then
					        	lanIPStr = lanIPStr ..  ", "
					        end
					    end
					end
    resTab.insertField (resultTab, "IP Address:", lanIPStr or '')

    lanIPStr = ""
						if (db.getAttribute("networkInfo", "_ROWID_", "1", "netWorkMode") == "3") then							
							i = 0
							if(lan6Rows) then
					    	    for k,v in pairs(lan6Rows) do
						    	    i = i + 1
							        local row6 = lan6Rows[i]
									lanIPStr = lanIPStr ..  row6["ipAddressTable.ipAddress"] .. " / " .. row6["ipAddressTable.ipv6PrefixLen"]
				   			        if(i ~= #lan6Rows) then
				    					lanIPStr = lanIPStr ..  ", "
					    	        end
								end
							end
						end
    resTab.insertField (resultTab, "IPv6 Address:", lanIPStr or '')

    resTab.insertField (resultTab, "DHCP Server:", dhcpStatus or '')
    resTab.insertField (resultTab, "DHCP Relay:", dhcpRelayStatus or '')
    resTab.insertField (resultTab, "DHCPv6 Server:", dhcp6Status or '')
    
    return resultTab
end

function wlanStatusGet ()
    local resultTab = {}

    local cardConfig = db.getRow ("dot11Card", "_ROWID_", "1")
			local wlInfo = {}
			if (cardConfig ~= nil) then
				wlInfo = db.getRow ("dot11Radio", "cardNo", cardConfig["dot11Card.cardNo"])
				wlInfo["dot11GlobalConfig.country"] = db.getAttribute ("dot11GlobalConfig", "_ROWID_", "1", "country") or ''
				wlInfo["dot11GlobalConfig.country"] = db.getAttribute ("dot11Countries", "countrycode", wlInfo["dot11GlobalConfig.country"], "country") or wlInfo["dot11GlobalConfig.country"]
				wlInfo["dot11Radio11n.puren"] = db.getAttribute ("dot11Radio11n", "radioNo", wlInfo["dot11Radio.radioNo"], "puren") or ''
		   	end
		   	if (cardConfig == nil) then
		   		cardConfig = {}
		   	end
		   	if (wlInfo == nil) then
		   		wlInfo = {}
		   	end
		   	local opFrq = ""
		   	if (wlInfo["dot11Radio.band"] == "b") then
		   		opFrq = "2.4GHz"
		   	elseif (wlInfo["dot11Radio.band"] == "a") then
		   		opFrq = "5GHz"
		   	end
		   		
      		if(wlInfo ["dot11Radio.opMode"] == "ng40-" or wlInfo ["dot11Radio.opMode"] == "ng") then
      			wlInfo ["dot11Radio.opMode"] = "N/G-Mixed";
      		elseif(wlInfo ["dot11Radio.opMode"] == "b and g") then	
      			wlInfo ["dot11Radio.opMode"] = "B/G-Mixed";
      		elseif(wlInfo ["dot11Radio.opMode"] == "g only") then	
      			wlInfo ["dot11Radio.opMode"] = "G Only";
      		end

      		if(wlInfo ["dot11Radio.opMode"] == "na40-" or wlInfo ["dot11Radio.opMode"] == "na") then
      			wlInfo ["dot11Radio.opMode"] = "N/A-Mixed";
      		elseif(wlInfo ["dot11Radio.opMode"] == "a only") then	
      			wlInfo ["dot11Radio.opMode"] = "A Only";
      		end

		if (wlInfo["dot11Radio11n.puren"] == "1") then
            		wlInfo ["dot11Radio.opMode"] = "N Only"; 
      		end
        
        print ("Operating Frequency: "..opFrq)
        print ("Mode: "..wlInfo["dot11Radio.opMode"])
        print ("Channel: "..wlInfo["dot11Radio.configuredChannel"])

        local i = 0
						local table = db.getTable("dot11VAP")
						for k,v in pairs(table) do
							i = i + 1							
							local row = table[i]
							local profileConfig = db.getRow ("dot11Profile", "profileName", row["dot11VAP.profileName"])
							if (profileConfig == nil) then
								profileConfig = {}
							end
							local encryption = profileConfig["dot11Profile.pairwiseCiphers"] or ""
							local authentication = profileConfig["dot11Profile.authMethods"] or ""
							if (profileConfig["dot11Profile.security"] == "OPEN")  then
								encryption = "NONE"
								authentication = "NONE"
						    elseif (profileConfig["dot11Profile.security"] == "WEP") then
						        encryption = profileConfig["dot11Profile.groupCipher"] or ""
						        authentication = profileConfig["dot11Profile.wepAuth"] or ""
							end							
							if( row["dot11VAP.vapEnabled"] == "1") then
                                
                                resTab.insertField (resultTab, "SSID", profileConfig["dot11Profile.ssid"] or '')
                                resTab.insertField (resultTab, "SECURITY", profileConfig["dot11Profile.security"] or '')
                                resTab.insertField (resultTab, "ENCRYPTION", encryption or '')
                                resTab.insertField (resultTab, "AUTHENTICATION", authentication or '')
                            end
			          end
    return resultTab
end

--
-- main function to get system status 
-- 
function sysStatusGet (args)
    printLabel ("System Info")    
    resultTab = sysInfoGet ()
    resTab.print (resultTab, 0)
    -- run program to NIMF Status and update DB
	local prog = db.getAttribute("environment", "name", "NIMFSTATUS_PROGRAM", "value")
	if (prog ~= nil) then
        util.appendDebugOut("Exec = " .. os.execute(prog .. " " .. DB_FILE_NAME .. " WAN1 2"))
        util.appendDebugOut("Exec = " .. os.execute(prog .. " " .. DB_FILE_NAME .. " WAN1 10"))
        util.appendDebugOut("Exec = " .. os.execute(prog .. " " .. DB_FILE_NAME .. " WAN2 2"))
        util.appendDebugOut("Exec = " .. os.execute(prog .. " " .. DB_FILE_NAME .. " WAN2 10"))
    end
    prog = db.getAttribute("environment", "name", "LINKSTATUS_PROGRAM", "value")
	if (prog ~= nil) then
		util.appendDebugOut("Exec = " .. os.execute(prog .. " " .. DB_FILE_NAME .. " WAN1"))
        util.appendDebugOut("Exec = " .. os.execute(prog .. " " .. DB_FILE_NAME .. " WAN2"))
        util.appendDebugOut("Exec = " .. os.execute(prog .. " " .. DB_FILE_NAME .. " DMZ"))
	end
	
	-- If IP Mode is IPv4/IPv6 then we need to call this binary.
	local ipMode = db.getAttribute ("networkInfo", "_ROWID_", "1", "netWorkMode") or '0'
	if (ipMode == "3") then
	    prog = db.getAttribute("environment", "name", "IPV6_GW_UPDATE", "value")
		if (prog ~= nil) then
			util.appendDebugOut("Exec = " .. os.execute(prog .. " " .. DB_FILE_NAME))
		end
	end
    printLabel ("WAN1 Information")
    resultTab = statusGet("WAN1")
    resTab.print (resultTab, 0)
    printLabel ("WAN2 Information")
    local configurablePort = db.getAttribute("ConfigPort","_ROWID_","1","LogicalIfName")
	if(configurablePort == "WAN2") then
        resultTab = statusGet ("WAN2")
        resTab.print (resultTab, 0)
    else
        resultTab = statusGet ("DMZ")
        resTab.print (resultTab, 0)
    end
    printLabel ("LAN Information")
    resultTab = lanStatusGet()
    resTab.print (resultTab, 0)

    printLabel ("Wireless LAN Information")
    resultTab = wlanStatusGet ()
    resTab.print (resultTab, 0)
end   

--
-- main function to get SNMP configuration information
-- 
function sysBOOTPGet (args)
    local resultTab1 = {}
    local resultTab2 = {}

    row = db.getRow ("bootpc", "bootpc._ROWID_", 1)
    if (row ~= nil) then
       resTab.insertYNField (resultTab1, row["bootpc.bootpUpgrade"] == "1",
			    "Enable Bootp upgrade")
       resTab.insertYNField (resultTab1, row["bootpc.bootTimeCheck"] == "1",
			    "Disable bootp check at boot time")
       resTab.insertField (resultTab1, "Bootp Server", row["bootpc.bootpServer"])
    else
       resTab.insertField (resultTab1, "", "BOOTP disabled")
    end
    
    cmd = db.getAttribute("environment","name","BOOTPC_DB_PROGRAM","value")
    cmd = cmd .. " " .. DB_FILE_NAME .. " " .. "bootpc"
    schedGet (resultTab2, cmd)

    printLabel ("Bootp Configuration")
    resTab.print (resultTab1, 0)
    printLabel ("Bootp Schedule Configuration")
    resTab.print (resultTab2, 0)
end

--
--
--
function sysUsageRepGet (args)
    local resultTab = {}

    schedGet (resultTab, "/pfrm2.0/bin/ifDevUsageRecord " .. DB_FILE_NAME)
    printLabel ("Usage Reports Schedule Configurations")
    resTab.print (resultTab, 0)    	
end

--
-- main function to get SNMP configuration information
-- 
function sysSNMPGet ()
    local resultTab = {}
    local resultTabACL = {}
    sysCfgTab = sysSNMPConfigGet()
    if (sysCfgTab ~= nil) then
       printLabel ("SNMP System Configuration")
       resTab.print (sysCfgTab, 0)
    end
       
    if (agentIP == nil) then
        local snmpTab = db.getTable("snmpTrap")
	local idx = 0
	for k,v in pairs(snmpTab) do
	    idx = idx + 1
	    local row = snmpTab[idx]
	    buildSNMP (resultTab, row)
        end
    else 
	local row = db.getRow ("snmpTrap", "snmpTrap.ipAddr", agentIP)	
	buildSNMP (resultTab, row)
    end
    printLabel ("Trap Agent IP Address")
    resTab.print (resultTab, 0)
    local snmpACL = db.getTable ("snmpAccessControl")
    local idxi = 0
    for k,v in pairs(snmpACL) do
        idxi = idxi + 1
        local row = snmpACL[idxi]
        buildSNMPACL(resultTabACL, row)
    end
    printLabel ("Access Control List")
    resTab.print (resultTabACL, 0)
end    
