-- utilties.lua
-- Gaurav Mathur
-- TeamF1
-- www.TeamF1.com

-- Modification History
-- 30apr10,db modified Factory Reset routine
-- 09apr10,db modified USB related routines to support udev
-- 24feb10,db modified added support for cat,caopy,mount,unmount,md5sum
-- 23dec07,gnm modified utilCLIFactoryReset to prompt for user confirmation
-- 07dec07 gnm written
--
-- Description
-- CLI utility routines

--
-- This routine implements the ping command for the CLI
function utilCLIPing(ipToPing)
    local pingprog = db.getAttribute ("environment", "name", "PING_PROGRAM", 
	  "value")
    local pingfile = db.getAttribute("environment", "name", "PING_FILE_NAME", 
	  "value")
    local x = os.execute (pingprog .. " " .. ipToPing)
    printCLIError (x)
end

--
-- This routine implements the cat command for the CLI
function utilCLICat(fileName)
    local x = os.execute ("/bin/cat " .. fileName)
end

--
-- This routine implements the reboot command for the CLI
function utilCLIReboot(args)
    db.setAttribute("reboot", "reboot._ROWID_", "1", "reboot", "1")
    local restartTime = db.getAttribute("environment", "name", "BOOTUP_TIME", "value")
    statusMessage = "Router will be up in " .. restartTime .. " seconds."
    printCLIError (statusMessage)	
end

--
-- This routine implements the md5sum command for the CLI
function utilCLIMd5sum(fileName)
    local x = os.execute ("/usr/bin/md5sum " .. fileName)
end

--
-- This routine implements the copy command for the CLI
function utilCLICopy(fileName, fileName1)
    local x = os.execute ("/bin/cp " .. fileName .. " " ..fileName1)
end

--
-- This routine implements the CLI command to restore factory default
-- settings on the system.
function utilCLIFactoryReset(args)
    local triesMax = 5 -- hard-coded
    local triesCur = 0
    local confirm = ""
    local message = "Preparing to restore saved settings from user-provided " .. 
		  "file.\nWARNING: Current configuration will be erased.\n"

    printCLIError (message)
    while (confirm ~= "Y" and confirm ~= "N" and triesCur < triesMax) do
	  printCLIError ("Are you sure you want to proceed [Y/N]?")
	  confirm = io.stdin:read'*l'
	  triesCur = triesCur + 1
    end

    if (confirm == "N" or (triesCur == triesMax)) then return end
    -- set factory as settings
	local fact_reset_program = db.getAttribute("environment", "name", "FACTORY_RESET_SCRIPT", "value")
	os.execute(fact_reset_program)

    utilCLIReboot()
end

--
-- This routine implements the firmwareVersion command for the CLI
function utilCLIFirmVer(args)
    local firmwareVer = db.getAttribute("system", "_ROWID_", "1", "firmwareVer")
    print ("Firmware Version : ".. firmwareVer)
end


--
--This routine is the helper rouine for utilCLIUsbTest
--
function utilCLIUsbTestHelp (ipAddr, fileName, folderName)
        
    x = os.execute ("tftp -g -r ".. fileName .." -l "..folderName..fileName.." ".. ipAddr)
    if ( x == 256) then
        print ("TFTP failed ")	
        return false
    end
    print ("Md5sum: ")
    x = os.execute ("/usr/bin/md5sum "..folderName..fileName)
    if ( x == 256) then
        print ("Md5Sum get Failed ")	
        return false
    end
    x = os.execute ("rm -rf "..folderName..fileName)
    if ( x == 256) then
        print ("File emoving Failed")	
        return false
    end
end


--
-- This routine implements the USB Testing command for the CLI
function utilCLIUsbTest(ipAddr, fileName)

    local modelName =  db.getAttribute("system", "_ROWID_", "1", "name")
    if (string.find (modelName,"DSR") == nil) then
        print ("This box doesn't belong to DSR family")
    end
    if (string.find (modelName,"1000") ~= nil) then
        file1 = io.open ("/var/usb1Info","r")
        file2 = io.open ("/var/usb2Info","r")
        if (file1 == nil or file2 == nil) then
            print ("Please Connect Devices to USB Ports")
            return false
        end
        fileContent1 = file1:read("*line")
        fileContent2 = file2:read("*line")
        if (string.find (fileContent1, "storage") == nil or string.find (fileContent2, "storage") == nil) then
            print ("Device connected is not storage device")
            return false
        end
        print ("Test for USB1:")
        utilCLIUsbTestHelp (ipAddr, fileName, "/usb1/")
        print ("Test for USB2:")
        utilCLIUsbTestHelp (ipAddr, fileName, "/usb2/")
    elseif (string.find (modelName,"500") ~= nil) then
        file1 = io.open ("/var/usb1Info","r")
        if (file1 == nil) then
            print ("Please Connect Device to USB Port")
            return false
        end
        fileContent1 = file1:read("*line")
        if (string.find (fileContent1, "storage") == nil) then
            print ("Device connected is not storage device")
            return false
        end
        print ("Test for USB1:")
        utilCLIUsbTestHelp (ipAddr, fileName, "/usb1/")
    else
        print ("This box doesn't belong to DSR-1000 or DSR-500 family")
    end
end

