--[[
#### Abhinendra Babu. T
#### TeamF1
#### www.TeamF1.com
#### Feb 29, 2008
#### File: vipSecure.lua
#### Description: ipsecVPN functions (ikePolicy, vpnPolicy, vpnWizard, modeConfig)
#### Revisions:
]]--

--************* Requires *************

--************* Initial Code *************

--package VPN Configurations
vipSecure = {}

--************* Functions *************

-- ikePolicy config
function vipSecure.ikeConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
    	return "ACCESS_DENIED", "ADMIN_REQD"
    end
    
   --[[ if (operation == "add") then
	local vpnTable = db.getTable("IpsecIKEPolicy")
	if (#vpnTable == 10) then
	    return "ERROR", "MAX_VPN_LIMIT_REACHED"
	end
    end
    --]] 
       	
    if (db.existsRow("IpsecIKEPolicy", "IKEPolicyName", inputTable["IpsecIKEPolicy.IKEPolicyName"])) then
	   	local id = db.existsRow("IpsecIKEPolicy", "IKEPolicyName", inputTable["IpsecIKEPolicy.IKEPolicyName"])  	
	    if(id ~= rowid) then
	   		return "ERROR", "IKEP_NAME_CONFIG_FAILED"
	   	end
    end   
    --db.beginTransaction() --begin transaction
    local valid = false
    
    -- Setting Local and Remote Identifiers in edit Operation
    if( operation == "edit" ) then
         if(inputTable["IpsecIKEPolicy.LocalIdentifierType"] == "0") then
	     	inputTable["IpsecIKEPolicy.LocalIdentifier"] = ""             
	 	 end
         
	 	if(inputTable["IpsecIKEPolicy.RemoteIdentifierType"] == "0") then
	     	inputTable["IpsecIKEPolicy.RemoteIdentifier"] = ""             
	    end
    end

    if (tonumber(inputTable["IpsecIKEPolicy.XAUTHType"]) > 1) then
	    inputTable["IpsecIKEPolicy.XAUTHType"]=inputTable["IpsecIKEPolicy.XAUTHType1"]
    else
	    inputTable["IpsecIKEPolicy.XAUTHType"]=inputTable["IpsecIKEPolicy.XAUTHType"]
    end

    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            valid = db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            valid = db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
            valid = db.delete(DBTable, inputTable)
        end
    end

    -- return
    if (valid) then
       -- db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        --db.rollback()
        return "ERROR", "IKEP_CONFIG_FAILED"
    end
end

-- ikePolicy Deletion
function vipSecure.ikeDeletePolicy (rowIds)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
    	return "ACCESS_DENIED", "ADMIN_REQD"
    end
    -- db.beginTransaction() --begin transaction
    local valid = true

    -- delete routes
    for k,v in pairs(rowIds) do      	
        local inTable = {}
        local vpnRow = db.getRow("IpsecVPNPolicy","_ROWID_",v)
        if(vpnRow["IpsecVPNPolicy.PolicyType"] == "1") then
        	local ikeName = db.getAttribute("IpsecVPNPolicy", "_ROWID_", v, "IKEPolicyName")         	
        	inTable["IpsecIKEPolicy._ROWID_"] = db.getAttribute("IpsecIKEPolicy","IKEPolicyName",ikeName,"_ROWID_")
    		
	    --[[	if db.existsRow("IpsecVPNPolicy", "IKEPolicyName", ikeName) then
	    	    return "ERROR", "IKEP_DELETE_DEPENDANT_FAILED"
	    	end
	    --]]
	    	valid = db.delete("IpsecIKEPolicy", inTable) 
	    	if(valid==false) then
	    	    break;
	    	end
    	end
    end	

    -- return
    if (valid) then
       -- db.commitTransaction(true)
        return "OK", "IKEP_DELETE_SUCCESS"
    else
        db.rollback()
        return "ERROR", "IKEP_DELETE_FAILED"
    end
end


-- Checking the usage of ikePolicy before Editing
function vipSecure.ikeEditCheck(Rowid)
    local tmpRow = db.getRow("IpsecIKEPolicy", "_ROWID_", Rowid)
    local tmpTable = db.getRows ("IpsecVPNPolicy", "IKEPolicyName", tmpRow["IpsecIKEPolicy.IKEPolicyName"])
    local i = 0
    local exist = "1"
    local usingAsBackupPolicy = false
    local backupPolicyUp = false
    if(tmpTable) then
        for k,v in pairs(tmpTable) do
		    i = i + 1
            local row = tmpTable[i]
 	        if(row["IpsecVPNPolicy.Status"] == "1") then
 	            exist = "0"
                break
 	        end
 	        if(row["IpsecVPNPolicy.TypeofPolicy"] ~= nil and
               row["IpsecVPNPolicy.TypeofPolicy"] == "2") then
 	            usingAsBackupPolicy = true
                break
 	        end
            -- Check if back up policy is up and running.
            if (row["IpsecVPNPolicy.RVGStatus"] == "1") then
                local bckPolicyStatus = db.getAttribute ("IpsecVPNPolicy", "VPNPolicyName", row["IpsecVPNPolicy.BackupPolicyName"], "Status") or ''
                    if (bckPolicyStatus == "1") then
                       	backupPolicyUp = true
                   	    break
                    end
            end
 	    end
 	end
    
    if(exist == "0") then
        return "ERROR", "IKEP_EDITCHECK_FAILED"
    elseif(usingAsBackupPolicy) then
        return "ERROR", "IKEP_USING_BY_BACKUP_POLICY"
    elseif(backupPolicyUp) then
        return "ERROR", "BCK_POLICY_UP_AND_RUNNING"
    else
        return "OK", "STATUS_OK"
    end
end

function vipSecure.getDulicatePolicies (inputTable)
    local vpnPolicies = nil
    if (inputTable == nil) then
        return vpnPolicies
    end
    if (inputTable["IpsecVPNPolicy.RemoteNetworkType"] == "0") then
        if (inputTable["IpsecVPNPolicy.LocalNetworkType"] == "0") then
            vpnPolicies = db.getRowsWhere ("IpsecVPNPolicy","Status = 1 and RemoteNetworkType = 0 and LocalNetworkType = 0")
        elseif (inputTable["IpsecVPNPolicy.LocalNetworkType"] == "1") then
            vpnPolicies = db.getRowsWhere ("IpsecVPNPolicy","Status = 1 and RemoteNetworkType = 0 and LocalNetworkType = 1 and LocalStartAddress = '" .. inputTable["IpsecVPNPolicy.LocalStartAddress"] .. "'")
        elseif (inputTable["IpsecVPNPolicy.LocalNetworkType"] == "2") then
            vpnPolicies = db.getRowsWhere ("IpsecVPNPolicy","Status = 1 and RemoteNetworkType = 0 and LocalNetworkType = 2 and LocalStartAddress = '" .. inputTable["IpsecVPNPolicy.LocalStartAddress"] .. "' and LocalEndAddress = '" .. inputTable["IpsecVPNPolicy.LocalEndAddress"] .. "'")
        elseif (inputTable["IpsecVPNPolicy.LocalNetworkType"] == "3") then
            vpnPolicies = db.getRowsWhere ("IpsecVPNPolicy","Status = 1 and RemoteNetworkType = 0 and LocalNetworkType = 3 and LocalStartAddress = '" .. inputTable["IpsecVPNPolicy.LocalStartAddress"] .. "' and LocalSubnetMask = '" .. inputTable["IpsecVPNPolicy.LocalSubnetMask"] .. "'")
        end
    elseif (inputTable["IpsecVPNPolicy.RemoteNetworkType"] == "1") then
        if (inputTable["IpsecVPNPolicy.LocalNetworkType"] == "0") then
            vpnPolicies = db.getRowsWhere ("IpsecVPNPolicy","Status = 1 and RemoteNetworkType = 1 and RemoteStartAddress = '" .. inputTable["IpsecVPNPolicy.RemoteStartAddress"] .. "' and LocalNetworkType = 0")
        elseif (inputTable["IpsecVPNPolicy.LocalNetworkType"] == "1") then
            vpnPolicies = db.getRowsWhere ("IpsecVPNPolicy","Status = 1 and RemoteNetworkType = 1 and RemoteStartAddress = '" .. inputTable["IpsecVPNPolicy.RemoteStartAddress"] .. "' and LocalNetworkType = 1 and LocalStartAddress = '" .. inputTable["IpsecVPNPolicy.LocalStartAddress"] .. "'")
        elseif (inputTable["IpsecVPNPolicy.LocalNetworkType"] == "2") then
            vpnPolicies = db.getRowsWhere ("IpsecVPNPolicy","Status = 1 and RemoteNetworkType = 1 and RemoteStartAddress = '" .. inputTable["IpsecVPNPolicy.RemoteStartAddress"] .. "' and LocalNetworkType = 2 and LocalStartAddress = '" .. inputTable["IpsecVPNPolicy.LocalStartAddress"] .. "' and LocalEndAddress = '" .. inputTable["IpsecVPNPolicy.LocalEndAddress"] .. "'")
        elseif (inputTable["IpsecVPNPolicy.LocalNetworkType"] == "3") then
            vpnPolicies = db.getRowsWhere ("IpsecVPNPolicy","Status = 1 and RemoteNetworkType = 1 and RemoteStartAddress = '" .. inputTable["IpsecVPNPolicy.RemoteStartAddress"] .. "' and LocalNetworkType = 3 and LocalStartAddress = '" .. inputTable["IpsecVPNPolicy.LocalStartAddress"] .. "' and LocalSubnetMask = '" .. inputTable["IpsecVPNPolicy.LocalSubnetMask"] .. "'")
        end
    elseif (inputTable["IpsecVPNPolicy.RemoteNetworkType"] == "2") then
        if (inputTable["IpsecVPNPolicy.LocalNetworkType"] == "0") then
            vpnPolicies = db.getRowsWhere ("IpsecVPNPolicy","Status = 1 and RemoteNetworkType = 2 and RemoteStartAddress = '" .. inputTable["IpsecVPNPolicy.RemoteStartAddress"] .. "' and RemoteEndAddress = '" .. inputTable["IpsecVPNPolicy.RemoteEndAddress"] .. "' and LocalNetworkType = 0")
        elseif (inputTable["IpsecVPNPolicy.LocalNetworkType"] == "1") then
            vpnPolicies = db.getRowsWhere ("IpsecVPNPolicy","Status = 1 and RemoteNetworkType = 2 and RemoteStartAddress = '" .. inputTable["IpsecVPNPolicy.RemoteStartAddress"] .. "' and RemoteEndAddress = '" .. inputTable["IpsecVPNPolicy.RemoteEndAddress"] .. "' and LocalNetworkType = 1 and LocalStartAddress = '" .. inputTable["IpsecVPNPolicy.LocalStartAddress"] .. "'")
        elseif (inputTable["IpsecVPNPolicy.LocalNetworkType"] == "2") then
            vpnPolicies = db.getRowsWhere ("IpsecVPNPolicy","Status = 1 and RemoteNetworkType = 2 and RemoteStartAddress = '" .. inputTable["IpsecVPNPolicy.RemoteStartAddress"] .. "' and RemoteEndAddress = '" .. inputTable["IpsecVPNPolicy.RemoteEndAddress"] .. "' and LocalNetworkType = 2 and LocalStartAddress = '" .. inputTable["IpsecVPNPolicy.LocalStartAddress"] .. "' and LocalEndAddress = '" .. inputTable["IpsecVPNPolicy.LocalEndAddress"] .. "'")
        elseif (inputTable["IpsecVPNPolicy.LocalNetworkType"] == "3") then
            vpnPolicies = db.getRowsWhere ("IpsecVPNPolicy","Status = 1 and RemoteNetworkType = 2 and RemoteStartAddress = '" .. inputTable["IpsecVPNPolicy.RemoteStartAddress"] .. "' and RemoteEndAddress = '" .. inputTable["IpsecVPNPolicy.RemoteEndAddress"] .. "' and LocalNetworkType = 3 and LocalStartAddress = '" .. inputTable["IpsecVPNPolicy.LocalStartAddress"] .. "' and LocalSubnetMask = '" .. inputTable["IpsecVPNPolicy.LocalSubnetMask"] .. "'")
        end
    elseif (inputTable["IpsecVPNPolicy.RemoteNetworkType"] == "3") then
        if (inputTable["IpsecVPNPolicy.LocalNetworkType"] == "0") then
            vpnPolicies = db.getRowsWhere ("IpsecVPNPolicy","Status = 1 and RemoteNetworkType = 3 and RemoteStartAddress = '" .. inputTable["IpsecVPNPolicy.RemoteStartAddress"] .. "' and RemoteSubnetMask = '" .. inputTable["IpsecVPNPolicy.RemoteSubnetMask"] .. "' and LocalNetworkType = 0")
        elseif (inputTable["IpsecVPNPolicy.LocalNetworkType"] == "1") then
            vpnPolicies = db.getRowsWhere ("IpsecVPNPolicy","Status = 1 and RemoteNetworkType = 3 and RemoteStartAddress = '" .. inputTable["IpsecVPNPolicy.RemoteStartAddress"] .. "' and RemoteSubnetMask = '" .. inputTable["IpsecVPNPolicy.RemoteSubnetMask"] .. "' and LocalNetworkType = 1 and LocalStartAddress = '" .. inputTable["IpsecVPNPolicy.LocalStartAddress"] .. "'")
        elseif (inputTable["IpsecVPNPolicy.LocalNetworkType"] == "2") then
            vpnPolicies = db.getRowsWhere ("IpsecVPNPolicy","Status = 1 and RemoteNetworkType = 3 and RemoteStartAddress = '" .. inputTable["IpsecVPNPolicy.RemoteStartAddress"] .. "' and RemoteSubnetMask = '" .. inputTable["IpsecVPNPolicy.RemoteSubnetMask"] .. "' and LocalNetworkType = 2 and LocalStartAddress = '" .. inputTable["IpsecVPNPolicy.LocalStartAddress"] .. "' and LocalEndAddress = '" .. inputTable["IpsecVPNPolicy.LocalEndAddress"] .. "'")
        elseif (inputTable["IpsecVPNPolicy.LocalNetworkType"] == "3") then
            vpnPolicies = db.getRowsWhere ("IpsecVPNPolicy","Status = 1 and RemoteNetworkType = 3 and RemoteStartAddress = '" .. inputTable["IpsecVPNPolicy.RemoteStartAddress"] .. "' and RemoteSubnetMask = '" .. inputTable["IpsecVPNPolicy.RemoteSubnetMask"] .. "' and LocalNetworkType = 3 and LocalStartAddress = '" .. inputTable["IpsecVPNPolicy.LocalStartAddress"] .. "' and LocalSubnetMask = '" .. inputTable["IpsecVPNPolicy.LocalSubnetMask"] .. "'")
        end
    end

    return vpnPolicies
end

-- ipsecVPN enable
function vipSecure.ipsecVPNPolicyEnable (rowids, enable)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    db.beginTransaction() --begin transaction
    local valid = false
    local isIkeUpdate = false
    local id = nil
    local ikeTable = {}
    local inputTable = {}
    
    if (enable) then
        inputTable["IpsecVPNPolicy.Status"] = "1"
    else
        inputTable["IpsecVPNPolicy.Status"] = "0"
    end
    local isNotUpdated = true

    for k,v in pairs(rowids) do    	
        local oldStatus = db.getAttribute("IpsecVPNPolicy", "_ROWID_", v, "Status")
        -- Check the local and remote traffic selector information
        -- In case of transport mode we should check for remote endpoint only.
        -- In case of Tunnel mode and if dhcp enabled over ipsec is enabled we should check for remote endpoint only
        -- In case of Tunnel mode and if dhcp over ipsec is disabled then we should check for Local network type and remote network type.
        if (enable) then
            local ipsecVpnConfig = db.getRow("IpsecVPNPolicy", "_ROWID_", v)           
            if(ipsecVpnConfig["IpsecVPNPolicy.TunnelOrTransport"] == "1" and  ipsecVpnConfig["IpsecVPNPolicy.EnableDhcpOverIPSec"] == "0") then
                local vpnPolicies = vipSecure.getDulicatePolicies (ipsecVpnConfig)
                if (vpnPolicies ~= nil and
                    #vpnPolicies == 1 and
                    vpnPolicies[1]["IpsecVPNPolicy._ROWID_"] ~= v and
                    vpnPolicies[1]["IpsecVPNPolicy.Status"] == "1" and vpnPolicies[1]["IpsecVPNPolicy.TunnelOrTransport"] == "1" and vpnPolicies[1]["IpsecVPNPolicy.EnableDhcpOverIPSec"] == "0") then
                    db.rollback()
                    return "ERROR", "CANT_ENABLE_TWO_POLICIES_WITH_SAME_TRAFFIC_SELECTORS"
                end            
            elseif(ipsecVpnConfig["IpsecVPNPolicy.TunnelOrTransport"] == "0") then                
                local sameRemoteEndPointRows = db.getRowsWhere("IpsecVPNPolicy","RemoteEndPoint = '" .. ipsecVpnConfig["IpsecVPNPolicy.RemoteEndPoint"] .. "'")                
                    if(sameRemoteEndPointRows ~= nil) then
                        util.appendDebugOut("I am here")
                        local j=0
                        for k,t in pairs(sameRemoteEndPointRows) do
                            j = j+1
                            local row = sameRemoteEndPointRows[j]
                            util.appendDebugOut("Status"..row["IpsecVPNPolicy.Status"])                            
                            if (v ~= row["IpsecVPNPolicy._ROWID_"] and row["IpsecVPNPolicy.Status"] == "1" ) then
                                db.rollback()
                                return "ERROR", "IPSEC_VPN_REMOTE_CONFIG_FAILED"
                            end
                        end
                   end               
            elseif(ipsecVpnConfig["IpsecVPNPolicy.TunnelOrTransport"] == "1" and ipsecVpnConfig["IpsecVPNPolicy.EnableDhcpOverIPSec"] == "1")then
                if(db.existsRow("IpsecVPNPolicy", "RemoteEndPoint", ipsecVpnConfig["IpsecVPNPolicy.RemoteEndPoint"]))then
                    local id = db.existsRow("IpsecVPNPolicy", "RemoteEndPoint", ipsecVpnConfig["IpsecVPNPolicy.RemoteEndPoint"])
                    if (id ~= ipsecVpnConfig["IpsecVPNPolicy._ROWID_"]) then
                        db.rollback()
                        return "ERROR", "IPSEC_VPN_REMOTE_CONFIG_FAILED"
                    end
                end
            end
        end

        -- Checking if already Enabled/Disabled
        if (oldStatus ~= inputTable["IpsecVPNPolicy.Status"]) then
            isNotUpdated = false
            -- check if policy is not manual policy
            if (db.getAttribute("IpsecVPNPolicy", "_ROWID_", v, "PolicyType") ~= "2") then
                local ikeName = db.getAttribute("IpsecVPNPolicy", "_ROWID_", v,  "IKEPolicyName") or ""
    	        id = db.getAttribute("IpsecIKEPolicy", "IKEPolicyName", ikeName, "_ROWID_")
                if (id ~= nil) then
                    ikeTable = db.getRow ("IpsecIKEPolicy", "_ROWID_", id)
                    if (enable) then
                        -- Check if update required Local and Remote Identifiers.
                        if (ikeTable["IpsecIKEPolicy.LocalIdentifier"] == "" or ikeTable["IpsecIKEPolicy.RemoteIdentifier"] == "") then
                            -- updating Local and Remote Identifiers.
                            local wanConfig = {}
                            local identifier = db.getAttribute("IpsecIKEPolicy", "_ROWID_", id, "LocalIdentifierType")
                            -- Checking if LocalIdentifier Type is IP Address or not
            	            if (identifier ~= nil and identifier == "0") then
                                local gateway = db.getAttribute("IpsecVPNPolicy", "_ROWID_", v, "LocalGateway")
                                -- Checking Gateway is WAN1 or WAN2
                                if (gateway == "0") then
	    	                        wanConfig = db.getRowWhere("ipAddressTable", "LogicalIfName = 'WAN1' AND addressFamily = 2")
            		            else
                                    wanConfig = db.getRowWhere("ipAddressTable", "LogicalIfName = 'WAN2' AND addressFamily = 2")
    	            	        end
                                ikeTable["IpsecIKEPolicy.LocalIdentifier"] = ""
                                if (wanConfig ~= nil) then
                                    ikeTable["IpsecIKEPolicy.LocalIdentifier"] = wanConfig["ipAddressTable.ipAddress"]
                                end
                            end
                            local identifier = db.getAttribute("IpsecIKEPolicy", "_ROWID_", id, "RemoteIdentifierType")
                            -- Checking if RemoteIdentifier Type is IP Address or not
                	        if (identifier ~= nil and identifier == "0") then
                                ikeTable["IpsecIKEPolicy.RemoteIdentifier"] = db.getAttribute("IpsecVPNPolicy", "_ROWID_", v, "RemoteEndPoint") or ''
                            end
                            isIkeUpdate = true
                        end
                    else
                        -- In Disable case we should clear 
                        -- Local and Remote End Points.
                        -- If all VPN Policies which are using this IKE policy
                        -- are disabled.
                        local vpnPoliciesTable = db.getRowsWhere ("IpsecVPNPolicy", "IKEPolicyName = '" .. ikeName .. "'")
                        isIkeUpdate = true
                        local i = 0
                        for m,n in pairs(vpnPoliciesTable) do
                            i = i + 1
                            local vpnPolicy = vpnPoliciesTable[i]
                            if (vpnPolicy["IpsecVPNPolicy.Status"] == "1" and vpnPolicy["IpsecVPNPolicy._ROWID_"] ~= v) then
                                isIkeUpdate = false
                                break
                            end
                        end
                        if (isIkeUpdate) then
                            if (ikeTable["IpsecIKEPolicy.LocalIdentifierType"] == "0") then
                                ikeTable["IpsecIKEPolicy.LocalIdentifier"] = ""
                            end
                            if (ikeTable["IpsecIKEPolicy.RemoteIdentifierType"] == "0") then
                                ikeTable["IpsecIKEPolicy.RemoteIdentifier"] = ""
                            end
                        end
                    end
                end
            end
            if (enable) then
                local priPolicyConfig = db.getRow ("IpsecVPNPolicy", "_ROWID_", v)
                -- Check if backup policy ids up and running.
                if (priPolicyConfig["IpsecVPNPolicy.RVGStatus"] == "1")then
                    local bckPolicyStatus = db.getAttribute ("IpsecVPNPolicy", "VPNPolicyName", priPolicyConfig["IpsecVPNPolicy.BackupPolicyName"], "Status") or ''
                    if (bckPolicyStatus == "1") then
                       	db.rollback()
                   	    return "ERROR", "BCK_POLICY_UP_AND_RUNNING"
                    end
                end

                if (priPolicyConfig["IpsecVPNPolicy.PolicyType"] == "1" and
                    priPolicyConfig["IpsecVPNPolicy.RVGStatus"] == "1" and
                    priPolicyConfig["IpsecVPNPolicy.BackupPolicyName"] ~= nil) then
                    configStatus, errCode = vipSecure.rvgConfigCheck (v)
                    if (configStatus ~= "OK") then
                       	db.rollback()
                   	    return "ERROR", errCode
                    end
                end
            end

            valid = db.update("IpsecVPNPolicy", inputTable, v)
            if (valid and isIkeUpdate) then
                valid = db.update("IpsecIKEPolicy", ikeTable, id)
                id = nil
                ikeTable = {}
                isIkeUpdate = false
                if (not valid) then
                    break
                end
            end
        end
    end

    if (isNotUpdated) then
        db.commitTransaction(true)
        if (enable) then
            return "OK", "IPSEC_VPN_ALREADY_ENABLED"
        else
            return "OK", "IPSEC_VPN_ALREADY_DISABLED"
        end        
    end
	
    -- return
    if (valid) then
        db.commitTransaction(true)
        if (enable) then
            return "OK", "IPSEC_VPN_ENABLE_SUCCESS"
        else
            return "OK", "IPSEC_VPN_DISABLE_SUCCESS"
        end
	else
		db.rollback()
        if (enable) then
            return "ERROR", "IPSEC_VPN_ENABLE_FAILED"
        else
            return "ERROR", "IPSEC_VPN_DISABLE_FAILED"
        end
    end
end


-- IPSEC VPN Policy Config
function vipSecure.ipsecVPNPolicyConfig(inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
    	return "ACCESS_DENIED", "ADMIN_REQD"
    end

    if (operation == "add") then
	    local vpnTable = db.getTable("IpsecVPNPolicy")
    	if (#vpnTable == 70) then
	        return "ERROR", "MAX_VPN_LIMIT_REACHED"
    	end
    end
    
    if (operation == "edit") then
        local policyConfig = db.getRow ("IpsecVPNPolicy", "_ROWID_", rowid)        
        if (policyConfig["IpsecVPNPolicy.RVGStatus"] == "1") then
            local bckPolicyStatus = db.getAttribute ("IpsecVPNPolicy", "VPNPolicyName", policyConfig["IpsecVPNPolicy.BackupPolicyName"], "Status") or ''
            if (bckPolicyStatus == "1") then
                local allKeys = db.getColNames("IpsecVPNPolicy")
                local isChanged = false
            	for k,v in pairs(allKeys) do
                    if ((v == "BackupPolicyName" and
                         inputTable["IpsecVPNPolicy" .. "." .. v] ~= nil and
                         inputTable["IpsecVPNPolicy" .. "." .. v] ~= policyConfig["IpsecVPNPolicy" .. "." .. v]) or
                        (v == "FailbackTime" and
                         inputTable["IpsecVPNPolicy" .. "." .. v] ~= nil and
                         inputTable["IpsecVPNPolicy" .. "." .. v] ~= policyConfig["IpsecVPNPolicy" .. "." .. v]) or
                        (v ~= "RVGStatus" and v ~= "BackupPolicyName" and v ~= "FailbackTime" and
                        inputTable["IpsecVPNPolicy" .. "." .. v] ~= nil and
                        inputTable["IpsecVPNPolicy" .. "." .. v] ~= policyConfig["IpsecVPNPolicy" .. "." .. v])) then
                        isChanged = true
                        break
                    end
                end
                if (isChanged) then
                    return "ERROR", "ONYL_RVG_STATUS_ALLOW_TO_CHANGE"
                end 
            end
        end
    end
    if (inputTable["IpsecVPNPolicy.PolicyType"] == "1") then
        -- For auto policies type, remote end point information should match with the
        -- vpn policies whcih are using same ike policy.
      --[[  local ikePolicyName = inputTable["IpsecVPNPolicy.IKEPolicyName"]
        local vpnPolicies = db.getRowsWhere ("IpsecVPNPolicy" , "IKEPolicyName = '" .. ikePolicyName .. "'")
        if ((#vpnPolicies == 1 and vpnPolicies[1]["IpsecVPNPolicy._ROWID_"] ~= rowid) or
            (#vpnPolicies > 1)) then
            local vpnPolicy = vpnPolicies[1]
            if (vpnPolicy["IpsecVPNPolicy.RemoteEndPointType"] ~= inputTable["IpsecVPNPolicy.RemoteEndPointType"] or
                vpnPolicy["IpsecVPNPolicy.RemoteEndPoint"] ~= inputTable["IpsecVPNPolicy.RemoteEndPoint"] or
                vpnPolicy["IpsecVPNPolicy.LocalGateway"] ~= inputTable["IpsecVPNPolicy.LocalGateway"]) then
                return "ERROR", "IPSEC_CANT_SHARE_IKE_FOR_DIFF_ENDPOINTS"
            end
        end  --]] 
    else
        if (db.existsRow("IpsecVPNPolicy", "RemoteEndPoint", inputTable["IpsecVPNPolicy.RemoteEndPoint"])) then
            local id = db.existsRow("IpsecVPNPolicy", "RemoteEndPoint", inputTable["IpsecVPNPolicy.RemoteEndPoint"])
	        if (id ~= rowid) then	        	
	            return "ERROR", "IPSEC_VPN_REMOTE_CONFIG_FAILED"
        	end
        end
    end

    if (db.existsRow("IpsecVPNPolicy", "VPNPolicyName", inputTable["IpsecVPNPolicy.VPNPolicyName"])) then
    	local id = db.existsRow("IpsecVPNPolicy", "VPNPolicyName", inputTable["IpsecVPNPolicy.VPNPolicyName"])
	        if (id ~= rowid) then
	            return "ERROR", "IPSEC_VPN_NAME_CONFIG_FAILED"
	        end
	end

	-- setting AutoPolicyType Gateway(0) / Client(1)
    if (inputTable["IpsecVPNPolicy.PolicyType"] == "1" or inputTable["IpsecVPNPolicy.PolicyType"] == "3") then
        if (inputTable["IpsecVPNPolicy.RemoteEndPointType"] == "1" and inputTable["IpsecVPNPolicy.RemoteNetworkType"] == "0" ) then
	        inputTable["IpsecVPNPolicy.AutoPolicyType"] = "1"
    	else
	        inputTable["IpsecVPNPolicy.AutoPolicyType"] = "0"
    	end	
    else
    	inputTable["IpsecVPNPolicy.AutoPolicyType"] = "0"
    end

    -- Making last octet as '0' in case of subnet configuration
    if (inputTable["IpsecVPNPolicy.LocalNetworkType"] == "3" and
        inputTable["IpsecVPNPolicy.LocalStartAddress"] ~= nil) then
        local ipaddr = util.split(inputTable["IpsecVPNPolicy.LocalStartAddress"] or "",".")
        ipaddr[4] = 0
        inputTable["IpsecVPNPolicy.LocalStartAddress"] = util.join (ipaddr, ".")
    end
    if (inputTable["IpsecVPNPolicy.RemoteNetworkType"] == "3" and
        inputTable["IpsecVPNPolicy.RemoteStartAddress"] ~= nil) then
        local ipaddr = util.split(inputTable["IpsecVPNPolicy.RemoteStartAddress"] or "",".")
        ipaddr[4] = 0
        inputTable["IpsecVPNPolicy.RemoteStartAddress"] = util.join (ipaddr, ".")
    end

    -- Checking the Local and Remote traffic selectors
    if(inputTable["IpsecVPNPolicy.TunnelOrTransport"] ~= "1" and inputTable["IpsecVPNPolicy.EnableDhcpOverIPSec"] ~= "1" or inputTable["IpsecVPNPolicy.TunnelOrTransport"] ~= "0") then
        local vpnPolicies = vipSecure.getDulicatePolicies (inputTable)
        if (operation == "add" and vpnPolicies ~= nil and #vpnPolicies ~= 0) then
            inputTable["IpsecVPNPolicy.Status"] = "0"               
                local ikePolicyName = inputTable["IpsecVPNPolicy.IKEPolicyName"]
                local vpnPolicies1 = db.getRowsWhere ("IpsecVPNPolicy" , "IKEPolicyName = '" .. ikePolicyName .. "'")
                if ((#vpnPolicies1 == 1 and vpnPolicies1[1]["IpsecVPNPolicy._ROWID_"] ~= rowid) or
                    (#vpnPolicies1 > 1)) then
                    local vpnPolicy = vpnPolicies1[1]
                    if (vpnPolicy["IpsecVPNPolicy.RemoteEndPointType"] == inputTable["IpsecVPNPolicy.RemoteEndPointType"] and
                        vpnPolicy["IpsecVPNPolicy.RemoteEndPoint"] == inputTable["IpsecVPNPolicy.RemoteEndPoint"]) then
                        return "ERROR", "IPSEC_VPN_INVALID_CONFIGURATION"
                    end
                end            
        elseif (operation == "edit" and vpnPolicies ~= nil and #vpnPolicies == 1 and vpnPolicies[1]["IpsecVPNPolicy._ROWID_"] ~= rowid) then
                inputTable["IpsecVPNPolicy.Status"] = "0"              
                local ikePolicyName = inputTable["IpsecVPNPolicy.IKEPolicyName"]
                local vpnPolicies1 = db.getRowsWhere ("IpsecVPNPolicy" , "IKEPolicyName = '" .. ikePolicyName .. "'")
                if ((#vpnPolicies1 == 1 and vpnPolicies1[1]["IpsecVPNPolicy._ROWID_"] ~= rowid) or
                    (#vpnPolicies1 > 1)) then
                    local vpnPolicy = vpnPolicies1[1]
                    if (vpnPolicy["IpsecVPNPolicy.RemoteEndPointType"] == inputTable["IpsecVPNPolicy.RemoteEndPointType"] and
                        vpnPolicy["IpsecVPNPolicy.RemoteEndPoint"] == inputTable["IpsecVPNPolicy.RemoteEndPoint"]) then
                        return "ERROR", "IPSEC_VPN_INVALID_CONFIGURATION"
                    end
                end            
        end
    end

   -- db.beginTransaction() --begin transaction
    local valid = false
    
    -- Add Policy
    if (operation == "add") then    
        if (inputTable["IpsecVPNPolicy.PolicyType"] ~= "2") then
            local ikeTable = db.getRow ("IpsecIKEPolicy", "IKEPolicyName", inputTable["IpsecVPNPolicy.IKEPolicyName"])
            local ikePolicyName = ikeTable["IpsecIKEPolicy.IKEPolicyName"] or ''
            local vpnPolicies = db.getRowsWhere ("IpsecVPNPolicy" , "IKEPolicyName = '" .. ikePolicyName .. "'")
            local isUpdateNeed = true
            -- Update required for IKE policy only if IKE policy is
            -- not sharing by any other vpn policies.
            for k,v in pairs(vpnPolicies) do
                if (v["IpsecIKEPolicy.Status"] == "1") then
                    isUpdateNeed = false
                end
            end

            if (isUpdateNeed) then
                isUpdateNeed = false
                if(ikeTable["IpsecIKEPolicy.LocalIdentifierType"] == "0") then
                    local wanConfig = {}
                    if (inputTable["IpsecVPNPolicy.LocalGateway"] == "0") then
                        wanConfig = db.getRowWhere("ipAddressTable", "LogicalIfName = 'WAN1' AND addressFamily = 2")
        	        else
                        wanConfig = db.getRowWhere("ipAddressTable", "LogicalIfName = 'WAN2' AND addressFamily = 2")
                    end
                    ikeTable["IpsecIKEPolicy.LocalIdentifier"] = ""
                    if (wanConfig ~= nil) then
                        ikeTable["IpsecIKEPolicy.LocalIdentifier"] = wanConfig["ipAddressTable.ipAddress"]
                    end
        	    end
            	idType = db.getAttribute("IpsecIKEPolicy", "IKEPolicyName", inputTable["IpsecVPNPolicy.IKEPolicyName"], "RemoteIdentifierType")
                if(ikeTable["IpsecIKEPolicy.RemoteIdentifierType"]  == "0") then
    	            ikeTable["IpsecIKEPolicy.RemoteIdentifier"] = inputTable["IpsecVPNPolicy.RemoteEndPoint"]
        	    end    
                valid = db.update("IpsecIKEPolicy", ikeTable, ikeTable["IpsecIKEPolicy._ROWID_"])
            else
                valid = true
            end
        else
            valid = true
        end

	-- Edit Policy
    elseif (operation == "edit") then
        local isIKEChanged = false
        ikeName = db.getAttribute("IpsecVPNPolicy", "_ROWID_", rowid, "IKEPolicyName")
        local remoteEndPoint = db.getAttribute("IpsecVPNPolicy", "_ROWID_", rowid, "RemoteEndPoint")
        --[[ Updating Local and Remote end points of IKE Policy ]]--
        if (ikeName ~= nil and ikeName ~= "") then
            -- Check if IKE Policy Changes
            -- Second condition true if Auto/Mode Config Policy Changed to Manual Policy
            if ((inputTable["IpsecVPNPolicy.IKEPolicyName"] ~= nil and ikeName ~= inputTable["IpsecVPNPolicy.IKEPolicyName"]) or inputTable["IpsecVPNPolicy.IKEPolicyName"] == nil) then
                isIKEChanged = true
                if (inputTable["IpsecVPNPolicy.IKEPolicyName"] == nil) then
                    inputTable["IpsecVPNPolicy.IKEPolicyName"] = ""
                end
                local ikeID = db.getAttribute("IpsecIKEPolicy", "IKEPolicyName", ikeName, "_ROWID_")
                if (ikeID ~= nil) then
                    local ikeOldTable = db.getRow ("IpsecIKEPolicy", "_ROWID_", ikeID)
                    local ikePolicyName = ikeOldTable["IpsecIKEPolicy.IKEPolicyName"] or ''
                    local vpnPolicies = db.getRowsWhere ("IpsecVPNPolicy" , "IKEPolicyName = '" .. ikePolicyName .. "'")
                    local isUpdateNeed = true
                    -- Update required for IKE policy only if IKE policy is
                    -- not sharing by any other vpn policies.
                    if (#vpnPolicies == 1) then
                        isUpdateNeed = true
                    else
                        for k,v in pairs(vpnPolicies) do
                            if (v["IpsecIKEPolicy._ROWID_"] ~= rowid and v["IpsecIKEPolicy.Status"] == "1") then
                                isUpdateNeed = false
                            end
                        end
                    end
                    if (isUpdateNeed) then
                        isUpdateNeed = false
                        if (ikeOldTable["IpsecIKEPolicy.LocalIdentifierType"] == "0") then
                            ikeOldTable["IpsecIKEPolicy.LocalIdentifier"] = ""
                            isUpdateNeed = true
                        end
                        if (ikeOldTable["IpsecIKEPolicy.RemoteIdentifierType"] == "0") then
                            ikeOldTable["IpsecIKEPolicy.RemoteIdentifier"] = ""
                            isUpdateNeed = true                    
                        end
                    end
                    if (isUpdateNeed) then
                        valid = db.update("IpsecIKEPolicy", ikeOldTable, ikeID)
                    else
                        valid = true
                    end
                end
            elseif (remoteEndPoint ~= inputTable["IpsecVPNPolicy.RemoteEndPoint"]) then
                valid = true
                isIKEChanged = true
            else
                valid = true
            end
        else
            -- Manual Policy changed to Auto/Mode Config Policy
            if (inputTable["IpsecVPNPolicy.IKEPolicyName"] ~= nil) then
                isIKEChanged = true
            end
            valid = true
       	end

    	local ikeTable = {}
	    if (valid) then
            -- Check if IKE Policy Changes
       	    if (inputTable["IpsecVPNPolicy.PolicyType"] ~= "2" and isIKEChanged == true) then
                id = db.getAttribute("IpsecIKEPolicy", "IKEPolicyName", inputTable["IpsecVPNPolicy.IKEPolicyName"], "_ROWID_")
                if (id ~= nil) then
                    local isUpdateNeed = false
                    local ikeTable = db.getRow ("IpsecIKEPolicy", "_ROWID_", id)
                    local ikePolicyName = ikeTable["IpsecIKEPolicy.IKEPolicyName"] or ''
                    local vpnPolicies = db.getRowsWhere ("IpsecVPNPolicy" , "IKEPolicyName = '" .. ikePolicyName .. "'")
                    local isUpdateNeed = true
                    -- Update required for IKE policy only if IKE policy is
                    -- not sharing by any other vpn policies.
                    for k,v in pairs(vpnPolicies) do
                        if (v["IpsecIKEPolicy.Status"] == "1") then
                            isUpdateNeed = false
                        end
                    end
                    local vpnPolStatus = db.getAttribute ("IpsecVPNPolicy", "_ROWID_", rowid, "Status") or ''
                    if (isUpdateNeed and vpnPolStatus == "1") then
                        isUpdateNeed = true
                        if (ikeTable["IpsecIKEPolicy.LocalIdentifierType"] == "0") then
                            isUpdateNeed = true
                            local wanConfig = {}
                            local gateway = db.getAttribute("IpsecVPNPolicy", "VPNPolicyName", inputTable["IpsecVPNPolicy.VPNPolicyName"], "LocalGateway")
                            if (gateway == "0") then
	                            wanConfig = db.getRowWhere("ipAddressTable", "LogicalIfName = 'WAN1' AND addressFamily = 2")
                            else
                                wanConfig = db.getRowWhere("ipAddressTable", "LogicalIfName = 'WAN2' AND addressFamily = 2")
            	            end
                            ikeTable["IpsecIKEPolicy.LocalIdentifier"] = ""
                            if (wanConfig ~= nil) then
                                ikeTable["IpsecIKEPolicy.LocalIdentifier"] = wanConfig["ipAddressTable.ipAddress"]
                            end                        
                        end                    
                        if (ikeTable["IpsecIKEPolicy.RemoteIdentifierType"] == "0") then
                            isUpdateNeed = true
                            ikeTable["IpsecIKEPolicy.RemoteIdentifier"] = inputTable["IpsecVPNPolicy.RemoteEndPoint"]
                        end
                    end
                    if (isUpdateNeed) then
                        valid = db.update("IpsecIKEPolicy", ikeTable, id)
                    else
                        valid = true
                    end
                end
            end
        end
    end
	
    --[[ Updating Redundant gateway configuration
    if (valid) then
        local ipsecConfig = db.getRow ("IpsecVPNPolicy", "_ROWID_", rowid)
        -- Updating Old Backup Policy information, if backup policy changed.
        if (operation == "edit") then
            if (ipsecConfig["IpsecVPNPolicy.RVGStatus"] == "1" and
                (inputTable["IpsecVPNPolicy.RVGStatus"] == nil or
                inputTable["IpsecVPNPolicy.RVGStatus"] == "0" or
                (inputTable["IpsecVPNPolicy.RVGStatus"] == "1" and
                 inputTable["IpsecVPNPolicy.BackupPolicyName"] ~= ipsecConfig["IpsecVPNPolicy.BackupPolicyName"]))) then
                local bckPolConfig = db.getRow ("IpsecVPNPolicy", "VPNPolicyName", ipsecConfig["IpsecVPNPolicy.BackupPolicyName"])
                if (bckPolConfig ~= nil) then
                    configStatus, errCode = vipSecure.rvgConfig (bckPolConfig, "1")
                    if (configStatus == "OK") then
                        valid = true
                    if ((inputTable["IpsecVPNPolicy.RVGStatus"] == nil or
                        inputTable["IpsecVPNPolicy.RVGStatus"] == "0") and
                        (ipsecConfig["IpsecVPNPolicy.BackupPolicyName"] ~= nil or
                        ipsecConfig["IpsecVPNPolicy.BackupPolicyName"] ~= "")) then
                        inputTable["IpsecVPNPolicy.BackupPolicyName"] = ""
                    end
                    else
                      	db.rollback()
               	        return "ERROR", errCode
                    end
                end
            end
        end ]]--

        --[[ Updating Backup Policy information
        if (inputTable["IpsecVPNPolicy.PolicyType"] == "1" and
            inputTable["IpsecVPNPolicy.RVGStatus"] == "1" and
            inputTable["IpsecVPNPolicy.BackupPolicyName"] ~= nil and
            inputTable["IpsecVPNPolicy.BackupPolicyName"] ~= ipsecConfig["IpsecVPNPolicy.BackupPolicyName"]) then
            configStatus, errCode = vipSecure.rvgConfigCheck (rowid)
            if (configStatus ~= "OK") then
               	db.rollback()
           	    return "ERROR", errCode
            end
            local bckPolicyConfig = db.getRow ("IpsecVPNPolicy", "VPNPolicyName", inputTable["IpsecVPNPolicy.BackupPolicyName"])
            if (bckPolicyConfig ~= nil) then
                configStatus, errCode = vipSecure.rvgConfig (bckPolicyConfig, "2")
                if (configStatus == "OK") then
                    valid = true
                else
                  	db.rollback()
               	    return "ERROR", errCode
                end
            else
               	db.rollback()
           	    return "ERROR", "BACKUP_POLICY_CONFIG_NOT_FOUND"
            end
        else
            valid = true
        end 
    end ]]--   
    if (valid) then    	    	
        valid = vipSecure.ipsecVPNConfig(inputTable, rowid, operation)
    end
    
    -- return
    if (valid) then
        --db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
    	--db.rollback()
	    return "ERROR", "IPSEC_VPN_CONFIG_FAILED"  
    end
end

-- Primary Policy Configuring Check to
-- configure Redundant VPN Gateway
function vipSecure.rvgConfigCheck (rowid)
    local priPolicyConfig = db.getRow ("IpsecVPNPolicy", "_ROWID_", rowid)

    if (priPolicyConfig == nil) then
            return "ERROR", "PRIMARY_POLICY_CONFIG_NOT_FOUND"
    else
        -- Check, if policy is manual policy
        if (priPolicyConfig["IpsecVPNPolicy.PolicyType"] == "2") then
            return "ERROR", "MANUAL_POLICY_CANT_SET_AS_BACKUP"
        -- Check, if policy is client policy
        elseif (priPolicyConfig["IpsecVPNPolicy.AutoPolicyType"] == "1") then
            return "ERROR", "CLIENT_POLICY_CANT_SET_AS_BACKUP"
        elseif (priPolicyConfig["IpsecVPNPolicy.IKEPolicyName"] ~= nil) then
            local ikeConfig = db.getRow ("IpsecIKEPolicy", "IKEPolicyName", priPolicyConfig["IpsecVPNPolicy.IKEPolicyName"])
            if (ikeConfig == nil) then
                return "ERROR", "IKE_POLICY_CONFIG_NOT_FOUND"
            -- Check, if DPD enabled or not
            elseif (ikeConfig["IpsecIKEPolicy.DPD"] == "0") then
                return "ERROR", "DPD_NOT_ENABLED"
            -- Check, if Direction is set as responder
            elseif (ikeConfig["IpsecIKEPolicy.Direction"] == "1") then
                return "ERROR", "DIRECTION_SET_TO_RESPONDER"
            -- Check, if XAuthConfig set to IPSecHost
            elseif (ikeConfig["IpsecIKEPolicy.XAUTHType"] == "2") then
                return "ERROR", "XAUTH_SET_TO_USER_DATABASE"
            end
        end
    end
    return "OK", ""
end

-- Configuring Redundant VPN Gateway
function vipSecure.rvgConfig (backupPolicyConfig, setFlag)
    local valid = false

    -- setting policy as primary policy
    if (setFlag == "1") then
        backupPolicyConfig["IpsecVPNPolicy.TypeofPolicy"] = "1"
        valid = vipSecure.ipsecVPNConfig(backupPolicyConfig, backupPolicyConfig["IpsecVPNPolicy._ROWID_"], "edit")
        if (valid) then
            return "OK", ""
        end
    -- setting policy as backup policy
    elseif (setFlag == "2") then
        -- Check, if policy already configured as back up policy
        if (backupPolicyConfig["IpsecVPNPolicy.TypeofPolicy"] ~= nil and
            backupPolicyConfig["IpsecVPNPolicy.TypeofPolicy"] == "2") then
            return "ERROR", "POLICY_ALREADY_IN_USE"
        else
            -- Check, if policy is manual policy
            if (backupPolicyConfig["IpsecVPNPolicy.PolicyType"] == "2") then
                return "ERROR", "MANUAL_POLICY_CANT_SET_AS_BACKUP"
            -- Check, if policy is client policy
            elseif (backupPolicyConfig["IpsecVPNPolicy.AutoPolicyType"] == "1") then
                return "ERROR", "CLIENT_POLICY_CANT_SET_AS_BACKUP"
            elseif (backupPolicyConfig["IpsecVPNPolicy.IKEPolicyName"] ~= nil) then
                local ikeConfig = db.getRow ("IpsecIKEPolicy", "IKEPolicyName", backupPolicyConfig["IpsecVPNPolicy.IKEPolicyName"])
                if (ikeConfig == nil) then
                    return "ERROR", "IKE_POLICY_CONFIG_NOT_FOUND"
                -- Check, if DPD enabled or not
                elseif (ikeConfig["IpsecIKEPolicy.DPD"] == "0") then
                    return "ERROR", "DPD_NOT_ENABLED"
                -- Check, if Direction is set as responder
                elseif (ikeConfig["IpsecIKEPolicy.Direction"] == "1") then
                    return "ERROR", "DIRECTION_SET_TO_RESPONDER"
                -- Check, if XAuthConfig set to IPSecHost
                elseif (ikeConfig["IpsecIKEPolicy.XAUTHType"] == "2") then
                    return "ERROR", "XAUTH_SET_TO_USER_DATABASE"
                end
            end
            backupPolicyConfig["IpsecVPNPolicy.TypeofPolicy"] = "2"
            backupPolicyConfig["IpsecVPNPolicy.Status"] = "0"
            valid = vipSecure.ipsecVPNConfig(backupPolicyConfig, backupPolicyConfig["IpsecVPNPolicy._ROWID_"], "edit")
            if (valid) then
                return "OK", ""
            end
        end
    end
    return "ERROR", "BACKUP_CONFIG_FAILED"
end

-- ipsecVPN config
function vipSecure.ipsecVPNConfig(inputTable, rowid, operation)
    if (db.typeAndRangeValidate(inputTable)) then    
        if (operation == "add") then
	       	return db.insert("IpsecVPNPolicy", inputTable)
        elseif (operation == "edit") then
	       	return db.update("IpsecVPNPolicy", inputTable, rowid)
        elseif (operation == "delete") then
	       	return db.delete("IpsecVPNPolicy", inputTable)
        end
    else
        return false
    end
end


-- IPSEC VPN Policy Delete
function vipSecure.ipsecVPNPolicyDelete (rowids)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	-- db.beginTransaction() --begin transaction
	local valid = false
    local vpnPolicies
	
	-- delete Ipsec VPN Policies
	for k,v in pairs(rowids) do
        local inTable = {}    	
        inTable["IpsecVPNPolicy._ROWID_"] = v;
        local policyType = db.getAttribute("IpsecVPNPolicy", "_ROWID_", v, "PolicyType")
        if (policyType ~= "2") then
    	    local ikeName = db.getAttribute("IpsecVPNPolicy", "_ROWID_", v, "IKEPolicyName")
            if (ikeName ~= nil) then
    	        local ikeID = db.getAttribute("IpsecIKEPolicy", "IKEPolicyName", ikeName, "_ROWID_")
                if (ikeID ~= nil) then
                    local ikeOldTable = db.getRow ("IpsecIKEPolicy", "_ROWID_", ikeID)
                    local ikePolicyName = ikeOldTable["IpsecIKEPolicy.IKEPolicyName"] or ''
                    vpnPolicies = db.getRowsWhere ("IpsecVPNPolicy" , "IKEPolicyName = '" .. ikePolicyName .. "'")
                    local isUpdateNeed = false
                    -- Update required for IKE policy only if IKE policy is
                    -- not sharing by any other vpn policies.
                    if (#vpnPolicies <= 1) then                    
                        if (ikeOldTable["IpsecIKEPolicy.LocalIdentifierType"] == "0") then
                            ikeOldTable["IpsecIKEPolicy.LocalIdentifier"] = ""
                            isUpdateNeed = true
                        end
                        if (ikeOldTable["IpsecIKEPolicy.RemoteIdentifierType"] == "0") then
                            ikeOldTable["IpsecIKEPolicy.RemoteIdentifier"] = ""
                            isUpdateNeed = true                    
                        end
                    end
                    if (isUpdateNeed) then
                        valid = db.update("IpsecIKEPolicy", ikeOldTable, ikeID)
                    else
                        valid = true
                    end
                else
                    valid = true
                end
            end
        else
            valid = true
        end

        -- Check if backup policy ids up and running.
        local priPolicyConfig = db.getRow ("IpsecVPNPolicy", "_ROWID_", v)
        if (priPolicyConfig["IpsecVPNPolicy.RVGStatus"] == "1")then
            local bckPolicyStatus = db.getAttribute ("IpsecVPNPolicy", "VPNPolicyName", priPolicyConfig["IpsecVPNPolicy.BackupPolicyName"], "Status") or ''
            if (bckPolicyStatus == "1") then
              	db.rollback()
           	    return "ERROR", "BCK_POLICY_UP_AND_RUNNING"
            end
       end

        -- Updating Backup policies
        if (valid) then
      	    local ipsecConfig = db.getRow("IpsecVPNPolicy", "_ROWID_", v)
            if (ipsecConfig ~= nil and
                ipsecConfig["IpsecVPNPolicy.RVGStatus"] ~= nil and
                ipsecConfig["IpsecVPNPolicy.RVGStatus"] == "1") then
                local bckPolicyConfig = db.getRow ("IpsecVPNPolicy", "VPNPolicyName", ipsecConfig["IpsecVPNPolicy.BackupPolicyName"])
                if (bckPolicyConfig ~= nil) then
                    configStatus, errCode = vipSecure.rvgConfig (bckPolicyConfig, "1")
                    if (configStatus == "OK") then
                        valid = true
                    end
                end
            else
                valid = true
            end
        end

        if (valid) then
	      valid = db.delete("IpsecVPNPolicy", inTable)
	    end
	    if (not valid) then
		    break
	    end
	end

	-- return
	if (valid) then
		--db.commitTransaction(true)
		return "OK", "IPSEC_VPN_DEL_SUCCESS"
	else
		db.rollback()
		return "ERROR", "IPSEC_VPN_DEL_FAILED"
	end
end

-- VPN wizard configuration
function vipSecure.wizardConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
    	return "ACCESS_DENIED", "ADMIN_REQD"
    end
    
    local MAXCOUNT = 70  -- Number of vpn policies for DSR 1000
    local DSR_500 = 35   -- Number of vpn policies for DSR 500
     
    if (UNIT_NAME == "DSR-500" or UNIT_NAME == "DSR-500N") then
			MAXCOUNT = DSR_500
	end
	
    if (operation == "add") then
	local vpnTable = db.getTable("IpsecVPNPolicy")	
	if (#vpnTable == MAXCOUNT) then
	    return "ERROR", "MAX_VPN_LIMIT_REACHED"
	end
    end
    
    if (db.existsRow("IpsecVPNPolicy", "RemoteEndPoint", inputTable["vpnConfig.RemoteIdentifier"])) then
    	return "ERROR", "IPSEC_VPN_REMOTE_CONFIG_FAILED"
    end

    if (db.existsRow("IpsecIKEPolicy", "IKEPolicyName", inputTable["vpnConfig.ConnectionName"])) then
	    return "ERROR", "IPSEC_VPN_NAME_CONFIG_FAILED"
    end

    if (db.existsRow("IpsecVPNPolicy", "VPNPolicyName", inputTable["vpnConfig.ConnectionName"])) then
    	return "ERROR", "IPSEC_VPN_NAME_CONFIG_FAILED"
    end

    db.beginTransaction() --begin transaction
    local valid = false
    
    local ikeTable = {}
    ikeTable["IpsecIKEPolicy.IKEPolicyName"]=inputTable["vpnConfig.ConnectionName"]
    if(tonumber(inputTable["vpnConfig.Peer"])==0) then
	    ikeTable["IpsecIKEPolicy.Direction"]="2"
    else
	    ikeTable["IpsecIKEPolicy.Direction"]="1"
    end
    if(tonumber(inputTable["vpnConfig.Peer"])==0) then
    	ikeTable["IpsecIKEPolicy.ExchangeMode"]="0"
    else
	    ikeTable["IpsecIKEPolicy.ExchangeMode"]="1"
    end
   	ikeTable["IpsecIKEPolicy.LocalIdentifierType"]=inputTable["vpnConfig.LocalIdentifierType"]
    ikeTable["IpsecIKEPolicy.LocalIdentifier"]=inputTable["vpnConfig.LocalIdentifier"]
   	ikeTable["IpsecIKEPolicy.RemoteIdentifierType"]=inputTable["vpnConfig.RemoteIdentifierType"]
    ikeTable["IpsecIKEPolicy.RemoteIdentifier"]=inputTable["vpnConfig.RemoteIdentifier"]
    ikeTable["IpsecIKEPolicy.Presharedkey"]=inputTable["vpnConfig.PresharedKey"]
    
    -- validate
    if (db.typeAndRangeValidate(ikeTable)) then
        if (operation == "add") then
            valid = db.insert("IpsecIKEPolicy", ikeTable)
        elseif (operation == "edit") then
            valid = db.update(DBTable, ikeTable, rowid)
        elseif (operation == "delete") then
            valid = db.delete(DBTable, ikeTable)
        end
    end

    local vpnTable = {}
    vpnTable["IpsecVPNPolicy.VPNPolicyName"]=inputTable["vpnConfig.ConnectionName"]
    if(tonumber(inputTable["vpnConfig.Peer"])==0) then
    	vpnTable["IpsecVPNPolicy.Netbios"]="1"
    	vpnTable["IpsecVPNPolicy.RemoteNetworkType"]="3"
	    vpnTable["IpsecVPNPolicy.AutoPolicyType"] = "0"
    else
    	vpnTable["IpsecVPNPolicy.Netbios"]="0"
	    vpnTable["IpsecVPNPolicy.RemoteNetworkType"]="0"
        vpnTable["IpsecVPNPolicy.AutoPolicyType"] = "1"
    end
   	vpnTable["IpsecVPNPolicy.LocalGateway"]=inputTable["vpnConfig.Interface"]
   	vpnTable["IpsecVPNPolicy.EnableDhcpOverIPSec"] = inputTable["IpsecVPNPolicy.EnableDhcpOverIPSec"]    
   	vpnTable["IpsecVPNPolicy.RemoteEndPointType"]=inputTable["vpnConfig.RemoteIdentifierType"]
    vpnTable["IpsecVPNPolicy.RemoteEndPoint"]=inputTable["vpnConfig.RemoteIdentifier"]
    local lanConfig = db.getRowWhere("ipAddressTable", "LogicalIfName = 'LAN' AND addressFamily = 2")
    local ipaddr = ""
    local subnet = ""
    if (lanConfig ~= nil) then
        ipaddr = lanConfig["ipAddressTable.ipAddress"]
        subnet = lanConfig["ipAddressTable.subnetMask"]
    end

    local ipAddr = util.split(ipaddr or "",".")
    ipAddr[4] = "0"
    ipaddr = util.join (ipAddr, ".")
    vpnTable["IpsecVPNPolicy.LocalStartAddress"]=ipaddr
    vpnTable["IpsecVPNPolicy.LocalSubnetMask"]=subnet
   
    if(tonumber(inputTable["vpnConfig.Peer"])==0) then
        ipAddr = util.split(inputTable["vpnConfig.RemoteIP"] or "",".")
        ipAddr[4] = "0"
        vpnTable["IpsecVPNPolicy.RemoteStartAddress"] = util.join (ipAddr, ".")
	    vpnTable["IpsecVPNPolicy.RemoteSubnetMask"] = inputTable["vpnConfig.RemoteSubnet"]
        local vpnPolicies = db.getRowsWhere ("IpsecVPNPolicy","RemoteStartAddress = '" .. vpnTable["IpsecVPNPolicy.RemoteStartAddress"] .. "' and RemoteSubnetMask = '" .. vpnTable["IpsecVPNPolicy.RemoteSubnetMask"] .. "'")
        if (vpnPolicies ~= nil and #vpnPolicies ~= 0) then
            vpnTable["IpsecVPNPolicy.Status"] = "0"
            --db.rollback()
            --return "ERROR", "IPSEC_VPN_INVALID_CONFIGURATION"
        end
    end


    if (valid) then
        vpnTable["IpsecVPNPolicy.IKEPolicyName"]=inputTable["vpnConfig.ConnectionName"]
        -- validate
    	if (db.typeAndRangeValidate(vpnTable)) then
            if (operation == "add") then
                valid = db.insert("IpsecVPNPolicy", vpnTable)
            elseif (operation == "edit") then
                valid = db.update(DBTable, vpnTable, rowid)
            elseif (operation == "delete") then
                valid = db.delete(DBTable, vpnTable)
            end
        end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "IKEP_CONFIG_FAILED"
    end
end

-- Mode Config Configuration
function vipSecure.modeConfigConfig(inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
    	    return "ACCESS_DENIED", "ADMIN_REQD"
	end
	
	if (db.existsRow("IpsecModeConfig", "ModeConfigRecordName", inputTable["IpsecModeConfig.ModeConfigRecordName"]) and operation == "add") then
	    return "ERROR", "IPSEC_MODE_NAME_CONFIG_FAILED"
    end

	db.beginTransaction() --begin transaction
	local valid = false

    -- Add Policy
    if (operation == "add") then
      	valid = vipSecure.modeConfigDBconfig(inputTable, rowid, operation)
    end

	-- Edit Policy
    if (operation == "edit") then
        valid = vipSecure.modeConfigDBconfig(inputTable, rowid, operation)
    end
	
	-- Delete Policy
    if (operation == "delete") then
    	valid = vipSecure.modeConfigDBconfig(inputTable, rowid, operation)
    end 
    
	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "IPSEC_MODE_CONFIG_FAILED"  
	end
end

function vipSecure.modeConfigDBconfig(inputTable, rowid, operation)
    if (operation == "add") then
	   	return db.insert("IpsecModeConfig", inputTable)
    elseif (operation == "edit") then
	   	return db.update("IpsecModeConfig", inputTable, rowid)
    elseif (operation == "delete") then
	   	return db.delete("IpsecModeConfig", inputTable)
    end
end

-- Delete Mode Config 
function vipSecure.modeConfigDelete (rowids)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false
	
	-- delete Ipsec VPN Policies
	for k,v in pairs(rowids) do
            local inTable = {}
            inTable["IpsecModeConfig._ROWID_"] = v;
	    local modeName = db.getAttribute("IpsecModeConfig", "_ROWID_", v, "ModeConfigRecordName")
	    if(db.existsRow("IpsecIKEPolicy", "ModeConfigRecordName", modeName)) then
	         return "ERROR", "IPSEC_MODE_DEL_DEPEND_FAILED"
	    else
	         valid = db.delete("IpsecModeConfig", inTable)
	    end
	    if (not valid) then
	    	break
	    end
	end
	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "IPSEC_MODE_DEL_FAILED"
	end
end

-- Connection Status 
function vipSecure.ConnStatusConfig (tableName, keyName,keyValue,attributeName,attributeValue,operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    
    db.beginTransaction() --begin transaction
    local valid = false

    -- validate
    if (operation == "Connect") then
		valid = db.setAttribute (tableName, keyName,keyValue,attributeName,attributeValue)
	else
		valid = db.setAttribute (tableName, keyName,keyValue,attributeName,attributeValue)
	end

    -- return
    if (valid) then
        db.commitTransaction()
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "VIPSECURE_CS_CONFIG_FAILED"
    end
end

-- XAUTH Users Config
function vipSecure.xauthConfig (inputTable, rowid, operation)
    -- validate
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false

    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            valid = db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            valid = db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
            valid = db.delete(DBTable, inputTable)
        end
    end

    if (valid and operation == "add") then
        if (inputTable["QuikVPNUserSupport"] ~= nil and inputTable["QuikVPNUserSupport"] == "1") then
            if (inputTable["dhcp.dhcpEnabled"] ~= nil and inputTable["dhcp.dhcpEnabled"] == "1") then
                local ifStaticConfig = db.getRowWhere ("ifStatic","LogicalIfName = 'LAN' and AddressFamily = 2")
                if (ifStaticConfig ~= nil) then
                    ifStaticConfig["ifStatic.StaticIp"] = inputTable["ifStatic.StaticIp"]
                    ifStaticConfig["ifStatic.NetMask"] = "255.255.255.0"
                    if (db.typeAndRangeValidate(ifStaticConfig)) then
                        valid = db.update("ifStatic",ifStaticConfig, ifStaticConfig["ifStatic._ROWID_"])
                    else
                        valid = false
                    end
                    local dhcpConfig = db.getRowWhere ("dhcp","logicalIfName = 'LAN'")
                    if (valid and dhcpConfig ~= nil) then
                        dhcpConfig ["dhcp.dhcpEnabled"] = "1"
                        dhcpConfig ["dhcp.startAddress"] = inputTable["dhcp.startAddress"]
                        dhcpConfig ["dhcp.endAddress"] = inputTable["dhcp.endAddress"]
                        if (db.typeAndRangeValidate(dhcpConfig)) then
                            valid = db.update("dhcp",dhcpConfig, dhcpConfig["dhcp._ROWID_"])
                        else
                            valid = false
                        end
                    else
                        valid = false
                    end
                else
                    valid = false
                end
            end
        end
    end

    -- return
	if (valid) then
		db.commitTransaction(true)
	        return "OK", "STATUS_OK"            
	else
		db.rollback()
		return "ERROR", "XAUTH_USERS_CONFIG_FAILED"
	end
end
