--[[
#### Hussain Vali.
#### TeamF1
#### www.TeamF1.com
#### July 25, 2008
#### File: vlan.lua
#### Description: vlan configuration functions
#### Revisions:
]]--


--************* Initial Code *************
--table packages
vlan = {}

--************* Functions *************
-- vlan config
function vlan.config (inputTable, rowid, operation)
    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            return db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            return db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
            return db.delete(DBTable, inputTable)
        end
    end
end

-- vlan status config
function vlan.vlanStatusConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transaction
	local valid = false
	
	valid = vlan.config (inputTable, rowid, operation)

	-- return
	if (valid) then
		db.commitTransaction(true)    
        return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "VLAN_STATUS_CONFIG_FAILED"
	end
end

-- vlan config
function vlan.vlanConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transaction
	local valid = false
	
    if (operation == "add" or operation == "edit") then
        if (DBTable == "VlanPortMgmt") then
            -- checking vlan membership state, when mode = "Access"
            if (inputTable["VlanPortMgmt.PortVlanMode"] == "0") then
                local portLogName = "vlan." .. db.getAttribute ("VlanPortMgmt", "_ROWID_", rowid, "PortLogicalName")
                local table = db.getTable ("vlan")
                local isNotAllow = false                
                local i = 0
	            for k,v in pairs(table) do
	                i = i + 1
                    local row = table[i]
                    if (row ~= nil and row[portLogName] ~= nil and row[portLogName] == "1" and row["vlan.vlanId"] ~= inputTable["VlanPortMgmt.DefaultPVID"]) then
                        isNotAllow = true
                        break
                    elseif ((row[portLogName] ~= nil or row[portLogName] == "0") and row["vlan.vlanId"] == inputTable["VlanPortMgmt.DefaultPVID"]) then
                        row[portLogName] = "1"
                        db.update ("vlan", row, row["vlan._ROWID_"])
                    end
                end
                if (isNotAllow) then
        	        db.rollback()
		            return "ERROR", "PORT_MODE_NOT_ALLOWED"
                end
            end
            if (db.getAttribute("vlan", "vlanId", inputTable["VlanPortMgmt.DefaultPVID"], "_ROWID_") == nil) then
                local vlanConfig = {}
                vlanConfig ["vlan.vlanId"] = inputTable["VlanPortMgmt.DefaultPVID"]
                --vlanConfig [portLogName] = "1"
                valid = db.insert("vlan", vlanConfig)
                if (not valid) then
        	        db.rollback()
		            return "ERROR", "VLAN_CONFIG_FAILED"
                end
            else
                local vlanRowId = db.getAttribute("vlan", "vlanId", inputTable["VlanPortMgmt.DefaultPVID"], "_ROWID_")
                local portLogName = db.getAttribute ("VlanPortMgmt", "_ROWID_", rowid, "PortLogicalName")
                valid = db.setAttribute ("vlan", "_ROWID_", vlanRowId, portLogName, 1)
                if (not valid) then
        	        db.rollback()
		            return "ERROR", "VLAN_CONFIG_FAILED"
                end
            end
        end

    --Checking for used vlans.
    elseif (operation == "delete") then
    	for k,v in pairs(inputTable) do
            local vlanId = db.getAttribute ("vlan","_ROWID_",v, "vlanId")
            local vlanName = db.getAttribute ("vlan","_ROWID_",v, "vlanName")
            if (vlanName) then
                local rows = db.getRowsWhere ("vlan", "vlanId="..vlanId.." and vlanName='"..vlanName.."'");
                -- if there are rows other than dummy ports 
                if (#rows > 1) then
            		db.rollback()
    		        return "ERROR", "VLAN_CONFIG_IN_USE"
                end
                -- delete the entries for vlanId from vlan
                rows = db.getRowsWhere ("vlan", "vlanId="..vlanId);
                for kk, vv in pairs (rows) do
                    valid = vlanConfig.config ({vv["vlan._ROWID_"]}, -1, "delete")
                end
            end
        end
    end

    local oldMode = nil
    local newMode = nil    
    local oldPvid = nil
    local newPvid = nil
    if (DBTable == "VlanPortMgmt") then
        oldMode = db.getAttribute ("VlanPortMgmt","_ROWID_",rowid,"PortVlanMode")
    	oldPvid = db.getAttribute ("VlanPortMgmt","_ROWID_",rowid,"DefaultPVID")
    end
    if (operation ~= "delete") then
    	valid = vlanConfig.config (inputTable, rowid, operation)
    end
    if (DBTable == "VlanPortMgmt") then
        newMode = db.getAttribute ("VlanPortMgmt","_ROWID_",rowid,"PortVlanMode")
	    newPvid = db.getAttribute ("VlanPortMgmt","_ROWID_",rowid,"DefaultPVID")
    end    

    --dummy updates for vlan rows if mode changed in VlanPortMgmt.
    if (valid and DBTable == "VlanPortMgmt") then
        if ((oldMode ~= nil and newMode ~= nil and oldMode ~= newMode) or (oldPvid ~= nil and newPvid ~= nil and oldPvid ~= newPvid)) then
            local portLogName = "vlan." .. db.getAttribute ("VlanPortMgmt", "_ROWID_", rowid, "PortLogicalName")
            local table = db.getTable ("vlan")
            local i = 0
	        for k,v in pairs(table) do
	            i = i + 1
                local row = table[i]
                local reqUpdate = false
                if (row ~= nil) then
                    if (row[portLogName] ~= nil and row[portLogName] == "1") then
                        reqUpdate = true
                    end
                    if (reqUpdate) then
                        db.setAttribute ("vlan", "_ROWID_", row["vlan._ROWID_"], db.getAttribute ("VlanPortMgmt", "_ROWID_", rowid, "PortLogicalName"), row[portLogName])
                   end
                end
            end            
        end
    end

    -- PVID cannot go beyond those in vlan table
    if (valid and DBTable == "VlanPortMgmt") then
        valid = db.getAttribute ("vlan", "vlanId", inputTable["VlanPortMgmt.DefaultPVID"], "vlanName")
    end

	-- return
	if (valid) then
		db.commitTransaction(true)

        -- access port should have only one vlan membership ..
        if (DBTable == "VlanPortMgmt" and inputTable["VlanPortMgmt.PortVlanMode"] == "0") then
            locTable = {}; 
            DBTable="vlan";
            locTable ["vlan.new_vlanId_"..inputTable["VlanPortMgmt.DefaultPVID"]] = inputTable["VlanPortMgmt.DefaultPVID"]
            vlan.portVlanConfig (locTable,rowid)
            DBTable="VlanPortMgmt";
        end

        return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "PORT_VLAN_CONFIG_FAILED"
	end
end

-- vlan config
function vlan.portVlanConfig (inputTable,rowid)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transaction
	local valid = false
    local isUpdated = false
	
    local portLogName = "vlan." .. db.getAttribute ("VlanPortMgmt", "_ROWID_", rowid, "PortLogicalName")
    --local defPVID = db.getAttribute ("VlanPortMgmt", "_ROWID_", rowid, "DefaultPVID")
    local table = db.getTable ("vlan")
    local i = 0
	for k,v in pairs(table) do
	    i = i + 1
        local row = table[i]
        local reqUpdate = false
        if (row ~= nil) then
            if (inputTable["vlan.row_" .. row["vlan._ROWID_"]] == nil) then
                if (row["vlan.portName"] ~= nil and "vlan." .. row["vlan.portName"] == portLogName) then
                    -- in any case, the current PVID must be mapped on the membership table ??
                    --if (defPVID ~= row["vlan.vlanId"]) then
                        valid = vlan.config ({row["vlan._ROWID_"]}, -1, "delete")
                    --end
                    isUpdated = true
                    if (not valid) then
                        break
                    end
                end                
            end
            if (inputTable["vlan.new_vlanId_" .. row["vlan.vlanId"]] ~= nil) then
                local tab = {}
                tab ["vlan.vlanId"] = inputTable["vlan.new_vlanId_" .. row["vlan.vlanId"]]
                tab ["vlan.vlanName"] = db.getAttribute ("vlan", "vlanId", tab ["vlan.vlanId"], "vlanName");
                tab ["vlan.portName"] = db.getAttribute ("VlanPortMgmt", "_ROWID_", rowid, "PortLogicalName")
               valid = vlan.config (tab, -1, "add")
            end
        end
    end

    if (not isUpdated) then
        valid = true
    end

	-- return
	if (valid) then
		db.commitTransaction(true)    
        return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "PORT_VLAN_CONFIG_FAILED"
	end
end
