--[[
#### Hussain Vali.
#### TeamF1
#### www.TeamF1.com
#### July 25, 2008
#### File: vlanConfig.lua
#### Description: vlan configuration functions
#### Revisions:
]]--


--************* Initial Code *************
--table packages
vlanConfig = {}

--************* Functions *************
-- vlan config
function vlanConfig.config (inputTable, rowid, operation)
    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            return db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            return db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
            return db.delete(DBTable, inputTable)
        end
    end
    return false
end

-- check if disabling vlan is allowed.
function vlanConfig.disableAllow (inputTable, rowid, operation)
	local lanIf = "LAN" --db.getAttribute("networkInterface","LogicalIfName","LAN","VirtualIfName") or ''
	local wan1If = "WAN1" --db.getAttribute("networkInterface","LogicalIfName","WAN1","VirtualIfName") or ''
	local wan2If = "WAN2" --db.getAttribute("networkInterface","LogicalIfName","WAN2","VirtualIfName") or ''
	local message = ""
	local allow = 1
	local tcpDumpRow = db.getRow ("tcpdump", "tcpdumpEnabled", "1")
	if (tcpDumpRow ~= nil ) then
		if (tcpDumpRow ["tcpdump.LogicalIfName"] ~= lanIf or
            tcpDumpRow ["tcpdump.LogicalIfName"] ~= wan1If or
            tcpDumpRow ["tcpdump.LogicalIfName"] ~= wan2If) then
			message = message .. "Packet Capture, "
			allow = 0
		end
	end
	local igmpRow = db.getRow ("igmp", "IgmpEnable", "1")
	if (igmpRow ~= nil ) then
		if (igmpRow["igmp.igmpLogicalIfName"] ~= "LAN") then
			message = message .. "IGMP Proxy, "
			allow = 0
		end
	end
	local upnpRow = db.getRow ("upnp", "upnpEnable", "1")
	if (upnpRow ~= nil ) then
		if (upnpRow["upnp.interfaceName"] ~= "bdg1") then
			message = message .. "UPnP, "
			allow = 0
		end
	end
	if (allow == 0) then
		return "ERROR", message
	else
		return "OK", message
	end
end

-- vlan status config
function vlanConfig.vlanStatusConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	local allowed = "OK"
	allowed, message  = vlanConfig.disableAllow (inputTable, rowid, operation)
	if (allowed == "ERROR" ) then
		message = "Please disable ".. message .. "on vlan enabled LANs."
		return "ERROR", message 
	end

	db.beginTransaction() --begin transaction
	local valid = false
	
	valid = vlanConfig.config (inputTable, rowid, operation)

    if(inputTable["VlanStatus.VlanState"] == "1" ) then
		--we need to revert dhcp to previous state
		local dhcpTable = db.getTable("dhcp")
		for i = 2,#(dhcpTable) do
			row = dhcpTable[i]
			row["dhcp.dhcpEnabled"] = row["dhcp.oldState"]
			DBTable = "dhcp"
			valid = vlanConfig.config(row, i, "edit");
		end
	else
		--we are here to disable dhcps
		local dhcpTable = db.getTable("dhcp")
		for i = 2,#(dhcpTable) do
			row = dhcpTable[i]
			local oldState = row["dhcp.dhcpEnabled"]
			row["dhcp.oldState"] = oldState
			row["dhcp.dhcpEnabled"] = "0"
			DBTable = "dhcp"
			valid = vlanConfig.config(row, i, "edit");
		end
	end

	-- return
	if (valid) then
		db.commitTransaction(true)    
        return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "VLAN_STATUS_CONFIG_FAILED"
	end
end

--vlan subnetting config
function vlanConfig.vlanSubnettingConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transaction
	local valid = false
	
	valid = vlanConfig.config (inputTable, rowid, operation)
	
	-- return
	if (valid) then
		db.commitTransaction(true)    
        return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "VLAN_SUBNETTING_CONFIG_FAILED"
	end
end

function vlanConfig.configDep (inputTable, rowid, operation)
	local valid1 = false

	if(operation == "delete")then
		for k,v in pairs(inputTable) do
			local vlanId = v
			valid1 = db.deleteRow("ifStatic","LogicalIfName","LAN"..vlanId)
			--valid1 = valid1 and db.deleteRow("dhcp","logicalIfName","LAN"..vlanId)
			valid1 = valid1 and db.setAttribute("dhcp","logicalIfName","LAN"..vlanId,"dhcpEnabled","0")
			valid1 = valid1 and db.setAttribute("dhcp","logicalIfName","LAN"..vlanId,"enableDnsProxy","0")
			valid1 = valid1 and db.deleteRow("DhcpRelay","logicalIfName","LAN"..vlanId)
			valid1 = valid1 and db.deleteRow("interVLANRouting","vlanId",vlanId)
			valid1 = valid1 and db.deleteRow("bridgeTable","bridgeIndex",vlanId)            
			valid1 = valid1 and db.deleteRow("tcpdump","LogicalIfName","LAN"..vlanId)
			valid1 = valid1 and db.deleteRow("zones","ZoneLogicalIfName","LAN"..vlanId)
			valid1 = valid1 and db.deleteRow("DhcpRelay","LogicalIfName","LAN"..vlanId)
		end
	elseif(operation == "add") then
        local staticIPAddr = "192.168.2.1"
        local dhcpStIPAddr = "192.168.2.100"
        local dhcpEndIPAddr = "192.168.2.254"
        local vlanCount = 2
        local thirdOctet = 2
        while (db.existsRow ("ifStatic", "StaticIp", staticIPAddr)  and
               vlanCount <= 16) do
            vlanCount = vlanCount + 1
            thirdOctet = thirdOctet + 1
            staticIPAddr = "192.168." .. thirdOctet .. ".1"
        end
        dhcpStIPAddr = "192.168." .. thirdOctet .. ".100"
        dhcpEndIPAddr = "192.168." .. thirdOctet .. ".254"

    	local ifStatic = {}
	    ifStatic["ifStatic.LogicalIfName"] = "LAN" .. inputTable["vlan.vlanId"]
	    ifStatic["ifStatic.AddressFamily"] = "2"
	    ifStatic["ifStatic.StaticIp"] = staticIPAddr
	    ifStatic["ifStatic.NetMask"] = "255.255.255.0"
	    ifStatic["ifStatic.Gateway"] = "0.0.0.0"
	    DBTable = "ifStatic"
	    valid1 = vlanConfig.config(ifStatic, "-1", "add");

        if (valid1) then
			local dhcp = {}
		if (operation == "add") then
			local dhcpRow = db.getRow("dhcp","logicalIfName","LAN".. inputTable["vlan.vlanId"])
			if (dhcpRow == nil) then
        			dhcp["dhcp.logicalIfName"] = "LAN" .. inputTable["vlan.vlanId"]
   	    			dhcp["dhcp.startAddress"] = dhcpStIPAddr
            			dhcp["dhcp.endAddress"] = dhcpEndIPAddr
   		    		dhcp["dhcp.dhcpEnabled"] = "1"
            			dhcp["dhcp.leaseTime"] = "24"
   		    		dhcp["dhcp.enableDnsProxy"] = "1"
            			DBTable = "dhcp"
            			valid1 = vlanConfig.config(dhcp, "-1", "add");
			else
				valid1 = valid1 and db.setAttribute("dhcp","logicalIfName","LAN"..inputTable["vlan.vlanId"],"dhcpEnabled","1")
				valid1 = valid1 and db.setAttribute("dhcp","logicalIfName","LAN"..inputTable["vlan.vlanId"],"enableDnsProxy","1")
			end
		end
        end

        if(valid1) then
            local DhcpRelay = {}
            DhcpRelay["DhcpRelay.LogicalIfName"] = "LAN" .. inputTable["vlan.vlanId"]
            DhcpRelay["DhcpRelay.virtualIfName"] = "bdg" .. inputTable["vlan.vlanId"]
            DhcpRelay["DhcpRelay.DhcpRelayStatus"] = "0"
            DhcpRelay["DhcpRelay.RelayGateway"] = "0.0.0.0"
            DBTable = "DhcpRelay"
            valid1 = vlanConfig.config(DhcpRelay,"-1", "add");
        end

        if(valid1) then
	        local zones = {}
        	zones["zones.ZoneLogicalIfName"] = "LAN" .. inputTable["vlan.vlanId"]
	        zones["zones.ZoneType"] = "SECURE"
        	DBTable = "zones"
	        valid1 = vlanConfig.config(zones,"-1", "add");
        end

        if(valid1) then
            local tcpdump = {}
            tcpdump["tcpdump.LogicalIfName"] = "LAN" .. inputTable["vlan.vlanId"]
            tcpdump["tcpdump.tcpdumpEnabled"] = "0"
            DBTable = "tcpdump"
            valid1 = vlanConfig.config(tcpdump,"-1", "add");
        end

        if(valid1) then
			local interVlan = {}
			interVlan["interVLANRouting.vlanId"] = inputTable["vlan.vlanId"]
			interVlan["interVLANRouting.status"] = inputTable["interVLANRouting.status"];
            interVlan["interVLANRouting.ifName"] = "bdg" .. inputTable["vlan.vlanId"];
			DBTable = "interVLANRouting"
			valid1 = vlanConfig.config(interVlan,"-1","add");
		end
    end

	return valid1 or true
end

-- vlan config
function vlanConfig.vlanConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transaction
	local valid = false
	
	if (operation == "add" and inputTable["interVLANRouting.status"] ~= nil and inputTable["insertRpt"] ~= "1") then
        local bridge = {}
	    bridge = db.getDefaults("bridgeTable")
		bridge["bridgeTable.bridgeIndex"] = inputTable["vlan.vlanId"]
    	bridge["bridgeTable.interfaceName"] = "bdg"..inputTable["vlan.vlanId"]
	    DBTable = "bridgeTable"
		valid = vlanConfig.config(bridge,"-1","add");
        DBTable = "vlan"
	end

    local boardName = db.getAttribute ("unitName","_ROWID_","1","unitName")
    if (boardName ~= nil and boardName ~= "WRVS5110") then
        boardName = true
    else
        boardName = false
    end
    --Checking configurable port configuration.
    --if it is set to WAN/DMZ, we should not allow
    --vlanid 4094 to configure. That is reserved for 
    --WAN/DMZ options.
    if (operation == "add" or operation == "edit") then
        if (boardName and DBTable == "vlan") then
            if (inputTable["vlan.vlanId"] == "4094") then
                local configPort = db.getAttribute ("ConfigPort","_ROWID_","1","LogicalIfName")
                if (configPort ~= nil and configPort ~= "LAN") then
        	        db.rollback()
    		        return "ERROR", "CONFIG_PORT_SET_AS_WANORDMZ"
                end
            end            
        elseif(boardName and DBTable == "VlanPortMgmt") then
            -- checking vlan membership state, when mode = "Access"
            if (inputTable["VlanPortMgmt.PortVlanMode"] == "0") then
                local portLogName = "vlan." .. db.getAttribute ("VlanPortMgmt", "_ROWID_", rowid, "PortLogicalName")
                local table = db.getTable ("vlan")
                local isNotAllow = false                
                local i = 0
	            for k,v in pairs(table) do
	                i = i + 1
                    local row = table[i]
                    if (row ~= nil and row[portLogName] ~= nil and row[portLogName] == "1" and row["vlan.vlanId"] ~= inputTable["VlanPortMgmt.DefaultPVID"]) then
                        isNotAllow = true
                        break
                    elseif ((row[portLogName] ~= nil or row[portLogName] == "0") and row["vlan.vlanId"] == inputTable["VlanPortMgmt.DefaultPVID"]) then
                        row[portLogName] = "1"
                        db.update ("vlan", row, row["vlan._ROWID_"])
                    end
                end
                if (isNotAllow) then
        	        db.rollback()
		            return "ERROR", "PORT_MODE_NOT_ALLOWED"
                end
            end
            if (inputTable["VlanPortMgmt.DefaultPVID"] == "4094") then
                local configPort = db.getAttribute ("ConfigPort","_ROWID_","1","LogicalIfName")
                if (configPort ~= nil and configPort ~= "LAN") then
        	        db.rollback()
		            return "ERROR", "CONFIG_PORT_SET_AS_WANORDMZ"
                end
            end
            if (db.getAttribute("vlan", "vlanId", inputTable["VlanPortMgmt.DefaultPVID"], "_ROWID_") == nil) then
                local vlanTable = db.getRows ("vlan", "portName", "dummy")
                local portLogName = "vlan." .. db.getAttribute ("VlanPortMgmt", "_ROWID_", rowid, "PortLogicalName")
                if (#vlanTable >= 17) then
        	        db.rollback()
		            return "ERROR", "VLAN_MAX_LIMIT_ERROR"
                end
                local vlanConfig = {}
                vlanConfig ["vlan.vlanId"] = inputTable["VlanPortMgmt.DefaultPVID"]
            else
                local vlanRowId = db.getAttribute("vlan", "vlanId", inputTable["VlanPortMgmt.DefaultPVID"], "_ROWID_")
                local portLogName = db.getAttribute ("VlanPortMgmt", "_ROWID_", rowid, "PortLogicalName")
            end
        elseif (DBTable == "VlanPortMgmt") then
            -- checking vlan membership state, when mode = "Access"
            if (inputTable["VlanPortMgmt.PortVlanMode"] == "0") then
                local portLogName = "vlan." .. db.getAttribute ("VlanPortMgmt", "_ROWID_", rowid, "PortLogicalName")
                local table = db.getTable ("vlan")
                local isNotAllow = false                
                local i = 0
	            for k,v in pairs(table) do
	                i = i + 1
                    local row = table[i]
                    if (row ~= nil and row[portLogName] ~= nil and row[portLogName] == "1" and row["vlan.vlanId"] ~= inputTable["VlanPortMgmt.DefaultPVID"]) then
                        isNotAllow = true
                        break
                    elseif ((row[portLogName] ~= nil or row[portLogName] == "0") and row["vlan.vlanId"] == inputTable["VlanPortMgmt.DefaultPVID"]) then
                        row[portLogName] = "1"
                        db.update ("vlan", row, row["vlan._ROWID_"])
                    end
                end
                if (isNotAllow) then
        	        db.rollback()
		            return "ERROR", "PORT_MODE_NOT_ALLOWED"
                end
            end
            if (db.getAttribute("vlan", "vlanId", inputTable["VlanPortMgmt.DefaultPVID"], "_ROWID_") == nil) then
                local vlanTable = db.getRows ("vlan", "portName", "dummy")
                local portLogName = "vlan." .. db.getAttribute ("VlanPortMgmt", "_ROWID_", rowid, "PortLogicalName")
                if (#vlanTable >= 17) then
        	        db.rollback()
		            return "ERROR", "VLAN_MAX_LIMIT_ERROR"
                end
                local vlanConfig = {}
                vlanConfig ["vlan.vlanId"] = inputTable["VlanPortMgmt.DefaultPVID"]
                valid = db.insert("vlan", vlanConfig)
                if (not valid) then
        	        db.rollback()
		            return "ERROR", "VLAN_CONFIG_FAILED"
                end
            else
                local vlanRowId = db.getAttribute("vlan", "vlanId", inputTable["VlanPortMgmt.DefaultPVID"], "_ROWID_")
                local portLogName = db.getAttribute ("VlanPortMgmt", "_ROWID_", rowid, "PortLogicalName")
            end
        end

    --Checking for used vlans.
    elseif (operation == "delete") then
    	for k,v in pairs(inputTable) do
            local vlanId = v
            local vlanName = db.getAttribute ("vlan","vlanId",vlanId, "vlanName")
            if (vlanName) then
                local rows = db.getRowsWhere ("vlan", "vlanId="..vlanId.." and vlanName='"..vlanName.."'");
                -- if there are rows other than dummy ports 
                if (#rows > 1) then
            		db.rollback()
    		        return "ERROR", "VLAN_CONFIG_IN_USE"
                end
                -- delete the entries for vlanId from vlan
                rows = db.getRowsWhere ("vlan", "vlanId="..vlanId);
                for kk, vv in pairs (rows) do
                    valid = vlanConfig.config ({vv["vlan._ROWID_"]}, -1, "delete")
                end
            end
        end
    end

    local oldMode = nil
    local newMode = nil    
    local oldPvid = nil
    local newPvid = nil
    if (DBTable == "VlanPortMgmt") then
        oldMode = db.getAttribute ("VlanPortMgmt","_ROWID_",rowid,"PortVlanMode")
    	oldPvid = db.getAttribute ("VlanPortMgmt","_ROWID_",rowid,"DefaultPVID")
    end
    if (operation ~= "delete") then
    	valid = vlanConfig.config (inputTable, rowid, operation)
    end
    if (DBTable == "VlanPortMgmt") then
        newMode = db.getAttribute ("VlanPortMgmt","_ROWID_",rowid,"PortVlanMode")
	    newPvid = db.getAttribute ("VlanPortMgmt","_ROWID_",rowid,"DefaultPVID")
    end    

    --dummy updates for vlan rows if mode changed in VlanPortMgmt.
    if (valid and DBTable == "VlanPortMgmt") then
        if ((oldMode ~= nil and newMode ~= nil and oldMode ~= newMode) or (oldPvid ~= nil and newPvid ~= nil and oldPvid ~= newPvid)) then
            local portLogName = "vlan." .. db.getAttribute ("VlanPortMgmt", "_ROWID_", rowid, "PortLogicalName")
            local table = db.getTable ("vlan")
            local i = 0
	        for k,v in pairs(table) do
	            i = i + 1
                local row = table[i]
                local reqUpdate = false
                if (row ~= nil) then
                    if (row[portLogName] ~= nil and row[portLogName] == "1") then
                        reqUpdate = true
                    end
                    if (reqUpdate) then
                        db.setAttribute ("vlan", "_ROWID_", row["vlan._ROWID_"], db.getAttribute ("VlanPortMgmt", "_ROWID_", rowid, "PortLogicalName"), row[portLogName])
                   end
                end
            end            
        end
    end

    -- PVID cannot go beyond those in vlan table
    if (valid and DBTable == "VlanPortMgmt") then
        valid = db.getAttribute ("vlan", "vlanId", inputTable["VlanPortMgmt.DefaultPVID"], "vlanName")
    end

    -- access port should have only one vlan membership ..
    if (valid and DBTable == "VlanPortMgmt" and inputTable["VlanPortMgmt.PortVlanMode"] == "0") then
        locTable = {}; 
        DBTable="vlan";
        locTable ["vlan.new_vlanId_"..inputTable["VlanPortMgmt.DefaultPVID"]] = inputTable["VlanPortMgmt.DefaultPVID"]
        valid = vlanConfig.portVlanConfig (locTable,rowid, 1)
        DBTable="VlanPortMgmt";
    -- otherwise, set the PVID vlan in the membership table
    elseif (valid and DBTable == "VlanPortMgmt") then
        local tab = {}
        tab ["vlan.vlanId"] = inputTable["VlanPortMgmt.DefaultPVID"]
        tab ["vlan.vlanName"] = db.getAttribute ("vlan", "vlanId", tab ["vlan.vlanId"], "vlanName");
        tab ["vlan.portName"] = db.getAttribute ("VlanPortMgmt", "_ROWID_", rowid, "PortLogicalName")
        db.insert("vlan", tab) -- ignore failure
    end

    -- Adding configuration for vlan subnetting.
	if (operation == "delete" or
        (operation == "add" and inputTable["interVLANRouting.status"] ~= nil and inputTable["insertRpt"] ~= "1")) then
        valid = vlanConfig.configDep (inputTable, rowid, operation)
        DBTable = "vlan"
	elseif (operation == "edit" and DBTable ~= "VlanPortMgmt") then
        local ifName = "bdg" .. inputTable["vlan.vlanId"] or ''
		local interVlan = db.getRow ("interVLANRouting", "ifName", ifName)
        if (interVlan ~= nil) then	        
    		interVlan["interVLANRouting.status"] = inputTable["interVLANRouting.status"]
    		DBTable = "interVLANRouting"
	    	valid = vlanConfig.config(interVlan, interVlan["interVLANRouting._ROWID_"], "edit");
        end
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
        return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "PORT_VLAN_CONFIG_FAILED"
	end
end

-- vlan config
function vlanConfig.portVlanConfig (inputTable,rowid, commitCheck)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

    -- temp, we need to reuse this foo under single commit 
    if (commitCheck ~= 1) then
    	db.beginTransaction() --begin transaction
    end
	local valid = false
    local isUpdated = false
	
    local portLogName = "vlan." .. db.getAttribute ("VlanPortMgmt", "_ROWID_", rowid, "PortLogicalName")
    local defPVID = db.getAttribute ("VlanPortMgmt", "_ROWID_", rowid, "DefaultPVID")
    local table = db.getTable ("vlan")
    local i = 0
	for k,v in pairs(table) do
	    i = i + 1
        local row = table[i]
        local reqUpdate = false
        if (row ~= nil) then
            if (inputTable["vlan.row_" .. row["vlan._ROWID_"]] == nil) then
                if (row["vlan.portName"] ~= nil and "vlan." .. row["vlan.portName"] == portLogName) then
                    -- in any case, the current PVID must be mapped on the membership table ??
                    if (defPVID ~= row["vlan.vlanId"]) then
                        valid = vlanConfig.config ({row["vlan._ROWID_"]}, -1, "delete") 
                    else
                        valid = true
                    end
                    isUpdated = true
                    if (not valid) then
                        break
                    end
                end                
            end
        end
    end

    i = 0
	for k,v in pairs(table) do
	    i = i + 1
        local row = table[i]
        local reqUpdate = false
        if (row ~= nil) then
            if (inputTable["vlan.new_vlanId_" .. row["vlan.vlanId"]] ~= nil) then
                local tab = {}
                tab ["vlan.vlanId"] = inputTable["vlan.new_vlanId_" .. row["vlan.vlanId"]]
                tab ["vlan.vlanName"] = db.getAttribute ("vlan", "vlanId", tab ["vlan.vlanId"], "vlanName");
                tab ["vlan.portName"] = db.getAttribute ("VlanPortMgmt", "_ROWID_", rowid, "PortLogicalName")
                vlanConfig.config (tab, -1, "add")
            end
        end
    end

    if (not isUpdated) then
        valid = true
    end

    -- temp, we need to reuse this foo under single commit 
    if (commitCheck == 1) then
        return valid;
    end

	-- return
	if (valid) then
		db.commitTransaction(true)    
        return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "PORT_VLAN_CONFIG_FAILED"
	end
end

-- vlan port mgmt update
function vlanConfig.portMgmtUpdate (inputTable)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transaction
	local valid = false
	
    local boardName = db.getAttribute ("unitName","_ROWID_","1","unitName")
    local portVlanConfig = db.getRowWhere ("VlanPortMgmt","PortName = 'OptionalPort'")
    if (portVlanConfig ~= nil) then
        if (inputTable["ConfigPort.LogicalIfName"] == "LAN") then
            
            if (boardName ~= nil and boardName ~= "WRVS5110") then
                portVlanConfig["VlanPortMgmt.PortType"] = inputTable["ConfigPort.LogicalIfName"]
                portVlanConfig["VlanPortMgmt.DefaultPVID"] = "1"
            else
                portVlanConfig["VlanPortMgmt.PortType"] = "BRIDGE"
            end
        else
            if (inputTable["ConfigPort.LogicalIfName"] == "WAN2") then
                portVlanConfig["VlanPortMgmt.PortType"] = "WAN"
            else
                portVlanConfig["VlanPortMgmt.PortType"] = inputTable["ConfigPort.LogicalIfName"]
            end
            if (boardName ~= nil and boardName ~= "WRVS5110") then
                portVlanConfig["VlanPortMgmt.DefaultPVID"] = "4094"
            else
                portVlanConfig["VlanPortMgmt.DefaultPVID"] = "1"
            end
                portVlanConfig["VlanPortMgmt.PortVlanMode"] = "0"            
                local updateError = false
                local portLogName = "vlan." .. portVlanConfig["VlanPortMgmt.PortLogicalName"]
                local table = db.getTable ("vlan")
                local i = 0
            	for k,v in pairs(table) do
	                i = i + 1
                    local row = table[i]
                    if (row["vlan.portName"] == portVlanConfig["VlanPortMgmt.PortLogicalName"]) then
                        valid =  db.delete ("vlan",{row["vlan._ROWID_"]})
                        if (not valid) then
                            updateError = true
                            break
                        end
                    end
                end
                if (updateError) then
                    db.rollback()
		            return "ERROR", "VLAN_UPDATE_CONFIG_FAILED"
                end
           end
        valid = db.update("VlanPortMgmt", portVlanConfig, portVlanConfig["VlanPortMgmt._ROWID_"])
        if (valid and (portVlanConfig["VlanPortMgmt.PortType"] == "LAN" or portVlanConfig["VlanPortMgmt.PortType"] == "BRIDGE") and boardName ~= nil) then
            local portLogName = "vlan." .. portVlanConfig["VlanPortMgmt.PortLogicalName"]
            local vlanConfigRow = db.getRow ("vlan","vlanId","1")
            if (vlanConfigRow ~= nil) then
                locTable = {}
                locTable ["vlan.vlanId"] = 1
                locTable ["vlan.vlanName"] = db.getAttribute ("vlan", "vlanId", "1", "vlanName")
                locTable ["vlan.portName"] = portVlanConfig["VlanPortMgmt.PortLogicalName"]
                valid =  db.insert ("vlan", locTable)
            end
            if (not valid) then
                db.rollback()
		        return "ERROR", "VLAN_UPDATE_CONFIG_FAILED"
            end
        end
    else
        valid = true        
    end

	-- return
	if (valid) then
		db.commitTransaction(true)    
        return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "PORT_VLAN_CONFIG_FAILED"
	end
end

function vlanConfig.vlanDeleteCheck (rows)
    for k,v in pairs(rows) do
        local row = db.getRow ("vlan","vlanId",v)
        local usingRows = db.getRowsWhere ("TrafficSelector","MatchType = 4 and VlanId = " .. row["vlan.vlanId"])
        if (usingRows ~= nil and #usingRows ~= 0) then
            return "ERROR", "VLAN_IN_USE_BY_BW_TRAFFIC_SELECTOR"
        end
    end
    return "OK", "STATUS_OK"
end

function vlanConfig.portVlanEditCheck (rowid)
    local portVlanConfig = db.getRow ("VlanPortMgmt","_ROWID_",rowid)
    local bwStatus = db.getAttribute ("BandwidthProfileStatus","_ROWID_","1","Status")
	if (portVlanConfig ~= nil) then
	local usingRows = db.getRowsWhere ("TrafficSelector","MatchType = 3 and PortName = '" .. portVlanConfig["VlanPortMgmt.PortName"] .. "'")
		if (usingRows ~= nil and #usingRows ~= 0) then
			if(bwStatus ~= "0") then 
			return "ERROR", "PORT_VLAN_IN_USE_BY_BW_TRAFFIC_SELECTOR"
			end
		end
	end
  
    return "OK", "STATUS_OK"
end

