--[[
#### Vivek Dutt Sharma.
#### TeamF1
#### www.TeamF1.com
#### Nov 3, 2009
#### File: algs.lua
#### Description: wanPort functions
#### Revisions:
]]--

--************* Initial Code *************
--package wan
wan = {}

--************* Functions *************
--[[
/******************************************************************************
*
* wan.config - hadles all data base manipulation operations.
*
* RETURNS: true or false
*/
]]--
function wan.config (inputTable, rowid, operation)
    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            return db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            return db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
            return false
        end
    else
        return false
    end
end

--[[
/******************************************************************************
*
* wan.portConfig - updates WAN port configurations
*
* This routine updates data base for the following configurations.
* 1. Set Port Speed
* 2. Set MAC Address Option
* 
* RETURNS: OK or ERROR
*/
]]--
function wan.portConfig (inputTable)
    local valid = false
    local PortMgmtRow = nil
    if(inputTable["Wan.LogicalIfName"] == "WAN1") then
	    PortMgmtRow = db.getRowWhere ("portMgmt","PortName= 'DedicatedWAN'")
	else
		PortMgmtRow = db.getRowWhere ("portMgmt","PortName= 'OptionalPort'")
	end
    
    --Port Management
    if(inputTable["Wan.portSpeed"] == "0") then
    	PortMgmtRow["portMgmt.AutoNegEnable"] = 1
    else	
    	PortMgmtRow["portMgmt.AutoNegEnable"] = 0
    end
    if(inputTable["Wan.portSpeed"] == "1" or inputTable["Wan.portSpeed"] == "3" or inputTable["Wan.portSpeed"] == "5" ) then
    	PortMgmtRow["portMgmt.DuplexMode"]= 0
    end	
    if(inputTable["Wan.portSpeed"] == "2" or inputTable["Wan.portSpeed"] == "4" or inputTable["Wan.portSpeed"] == "6" ) then
    	PortMgmtRow["portMgmt.DuplexMode"] = 1
    end    
    if(inputTable["Wan.portSpeed"] == "1" or inputTable["Wan.portSpeed"] == "2") then
    	PortMgmtRow["portMgmt.PortSpeed"]= 10
    end
    if(inputTable["Wan.portSpeed"]=="3" or inputTable["Wan.portSpeed"]=="4") then
    	PortMgmtRow["portMgmt.PortSpeed"]= 100
    end
    if(inputTable["Wan.portSpeed"]=="5" or inputTable["Wan.portSpeed"]=="6") then
    	PortMgmtRow["portMgmt.PortSpeed"]= 1000
    end

    DBTable = "portMgmt"
	valid = wan.config (PortMgmtRow,PortMgmtRow["portMgmt._ROWID_"], "edit")
	if (not valid) then		
		return "ERROR", "PORT_MGMT_CONFIG_FAILED"
	end

    -- getting NimfConf table rows
    local NimfTable = db.getRowsWhere ("NimfConf","LogicalIfName = '" .. inputTable["Wan.LogicalIfName"] .. "' and AddressFamily = 2")       
    
    if (NimfTable == nil) then
        return "ERROR", "NIMF_CONFIG_NOTFOUND"
    end 
    
    -- config NIMF
    -- Set the wan port configuration
    local i=0 
    for k,v in pairs(NimfTable) do
        i = i + 1	      													
		local NimfRow = NimfTable[i]
	    NimfRow["NimfConf.UseDefaultMtu"] = inputTable["Wan.DefMtu"]
        -- NimfRow["NimfConf.SetMacAddress"] = inputTable["Wan.UseMac"]              
        if (inputTable["Wan.DefMtu"] == "0") then
            NimfRow["NimfConf.MtuSize"] = inputTable["Wan.MtuSize"]
        else
            if (NimfRow["NimfConf.RequireLogin"] == "0") then
                NimfRow["NimfConf.MtuSize"] = "1500"
            else
                NimfRow["NimfConf.MtuSize"] = "1492"
            end
        end        
    --[[    if (inputTable["Wan.UseMac"] == "2") then
           NimfRow["NimfConf.MacAddress"] = inputTable["Wan.MacAddr"]
        end
     --]]
            
        DBTable = "NimfConf"
        valid = wan.config (NimfRow ,NimfRow["NimfConf._ROWID_"], "edit")

        if (not valid ) then
            return "ERROR", "NIMF_CONFIG_NOTFOUND"
        end     
    end

    return "OK", "STATUS_OK"
end

--[[
/******************************************************************************
*
* wan.wanConfigSetWrap - Wrapper function to set different options of WAN1 and WAN2
*
* This routine updates data base for the following configurations.
* 1. Checks for Multicast MAC Address
* 2. Sets various wan port options.
* 
* RETURNS: ACCESS_DENIED or OK or ERROR
*/
]]--
function wan.wanConfigSetWrap (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	
--[[  Removing the following options as we are removing the mac address option in wanPortSetUp page --]]
   --[[
    if(inputTable["Wan.UseMac"] == "2" ) then
        local macAddr = inputTable["Wan.MacAddr"]
        local secondBit = string.sub(macAddr,2,2)
        if(secondBit == "1" or secondBit == "3" or secondBit == "5" or secondBit == "7" or secondBit == "9" or secondBit == "b" or secondBit == "d" or secondBit == "f" or secondBit == "B" or secondBit == "D" or secondBit == "F") then
            return "ERROR", "MULTICAST_MAC_ERROR"
        end
    end
    --]]

    --[[
    if(inputTable["Wan.UseMac2"] == "2" ) then
        local macAddr = inputTable["Wan.MacAddr2"]
        local secondBit = string.sub(macAddr,2,2)
        if(secondBit == "1" or secondBit == "3" or secondBit == "5" or secondBit == "7" or secondBit == "9" or secondBit == "b" or secondBit == "d" or secondBit == "f" or secondBit == "B" or secondBit == "D" or secondBit == "F") then
            return "ERROR", "MULTICAST_MAC_ERROR"
        end
    end
    --]]

    db.beginTransaction() --begin transaction
	local valid = false

    -- Configure PING through WAN Port
    local attackChecksConfig = db.getRow ("AttackChecks", "_ROWID_", "1")
    if (attackChecksConfig ~= nil) then
        attackChecksConfig["AttackChecks.ExternalPing"] = inputTable["AttackChecks.ExternalPing"]
        DBTable = "AttackChecks"
        valid = wan.config (attackChecksConfig, "1", "edit")
    end

    -- Configure WAN1 Port Settings
    if (valid) then
        local wan1PortConfig = {}
        wan1PortConfig["Wan.LogicalIfName"] = "WAN1"
        wan1PortConfig["Wan.DefMtu"] = inputTable["Wan.DefMtu"]
        if (inputTable["Wan.DefMtu"] == "0") then
            wan1PortConfig["Wan.MtuSize"] = inputTable["Wan.MtuSize"]
        end
        wan1PortConfig["Wan.portSpeed"] = inputTable["Wan.portSpeed"]        
        local errFg, stCode = wan.portConfig (wan1PortConfig)
        if (errFg ~= "OK") then
		    db.rollback()
    		return errFg, stCode
        else
            valid = true
        end
    end

    -- Configure WAN2 Port Settings
    if (valid) then
        local wan2PortConfig = {}
        wan2PortConfig["Wan.LogicalIfName"] = "WAN2"
        wan2PortConfig["Wan.DefMtu"] = inputTable["Wan.DefMtu2"]
        if (inputTable["Wan.DefMtu2"] == "0") then
            wan2PortConfig["Wan.MtuSize"] = inputTable["Wan.MtuSize2"]
        end
        wan2PortConfig["Wan.portSpeed"] = inputTable["Wan.portSpeed2"]        
        local errFg, stCode = wan.portConfig (wan2PortConfig)
        if (errFg ~= "OK") then
		    db.rollback()
    		return errFg, stCode
        else
            valid = true
        end
    end

    -- return "OK", if succeed all updates
    -- or
    -- return "ERROR", if update failed and also Roolback the database.
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "NIMF_CONFIG_FAILED"
	end
end

--[[
/******************************************************************************
* wan.wanConfigGetWrap - Wrapper function to get WAN Configuration
*
* RETURNS: A lua table contains WAN configuration
*/
]]--
function wan.wanConfigGetWrap (wanLogicalIfName)
    local logIfName = wanLogicalIfName   
    local outTable= wan.wanAdvConfigGet (logIfName )   
    return outTable
end

--[[
/******************************************************************************
* wan.wanAdvConfigGet - Wrapper function to get WAN Advanced Configurations
*
* RETURNS: A lua table contains WAN Advanced Configurations
*/
]]--
function wan.wanAdvConfigGet (logIfName)
    local outTable = {}
    local PortMgmtRow 
    if(logIfName == "WAN1") then
	    PortMgmtRow = db.getRowWhere ("portMgmt","PortName= 'DedicatedWAN'")
	else
		PortMgmtRow = db.getRowWhere ("portMgmt","PortName= 'OptionalPort'")
	end 
    local NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. logIfName .. "' and AddressFamily = 2")
    if (NimfRow ~= nil) then
        outTable["Wan.DefMtu"] = NimfRow["NimfConf.UseDefaultMtu"] or ''
        outTable["Wan.MtuSize"] = NimfRow["NimfConf.MtuSize"] or ''
        --outTable["Wan.UseMac"] = NimfRow["NimfConf.SetMacAddress"] or ''    

        --outTable["Wan.MacAddr"] = NimfRow["NimfConf.MacAddress"] or ''
        if (NimfRow["NimfConf.RequireLogin"] == "0") then
            outTable["Wan.MtuSizeMin"] = "68"
            outTable["Wan.MtuSizeMax"] = "1500"
        else
            outTable["Wan.MtuSizeMin"] = "128"
            outTable["Wan.MtuSizeMax"] = "1492"
        end        
    end
    if ( PortMgmtRow["portMgmt.AutoNegEnable"] == "0" ) then
        if (PortMgmtRow["portMgmt.PortSpeed"] == "10" and  PortMgmtRow["portMgmt.DuplexMode"] == "0") then
            outTable["Wan.portSpeed"] = "1"
        end
        if (PortMgmtRow["portMgmt.PortSpeed"] == "10" and  PortMgmtRow["portMgmt.DuplexMode"] == "1") then
            outTable["Wan.portSpeed"] = "2"
        end
        if (PortMgmtRow["portMgmt.PortSpeed"] == "100" and  PortMgmtRow["portMgmt.DuplexMode"] == "0") then
            outTable["Wan.portSpeed"] = "3"
        end
        if (PortMgmtRow["portMgmt.PortSpeed"] == "100" and  PortMgmtRow["portMgmt.DuplexMode"] == "1") then
            outTable["Wan.portSpeed"] = "4"
        end
        if (PortMgmtRow["portMgmt.PortSpeed"] == "1000" and  PortMgmtRow["portMgmt.DuplexMode"] == "0") then
            outTable["Wan.portSpeed"] = "5"
        end
        if (PortMgmtRow["portMgmt.PortSpeed"] == "1000" and  PortMgmtRow["portMgmt.DuplexMode"] == "1") then
            outTable["Wan.portSpeed"] = "6"
        end
    else
        outTable["Wan.portSpeed"] = "0"
    end

	return outTable
end
