--[[
#### Hussain Vali.
#### TeamF1
#### www.TeamF1.com
#### Dec 19, 2009
#### File: wanWizard.lua
#### Description: WAN Wizard functions
#### Revisions:
]]--


--table packages
wanWizard = {}

--************* Functions *************
--[[
/******************************************************************************
*
* wanWizard.config - hadles all data base manipulation operations.
*
* RETURNS: true or false
*/
]]--

function wanWizard.config (inputTable, rowid, operation)
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
           return db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
           return db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
           return db.delete(DBTable, inputTable)
        end
    else
        return false
    end
end

--[[
/******************************************************************************
*
* wanWizard.wanWizardConfig - updates configurations from WAN Wizard
*
* This routine updates data base for the following configurations.
* 1. Updates default admini user password.
* 2. Sets timezone ntp
* 3. Configures WAN
* 
* RETURNS: ACCESS_DENIED or OK or ERROR
*/
]]--

function wanWizard.wanWizardConfig (inputTable)
    --Local Variales
    local DEFAULT_USER_FLAG = "1"
    local ADMIN_USER_TYPE = "3"
    local NTP_ROWID = "1"
    local LOGICAL_IF_NAME = "WAN1"
    local ADDRESS_FAMILY = "2"
    local PPPOE_PROFILE_MAX_LIMIT = 2  -- limit specification for the pppoe profile	

	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	
	db.beginTransaction() --begin transaction
	local valid = false

    -- Setting Default Admin User Password
	local adminConfig = db.getRowWhere ("USERDBUsers", "UserType = " .. ADMIN_USER_TYPE .. " and DefaultUser = " .. DEFAULT_USER_FLAG)
	if (adminConfig ~= nil) then
        if (inputTable["USERDBUsers.Password"] and util.isAllMasked (inputTable["USERDBUsers.Password"]) == false) then
            adminConfig["USERDBUsers.Password"] = inputTable["USERDBUsers.Password"]
            DBTable = "USERDBUsers"
            valid = wanWizard.config (adminConfig, adminConfig["USERDBUsers._ROWID_"], "edit")
        else
            valid = true
        end
    else
		db.rollback()
		return "ERROR", "ADMIN_USER_CONFIG_NOT_FOUND"
	end

    -- Setting timezone configuration
    if (valid) then
        local ntpConfig = db.getRow ("ntp", "_ROWID_", NTP_ROWID)
        if (ntpConfig ~= nil) then
            ntpConfig["ntp.timezone"] = inputTable["ntp.timezone"]
            DBTable = "ntp"
            valid = wanWizard.config (ntpConfig, ntpConfig["ntp._ROWID_"], "edit")
        else
    		db.rollback()
	    	return "ERROR", "NTP_CONFIG_NOT_FOUND"
        end
    end

    -- Setting WAN Configuration
    if (valid) then
        -- return error if wan Configuration not found
        local NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. LOGICAL_IF_NAME .. "' and AddressFamily = " .. ADDRESS_FAMILY)

        -- Retrying for configuration in NimfConf.
        local retryIdx = 1
        while (NimfRow == nil and retryIdx <= 3) do
            NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. LOGICAL_IF_NAME .. "' and AddressFamily = " .. ADDRESS_FAMILY)
            retryIdx = retryIdx + 1
        end

        if (NimfRow == nil) then
    		db.rollback()
            return "ERROR", "NIMF_CONFIG_NOTFOUND"
        end

        local rowid = NimfRow["NimfConf._ROWID_"]
        local NimfConfTable = {}
        local wanConfTable = {}
		
        -- Disabling PPPOE Profiles if ISP type is not PPPOE   
        if (NimfRow["NimfConf.ConnectionType"] == "pppoe" and inputTable["Wan.IspType"] ~= "2") then
            local where1 = "LogicalIfName = '" .. LOGICAL_IF_NAME .. "' and AddressFamily = 2"
            local where2 = "LogicalIfName = '" .. LOGICAL_IF_NAME .. "'"
            local nimfRow = db.getRowsWhere("NimfConf", where1)
            local pppoeProfileRow = db.getRowsWhere("PppoeProfile", where2)

           --[[ if(nimfRow ~= nil) then
                local i=0
                for k,v in pairs(nimfRow) do
                    i = i + 1
                    local row = nimfRow[i]
                    if (row["NimfConf.IfStatus"] == "1") then
                        row["NimfConf.IfStatus"] = 0
                        DBTable = "NimfConf"
                        valid = wanWizard.config (row, row["NimfConf._ROWID_"], "edit")
                        if(not valid) then
                            db.rollback()
                            return "ERROR", "DISABLE_OP_FAILED" 
                        end 
                    end
                end
            end --]]

            if(pppoeProfileRow ~= nil) then
                local i=0
                for k,v in pairs(pppoeProfileRow) do
                    i = i + 1
                    local row = pppoeProfileRow[i]
                    if (row["PppoeProfile.Status"] == "1") then
                        row["PppoeProfile.Status"] = 0
                        DBTable = "PppoeProfile"
                        valid = wanWizard.config (row, row["PppoeProfile._ROWID_"], "edit")
                        if(not valid) then
                            db.rollback()
                            return "ERROR", "DISABLE_OP_FAILED"
                        end
                    end
                end
            end

            -- Enabling default WAN Configuration
            NimfConfTable["NimfConf.IfStatus"] = "1"
        end       
        
    	-- config NIMF
        NimfConfTable["NimfConf.LogicalIfName"] = LOGICAL_IF_NAME
        if (inputTable["Wan.IspType"] == "0") then    -- dhcpc
            NimfConfTable["NimfConf.RequireLogin"] = "0"
            NimfConfTable["NimfConf.ConnectionType"] = "dhcpc"
            if (inputTable["Wan.UseMac"] ~= nil) then
                NimfConfTable["NimfConf.SetMacAddress"] = inputTable["Wan.UseMac"]
                if (inputTable["Wan.UseMac"] == "2") then
                    NimfConfTable["NimfConf.MacAddress"] = inputTable["Wan.macAddress"]
                end
            end

            wanConfTable["Dhcpc.LogicalIfName"] = LOGICAL_IF_NAME
            wanConfTable["Dhcpc.HostName"] = inputTable["Wan.HostName"]
            wanConfTable["Dhcpc.GetDnsFromIsp"] = inputTable["Wan.IsDNSDyn"]
            if (inputTable["Wan.IsDNSDyn"] == "0") then
                wanConfTable["Dhcpc.PrimaryDns"] = inputTable["Wan.PriDnsIpAddr"]
                wanConfTable["Dhcpc.SecondaryDns"] = inputTable["Wan.SecDnsIpAddr"]
            end
            DBTable = "Dhcpc"
            if (db.existsRowWhere("Dhcpc", "LogicalIfName = '".. LOGICAL_IF_NAME .. "'")) then
                valid = wanWizard.config(wanConfTable, db.getAttribute ("Dhcpc", "LogicalIfName", LOGICAL_IF_NAME, "_ROWID_"), "edit")
            else
                valid = wanWizard.config(wanConfTable, "-1", "add")
            end

        elseif (inputTable["Wan.IspType"] == "1") then -- ifStatic
            NimfConfTable["NimfConf.RequireLogin"] = "0"
            NimfConfTable["NimfConf.ConnectionType"] = "ifStatic"

            wanConfTable["ifStatic.LogicalIfName"] = LOGICAL_IF_NAME            
            wanConfTable["ifStatic.StaticIp"] = inputTable["Wan.IpAddr"]
            wanConfTable["ifStatic.NetMask"] = inputTable["Wan.SnetMask"]
            wanConfTable["ifStatic.Gateway"] = inputTable["Wan.GwIpAddr"]
            wanConfTable["ifStatic.PrimaryDns"] = inputTable["Wan.PriDnsIpAddr"]
            wanConfTable["ifStatic.SecondaryDns"] = inputTable["Wan.SecDnsIpAddr"]
            wanConfTable["ifStatic.AddressFamily"] = ADDRESS_FAMILY
            DBTable = "ifStatic"
            if (db.existsRowWhere("ifStatic", "LogicalIfName = '" .. LOGICAL_IF_NAME .. "' and AddressFamily = " .. ADDRESS_FAMILY)) then
                valid = wanWizard.config(wanConfTable, db.getAttribute ("ifStatic", "LogicalIfName", LOGICAL_IF_NAME, "_ROWID_"), "edit")
            else
                valid = wanWizard.config(wanConfTable, "-1", "add")
            end

        elseif (inputTable["Wan.IspType"] == "2") then -- PPPoE
           --[[ local wanRowCount = db.getAttribute("PppoeProfile", "LogicalIfName", LOGICAL_IF_NAME, "count(*)") or '0'
            if(tonumber(wanRowCount) >= PPPOE_PROFILE_MAX_LIMIT) then
        		db.rollback()
                return "ERROR", "PPPOE_PROFILES_LIMIT_ERROR"
            end
			
            if(db.existsRow("PppoeProfile", "ProfileName", inputTable["Wan.ProfileName"])) then
        		db.rollback()
                return "ERROR", "PROFILE_NAME_ALREADY_EXIST"            
            end
            

            -- pppoe configuration  
            local conId = 1             
            while(conId <= 8) do
                if(db.existsRow("NimfConf", "ConnectionId", conId)) then
                    conId = conId + 1
                else
                    break;
                end
            end  
            --]]

            --Adding in pppoe table
            local pppoeProfileTable = {}
            local operation = ""
            local rowidPppoeProfile = ""
            
            -- Check whether any WAN1 profile exists or not.If it exists then update else add                      
            local pppoeProfileRow = db.getRows ("PppoeProfile","LogicalIfName",LOGICAL_IF_NAME)                   
            if(#pppoeProfileRow ~= 0) then            	
            	 operation = "edit"            	 
            	 rowidPppoeProfile = pppoeProfileRow[1]["PppoeProfile._ROWID_"]            	  
           	else
           		 operation = "add"
           		 rowidPppoeProfile = "-1"
           	end            	
            pppoeProfileTable["PppoeProfile.LogicalIfName"] = LOGICAL_IF_NAME
            pppoeProfileTable["PppoeProfile.ProfileName"] = inputTable["Wan.ProfileName"]
            pppoeProfileTable["PppoeProfile.Status"] = "1"
            pppoeProfileTable["PppoeProfile.AuthOpt"] = "1" -- Auto-negotiate
            pppoeProfileTable["PppoeProfile.UserName"] = inputTable["Wan.LoginName"]
            pppoeProfileTable["PppoeProfile.Password"] = inputTable["Wan.LoginPassword"]
            pppoeProfileTable["PppoeProfile.GetIpFromIsp"] = inputTable["Wan.IsGetIpDyn"]
            if(inputTable["Wan.IsGetIpDyn"] == "0") then
                pppoeProfileTable["PppoeProfile.StaticIp"] = inputTable["Wan.IpAddr"]
                pppoeProfileTable["PppoeProfile.NetMask"] = inputTable["Wan.SnetMask"]
            end
            pppoeProfileTable["PppoeProfile.GetDnsFromIsp"] = inputTable["Wan.IsDNSDyn"]
            if(inputTable["Wan.IsDNSDyn"] == "0") then
                pppoeProfileTable["PppoeProfile.PrimaryDns"] = inputTable["Wan.PriDnsIpAddr"]
                pppoeProfileTable["PppoeProfile.SecondaryDns"] = inputTable["Wan.SecDnsIpAddr"]
            end           
            pppoeProfileTable["PppoeProfile.IdleTimeOutFlag"] = inputTable["Wan.IdleTimeOutFlag"]
            if (inputTable["Wan.IdleTimeOutFlag"] == "1") then
                pppoeProfileTable["PppoeProfile.IdleTimeOutValue"] = inputTable["Wan.IdleTime"]
            end

            DBTable = "PppoeProfile"
            valid = wanWizard.config(pppoeProfileTable,rowidPppoeProfile, operation)

            if (valid) then
                local pppoeTable={}
                pppoeTable["Pppoe.LogicalIfName"] = LOGICAL_IF_NAME
                pppoeTable["Pppoe.UserName"] = inputTable["Wan.LoginName"]
                pppoeTable["Pppoe.Password"] = inputTable["Wan.LoginPassword"]
                pppoeTable["Pppoe.GetIpFromIsp"] = inputTable["Wan.IsGetIpDyn"]
                if(inputTable["Wan.IsGetIpDyn"] == "0") then
                    pppoeTable["Pppoe.StaticIp"] = inputTable["Wan.IpAddr"]
                    pppoeTable["Pppoe.NetMask"] = inputTable["Wan.SnetMask"]
                end
                pppoeTable["Pppoe.GetDnsFromIsp"] = inputTable["Wan.IsDNSDyn"]
                if(inputTable["Wan.IsDNSDyn"] == "0") then
                    pppoeTable["Pppoe.PrimaryDns"] = inputTable["Wan.PriDnsIpAddr"]
                    pppoeTable["Pppoe.SecondaryDns"] = inputTable["Wan.SecDnsIpAddr"]
                end           
                pppoeTable["Pppoe.IdleTimeOutFlag"] = inputTable["Wan.IdleTimeOutFlag"]
                pppoeTable["Pppoe.IdleTimeOutValue"] = inputTable["Wan.IdleTime"]
                pppoeTable["Pppoe.ProfileName"] = inputTable["Wan.ProfileName"]
               -- pppoeTable["Pppoe.ConnectionId"] = conId
       
                DBTable = "Pppoe"
                valid = wanWizard.config(pppoeTable, "1", "edit")
            end

            -- Adding for MultiPppoe table
        --[[    if (valid) then
                local multiPPoETable={}        
                multiPPoETable["MultiPppoe.LogicalIfName"] = LOGICAL_IF_NAME
                multiPPoETable["MultiPppoe.ConnectionId"] = conId

                DBTable = "MultiPppoe"    
                valid = wanWizard.config(multiPPoETable, "-1", "add")
            end  --]]

            -- NimfConf configuration             
          --[[  if(valid) then
                local nimfTable={}
                local addressFamily = 2
                local connectiotype = "pppoe"
                local ifStatus = 1
                local requireLogin = 0
                local mtuSize = 1500
                local portSpeed = 1
                local setMacAddr = 0
                local useDefaultMtu = 1

                nimfTable["NimfConf.LogicalIfName"] = LOGICAL_IF_NAME
                nimfTable["NimfConf.AddressFamily"] = addressFamily
                nimfTable["NimfConf.ConnectionType"] = connectiotype
                nimfTable["NimfConf.IfStatus"] = ifStatus
                nimfTable["NimfConf.RequireLogin"] = requireLogin
                nimfTable["NimfConf.MtuSize"] = mtuSize
                nimfTable["NimfConf.PortSpeed"] = portSpeed
                nimfTable["NimfConf.SetMacAddress"] = setMacAddr
                nimfTable["NimfConf.UseDefaultMtu"] = useDefaultMtu
            --  nimfTable["NimfConf.ConnectionId"] = conId
            
                DBTable = "NimfConf"
                valid = wanWizard.config(nimfTable, "-1", "add")
            end --]]

            NimfConfTable["NimfConf.RequireLogin"] = "1"
            NimfConfTable["NimfConf.ConnectionType"] = "pppoe"
      		NimfConfTable["NimfConf.IfStatus"] = "1"

        elseif (inputTable["Wan.IspType"] == "3") then -- PPTP
            NimfConfTable["NimfConf.RequireLogin"] = "1"
            NimfConfTable["NimfConf.ConnectionType"] = "pptp"

            wanConfTable["Pptp.LogicalIfName"] = LOGICAL_IF_NAME
            wanConfTable["Pptp.UserName"] = inputTable["Wan.LoginName"]
            if (util.isAllMasked (inputTable["Wan.LoginPassword"]) == false) then
                wanConfTable["Pptp.Password"] = inputTable["Wan.LoginPassword"]
            end
            wanConfTable["Pptp.MyIp"] = inputTable["Wan.MyIpAddr"]
            wanConfTable["Pptp.ServerIp"] = inputTable["Wan.SvrIpAddr"]
            wanConfTable["Pptp.GetIpFromIsp"] = inputTable["Wan.IsGetIpDyn"]
            wanConfTable["Pptp.GetDnsFromIsp"] = inputTable["Wan.IsDNSDyn"]
            if (inputTable["Wan.IsGetIpDyn"] == "0") then
                wanConfTable["Pptp.StaticIp"] = inputTable["Wan.IpAddr"]
                wanConfTable["Pptp.NetMask"] = inputTable["Wan.SnetMask"]
            end
            if (inputTable["Wan.IsDNSDyn"] == "0") then
                wanConfTable["Pptp.PrimaryDns"] = inputTable["Wan.PriDnsIpAddr"]
                wanConfTable["Pptp.SecondaryDns"] = inputTable["Wan.SecDnsIpAddr"]
            end
            wanConfTable["Pptp.IdleTimeOutFlag"] = inputTable["Wan.IdleTimeOutFlag"]
            if (inputTable["Wan.IdleTimeOutFlag"] == "1") then
                wanConfTable["Pptp.IdleTimeOutValue"] = inputTable["Wan.IdleTime"]
            end
            DBTable = "Pptp"
            if (db.existsRowWhere("Pptp", "LogicalIfName = '".. LOGICAL_IF_NAME .. "'")) then
                valid = wanWizard.config(wanConfTable, db.getAttribute ("Pptp", "LogicalIfName", LOGICAL_IF_NAME, "_ROWID_"), "edit")
            else
                valid = wanWizard.config(wanConfTable, "-1", "add")
            end

        elseif (inputTable["Wan.IspType"] == "4") then -- L2TP
            NimfConfTable["NimfConf.RequireLogin"] = "1"
            NimfConfTable["NimfConf.ConnectionType"] = "l2tp"

            wanConfTable["L2tp.LogicalIfName"] = LOGICAL_IF_NAME
            wanConfTable["L2tp.UserName"] = inputTable["Wan.LoginName"]
            if (util.isAllMasked (inputTable["Wan.LoginPassword"]) == false) then
                wanConfTable["L2tp.Password"] = inputTable["Wan.LoginPassword"]
            end
            wanConfTable["L2tp.MyIp"] = inputTable["Wan.MyIpAddr"]
            wanConfTable["L2tp.ServerIp"] = inputTable["Wan.SvrIpAddr"]
            wanConfTable["L2tp.GetIpFromIsp"] = inputTable["Wan.IsGetIpDyn"]
            wanConfTable["L2tp.GetDnsFromIsp"] = inputTable["Wan.IsDNSDyn"]
            if (inputTable["Wan.IsGetIpDyn"] == "0") then
                wanConfTable["L2tp.StaticIp"] = inputTable["Wan.IpAddr"]
                wanConfTable["L2tp.NetMask"] = inputTable["Wan.SnetMask"]
            end
            if (inputTable["Wan.IsDNSDyn"] == "0") then
                wanConfTable["L2tp.PrimaryDns"] = inputTable["Wan.PriDnsIpAddr"]
                wanConfTable["L2tp.SecondaryDns"] = inputTable["Wan.SecDnsIpAddr"]
            end
            wanConfTable["L2tp.IdleTimeOutFlag"] = inputTable["Wan.IdleTimeOutFlag"]
            if (inputTable["Wan.IdleTimeOutFlag"] == "1") then
                wanConfTable["L2tp.IdleTimeOutValue"] = inputTable["Wan.IdleTime"]
            end
            DBTable = "L2tp"
            if (db.existsRowWhere("L2tp", "LogicalIfName = '".. LOGICAL_IF_NAME .. "'")) then
                valid = wanWizard.config(wanConfTable, db.getAttribute ("L2tp", "LogicalIfName", LOGICAL_IF_NAME, "_ROWID_"), "edit")
            else
                valid = wanWizard.config(wanConfTable, "-1", "add")
            end
        end

        if (valid) then
            DBTable = "NimfConf"
            valid = wanWizard.config(NimfConfTable, rowid, "edit")
        end
    end

    -- return "OK", if succeed all updates
    -- or
    -- return "ERROR", if update failed and also Roolback the database.
	if (valid) then
		db.commitTransaction()
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "WAN_WIZARD_CONFIG_FAILED"
	end
end
