--[[
#### Adarsh Uppula
#### TeamF1
#### www.TeamF1.com
#### June 29, 2007

#### File: web.lua
#### Description: Web functions.

#### Revisions:
None
]]--


--************* Requires *************

require "teamf1lualib/login"

--************* Initial Code *************

-- package web
web = {}

--************* Functions *************

-- Go to given page if logged in.
-- Assumes login validation already done.
function web.goToPage (page, htm, clearbutton)
	util.appendDebugOut("Wanting to go to page: " .. page .. "<br>")
	-- clear button press
	if clearbutton then ButtonType = nil end
	-- if htm
	if htm then page = page .. ".htm" end
	-- check if good page
	if (page == PLATFORM_PAGE) then
		cgilua.lp.include(HOME_PAGE)
	elseif (page ~= nil and util.fileExists(page)) then
		if (rebootFlag) then
			util.appendDebugOut("Going to reboot page: " .. REBOOT_PAGE .. "<br>")
			cgilua.lp.include(REBOOT_PAGE) --go to next page
		else
                        util.appendDebugOut("Going to the page: " .. page .. "<br>")
                        cgilua.lp.include(page) --go back to current page
		end
	else
		cgilua.lp.include(HOME_PAGE)
	end
end

-- Run given page if logged in.
-- Assumes login validation already done.
function web.runPage (page)
	-- check if good page
	if (cgi.page == PLATFORM_PAGE) then
		cgilua.lp.include(HOME_PAGE)
	elseif (page ~= nil and util.fileExists(page)) then
		dofile(page)
	else
		cgilua.lp.include(HOME_PAGE)
	end
end

-- Put file as string.
function web.includeFile (filepath)
	cgilua.put(util.fileToString(filepath))
end

-- Put file as string after lua procesing.
function web.includeMenu (filepath)
	cgilua.lp.include(filepath)
end

-- Convert given cgi table (only fields) to a LUA table.
-- Returns nil on error.
function web.cgiToLuaTable (cgitable)
	local luaTable = {}
	local splitparts = {}
	local keyStr = ""
	for k,v in pairs(cgitable) do
		splitparts = util.split(k, ".")
		if (#splitparts == 2) then
			-- for checkbox fields
			if (type(v) == "table" and #v == 2) then
				luaTable[splitparts[1] .. "." .. splitparts[2]] = v[1]
			-- for file upload
			elseif (type(v) == "table" and #v == 4) then
				luaTable[splitparts[1] .. "." .. splitparts[2]] = v
			-- regular field
			else
				luaTable[splitparts[1] .. "." .. splitparts[2]] = v
			end
		end
	end

	-- return
	return luaTable
end

-- Search given cgi table for button and return its (fixed) name.
function web.cgiFindButton (cgi)
	local field, splitted, value = web.cgiSearch(cgi, "button")
	if (splitted ~= nil and (splitted[#splitted] == "x" or splitted[#splitted] == "y")) then
		table.remove(splitted, #splitted)
		field = splitted[1]
		for k,v in pairs(splitted) do
			if (k ~= 1) then
				field = field .. "." .. v
			end
		end
	end
	return field, splitted, value
end

-- Search given cgi table and return value where key's
-- first part is given token. Returns table if value is not
-- a table with key "1".
-- Returns nil on error.
function web.cgiSearch (table, token)
	local splitted = {}
	local tab = {}
	for k,v in pairs(table) do
		splitted = util.split(k, ".")
		if (#splitted >= 1 and splitted[1] == token) then
			if (type(v) == "table") then
				return k, splitted, v
			else
				tab["1"] = v
				return k, splitted, tab
			end
		end
	end
	
	--none found
	return nil, nil, nil
end

-- If value true, return "checked", else "".
function web.checkboxSelected (value)
	--if equal
	if value then
		return "CHECKED"
	else
		return ""
	end
end

-- If value true, return "selected", else "".
function web.radioSelected (value)
	--if equal
	if value then
		return "CHECKED"
	else
		return ""
	end
end

-- If value true, return "selected", else "".
function web.dropdownSelected (value)
	--if equal
	if value then
		return "SELECTED"
	else
		return ""
	end
end

-- If value true, return "disabled", else "".
function web.inputDisabled (value)
	--if equal
	if value then
		return "DISABLED"
	else
		return ""
	end
end

-- Return argument corresponding to index (a string).
function web.getArgsFromIndex (index, arg0, arg1, arg2, arg3)
	if index == "0" then
		return arg0
	elseif index == "1" then
		return arg1
	elseif index == "2" then
		return arg2
	elseif index == "3" then
		return arg3
	end
end

-- Let user download given file as given name.
function web.download (filepath, filename)
	cgilua.header("Content-type", "application/octet-stream;")
	cgilua.header("Content-disposition", "attachment; filename=" .. filename)
	cgilua.put("\n" .. util.fileToString(filepath))
end

