-- websiteFilteringCLI.lua
-- Ravindranath
-- TeamF1
-- www.TeamF1.com
-- 02/03/10
--
-- Description
-- CLI interface set and get security website filtering routines

require "teamf1lualib/fwView"
--content filtering
function contentFilteringCfgInit (args)
    configRow = db.getRow("ContentFiltering", "_ROWID_", "1")
    local configRowWeb = db.getRow("WebComponents","_ROWID_", "1")
    configRow = util.tableAppend(configRow, configRowWeb)
    return 1, configRow
end

function contentFilteringCfgSave (configRow)
    errorFlag = "ERROR"
    statusCode = ""
    statusMessage = ""
    DBTable = "ContentFiltering"
    -- update Configuration
    	local procEntry = db.getAttribute("environment", "name", "HW_OFFLOAD_PROC_ENTRY", "value")
    	if (configRow["ContentFiltering.Status"] == "1") then
    		util.appendDebugOut("Exec: " .. os.execute("echo 0 >" .. procEntry))
	elseif ( configRow["ContentFiltering.Status"] == "0") then
    		util.appendDebugOut("Exec: " .. os.execute("echo 1 >" .. procEntry))
    	end
    		errorFlag, statusCode = fwView.contentFileteringConfig(configRow, "1", "edit")
    	
    	-- save db if no error
    	if (errorFlag == "OK") then db.save() end
    	statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage
end

function contentFilteringCfgInputVal (configRow)
	if (configRow["ContentFiltering.Status"] == "0")then
		printCLIError("Please Enable content filtering first")
		return false
	end
    return true
end

function contentFilteringCfgGet (args)
    local row1 = db.getRow ("ContentFiltering", "_ROWID_", "1")
    local row2 = db.getRow ("WebComponents", "_ROWID_", "1")
    if (row1 == nil and row2 == nil) then 
    print ("Content Filtering Configurations Not Available\n")
    else
    printLabel("Content Filtering")
    print ("WAN Security Checks\n")
    if (row1["ContentFiltering.Status"] == "1") then
    print ("Content Filtering : Enabled\n")
    else
    print ("Content Filtering : Disabled\n")
    end
    print ("LAN Security Checks\n")
    if(row2["WebComponents.ProxyEnable"] == "1") then
    print ("Proxy : Enabled\n")
    else
    print ("Proxy : Disabled\n")
    end
    if(row2["WebComponents.JavaComponentEnable"] == "1") then
    print ("Java : Enabled\n")
    else
    print ("Java : Disabled\n")
    end
    if(row2["WebComponents.ActivexControlEnable"] == "1") then
    print ("ActiveX : Enabled\n")
    else
    print ("ActiveX : Disabled\n")
    end
    if(row2["WebComponents.CookiesEnable"] == "1") then
    print ("Cookies : Enabled\n")
    else
    print ("Cookies : Disabled\n")
    end
    end
end

--trusted domains
function trustedDomainsCfgInit (args)
    -- for Add
    RowId = args[1]
    if (RowId == nil) then
	configRow = db.getDefaults(true, "TrustedDomains")
    -- for edit
    else
	configRow = db.getRow("TrustedDomains", "TrustedDomains._ROWID_",RowId)
    end
    if(configRow == nil)then
       print("Row doesnt exist probably deleted. Please exit from this mode\n")
       return -1, {}
    end
    return 1,configRow
end

function trustedDomainsCfgSave (configRow)
    errorFlag = ""
    statusCode = ""
    statusMessage = ""
    DBTable = "TrustedDomains"

    RowId = configRow ["TrustedDomains._ROWID_"]
    -- for add
    if (RowId == nil) then
	configRow["TrustedDomains.OID"] = nil 
	errorFlag, statusCode = fwView.trustedDomainConfig(configRow, "-1", "add")
    else
	errorFlag, statusCode = fwView.trustedDomainConfig(configRow, RowId, "edit")
    end

    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage
end

function trustedDomainsCfgDel (args)
    errorFlag = ""
    statusCode = ""
    statusMessage = ""
    DBTable = "TrustedDomains"
    rows = {}
    local configRow={}
    configRow=db.getRow("TrustedDomains","_ROWID_",args[1]) 
    if (configRow == nil)then
		errorFlag="ERROR"
	        statusMessage="Row doesnt exist"
                printCLIError("Row doesnt exist")
		return errorFlag, statusMessage
    else
    rows["rowid"] = configRow["TrustedDomains._ROWID_"]
	if (rows) then
		errorFlag, statusCode = fwView.deleteDomains (rows)
	end
    -- save db if no error
    	if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    end
    return errorFlag, statusMessage
end

function trustedDomainsCfgInputVal (configRow)
    local testRow=db.getRow("ContentFiltering","_ROWID_","1")
    if (testRow["ContentFiltering.Status"] == "0")then
	printCLIError("please Enable content filtering first")
	return false
    end 
    if (configRow["TrustedDomains.KeywordToAllow"] == nil or configRow["TrustedDomains.KeywordToAllow"] == "")then
	printCLIError("please enter a valid url")
	return false
    end
    return true
end

function trustedDomainsCfgGet (args)
    local resultTab = {}
    print("List of available  Approved URLS")
    local i = 0
    local table = db.getTable("TrustedDomains")
    if (table == nil) then
        print ("trusted domains configurations unavailable")
    else
        for k,v in pairs(table) do
        i = i + 1
        local row = table[i]
        if (i ~= nil) then
            resTab.insertField (resultTab, "ROW ID", row["TrustedDomains._ROWID_"] or "")
            resTab.insertField (resultTab, "Domain", row["TrustedDomains.KeywordToAllow"] or "")
        end
        end
    end
    resTab.print (resultTab, 0)
end


--blocked keywords
function blockedKeywordsCfgInit (args)
   -- for Add
    RowId = args[1]
    if (RowId == nil) then
	configRow = db.getDefaults(true, "BlockSites")
    -- for edit
    else
	configRow = db.getRow("BlockSites", "BlockSites._ROWID_",RowId)	
    end
    if(configRow == nil)then
       print("Row doesnt exist probably deleted. Please exit from this mode\n")
       return -1, {}
    end
    return 1,configRow
end

function blockedKeywordsCfgSave (configRow)
    errorFlag = ""
    statusCode = ""
    statusMessage = ""
    DBTable = "BlockSites"
    RowId = configRow ["BlockSites._ROWID_"]
    -- for add
    if (RowId == nil) then
	configRow["BlockSites.OID"] = nil 
	configRow["BlockSites.KeywordBlockingTo"]="1"
	configRow["BlockSites.Status"]="1"
	errorFlag, statusCode = fwView.blockedKeywordConfig(configRow, "-1", "add")
    else
	errorFlag, statusCode = fwView.blockedKeywordConfig(configRow, RowId, "edit")
    end

    -- save db if no error
    if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    return errorFlag, statusMessage
end

function blockedKeywordsCfgDel (args)
    errorFlag = ""
    statusCode = ""
    statusMessage = ""
    DBTable = "BlockSites"
    rows = {}
    local configRow={}
    configRow=db.getRow("BlockSites","_ROWID_",args[1]) 
    if (configRow == nil)then
		errorFlag="ERROR"
	        statusMessage="Row doesnt exist"
                printCLIError("Row doesnt exist")
		return errorFlag, statusMessage
    else
    rows["rowid"] = configRow["BlockSites._ROWID_"]
	if (rows) then
		errorFlag, statusCode = fwView.deleteKeywords (rows)
	end
    -- save db if no error
    	if (errorFlag == "OK") then db.save() end
    statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    end
    return errorFlag, statusMessage

end

function blockedKeywordsCfgInputVal (configRow)
    local testRow=db.getRow("ContentFiltering","_ROWID_","1")
    if (testRow["ContentFiltering.Status"] == "0")then
	printCLIError("please Enable content filtering first")
	return false
    end 
    if (configRow["BlockSites.KeywordToBlock"] == nil or configRow["BlockSites.KeywordToBlock"] == "")then
		printCLIError("Enter valid blocked_keyword")
		return false
    end
    return true
end

function blockedKeywordsStatusModify (args)
    DBTable = "BlockSites"
    rows = {}
    local configRow={}
    configRow=db.getRow("BlockSites","_ROWID_",args[1]) 
    if (configRow == nil)then
		errorFlag="ERROR"
	        statusMessage="Row doesnt exist"
                printCLIError("Row doesnt exist")
		return errorFlag, statusMessage
    else
    rows["rowid"] = args[1]
    	if (rows) then
		errorFlag, statusCode = fwView.modifyKeywordStatus (rows,args[2]) 
    	end
	-- save db if no error
    	if (errorFlag == "OK") then db.save() end
	statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE)
    end
    return errorFlag, statusMessage
end

function blockedKeywordsCfgGet (args)
    local resultTab = {}

    printLabel ("Blocked Keywords")
    print ("List of available Blocked Keywords\n")
    local i = 0
    local status = ""
    local table = db.getTable ("BlockSites")
    if (table == nil) then
        print ("Blocked Keywords configuration unavailable")
    else
        for k,v in pairs(table) do
        i = i + 1
        local row = table[i]
        if (i ~= nil) then
        resTab.insertField (resultTab, "ROW ID", row["BlockSites._ROWID_"]or "")
        if(row["BlockSites.Status"] == "1")then
                status = "Enabled"
            elseif (row["BlockSites.Status"] == "0") then
                status = "Disabled"
            end
        resTab.insertField (resultTab, "Status", status)
        resTab.insertField (resultTab, "Blocked Keyword ", row["BlockSites.KeywordToBlock"] or "")
        end
        end
   end
   resTab.print (resultTab, 0)
end
