--[[
#### Chaitanya Dandamudi
#### TeamF1
#### www.TeamF1.com
#### December 12, 2007

#### File: x509.lua
#### Description: x509 functions

#### Revisions:
01d, 22mar10, rac changed to capture extra error messages while adding active 
              self certificates(part of 17062 spr fix)
01c, 15nov08, til changed signature of certLoad for identifying the input
              certificate type(ca/self).   
01b, 22oct08, til included support for httpdMgmt tbl update
              on deletion of active certificate.
]]--


--************* Requires *************

require "x509Lib"

--************* Initial Code *************

--package x509
x509 = {}

--table packages
cert = {}

--************* Functions *************

-- x509 inputValidate
function x509.inputValidate (inputTable, operation)
        if (true) then
            return db.typeAndRangeValidate(inputTable)
        end
        return false
end
        
-- x509 configuration
function x509.uploadTrustedCACert (inputTable)
    -- if not allowed to edit
        if (ACCESS_LEVEL ~= 0) then
                NextPage = "uploadTrustedCertificate"
                return "ACCESS_DENIED", "ADMIN_REQD"
        end
        db.beginTransaction() --begin transaction
        local valid = false

        -- uploading Certificate
       	util.appendDebugOut(util.tableToStringRec(inputTable))
    	local fileSize = inputTable["file.upload"]["filesize"]
	local fileName = inputTable["file.upload"]["filename"]
    	local fileHandle =  inputTable["file.upload"]["file"]
	-- moving file to required path
	local fileName = cgilua.cookies.get("TeamF1Login")
	local valid = os.execute("mv" .. " " .. "/tmp/" .. 
                             fileName .. " /tmp/" .. "temp.crt")

    local certInputTable = {}
    local certData
    certData = util.fileToString("/tmp/temp.crt")
    
    local status,name,subjectName,issuerName,expiryTime,serialNumber,format =
                                                    x509Lib.certLoad(certData, "ca")
                                                    
    local valid = os.execute("rm" .. " /tmp/" .. "temp.crt")

    if (status == -1) then
        return "ERROR", "X509_INVALID_CA"
    end 
 
    certInputTable["cert.name"] = name
    certInputTable["cert.subjectName"] = subjectName
    certInputTable["cert.issuerName"] = issuerName
    certInputTable["cert.expiryTime"] = expiryTime
    certInputTable["cert.serialNumber"] = serialNumber
    certInputTable["cert.format"] = format
    certInputTable["cert.certType"] = "ca"
    certInputTable["cert.certData"] = certData
    valid = db.insert("cert",certInputTable)
	
    -- return
	if (valid) then
		db.commitTransaction(true)		
		return "OK", "X509_CA_ADD_SUCCESS"
	else
		db.rollback()
		return "ERROR", "X509_CA_ADD_FAILURE"
	end
end

--Trusted Cert Delete
function x509.trustedCACertDelete (rowIds)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions

	-- delete Trusted Certificates
        for k,v in pairs(rowIds) do
            local inTable = {}
            inTable["cert._ROWID_"] = v;
          
            -- delete <hash_of_certificate>.0 file
           local status,subjectHash = x509Lib.getCertHash(v)
          
           if (status == -1) then
               return "ERROR", "X509_CA_DEL_FAILURE"
           end     
           
           os.execute("rm" .. " /var/certs/ca/" .. subjectHash .. ".0" )
          
           -- delete CA<rowId>.crt file 
           os.execute("rm" .. " /var/certs/ca/" .. "CA" .. v .. ".crt" )
         
           valid = db.delete("cert", inTable)
	end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "X509_CA_DEL_SUCCESS"
	else
		db.rollback()
		return "ERROR", "X509_CA_DEL_FAILURE"
	end

end

--Active Self Cert Delete
function x509.activeSelfCertDelete (rowIds)
      
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions

	-- delete Trusted Certificates
        for k,v in pairs(rowIds) do
            local inTable = {}
            inTable["cert._ROWID_"] = v;
		    valid = db.delete("cert", inTable)
     	end

	if (valid) then
        -- Restart httpd
        -- get the column 'certEvent''s value from httpsMgmt
        httpsMgmtRow = db.getRow ("httpsMgmt","_ROWID_","1")
        -- toggle the value of 'certEvent'
        if (httpsMgmtRow) then 
            if (httpsMgmtRow["httpsMgmt.certEvent"] == "1") then
                httpsMgmtRow["httpsMgmt.certEvent"] = "0"
            elseif (httpsMgmtRow["httpsMgmt.certEvent"] == "0") then
                httpsMgmtRow["httpsMgmt.certEvent"] = "1"
            end -- certEventFlag    
            -- update the table 'httpsMgmt' with new 'certEvent' value.
            valid = db.update("httpsMgmt", httpsMgmtRow, "1")
        end --httpsMgmtRow
        
        -- commit the transaction
	    db.commitTransaction(true)
	    -- return
		return "OK", "X509_SELF_DEL_SUCCESS"
	else
		db.rollback()
		return "ERROR", "X509_SELF_DEL_FAILURE"
	end

end


-- Generate Self Certificate
function x509.genSelfCert (inputTable)
    -- if not allowed to edit
        if (ACCESS_LEVEL ~= 0) then
                NextPage = "generateSelfCertificateRequest"
                return "ACCESS_DENIED", "ADMIN_REQD"
        end
        db.beginTransaction() --begin transaction
        local valid = false
        -- overright with new certificate for same applications, currently
        -- it is supporting for https and ipsec, in future we may support more
        -- applications.	
        httpsConfigRow = db.getRow ("x509SelfCertReq","appType","https")
    	ipsecConfigRow = db.getRow ("x509SelfCertReq","appType","ipsec")
        -- append CN= for subjectName if '=' is not found in the input string 
        local stIdx,endIdx = string.find(inputTable["x509SelfCertReq.subjectName"],"=")
        if (stIdx == nil and endIdx == nil) then
            inputTable["x509SelfCertReq.subjectName"] = "CN=" .. inputTable["x509SelfCertReq.subjectName"]
        end
        if (inputTable["x509SelfCertReq.appType"] == "https") then
             if (httpsConfigRow) then
                 -- clear the existing table's hidden data before update. 
                 inputTable["x509SelfCertReq.selfCertData"] = ""
                 inputTable["x509SelfCertReq.selfKeyData"] = ""
                 valid = db.update("x509SelfCertReq",inputTable,
                          httpsConfigRow["x509SelfCertReq._ROWID_"])
             else             
	             valid = db.insert("x509SelfCertReq", inputTable)
             end              
        elseif (inputTable["x509SelfCertReq.appType"] == "ipsec") then
             if (ipsecConfigRow) then
                 -- clear the existing table's hidden data before update. 
                 inputTable["x509SelfCertReq.selfCertData"] = ""
                 inputTable["x509SelfCertReq.selfKeyData"] = ""
                valid = db.update("x509SelfCertReq",inputTable,
                          ipsecConfigRow["x509SelfCertReq._ROWID_"])
             else             
	             valid = db.insert("x509SelfCertReq", inputTable)
             end              
        end
    
     -- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "X509_SELF_GEN_FAILURE"
	end
end
 

--Self Cert Delete
function x509.selfCertDelete (rowIds)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions

	-- delete Self Certificates
        for k,v in pairs(rowIds) do
            local inTable = {}
            inTable["x509SelfCertReq._ROWID_"] = v;
		    valid = db.delete("x509SelfCertReq", inTable)
	end

    -- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "X509_SELF_DEL_SUCCESS"
	else
		db.rollback()
		return "ERROR", "X509_SELF_DEL_FAILURE"
	end

end


-- upload Active Self Certificates
function x509.uploadSelfCert (inputTable)
    -- if not allowed to edit
        if (ACCESS_LEVEL ~= 0) then
                NextPage = "uploadActiveSelfCertificate"
                return "ACCESS_DENIED", "ADMIN_REQD"
        end
        db.beginTransaction() --begin transaction
        local valid = false

        -- uploading Certificate
       	util.appendDebugOut(util.tableToStringRec(inputTable))
    	local fileSize = inputTable["file.upload"]["filesize"]
	local fileName = inputTable["file.upload"]["filename"]
    	local fileHandle =  inputTable["file.upload"]["file"]
	-- moving file to required path
	local fileName = cgilua.cookies.get("TeamF1Login")
	local valid = os.execute("mv" .. " " .. "/tmp/" .. 
                             fileName .. " /tmp/" .. "temp.crt")

    local certInputTable = {}
    local certData
    certData = util.fileToString("/tmp/temp.crt")
    
    local status,name,subjectName,issuerName,expiryTime,serialNumber,format =
                                                    x509Lib.certLoad(certData, "self")
                                                    
    local valid = os.execute("rm" .. " /tmp/" .. "temp.crt")

    if (status == -1) then
        return "ERROR", "X509_INVALID_SELF"
    elseif (status == 27) then        
        return "ERROR", "X509_INVALID_SELF_CA"
    elseif (status == 26) then        
        return "ERROR", "X509_INVALID_SELF_PURPOSE"        
    end   
 
    certInputTable["cert.name"] = name
    certInputTable["cert.subjectName"] = subjectName
    certInputTable["cert.issuerName"] = issuerName
    certInputTable["cert.expiryTime"] = expiryTime
    certInputTable["cert.serialNumber"] = serialNumber
    certInputTable["cert.format"] = format
    certInputTable["cert.certType"] = "self"
    certInputTable["cert.certData"] = certData
    valid = db.insert("cert",certInputTable)

    -- return
	if (valid) then
		db.commitTransaction(true)		
		return "OK", "X509_SELF_ADD_SUCCESS"
	else
		db.rollback()
		return "ERROR", "X509_SELF_ADD_FAILURE"
	end
end
