#!/bin/sh
#
# usbProgram - script to handle USB devices
#
# Copyright (c) 2010, TeamF1, Inc.
#
# modification history
# --------------------
# 01a,02apr10,anp  written.
#
# DESCRIPTION
# This script is called by the udev and it handles action on different USB
# devices.
#

# macros
USB1_FILE=/var/usb1Info
USB2_FILE=/var/usb2Info
DEV_DIR=/var/dev
LED_BIN=/pfrm2.0/bin/ledctrl
USB1_LINK=/usb1
USB2_LINK=/usb2
USB1_FILE_LIST=/var/usb_filelist1.txt
USB2_FILE_LIST=/var/usb_filelist2.txt
USB1_STATUS=/var/usb_status1.txt
USB2_STATUS=/var/usb_status2.txt

# for debugging purpose
echo "Inside $0 $ACTION:$1:$DEVTYPE:$ID_VENDOR:$ID_MODEL" > /tmp/debug;

# handles a device remove
if [ $ACTION == "remove" ]; then
    product_id=$(head -n 1 /proc/productData/product_model_id);
    
    # USB1
    if [ $1 == "1-1.1" ]; then
        usbFile=$USB1_FILE;
        $LED_BIN usb1 off;
        rm -f $USB1_LINK;
        rm -f $USB1_FILE_LIST;
        rm -f $USB1_STATUS;
    # USB2
    elif [ $1 == "1-1.2" ]; then
        usbFile=$USB2_FILE;
        $LED_BIN usb2 off;
        rm -f $USB2_LINK;
        rm -f $USB2_FILE_LIST;
        rm -f $USB2_STATUS;
    fi
    
    # USB1
    if [ $product_id == "DSR-500N" ]; then
        usbFile=$USB1_FILE;
        $LED_BIN usb1 off;
        rm -f $USB1_LINK;
        rm -f $USB1_FILE_LIST;
        rm -f $USB1_STATUS;
    fi

    if [ ! -e $usbFile ]; then
        exit 0;
    fi

    devType=$(head -n 1 $usbFile | cut -d : -f3)
    mountPoint=$(head -n 1 $usbFile | cut -d : -f4)   
    device=$(head -n 1 $usbFile | cut -d : -f5)

    umount $mountPoint;
    rm -rf $mountPoint;
    rm -f $usbFile;
fi

# handles a device add
if [ $ACTION == "add" ]; then
    if [ -e "$DEV_DIR/$1" ]; then
        product_id=$(head -n 1 /proc/productData/product_model_id);

        if [ $product_id == "DSR-500N" ] && [ $ID_VENDOR == "ConnMgr" ]; then
            echo -2 > $USB1_STATUS;
            exit 0; # 3G is not supported in a DSR-500N
        fi

        # create a mount point
        mkdir -p /usb/$1;

        # mount the device
        if [ $ID_FS_USAGE == "filesystem" ] || [ $DEVTYPE == "partition" ]; then
            mount $DEV_DIR/$1 /usb/$1;
            mountPoint="/usb/$1";
            devType="storage";
            device="$DEV_DIR/$1";
        fi

        # specific to DWM 3G cards
        if [ $ID_VENDOR == "ConnMgr" ]; then
            touch /usb/$1/wcdma.cfg;
            devType="3G";
        fi

        endPoint=$(echo $ID_PATH | cut -d : -f2);

        if [ $mountPoint != "" ]; then
            # provide important information about the device to the appications
            if [ $endPoint == "1.1" ]; then
                echo "$ID_VENDOR:$ID_MODEL:$devType:$mountPoint:$device" > $USB1_FILE;

                # turn LED ON
                $LED_BIN usb1 on;

                # create a link for user applications
                ln -sf $mountPoint $USB1_LINK;

                # create a file list for firmware upgrade                
                if [ $devType == "storage" ]; then 
                    echo 1 > $USB1_STATUS;
                    ls $USB1_LINK/ > $USB1_FILE_LIST;
                else
                    echo -2 > $USB1_STATUS;
                fi
            elif [ $endPoint == "1.2" ]; then
                echo "$ID_VENDOR:$ID_MODEL:$devType:$mountPoint:$device" > $USB2_FILE;

                # turn LED ON
                $LED_BIN usb2 on;
                ln -sf $mountPoint $USB2_LINK;

                # create a file list for firmware upgrade                
                if [ $devType == "storage" ]; then 
                    echo 1 > $USB2_STATUS;
                    ls $USB2_LINK/ > $USB2_FILE_LIST;
                else
                    echo -2 > $USB2_STATUS;
                fi
            elif [ $endPoint == "1" ]; then
                echo "$ID_VENDOR:$ID_MODEL:$devType:$mountPoint:$device" > $USB1_FILE;

                # turn LED ON
                $LED_BIN usb1 on;

                # create a link for user applications
                ln -sf $mountPoint $USB1_LINK;

                # create a file list for firmware upgrade                
                if [ $devType == "storage" ]; then 
                    echo 1 > $USB1_STATUS;
                    ls $USB1_LINK/ > $USB1_FILE_LIST;
                else
                    echo -2 > $USB1_STATUS;
                fi
            fi
        fi
    fi
fi
