BEGIN;
create table dbUpdateRegisterTbl (
    compName text NOT NULL,
    stopIfError integer NOT NULL,
    waitForMe   integer NOT NULL,
    tableName text NOT NULL,
    rowIndex  integer,
    onUpdate integer NOT NULL,
    onAdd   integer NOT NULL,
    onDelete integer NOT NULL
); 

-- table used by evtDsptch to make sure all writes are done before
-- notifying backend components
-- a write to this table causes sqlite to complete the earlier write

create table sqliteLock
(
lock integer
);

create table ifDevEventTbl   
(
   ifName text NOT NULL,
   compName text NOT NULL,
   userArg text NOT NULL,
   ifAdded integer NOT NULL,
   ifDeleted integer NOT NULL,
   flagsChange integer NOT NULL,
   macAddrChange integer NOT NULL,
   ipChange integer NOT NULL,
   ip6Change integer NOT NULL,
   mask     integer
);

CREATE TABLE tableValidation
(
	tableName  text NOT NULL,
	function  text NOT NULL,
	 PRIMARY KEY (tableName)
)
;

CREATE TABLE tableDefaults
(
	tableName  text NOT NULL,
	columnName  text NOT NULL,
	defValue  text ,
	minValue  text ,
	maxValue  text ,
	PRIMARY KEY (tableName,columnName)
)
;

CREATE TABLE system
(
	name  text NOT NULL,
	firmwareVer  text,
	firmwarePri  text,
	firmwareSec  text,
    firmwareTime text,
    firmCSum     text,
    vendor text NOT NULL,
    sysContact text,
    sysLoc  text,
	PRIMARY KEY (name)
)
;
create TABLE systemConfig
(
    configMajorVer  integer,
    configMinorVer  integer,
    checksum        integer
);

create table dbUpdateRegisterProgram
(
    progName text NOT NULL,
    stopIfError integer NOT NULL,
    waitForMe   integer NOT NULL,
    tableName text NOT NULL,
    rowIndex  integer,
    onUpdate integer NOT NULL,
    onAdd   integer NOT NULL,
    onDelete integer NOT NULL
);

CREATE TABLE saveTables
(
	tableName  text NOT NULL,
	PRIMARY KEY (tableName)
)
;
CREATE TABLE saveTables2
(
        TableName  text NOT NULL,
        PRIMARY KEY (tableName)
);


CREATE TABLE environment
(
	name  text NOT NULL,
	value  text,
	PRIMARY KEY (name)
)
;

CREATE TABLE tableMaxRecords
    (
    tableName text PRIMARY KEY,
    maxRecords integer NOT NULL
    );

CREATE TABLE stringsMap
(
    stringId  text NOT NULL,
    en_US     text NOT NULL,
    PRIMARY KEY (stringId)
); 
CREATE TABLE ethernet
(
	interfaceName  text NOT NULL,
	poe  integer NOT NULL,
	duplex  text NOT NULL,
	vlanId  integer NOT NULL,
	vlanEnabled integer NOT NULL,
	trunkPort integer NOT NULL,
	macAddress  text,
    edgePortStatus  bool,
    ethernetEdgePortCost  integer,
	 PRIMARY KEY (interfaceName),
	FOREIGN KEY (vlanId) REFERENCES vlan(vlanId),
    FOREIGN KEY (interfaceName) REFERENCES networkInterface(interfaceName)
)
;

insert into tableDefaults values ("ethernet","vlanEnabled","0","","");
insert into tableDefaults values ("ethernet","trunkPort","0","","");

insert into stringsMap values ("ETHERNET_PORT_CONFIG_FAILED","Ethernet port configuration failed");
insert into stringsMap values ("ETHERNET_STAT_CONFIG_FAILED","Ethernet statistics refresh configuration failed");
-- Tables used in TeamF1's Network Interface Management (NIM) Framework 1.0

-- NimfConf - this table stores the nimf related helper parameters.

-- Note: This table holds parameters which are necessary for NIM Framework and
-- are generic enough to handle all the connection types. The connection type 
-- specific configuration will go into their respective connType.sql files.

create table NimfConf
    (
    OID                      integer,           -- OID
    LogicalIfName            text     NOT NULL, -- interface names like 'lan1', 'wan2', etc.
    AddressFamily            integer  NOT NULL, -- address family 
    RequireLogin             boolean  NOT NULL, -- specifies, if ISP requires login
    ConnectionType           text             , -- connection type like dhcp,pppoe
    ConnectionType6          text             , -- connection type like dhcp,pppoe for IPv6
    IfStatus                 integer  NOT NULL, -- nimf status like enable(1)/disable(0)
    UseDefaultMtu            boolean  NOT NULL, -- if we want to use default MTU
    MtuSize                  integer  NOT NULL, -- MTU size
    PortSpeed                integer  NOT NULL, -- port speed of a wan interface
    SetMacAddress            integer  NOT NULL, -- mac address options
    MacAddress               text,              -- mac address
    PRIMARY KEY (LogicalIfName, AddressFamily), -- primary key
    FOREIGN KEY (LogicalIfName) REFERENCES networkInterface (LogicalIfName)
    );

insert into dbUpdateRegisterTbl values ("UMI_COMP_NIMF", 0, 0, "NimfConf", 0, 1, 1, 1);
insert into stringsMap values ("IPV6WAN_CONFIG_FAILED", "IPV6 WAN Configuration Failed");

-- WanMode - table for storing WAN mode configuration.
create table WanMode
    (
    OID                      integer,           -- OID
    Wanmode                  integer  NOT NULL, -- wan mode (dedicated, autorollover or load balancing)
    AddressFamily            integer  NOT NULL, -- address family
    DedicatedLogicalIfName   text     NOT NULL, -- nimf id of dedicated WAN
    FailoverPriLogicalIfName text     NOT NULL, -- from where to fail over
    FailoverSecLogicalIfName text     NOT NULL, -- where to fail over
    FailureDetectionMeth     integer  NOT NULL, -- detection method
    RetryTime                integer  NOT NULL, -- time to retry
    RetryAttempts            integer  NOT NULL, -- number of attempts to retry
    -- add newer entries here

    FOREIGN KEY (DedicatedLogicalIfName) REFERENCES networkInterface (LogicalIfName),
    FOREIGN KEY (FailoverPriLogicalIfName) REFERENCES networkInterface (LogicalIfName),
    FOREIGN KEY (FailoverSecLogicalIfName) REFERENCES networkInterface (LogicalIfName)
    );

insert into dbUpdateRegisterTbl values ("UMI_COMP_NIMF", 0, 0, "WanMode", 0, 1, 0, 1);

-- WanFailure - table for storing IPs for failure detection.
create table WanFailure
    (
    OID                      integer,           -- OID
    LogicalIfName            text     NOT NULL, -- interface names like 'lan', 'wan2', etc.
    AddressFamily            integer  NOT NULL, -- address family
    PingIp                   text,              -- IP which is a ping destination
    DnsIp                    text,              -- IP which is used for DNS lookup
    -- add newer entries here

    PRIMARY KEY (LogicalIfName, AddressFamily), -- primary key
    FOREIGN KEY (LogicalIfName) REFERENCES networkInterface (LogicalIfName)
    );

insert into dbUpdateRegisterTbl values ("UMI_COMP_NIMF", 0, 0, "WanFailure", 0, 1, 0, 1);

-- NimfStatus - table for storing different status's of NIMF.
create table NimfStatus
    (
    OID                      integer,           -- OID
    LogicalIfName            text     NOT NULL, -- interface names like 'lan', 'wan2', etc.
    AddressFamily            integer  NOT NULL, -- address family
    Nimfstatus               text     NOT NULL, -- wan status like connected, disconnected, connecting etc. 
    -- add newer entries here
    WanUpTime                text     NOT NULL,
    Linkstatus               text     NOT NULL, --Link status like LINK UP or LINK DOWN.  

    PRIMARY KEY (LogicalIfName, AddressFamily), -- primary key
    FOREIGN KEY (LogicalIfName) REFERENCES networkInterface (LogicalIfName)
    );

-- NimfTrigger - table for triggering NIMF
create table NimfTrigger
    (
    OID                      integer,           -- OID
    LogicalIfName            text     NOT NULL, -- logical interface name 
    AddressFamily            integer  NOT NULL, -- address family    
    Status                   boolean,           -- trigger field
    PRIMARY KEY (LogicalIfName, AddressFamily), -- primary key
    FOREIGN KEY (LogicalIfName) REFERENCES networkInterface (LogicalIfName)
    );
-- MacTable - table for MAC refresh
create table MacTable
    (
    LogicalIfName            text     NOT NULL, -- logical interface name 
    Flag                     integer  NOT NULL, 
    PRIMARY KEY (LogicalIfName),                -- primary key
    FOREIGN KEY (LogicalIfName) REFERENCES networkInterface (LogicalIfName)
    );

-- NimfMtu - table for storing solution specific interface MTU values.
-- This MTU will be set on interface by NIMF. For PPP connection types we will
-- negotiate MTU in the backend connection type components.
create table NimfMtu
    (
    LogicalIfName            text     NOT NULL, -- logical interface name
    DefMtu                   integer  NOT NULL, -- default Mtu 
    MinMtu                   integer  NOT NULL, -- Minimum Mtu 
    MaxMtu                   integer  NOT NULL, -- Maximum Mtu
    PRIMARY KEY (LogicalIfName) -- primary key
    );

create table NimfMac
    (
    LogicalIfName            text     NOT NULL, -- logical interface name
    macAddress               text,              -- mac address
    PRIMARY KEY (LogicalIfName) -- primary key
    );

create table NimfLBStatus
     (
     OID                     integer,            -- OID
     wanStatus               integer    NOT NULL -- wanStatus 
     );

insert into dbUpdateRegisterTbl values ("UMI_COMP_NIMF", 0, 0, "NimfLBStatus", 0, 1, 1, 1);

create table NimfAFStatus
     (
     OID                     integer,            -- OID
     wanStatus               integer    NOT NULL -- wanStatus 
     );

insert into dbUpdateRegisterTbl values ("UMI_COMP_NIMF", 0, 0, "NimfAFStatus", 0, 1, 1, 1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"NimfAFStatus",0,1,1,1);


-- strings mappings
insert into stringsMap values ("NIMF_CONFIG_NOTFOUND","Internet Configuration Not Found");
insert into stringsMap values ("NIMF_CONFIG_FAILED","Internet Configuration Failed");
insert into stringsMap values ("IDLE_TIMEOUT_ERROR","Please Remove the idle timeout option for Dedicated/Secondary WAN");
insert into stringsMap values ("LOAD_BALANCE_ERROR","PPPOE profile selected has idle time configured whereas WAN is set in Load Balancing mode");
insert into ifDevEventTbl values ("ppp1", "UMI_COMP_NIMF", "NULL", 1, 1, 0, 0, 0, 0, 1);
insert into ifDevEventTbl values ("ppp2", "UMI_COMP_NIMF", "NULL", 1, 1, 0, 0, 0, 0, 1);
insert into ifDevEventTbl values ("ppp101", "UMI_COMP_NIMF", "NULL", 1, 1, 0, 0, 0, 0, 1);
insert into ifDevEventTbl values ("ppp201", "UMI_COMP_NIMF", "NULL", 1, 1, 0, 0, 0, 0, 1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_NIMF", 0, 0, "NimfTrigger", 0, 1, 1, 1);
insert into stringsMap values ("MULTICAST_MAC_ERROR","MAC Address configured is a Multicast MAC address");

CREATE TABLE networkInterface
    (
    interfaceName   text    NOT NULL,
    LogicalIfName   text    NOT NULL,
    VirtualIfName   text    NOT NULL,
    ConnectionType  text    NOT NULL,
    ConnectionType6 text    NOT NULL,
    ifType          text,
    MTU             integer,
    ipaddr          text,
    subnetmask      text,
    gateway         text,
    dns1            text,
    dns2            text,
    DHCP            integer,
    IfStatus        integer NOT NULL,
    refreshInterval integer NOT NULL,
    autoRefreshFlag integer NOT NULL,
    capabilities    integer NOT NULL,
    restartFwForMe  integer DEFAULT 0, --1 restart the firewall,  0-don't restart    
    ifGroupId       integer,--1 secure, 2 insecure, 3 dmz, 4 IPV6, 5 WIRELESS etc    
    PRIMARY KEY (interfaceName,LogicalIfName)
    );

CREATE TABLE Lan
    (
    LogicalIfName   text    NOT NULL,
    Mtu             integer NOT NULL,
    IpAddress       text    NOT NULL,
    SubnetMask      text    NOT NULL,
    IfStatus        boolean NOT NULL,
    PRIMARY KEY (LogicalIfName),
    FOREIGN KEY (LogicalIfName) REFERENCES networkInterface(LogicalIfName)
    );

CREATE TABLE interfaceStats
(
    interfaceName  text NOT NULL,
    rx_packets  text NOT NULL,
    rx_packets_prev  text NOT NULL,
    tx_packets  text NOT NULL,
    tx_packets_prev  text NOT NULL,
    rx_bytes  text NOT NULL,
    rx_bytes_prev  text NOT NULL,
    tx_bytes  text NOT NULL,
    tx_bytes_prev  text NOT NULL,
    rx_errors  text NOT NULL,
    rx_errors_prev  text NOT NULL,
    tx_errors  text NOT NULL,
    tx_errors_prev  text NOT NULL,
    rx_dropped  text NOT NULL,
    rx_dropped_prev  text NOT NULL,
    tx_dropped  text NOT NULL,
    tx_dropped_prev  text NOT NULL,
    multicast  text NOT NULL,
    multicast_prev  text NOT NULL,
    collisions  text NOT NULL,
    collisions_prev  text NOT NULL,
    PRIMARY KEY (interfaceName),
    FOREIGN KEY (interfaceName) REFERENCES networkInterface(interfaceName)
)
;

CREATE TABLE interfaceStatsParams
(
    ifType  text NOT NULL,
    refreshInterval integer NOT NULL,
    autoRefreshFlag integer NOT NULL
)
;

CREATE TABLE interfaceUsage
(
	interfaceName    text NOT NULL,
    entryNum         integer NOT NULL,
    entryTime        DATE NOT NULL,
	rx_packets       text NOT NULL,
    rx_packet_rate   text NOT NULL,
	tx_packets       text NOT NULL,
    tx_packet_rate   text NOT NULL,
	rx_bytes         text NOT NULL,
    rx_bytes_rate    text NOT NULL,
	tx_bytes         text NOT NULL,
    tx_bytes_rate    text NOT NULL,
	rx_errors        text NOT NULL,
    rx_error_rate    text NOT NULL,
	tx_errors        text NOT NULL,
    tx_error_rate    text NOT NULL,
	rx_dropped       text NOT NULL,
    rx_dropped_rate  text NOT NULL,
	tx_dropped       text NOT NULL,
    tx_dropped_rate  text NOT NULL,
	multicast        text NOT NULL,
    multicast_rate   text NOT NULL,
	collisions       text NOT NULL,
    collisions_rate  text NOT NULL,
    PRIMARY KEY (interfaceName, entryNum),
    FOREIGN KEY (interfaceName) REFERENCES networkInterface(interfaceName)
)
;

create table ipAddressTable
(
    LogicalIfName text NOT NULL,        -- hardware interface name
    addressFamily integer,              -- Address family
    isStatic boolean,                   -- static assigned or dynamic
    ipAddress text,                     -- IP address
    ipDstAddres text,                   -- End point for P-to-P
    ipv6PrefixLen integer,              -- IPv6 prefix length
    subnetMask text,                    -- subnet mask
    PRIMARY KEY(LogicalIfName,addressFamily,ipAddress), -- this modification will avoid same multiple ipaddress
                                                        -- for particular logicalifname and addressfamily.
    FOREIGN KEY (LogicalIfName) REFERENCES networkInterface(LogicalIfName)
);

create table resolverConfig
(
    LogicalIfName    text    NOT NULL,  -- WAN1, WAN2, LAN
    addressFamily    integer NOT NULL,  -- Address family
    nameserver1      text            ,  -- DNS nameserver 1 
    nameserver2      text            ,  -- DNS nameserver 2
    nameserver3      text            ,  -- DNS nameserver 1 
    nameserver4      text            ,  -- DNS nameserver 2 
    PRIMARY KEY (LogicalIfName, addressFamily),
    FOREIGN KEY (LogicalIfName) REFERENCES networkInterface(LogicalIfName)
);

create table  defaultRouters
(
  LogicalIfName    text    NOT NULL,    -- WAN1, WAN2, LAN  
  addressFamily    integer NOT NULL,    -- Address family   
  nextHopGateway   text            ,    -- Next hop gateway
  interfaceName    text            ,    -- for interface route
  PRIMARY KEY (LogicalIfName, addressFamily),
  FOREIGN KEY (LogicalIfName) REFERENCES networkInterface(LogicalIfName)
);

CREATE TABLE ifUsageReportConfig
    (
    maxSnaps integer NOT NULL,
    currSnap integer NOT NULL
    );

CREATE TABLE networkInfo
(
	netWorkMode integer NOT NULL -- 1 = LAN IPv4, WAN IPv4
                                 -- 2 = LAN Dual, WAN IPv4   
                                 -- 3 = LAN Dual, WAN Dual
);

CREATE TABLE ipv6Interfaces
(
    interfaceName  text NOT NULL   -- ipv6 enabled interface name 
);

insert into tableDefaults values ("ifUsageReportConfig","maxSnaps","24","1","99");
    
insert into stringsMap values ("IFDEV_CONFIG_FAILED","Interface configuration failed");
insert into stringsMap values ("IFDEV_USAGE_CONFIG_FAILED","Usage reports configuration failed");
insert into stringsMap values ("IFDEV_RESET_STATS_FAILED","Reset of interface statistics failed");
insert into stringsMap values ("IP_MODE_SET_FAILED","IP Mode Configuration Failed");
insert into stringsMap values ("AUTOREFRESH_CONFIG_FAILED","Auto refresh Configuration Failed");
insert into stringsMap values ("CONFIG_CHECKSUM_FAILURE","Failed to update configuration (checksum error)");
CREATE TABLE cert
(
    name text,                 -- user identifier
	subjectName  text NOT NULL,
	issuerName   text NOT NULL,
	expiryTime   text NOT NULL,	
	serialNumber text NOT NULL,-- The cert serial number	
	format	     text,         -- the format of the certificate data; 
                               -- for future use	
	certType     text NOT NULL,-- cert type: "ca" = root/CA certificate, 
                               --            "self"=self certificate
	certData     text NOT NULL,-- Certificate data
    PRIMARY KEY (subjectName)
)
;

insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"cert", 
                                         0, 1,1,1);
CREATE TABLE x509SelfCertReq
(
    requestName text NOT NULL,
    subjectName text NOT NULL,
    hashAlgo integer,
    signatureAlgo integer,
    signatureKeyLength integer,
    ipAddress text,
    domainName text,
    emailAddress text,
    selfCertData text,
    selfKeyData text,
    appType text,
    requestStatus integer,
    PRIMARY KEY (requestName)
);

CREATE TABLE x509DbUpdateFlag
(
    dbUpdateFlag integer
);

insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1, 
                                        "x509SelfCertReq", 0, 1,1,1);

insert into stringsMap values ("X509_INVALID_CA","Can't Upload Invalid Trusted Certificate"); 
insert into stringsMap values ("X509_CA_ADD_SUCCESS","Added Trusted Certificate"); 
insert into stringsMap values ("X509_CA_ADD_FAILURE","Cannot Add Trusted Certificate"); 
insert into stringsMap values ("X509_CA_DEL_SUCCESS","Selected Trusted CAs Deleted"); 
insert into stringsMap values ("X509_CA_DEL_FAILURE","Cannot Delete Trusted CAs"); 
insert into stringsMap values ("X509_SELF_GEN_FAILURE","Generate Self Certificate Request Failed");
insert into stringsMap values ("X509_SELF_DEL_SUCCESS","Selected Self Certificates Deleted");
insert into stringsMap values ("X509_SELF_DEL_FAILURE","Cannot Delete Self Certificates");
insert into stringsMap values ("X509_SELF_ADD_SUCCESS","Added Active Self Certificate");
insert into stringsMap values ("X509_SELF_ADD_FAILURE","Cannot Add Active Self Certificate");
insert into stringsMap values ("X509_INVALID_SELF","Can't Upload Invalid Self Certificate"); 
insert into stringsMap values ("X509_INVALID_SELF_PURPOSE","Invalid purpose, Can't Upload Self Certificate"); 
insert into stringsMap values ("X509_INVALID_SELF_CA","No trusted certificate found, Can't Upload Self Certificate"); 

CREATE table smtpServer
(
    entryName    text,
    emailServer  text,
    smtpPort     text,
    fromAddr     text,
    toAddr       text,
    toAddr2       text,
    toAddr3       text,
    auth         integer,
    userName     text,
    passWord     text,
    respondIdentd   integer,
    PRIMARY KEY (entryName)
);

insert into tableDefaults values ("smtpServer","smtpPort","25","1","65536");

CREATE TABLE ntp
(
   timezone  integer NOT NULL,
   enabled  integer NOT NULL,
   autoDaylight  integer NOT NULL,
   useDefServers  integer NOT NULL,
   server1  text NOT NULL,
   server2  text NOT NULL,
   reSyncNtpVal integer NOT NULL,
   networkMode	integer	NOT NULL,
   sharedSecretStatus boolean,
   authKeyID  integer,
   authKeyValue text		
)
;

CREATE TABLE ntpDefServers
    (
    serverName text NOT NULL,
    PRIMARY KEY (serverName)
    )
;

CREATE TABLE manualTime
(
   ManualTimeStatus boolean NOT NULL, 
   Timezone  integer NOT NULL,
   AutoDaylight  integer NOT NULL
)
;

insert into tableDefaults values ("manualTime", "ManualTimeStatus", "0", "0", "1");
insert into tableDefaults values ("ntp", "sharedSecretStatus", "0", "0", "1");

insert into dbUpdateRegisterTbl values ("UMI_COMP_NTP",1,1,"ntp", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_NTP",1,1,"ntpDefServers", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_NTP",1,0,"manualTime", 0, 0,1,0);

insert into stringsMap values ("TIME_STATUS_NTP_FAILED","NTP configuration failed");
insert into stringsMap values ("TIME_STATUS_TIMEDATE_FAILED","Time/Date configuration Failed");
insert into stringsMap values ("MANUAL_TIME_CONFIG_FAILED","Manual Time/Date configuration Failed");
CREATE TABLE crontab
(
    unit        integer NOT NULL,
    interval    integer,
    minute      integer,
    hour        integer,
    dayOfMonth  integer,
    month       integer,
    dayOfWeek   integer,
    command     text
);
insert into tableDefaults values ("crontab","unit","0","","");
insert into tableDefaults values ("crontab","interval","0","","");
insert into tableDefaults values ("crontab","minute","0","0","59");
insert into tableDefaults values ("crontab","hour","-1","","");
insert into tableDefaults values ("crontab","dayOfMonth","-1","","");
insert into tableDefaults values ("crontab","month","-1","","");
insert into tableDefaults values ("crontab","dayOfWeek","-1","","");
insert into dbUpdateRegisterTbl values ("UMI_COMP_CRON",1,1,"crontab", 0,1,1,1);

insert into stringsMap values ("CRONTAB_CONFIG_FAILED","Crontab configuration failed");



CREATE TABLE oidRecord
    (
    OID             integer,    -- next instance id to be used
    tableName       text        -- table name
    
    );

insert into oidRecord values (1, "route");




insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"bridgePorts", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"bridgeTable", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",0,1,"networkInterface", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"users", 0, 1,1,1);

insert into stringsMap values ("NONE","");
insert into stringsMap values ("STATUS_OK","Operation succeeded");
insert into stringsMap values ("ADMIN_REQD","Administrator privileges are required");
insert into stringsMap values ("PASSWORD_CHANGED","Password Changed. Please login again");
insert into stringsMap values ("ERR_EMPTY_VALUE_ENTERED","Field cannot be left blank");
insert into stringsMap values ("ERR_STRING_LENGTH","Length of the string entered is incorrect");
insert into stringsMap values ("ERR_START_IP_GREATER","Start IP cannot be greater than end IP");
insert into stringsMap values ("ERR_PASSWORD_EMPTY","Password field cannot be left empty");
insert into stringsMap values ("ERR_PASSWORD_MISMATCH","Passwords entered do not match");
insert into stringsMap values ("INVALID_VALUE_ERR_ALPHABET","Only Numbers allowed for the field");
insert into stringsMap values ("INVALID_VALUE_ERR_ALPHA","Only Numbers allowed for the field");
insert into stringsMap values ("INVALID_VALUE_ERR_SPCLCHAR","Special Characters not allowed for the field");
insert into stringsMap values ("ERR_START_GREATER_THAN_END","Start value cannot be greater than end value");
insert into stringsMap values ("ERR_BOUND_INVALID_RANGE","Value entered does not fall in allowed range");
insert into stringsMap values ("INVALID_SUBNET_EMPTY_OCTET_1","Octet 1 of subnet field cannot be left blank");
insert into stringsMap values ("INVALID_SUBNET_EMPTY_OCTET_2","Octet 2 of subnet field cannot be left blank");
insert into stringsMap values ("INVALID_SUBNET_EMPTY_OCTET_3","Octet 3 of subnet field cannot be left blank");
insert into stringsMap values ("INVALID_SUBNET_EMPTY_OCTET_4","Octet 4 of subnet field cannot be left blank");
insert into stringsMap values ("ERR_INVALIDSUBNET_OCTET_1","Invalid value entered for subnet octet 1");
insert into stringsMap values ("ERR_INVALIDSUBNET_OCTET_2","Invalid value entered for subnet octet 2");
insert into stringsMap values ("ERR_INVALIDSUBNET_OCTET_3","Invalid value entered for subnet octet 3");
insert into stringsMap values ("ERR_INVALIDSUBNET_OCTET_4","Invalid value entered for subnet octet 4");
insert into stringsMap values ("INVALID_SUBNET_ERR1_OCTET_1","Subnet Octet1 should be 255");
insert into stringsMap values ("INVALID_SUBNET_ERR2_OCTET_4","Subnet Octet4 should not be 254");
insert into stringsMap values ("INVALID_SUBNET_ERR3_OCTET_4","Subnet Octet4 should not be 255");
insert into stringsMap values ("INVALID_SUBNET_ERR4_OCTET_3","Subnet Octet 3 should be 0");
insert into stringsMap values ("INVALID_SUBNET_ERR5_OCTET_4","Subnet Octet 4 should be 0");
insert into stringsMap values ("INVALID_SUBNET_ERR6_OCTET_2","Subnet Octet 2 should be 255 for class B");
insert into stringsMap values ("INVALID_SUBNET_ERR7_OCTET_2","Subnet Octet 2 should be 255 for class C");
insert into stringsMap values ("INVALID_SUBNET_ERR8_OCTET_3","Subnet Octet 3 should be 255 for class C");
insert into stringsMap values ("INVALID_IPCLASS_ERR9","IP Class not valid");
insert into stringsMap values ("INVALID_IP_CHECK_NUMBER_OCTETS","Number of octets in a IPv4 address should be 4");
insert into stringsMap values ("INVALID_IP_ERR_ALPHABET","Alphabets are not allowed in IPv4 address");
insert into stringsMap values ("INVALID_IP_ERR_SPCLCHAR","Special Characters are not allowed in IPv4 address"); 
insert into stringsMap values ("INVALID_IP_EMPTY_OCTET_1","Octet 1 of IP address cannot be left blank");
insert into stringsMap values ("INVALID_IP_EMPTY_OCTET_2","Octet 2 of IP address cannot be left blank");
insert into stringsMap values ("INVALID_IP_EMPTY_OCTET_3","Octet 3 of IP address cannot be left blank");
insert into stringsMap values ("INVALID_IP_EMPTY_OCTET_4","Octet 4 of IP address cannot be left blank");
insert into stringsMap values ("INVALID_IP_OCTET_1","Enter a value between 1 to 223 for octet 1");
insert into stringsMap values ("INVALID_IP_OCTET_2","Enter a value between 1 to 255 for octet 2");
insert into stringsMap values ("INVALID_IP_OCTET_3","Enter a value between 1 to 255 for octet 3");
insert into stringsMap values ("INVALID_IP_OCTET_4","Enter a value between 1 to 254 for octet 4");
insert into stringsMap values ("INVALID_SUBNET_ERR_ALPHABET","Alphabets are not allowed in IP subnet maks");
insert into stringsMap values ("INVALID_SUBNET_ERR_SPCLCHAR","Special Characters are not allowed in IP subnet maks");
insert into stringsMap values ("ERROR_MAC_NUMBER_OF_OCTETS","Invalid MAC Address");
insert into stringsMap values ("INVALID_MAC_ERR_SPCLCHAR","Special Characters are not allowed in MAC field");
insert into stringsMap values ("ERR_MAC_EMPTY_VALUE_OCTET_1","Octet 1 of MAC field cannot be left empty");
insert into stringsMap values ("ERR_MAC_EMPTY_VALUE_OCTET_2","Octet 2 of MAC field cannot be left empty");
insert into stringsMap values ("ERR_MAC_EMPTY_VALUE_OCTET_3","Octet 3 of MAC field cannot be left empty");
insert into stringsMap values ("ERR_MAC_EMPTY_VALUE_OCTET_4","Octet 4 of MAC field cannot be left empty");
insert into stringsMap values ("ERR_MAC_EMPTY_VALUE_OCTET_5","Octet 5 of MAC field cannot be left empty");
insert into stringsMap values ("ERR_MAC_EMPTY_VALUE_OCTET_6","Octet 6 of MAC field cannot be left empty");
insert into stringsMap values ("ERR_TWOCHAR_OCTET_1","Only 2 characters allowed in octet 1");
insert into stringsMap values ("ERR_TWOCHAR_OCTET_2","Only 2 characters allowed in octet 2");
insert into stringsMap values ("ERR_TWOCHAR_OCTET_3","Only 2 characters allowed in octet 3");
insert into stringsMap values ("ERR_TWOCHAR_OCTET_4","Only 2 characters allowed in octet 4");
insert into stringsMap values ("ERR_TWOCHAR_OCTET_5","Only 2 characters allowed in octet 5");
insert into stringsMap values ("ERR_TWOCHAR_OCTET_6","Only 2 characters allowed in octet 6");
insert into stringsMap values ("ERR_ONLYHEX_OCTET_1","Only a-f and 0-9 allowed in octet 1");
insert into stringsMap values ("ERR_ONLYHEX_OCTET_2","Only a-f and 0-9 allowed in octet 2");
insert into stringsMap values ("ERR_ONLYHEX_OCTET_3","Only a-f and 0-9 allowed in octet 3");
insert into stringsMap values ("ERR_ONLYHEX_OCTET_4","Only a-f and 0-9 allowed in octet 4");
insert into stringsMap values ("ERR_ONLYHEX_OCTET_5","Only a-f and 0-9 allowed in octet 5");
insert into stringsMap values ("ERR_ONLYHEX_OCTET_6","Only a-f and 0-9 allowed in octet 6");
insert into stringsMap values ("ERR_LEASETIME_EMPTY_VALUE","Lease Time field cannot be left empty");
insert into stringsMap values ("INVALID_LEASETIME_ERR_SPCLCHAR","Special chracter not allowed in lease time field");
insert into stringsMap values ("INVALID_LEASETIME_ERR_ALPHA","Alphabet not allowed in Lease time field");
insert into stringsMap values ("ERR_LEASETIME_INVALIDRANGEVALUE","Lease time entered is outside allowed range");
insert into stringsMap values ("ERR_MTU_EMPTY_VALUE","MTU field cannot be left empty");
insert into stringsMap values ("INVALID_MTUSIZE_ERR_ALPHA","Alphabet not allowed in MTU field");
insert into stringsMap values ("INVALID_MTUSIZE_ERR_SPCLCHAR","Special chracter not allowed in MTU field");
insert into stringsMap values ("ERR_MTUSIZE_INVALIDRANGEVALUE","MTU value entered falls outside allowed range");
insert into stringsMap values ("ERR_PORT_EMPTY_VALUE","Port field cannot be left empty");
insert into stringsMap values ("INVALID_PORT1_ERR_ALPHA","Alphabet not allowed in port field");
insert into stringsMap values ("INVALID_PORT1_ERR_SPCLCHAR","Special character not allowed in port1 field");
insert into stringsMap values ("INVALID_PORT2_ERR_ALPHA","Alphabet not allowed in port2 field");
insert into stringsMap values ("INVALID_PORT2_ERR_SPCLCHAR","Special character not allowed in port2 field");
insert into stringsMap values ("ERR_PORT1_GREATERTHAN_PORT2","Port1 entered is greater than Port2");
insert into stringsMap values ("ERR_PORT1_INVALID","Invalid port number");
insert into stringsMap values ("ERR_IDLETIME_EMPTY_VALUE","Idle time field cannot be left empty");
insert into stringsMap values ("INVALID_IDLETIME_ERR_ALPHA","Alphabet not allowed in idle time field");
insert into stringsMap values ("INVALID_IDLETIME_ERR_SPCLCHAR","Special character not allowed in idle time field");
insert into stringsMap values ("ERR_IDLETIME_INVALIDRANGEVALUE","Idle time entered is outside allowed limits");
insert into stringsMap values ("ERR_ADV_INV_EMPTY_VALUE","Empty field not allowed");
insert into stringsMap values ("INVALID_ADV_INV_ERR_ALPHA","Alphabet not allowed");
insert into stringsMap values ("INVALID_ADVINTERVAL_ERR_SPCLCHAR","Special character not allowed");
insert into stringsMap values ("ERR_ADV_INV_INVALIDRANGEVALUE","Advertisement interval entered falls ouside alowed range");
insert into stringsMap values ("INVALID_RADVDMTU_ERR_ALPHA","Alphabet not allowed");
insert into stringsMap values ("INVALID_RADVDMTU_ERR_SPCLCHAR","Special character not allowed");
insert into stringsMap values ("ERR_RADVDMTU_INVALIDRANGEVALUE","MTU value entered falls outside allowed range"); 
insert into stringsMap values ("INVALID_ROUTERLIFETIME_ERR_ALPHA","Alphabet not allowed in lifetime field");
insert into stringsMap values ("INVALID_ROUTERLIFETIME_ERR_SPCLCHAR","Special character not allowed in life time field");
insert into stringsMap values ("ERR_ROUTERLIFETIME_INVALIDRANGEVALUE","Life time entered falls ouside alowed range");
insert into stringsMap values ("INVALID_BANDWIDTH_ERR_ALPHA","Alphabet not allowed"); 
insert into stringsMap values ("INVALID_BANDWIDTH_ERR_SPCLCHAR","Special character not allowed"); 
insert into stringsMap values ("ERR_BANDWIDTH_INVALIDRANGEVALUE","Value entered falls outside allowed range");
insert into stringsMap values ("INVALID_IDLETIMEOUT_ERR_ALPHA","Alphabet not allowed in idle time out field");
insert into stringsMap values ("INVALID_IDLETIMEOUT_ERR_SPCLCHAR","Special character not allowed in idle timeout field");
insert into stringsMap values ("ERR_IDLETIMEOUT_INVALIDRANGEVALUE","Value entered falls outside allowed range"); 
insert into stringsMap values ("ERR_PRESHAREKEY_EMPTY_VALUE","Pre-Shared field cannot be left empty"); 
insert into stringsMap values ("ERR_PRESHAREKEY_INVALID_KEY_LEN","Please enter key with length 8-49 characters");
insert into stringsMap values ("INVALID_MAXQUERRYTIME_ERR_ALPHA","Alphabets not allowed in querry time field"); 
insert into stringsMap values ("INVALID_MAXQUERRYTIME_ERR_SPCLCHAR","Special Character not allowed in querry time field");
insert into stringsMap values ("ERR_MAXQUERRYTIME_INVALIDRANGEVALUE","Query time entered falls outside range");
insert into stringsMap values ("INVALID_ROBVARIABLE_ERR_ALPHA","Alphabets not allowed in robustness field"); 
insert into stringsMap values ("INVALID_ROBVARIABLE_ERR_SPCLCHAR","Special character not allowed in robustness field");
insert into stringsMap values ("ERR_ROBVARIABLE_INVALIDRANGEVALUE","Enter a value between 2 to 99 for robustness field");
insert into stringsMap values ("INVALID_QUERRYINV_ERR_ALPHA","Alphabets not allowed in querry interval field");
insert into stringsMap values ("INVALID_QUERRYINV_ERR_SPCLCHAR","Special character not allowed in querry interval field");
insert into stringsMap values ("ERR_QUERRYINV_INVALIDRANGEVALUE","enter a value between 100-99999 for query interval");
insert into stringsMap values ("INVALID_DETPERIOD_ERR_ALPHA","Alphabet not allowed in detection period field"); 
insert into stringsMap values ("INVALID_DETPERIOD_ERR_SPCLCHAR","Special character not allowed in detection period field");
insert into stringsMap values ("ERR_DETPERIOD_INVALIDRANGEVALUE","enter value between 10-999 for detection period");
insert into stringsMap values ("INVALID_RECONNECT_ERR_ALPHA","Alphabet not allowed in reconnect field");
insert into stringsMap values ("INVALID_RECONNECT_ERR_SPCLCHAR","Special characeter not allowed in re-connect field");
insert into stringsMap values ("ERR_RECONNECT_INVALIDRANGEVALUE","Value entered falls outside allowed range"); 
insert into stringsMap values ("ERR_IPV6_EMPTY_VALUE","Empty IPv6 Address");
insert into stringsMap values ("CHECK_NUMBER_OCTETS_IPV6","Check the number of octect in ipV6 address field");
insert into stringsMap values ("CHECK_COLON_POSITION_IPV6","Check colon position in ipv6 address"); 
insert into stringsMap values ("INVALID_IPV6ADDR_CHAR","Only hex character allowed in ipv6 address");
insert into stringsMap values ("INVALID_IPV6ADDR_CHECK_COLONS","Check colon position in ipv6 address");
insert into stringsMap values ("CHECK_OCTET_IPV4_TYPE","IPV4 address string can occur only once");
insert into stringsMap values ("INVALID_IPV4ADRR_NOT_AT_END","IPV4 string can be present only at the end");
insert into stringsMap values ("INVALID_IPV6ADDR_CHECK_OCTECT_POSITION","Ckeck octet position");
insert into stringsMap values ("INVALID_IPV6ADDR_CHECK_NUMBER_OCTETS","Check the number of octets present");
insert into stringsMap values ("INVALID_IPV6ADDR_CHECK_OCTET","Check the octet positons");
insert into stringsMap values ("FIRST_GREATER_THAN_SECOND_IP_V6","ipv6 start address greater than end address");
insert into stringsMap values ("BOTH_IP_EQUAL_V6","Error, both ipv6 address are equal");
insert into stringsMap values ("INVALID_PREFIX_LENGTH_ERR_ALPHA","Alphabet not allowed in prefix length field");
insert into stringsMap values ("INVALID_PREFIX_LENGTH_ERR_SPCLCHAR","Special character not allowed in prefix length field");
insert into stringsMap values ("ERR_PREFIX_LENGTH_INVALIDRANGEVALUE","Enter value between 1-127 for prefix length");
insert into stringsMap values ("INVALID_METRIC_ERR_ALPHA","Alphabet not allowed for metric field");
insert into stringsMap values ("INVALID_METRIC_ERR_SPCLCHAR","Special character not allowed in metric field"); 
insert into stringsMap values ("ERR_METRIC_INVALIDRANGEVALUE","enter value between 2-15 for metric field");
insert into stringsMap values ("INVALID_REBIND_TIME_ERR_ALPHA","Alphabet not allowed in rebind time field");
insert into stringsMap values ("INVALID_REBIND_TIME_ERR_SPCLCHAR","Special character not allowed in rebind time field");
insert into stringsMap values ("ERR_BIND_TIME_INVALIDRANGEVALUE","enter value between 0-604800 for rebind time");
insert into stringsMap values ("INVALID_LOGIN_MASK_ERR_ALPHA","Alphabet not allowed");
insert into stringsMap values ("INVALID_LOGIN_MASK_ERR_SPCLCHAR","Special character not allowed");
insert into stringsMap values ("ERR_LOGIN_MASK_INVALIDRANGEVALUE","Value entered falls outside allowed range");
insert into stringsMap values ("INVALID_FAILOVER_ERR_ALPHA","Alphabet not allowed in failover field");
insert into stringsMap values ("INVALID_FAILOVER_ERR_SPCLCHAR","Special character not allowed in failover field"); 
insert into stringsMap values ("ERR_FAILOVER_INVALIDRANGEVALUE","enter a value between 2-999 for failover"); 
insert into stringsMap values ("INVALID_RETRY_INV_ERR_ALPHA","Alphabet not allowed in retyr interval field");
insert into stringsMap values ("INVALID_RETRY_INV_ERR_SPCLCHAR","Special character not allowed in retrt interval field");
insert into stringsMap values ("ERR_RETRY_INV_INVALIDRANGEVALUE","enter a value between 30-999 for retry interval");
insert into stringsMap values ("INVALID_TCP_RANGE_ERR_ALPHA","Alphabet not allowed");
insert into stringsMap values ("INVALID_TCP_RANGE_ERR_SPCLCHAR","Special character not allowed"); 
insert into stringsMap values ("ERR_TCP_INVALIDRANGEVALUE","Value entered falls outside allowed range");
insert into stringsMap values ("INVALID_TIME_ERR_ALPHA","Alphabet not allowed in time field");
insert into stringsMap values ("INVALID_TIME_ERR_SPCLCHAR","Special character not allowed in time field");
insert into stringsMap values ("ERR_INVALID_HOUR_SETTING","enter value between 1-12 for hour field");
insert into stringsMap values ("ERR_INVALID_HOUR24_SETTING","enter value between 0-23 for hour field");
insert into stringsMap values ("ERR_INVALID_MINUTE_SETTING","enter value between 1-59 for minute field");
insert into stringsMap values ("ERR_INVALID_MONTH_SETTING","enter value between 1-12 for month field");
insert into stringsMap values ("ERR_INVALID_DATE_SETTING","enter value between 1-31 for cate field");
insert into stringsMap values ("ERR_INVALID_SECOND_SETTING","enter value between 1-59 for second field");
insert into stringsMap values ("INVALID_MD5_KEY_ERR_ALPHA","alphabet not allowed in MD5 key field");
insert into stringsMap values ("INVALID_MD5_KEY_ERR_SPCLCHAR","special characters not allowed in md5 key field");
insert into stringsMap values ("ERR_MD5KEY_INVALIDRANGEVALUE","enter value between 1-255 for md5 key field");
insert into stringsMap values ("ERR_SALIFE_EMPTY_VALUE","SA-LifeTime field empty");
insert into stringsMap values ("INVALID_SA_TIME_ERR_ALPHA","alphabet not allowed in sa-lifetime field");
insert into stringsMap values ("INVALID_SA_TIME_ERR_SPCLCHAR","Special character not allowed in sa-liftime field");
insert into stringsMap values ("ERR_SA_LIFE_TIME","SA-Lifteime value entered is out of range");
insert into stringsMap values ("ERR__VPN_KEY_EMPTY_VALUE","vpn key field cannot be empty");
insert into stringsMap values ("VPN_INCORRECT_KEY_LENGTH","VPN key length entered is out of range");
insert into stringsMap values ("ERROR_NAME_EMPTY","Empty field value"); 
insert into stringsMap values ("INAVLID_CHARACTER_IN_NAME","Invalid characters"); 
insert into stringsMap values ("INVALID_UPNP_INV_ERR_ALPHA","Alphabet not allowed in upnp interval field");
insert into stringsMap values ("INVALID_UPNP_INV_ERR_SPCLCHAR","Special character not allowed in interval field");
insert into stringsMap values ("ERR_UPNP_ADV_INV_INVALIDRANGEVALUE","enter value between 1-86400");
insert into stringsMap values ("INVALID_AD_TIME_ERR_ALPHA","Alphabet not allowed in Adv time field");
insert into stringsMap values ("INVALID_AD_TIME_ERR_SPCLCHAR","Special character not allowed in adv time field");
insert into stringsMap values ("ERR_UPNP_ADV_TIME_INVALIDRANGEVALUE","enter a value between 1-255 for avd time field");
insert into stringsMap values ("INVALID_PROFILE_ERR_ALPHA","alphabet not allowed for field in profile page");
insert into stringsMap values ("INVALID_PROFILE_ERR_SPCLCHAR"," Special characters not allowed for field in profile page");
insert into stringsMap values ("ERR_INVALIDRANGEVALUE_WL","enter a value between 1-36000 for the field");
insert into stringsMap values ("INVALID_RADIUS_ERR_ALPHA","alphabet not allowed in radius field");
insert into stringsMap values ("INVALID_RADIUS_ERR_SPCLCHAR","special characters not allowed in radius field");
insert into stringsMap values ("ERR_INVALIDRANGEVALUE_RADIUS","enter a value between 1-999 for radius fields");
CREATE TABLE bridgePorts
(
	interfaceName  text NOT NULL,
    portEnabled     integer NOT NULL,
	bridgeInterface  text NOT NULL,
    vlanEnabled     integer NOT NULL,
    vlanId          integer,
    trunk           integer,
    PRIMARY KEY (interfaceName),
    FOREIGN KEY (interfaceName) REFERENCES networkInterface(interfaceName)
)
;



CREATE TABLE bridgeTable
(
	interfaceName  text NOT NULL,
	macAddress  text,
    bridgeIndex integer NOT NULL,
    PRIMARY KEY (interfaceName),
    FOREIGN KEY (interfaceName) REFERENCES networkInterface(interfaceName)
)
;

insert into tableDefaults values ('bridgeTable','bridgeIndex','1','','');

insert into dbUpdateRegisterTbl values ("UMI_COMP_LOGGING",1,1,"logging", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_LOGGING",1,1,"logConfig", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_LOGGING",1,1,"compFacilityMap", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_LOGGING",1,1,"sysLogInfo", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_LOGGING",1,1,"logEventEmailConfig", 0, 1,1,1);

CREATE TABLE logging
(
    maxEventLogRows  integer NOT NULL,
    saveLogs    integer NOT NULL
);

insert into logging values (1000, 0);

CREATE TABLE logConfig
(
	facility  text NOT NULL,
    facilityId integer NOT NULL,
	emergency  integer NOT NULL,
	alert  integer NOT NULL,
	critical  integer NOT NULL,
	error  integer NOT NULL,
	warning  integer NOT NULL,
	notice  integer NOT NULL,
	information  integer NOT NULL,
	debug  integer NOT NULL,
	PRIMARY KEY (facilityId)
)
;

CREATE TABLE logEventEmailConfig
(
	facility  text NOT NULL,
    facilityId integer NOT NULL,
	emergency  integer NOT NULL,
	alert  integer NOT NULL,
	critical  integer NOT NULL,
	error  integer NOT NULL,
	warning  integer NOT NULL,
	notice  integer NOT NULL,
	information  integer NOT NULL,
	debug  integer NOT NULL,
	PRIMARY KEY (facilityId)
)
;

CREATE TABLE compFacilityMap
(
	component  text NOT NULL,
	facilityId  integer NOT NULL,
	PRIMARY KEY (component ,facilityId )
)
;


CREATE TABLE sysLogInfo
(
	serverName  text NOT NULL,      -- servername or ip
	serverPort  integer DEFAULT 514,
    facilityId  integer,            --Ex: kernel 0
    severity    integer,            --Ex: emergency 1
    Enable      integer,            -- 1- enable , 0- disable
    serverId    integer             -- i th syslog server (0 to 7)
)
;

insert into tableDefaults values("sysLogInfo","serverPort","514","","");
insert into tableDefaults values("sysLogInfo","facilityId","255","","");
insert into tableDefaults values("sysLogInfo","severity","0","","");
insert into tableDefaults values("sysLogInfo","Enable","1","0","1");
insert into tableDefaults values("sysLogInfo","serverId","0","0","7");

--local0 facility (wireless)
insert into compFacilityMap values ("UMI_COMP_UDOT11", 16);
insert into compFacilityMap values ("UMI_COMP_KDOT11", 16);
insert into compFacilityMap values ("UMI_COMP_PNAC", 16);
insert into compFacilityMap values ("UMI_COMP_PNAC_USR", 16);
insert into compFacilityMap values ("UMI_COMP_RAD", 16);
insert into compFacilityMap values ("UMI_COMP_EAP_RAD", 16);
--local1 facility (UTM)
insert into compFacilityMap values ("UMI_COMP_IPS", 17);
insert into compFacilityMap values ("UMI_COMP_URLFILTER", 17);
insert into compFacilityMap values ("UMI_COMP_TM_PROTECTLINK", 17);
-- system facility
insert into compFacilityMap values ("UMI_COMP_LOGGING", 3);
insert into compFacilityMap values ("UMI_COMP_EVTDSPTCH", 3);
insert into compFacilityMap values ("UMI_COMP_PLATFORM", 3);
insert into compFacilityMap values ("UMI_COMP_REBOOT", 3);
-- kernel facility
insert into compFacilityMap values ("UMI_COMP_KERNEL", 0);


CREATE TABLE eventLog
(
	component  text NOT NULL,
	facilityId integer NOT NULL,
	logLevel  integer NOT NULL,
	textMessage  text NOT NULL,
    binMessage blob,
    FOREIGN KEY (component,facilityId) REFERENCES compFacilityMap(component,facilityId)
)
;

CREATE TABLE Events
(
	date DATE NOT NULL,
	eventMsg text NOT NULL
)
;

CREATE TABLE emailLogs
(
    mailLogs integer,
    entryName text NOT NULL,
    logId   text
)
;

insert into tableDefaults values("emailLogs","entryName","LOGGING","","");

insert into stringsMap values ("LOGGING_SYSLOG_CONFIG_FAILED","Syslog configuration failed");
insert into stringsMap values ("LOGGING_CONFIG_FAILED","Logging configuration failed");
insert into stringsMap values ("LOGGING_EMAILLOGS_CONFIG_FAILED","Email logs configuration failed");
insert into stringsMap values ("LOGGING_SMTP_CONFIG_FAILED","SMTP server configuration Failed");
insert into stringsMap values ("LOGGING_CRONTAB_CONFIG_FAILED","CRONTAB configuration failed");
insert into stringsMap values ("LOGGING_EMAIL_PASSED","Mail sent and logs cleared");
insert into stringsMap values ("LOGGING_EMAIL_FAILED","Unable to send mail");
insert into stringsMap values ("LOGGING_EMAIL_DISABLED","Email logs currently disabled. Please enable and try again.");
create TABLE reboot 
(
    reboot integer NOT NULL, 
    rebootTime integer NOT NULL
);

insert into reboot values (0,5);

insert into dbUpdateRegisterTbl values ("UMI_COMP_REBOOT",1,1,"reboot", 0, 1,1,1);
CREATE TABLE FirewallRules6
(
  _metadata text,                       -- metadata for internal use 
  OID integer NULL,
  RuleType text NOT NULL,               -- GUI (HIDDEN): SECURE_INSECURE
  Status integer NOT NULL,              -- GUI (HIDDEN)
  ServiceName text NULL,                -- GUI: Service
  Action text NULL,                     -- GUI: Action
  ScheduleName text NULL,               -- GUI: Select Schedule
  SourceAddressType integer NULL,       -- GUI: Src Users
  SourceAddressStart text NULL,         -- GUI: Start (Src Users)
  SourceAddressEnd text NULL,           -- GUI: Finish (Src Users)
  SourceAddressPrefixLength text NULL,  -- GUI: Prefix Length (Src Users)
  DestinationAddressType integer NULL,  -- GUI: Dest. Users
  DestinationAddressStart text NULL,    -- GUI: Start (Dest. Users)
  DestinationAddressEnd text NULL,      -- GUI: Finish (Dest. Users)
  DestinationAddressPrefixLength text NULL,  -- GUI: Prefix Length (Dest. Users)
  LogLevel integer NULL,                -- GUI: Log
  Ipv6HeaderType  text NULL,
  LogPrefix text NULL,     
  FromZoneType text NULL,
  ToZoneType text NULL,
  ConnectionState text NULL,
  Protocol integer NULL,
  DestinationPortStart integer NULL,
  DestinationPortEnd integer NULL,
  FOREIGN KEY (ScheduleName) REFERENCES Schedules (ScheduleName),
  FOREIGN KEY (ServiceName) REFERENCES Services (ServiceName)
)
;

CREATE TABLE FirewallRules
(
  _metadata text,                       -- metadata for internal use 
  OID integer NULL,
  RuleType text NOT NULL,               -- GUI (HIDDEN): SECURE_INSECURE
  Status integer NOT NULL,              -- GUI (HIDDEN)
  ServiceName text NULL,                -- GUI: Service
  TypeOfService integer NULL,           -- GUI: Qos. 
  Action text NULL,                     -- GUI: Action
  ScheduleName text NULL,               -- GUI: Select Schedule
  SourceAddressType integer NULL,       -- GUI: LAN/DMZ Users
  SourceAddressStart text NULL,         -- GUI: Start (LAN/DMZ Users)
  SourceAddressEnd text NULL,           -- GUI: Finish (LAN/DMZ Users)
  LogLevel integer NULL,                -- GUI: Log
  DestinationAddressType integer NULL,  -- GUI: WAN/DMZ Users
  DestinationAddressStart text NULL,    -- GUI: Start (WAN/DMZ Users)
  DestinationAddressEnd text NULL,      -- GUI: Finish (WAN/DMZ Users)
  SNATAddressType integer NULL,         -- GUI: NAT IP
  SNATAddress text NULL,                -- GUI: NAT IP Address
  SNATInterface text NULL,              -- GUI: NAT Single IP Is On
  DestinationPublicInterface text NULL, -- GUI: WAN Destination IP Address
  DestinationPublicAddress text NULL,   -- GUI: WAN Destination IP Address
                                        -- this value is 'OTHER' for public ipAddr
  DNATAddress text NULL,                -- GUI: Send to LAN/DMZ Server
  DNATPortEnable integer NULL,          -- GUI: Translate to Port Number
  DNATPort integer NULL,                -- GUI: Translate to Port Number
  FromZoneType text NULL,
  ToZoneType text NULL,
  ConnectionState text NULL,
  Protocol integer NULL,
  DestinationPortStart integer NULL,
  DestinationPortEnd integer NULL,
  SourcePort integer NULL, 
  TrafficLimit integer NULL,
  MaximumTrafficLimit integer NULL,
  ConnectionLimit integer NULL,
  TcpMssValue integer NULL,
  WebContent integer NULL,
  MacSource  text NULL,
  BwLimitMark integer NULL,
  DscpMark text NULL,
  DscpMatch integer NULL,
  PriorityId integer NULL DEFAULT 0,
  FOREIGN KEY (ScheduleName) REFERENCES Schedules (ScheduleName),
  FOREIGN KEY (ServiceName) REFERENCES Services (ServiceName),
  FOREIGN KEY (DestinationPublicInterface) REFERENCES Zones (ZoneLogicalIfName),
  FOREIGN KEY (SNATInterface) REFERENCES Zones (ZoneLogicalIfName)
)
;
insert into tableDefaults values ("FirewallRules","Status","1","0","2");
insert into tableDefaults values ("FirewallRules6","Status","1","0","1");
insert into stringsMap values ("FW_CONFIG_FAILED","Firewall Configuration Failed");
insert into stringsMap values ("FW_DEFAULT_CONFIG _FAILED","Default Outbound Policy Configuration Failed");
insert into stringsMap values ("FW_RULES_ENABLED_FAILED","Selected Firewall Rule(s) Enabling Failed");
insert into stringsMap values ("FW_RULES_SERVICE_NOT_FOUND","Selected Firewall Rule(s) Service Not Avaialble");
insert into stringsMap values ("FW_RULES_SCHED_NOT_FOUND","Selected Firewall Rule(s) Schedule Not Avaialble");
insert into stringsMap values ("FW_RULES_ENABLED","Selected Firewall Rule(s) Enabled Successfully");
insert into stringsMap values ("FW_RULES_DISABLED_FAILED","Selected Firewall Rule(s) Disabling Failed");
insert into stringsMap values ("FW_RULES_DISABLED","Selected Firewall Rule(s) Disabled Successfully");
insert into stringsMap values ("FW_RULES_DELETE_FAILED","Selected Firewall Rule(s) Delete Failed");
insert into stringsMap values ("FW_RULES_DELETED","Selected Firewall Rule(s) Deleted Successfully");
insert into stringsMap values ("FW_IN_RULE_CONFIG_FAILED","Firewall LAN WAN Inbound Rule Configuration Failed");
insert into stringsMap values ("FW_OUT_RULE_CONFIG_FAILED","Firewall LAN WAN Outbound Rule Configuration Failed");
insert into stringsMap values ("FW_SCHED_REQUIRE","Schedule Is Not Available, Add Schedule First ");

 

insert into ifDevEventTbl values("bdg1","UMI_COMP_FIREWALL","",1,1,1,0,0,0,0);
insert into ifDevEventTbl values("eth1","UMI_COMP_FIREWALL","",1,1,1,0,0,0,0);
insert into ifDevEventTbl values("sit0","UMI_COMP_FIREWALL","",1,1,1,0,0,0,0);

CREATE TABLE fwPriorityMap
(
    OID  integer NULL,
    RuleType        text NOT NULL,
    PriorityStr     text NULL,
    ResetStr        integer NULL DEFAULT 0   -- For avoiding factory reset from B51    
)
;

CREATE TABLE Services
(
  OID integer NULL,
  ServiceName text NOT NULL,    -- name of the service
  Protocol integer NOT NULL,       -- TCP/UDP/ICMP 
  DestinationPortStart integer NULL,    -- first TCP/UDP port or ICMP type
  DestinationPortEnd integer NULL,      -- last TCP/UDP port
  TypeOfService integer NULL,         -- type of service
  IsDefault boolean NOT NULL,
  PRIMARY KEY (ServiceName)
)
;
insert into tableDefaults values ("Services","TypeOfService","0","0","16");
insert into stringsMap values ("SERVICE_IN_USE","Service Is In Use, Disable/Delete Firewall Rule First ");
insert into stringsMap values ("SERVICE_IN_USE_DEL_RULE","Service Is In Use, Delete Firewall Rule First ");
insert into stringsMap values ("SERVICE_DELETE_FAILED","Service(s) Delete Failed");
insert into stringsMap values ("SERVICE_DELETE_OK","Selected Service(s) Deleted Successfully");
insert into stringsMap values ("SERVICE_CONFIG_FAILED","Service Configuration Failed");

CREATE TABLE Schedules
(
  OID integer NULL,
  ScheduleName text NOT NULL,
  Days integer NULL,
  AllDay boolean NOT NULL,
  StartTimeHours integer NULL,
  StartTimeMins integer NULL,
  EndTimeHours integer NULL,
  EndTimeMins integer NULL,
  PRIMARY KEY (ScheduleName)
);

insert into stringsMap values ("SCHEDULE_IN_USE","Schedule Is In Use, Disable/Delete Firewall Rule First ");
insert into stringsMap values ("SCHEDULE_IN_USE_DEL_RULE","Schedule Is In Use, Delete Firewall Rule First ");
insert into stringsMap values ("SCHEDULE_DELETE_FAILED","Schedule(s) Delete Failed");
insert into stringsMap values ("SCHEDULE_DELETE_OK","Selected Schedule(s) Deleted Successfully");
insert into stringsMap values ("SCHEDULE_CONFIG_FAILED","Schedule Configuration Failed");
insert into tableDefaults values ("Schedules","Days","127","0","127");
insert into tableDefaults values ("Schedules","AllDay","1","0","1");

CREATE TABLE BlockSites      -- table for blocking sites with keyword
(
  OID           integer NULL,
  KeywordToBlock    text NOT NULL,
  KeywordBlockingTo text NOT NULL,
  Status boolean NOT NULL,
  PRIMARY KEY (KeywordToBlock,KeywordBlockingTo)
)
;

insert into stringsMap values ("GROUPNAME_BLOCKED_KEYWORD_MAP_IN_USE","Selected Group Name(s) are using in Keyword Blocking");

CREATE TABLE PortTriggering
(
  OID                integer NULL,
  RuleName           text NOT NULL,                 -- port triggering rule name
  RuleStatus         integer NOT NULL,              -- enable/disable port trigger
  Interface          text    NOT NULL,               --rules for specific interface LAN/DMZ
  Protocol           integer NOT NULL,              -- protocol name 1-tcp,2-udp
  OutgoingPortStart  integer NOT NULL,              -- outgoing start port
  OutgoingPortEnd    integer NOT NULL,              -- outgoing end port
  IncomingPortStart  integer NOT NULL,              -- incoming start port
  IncomingPortEnd    integer NOT NULL,              -- incoming end port  
  PRIMARY KEY (RuleName)                            -- primary key
)
;
insert into stringsMap values ("FW_PORT_TRG_CONFIG_FAILED","Port Triggering Rule Configuration Failed");
insert into stringsMap values ("FW_PORT_TRG_RULES_DELETED","Selected Rule(s) Deleted Successfully");
insert into stringsMap values ("FW_PORT_TRG_RULES_DELETE_FAILED","Rule(s) Delete Failed");


CREATE TABLE PortTriggerStatus
(
OID  integer NULL,
LanIpAddr text NOT NULL,
OpenPortRangeStart integer NOT NULL,
OpenPortRangeEnd integer NOT NULL,
TimeRemaining integer NOT NULL
)
;

CREATE TABLE AttackChecks
(
  ExternalPing boolean,
  StealthMode boolean NULL,
  TcpFlood boolean NULL,
  UdpFlood boolean NULL,
  PingReplyOnLan boolean NULL,
  DnsProxy boolean NULL,
  IpsecPassthrough boolean NULL,
  PptpPassthrough boolean NULL,
  L2tpPassthrough boolean NULL
)
;

insert into stringsMap values ("FW_AC_CONFIG_FAILED","Attack checks Configuration Failed");

CREATE TABLE AttackChecks6
(
  ExternalPing boolean    -- enable/disable respond to ping on IPv6 WAN port
)
;

CREATE TABLE accessMgmt
(
    accessMgmtEnable integer NOT NULL,
    serviceType integer NOT NULL,
    accessMgmtIP1 text,
    accessMgmtIP2 text ,
    accessType integer NOT NULL,
    port integer NOT NULL,
    PRIMARY KEY (serviceType)
)
;


insert into stringsMap values("ACCESSMGMT_CONFIG_FAILED","Accesss Management Configuration Failed");

CREATE TABLE RemoteSnmp
(
    Status integer NOT NULL
)
;

CREATE TABLE ContentFiltering
(
  OID    integer NULL, 
  Status boolean NOT NULL                      --enable/disable content filter
  
)
;
insert into tableDefaults values ("ContentFiltering","Status","0","0","1");



CREATE TABLE WebComponents
(
  OID    integer NULL, 
  ProxyEnable           boolean NULL,       --Enable/Disable proxy to block
  JavaComponentEnable   boolean NULL,       --Enable/Disable Java component 
  ActivexControlEnable  boolean NULL,       --1-Enable/ 0-Disable 
  CookiesEnable         boolean NULL
)
;

insert into tableDefaults values ("WebComponents","ProxyEnable","0","0","1");
insert into tableDefaults values ("WebComponents","JavaComponentEnable","0","0","1");
insert into tableDefaults values ("WebComponents","ActivexControlEnable","0","0","1");
insert into tableDefaults values ("WebComponents","CookiesEnable","0","0","1");



CREATE TABLE TrustedDomains      -- table for blocking sites with keyword
(
  OID    integer NULL, 
  KeywordToAllow  text NULL,
  PRIMARY KEY (KeywordToAllow)
)
;
insert into stringsMap values ("FW_CF_CONFIG_FAILED","Content Filtering Configuration Failed");
insert into stringsMap values ("FW_KEYWORD_CONFIG_FAILED","Blocked Keyword Configuration Failed");
insert into stringsMap values ("FW_DOMAINS_CONFIG_FAILED","Trusted Domain Configuration Failed");
insert into stringsMap values ("FW_BLOCKED_KEYWORDS_DELETE_FAILED","Blocked Keyword Delete Failed");
insert into stringsMap values ("FW_BLOCKED_KEYWORDS_DELETED","Blocked Keyword(s) Deleted Successfully");
insert into stringsMap values ("FW_TRUSTED_DOMAINS_DELETE_FAILED","Trusted Domain Delete Failed");
insert into stringsMap values ("FW_TRUSTED_DOMAINS_DELETED","Trusted Domain(s) Deleted Successfully");


CREATE TABLE NatTable
(
  OID     integer NULL, 
  Enable  boolean NOT NULL
)
;

insert into stringsMap values ("ROUTING_MODE_CONFIG_FAILED","Routing Mode Configuration Failed");

-- table for firewall IP MAC binding
CREATE TABLE IPMACBinding 
( 
 OID integer NULL,
 Name               text NOT NULL,          --name of the entry for IPMACBinding
 SourceMACAddress   text NOT NULL,         
 IPAddress          text NOT NULL,
 LogStatus          boolean NOT NULL,
 PRIMARY KEY (Name)
)
;

-- table for firewall MAC filtering configuration
CREATE TABLE MACFiltering 
( 
 OID integer NULL,
 MACFilterEnable    boolean NOT NULL,       --Enable/Disable Mac Filter status
 MACAddressPolicy  boolean NOT NULL         --policy for the MAC Address
)
;

-- table for MAC address of  firewall MAC filtering 
CREATE TABLE SourceMACAddress 
( 
 OID integer NULL,
 SourceMACAddress      text NOT NULL,      
 PRIMARY KEY (SourceMACAddress)
)
;


-- table for IP MAC binding Email configuration
CREATE TABLE IPMACEmailLogsConfig
(
 OID integer NULL,
 IPMACVoilationEmailLogsSet boolean NOT NULL    
)
;

insert into stringsMap values ("FW_MAC_DELETE_FAILED","Mac address Deletion Failed");
insert into stringsMap values ("FW_MAC_DELETED","Selected Mac Address Deleted");
insert into stringsMap values ("FW_IPMAC_DELETE_FAILED","Ip/Mac Binding Deletion Failed");
insert into stringsMap values ("FW_IPMAC_DELETED","Selected Ip/Mac Bindings Deleted");
insert into stringsMap values ("FW_MACFILTERING_CONFIG_FAILED","Source Mac Filtering Configuration Failed");
insert into stringsMap values ("FW_IPMACBINDING_CONFIG_FAILED","IpMacBinding Configuration Failed");



CREATE TABLE ProtocolBinding
(
    OID integer,
    ServiceName text,
    Enabled boolean NOT NULL,
    LocalGatewayType integer NOT NULL,    
    SourceNetworkType integer NOT NULL,
    SourceNetworkStart text,
    SourceNetworkEnd text,
    DestinationNetworkType integer NOT NULL,
    DestinationNetworkStart text,
    DestinationNetworkEnd text,
    FOREIGN KEY (ServiceName) REFERENCES Services (ServiceName)
);

insert into tableDefaults values ("ProtocolBinding","Enabled","1","0","1");
insert into stringsMap values ("PB_RULES_ENABLED_FAILED","Selected Protocol Binding Rule(s) Enabling Failed");
insert into stringsMap values ("PB_RULES_ENABLED","Selected Protocol Binding Rule(s) Enabled Successfully");
insert into stringsMap values ("PB_RULES_DISABLED_FAILED","Selected Protocol Binding Rule(s) Disabling Failed");
insert into stringsMap values ("PB_RULES_DISABLED","Selected Protocol Binding Rule(s) Disabled Successfully");
insert into stringsMap values ("PB_RULES_DELETE_FAILED","Selected Protocol Binding Rule(s) Delete Failed");
insert into stringsMap values ("PB_RULES_DELETED","Selected Protocol Binding Rule(s) Deleted Successfully");
insert into stringsMap values ("PB_RULE_CONFIG_FAILED","Protocol Binding Rule Configuration Failed");

CREATE TABLE FwDosAttackSettings  --table Dos Attack ICMP flood and SYN floodsettings
(
OID                     integer  NULL,
SynFloodDetectRate      integer NOT NULL,
EchoStormFloodRate      integer NOT NULL,
IcmpFloodRate           integer NOT NULL
)
;

CREATE TABLE FirewallLogs
(
OID                            integer NULL,
LANToWANAccept                 boolean NOT NULL,   --LAN to WAN Accept packets logs
LANToWANDrop                   boolean NOT NULL,   --LAN to WAN Drop packets logs
WANToLANAccept                 boolean NOT NULL,   --WAN to LAN accept packets logs		 
WANToLANDrop                   boolean NOT NULL,   --WAN to LAN Drop packets logs	 
LANToDMZAccept                 boolean NOT NULL,   --LAN to DMZ	Accept packets logs 
LANToDMZDROP                   boolean NOT NULL,   --LAN to DMZ drop packets logs	 
DMZToWANAccept                 boolean NOT NULL,   --DMZ to WAN Accept packets logs	 
DMZToWANDrop                   boolean NOT NULL,   --DMZ to WAN drop packets logs		 
DMZToLANAccept                 boolean NOT NULL,   --DMZ to LAN Accept packets logs	 
DMZToLANDrop                   boolean NOT NULL,   --DMZ to LAN drop packets logs 
WANToDMZDrop                   boolean NOT NULL,   --WAN to DMZ drop packets logs
WANToDMZAccept                 boolean NOT NULL,   --WAN to DMZ Accept packets logs
UnicastTraffic                 boolean NOT NULL,   --All unicast traffic logs	 
BroadCastORMulticastTraffic    boolean NOT NULL,   --All Broadcast/mulitcast traffic logs
SourceMACFilter                boolean NOT NULL,   --source MAC filter logs
SessionLimitLogs               boolean NOT NULL,	  	  
BandwidthLimitLogs             boolean NOT NULL
)
;


CREATE TABLE FirewallLogs6
(
OID                            integer NULL,
LANToWANAccept                 boolean NOT NULL,   --LAN to WAN Accept packets logs
LANToWANDrop                   boolean NOT NULL,   --LAN to WAN Drop packets logs
WANToLANAccept                 boolean NOT NULL,   --WAN to LAN accept packets logs		 
WANToLANDrop                   boolean NOT NULL    --WAN to LAN Drop packets logs	 
)
;


CREATE TABLE OtherSystemLogs
(
OID                     integer NULL,
ChangeOfTimeByNTPLogs  	boolean NOT NULL, 	 
LoginAttemptsLogs       boolean NOT NULL,
SecureLoginAttempts 	boolean NOT NULL,	 
RebootLogs              boolean NOT NULL,
WANStatusLogs  	 	    boolean NOT NULL,
ResolvedDNSNames        boolean NOT NULL
);

CREATE TABLE IcsaSettings               --table for ICSA settings
(
 OID integer,
 BlockICMPNotification boolean,
 BlockFragPkts         boolean,
 BlockMulticastPkts    boolean
)
;



CREATE TABLE GlobalSettings
(
    OID integer,
    MaxUnidentifiedSessions integer NOT NULL,
    MaxHalfOpenSessions integer NOT NULL,
    TcpSessionTimoutDuration integer NOT NULL,
    UdpSessionTimoutDuration integer NOT NULL,
    OtherSessionTimeoutDuration integer NOT NULL,
    TcpSessionCleanupLatency integer NOT NULL
)
;
CREATE TABLE FwTrafficMeter
(
    OID integer,
    TrafficMeterStatus      boolean NOT NULL, -- enable/disable Trafficmeter
    WanId                   integer NOT NULL, -- wan mode 1-WAN1, 2-WAN2
    TrafficLimitType        integer,          -- 0 -No limit, 1-Download only, 2-Both Directions
    TrafficMonthlyLimit       integer,     -- monthly limit
    IncreaseLimitStatus       boolean,     -- Enable/disable increase limit
    IncreaseLimitBy           integer,     -- Extra traffic limit for this month
    RestartCounter            boolean,     -- 0 -Restart the counters now
                                           -- 1 -Restart counter on specific time 
    TimeHH                    integer,     -- hours
    TimeMM                    integer,     -- minutes
    DayOfMonth                integer,     -- Day of the month
    SendEmailReport           integer,     -- sends the email report before restart
    BlockTraffic              boolean,     -- 0 -Block All traffic when limit is reached
                                           -- 1 - Block All traffic Except Email
    SendEmailAlert            boolean,     -- Enable/Disable Email Alert
    PRIMARY KEY (WanId)
)    
;
CREATE TABLE TrafficCounters
(
OID     integer NULL,
WanId   integer NOT NULL,
TotalInCounter NUMERIC,
TotalOutCounter NUMERIC,
HttpInCounter NUMERIC,
HttpOutCounter NUMERIC,
EmailInCounter NUMERIC,
EmailOutCounter NUMERIC,
TfMtrlimitStatus  NUMERIC,     -- 1- traffic meter limit reached
                               -- 0- traffic meter limit not reached
PRIMARY KEY (WanId)
)
;

CREATE TABLE TrafficStatistics      -- table for traffic meter statistics
(
 OID             integer  NULL,
 WanId           integer NOT NULL,      -- 1= wan1 2=wan2
 StartTime       text     NULL,         --traffic Date/Time
 StartTimeReset  integer,         -- 1= ResetTime 0=Don't Reset
 OutgoingTrafVol integer  NULL,  --outgoing traffic volume
 IncomingTrafVol integer  NULL,  -- incoming traffic volume
 AvgPerDay       integer  NULL,  --Average per day
 StandardLimit   integer  NULL,  -- % of stadard limit
 ThisMonthLimit  integer  NULL,   -- %of this month limit
 PRIMARY KEY (WanId)
)
;

CREATE TABLE OneToOneNat
(
    OID             integer NULL,
    WanIfName       text  NOT NULL, -- WAN1/WAN2 etc
    PrivateIPstart  text NOT NULL, -- IP address in LAN
    PublicIPstart   text NOT NULL, -- public ip address
    PublicIPmask    text NOT NULL,    -- public IP address maks ex: 255.255.255.0
    RangeLen        integer  NULL, -- no of address from the start ip address
    PRIMARY KEY (PrivateIPstart)
)
;

CREATE TABLE OneToOneNatService
(
    OID             integer NULL,
    LanServerIp     text NOT NULL,  
    serviceName     text NOT NULL,  --service to be forwarded for LAN host
    WanIfName       text  NOT NULL, -- WAN1/WAN2 etc
    PRIMARY KEY (serviceName)
)
;

CREATE TABLE DefaultIfGrpInfo
(
    OID  integer NULL,
    LogicalIfName        text NOT NULL,
    VirtualIfName        text NOT NULL,
    IfGroupId            integer NOT NULL,
    PRIMARY KEY (LogicalIfName, VirtualIfName)
)
;

-- AlgConf - table for ALG configuration.
CREATE TABLE AlgConf
    (
    OID   Integer     NULL, -- OID
    Pptp  Boolean NOT NULL, -- PPTP ALG
    Ipsec Boolean NOT NULL, -- IPSEC ALG
    Rtsp  Boolean NOT NULL, -- RTSP ALG
    Sip   Boolean NOT NULL, -- SIP ALG
    H323  Boolean NOT NULL, -- H323 ALG
    Smtp  Boolean NOT NULL, -- SMTP ALG
    Dns   Boolean NOT NULL, -- DNS ALG
    Tftp  Boolean NOT NULL  -- TFTP ALG
    );

-- BW Mon table
CREATE TABLE BwMonStat
(
    OID   Integer     NULL, -- OID
    LogicalIfName     text NOT NULL,
    AppName              text NOT NULL,    
    Counter              text NOT NULL,    
    PRIMARY KEY (LogicalIfName, AppName)
);

--Default values 
--Lan
insert into BwMonStat values (0, "LAN", "HTTP", "0");
insert into BwMonStat values (0, "LAN", "HTTPS", "0");
insert into BwMonStat values (0, "LAN", "DNS", "0");
insert into BwMonStat values (0, "LAN", "SMTP", "0");
insert into BwMonStat values (0, "LAN", "IMAP2", "0");
insert into BwMonStat values (0, "LAN", "IMAP3", "0");
insert into BwMonStat values (0, "LAN", "POP3", "0");
insert into BwMonStat values (0, "LAN", "SSH", "0");
insert into BwMonStat values (0, "LAN", "TELNET", "0");
insert into BwMonStat values (0, "LAN", "SNMP", "0");
insert into BwMonStat values (0, "LAN", "NFS", "0");
--Wan1
insert into BwMonStat values (0, "WAN1", "HTTP", "0");
insert into BwMonStat values (0, "WAN1", "HTTPS", "0");
insert into BwMonStat values (0, "WAN1", "DNS", "0");
insert into BwMonStat values (0, "WAN1", "SMTP", "0");
insert into BwMonStat values (0, "WAN1", "IMAP2", "0");
insert into BwMonStat values (0, "WAN1", "IMAP3", "0");
insert into BwMonStat values (0, "WAN1", "POP3", "0");
insert into BwMonStat values (0, "WAN1", "SSH", "0");
insert into BwMonStat values (0, "WAN1", "TELNET", "0");
insert into BwMonStat values (0, "WAN1", "SNMP", "0");
insert into BwMonStat values (0, "WAN1", "NFS", "0");
--Wan2
insert into BwMonStat values (0, "WAN2", "HTTP", "0");
insert into BwMonStat values (0, "WAN2", "HTTPS", "0");
insert into BwMonStat values (0, "WAN2", "DNS", "0");
insert into BwMonStat values (0, "WAN2", "SMTP", "0");
insert into BwMonStat values (0, "WAN2", "IMAP2", "0");
insert into BwMonStat values (0, "WAN2", "IMAP3", "0");
insert into BwMonStat values (0, "WAN2", "POP3", "0");
insert into BwMonStat values (0, "WAN2", "SSH", "0");
insert into BwMonStat values (0, "WAN2", "TELNET", "0");
insert into BwMonStat values (0, "WAN2", "SNMP", "0");
insert into BwMonStat values (0, "WAN2", "NFS", "0");

-- Event handlers for Firewall
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",0,1,"LanGroup",0,1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",0,1,"LanHost",0,1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",1,1,"Services",0,1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",1,1,"Schedules",0,1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",1,1,"FirewallRules", 0,1,1,1);  
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",1,1,"FirewallRules6", 0,1,1,1); 
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",1,1,"PortTriggering",0,1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",1,1,"AttackChecks",
                                        0,1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",1,1,"AttackChecks6",
                                        0,1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",1,1,"accessMgmt", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",1,1,"RemoteSnmp", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",1,1,"ContentFiltering",0,1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",1,1,"WebComponents",
                                        0,1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",1,1,"TrustedDomains",0,1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",1,1,"BlockSites",0,1,1,1);   
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",0,1,"NatTable",0,1,1,1);
-- Register networkInterface table to update firewall Rules.
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",0,1,"networkInterface",0,1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",0,1,"fwPriorityMap",0,1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",0,1,"IPMACBinding",0,1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",0,1,"SourceMACAddress",0,1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",0,1,"MACFiltering",0,1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",0,1,"FwDosAttackSettings",0,1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",0,1,"FirewallLogs",0,1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",0,1,"FirewallLogs6",0,1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",0,1,"IcsaSettings",0,1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",0,1,"GlobalSettings",0,1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",0,1,"WanMode",0,1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",0,1,"ProtocolBinding",0,1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",0,1,"FwTrafficMeter",0,1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",0,1,"dhcp",0,1,1,1);
-- Register Zones table to update firewall Rules.
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",0,1,"Zones",6,1,1,1);

insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",0,1,"OneToOneNat",0,1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",0,1,"OneToOneNatService",0,1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",1,1,"DefaultIfGrpInfo",0,1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",1,1,"interVLANRouting",0,1,1,1);

insert into stringsMap values ("CONFIG_FAILED","One to one nat config failed");

-- Register AlgConf for firewall
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",0,1,"AlgConf",0,1,1,1);

-------------------------------------------------------------------------------
--                        IPv4 Firewall Default Rules
-------------------------------------------------------------------------------
-- NAT Defaults:
-- =============
-- -A PREROUTING -j fwDhcpRelay
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_NAT_IB_REDIRECT_RULE", 1, "fwDhcpRelay", "ANY", "ANY");

-- -A PREROUTING -j preOneToOneNat
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_NAT_IB_INTERNAL_DNAT_GRP", 1, "preOneToOneNat", "ANY", "ANY");


-- -A PREROUTING -m set --set Dnat dst -j INSECURE_SECURE
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_NAT_IB_INTERNAL_DNAT_GRP", 1, "INSECURE_SECURE", "ANY", "ANY");

-- -A PREROUTING -m set --set Dnat dst -j INSECURE_PUBLIC
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_NAT_IB_INTERNAL_DNAT_GRP", 1, "INSECURE_PUBLIC", "ANY", "ANY");

-- -A PREROUTING -i <wan<N>Iface> -j TRIGGER --trigger-type dnat
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_NAT_IB_TRIGGER_DNAT", 1, "FW_TRIGGER_DNAT", "INSECURE", "ANY");

-- -A PREROUTING -i <lanIface> -j DEFAULT_WEB_SERVICE
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_NAT_IB_REDIRECT_RULE", 1, "DEFAULT_WEB_SERVICE", "SECURE", "ANY");

-- -A PREROUTING -i <lanIface> -j REDIRECT_RULE
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_NAT_IB_REDIRECT_RULE", 1, "REDIRECT_RULE", "SECURE", "ANY");


-- -A POSTROUTING  -j postOneToOneNat
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_NAT_OB_INTERNAL_INSECURE", 1, "postOneToOneNat", "ANY", "INSECURE");

-- -A POSTROUTING -o <wan<N>Iface> -j SECURE_INSECURE
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_NAT_OB_INTERNAL_INSECURE", 1, "SECURE_INSECURE", "ANY", "INSECURE");

-- -A POSTROUTING -o <wan<N>Iface> -j PUBLIC_INSECURE
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_NAT_OB_INTERNAL_INSECURE", 1, "PUBLIC_INSECURE", "ANY", "INSECURE");

-- -A POSTROUTING -o <wan<N>Iface> -j SNAT --to-source <wan<N>Addr>  
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_NAT_OB_SNAT_RULE", 1, "FW_SNAT_INSECURE","ANY", "INSECURE");

-- -A POSTROUTING -m mark --mark 0x1111 -j SNAT --to-source <lanAddr>
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_NAT_OB_SNAT_MARK_DNAT", 1, "FW_SNAT_SECURE","ANY", "ANY");

-- Firewall Defaults:
-- ==================
-- -A INPUT -i <loIface> -j ACCEPT
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_IB_LOOP_BACK", 1, "ACCEPT", "LOCAL", "ANY");

-- -A INPUT -m state --state RELATED,ESTABLISHED -j ACCEPT
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, ConnectionState) values ("edit=0","FW_FILTER_IB_STATE", 1, "ACCEPT", "ANY", "ANY", "RELATED,ESTABLISHED");

-- -A INPUT -m mark --mark 0xF1DEAD -j ACCEPT
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_IB_IPSEC_MARK", 1, "ACCEPT", "ANY", "ANY");

-- -A fwInputLnA -j ACCEPT
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_IN_ACCEPT", 1, "ACCEPT", "ANY", "ANY");


-- A INPUT -i <lanIface> -p tcp --dport 49152:49153 -j fwInputLnA
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart, DestinationPortEnd) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "fwInputLnA", "SECURE", "ANY", 6, 49152, 49153);

-- A INPUT -i <lanIface> -p udp --dport 1900 -j fwInputLnA
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType,
Protocol, DestinationPortStart) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "fwInputLnA", "SECURE", "ANY", 17, 1900);

-- A INPUT -i <lanIface> -p 41 -j fwInputLnA
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "fwInputLnA", "SECURE", "ANY", 41);

-- A INPUT -i <wanIface> -p 41 -j fwInputLnA
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "fwInputLnA", "INSECURE", "ANY", 41);

-- -A INPUT -i <lanIface> -p tcp --syn --dport FW_HTTP_PROXY_PORT -j fwInputLnA

insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0", "FW_FILTER_IB_SYN_SERVICE", 1, "fwInputLnA", "SECURE", "ANY", 6, 8080);

-- -A INPUT -i <lanIface> -p tcp --syn --dport FW_TELNET_PORT -j fwInputLnA
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","FW_FILTER_IB_SYN_SERVICE", 1, "fwInputLnA", "SECURE", "ANY", 6, 23);

-- -A INPUT -i <lanIface> -p tcp --syn --dport  FW_HTTP_PORT -j fwInputLnA
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","FW_FILTER_IB_SYN_SERVICE", 1, "fwInputLnA", "SECURE", "ANY", 6, 80);

-- -A INPUT -i <lanIface> -p tcp --syn --dport FW_HTTPS_PORT -j fwInputLnA
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","FW_FILTER_IB_SYN_SERVICE", 1, "fwInputLnA", "SECURE", "ANY", 6, 443);

-- -A INPUT -i <lanIface> -p tcp --syn --dport FW_SSH_PORT -j fwInputLnA
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType,Protocol, DestinationPortStart) values ("edit=0","FW_FILTER_IB_SYN_SERVICE", 1,"fwInputLnA", "SECURE", "ANY", 6, 22);

-- -A INPUT -i <lanIface> -p udp --dport 67:68 -j fwInputLnA
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart, DestinationPortEnd) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "fwInputLnA", "SECURE", "ANY", 17, 67, 68);

-- -A INPUT -i <lanIface> -p udp --dport <ripPort> -j fwInputLnA
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType,
    Protocol, DestinationPortStart) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "fwInputLnA", "SECURE", "ANY", 17, 520);


-- -A INPUT -i <lanIface> -p udp --dport 161 -j fwInputLnA
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "fwInputLnA", "SECURE", "ANY", 17, 161);

-- -A INPUT -i <lanIface> -p icmp -j fwInputLnA
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "fwInputLnA", "SECURE", "ANY", 1);

-- -A INPUT -i <dmzIface> -p icmp -j fwInputLnA
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "fwInputLnA", "PUBLIC", "ANY", 1);

-- -A INPUT -i <dmzIface> -p icmp --icmp-type 8 -m limit --limit 10/second --limit-burst 30/second -j fwInputLnA
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart, TrafficLimit, MaximumTrafficLimit) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "fwInputLnA", "PUBLIC", "ANY", 1, 8, 36000, 30);

-- -A INPUT -i <dmzIface> -p udp --dport 67:68 -j fwInputLnA
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart, DestinationPortEnd) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "fwInputLnA", "PUBLIC", "ANY", 17, 67, 68);

-- -A INPUT -p udp -j fwInDhcpRelay  
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "fwInDhcpRelay", "ANY", "ANY", 17);

-- -A INPUT -p udp -j fwInputDnsAndDhcp  
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "fwInputDnsAndDhcp", "ANY", "ANY", 17);

-- -A INPUT -p 50 -j fwInputLnA  (ESP);
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "fwInputLnA", "ANY", "ANY", 50);

-- -A INPUT -p 51 --j fwInputLnA (AH) 
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "fwInputLnA", "ANY", "ANY", 51);

-- -A INPUT -j fwRipInput
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_IB_INTERNAL", 1, "fwRipInput", "ANY", "ANY");

-- -A INPUT -p udp --dport 500 -j fwInputLnA
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "fwInputLnA", "ANY", "ANY", 17, 500);

-- -A INPUT -p udp --dport 4500 -j fwInputLnA
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "fwInputLnA", "ANY", "ANY", 17, 4500);

-- -A INPUT -j fwPPTPIn
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "fwPPTPIn", "ANY", "ANY");

-- -A INPUT -j fwInBypass
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_IB_INTERNAL", 1, "fwInBypass", "ANY", "ANY");

-- -A INPUT -p icmp -j fwInFlood
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "fwInFlood", "ANY", "ANY",1);

-- -A INPUT -j fwStealth
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_IB_INTERNAL", 1, "fwStealth", "ANY", "ANY");

-- -A INPUT -j fwInFltrEnd
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType)
values ("edit=0","FW_FILTER_IB_SERVICE", 1, "fwInFltrEnd", "ANY", "ANY");

-- Bonjour Specific rule
-- -A INPUT -p udp --dport <mDNSPort> -j ACCEPT
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType,
    Protocol, DestinationPortStart) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "fwInputLnA", "ANY", "ANY", 17, 5353);

-- Bonjour Specific rule
-- -A INPUT -p udp --sport <mDNSPort> -j ACCEPT
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType,
    Protocol, SourcePort) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "fwInputLnA", "ANY", "ANY", 17, 5353);

-- -A fwdAccept -j ACCEPT
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType,
    ToZoneType) values ("edit=0","IPS_QUEUE", 1, "ACCEPT", "ANY", "ANY");

-- -A FORWARD -j CONFIGFORWARD
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_INTERNAL_SUBSET", 1, "CONFIGFORWARD", "ANY", "ANY");

-- -A FORWARD -m state --state RELATED,ESTABLISHED -j ACCEPT
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, ConnectionState) values ("edit=0","FW_FILTER_FWD_STATE", 1, "ACCEPT", "ANY", "ANY", "RELATED,ESTABLISHED");

-- -A FORWARD -i <loIface> -j ACCEPT
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_LOOP_BACK", 1, "ACCEPT", "LOCAL", "ANY");

-- -A FORWARD -m mark --mark 0xF1DEAD -j ACCEPT
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_IPSEC_MARK", 1, "ACCEPT", "ANY", "ANY");

-- -A FORWARD -o <wan<N>Iface>  fwVpnCheck
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_SERVICE", 1, "fwVpnCheck", "ANY", "INSECURE");

-- -A FORWARD -i <lanIface> -j fwSrcMacFltr
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_INTERNAL_SUBSET", 1, "fwSrcMacFltr", "SECURE", "ANY");

-- -A FORWARD -i <lanIface> -j fwBindIpToMac
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_INTERNAL_SUBSET", 1, "fwBindIpToMac", "SECURE", "ANY");

-- -A FORWARD -i <lanIface> -j fwBindMacToIp
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_INTERNAL_SUBSET", 1, "fwBindMacToIp", "SECURE", "ANY");

-- -A FORWARD -i -j fwConnLimitMatch
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_INTERNAL_SUBSET", 1, "fwConnLimitMatch", "ANY", "ANY");

-- -A FORWARD -i <lanIface> -o <dmzIface> -j SECURE_PUBLIC
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_INTERNAL_SUBSET", 1, "SECURE_PUBLIC", "SECURE", "PUBLIC");

-- -A FORWARD -i <dmzIface> -o <lanIface>  -j PUBLIC_SECURE
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_INTERNAL_SUBSET", 1, "PUBLIC_SECURE", "PUBLIC", "SECURE");

-- -A FORWARD -i <dmzIface> -o <wan<N>Iface> -j PUBLIC_INSECURE
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_INTERNAL_SUBSET", 1, "PUBLIC_INSECURE", "PUBLIC", "INSECURE");

-- -A FORWARD -i <lanIface> -o <wan<N>Iface> -j SECURE_INSECURE
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_INTERNAL_SUBSET", 1, "SECURE_INSECURE", "SECURE", "INSECURE");

-- -A FORWARD  -j fwTrigger
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_INTERNAL_SUBSET", 1, "fwTrigger", "ANY", "ANY");

-- -A FORWARD -j fwdOneToOneNat 
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_INTERNAL_SUBSET", 1, "fwdOneToOneNat", "ANY", "ANY");


-- -A FORWARD -j fwFltrIb 
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_INTERNAL_SUBSET", 1, "fwFltrIb", "ANY", "ANY");

-- -A FORWARD -j fwInterVlanRouting
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_INTER_VLAN_ROUTING", 1, "fwInterVlanRouting", "ANY", "ANY");

-- -A FORWARD -m connmark --mark <wan1Mark/wan2Mark> -i <wan1> -o <lanIface> -j INSECURE_SECURE
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_INTERNAL_SUBSET_LB", 1, "INSECURE_SECURE", "INSECURE", "SECURE");

-- -A FORWARD -m connmark --mark <wan1Mark/wan2Mark> -i <lanIface> -o <lanIface> -j INSECURE_SECURE
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_INTERNAL_SUBSET_LB_LAN_WAN1", 1, "INSECURE_SECURE", "SECURE", "SECURE");

-- -A FORWARD -m connmark --mark <wan1Mark/wan2Mark> -i <lanIface> -o <lanIface> -j INSECURE_SECURE
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_INTERNAL_SUBSET_LB_LAN_WAN2", 1, "INSECURE_SECURE", "SECURE", "SECURE");

-- -A FORWARD -m connmark --mark <wan1Mark/wan2Mark> -i <wan1> -o <dmzIface> -j INSECURE_PUBLIC
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_INTERNAL_SUBSET_LB", 1, "INSECURE_SECURE", "INSECURE", "PUBLIC");

-- -A FORWARD -m connmark --mark <wan1Mark/wan2Mark> -i <dmzIface> -o <dmzIface> -j INSECURE_PUBLIC
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_INTERNAL_SUBSET_LB", 1, "INSECURE_SECURE", "PUBLIC", "PUBLIC");

-- -A FORWARD -m connmark --mark <wan1Mark/wan2Mark> -j TRIGGER --trigger-type in
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_INTERNAL_SUBSET_TRIGR_LB_WAN", 1, "FW_TRIGGER_IN", "ANY", "ANY");

-- -A FORWARD -m connmark --mark 0x888 -j TRIGGER --trigger-type in
--insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_INTERNAL_SUBSET_TRIGR_LB_WAN2", 1, "FW_TRIGGER_IN", "ANY", "ANY");


-- this rule  is to allow all the lan-lan[vlans included] traffic 
-- -A FORWARD -i <fwlanIface>  -o <fwlanIface> -j fwAccept 
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_DEFAULT", 1, "fwdAccept", "SECURE", "SECURE");

-- -A FORWARD -i <fwlanIface> -j DEFAULT_POLICY 
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_DEFAULT", 1, "DEFAULT_POLICY", "SECURE", "ANY");

-- this rule for accepting dmz static routing traffic 
-- -A FORWARD -i <fwdmzIface>  -o <fwdmzIface> -j INSECURE_SECURE
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_INTERNAL_SUBSET", 1, "fwdAccept", "PUBLIC", "PUBLIC");

-- -A FORWARD -i <fwdmzIface> -j INSECURE_SECURE
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_INTERNAL_SUBSET", 1, "INSECURE_SECURE", "PUBLIC", "ANY");

-- -A FORWARD -i <fwdmzIface> -j INSECURE_PUBLIC
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_INTERNAL_SUBSET", 1, "INSECURE_PUBLIC", "PUBLIC", "ANY");

-- -A FORWARD -j fwFwdFltrEnd
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_SERVICE", 1, "fwFwdFltrEnd", "ANY", "ANY");

-- -A fwFltrIb -m connmark --mark 0x2222 -i <wan<N>Iface> -o <lanIface> -j INSECURE_SECURE 
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_INTERNAL_DNAT_CONNMARK", 1, "INSECURE_SECURE", "INSECURE", "SECURE");

-- -A fwFltrIb -m connmark --mark 0x2222 -i <lanIface> -o <lanIface> -j INSECURE_SECURE
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_INTERNAL_DNAT_CONNMARK", 1, "INSECURE_SECURE", "SECURE", "SECURE");

-- -A fwFltrIb -m connmark --mark 0x2222 -i <wan<N>Iface> -o <dmzIface> -j INSECURE_PUBLIC
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_INTERNAL_DNAT_CONNMARK", 1, "INSECURE_PUBLIC", "INSECURE", "PUBLIC");

-- -A fwFltrIb -m connmark --mark 0x2222 -i <dmzIface> -o <dmzIface> -j INSECURE_PUBLIC
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_INTERNAL_DNAT_CONNMARK", 1, "INSECURE_PUBLIC", "PUBLIC", "PUBLIC");

-- -A FORWARD -m connmark --mark 0x2222 -j TRIGGER --trigger-type in
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_INTERNAL_DNAT_CONNMARK", 1, "FW_TRIGGER_IN", "ANY", "ANY");

-- -A fwPPTPBox -p tcp --dport FW_PPTP_PORT -j ACCEPT
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","FW_PPTP_BOX", 1, "ACCEPT", "ANY", "ANY", 6, 1723);

-- -A fwPPTPBox -p gre -j ACCEPT
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType,
Protocol) values ("edit=0","FW_PPTP_BOX", 1, "ACCEPT", "ANY", "ANY", 47);

-- -A fwPPTPIn -p tcp --dport FW_PPTP_PORT -j ACCEPT
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","FW_PPTP_IN", 1, "ACCEPT", "ANY", "ANY", 6, 1723);

-- -A fwPPTPIn -p gre -j ACCEPT
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType,
Protocol) values ("edit=0","FW_PPTP_IN", 1, "ACCEPT", "ANY", "ANY", 47);


-- Mangle Defaults:
-- =================

-- -A OUTPUT -o <lanIface> -p tcp --sport 80 -j TOS --set-tos 0x16
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, SourcePort, TypeOfService) values ("edit=0","FW_MANGLE_OUT_TOS", 1, "TOS", "ANY", "SECURE", 6, 80, 16);

-- -A FORWARD -i <lanIface> -o <lanIface> -m connmark --mark 0x2222 -j MARK --set-mark 0x1111 
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_MANGLE_FWD_INTERNAL_DNAT_CONNMARK", 1, "MARK", "SECURE", "SECURE");

-- -A FORWARD -i <dmzIface> -o <dmzIface> -m connmark --mark 0x2222 -j MARK --set-mark 0x1111
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_MANGLE_FWD_INTERNAL_DNAT_CONNMARK", 1, "MARK", "PUBLIC", "PUBLIC");
.timeout 1000

-- -A FORWARD -i <lan<N>Iface> -o <wan<N>Iface> -j SECURE_INSECURE
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_MANGLE_FWD_INTERNAL_SUBSET", 1, "SECURE_INSECURE", "SECURE", "INSECURE");

-- -A FORWARD -i <dmz<N>Iface> -o <wan<N>Iface> -j PUBLIC_INSECURE
insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_MANGLE_FWD_INTERNAL_SUBSET", 1, "PUBLIC_INSECURE", "PUBLIC", "INSECURE");

-- Account Rules

insert into FirewallRules (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_MANGLE_FWD_INTERNAL_SUBSET", 1, "PUBLIC_INSECURE", "PUBLIC", "INSECURE");


-------------------------------------------------------------------------------
--                        IPv6 Firewall Default Rules
-------------------------------------------------------------------------------

-- INPUT CHAIN

-- $IPTABLES_BIN  -A INPUT -m state --state ESTBLISHED,RELATED -j ACCEPT
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, ConnectionState) values ("edit=0","FW_FILTER_IB_STATE", 1, "ACCEPT", "ANY", "ANY", "RELATED,ESTABLISHED");

-- $IPTABLES_BIN  -A INPUT -j fwLanInFilter
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "fwLanInFilter", "ANY", "ANY");

-- $IPTABLES_BIN  -A INPUT -s ::1/128 -d ::1/128 -j ACCEPT
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, SourceAddressType, SourceAddressStart, SourceAddressPrefixLength, DestinationAddressType, DestinationAddressStart, DestinationAddressPrefixLength) values ("edit=0","FW_FILTER_IB_LOOP_BACK", 1, "ACCEPT", "ANY", "ANY", 1,"::1",128, 1, "::1",128);

-- $IPTABLES_BIN  -A INPUT -i lo -j ACCEPT
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_IB_LOOP_BACK", 1, "ACCEPT", "LOCAL", "ANY");

-- $IPTABLES_BIN -A  INPUT -p icmpv6 -m icmpv6 --icmpv6-type 133/0 -j ACCEPT
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "ACCEPT", "ANY", "ANY", 58, " 133");

-- $IPTABLES_BIN -A  INPUT -p icmpv6 -m icmpv6 --icmpv6-type 134/0 -j ACCEPT
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "ACCEPT", "ANY", "ANY", 58, " 134");

-- $IPTABLES_BIN -A  INPUT -p icmpv6 -m icmpv6 --icmpv6-type 135/0 -j ACCEPT
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "ACCEPT", "ANY", "ANY", 58, " 135");

-- $IPTABLES_BIN -A  INPUT -p icmpv6 -m icmpv6 --icmpv6-type 136/0 -j ACCEPT
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "ACCEPT", "ANY", "ANY", 58, " 136");

-- $IPTABLES_BIN  -A INPUT -p udp -m udp --dport 546  -j ACCEPT
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol,DestinationPortStart) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "ACCEPT", "ANY", "ANY", 17, 546);

-- $IPTABLES_BIN  -A INPUT -p udp -m udp --dport 547  -j ACCEPT
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "ACCEPT", "ANY", "ANY", 17, 547);

-- $IPTABLES_BIN  -A INPUT -j fwWanInFilter
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "fwWanInFilter", "ANY", "ANY");

-- $IPTABLES_BIN  -A INPUT  -j ipv6HdrFilter
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "ipv6HdrFilter", "ANY", "ANY");

-- $IPTABLES_BIN  -A INPUT -j fwSysInputDrop
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_IB_SERVICE", 1, "fwSysInputDrop", "ANY", "ANY");


-- FORWARD CHAIN
--
-- $IPTABLES_BIN  -A   -j ACCEPT
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_ACCEPT", 1, "ACCEPT", "ANY", "ANY");

-- $IPTABLES_BIN  -A FORWARD -m state --state ESTBLISHED,RELATED -j ACCEPT
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, ConnectionState) values ("edit=0","FW_FILTER_FWD_STATE", 1, "ACCEPT", "ANY", "ANY", "RELATED,ESTABLISHED");


-- $IPTABLES_BIN  -A FORWARD -j fwForwardFilter
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_SERVICE", 1, "fwForwardFilter", "ANY", "ANY");

-- $IPTABLES_BIN  -A FORWARD -j icmpv6ForwardFilter
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_SERVICE", 1, "icmpv6ForwardFilter", "ANY", "ANY");

-- $IPTABLES_BIN  -A FORWARD  -j ipv6HdrFilter
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_SERVICE", 1, "ipv6HdrFilter", "ANY", "ANY");

-- $IPTABLES_BIN  -A FORWARD -j ipv6BadAddrs
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_SERVICE", 1, "ipv6BadAddrs", "ANY", "ANY");

-- $IPTABLES_BIN  -A FORWARD -d fe80::/10 -o <wan> -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, DestinationAddressType, DestinationAddressStart, DestinationAddressPrefixLength) values ("edit=0","FW_FILTER_FWD_SERVICE", 1, "DROP", "ANY", "INSECURE", 1, "fe80::",10);

-- $IPTABLES_BIN  -A FORWARD -i <wan> -s fe80::/10 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, SourceAddressType, SourceAddressStart,SourceAddressPrefixLength) values ("edit=0","FW_FILTER_FWD_SERVICE", 1, "DROP", "INSECURE", "ANY", 1, "fe80::",10);

-- $IPTABLES_BIN  -A FORWARD -s ff00::/8 -j DROP 
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, SourceAddressType, SourceAddressStart,SourceAddressPrefixLength) values ("edit=0","FW_FILTER_FWD_SERVICE", 1, "DROP", "ANY", "ANY", 1, "ff00::",8);

-- $IPTABLES_BIN  -A FORWARD -j fwForwardAccept
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_SERVICE", 1, "fwForwardAccept", "ANY", "ANY");

-- $IPTABLES_BIN  -A FORWARD -j fwForwardDrop
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_SERVICE", 1, "fwForwardDrop", "ANY", "ANY");



-- fwLanMulticastFilter  - Filter for multicast packet from LAN to WAN
--
-- $IPTABLES_BIN  -A fwLanMulticastFilter -d ff08::/16 -j ACCEPT
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, DestinationAddressType, DestinationAddressStart, DestinationAddressPrefixLength)
 values ("edit=0","fwLanMulticastFilter", 1, "ACCEPT", "ANY", "ANY", 1, "ff08::",16);

-- $IPTABLES_BIN  -A fwLanMulticastFilter -d ff0e::/16 -j ACCEPT
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType,  DestinationAddressType, DestinationAddressStart, DestinationAddressPrefixLength) values ("edit=0","fwLanMulticastFilter", 1, "ACCEPT", "ANY", "ANY", 1, "ff0e::",16);

-- $IPTABLES_BIN  -A fwLanMulticastFilter -d ff00::/8 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType,
DestinationAddressType, DestinationAddressStart, DestinationAddressPrefixLength)
values ("edit=0","fwLanMulticastFilter", 1, "DROP", "ANY", "ANY", 1, "ff00::",8);


-- fwForwardFilter - FORWARD filter

-- $IPTABLES_BIN  -A fwForwardFilter -i <lan> -o <wan> -j fwForwardOut
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_FLTR", 1, "fwForwardOut", "SECURE", "INSECURE"); 

-- $IPTABLES_BIN  -A fwForwardFilter -i <wan> -o <lan> -j fwForwardIn
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_FLTR", 1, "fwForwardIn", "INSECURE", "SECURE");

-- $IPTABLES_BIN  -A fwForwardFilter -i <isatap+> -o <isatap+> -j ACCEPT
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_FLTR", 1, "ACCEPT", "SECURE", "SECURE"); 


-- fwForwardOut - For Packets being forwarded to internet

--$IPTABLES_BIN -A fwForwardOut  -j SECURE_INSECURE 
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_OUT", 1, "SECURE_INSECURE", "ANY", "ANY"); 

--$IPTABLES_BIN  -A fwForwardOut  -p udp -m udp --dport 53 -j ACCEPT
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","FW_FILTER_FWD_OUT", 1, "ACCEPT", "ANY", "ANY", 17, 53);

-- $(IPTABLES_BIN -A fwForwardOut  -j fwLanMulticastFilter
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType)   values ("edit=0","FW_FILTER_FWD_OUT", 1, "fwLanMulticastFilter", "ANY", "ANY");

-- $IPTABLES_BIN  -A fwForwardOut  -s fec0::/10 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, SourceAddressType, SourceAddressStart,SourceAddressPrefixLength) values ("edit=0","FW_FILTER_FWD_OUT", 1, "DROP", "ANY", "ANY", 1, "fec0::",10);


-- fwForwardIn - For Packets being forwarded from internet

-- $IPTABLES_BIN  -A fwForwardIn  -j INSECURE_SECURE
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_FWD_IN", 1, "INSECURE_SECURE", "ANY", "ANY"); 

-- $IPTABLES_BIN  -A fwForwardIn  -d fec0::/10 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, DestinationAddressType, DestinationAddressStart,DestinationAddressPrefixLength) values ("edit=0","FW_FILTER_FWD_IN", 1, "DROP", "ANY", "ANY", 1, "fec0::",10);


-- fwWanInFilter - Packets from the WAN network to the BOX.

-- $IPTABLES_BIN  -A fwWanInFilter  -i WAN -j fwWanInFilter2
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","fwWanInFilter", 1, "fwWanInFilter2", "INSECURE", "ANY");

-- fwWanInFilter2

-- $IPTABLES_BIN  -A fwWanInFilter2  -j icmpv6WanInputFilter
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","fwWanInFilter2", 1, "icmpv6WanInputFilter", "ANY", "ANY");

-- $IPTABLES_BIN  -A fwWanInFilter2  -s ff00::/8 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, SourceAddressType, SourceAddressStart,SourceAddressPrefixLength) values ("edit=0","fwWanInFilter2", 1, "DROP", "ANY", "ANY", 1, "ff00::",8);


-- fwLanInFilter - Packets from the LAN network to the BOX.
--
-- $IPTABLES_BIN  -A fwLanInFilter -i $LAN_IF  -j ACCEPT
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","FW_FILTER_IN_FILTER", 1, "ACCEPT", "SECURE", "ANY");


-- WAN ICMPv6 filter 


-- $IPTABLES_BIN -A  icmpv6WanInputFilter -p icmpv6 -m icmpv6 --icmpv6-type 1/0 -j ACCEPT
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","icmpv6WanInputFilter", 1, "ACCEPT", "ANY", "ANY", 58, "1");

-- $IPTABLES_BIN -A  icmpv6WanInputFilter -p icmpv6 -m icmpv6 --icmpv6-type 2/0 -j ACCEPT
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","icmpv6WanInputFilter", 1, "ACCEPT", "ANY", "ANY", 58, "2");

-- $IPTABLES_BIN -A  icmpv6WanInputFilter -p icmpv6 -m icmpv6 --icmpv6-type 3/0 -j ACCEPT
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","icmpv6WanInputFilter", 1, "ACCEPT", "ANY", "ANY", 58, "3");

-- $IPTABLES_BIN -A  icmpv6WanInputFilter -p icmpv6 -m icmpv6 --icmpv6-type 4/0 -j ACCEPT
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","icmpv6WanInputFilter", 1, "ACCEPT", "ANY", "ANY", 58, " 4");

-- $IPTABLES_BIN -A  icmpv6WanInputFilter -p icmpv6 -m icmpv6 --icmpv6-type 130/0 -j ACCEPT
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","icmpv6WanInputFilter", 1, "ACCEPT", "ANY", "ANY", 58, " 130");

-- $IPTABLES_BIN -A  icmpv6WanInputFilter -p icmpv6 -m icmpv6 --icmpv6-type 131/0 -j ACCEPT
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","icmpv6WanInputFilter", 1, "ACCEPT", "ANY", "ANY", 58, " 131");

-- $IPTABLES_BIN -A  icmpv6WanInputFilter -p icmpv6 -m icmpv6 --icmpv6-type 132/0 -j ACCEPT
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","icmpv6WanInputFilter", 1, "ACCEPT", "ANY", "ANY", 58, " 132");


-- ipv6HdrFilter - IPv6 header filtering


-- $IPTABLES_BIN  -A ipv6HdrFilter -m ipv6header --header route --soft -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Ipv6HeaderType) values ("edit=0","ipv6HdrFilter", 1, "DROP", "ANY", "ANY", " route");

-- $IPTABLES_BIN  -A ipv6HdrFilter -m ipv6header --header frag --soft -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Ipv6HeaderType) values ("edit=0","ipv6HdrFilter", 1, "DROP", "ANY", "ANY", " frag");


-- Ultility Chains


-- $IPTABLES_BIN  -A fwSysInputDrop -j LOG --log-prefix "SYSTEM_INPUT"   \
    --log-level=debug 
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, LogPrefix, LogLevel) values ("edit=0","fwSysInputDrop", 1, "LOG", "ANY", "ANY", "SYSTEM_INPUT", 1);

-- $IPTABLES_BIN  -A fwSysInputDrop -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","fwSysInputDrop", 1, "DROP", "ANY", "ANY");


-- $IPTABLES_BIN  -A fwForwardDrop -j LOG --log-prefix "SYSTEM_FORWARD"  \
    --log-level=debug 
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, LogPrefix, LogLevel) values ("edit=0","fwForwardDrop", 1, "LOG", "ANY", "ANY", "SYSTEM_FORWARD", 1);

-- $IPTABLES_BIN  -A fwForwardDrop -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","fwForwardDrop", 1, "DROP", "ANY", "ANY");


-- $IPTABLES_BIN  -A fwForwardAccept -i <lan> -o <wan> -j DEFAULT_POLICY
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","fwForwardAccept", 1, "DEFAULT_POLICY", "SECURE", "INSECURE");


-- $IPTABLES_BIN  -A fwForwardAccept -i <lan> -o <isatap+> -j ACCEPT
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType) values ("edit=0","fwForwardAccept", 1, "ACCEPT", "SECURE", "SECURE-ISATAP");



-- LAN->WAN ICMPv6 filter 

-- $IPTABLES_BIN -A  icmpv6ForwardFilter -p icmpv6 -m icmpv6 --icmpv6-type 128/0 -j RETURN
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","icmpv6ForwardFilter", 1, "RETURN", "ANY", "ANY", 58, " 128");

-- $IPTABLES_BIN -A  icmpv6ForwardFilter -p icmpv6 -m icmpv6 --icmpv6-type 129/0 -j RETURN
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","icmpv6ForwardFilter", 1, "RETURN", "ANY", "ANY", 58, " 129");

-- $IPTABLES_BIN -A  icmpv6ForwardFilter -p icmpv6 -m icmpv6 --icmpv6-type 1/0 -j ACCEPT
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","icmpv6ForwardFilter", 1, "ACCEPT", "ANY", "ANY", 58, " 1");


-- $IPTABLES_BIN -A  icmpv6ForwardFilter -p icmpv6 -m icmpv6 --icmpv6-type 2/0 -j ACCEPT
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","icmpv6ForwardFilter", 1, "ACCEPT", "ANY", "ANY", 58, "2");


-- $IPTABLES_BIN -A  icmpv6ForwardFilter -p icmpv6 -m icmpv6 --icmpv6-type 3/0 -j ACCEPT
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","icmpv6ForwardFilter", 1, "ACCEPT", "ANY", "ANY", 58, "3");


-- $IPTABLES_BIN -A  icmpv6ForwardFilter -p icmpv6 -m icmpv6 --icmpv6-type 4/0 -j ACCEPT
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","icmpv6ForwardFilter", 1, "ACCEPT", "ANY", "ANY", 58, "4");


-- $IPTABLES_BIN -A  icmpv6ForwardFilter -p icmpv6 -m icmpv6 --icmpv6-type 130/0 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","icmpv6ForwardFilter", 1, "DROP", "ANY", "ANY", 58, "130");

-- $IPTABLES_BIN -A  icmpv6ForwardFilter -p icmpv6 -m icmpv6 --icmpv6-type 131/0 -j  DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","icmpv6ForwardFilter", 1, "DROP", "ANY", "ANY", 58, "131");

-- $IPTABLES_BIN -A  icmpv6ForwardFilter -p icmpv6 -m icmpv6 --icmpv6-type 132/0 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","icmpv6ForwardFilter", 1, "DROP", "ANY", "ANY", 58, "132");


-- $IPTABLES_BIN -A  icmpv6ForwardFilter -p icmpv6 -m icmpv6 --icmpv6-type 133/0 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","icmpv6ForwardFilter", 1, "DROP", "ANY", "ANY", 58, "133");

-- $IPTABLES_BIN -A  icmpv6ForwardFilter -p icmpv6 -m icmpv6 --icmpv6-type 134/0 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","icmpv6ForwardFilter", 1, "DROP", "ANY", "ANY", 58, "134");


-- $IPTABLES_BIN -A  icmpv6ForwardFilter -p icmpv6 -m icmpv6 --icmpv6-type 135/0 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","icmpv6ForwardFilter", 1, "DROP", "ANY", "ANY", 58, "135");

-- $IPTABLES_BIN -A  icmpv6ForwardFilter -p icmpv6 -m icmpv6 --icmpv6-type 136/0 -j  DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","icmpv6ForwardFilter", 1, "DROP", "ANY", "ANY", 58, "136");

-- $IPTABLES_BIN -A  icmpv6ForwardFilter -p icmpv6 --icmpv6-type 139 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","icmpv6ForwardFilter", 1, "DROP", "ANY", "ANY", 58, "139");

-- $IPTABLES_BIN -A  icmpv6ForwardFilter -p icmpv6 --icmpv6-type 140 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","icmpv6ForwardFilter", 1, "DROP", "ANY", "ANY", 58, "140");


-- $IPTABLES_BIN -A  icmpv6ForwardFilter -p icmpv6 --icmpv6-type 138 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","icmpv6ForwardFilter", 1, "DROP", "ANY", "ANY", 58, "138");


-- $IPTABLES_BIN -A  icmpv6ForwardFilter -p icmpv6 --icmpv6-type redirect -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, Protocol, DestinationPortStart) values ("edit=0","icmpv6ForwardFilter", 1, "DROP", "ANY", "ANY", 58, "redirect");



-- Drop all IPv4 mapped IPv6 addresses.

-- $IPTABLES_BIN  -A ipv6BadAddrs -s 0:0:0:0:0:0:ffff::/96 -j DROP 
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, SourceAddressType, SourceAddressStart,SourceAddressPrefixLength) values ("edit=0","ipv6BadAddrs", 1, "DROP", "ANY", "ANY", 1, "0:0:0:0:0:0:ffff::",96);

-- $IPTABLES_BIN  -A ipv6BadAddrs -d 0:0:0:0:0:0:ffff::/96 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, DestinationAddressType, DestinationAddressStart,DestinationAddressPrefixLength) values ("edit=0","ipv6BadAddrs", 1, "DROP", "ANY", "ANY", 1, "0:0:0:0:0:0:ffff::",96);

-- $IPTABLES_BIN  -A ipv6BadAddrs -s ::224.0.0.0/100 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, SourceAddressType, SourceAddressStart,SourceAddressPrefixLength) values ("edit=0","ipv6BadAddrs", 1, "DROP", "ANY", "ANY", 1, "::224.0.0.0",100);

-- $IPTABLES_BIN  -A ipv6BadAddrs -d ::224.0.0.0/100 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, DestinationAddressType, DestinationAddressStart,DestinationAddressPrefixLength) values ("edit=0","ipv6BadAddrs", 1, "DROP", "ANY", "ANY",1, "::224.0.0.0",100);

-- $IPTABLES_BIN  -A ipv6BadAddrs -s ::127.0.0.0/104 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, SourceAddressType, SourceAddressStart,SourceAddressPrefixLength) values ("edit=0","ipv6BadAddrs", 1, "DROP", "ANY", "ANY", 1, "::127.0.0.0",104); 
-- $IPTABLES_BIN  -A ipv6BadAddrs -d ::127.0.0.0/104 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, DestinationAddressType, DestinationAddressStart,DestinationAddressPrefixLength) values ("edit=0","ipv6BadAddrs", 1, "DROP", "ANY", "ANY", 1, "::127.0.0.0",104);

-- $IPTABLES_BIN  -A ipv6BadAddrs -s ::0.0.0.0/104 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, SourceAddressType, SourceAddressStart,SourceAddressPrefixLength) values ("edit=0","ipv6BadAddrs", 1, "DROP", "ANY", "ANY", 1, "::0.0.0.0",104);

-- $IPTABLES_BIN  -A ipv6BadAddrs -d ::0.0.0.0/104 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, DestinationAddressType, DestinationAddressStart,DestinationAddressPrefixLength) values ("edit=0","ipv6BadAddrs", 1, "DROP", "ANY", "ANY", 1, "::0.0.0.0",104);

-- $IPTABLES_BIN  -A ipv6BadAddrs -s ::255.0.0.0/104 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, SourceAddressType, SourceAddressStart,SourceAddressPrefixLength) values ("edit=0","ipv6BadAddrs", 1, "DROP", "ANY", "ANY", 1, "::255.0.0.0",104);

-- $IPTABLES_BIN  -A ipv6BadAddrs -d ::255.0.0.0/104 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, DestinationAddressType, DestinationAddressStart,DestinationAddressPrefixLength) values ("edit=0","ipv6BadAddrs", 1, "DROP", "ANY", "ANY", 1, "::255.0.0.0",104);

-- $IPTABLES_BIN  -A ipv6BadAddrs -s ::0.0.0.0/96 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, SourceAddressType, SourceAddressStart,SourceAddressPrefixLength) values ("edit=0","ipv6BadAddrs", 1, "DROP", "ANY", "ANY", 1, "::0.0.0.0",96);

-- $IPTABLES_BIN  -A ipv6BadAddrs -d ::0.0.0.0/96 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, DestinationAddressType, DestinationAddressStart,DestinationAddressPrefixLength) values ("edit=0","ipv6BadAddrs", 1, "DROP", "ANY", "ANY", 1, "::0.0.0.0",96);

-- $IPTABLES_BIN  -A ipv6BadAddrs -s 2002:e000::/20 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, SourceAddressType, SourceAddressStart,SourceAddressPrefixLength) values ("edit=0","ipv6BadAddrs", 1, "DROP", "ANY", "ANY", 1, "2002:e000::",20);

-- $IPTABLES_BIN  -A ipv6BadAddrs -d 2002:e000::/20 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, DestinationAddressType, DestinationAddressStart,DestinationAddressPrefixLength) values ("edit=0","ipv6BadAddrs", 1, "DROP", "ANY", "ANY", 1, "2002:e000::",20);

-- $IPTABLES_BIN  -A ipv6BadAddrs -s 2002:7f00::/24 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, SourceAddressType, SourceAddressStart,SourceAddressPrefixLength) values ("edit=0","ipv6BadAddrs", 1, "DROP", "ANY", "ANY", 1, "2002:7f00::",24);

-- $IPTABLES_BIN  -A ipv6BadAddrs -d 2002:7f00::/24 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, DestinationAddressType, DestinationAddressStart,DestinationAddressPrefixLength) values ("edit=0","ipv6BadAddrs", 1, "DROP", "ANY", "ANY", 1, "2002:7f00::",24);

-- $IPTABLES_BIN  -A ipv6BadAddrs -s 2002:0000::/24 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, SourceAddressType, SourceAddressStart,SourceAddressPrefixLength) values ("edit=0","ipv6BadAddrs", 1, "DROP", "ANY", "ANY", 1, "2002:0000::",24);

-- $IPTABLES_BIN  -A ipv6BadAddrs -d 2002:0000::/24 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, DestinationAddressType, DestinationAddressStart,DestinationAddressPrefixLength) values ("edit=0","ipv6BadAddrs", 1, "DROP", "ANY", "ANY", 1, "2002:0000::",24);

-- $IPTABLES_BIN  -A ipv6BadAddrs -s 2002:ff00::/24 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, SourceAddressType, SourceAddressStart,SourceAddressPrefixLength) values ("edit=0","ipv6BadAddrs", 1, "DROP", "ANY", "ANY", 1, "2002:ff00::",24);

-- $IPTABLES_BIN  -A ipv6BadAddrs -d 2002:ff00::/24 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, DestinationAddressType, DestinationAddressStart,DestinationAddressPrefixLength) values ("edit=0","ipv6BadAddrs", 1, "DROP", "ANY", "ANY", 1, "2002:ff00::",24);

-- $IPTABLES_BIN  -A ipv6BadAddrs -s 2002:0a00::/24 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, SourceAddressType, SourceAddressStart,SourceAddressPrefixLength) values ("edit=0","ipv6BadAddrs", 1, "DROP", "ANY", "ANY", 1, "2002:0a00::",24);

-- $IPTABLES_BIN  -A ipv6BadAddrs -d 2002:0a00::/24 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, DestinationAddressType, DestinationAddressStart,DestinationAddressPrefixLength) values ("edit=0","ipv6BadAddrs", 1, "DROP", "ANY", "ANY", 1, "2002:0a00::",24);

-- $IPTABLES_BIN  -A ipv6BadAddrs -s 2002:ac10::/28 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, SourceAddressType, SourceAddressStart,SourceAddressPrefixLength) values ("edit=0","ipv6BadAddrs", 1, "DROP", "ANY", "ANY", 1, "2002:ac10::",24);

-- $IPTABLES_BIN  -A ipv6BadAddrs -d 2002:ac10::/28 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, DestinationAddressType, DestinationAddressStart,DestinationAddressPrefixLength) values ("edit=0","ipv6BadAddrs", 1, "DROP", "ANY", "ANY",1, "2002:ac10::",24);

-- $IPTABLES_BIN  -A ipv6BadAddrs -s 2002:c0a8::/32 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, SourceAddressType, SourceAddressStart,SourceAddressPrefixLength) values ("edit=0","ipv6BadAddrs", 1, "DROP", "ANY", "ANY", 1, "2002:c0a8::",32);

-- $IPTABLES_BIN  -A ipv6BadAddrs -d 2002:c0a8::/32 -j DROP
insert into FirewallRules6 (_metadata, RuleType, Status, Action, FromZoneType, ToZoneType, DestinationAddressType, DestinationAddressStart,DestinationAddressPrefixLength) values ("edit=0","ipv6BadAddrs", 1, "DROP", "ANY", "ANY", 1, "2002:c0a8::",32);
CREATE TABLE ddns
(
        OID integer ,
        interfaceName text NOT NULL,
        ddnsService integer NOT NULL,
        hostname text ,
        username text ,
        password text ,
        timePeriod integer ,
        wildflag integer 
)
;

CREATE TABLE ddnsStatus
(
        OID integer ,
        interfaceName text NOT NULL,
        Status text , 
        orayType text ,
        orayDomain text 
)
;

insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"ddns", 0, 1,1,1);
insert into stringsMap values ("DDNS_CONFIG_FAILED","DDNS Configuration Failed");
CREATE TABLE LanGroup
(
    OID           integer,
    GroupName     text NOT NULL,
    PRIMARY KEY   (GroupName)
)
;

CREATE TABLE LanHost
(
    OID           integer,
    GroupName     text NOT NULL,
    IPAddr        text,
    PRIMARY KEY   (IPAddr)
    FOREIGN KEY (GroupName) REFERENCES LanGroup (GroupName)
)
;

CREATE TABLE AvailableLanHost
(
    OID           integer,
    Name          text NOT NULL,
    IPAddr        text,
    MacAddr       text NOT NULL,
    PRIMARY KEY   (IPAddr)
)
;

insert into stringsMap values ("LAN_HOST_CONFIG_FAILED","LAN Hosts Configuration Failed");
insert into stringsMap values ("LAN_GROUPS_CONFIG_FAILED","LAN groups Configuration Failed");
insert into stringsMap values ("GROUPS_NAME_ALREADY_EXIXSTS","Groups Name already Exists");
-- SQL table for ifStatic back-end component.

-- ifStatic table
CREATE TABLE ifStatic
    (
    OID                      integer,           -- OID
    LogicalIfName            text     NOT NULL, -- interface names like 'lan1', 'wan2', etc.
    AddressFamily            integer NOT NULL,  -- address family.
    IspName                  integer,           -- name of the service provider
    StaticIp                 text     NOT NULL, -- ip which we want to assign
    NetMask                  text     NOT NULL, -- net mask for the above IP
    PrefixLength             integer          , -- address family.
    Gateway                  text     NOT NULL, -- gateway address
    PrimaryDns               text,              -- if we want to specify our DNS
    SecondaryDns             text,              -- if we want to specify our DNS
    -- add your new entries here

    PRIMARY KEY (LogicalIfName, AddressFamily),                -- primary key
    FOREIGN KEY (LogicalIfName) REFERENCES networkInterface (LogicalIfName)
    );

CREATE TABLE ifStaticAliasAddrTbl
    (
    LogicalIfName            text    NOT NULL, -- interface names like 'lan1', 'wan2', etc.
    AddressFamily            integer NOT NULL, -- address family.        
    StaticIp                 text    NOT NULL, -- ip which we want to assign
    NetMask                  text            , -- net mask for the above IP
    PrefixLength             integer         , -- Prefix Length.    
    PRIMARY KEY (LogicalIfName, AddressFamily, StaticIp),-- primary key
    FOREIGN KEY (LogicalIfName) REFERENCES networkInterface (LogicalIfName)    
    );

insert into stringsMap values ("IP_CONFLICTING_WITH_LAN_IPADDR","Configured DMZ IP address conflicting with LAN IP address");
insert into stringsMap values ("IP_CONFLICTING_WITH_DMZ_IPADDR","Configured LAN IP address conflicting with DMZ IP address");
CREATE TABLE ZoneTypes
(
  OID integer NULL,
  ZoneType text NOT NULL, -- SECURE/INSECURE/PUBLIC
  zoneIfGroupId integer,
  PRIMARY KEY (ZoneType)
)
;

CREATE TABLE Zones
(
  OID integer NULL,
  ZoneType text NOT NULL, --  SECURE/INSECURE/PUBLIC
  ZoneLogicalIfName text NOT NULL, --  LAN/WAN1/WAN2/DMZ
  -- this mark is for FW
  LogicalIfNameMark integer NULL,           
  PRIMARY KEY (ZoneLogicalIfName),
  FOREIGN KEY (ZoneType) REFERENCES ZoneTypes (ZoneType),
  FOREIGN KEY (ZoneLogicalIfName) REFERENCES networkInterface (logicalIfName)
)
;

CREATE TABLE ZoneAttr
(
  OID              integer NULL,
  LogicalIfName    text NOT NULL, --  LAN/WAN1/WAN2/DMZ
  ZoneAssTableId   integer,
  ZoneMark         integer, 
  ZonePBMark       integer, 
  PRIMARY KEY      (LogicalIfName),
  FOREIGN KEY      (LogicalIfName) REFERENCES Zones (ZoneLogicalIfName)
)
;
-- This file contains tables related to Bandwidth Limit Functionality.
   -- 1. BandWidthProfileStatus Table
   -- 2. BandWidthProfile Table
   -- 3. TrafficSelector Table


-- Enable/Disable BW Profiles
-- SPECS 
-- 1. This table will hold status of Bandwidth Profile functionality,whether its
--    enable or disable.
      -- When enable, default qdisc/class/filter will get added in backend. And
      -- for that event will be generated. And it will call bandwidthInit. 

-- GUI Changes proposed.
-- 1. Add a radio button called Enable/Disable on bandwidthProfiles.htm

CREATE TABLE BandWidthProfileStatus
(
    OID     integer NULL,
    Status  boolean NOT NULL -- Status of Bandwidth Profile functionality.Defaultly it should be enable.
);


-- BandWidth Profiles table
-- SPECS
    -- 1. This table is for Adding bandwidth profiles.
       -- When user adds bandwidth profile, the database will get the values
       -- entered by user. No event will get generated here.  

--GUI Changes proposed.
    -- 1.  Add radio and priority radio buttons.If rate is selected, grayout
    --     priority dropdown. If priority is selected, grayout Min. rate/Max. rate.
    -- 2.  Remove Direction dropdown.
    -- 3.  Add WAN Iface dropdown as in FVX538 (for Auto failover and load balancing mode).

CREATE TABLE BandWidthProfile
(
    OID         integer     NULL,
    ProfileName text    NOT NULL,   -- Unique Profile Name PRIMARY KEY.
    ProfileType boolean NOT NULL,   -- It can be rate(value - 0)/priority(value - 1) according to 
                                    -- radio button.When rate is enabled, allow to add MaximumRate and
                                    -- MinimumRate and disallow Priority file  
                                    -- If Priority is enabled disallow to add MaximumRate and 
                                    -- MinimumRate and allow to select priority from 
                                    -- Priority dropdown.
    Priority    integer     NULL,   -- Priority (Dropdown).
                                    -- (LOW(10% - value - 1)/MEDIUM(30% - value -2 )/HIGH(60% - value 3))
    MinimumRate integer     NULL,   -- Minimum Bandwidth provided by user. 
    MaximumRate integer     NULL,   -- Maximum Bandwidth provided by user.
    WANID       text        NULL,   -- WAN interface selection if DUT is in 
                                    -- Auto Failover or Load balancing mode.
                                    -- Default WAN interface should be WAN1.    
    DscpMark    text    NOT NULL,   -- marking value, which should be used for 
                                    -- dscp bits for classification. DscpMark value should be taken
                                    -- from DscpClassList table as 
                                    -- explained for ClassId
    ClassId     integer NOT NULL,   -- ClassId values should be taken from DscpClassList,
                                    -- for first profile, first set of DscpMark and ClassId.
                                    -- For second profile second set of values, like that.     

    -- add your new entries here.

    PRIMARY KEY (ProfileName)

);


CREATE TABLE BandWidthProfileSpeed
(
    OID         integer     NULL,
    Wan1Speed   NOT        NULL,
    Wan2Speed   NOT        NULL
);

insert into stringsMap values ("BWPROFILE_CONFIG_FAILED", "Bandwidth Profile(s) Configuration Failed");
insert into stringsMap values ("BWPROFILESTATUS_CONFIG_FAILED", "Bandwidth Profile Status Configuration Failed");
insert into stringsMap values ("BWPROFILE_DSCP_CONFIG_FAILED", "Bandwidth Profiles reached to maximum limit(15).");
insert into stringsMap values ("BWPROFILE_USE_DELETE_FAILED", "Bandwidth Profile(s) Delete Failed as it is being used by traffic selector");
insert into stringsMap values ("BWPROFILE_DELETE_FAILED", "Bandwidth Profile(s) Delete Failed");
insert into stringsMap values ("BWPROFILE_INUSE_EDIT_FAILED", "Bandwidth profile edit failed as it is being used by traffic selector");

-- Traffic Selector table
-- SPECS
    -- 1. This table has fields for selecting particular profile from added
    --    profiles. And apply that with proper iptables rule with it.
    -- 2. Event will get generated on the applying on this page.And according to
    --    selected BandWidth profile, the tc rules will get applied and
    --    according to values in Traffic selector table iptables rules will get
    --    applied.

-- GUI Changes Proposed.    
    -- 1. Remove Service & IP radio button. Only Service dropdown is needed.
    -- 2. radio button for IP is needed.

CREATE TABLE TrafficSelector
(
    OID         integer     NULL,
    ProfileName text        NULL,   -- FOREIGN KEY from BandWidthProfile table.
    ServiceName text        NULL,   -- FOREIGN KEY from Services table.
    MatchType   integer NOT NULL,   -- The Values of the MatchType are 
                                    -- IP(0) /MAC(1) /Port number(2) / VLAN Name(3).
    IPAddress   text        NULL,   -- Ip Address given by user.
    SubnetMask  text        NULL,   -- Subnet Mask.
    MACAddress  text        NULL,   -- Mac Address.
    PortName    text        NULL,   -- Port Name (Drop down).
    VlanId      integer     NULL,   -- Vlan Id ( Drop down).
    DscpValue   integer     NULL,   -- Dscp Value set
    VapName     text        NULL,   -- Access Point Name

    -- add your new entries here.

--foreign keys 

    FOREIGN KEY (ProfileName) REFERENCES BandWidthProfile (ProfileName), 
    FOREIGN KEY (ServiceName) REFERENCES Services (ServiceName),
    FOREIGN KEY (PortName) REFERENCES VlanPortMgmt (PortName)
);

insert into stringsMap values ("TRAF_SELECTOR_CONFIG_FAILED", "Traffic Selector Configuration Failed");
insert into stringsMap values ("TRAF_SELECTOR_DELETE_FAILED", "Traffic Selector(s) Delete Failed");

CREATE TABLE DscpClassList
(
    OID         integer     NULL,
    DscpMark    text    NOT NULL,
    ClassId     integer unique NOT NULL,
    Status      boolean NOT NULL,

    PRIMARY KEY (DscpMark)
);

insert into tableDefaults values ("DscpClassList", "Status", "0", "", "");
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"BandWidthProfile",0,1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"BandWidthProfileStatus",0,1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"TrafficSelector",0,1,1,1);
-- switchdevice Table
--

CREATE TABLE SwitchDevice
(
    OID               integer   NULL,
    SwitchDeviceName  text      NOT NULL,
	PRIMARY KEY (SwitchDeviceName)
)
;

CREATE TABLE portStats
(
OID integer NULL,
portId  integer,
TxBytes NUMERIC DEFULT ZERO,
TxDropPkts NUMERIC DEFULT ZERO,
RxBytes NUMERIC DEFULT ZERO,
RxDropPkts NUMERIC DEFULT ZERO,
PRIMARY KEY(portId)
)
;

-- PowerModeConf - table for power mode configuration
CREATE TABLE PowerModeConf
    (
    PowerMode  Boolean NOT NULL,         -- triggers the backend, set power mode
    LengthMode Boolean NOT NULL         -- triggers the backend, set length mode
    );

INSERT INTO dbUpdateRegisterTbl VALUES ("UMI_COMP_PLATFORM", 0, 1, "PowerModeConf", 0, 1, 1, 0);
INSERT INTO dbUpdateRegisterProgram VALUES ("/pfrm2.0/bin/powerModeConfig", 1, 1, "PowerModeConf", 0, 1, 1, 0);



-- vlan Table
--
-- This table will hold vlan information which includes vlanId and vlan membership info.
-- 
-- Cases to be considered:
-- 1) If creating a new vlan where no port membership exists, use portName="dummy"
--    which will be ignored by the backend from configurations.
-- 2) when deleting a vlan, delete all the associated port entries and not just
--    the "dummy" entry to deconfigure all the ports on that vlan.
-- 3) Mode change in VlanPortMgmt requires an update on this table as well to change 
--    the memberships as per the mode. eg: trunk -> access mode change
-- 4) when deleting vlan, make sure that there are no ports currently using it
--    to avoid inconsitencies in frontend

CREATE TABLE vlan
(
    OID             integer   NULL,
    portName        text      NOT NULL,      --  PortLogicalName in VlanPortMgmt
	vlanId          integer   NOT NULL,      --  port is member of vlanID (Range: 1-4094) 
    vlanName        text      NOT NULL,      --  vlan name needed only for display
	PRIMARY KEY (vlanId, portName)
)
;

-- vlan table default entries
insert into tableDefaults values ("vlan","vlanId","","1","4094");
insert into stringsmap values ("VLAN_NAME_ALREADY_IN_USE","VLAN name already in use");

-- vlan status table
--
-- This table will hold vlan state information.
-- 
-- Cases to be Considered:
-- (none)
--

CREATE TABLE VlanStatus
(
    OID             integer   NULL,
    VlanState       boolean   NOT NULL,    -- vlanState is either Enable/Disable
    PRIMARY KEY (VlanState)
)
;

-- VlanPortMgmt table
--
-- This table will hold all port specific vlan information.
--
-- Backend Values:
-- Field            GUI          Backend Value
-- PortVlanMode   Access(Mode)         0
-- PortVlanMode   General(Mode)        1
-- PortVlanMode   Trunk(Mode)          2
-- PortType       (none)         "BRIDGE"/<ANY_STRING>
--
-- Cases to be considered:
-- 1) If Port is configured as access port(Mode: Access), then VLAN Membership
--    section should be updated to use only PVID
-- 2) The backend configuration is applied only if the portType=="BRIDGE" which
--    indicates that the port is part of the bridge (bdg) and be configured.
--
-- Description regarding the modes:
-- Access Mode : In this mode, the interface accepts untagged packets and can 
--               be a member of only one vlan, the DefaultPVID vlan.
-- Trunk Mode  : In this mode, the interface will accept only tagged packets and
--               can be a member of multiple vlans.
-- General Mode: In this mode, the interface will accept both tagged, untagged 
--               packets and can be a member of multiple vlans with a check that
--               untagged packets are handled on vlan matching the port's PVID.

CREATE TABLE VlanPortMgmt
(
    OID               integer   NULL,
    PortNumber        integer   NOT NULL,      --  unique port number
	PortName          text      NOT NULL,      --  Used for display
    PortLogicalName   text      NOT NULL,      --  Logical portName used in vlan table
    PortType          text      NOT NULL,      --  Port is part of a BRIDGE (or) ...
    PhyInterfaceName  text      NOT NULL,      --  physical interface attached to this port
	PortVlanMode      integer   NOT NULL,      --  Indicates access/general/trunk modes.
    DefaultPVID       integer   NOT NULL,      --  Indicates PVID(1-4094) field. 
	PRIMARY KEY (PortNumber)
)
;

-- VlanPortMgmt table default entries
insert into tableDefaults values ("VlanPortMgmt","portVlanMode", "","0","2");

-- comps. to be notified on updates
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM", 0, 1, "vlan", 0, 1, 1, 1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM", 0, 1, "VlanStatus", 0, 1, 1, 1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM", 0, 1, "VlanPortMgmt", 0, 1, 1, 1);

-- stringIds used for GUI message mapping
insert into stringsmap values ("VLAN_ADD_CONFIG_FAILED","Add VLAN configuration failed.");
insert into stringsmap values ("VLAN_EDIT_CONFIG_FAILED","Edit VLAN configuration failed.");
insert into stringsmap values ("VLAN_DELETE_CONFIG_FAILED","Delete VLAN configuration failed.");
insert into stringsmap values ("VLAN_STATUS_CONFIG_FAILED","VLAN configuration failed.");
insert into stringsmap values ("PORT_VLAN_CONFIG_FAILED","Port VLAN configuration failed");
insert into stringsmap values ("VLAN_CONFIG_FAILED","VLAN configuration failed");
insert into stringsmap values ("VLAN_CONFIG_IN_USE","VLANID is in use");
insert into stringsmap values ("VLAN_UPDATE_CONFIG_FAILED","VLAN Configuration Update Failed");

-- save tables order
-- insert into saveTables (TableName) values("VlanStatus");
-- insert into saveTables (TableName) values("VlanPortMgmt");
-- insert into saveTables (TableName) values("vlan");

CREATE TABLE interVLANRouting
(
    vlanId  integer NOT NULL,
    ifName  text    NOT NULL,    
    status  boolean NOT NULL
);


-- Port Mgmt Table
--
-- This table contains information regarding port management.
--
-- Backend values:
-- Backend Field        GUI            Backend value
--   PortStatus     Enable(Status)           1 
--   PortStatus     Disable(Status)          0
--   AutoNegEnable     AUTO            0(Disable),1(Enable)   
--   PortSpeed        Speed(10Mbps)          10            
--                    Speed(100Mbps)         100
--                    Speed(1000Mbps)        1000
--   DuplexMode     Duplex(Half)             0
--                  Duplex(Full)             1

CREATE TABLE PortMgmt
(
    OID               integer   NULL,
    PortName          text      NOT NULL,   -- Used for display and is Unique.
    PortPhyID         integer   NULL,       -- PhyID of port, filled by backend in default configuration
    InterfaceName     text      NULL,       -- Physical interface Name, filled in default configuration
    PortStatus        boolean   NOT NULL,   -- Port Status (enable/disable)
    AutoNegEnable     boolean   NOT NULL,   -- Auto negotiation enable/disable
    DuplexMode        boolean   NOT NULL,   -- Duplex Mode (HALF/FULL)
    PortSpeed         integer   NULL,       -- port speed (10/100/1000Mbps)
    MacAddress        text      NULL,       -- MAC Address of the port
    DefaultMacAddress text      NULL,       -- Default MAC Address of the port
    SetMacAddress     integer   NULL,      
    Mtu               integer   NULL,       -- Mtu size of the port
	PRIMARY KEY (PortName)
)
;
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM", 0, 1, "PortMgmt", 0, 1, 1, 1);
insert into stringsmap values ("PORT_MGMT_CONFIG_FAILED","Port Management configuration failed.");

-- Port Mirroring Table
--
-- This table contains information regarding port mirroring.
--
-- Backend values:
-- Backend Field               GUI                     Backend value
-- MirroringEnable     enable Port Mirroring           1(Yes), 0(No)   
-- CapturePort         Mirror all LAN Ports to         port1/port2/port3/port4

CREATE TABLE PortMirroring
(
    OID               integer   NULL,
    MirroringEnable   boolean   NOT NULL,
    CapturePort       text      NOT NULL,
	PRIMARY KEY (MirroringEnable)
)
;
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM", 0, 1, "PortMirroring", 0, 1, 1, 1);
insert into tableDefaults values ("PortMirroring","CapturePort","Port1","","");

-- Tables used in TF1's Configurable port management handlers 

CREATE TABLE ConfigPort
(
    OID integer,                    -- OID 
    LogicalIfName text NOT NULL     -- value can be LAN/WAN2/DMZ 
                                    -- Add new entries here  
)
;
insert into stringsMap values ("CONFIGPORT_CONFIG_FAILED","Configuration of PORT is Failed");
insert into stringsMap values ("CONFIG_LAN_IN_USE_BY_BW_TRAFFIC_SELECTOR","Optional Port(LAN) is using by bandwidth profiles traffic selectors");
insert into stringsMap values ("CONFIG_LAN_IN_USE_BY_PORT_MIRRORING","Optional Port(LAN) is in use by port mirroring configuration");
insert into stringsMap values ("ALL_PORTS_DISABLED","Operation Failed. Router cannot be accessible if all ports are disabled.");
CREATE TABLE dot11Card
(
	cardNo  integer NOT NULL,
	cardName  text NOT NULL,
    txEnabled integer NOT NULL,
    rxEnabled integer NOT NULL,
    agcEnabled integer NOT NULL,
    txCancellation integer NOT NULL,
    rxMaxGain integer NOT NULL,
    txLO integer NOT NULL,
    rxLO integer NOT NULL,
	 PRIMARY KEY (cardNo)
)
;



CREATE TABLE dot11Radio
(
	interfaceName  text NOT NULL,
	radioNo  integer NOT NULL,
	cardNo  integer NOT NULL,
	band  text NOT NULL,
	currentChannel  integer NOT NULL,
	configuredChannel  text NOT NULL,
	radioenabled  integer NOT NULL,
	txPower  integer NOT NULL,
	path  integer NOT NULL,
	rogueAPEnabled  integer NOT NULL,
	rxDiversity  integer NOT NULL,
    dothEnabled integer NOT NULL,
    dothMaxPower integer NOT NULL,
    dothMarkDfs integer NOT NULL,
    opMode  text NOT NULL,
	beaconInterval  integer NOT NULL,
	dtimInterval  integer NOT NULL,
	rtsThreshold  integer NOT NULL,
	fragThreshold  integer NOT NULL,
	preambleMode  text NOT NULL,
	rtsCtsProtect  integer NOT NULL,
	shortRetry  integer NOT NULL,
	longRetry  integer NOT NULL,
	uapsd  integer NOT NULL,
    txRate integer NOT NULL,
	PRIMARY KEY (radioNo),
    FOREIGN KEY (cardNo) REFERENCES dot11Card(cardNo)
)
;

insert into tableDefaults values ("dot11Radio","txRate","0","","");

CREATE TABLE dot11Interface
(
	interfaceName  text NOT NULL,
	radioNo  integer NOT NULL,
	vapName  text NOT NULL,
	macAddress  text NOT NULL,
	 PRIMARY KEY (interfaceName),
    FOREIGN KEY (interfaceName) REFERENCES networkInterface(interfaceName),
    FOREIGN KEY (radioNo) REFERENCES dot11Radio(radioNo)
)
;



CREATE TABLE dot11ACL
(
	macAddress  text NOT NULL,
    vapName  text NOT NULL,
    PRIMARY KEY (vapName, macAddress),
    FOREIGN KEY (vapName) REFERENCES dot11VAP(vapName)
)
;


CREATE TABLE dot11VAP
(
	vapName  text NOT NULL,
	vapEnabled  integer NOT NULL,
    opMode  text NOT NULL,
	profileName  text NOT NULL,
	beaconInterval  integer NOT NULL,
	dtimInterval  integer NOT NULL,
	maxClients  integer NOT NULL,
	rate1Mbps  integer NOT NULL,
	rate2Mbps  integer NOT NULL,
	rate5_5Mbps  integer NOT NULL,
	rate6Mbps  integer NOT NULL,
	rate9Mbps  integer NOT NULL,
	rate11Mbps  integer NOT NULL,
	rate12Mbps  integer NOT NULL,
	rate18Mbps  integer NOT NULL,
	rate24Mbps  integer NOT NULL,
	rate36Mbps  integer NOT NULL,
	rate48Mbps  integer NOT NULL,
	rate54Mbps  integer NOT NULL,
	rtsThreshold  integer NOT NULL,
	fragThreshold  integer NOT NULL,
	preambleMode  text NOT NULL,
	rtsCtsProtect  integer NOT NULL,
	shortRetry  integer NOT NULL,
	longRetry  integer NOT NULL,
	txPower  integer NOT NULL,
	dot11Mode  text NOT NULL,
	apIsolation  integer NOT NULL,
    defACLPolicy  text NOT NULL,
    iappEnabled  integer NOT NULL,
    uapsd  integer NOT NULL,
    radioList text, -- we use this only to trigger updates,
                    -- in case radio selection has changed
    txRate integer NOT NULL,
    puren integer NOT NULL,
    PRIMARY KEY (vapName),
    FOREIGN KEY (profileName) REFERENCES dot11Profile(profileName)
)
;

insert into tableDefaults values ("dot11VAP","vapEnabled","1","","");
insert into tableDefaults values ("dot11VAP","trunkPort","0","","");
insert into tableDefaults values ("dot11VAP","beaconInterval","100", "40", "3500");
insert into tableDefaults values ("dot11VAP","dtimInterval","1","1","255");
insert into tableDefaults values ("dot11VAP","rate1Mbps","1","","");
insert into tableDefaults values ("dot11VAP","rate2Mbps","1","","");
insert into tableDefaults values ("dot11VAP","rate5_5Mbps","1","","");
insert into tableDefaults values ("dot11VAP","rate6Mbps","1","","");
insert into tableDefaults values ("dot11VAP","rate9Mbps","1","","");
insert into tableDefaults values ("dot11VAP","rate11Mbps","1","","");
insert into tableDefaults values ("dot11VAP","rate12Mbps","1","","");
insert into tableDefaults values ("dot11VAP","rate18Mbps","1","","");
insert into tableDefaults values ("dot11VAP","rate24Mbps","1","","");
insert into tableDefaults values ("dot11VAP","rate36Mbps","1","","");
insert into tableDefaults values ("dot11VAP","rate48Mbps","1","","");
insert into tableDefaults values ("dot11VAP","rate54Mbps","1","","");
insert into tableDefaults values ("dot11VAP","rtsThreshold","2346","256","2346");
insert into tableDefaults values ("dot11VAP","fragThreshold","2346","257","2346");
insert into tableDefaults values ("dot11VAP","preambleMode","Long","","");
insert into tableDefaults values ("dot11VAP","rtsCtsProtect","0","","");
insert into tableDefaults values ("dot11VAP","shortRetry","16","0","128");
insert into tableDefaults values ("dot11VAP","longRetry","16","0","128");
insert into tableDefaults values ("dot11VAP","txPower","31","1","31");
insert into tableDefaults values ("dot11VAP","dot11Mode","AP","","");
insert into tableDefaults values ("dot11VAP","opMode","b and g","","");
insert into tableDefaults values ("dot11VAP","defACLPolicy","Open","","");
insert into tableDefaults values ("dot11VAP","iappEnabled","0","","");
insert into tableDefaults values ("dot11VAP","bssidSecret","","","");
insert into tableDefaults values ("dot11VAP","radioList","","","");
insert into tableDefaults values ("dot11VAP", "maxClients","8","","");
insert into tableDefaults values ("dot11VAP","uapsd","0","","");
insert into tableDefaults values ("dot11VAP","txRate","0","","");
insert into tableDefaults values ("dot11VAP","puren","0","","");

CREATE TABLE dot11VAPStats
(
	interfaceName  text NOT NULL,
	is_rx_badversion  integer NOT NULL,
	is_rx_tooshort  integer NOT NULL,
	is_rx_wrongbss  integer NOT NULL,
	is_rx_dup  integer NOT NULL,
	is_rx_wrongdir  integer NOT NULL,
	is_rx_mcastecho  integer NOT NULL,
	is_rx_notassoc  integer NOT NULL,
	is_rx_noprivacy  integer NOT NULL,
	is_rx_unencrypted  integer NOT NULL,
	is_rx_wepfail  integer NOT NULL,
	is_rx_decap  integer NOT NULL,
	is_rx_mgtdiscard  integer NOT NULL,
	is_rx_ctl  integer NOT NULL,
	is_rx_beacon  integer NOT NULL,
	is_rx_rstoobig  integer NOT NULL,
	is_rx_elem_missing  integer NOT NULL,
	is_rx_elem_toobig  integer NOT NULL,
	is_rx_elem_toosmall  integer NOT NULL,
	is_rx_elem_unknown  integer NOT NULL,
	is_rx_badchan  integer NOT NULL,
	is_rx_chanmismatch  integer NOT NULL,
	is_rx_nodealloc  integer NOT NULL,
	is_rx_ssidmismatch  integer NOT NULL,
	is_rx_auth_unsupported  integer NOT NULL,
	is_rx_auth_fail  integer NOT NULL,
	is_rx_auth_countermeasures  integer NOT NULL,
	is_rx_assoc_bss  integer NOT NULL,
	is_rx_assoc_notauth  integer NOT NULL,
	is_rx_assoc_capmismatch  integer NOT NULL,
	is_rx_assoc_norate  integer NOT NULL,
	is_rx_assoc_badwpaie  integer NOT NULL,
	is_rx_deauth  integer NOT NULL,
	is_rx_disassoc  integer NOT NULL,
	is_rx_badsubtype  integer NOT NULL,
	is_rx_nobuf  integer NOT NULL,
	is_rx_decryptcrc  integer NOT NULL,
	is_rx_ahdemo_mgt  integer NOT NULL,
	is_rx_bad_auth  integer NOT NULL,
	is_rx_unauth  integer NOT NULL,
	is_rx_badkeyid  integer NOT NULL,
	is_rx_ccmpreplay  integer NOT NULL,
	is_rx_ccmpformat  integer NOT NULL,
	is_rx_ccmpmic  integer NOT NULL,
	is_rx_tkipreplay  integer NOT NULL,
	is_rx_tkipformat  integer NOT NULL,
	is_rx_tkipmic  integer NOT NULL,
	is_rx_tkipicv  integer NOT NULL,
	is_rx_badcipher  integer NOT NULL,
	is_rx_nocipherctx  integer NOT NULL,
	is_rx_acl  integer NOT NULL,
	is_rx_ffcnt  integer NOT NULL,
	is_rx_badathtnl  integer NOT NULL,
	is_tx_nobuf  integer NOT NULL,
	is_tx_nonode  integer NOT NULL,
	is_tx_unknownmgt  integer NOT NULL,
	is_tx_badcipher  integer NOT NULL,
	is_tx_nodefkey  integer NOT NULL,
	is_tx_noheadroom  integer NOT NULL,
	is_tx_ffokcnt  integer NOT NULL,
	is_tx_fferrcnt  integer NOT NULL,
	is_scan_active  integer NOT NULL,
	is_scan_passive  integer NOT NULL,
	is_node_timeout  integer NOT NULL,
	is_crypto_nomem  integer NOT NULL,
	is_crypto_tkip  integer NOT NULL,    
	is_crypto_tkipenmic  integer NOT NULL,
	is_crypto_tkipdemic  integer NOT NULL,
	is_crypto_tkipcm  integer NOT NULL,
	is_crypto_ccmp  integer NOT NULL,
	is_crypto_wep  integer NOT NULL,
	is_crypto_setkey_cipher  integer NOT NULL,
	is_crypto_setkey_nokey  integer NOT NULL,
	is_crypto_delkey  integer NOT NULL,
	is_crypto_badcipher  integer NOT NULL,
	is_crypto_nocipher  integer NOT NULL,
	is_crypto_attachfail  integer NOT NULL,
	is_crypto_swfallback  integer NOT NULL,
	is_crypto_keyfail  integer NOT NULL,
	is_crypto_enmicfail  integer NOT NULL,
	is_ibss_capmismatch  integer NOT NULL,
	is_ibss_norate  integer NOT NULL,
	is_ps_unassoc  integer NOT NULL,
	is_ps_badaid  integer NOT NULL,
	is_ps_qempty  integer NOT NULL,
	 PRIMARY KEY (interfaceName),
    FOREIGN KEY (interfaceName) REFERENCES dot11Interface(interfaceName)
)
;



CREATE TABLE dot11STA
(
	interfaceName  text NOT NULL,
	macAddress  text NOT NULL,
	security  text NOT NULL,
	authentication  text NOT NULL,
	cipher  text NOT NULL,
	timeConnected  text NOT NULL,
	 PRIMARY KEY (macAddress),
    FOREIGN KEY (interfaceName) REFERENCES dot11Interface(interfaceName)
)
;



CREATE TABLE dot11NodeStats
(
	interfaceName  text NOT NULL,
	macAddress  text NOT NULL,
	ns_rx_data  integer NOT NULL,
	ns_rx_mgmt  integer NOT NULL,
	ns_rx_ctrl  integer NOT NULL,
	ns_rx_ucast  integer NOT NULL,
	ns_rx_mcast  integer NOT NULL,
	ns_rx_bytes  integer NOT NULL,
	ns_rx_beacons  integer NOT NULL,
	ns_rx_proberesp  integer NOT NULL,
	ns_rx_dup  integer NOT NULL,
	ns_rx_noprivacy  integer NOT NULL,
	ns_rx_wepfail  integer NOT NULL,
	ns_rx_demicfail  integer NOT NULL,
	ns_rx_decap  integer NOT NULL,
	ns_rx_defrag  integer NOT NULL,
	ns_rx_disassoc  integer NOT NULL,
	ns_rx_deauth  integer NOT NULL,
	ns_rx_decryptcrc  integer NOT NULL,
	ns_rx_unauth  integer NOT NULL,
	ns_rx_unencrypted  integer NOT NULL,
	ns_tx_data  integer NOT NULL,
	ns_tx_mgmt  integer NOT NULL,
	ns_tx_ucast  integer NOT NULL,
	ns_tx_mcast  integer NOT NULL,
	ns_tx_bytes  integer NOT NULL,
	ns_tx_probereq  integer NOT NULL,
	ns_tx_uapsd  integer NOT NULL,
	ns_tx_novlantag  integer NOT NULL,
	ns_tx_vlanmismatch  integer NOT NULL,
	ns_tx_eosplost  integer NOT NULL,
	ns_ps_discard  integer NOT NULL,
	ns_uapsd_triggers  integer NOT NULL,
	ns_tx_assoc  integer NOT NULL,
	ns_tx_assoc_fail  integer NOT NULL,
	ns_tx_auth  integer NOT NULL,
	ns_tx_auth_fail  integer NOT NULL,
	ns_tx_deauth  integer NOT NULL,
	ns_tx_deauth_code  integer NOT NULL,
	ns_tx_disassoc  integer NOT NULL,
	ns_tx_disassoc_code  integer NOT NULL,
	ns_psq_drops  integer NOT NULL,
	 PRIMARY KEY (macAddress),
    FOREIGN KEY (macAddress) REFERENCES dot11STA(macAddress),
    FOREIGN KEY (interfaceName) REFERENCES dot11Interface(interfaceName)
)
;



CREATE TABLE dot11RogueAP
(
	macAddress  text NOT NULL,
	SSID  text NOT NULL,
	security  text NOT NULL,
	pairwiseCiphers  text NOT NULL,
	authMethods  text NOT NULL,
	timeLastSeen  integer,
	 PRIMARY KEY (macAddress, SSID, security, pairwiseCiphers, authMethods)
)
;


CREATE TABLE dot11AuthorizedAP
(
	macAddress  text NOT NULL,
	SSID  text NOT NULL,
	security  text NOT NULL,
	pairwiseCiphers  text,
	authMethods  text,
	 PRIMARY KEY (macAddress, SSID, security, pairwiseCiphers, authMethods)
)
;

CREATE TABLE dot11Profile
(
	profileName  text NOT NULL,
	ssid  text NOT NULL,
	authserver  text ,
	broadcastSSID  integer NOT NULL,
	pskPassAscii  text ,
	pskPassHex  text ,
	wepkey0  text ,
	wepkey1  text ,
	wepkey2  text ,
	wepkey3  text ,
	defWepkeyIdx  integer ,
	groupCipher  text ,
	pairwiseCiphers  text ,
	authMethods  text ,
	security  text NOT NULL,
	authTimeout  integer ,
	assocTimeout  integer ,
	groupKeyUpdateInterval  integer ,
	pmksaLifetime  integer ,
	dot1xReauthInterval  integer ,
	wepAuth  text ,
    preAuthStatus integer NOT NULL,
    qosEnable integer NOT NULL,
    defaultCos text NOT NULL,
    dscpCosMap0 text  NOT NULL,
    dscpCosMap1 text  NOT NULL,
    dscpCosMap2 text  NOT NULL,
    dscpCosMap3 text  NOT NULL,
    dscpCosMap4 text  NOT NULL,
    dscpCosMap5 text  NOT NULL,
    dscpCosMap6 text  NOT NULL,
    dscpCosMap7 text  NOT NULL,
    dscpCosMap8 text  NOT NULL,
    dscpCosMap9 text  NOT NULL,
    dscpCosMap10 text  NOT NULL,
    dscpCosMap11 text  NOT NULL,
    dscpCosMap12 text  NOT NULL,
    dscpCosMap13 text  NOT NULL,
    dscpCosMap14 text  NOT NULL,
    dscpCosMap15 text  NOT NULL,
    dscpCosMap16 text  NOT NULL,
    dscpCosMap17 text  NOT NULL,
    dscpCosMap18 text  NOT NULL,
    dscpCosMap19 text  NOT NULL,
    dscpCosMap20 text  NOT NULL,
    dscpCosMap21 text  NOT NULL,
    dscpCosMap22 text  NOT NULL,
    dscpCosMap23 text  NOT NULL,
    dscpCosMap24 text  NOT NULL,
    dscpCosMap25 text  NOT NULL,
    dscpCosMap26 text  NOT NULL,
    dscpCosMap27 text  NOT NULL,
    dscpCosMap28 text  NOT NULL,
    dscpCosMap29 text  NOT NULL,
    dscpCosMap30 text  NOT NULL,
    dscpCosMap31 text  NOT NULL,
    dscpCosMap32 text  NOT NULL,
    dscpCosMap33 text  NOT NULL,
    dscpCosMap34 text  NOT NULL,
    dscpCosMap35 text  NOT NULL,
    dscpCosMap36 text  NOT NULL,
    dscpCosMap37 text  NOT NULL,
    dscpCosMap38 text  NOT NULL,
    dscpCosMap39 text  NOT NULL,
    dscpCosMap40 text  NOT NULL,
    dscpCosMap41 text  NOT NULL,
    dscpCosMap42 text  NOT NULL,
    dscpCosMap43 text  NOT NULL,
    dscpCosMap44 text  NOT NULL,
    dscpCosMap45 text  NOT NULL,
    dscpCosMap46 text  NOT NULL,
    dscpCosMap47 text  NOT NULL,
    dscpCosMap48 text  NOT NULL,
    dscpCosMap49 text  NOT NULL,
    dscpCosMap50 text  NOT NULL,
    dscpCosMap51 text  NOT NULL,
    dscpCosMap52 text  NOT NULL,
    dscpCosMap53 text  NOT NULL,
    dscpCosMap54 text  NOT NULL,
    dscpCosMap55 text  NOT NULL,
    dscpCosMap56 text  NOT NULL,
    dscpCosMap57 text  NOT NULL,
    dscpCosMap58 text  NOT NULL,
    dscpCosMap59 text  NOT NULL,
    dscpCosMap60 text  NOT NULL,
    dscpCosMap61 text  NOT NULL,
    dscpCosMap62 text  NOT NULL,
    dscpCosMap63 text  NOT NULL,
    wmeEnabled integer NOT NULL,
	 PRIMARY KEY (profileName),
    FOREIGN KEY (authserver) REFERENCES radiusClient(authserver)
)
;

create table dot11Countries
(
    country text,
    countrycode int
);

create table dot11CountriesRegDmnMapping
(
    country text,
    regDomain5Ghz  text,
    regDomain2Ghz text,
    allow11g int,
    allow11na40 int,
    allow11ng40 int
);

create table dot11RegDmnChannelMapping 
(
    regDomain text,
    channelNo int,
    frequency int,
    chan11a   int,
    chan11na int,
    chan11b   int,
    chan11g   int,
    chan11ng int,
    ht40 int -- 40Ghz channel width
); 

create table dot11GlobalConfig
(
    country text,
    iappBssidSecret text
);

create table dot11WPS
(
    vapName       text, -- ap name
    wpsEnabled    int NOT NULL,  -- wps enabled/disabled
    PRIMARY KEY (vapName)
);

create table dot11WPSSessStatus
(
    sessionMsg    text, -- wps session status messages
    sessionStatus int NOT NULL  -- wps session status
);

insert into dot11WPSSessStatus values ("","0");

insert into stringsMap values ("WPS_CONFIG_FAILED","WPS Configuration Failed.");
insert into stringsMap values ("WPS_QUERRY_INPROCESS","WPS Station Adding in Process. Please try after some time to add new Station");
insert into stringsMap values ("AP_DISABLED_ERROR","Enable AP to add WPS Stations");
insert into stringsMap values ("WPA_VAP_USE_ERROR","Please disable WPS on AP using this profile");

insert into dot11CountriesRegDmnMapping values ("USA", "FCC3","FCCA", 1, 1, 1);
insert into dot11CountriesRegDmnMapping values ("INDIA", "APL6","WORLD", 1, 0, 1);
insert into dot11CountriesRegDmnMapping values ("CANADA", "FCC2","FCCA", 1, 1, 1);
--insert into dot11CountriesRegDmnMapping values ("KOREA", "--","--", 1, 1, 1);
--insert into dot11CountriesRegDmnMapping values ("JAPAN", "--","--", 1, 1, 1);
insert into dot11CountriesRegDmnMapping values ("HONG KONG", "FCC2","WORLD", 1, 1, 1);
insert into dot11CountriesRegDmnMapping values ("CHINA", "APL1","WORLD", 1, 1, 1);
insert into dot11CountriesRegDmnMapping values ("AUSTRALIA", "FCC2","WORLD", 1, 1, 1);
insert into dot11CountriesRegDmnMapping values ("SINGAPORE", "APL6","WORLD", 1, 1, 1);
insert into dot11CountriesRegDmnMapping values ("THAILAND", "NULL1","WORLD", 1, 0, 1);
insert into dot11CountriesRegDmnMapping values ("BRUNEI DARUSSALAM", "APL1","WORLD", 1, 1, 1);
insert into dot11CountriesRegDmnMapping values ("INDONESIA", "APL1","WORLD", 1, 0, 1);
insert into dot11CountriesRegDmnMapping values ("NEW ZEALAND", "FCC2","ETSIC", 1, 0, 1);
insert into dot11CountriesRegDmnMapping values ("PHILIPPINES", "APL1","WORLD", 1, 1, 1);
insert into dot11CountriesRegDmnMapping values ("VIETNAM", "NULL1","WORLD", 1, 0, 1);

insert into dot11RegDmnChannelMapping values ("FCCA",  1,2412, 0, 0,1, 1, 1, 0);
insert into dot11RegDmnChannelMapping values ("FCCA",  2,2417, 0, 0,1, 1, 1, 0);
insert into dot11RegDmnChannelMapping values ("FCCA",  3,2422, 0, 0,1, 1, 1, 1);
insert into dot11RegDmnChannelMapping values ("FCCA",  4,2427, 0, 0,1, 1, 1, 1);
insert into dot11RegDmnChannelMapping values ("FCCA",  5,2432, 0, 0,1, 1, 1, 1);
insert into dot11RegDmnChannelMapping values ("FCCA",  6,2437, 0, 0,1, 1, 1, 1);
insert into dot11RegDmnChannelMapping values ("FCCA",  7,2442, 0, 0,1, 1, 1, 1);
insert into dot11RegDmnChannelMapping values ("FCCA",  8,2447, 0, 0,1, 1, 1, 1);
insert into dot11RegDmnChannelMapping values ("FCCA",  9,2452, 0, 0,1, 1, 1, 1);
insert into dot11RegDmnChannelMapping values ("FCCA", 10,2457, 0, 0,1, 1, 1, 0);
insert into dot11RegDmnChannelMapping values ("FCCA", 11,2462, 0, 0,1, 1, 1, 0);
insert into dot11RegDmnChannelMapping values ("FCCA", 12,2467, 0, 0,0, 0, 1, 0);
insert into dot11RegDmnChannelMapping values ("FCCA", 13,2472, 0, 0,0, 0, 1, 0);

insert into dot11RegDmnChannelMapping values ("FCC2", 36, 5180, 1, 0,0,0,0,0);
insert into dot11RegDmnChannelMapping values ("FCC2", 38, 5190, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("FCC2", 40, 5200, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("FCC2", 44, 5220, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("FCC2", 46, 5230, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("FCC2", 48, 5240, 1, 0,0,0,0,0);
insert into dot11RegDmnChannelMapping values ("FCC2", 52, 5260, 1, 0,0,0,0,0);
insert into dot11RegDmnChannelMapping values ("FCC2", 54, 5270, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("FCC2", 56, 5280, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("FCC2", 60, 5300, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("FCC2", 62, 5310, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("FCC2", 64, 5320, 1, 0,0,0,0,0);
insert into dot11RegDmnChannelMapping values ("FCC2", 149, 5745, 1, 0,0,0,0,0);
insert into dot11RegDmnChannelMapping values ("FCC2", 153, 5765, 1, 0,0,0,0,0);
insert into dot11RegDmnChannelMapping values ("FCC2", 157, 5785, 1, 0,0,0,0,0);
insert into dot11RegDmnChannelMapping values ("FCC2", 159, 5795, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("FCC2", 161, 5805, 1, 0,0,0,0,0);
insert into dot11RegDmnChannelMapping values ("FCC2", 165, 5825, 1, 0,0,0,0,0);

insert into dot11RegDmnChannelMapping values ("FCC3", 36, 5180, 1, 0,0,0,0,0);
insert into dot11RegDmnChannelMapping values ("FCC3", 38, 5190, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("FCC3", 40, 5200, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("FCC3", 44, 5220, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("FCC3", 46, 5230, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("FCC3", 48, 5240, 1, 0,0,0,0,0);
insert into dot11RegDmnChannelMapping values ("FCC3", 52, 5260, 1, 0,0,0,0,0);
insert into dot11RegDmnChannelMapping values ("FCC3", 54, 5270, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("FCC3", 56, 5280, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("FCC3", 60, 5300, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("FCC3", 62, 5310, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("FCC3", 64, 5320, 1, 0,0,0,0,0);
insert into dot11RegDmnChannelMapping values ("FCC3", 100, 5500, 1, 0,0,0,0,0);
insert into dot11RegDmnChannelMapping values ("FCC3", 104, 5520, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("FCC3", 108, 5540, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("FCC3", 112, 5560, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("FCC3", 116, 5580, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("FCC3", 120, 5600, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("FCC3", 124, 5620, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("FCC3", 128, 5640, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("FCC3", 132, 5660, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("FCC3", 136, 5680, 1, 0,0,0,0,0);
insert into dot11RegDmnChannelMapping values ("FCC3", 140, 5700, 1, 0,0,0,0,0);
insert into dot11RegDmnChannelMapping values ("FCC3", 149, 5745, 1, 0,0,0,0,0);
insert into dot11RegDmnChannelMapping values ("FCC3", 153, 5765, 1, 0,0,0,0,0);
insert into dot11RegDmnChannelMapping values ("FCC3", 157, 5785, 1, 0,0,0,0,0);
insert into dot11RegDmnChannelMapping values ("FCC3", 159, 5795, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("FCC3", 161, 5805, 1, 0,0,0,0,0);
insert into dot11RegDmnChannelMapping values ("FCC3", 165, 5825, 1, 0,0,0,0,0);

insert into dot11RegDmnChannelMapping values ("APL1", 149, 5745, 1, 0,0,0,0,0);
insert into dot11RegDmnChannelMapping values ("APL1", 151, 5755, 0, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("APL1", 153, 5765, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("APL1", 157, 5785, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("APL1", 159, 5795, 0, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("APL1", 161, 5805, 1, 0,0,0,0,0);
insert into dot11RegDmnChannelMapping values ("APL1", 165, 5825, 1, 0,0,0,0,0);

insert into dot11RegDmnChannelMapping values ("APL6", 36, 5180, 1, 0,0,0,0,0);
insert into dot11RegDmnChannelMapping values ("APL6", 38, 5190, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("APL6", 40, 5200, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("APL6", 44, 5220, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("APL6", 46, 5230, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("APL6", 48, 5240, 1, 0,0,0,0,0);
insert into dot11RegDmnChannelMapping values ("APL6", 52, 5260, 1, 0,0,0,0,0);
insert into dot11RegDmnChannelMapping values ("APL6", 54, 5270, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("APL6", 56, 5280, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("APL6", 60, 5300, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("APL6", 62, 5310, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("APL6", 64, 5320, 1, 0,0,0,0,0);
insert into dot11RegDmnChannelMapping values ("APL6", 149, 5745, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("APL6", 153, 5765, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("APL6", 157, 5785, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("APL6", 159, 5795, 1, 1,0,0,0,1);
insert into dot11RegDmnChannelMapping values ("APL6", 161, 5805, 1, 0,0,0,0,0);
insert into dot11RegDmnChannelMapping values ("APL6", 165, 5825, 1, 0,0,0,0,0);

insert into dot11RegDmnChannelMapping values ("ETSIC",  1,2412, 0, 0,1, 1, 1, 0);
insert into dot11RegDmnChannelMapping values ("ETSIC",  2,2417, 0, 0,1, 1, 1, 0);
insert into dot11RegDmnChannelMapping values ("ETSIC",  3,2422, 0, 0,1, 1, 1, 1);
insert into dot11RegDmnChannelMapping values ("ETSIC",  4,2427, 0, 0,1, 1, 1, 1);
insert into dot11RegDmnChannelMapping values ("ETSIC",  5,2432, 0, 0,1, 1, 1, 1);
insert into dot11RegDmnChannelMapping values ("ETSIC",  6,2437, 0, 0,1, 1, 1, 1);
insert into dot11RegDmnChannelMapping values ("ETSIC",  7,2442, 0, 0,1, 1, 1, 1);
insert into dot11RegDmnChannelMapping values ("ETSIC",  8,2447, 0, 0,1, 1, 1, 1);
insert into dot11RegDmnChannelMapping values ("ETSIC",  9,2452, 0, 0,1, 1, 1, 1);
insert into dot11RegDmnChannelMapping values ("ETSIC", 10,2457, 0, 0,1, 1, 1, 1);
insert into dot11RegDmnChannelMapping values ("ETSIC", 11,2462, 0, 0,1, 1, 1, 1);
insert into dot11RegDmnChannelMapping values ("ETSIC", 12,2467, 0, 0,1, 1, 1, 0);
insert into dot11RegDmnChannelMapping values ("ETSIC", 13,2472, 0, 0,1, 1, 1, 0);

insert into dot11RegDmnChannelMapping values ("WORLD", 1,2412, 0, 0,1, 1, 1, 0);
insert into dot11RegDmnChannelMapping values ("WORLD", 2,2417, 0, 0,1, 1, 1, 0);
insert into dot11RegDmnChannelMapping values ("WORLD", 3,2422, 0, 0,1, 1, 1, 1);
insert into dot11RegDmnChannelMapping values ("WORLD", 4,2427, 0, 0,1, 1, 1, 1);
insert into dot11RegDmnChannelMapping values ("WORLD", 5,2432, 0, 0,1, 1, 1, 1);
insert into dot11RegDmnChannelMapping values ("WORLD", 6,2437, 0, 0,1, 1, 1, 1);
insert into dot11RegDmnChannelMapping values ("WORLD", 7,2442, 0, 0,1, 1, 1, 1);
insert into dot11RegDmnChannelMapping values ("WORLD", 8,2447, 0, 0,1, 1, 1, 1);
insert into dot11RegDmnChannelMapping values ("WORLD", 9,2452, 0, 0,1, 1, 1, 1);
insert into dot11RegDmnChannelMapping values ("WORLD", 10,2457, 0, 0,1, 1, 1, 0);
insert into dot11RegDmnChannelMapping values ("WORLD", 11,2462, 0, 0,1, 1, 1, 0);
insert into dot11RegDmnChannelMapping values ("WORLD", 12,2467, 0, 0,1, 1, 1, 0);
insert into dot11RegDmnChannelMapping values ("WORLD", 13,2472, 0, 0,1, 1, 1, 0);

CREATE TABLE dot11IAPPRemoteAP
(
    bssid  text NOT NULL,
    ipAddr  text,
    bssidSecret text,
    vlanId integer,
    neighbor text,
    PRIMARY KEY (bssid)
)
;

insert into tableDefaults values ("dot11IAPPRemoteAP", "bssid","","","");
insert into tableDefaults values ("dot11IAPPRemoteAP", "ipAddr","","","");
insert into tableDefaults values ("dot11IAPPRemoteAP", "bssidSecret","","","");
insert into tableDefaults values ("dot11IAPPRemoteAP", "vlanId","","1","4095");
insert into tableDefaults values ("dot11IAPPRemoteAP", "neighbor","","","");

insert into tableDefaults values ("dot11Profile", "authTimeout","10","1","3600");
insert into tableDefaults values ("dot11Profile", "assocTimeout","10","1","3600");
insert into tableDefaults values ("dot11Profile", "groupKeyUpdateInterval","3600","1","36000");
insert into tableDefaults values ("dot11Profile", "pmksaLifetime","3600","1","36000");
insert into tableDefaults values ("dot11Profile", "dot1xReauthInterval","3600","1","36000");
insert into tableDefaults values ("dot11Profile", "preAuthStatus","0","","");
insert into tableDefaults values ("dot11Profile","qosEnable","0","","");
insert into tableDefaults values ("dot11Profile","defaultCos","Background","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap0","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap1","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap2","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap3","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap4","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap5","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap6","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap7","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap8","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap9","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap10","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap11","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap12","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap13","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap14","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap15","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap16","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap17","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap18","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap19","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap20","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap21","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap22","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap23","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap24","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap25","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap26","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap27","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap28","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap29","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap30","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap31","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap32","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap33","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap34","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap35","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap36","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap37","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap38","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap39","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap40","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap41","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap42","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap43","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap44","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap45","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap46","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap47","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap48","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap49","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap50","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap51","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap52","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap53","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap54","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap55","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap56","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap57","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap58","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap59","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap60","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap61","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap62","Default","","");
insert into tableDefaults values ("dot11Profile","dscpCosMap63","Default","","");
insert into tableDefaults values ("dot11Profile","wmeEnabled","1","","");
insert into tableDefaults values ("dot11WPS", "vapName","","","");
insert into tableDefaults values ("dot11WPS", "wpsEnabled","0","","");

insert into tableDefaults values ("dot11WPSSessStatus", "sessionMsg","","","");
insert into tableDefaults values ("dot11WPSSessStatus", "sessionStatus","0","","");

insert into dbUpdateRegisterTbl values ("UMI_COMP_UDOT11",1,1,"dot11VAP", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_UDOT11",1,1,"dot11Profile", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_UDOT11",1,1,"dot11Interface", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_UDOT11",0,1,"dot11Radio", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_UDOT11",0,1,"dot11Radio11n", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_UDOT11",0,1,"dot11GlobalConfig", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_UDOT11",1,1,"dot11AuthorizedAP", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_UDOT11",1,1,"dot11RogueAP", 0, 0,0,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_UDOT11",1,1,"dot11ACL", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"dot11Radio", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",0,1,"dot11VAP", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"dot11Card", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_IAPP",0,0,"dot11VAP", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_IAPP",0,0,"dot11IAPPRemoteAP", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_IAPP",0,0,"networkInterface", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_IAPP",0,0,"dot11GlobalConfig", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_UDOT11",0,0,"dot11WPS", 0, 1,1,1);

insert into stringsMap values ("DOT11_STAT_CONFIG_FAILED","Wireless statistics refresh configuration failed");
insert into stringsMap values ("DOT11_AP_STAT_CONFIG_FAILED","AP statistics refresh configuration failed");
insert into stringsMap values ("DOT11_ROGUE_AP_MOVE_FAILED","Unable to move rogue AP");
insert into stringsMap values ("DOT11_AUTH_AP_CONFIG_FAILED","Authorized AP configuration failed");
insert into stringsMap values ("DOT11_PROFILE_CONFIG_FAILED","Profile configuration failed");
insert into stringsMap values ("DOT11_ADV_PROFILE_CONFIG_FAILED","Advanced profile configuration failed");
insert into stringsMap values ("DOT11_QOS_CONFIG_FAILED","QoS configuration failed");
insert into stringsMap values ("DOT11_AP_DEL_FAILED","Unable to delete AP");
insert into stringsMap values ("DOT11_AP_ENABLE_FAILED","Enabling of AP failed");
insert into stringsMap values ("DOT11_AP_DISABLE_FAILED","Disabling of AP failed");
insert into stringsMap values ("DOT11_AP_CONFIG_FAILED","AP configuration failed");
insert into stringsMap values ("DOT11_ADV_AP_CONFIG_FAILED","Advanced AP configuration failed");
insert into stringsMap values ("DOT11_ACL_CONFIG_FAILED","ACL configuration failed");
insert into stringsMap values ("DOT11_RADIO_CONFIG_FAILED","Radio configuration failed");
insert into stringsMap values ("DOT11_CARD_CONFIG_FAILED","Card configuration failed");
insert into stringsMap values ("DOT11_IAPP_CONFIG_FAILED_BSSID","IAPP configuration failed (BSSID Secret)");
insert into stringsMap values ("DOT11_IAPP_ENABLE_FAILED","IAPP enable failed");
insert into stringsMap values ("DOT11_IAPP_DISABLE_FAILED","IAPP disable failed");
insert into stringsMap values ("DOT11_IAPP_CONFIG_FAILED_REMOTE","IAPP configuration failed (Remote AP)");
insert into stringsMap values ("DOT11_WDS_CONFIG_FAILED","Failed to configure WDS (Radio is in use by other APs)");
insert into stringsMap values ("DOT11_RADIO_CHAN_MUTEX_FAILURE","Channel configuration failed (Same channel already configured on a different radio)");
insert into stringsMap values ("DOT11_RADIO_CHAN_DWIDTH_FAILURE","Warning : Channel falls within 25Mhz range of channel configured on another radio (This might affect normal operation)");
insert into stringsMap values ("MAX_CLIENT_ERROR","Configuration Failed.Max Clients configured exceeds total number of clients allowed for the radio");
insert into stringsMap values ("DOT11_WPS_PIN_INVALID","Invalid PIN. Please enter correct PIN.");
insert into stringsMap values ("DELETE_FROM_WPS","This VAP is being used in WPS, cannot delete.");
insert into stringsMap values ("DISABLE_FROM_WPS","This VAP is being used in WPS, cannot disable.");
insert into stringsMap values ("CAN_NOT_EDIT_ACCESS_CONTROL","This VAP is being used in WPS, cannot edit.");
insert into stringsMap values ("AP_USING_PROFILE","Selected Profile(s) are using by a VAP.");
insert into stringsMap values ("WPA_LENGTH_FAILED","Invalid password length. Please enter a password having 8 to 63 characters.");
insert into stringsMap values ("WEP_LENGTH_FAILED","Failed to configure WEP key as the length is invalid.");
insert into stringsMap values ("WEP_ASCII_LENGTH_FAILED","ASCII key should contain 5(64 Bit) or 13(128 Bit) characters.");
CREATE TABLE radiusClient
(
	authserver  text NOT NULL,
	authport  integer NOT NULL,
	authsecret  text NOT NULL,
	authtimeout  integer NOT NULL,
	authretries  integer NOT NULL,
	acctserver  text ,
	acctport  integer ,
	acctsecret  text ,
	 PRIMARY KEY (authserver)
)
;
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"radiusClient", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_EAP_RAD",1,1,"radiusClient", 0, 1,1,1);

insert into stringsMap values ("RADIUS_CONFIG_FAILED","RADIUS configuration failed");

-- ipsecVPN Table
CREATE TABLE IpsecVPNPolicy
    (
    TunnelOrTransport       boolean   NOT NULL,      -- 1 for tunnel mode, 0 for transport, default:1
    EnableDhcpOverIPSec     boolean   NOT NULL,      -- 1 for enabling and 0 for disabling, default: 0    
    OID                     integer   NULL,          -- OID
    VPNPolicyName           text      NOT NULL UNIQUE,-- VPN Policy Name
    PolicyType              integer   NOT NULL,      -- Type of the Policy (Manual/Auto/ModeConfig)
    LocalGateway            integer   NULL,          -- Type of Local GateWay (Dedicated/Secondary)
    RemoteEndPointType      boolean   NOT NULL,      -- Type of Remote GateWay (IPAddress/FQDN)
    RemoteEndPoint          text      NOT NULL,      -- IP or FQDN name
    Netbios                 boolean   NULL,          -- enable/disable netbios
    Rollover                boolean   NULL,          -- enable/disable rollover
    EnableKeepAlive         boolean   NULL,          -- enable/disable KeepAlive
    RemotePingIPAddress     text      NULL,          -- IP address of th remote host to check KA   
    KeepAlivePeriod         integer   NULL,          -- KeepAlive Detection Period 
    KeepAliveFailureCount   integer   NULL,          -- Reconnect after failure count for KA
    EncryptionAlgorithm     integer   NOT NULL,      -- EncryptionAlgorithm
    KeyLength               integer   NULL,          -- Key length for variable key-size algorithms like blowfish and cast128  
    AuthAlgorithm           integer   NOT NULL,      -- AuthAlgorithm
    LocalNetworkType        integer   NOT NULL,      -- Type of local traffic selectors (Any/Single/Range/Subnet)   
    LocalStartAddress       text      NULL,          -- Local network Start Address
    LocalEndAddress         text      NULL,          -- Local network End Address
    LocalSubnetMask         text      NULL,          -- Local network Subnet Mask
    RemoteNetworkType       integer   NOT NULL,      -- Type of Remote traffic selectors (Any/Single/Range/Subnet)   
    RemoteStartAddress      text      NULL,          -- Remote network Start Address
    RemoteEndAddress        text      NULL,          -- Remote network End Address
    RemoteSubnetMask        text      NULL,          -- Remote network Subnet Mask
    SPIIn                   text      NULL,          -- SPI IN
    SPIOut                  text      NULL,          -- SPI OUT
    EncryptionKeyIn         text      NULL,          -- Key In for Encryption 
    EncryptionKeyOut        text      NULL,          -- Key Out for Encryption 
    AuthKeyIn               text      NULL,          -- Key In for Authentication
    AuthKeyOut              text      NULL,          -- Key Out for Authentication
    AutoPolicyType          integer   NULL,          -- Policy Type (Client/Gate Way)
    SALifeTime              integer   NULL,          -- SA LifeTime 
    SALifeTimeType          boolean   NULL,          -- SA LifeTime Type     
    PFSKeyGroup             integer   NULL,          -- PFSKeyGroup (DH Group 1/2/3)
    IKEPolicyName           text      NULL,          -- IKE Policy Name
    Status                  boolean   NOT NULL,      -- status of the VPN policy (enable/disable)
    RVGStatus               boolean   NOT NULL,
    BackupPolicyName        text      NULL,
    FailbackTime            integer   NULL,
    TypeofPolicy            integer   NULL,          -- Primary or Backup policy
    NodeId                  integer   primary key autoincrement,

    -- add your new entries here
    
    FOREIGN KEY (IKEPolicyName) REFERENCES IpsecIKEPolicy (IKEPolicyName)    -- foreign key    
    );

insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"IpsecVPNPolicy",0,1,1,1);

insert into stringsMap values ("IPSEC_VPN_DEL_FAILED","Ipsec VPN Delete Failed");
insert into stringsMap values ("IPSEC_VPN_ENABLE_FAILED","Ipsec VPN Enable Failed");
insert into stringsMap values ("IPSEC_VPN_DISABLE_FAILED","Ipsec VPN Disable Failed");
insert into stringsMap values ("IPSEC_VPN_ALREADY_ENABLED","Selected All Policies Already Enabled");
insert into stringsMap values ("IPSEC_VPN_ALREADY_DISABLED","Selected All Policies Already Disabled");
insert into stringsMap values ("IPSEC_VPN_CONFIG_FAILED","Ipsec VPN Configuration Failed");
insert into stringsMap values ("IPSEC_VPN_INVALID_CONFIGURATION","Invalid Configuration, Can't Add/Edit Policy");
insert into stringsMap values ("IPSEC_CANT_SHARE_IKE_FOR_DIFF_ENDPOINTS","Can't share IKE policy with different remote/local endpoint");
insert into stringsMap values ("ONYL_RVG_STATUS_ALLOW_TO_CHANGE","Only Redundant VPN Gateway status is allowed to change when backup policy is up.");
insert into stringsMap values ("PRIMARY_POLICY_CONFIG_NOT_FOUND","Primary VPN Policy configuration not found");
insert into stringsMap values ("MANUAL_POLICY_CANT_SET_AS_BACKUP","Backup policy setting is not allowed for manual policies");
insert into stringsMap values ("CLIENT_POLICY_CANT_SET_AS_BACKUP","Redundant VPN Gateway is not for client policies");
insert into stringsMap values ("IKE_POLICY_CONFIG_NOT_FOUND","IKE Policy configuration not found");
insert into stringsMap values ("DPD_NOT_ENABLED","Please enable DPD for this policy If you want to enable back up tunnel");
insert into stringsMap values ("DIRECTION_SET_TO_RESPONDER","Redundant VPN Gateway is for initiators only");
insert into stringsMap values ("XAUTH_SET_TO_USER_DATABASE","Please disable XAuth user database option If you want to enable back up tunnel");
insert into stringsMap values ("POLICY_ALREADY_IN_USE","Selected Policy is already using as backup policy");

insert into tableDefaults values ("IpsecVPNPolicy","TunnelOrTransport","1","","");
insert into tableDefaults values ("IpsecVPNPolicy","EnableDhcpOverIPSec","0","","");
insert into tableDefaults values ("IpsecVPNPolicy","PolicyType","1","","");
insert into tableDefaults values ("IpsecVPNPolicy","LocalGateway","0","","");
insert into tableDefaults values ("IpsecVPNPolicy","RemoteEndPointType","0","","");
insert into tableDefaults values ("IpsecVPNPolicy","Netbios","0","0","1");
insert into tableDefaults values ("IpsecVPNPolicy","Rollover","0","0","1");
insert into tableDefaults values ("IpsecVPNPolicy","EnableKeepAlive","0","0","1");
insert into tableDefaults values ("IpsecVPNPolicy","KeepAlivePeriod","10","","");
insert into tableDefaults values ("IpsecVPNPolicy","KeepAliveFailureCount","3","","");
insert into tableDefaults values ("IpsecVPNPolicy","EncryptionAlgorithm","2","","");
insert into tableDefaults values ("IpsecVPNPolicy","KeyLength","0","","");
insert into tableDefaults values ("IpsecVPNPolicy","AuthAlgorithm","2","","");
insert into tableDefaults values ("IpsecVPNPolicy","LocalNetworkType","3","0","3");
insert into tableDefaults values ("IpsecVPNPolicy","RemoteNetworkType","3","0","3");
insert into tableDefaults values ("IpsecVPNPolicy","SPIIn","0x","","");
insert into tableDefaults values ("IpsecVPNPolicy","SPIOut","0x","","");
insert into tableDefaults values ("IpsecVPNPolicy","SALifeTime","3600", "", "");
insert into tableDefaults values ("IpsecVPNPolicy","SALifeTimeType","0", "0", "1");
insert into tableDefaults values ("IpsecVPNPolicy","PFSKeyGroup","2", "", "");
insert into tableDefaults values ("IpsecVPNPolicy","Status","1", "", "");
insert into tableDefaults values ("IpsecVPNPolicy","AutoPolicyType","0", "", "");
insert into tableDefaults values ("IpsecVPNPolicy","RVGStatus","0","","");
insert into tableDefaults values ("IpsecVPNPolicy","FailbackTime","30","30","3600");

-- IpsecIKEPolicy Table
CREATE TABLE IpsecIKEPolicy
    (
    OID                     integer         NULL,        -- OID   
    IKEPolicyName           text            NOT NULL,    -- IKE PolicyName
    Direction               integer         NOT NULL,    -- Direction (Responder/Initiator/Both)
    ExchangeMode            boolean         NOT NULL,    -- ExchangeMode (Main/Aggressive)    
    ModeConfigStatus        boolean         NOT NULL,    -- enable/disable ModeConfig 
    ModeConfigRecordName    text            NULL,        -- ModeConfigRecordName
    LocalIdentifierType     integer         NOT NULL,    -- LocalIdentifierType (IP/FQDN/User FQDN/DER ASN1 DN)
    LocalIdentifier         text            NULL,        -- LocalIdentifier
    RemoteIdentifierType    integer         NOT NULL,    -- RemoteIdentifierType (IP/FQDN/User FQDN/DER ASN1 DN)
    RemoteIdentifier        text            NULL,        -- RemoteIdentifier
    EncryptionAlgorithm     integer         NOT NULL,    -- EncryptionAlgorithm
    AuthAlgorithm           integer         NOT NULL,    -- AuthAlgorithm
    AuthType                integer         NOT NULL,    -- Authentication Type (presharedkey/RSA signature) 
    Presharedkey            text            NULL,        -- Presharedkey
    DHGroup                 integer         NOT NULL,    -- DH Group 1/2/3
    SALifeTime              long integer    NOT NULL,    -- SA Life Time
    DPD                     boolean         NOT NULL,    -- enable/disable DPD
    DPDDetectionPeriod      integer         NULL,        -- DPD Detection Period
    DPDFailureCount         integer         NULL,        -- DPD Failure Count   
    XAUTHType               integer         NOT NULL,    -- XAUTH Type (Edgedevice/Ipsec Host/ Radius CHAP/PAP/User DB)
    UserName                text            NULL,        -- UserName for Ipschost
    Password                text            NULL,        -- Password for Ipschost
    NatTraversal            boolean         NOT NULL,    -- 1-On, 0-off default:1
    NatKeepAliveTime        integer         NOT NULL,    -- Interval between sending keep-alive packets, in seconds 

    -- add your new entries here
    
    PRIMARY KEY (IKEPolicyName),                         -- primary key
    FOREIGN KEY (ModeConfigRecordName) REFERENCES IpsecModeConfig (ModeConfigRecordName)   -- foreign key
    );

insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"IpsecIKEPolicy",0,1,1,1);

insert into tableDefaults values ("IpsecIKEPolicy","Direction","2","0","2");
insert into tableDefaults values ("IpsecIKEPolicy","ModeConfigStatus","0","","");
insert into tableDefaults values ("IpsecIKEPolicy","EncryptionAlgorithm","2","","");
insert into tableDefaults values ("IpsecIKEPolicy","AuthAlgorithm","2","","");
insert into tableDefaults values ("IpsecIKEPolicy","AuthType","0","","");
insert into tableDefaults values ("IpsecIKEPolicy","DHGroup","2","","");
insert into tableDefaults values ("IpsecIKEPolicy","SALifeTime","28800","","");
insert into tableDefaults values ("IpsecIKEPolicy","DPD","0","","");
insert into tableDefaults values ("IpsecIKEPolicy","DPDDetectionPeriod","10","","");
insert into tableDefaults values ("IpsecIKEPolicy","DPDFailureCount","3","","");
insert into tableDefaults values ("IpsecIKEPolicy","XAUTHType","0","","");
insert into tableDefaults values ("IpsecIKEPolicy","NatTraversal","1","","");
insert into tableDefaults values ("IpsecIKEPolicy","NatKeepAliveTime","20","","");
insert into stringsMap values ("IKEP_CONFIG_FAILED","IKE Policy Configuration Failed");
insert into stringsMap values ("IKEP_DELETE_FAILED","IKE Policy Deletion Failed");
insert into stringsMap values ("IKEP_DELETE_DEPENDANT_FAILED","IKE Policy Cannot be deleted, being used in VPN Policy");
insert into stringsMap values ("IKEP_DELETE_SUCCESS","Delete Operation Succeeded");
insert into stringsMap values ("IKEP_EDITCHECK_FAILED","Please disable corresponding VPN policy before making changes to this IKE policy");
insert into stringsMap values ("CANT_ENABLE_TWO_POLICIES_WITH_SAME_TRAFFIC_SELECTORS","Can't enable two policies with same traffic selectors");

--IpsecModeConfig Table
CREATE TABLE IpsecModeConfig
    (
    OID                     integer   NULL,         -- OID
    ModeConfigRecordName    text      NOT NULL,     -- ModeConfig RecordName
    FirstPoolStartIP        text      NOT NULL,     -- First Pool Start IP
    FirstPoolEndIP          text      NOT NULL,     -- First Pool End IP
    SecondPoolStartIP       text      NULL,         -- Second Pool StartIP
    SecondPoolEndIP         text      NULL,         -- Second Pool End IP
    ThirdPoolStartIP        text      NULL,         -- Third Pool Start IP
    ThirdPoolEndIP          text      NULL,         -- Third Pool End IP
    WINSServerPrimaryIP     text      NULL,         -- WINSServer Praimary IP
    WINSServerSecondaryIP   text      NULL,         -- WINSServer Secondary IP
    DNSServerPrimaryIP      text      NULL,         -- DNSServer Praimary IP
    DNSServerSecondaryIP    text      NULL,         -- DNSServer Secondary IP

    -- add your new entries here
    PRIMARY KEY (ModeConfigRecordName)              -- primary key
    );

insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"IpsecModeConfig",0,1,1,1);

insert into stringsMap values ("IPSEC_MODE_DEL_FAILED","Ipsec Mode Delete Failed");
insert into stringsMap values ("IPSEC_MODE_CONFIG_FAILED", "Ipsec Mode Configuration Failed");
insert into stringsMap values ("IPSEC_MODE_DEL_DEPEND_FAILED","Ipsec Mode Delete Failed as Mode is used in IkePolicy");
-- ConnectionStatus Table
CREATE TABLE ConnectionStatus
    (
    OID                 integer       NULL,         -- OID   
    PolicyName          text          NULL,         -- Policy Name
    TxData              text          NULL,         -- Tx Data  
    TxPackets           long integer  NULL,         -- Tx Packets
    ConnectionStatus    text          NULL,         -- Connction Status
    ActionStatus        boolean       NULL,         -- Is Action allowed    
    Action              boolean       NULL,         -- Action (Connect/DROP)
    NodeId              integer       NULL,         -- unique ID 
    OutSPI              text          NULL,         -- SPI
    RemoteEndpoint      text          NULL,         -- Remote End Point
    IKEPort             integer       NULL,         -- IKE port Number
    AutopolicyType      boolean       NULL,         -- Autopolicytype         
    
    -- add your new entries here
    PRIMARY KEY (PolicyName)            -- primary key
    );

-- This table is used to maintain the reference count of a ikepolicy usage
-- This table is not saved across reboots. This is used only to get/set the reference count os ike policy usage
CREATE TABLE IKEPolicyUsageRefCountTable 
(
    OID   integer NULL,
    PolicyName text NOT NULL,
    RefCount integer NOT NULL,

    PRIMARY KEY (PolicyName),
    FOREIGN KEY (PolicyName) REFERENCES IpsecIKEPolicy (IKEPolicyName)   -- foreign key
);
    
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"ConnectionStatus",0,1,0,0);
insert into stringsMap values ("VIPSECURE_CS_CONFIG_FAILED","Connection Status update Failed");
insert into stringsMap values ("IPSEC_VPN_ENABLE_SUCCESS","Ipsec VPN Policy(s) Enabled Successfully");
insert into stringsMap values ("IPSEC_VPN_DISABLE_SUCCESS","Ipsec VPN Policy(s) Disabled Successfully");
insert into stringsMap values ("IPSEC_VPN_REMOTE_CONFIG_FAILED","Ipsec VPN Configuration Failed as the Remote End Point is already in use.");
insert into stringsMap values ("IPSEC_VPN_NAME_CONFIG_FAILED","VPN Configuration Failed as the Policy Name is already in use.");
insert into stringsMap values ("IKEP_NAME_CONFIG_FAILED","IKE Policy Configuration Failed as the Policy Name is already in use");
insert into stringsMap values ("IPSEC_MODE_NAME_CONFIG_FAILED", "Ipsec Mode Configuration Failed as the Record Name is already in use");
insert into stringsMap values ("IPSEC_VPN_DEL_SUCCESS","Ipsec VPN Policy(s) Deleted Successfully");
insert into tableDefaults values ("ConnectionStatus", "TxData", "0", "", "");
insert into tableDefaults values ("ConnectionStatus", "TxPackets", "0", "", "");
insert into tableDefaults values ("ConnectionStatus", "ConnectionStatus", "0", "", "");
insert into tableDefaults values ("ConnectionStatus", "Action", "0", "", "");

CREATE TABLE XAuthUsers
(
    OID integer NULL,
    UserName text NOT NULL,
    Password text NOT NULL,
    UserType integer NULL,
    AllowChangePassword boolean NULL,
    PRIMARY KEY (UserName)
);

insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"XAuthUsers",0,1,1,1);

insert into tableDefaults values ("XAuthUsers", "UserType", "0", "", "");
insert into stringsMap values ("XAUTH_USERS_CONFIG_FAILED","XAuth User Configuration Failed");
insert into stringsMap values ("XAUTH_WRONGE_OLD_PASSWORD","Wrong Old Password");
insert into stringsMap values ("USER_NAME_ALREADY_EXISTS","User name already exists");
insert into stringsMap values ("MAX_VPN_LIMIT_REACHED","Can not add policies as maximum limit reached.");
-- This Table is for back end maintanance purpose. In GUI point of view it is a
-- dummy table. Here we will save the presharedkey, NAT IP if the client is
-- behind NAT.

CREATE TABLE QuickVPNUsersStatus
(
    OID             integer NULL,
    UserName        text    NOT NULL,
    Presharedkey    text    NULL,
    Status          boolean NOT NULL,
    ServerIP        text    NULL, -- IP of server.
    PublicIP        text    NULL, -- This is NAT IP if the client is behind NAT and in normal case it is client IP
    ClientIP        text    NULL, -- This is IP if the client.
    IkeBlockName    text    NULL, -- This is for with ike block it is sharing
    NodeId          integer NOT NULL,

    PRIMARY KEY (UserName)
);
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"QuickVPNUsersStatus",0,1,0,0);

-- This is table to exchange th data between racoon and rvgd daemon. This is
-- specific to redundant vpn gateway feature
CREATE TABLE VipsecureDeadPeer
(
    OID             integer NULL,
    PolicyName      text    NOT NULL,

    PRIMARY KEY (PolicyName)
);

-- This table stores the IP addresses that we assign to 'DHCP over IPsec' 
-- clients
CREATE TABLE IPsecDhcpClientIPRange
(
    StartIPAddress  text NOT NULL,
    EndIPAddress    text NOT NULL,
    SubnetMask      text NOT NULL,
    PRIMARY KEY (StartIPAddress,EndIPAddress,SubnetMask)
);
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"IPsecDhcpClientIPRange",0,1,1,1);
insert into stringsMap values ("DHCP_IPSEC_FAILED","DHCP Over IPsec Configuration Failed.");
CREATE TABLE sixToFourTunnel
    (
    tunnelStatus        integer NOT NULL,    -- enabled or disabled    
    LogicalIfName       text NOT NULL,       -- IPv4 interface to use
    interfaceName       text NOT NULL,       -- 6to4 interface name 
    localAddr           text NOT NULL,       -- local tunnel end-point   
    destAddr            text,                -- destination tunnel end-point
    PRIMARY KEY (LogicalIfName),
    FOREIGN KEY (LogicalIfName) REFERENCES networkInterface(LogicalIfName)
    );
insert into tableDefaults values ("sixToFourTunnel", "tunnelStatus", "1", "", "");
insert into tableDefaults values ("sixToFourTunnel", "LogicalIfName", "WAN1", "", "");
insert into tableDefaults values ("sixToFourTunnel", "interfaceName", "eth0", "", "");
insert into tableDefaults values ("sixToFourTunnel", "localAddr", "192.168.5.10", "", "");
insert into stringsMap values ("6TO4_TUNNEL_CONFIG_FAILED", "6 to 4 Tunnel Configuration Failed");    
CREATE TABLE isatapTunnel
    (
    isatapTunnelStatus  integer NOT NULL,
    isatapPrefix        text,    -- 64 bit ISATAP prefix
    useLanAddress       integer NOT NULL, -- use LAN address.
    localIPv4Address    text,    -- The IPV4 address to use as local endpoint
    isIsatapRouter      integer, -- Is this a ISATAP Router
    preferredRouter     text,    -- Preferred Router if this is a ISATAP host
    interfaceName       text,    -- tunnel interface name
    transportIfName     text NOT NULL, -- transport interface      
    PRIMARY KEY (isatapPrefix)
    );
insert into tableDefaults values ("isatapTunnel", "isatapTunnelStatus", "1", "", "");
insert into tableDefaults values ("isatapTunnel", "useLanAddress", "1", "", "");
insert into stringsMap values ("ISATAP_TUNNEL_DELETE_SUCCESS", "ISATAP Tunnel(s) Deleted Successfully");
insert into stringsMap values ("ISATAP_TUNNEL_DELETE_FAILED", "ISATAP Tunnel(s) Deletion Failed");
insert into stringsMap values ("ISATAP_TUNNEL_CONFIG_FAILED", "ISATAP Tunnel Configuration Failed");    
CREATE TABLE sshd
(
        port integer NOT NULL,
        protocol integer NOT NULL,
        listenAddr text NOT NULL,
        keyBits integer NOT NULL,
        rootEnable integer NOT NULL,
        rsaEnable integer NOT NULL,
        dsaEnable integer NOT NULL,
        passEnable integer NOT NULL,
        emptyPassEnable integer NOT NULL,
        sftpEnable integer NOT NULL,
        scpEnable integer NOT NULL,
        sshdEnable integer NOT NULL,
        privSep   integer NOT NULL
)
;

insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"sshd", 0, 1,1,1);
insert into stringsmap values ("SSHD_CONFIG_FAILED","SSHD configuration failed.");
-- Modification History
-- 
-- 01b,27feb10,ar  included new column for RefererValue to support CSRF fix.
-- 01a,22oct08,til included new colume to handle certificate updates 
--
--

--
-- This file includes sslvpn component related Tables like -
--
--  1. sslvpn Users
--  2. sslvpn Groups
--  3. sslvpn Domains
--  4. sslvpn PortalLayouts
--  5. sslvpn Policies
--  6. sslvpn Resources
--  7. sslvpn ClientConfiguration
--  8. sslvpn PortForwarding
--  9. sslvpn ActiveUsersStatus / sslvpn Connection Status
-- 10. sslvpn UsersState 
--
--  TODO :
--  0. Review
--  1. Include Error Macros
--  2. Rectifications, if any after getting Reviewed.
--

--
-- Portal Layout Table
--
--  SPECS(SPECIFICATIONS) :
--
--  1. DefaultPortal is used as a hidden key to mark(*) a layout as
--      'system Generated' PortalLayout or user defined 'display Default portal'.
--      DefaultPortal takes 0 (not a default Portal), 1 (system Default), 2(user Configured Display Default) 
--  
--  2. Portal Layout's 'Banner Message' @ ADD/EDIT is shown as 'Description' in
--      the list of PortalLayouts @ PortalLayouts Main page.
--      Its better to change 'Description' in GUI as 'Banner Message' to avoid confusion.
-- 
--  3. We can completely avoid the field 'Description' from displaying in the List of layouts as Very lengthy
--      entries in the field causes some weird results.
--
--  4. A Portal if it is  DefaultPortal cannot be deleted.
--      i.e, If DefaulPortal == 1 or 2, mark the PortalLayout with * and Grey out the
--      checkBox used for Deleting the row entry.
--  
--  5. A Portal canot be deleted if its UseCount > 0(=> if any of domains use it).
--     i.e, If a portal was Configured for any of the domains, such Portal
--     cannot be deleted. 
--     Error = "Portal cannot be deleted if Domains are Configured for it" should be thrown.
--  
--  6. When Domain is deleted/added, the 'UseCount' of the associated PortalLayout should be 
--      Updated(decremented/incremented).
--
--  7. PortalURL for a PortalLayoout entry should be Manufactured by the GUI/LUA, 
--      as "https://$wanIP/portal/$PortalLayoutName"  using wanIP field obtained from 
--      WAN Interface Table.
--  
--  8. Setting a portal with no domains configured, as Default portal, should throw an error.
--     i.e, clicking 'default' for a portal with UseCount == 0,should throw an error.   
--     error : "Portal is not configured for any domain. set as default Failed."
--  

CREATE TABLE SSLVPNPortalLayout
(
    OID                                 integer     NULL,
    PortalLayoutName                    text        NOT NULL,   -- unique PortalName, PRIMARY KEY.
    DefaultPortal                       integer     NULL,       -- Hidden value to mark the portal as system default.
    PortalSiteTitle                     text        NULL,       -- Displayed in Window's Title Bar for UserPortal 
    BannerTitle                         text        NULL,       -- Dispalyed in Login Page of the portal.
    BannerMessage                       text        NULL,       -- @ alias to 'Description'. 
                                                                -- Dispalyed in Login Page of the portal.
    DisplayBanner                       boolean     NULL,       -- option to Enable 'Display BannnerMessage'
    EnableHTTPMetaTags                  boolean     NULL,       -- option to Enable 'HTTP Tags for Cache Control'.  
    EnableActiveXWebCacheCleaner        boolean     NULL,       -- option to Enable Active X web Cache Control.
    EnableVPNTunnel                     boolean     NULL,       -- option to Enable Display VPNTunnel in userPortal.   
    EnablePortForwarding                boolean     NULL,       -- option to Enable Display PF in userPortal.   
    UseCount                            integer     NULL,       -- The number of Domains configured for this Portal.    
    PortalURL                           text        NULL,       -- shortCut link to UserPortal page.
    PRIMARY KEY (PortalLayoutName)
)
;

insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM", 0, 1, "SSLVPNPortalLayout", 0, 1, 1, 1);
insert into tableDefaults values ("SSLVPNPortalLayout", "DefaultPortal", "0", "", "");
insert into tableDefaults values ("SSLVPNPortalLayout", "EnableVPNTunnel", "1", "", ""); -- Enabling VP by default
insert into tableDefaults values ("SSLVPNPortalLayout", "UseCount", "0", "", "");
insert into stringsMap values ("SSLVPN_PORTAL_LAYOUT_CONFIG_FAILED","PortalLayout(s) Configuration Failed");
insert into stringsMap values ("SSLVPN_PORTAL_LAYOUT_HAS_DOMAIN_DEL_FAILED","PortalLayout can not Deleted if it contains Domain(s)");
insert into stringsMap values ("SSLVPN_PORTAL_LAYOUT_NO_DOMAIN_SET_AS_DEFAULT_FAILED","Portal cannot be set as default if no domain uses it");
insert into stringsMap values ("SSLVPN_PORTAL_LAYOUT_DELETE_SUCCESS","PortalLayout(s) Deleted Successfully");

--
-- Domains Table
--
-- SPECS :
--
--  1. 'AuthenticationType', decides which Fields other than DomainName and
--      PortalLayoutName, should get disabled or enabled in the Add/Edit pages
--      of Domain configuration. In other words, Only related fields of 
--      'AuthenticationType' should be Enabled. 
--     Please refer platform1.0's add domain page.
--
--  2. 'AuthenticationType' Mapping : 
--      RULES:
--           For DropDown Entry,
--                             [GUI]   : [values needed by Backend]
--           case local User Database  : set value as "local";
--           case Radius-PAP           : set value as "radius_pap";
--           case Radius-CHAP          : set value as "radius_chap";
--           case Radius-MSCHAP        : set value as "radius_mschap";
--           case Radius-MACHAPv2      : set value as "radius_mschapv2";
--           case NT Domain            : set value as "ntdomain";
--           case Active Directory     : set value as "active-directory";
--           case LDAP                 : set value as "ldap";			
--    To avoid any Confusion we can display in the DropDown List of
--    Authentication Type, the same values what we set for 'AuthenticationType'. 
--
--  3. 'DefaultDomain' is a flag to indicate that Domain is system generated.  
--      It is used as hidden key to mark(*) in GUI as Default Domain.
--      'DefaultDomain' takes values 0 (not a default), 1 (default Domain).
--      Default Domains Cannot be deleted.
--
--  4. A Domain Should not be deleted if Users are Configured for it.
--      Error = "Domain cannot be deleted if Users are Configured for it." should be thrown.
-- 
--  5. when a Domain is created, its corresponding Default group(group with the same name as that of Domain)
--      should also be created in the GroupTable and the DefaultGroup flag should be set to 1 for that Group.
-- 
--  6. Domain Should NOT be Deleted if its portal is a Default Portal(ie when DefaultPortal == 1 or 2)
--      Error: "Domain Cannot if its portal is set as Default".
--     For a Domain, 'DefaultPortal' value  of its portal can be obtained using 
--      portalLayoutName reference to PortalLayoutTable. 
--          

CREATE TABLE SSLVPNDomain
(
    OID                               integer     NULL,
    DomainName                        text        NOT NULL,   -- Unique DomainName, PRIMARY KEY
    PortalLayoutName                  text        NOT NULL,   -- Portal for this Domain. 
    AuthenticationType                text        NOT NULL,   -- Authentication type.
    AuthenticationServer              text        NULL,       -- Authenticating server address.
    SecondAuthenticationServer        text        NULL,       -- Authenticating server address.
    AuthenticationRadiusSecret        text        NULL,       -- Radius secret.
    SecondAuthenticationRadiusSecret  text        NULL,       -- Radius secret.
    NTDomainWorkGroup                 text        NULL,       -- Work Group for NT Domain.
    LDAPBaseDN                        text        NULL,       -- Base DomainName for LDAP Authentication.
    ActiveDirectoryDomain             text        NULL,       -- AD domain Name for AD Authentication.
    DefaultDomain                     boolean     NULL,       -- Flag to indicate that the domain is a System Generated Domain.                                                            -- Default value is set to 0.    
    PRIMARY KEY (DomainName),
    FOREIGN KEY (PortalLayoutName) REFERENCES SSLVPNPortalLayout (PortalLayoutName)

)
;

insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM", 0, 1, "SSLVPNDomain", 0, 1, 1, 1);
insert into tableDefaults values ("SSLVPNDomain", "DefaultDomain", "0", "", "");
insert into stringsMap values ("SSLVPN_DOMAIN_CONFIG_FAILED","Domain(s) Configuration Failed");
insert into stringsMap values ("SSLVPN_DOMAIN_HAS_USERS_DOM_DEL_FAILED","Can not Delete a Domain if it contains User(s)");
insert into stringsMap values ("SSLVPN_PORTAL_IS_DEFAULT_DOMAIN_DEL_FAILED","Cannot delete the Only domain, As its portal is set as Default");
insert into stringsMap values ("SSLVPN_PORTAL_IS_DEFAULT_DOMAIN_EDIT_FAILED","Cannot change the protal for the only domain, As its portal is set as Default");

--
-- Groups Table
--
--  SPECS:
--
--  1. In Group Edit Page,
--      RULE:
--      Ldap Attributes fields should be enabled only if AuthenticationType == 'ldap'.
--      'AuthenticationType' for a Group could be obtained from DomainTable using the 'DomainName'
--      refrence .
--   
--  2. 'DefaultGroup' is the attribute to indicate that it's created along with the
--      creation of a domain. It is used as hidden key to mark(*) in GUI as Default
--      Group.
--      DefaultGroup takes values = 0 (Not a default Group), 1 (Default group).
--      Default Groups cannot be deleted.
--
--  3. when Domain is created, its corresponding Default group(group with the same name as that of Domain)
--      should also be created in the GroupTable and the DefaultGroup flag should be set to 1 for that Group.
      
--  4. Group, if has Users Configured for it, cannot be deleted.
--      Error : "Group cannot be deleted if Users are configured for it" should be thrown.
--
--  5. GUI REVIEW/proposal:
--      a. Can the LDAP Attributes be moved to ADD Page so that the user dont miss configuring
--          LDAP attributes.(Presently LDAP Attr are configured in EDIT page of Groups)
--      b. If LDAP Attributes are Moved to ADD page, 
--          We need to use AuthenticationType of selected DomainName to enable or disable the LDAP Attributes.   
--          The above same mechanism is followed in the EDIT page too.
-- 

CREATE TABLE SSLVPNGroup
(
    OID                         integer     NULL,
    GroupName                   text        NOT NULL,   --  Unique Group name, PRIMARY KEY.
    DomainName                  text        NOT NULL,   --  Domain to which This Group Belongs.       
    GroupTimeOut                integer     NOT NULL,   --  Idle TimeOut of Group, Defaul is 10.
    LDAPAttribute1              text        NULL,       --  LDAP Attributes.
    LDAPAttribute2              text        NULL,       --  LDAP Attributes.
    LDAPAttribute3              text        NULL,       --  LDAP Attributes.
    LDAPAttribute4              text        NULL,       --  LDAP Attributes.    
    DefaultGroup                boolean     NULL,       --  Flag to indicate that the Group is a Domain Generated Group.
                                                        --  Default valu is set to 0.
    PRIMARY KEY (GroupName),
    FOREIGN KEY (DomainName) REFERENCES SSLVPNDomain (DomainName)    
)
;

insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM", 0, 1, "SSLVPNGroup", 0, 1, 1, 1);
insert into tableDefaults values ("SSLVPNGroup", "DefaultGroup", "0", "", "");
insert into tableDefaults values ("SSLVPNGroup", "GroupTimeOut", "5", "", "");
insert into tableDefaults values ("SSLVPNGroup", "LDAPAttribute1", "", "", "");
insert into tableDefaults values ("SSLVPNGroup", "LDAPAttribute2", "", "", "");
insert into tableDefaults values ("SSLVPNGroup", "LDAPAttribute3", "", "", "");
insert into tableDefaults values ("SSLVPNGroup", "LDAPAttribute4", "", "", "");
insert into stringsMap values ("SSLVPN_GROUP_CONFIG_FAILED","Groups Configuration Failed");
insert into stringsMap values ("SSLVPN_GROUP_HAS_USER_DEL_FAILED","Group cannot be deleted if it contains User(s)");
insert into stringsMap values ("SSLVPN_GROUP_IS_DEFAULT_DEL_FAILED", "Default Group(s) cannot be deleted");

--
-- Users Table
--
--  SPECS:
--
--  1. IPSEC user should be removed from the userType as we already have 
--      Xauth users Page.  
--
--  2. Use 'AuthenticationType' as hidden key to Identify which Groups(selected
--      from GroupList) Could support selected 'userType'. 
--      RULES:
--      An 'Adminstrator' userType can belong ONLY to a group which has "local" authenticationType. 
--
--  3. a. 'AutheticationType' is only for display purpose and for RULE @ SPECS 2(above)
--     b. 'DomainName' is only for Display Purpose @ 'list of users'. 
--     c. 'AuthenticationType' and 'DomainName' are NOT configurable for User and can be obtained 
--          using DomainName reference to the DomainTable.
--      
--  4. 'DefaultUser' indicates that the User is a system generated user and marks(*) such users.
--      Default user belongs to a default group which belongs to a default domain.
--      DefaultUser takes values 0 (NOT a default user), 1 (default user)
--
--  5. 'UserType' Mapping:
--      RULES:
--      For DropDown Entry,     
--      case Adminstrator : set UserType as 3 and display in the table as "Administrator"
--      case SSLVPN User  : set UserType as 0 and display in the table as "SSLVPN User"
--
--  6. When a User is Deleted, his/her DefinedAddresses Table and
--      DefinedBrowsers Tables should be Deleted. 
--
--  7. A Domain Should not be deleted if Users are Configured for it.
--      Error = "Domain cannot be deleted if Users are Configured for it." should be thrown.
-- 
--  8. GUI REVIEW/proposal:
--      Is it {LastName and FirstName} or {MiddleName and LastName} ?

-- ****  AdministratorPassword field is only to show in GUI.
--       No need to store this value in table. we need to call one
--       function in userLuaLib.c file written by comp dev.
--       It will validate the administrator password
--       and send ok or error, based on that value we need to update the password
--       while editing the user info. This is the password of the
--       administrator (current logged in user), the one who is trying to change the Password.
-- 

CREATE TABLE SSLVPNUser
(
    OID                     integer         NULL,
    UserName                text            NOT NULL,   --  Unique UserName(login name), PRIMARY KEY
    FirstName               text            NULL,       --  Middle Name
    LastName                text            NULL,       --  Last Name     
    GroupName               text            NOT NULL,   --  Group to which this User Belongs, FOREIGN KEY.
    UserType                integer         NOT NULL,   --  Type of the User.
    UserTimeOut             integer         NOT NULL,   --  Idle TimeOut for User, Default value 10 secs 
    DenyLogin               boolean         NOT NULL,   --  Option to Block Login, Default is FALSE(0) ie., ALLOW        
    DenyLoginFromWan        boolean         NOT NULL,   --  Option to Block Login From WAN, -  
                                                        --   - Default as FALSE(0) ie., ALLOW    
    LoginFromIP             boolean         NOT NULL,   --  Option to Allow Login From this Address. - 
                                                        --   - default value is FALSE(0) ie., Deny 
    LoginFromBrowser        boolean         NOT NULL,   --  Option to Allow Login From this Browser. - 
                                                        --   - Default is FALSE(0), ie DENY
    Password                text            NULL,       --  password / New Password in Edit page.
    DefaultUser             boolean         NULL,       --  Flag to indicate that the user is a system generated User.
    PRIMARY KEY (UserName),
    FOREIGN KEY (GroupName) REFERENCES SSLVPNGroup(GroupName)
)
;
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM", 0, 1, "SSLVPNUser", 0, 1, 1, 1);
insert into tableDefaults       values ("SSLVPNUser", "DefaultUser", "0", "", "");
insert into tableDefaults       values ("SSLVPNUser", "UserTimeOut", "10", "", "");
insert into tableDefaults       values ("SSLVPNUser", "DenyLogin", "0", "", "");
insert into tableDefaults       values ("SSLVPNUser", "DenyLoginFromWan", "0", "", "");
insert into tableDefaults       values ("SSLVPNUser", "EditPassWord", "0", "", "");
insert into tableDefaults       values ("SSLVPNUser", "LoginFromIP", "0", "", "");
insert into tableDefaults       values ("SSLVPNUser", "LoginFromBrowser", "0", "", "");
insert into stringsMap          values ("SSLVPN_USER_CONFIG_FAILED","SSLVPN User(s) Configuration Failed");
insert into stringsMap          values ("WRONG_USER_PASSWORD","Wrong Login User Password");
insert into stringsMap          values ("USER_SESSION_NOT_FOUND","Failed to get login user's session to check login users password");
insert into stringsMap          values ("USERNAME_CHANGED","User Name Changed");

--
-- UserLoginPolicy's Defined Address Table
--
-- SPECS:
--
--  1.  if any address entries are added, UserName field should be
--      filled up with the UserName for whom this address entry is getting
--      added. 
--      This was achieved platform1.0 by passing UserName across LoginPolicies tabs.
--
--  2. 'AddressType' values 
--      RULES: 
--      case IP Address: set AddressType = 1 and display as "IP Address";
--      case IP Network: set AddressType = 2 and display as "IP Network";
--
--  3. If addressType is 'IP Network', MaskLength field should be enabled/editable .
--     If addressType is 'IP Address', MaskLength field should be disabled/Not editable but should set value as 32.
--      By default 'MaskLength' is set to 32 so that it can be used a PRIMARY KEY, as in case of IP Networks two 
--      different MaskLength can be applied to same Network Address. 
--
--  4. For a Given User : 
--      a. Address and Mask Lenth combination Forms the PRIMARY KEY.  
--      b. All the configured IPs are subjected to same LoginPolicy PERMISSION. 
--      c. Obviously same UserName.
--
--  5. When a user is deleted, all the entries of DefinedAddress of that user
--      should also be deleted.
--  
--

CREATE TABLE SSLVPNUserLoginPolicyDefinedAddress
(
    OID                     integer         NULL,
    UserName                text            NOT NULL,       -- UserName to whom these Addresses Apply, FOREIGN KEY
    AddressType             integer         NOT NULL,       -- Option to indicate that Address is a single or a Network.
    Address                 text            NOT NULL,       -- IP Address string. 
    MaskLength              integer         NOT NULL,       -- Mask Length. 
    PRIMARY KEY (UserName, Address, MaskLength),
--   FOREIGN KEY (UserName) REFERENCES SSLVPNUser (UserName) 
    FOREIGN KEY (UserName) REFERENCES USERDBUsers (UserName)
)
;
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM", 0, 1, "SSLVPNUserLoginPolicyDefinedAddress", 0, 1, 1, 1);
insert into tableDefaults       values ("SSLVPNUserLoginPolicyDefinedAddress", "MaskLength", "32", "", "");

--
-- USerLoginPolicy's Defined Browsers Table
--
-- SPECS:
--
--  1. 'ClientBrowserName' Values:
--      RULES:
--      case Internet Expolorer : 
--        set ClientBrowserName value as "MSIE" and  display as "Internet Explorer"
--      case Opera :
--        set value as "Opera"
--      case Netscape Navigator:
--        set value as "Navigator" and display as "Netscape Navigator"
--      case Firefox :
--        set value as "Firefox"
--      case Mozilla:
--        set value as "Mozilla" 
--
--  2.  if any Browsers entries are added, UserName field should be
--      filled up with the UserName for whom this address entry is getting
--      added.
--
--  4. For a Given User : 
--      a. All the Configured Browsers are subjected to same loginPolicy PERMISSION
--      b. Obviously same UserName .
--
--  5. When a user is deleted, all the entries of DefinedBrowser of that user
--      should also be deleted.
--
--

CREATE TABLE SSLVPNUserLoginPolicyDefinedBrowser
(
    OID                     integer         NULL,
    UserName                text            NOT NULL,       -- For whom this Browser Rule Apply.
    ClientBrowserName       text            NOT NULL,       -- Browser Name 
    PRIMARY KEY (UserName, ClientBrowserName),
--    FOREIGN KEY (UserName) REFERENCES SSLVPNUser (UserName)
    FOREIGN KEY (UserName) REFERENCES USERDBUsers (UserName)  
)
;
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM", 0, 1, "SSLVPNUserLoginPolicyDefinedBrowser", 0, 1, 1, 1);

--
-- Resource Table
--
--  SPEC:
--  
--  1. 'ServiceType' values mapping:
--      RULES:
--      case VPN Tunnel      : set value as "VIRTUAL-PASSAGE" and display as 'VPN Tunnel'
--      case Port Forwarding : set value as "VIRTUAL-TRANSPORT" and display as 'Port Forwarding'
--      case All             : set value as "all" and display as 'All'
--
--  2. 'UseCount' is a counter of number of policies using a particular Resource.
--      when a resource is created, initiate the counter to 0.
--
--  3. Cannot delete a Resource if some Policy is using it.
--     i.e, if UseCount > 0, donot delete the Resource.
--      Error : "Resource in use, Cannot delete the Resource" should be Thrown.
--  
--  4. 'UseCount' for a resource should be updated when ever a corresponding policy updates its 'resource'.
--      i.e, 'UseCount' for a resource(r1) should be incremented where ever a policy configures
--      'ResourceName' as 'r1' and decremented when the policy changes its ResourceName from r1 to r2 or 
--      when the policy Configuring r1 as its resourceName is deleted.
--      The 'UseCount' has to be updated using 'ResourceName' reference to 'ResourceTable' in the 'PolicyTable'.
--

CREATE TABLE SSLVPNResource
(
    OID                     integer         NULL,
    ResourceName            text            NOT NULL,   -- Unique Resource Name, PRIMARY KEY.
    ServiceType             text            NOT NULL,   -- Service type on which the Resource is Configured.
    UseCount                integer         NULL,       -- The number of Policies configured over this ResourceName.
    PRIMARY KEY (ResourceName)
)
;

insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM", 0, 1, "SSLVPNResource", 0, 1, 1, 1);
insert into tableDefaults values ("SSLVPNResource", "UseCount", "0", "", "");
insert into stringsMap values ("SSLVPN_RESOURCE_CONFIG_FAILED","SSLVPN Resource(s) Configuration Failed");
insert into stringsMap values ("SSLVPN_RESOURCE_IN_USE_DELETE_FAILED","Resource in use, Cannot delete the Resource");
insert into stringsMap values ("NO_RESOURCE_OBJECTS_CONFIGURED","No Objects Configured for the selected Resource");
insert into stringsMap values ("CANNOT_DELETE_ONLY_RESOURCE","Deleting the Only resourceObject of a resource will make the corresponding policy IN EFFECTIVE");

--
-- ResourceObject Table 
-- 
-- SPECS:
--
--  1. Every ResourceObject entry belongs to one of the Reource.
--      So, ResourceName should be passed to the page 'ResourceEdit'
--      where we configure ResourceObjects.
--  
--  2. Port ="BeginPort-EndPort"; if EndPort is Entered in GUI
--     Port = BeginPort         ; if EndPort is NOT Entered in GUI
--  
--  3. All the ResourceObjects belonging to a Resource should be deleted
--      when corresponding Resource is Deleted.
--  
--  4. 'ObjectType' Mapping
--     RULES:
--      For DropDown selected(Object type) @ resource Configure,     
--      case IP Address: set ObjectType = 1 and display In the table as "IP Address";
--      case IP Network: set ObjectType = 2 and display In the table as "IP Network";
--
--  5. GUI REVIEW : field 'IP Address/Name' should be 'IP Address' @ resourceConfigure(vpn->sslvpn->resource->add)  
--      As FQDN is not supported in the Backend.
-- 
--  6. GUI REVIEW : No need of 2 Fields : IP address/Name, Network Address @ resourceConfigure
--      (vpn->sslvpn->resource->add) Rather we can sigle Field 'ObjectAddress'
--
--  7. if 6. is implemented:
--      When ObjectType == IP Network, Only then 'Mask Length' should be Enabled.
--     else
--      When ObjectType == IP Network, Only then both 'Network Address' and 'Mask Length' should be Enabled.             
--
--  8. GUI REVIEW : No Resource Objects Table in the Resource Add page.
--  
--  9. DEfault Value is Given to port is " "(single space), just to make Port as PRIMARY KEY
--      And in the Backend, we will strip any spaces to pass empty string, if user Didnot 
--      configure port.    
--

CREATE TABLE SSLVPNResourceObject
(
    OID                     integer         NULL,
    ResourceName            text            NOT NULL,       --  Resource to which this Object Belongs to. 
    ObjectType              integer         NOT NULL,       --  IP Address / IP Network
                                                            --   - If IP Address, MaskLength will be ignored by backend.
    ObjectAddress           text            NOT NULL,       --  Address of the Object
    MaskLength              integer         NOT NULL,       --  Mask length, default set to 32 
    StartPort               integer         NULL,           --  Start Port.
    EndPort                 integer         NULL,           --  End Port.    
    PRIMARY KEY (ResourceName, ObjectAddress, MaskLength, StartPort, EndPort),
    FOREIGN KEY (ResourceName) REFERENCES SSLVPNResource (ResourceName)
)
;

insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM", 0, 1, "SSLVPNResourceObject", 0, 1, 1, 1);
insert into tableDefaults values ("SSLVPNResourceObject", "Port", " ", "", ""); -- default as " " single space
insert into tableDefaults values ("SSLVPNResourceObject", "MaskLength", "32", "", "");
insert into stringsMap values ("SSLVPN_RESOURCE_IN_USE_DEL_POLCY_FIRST","Resource is in use, delete the policy first");
insert into stringsMap values ("SSLRESOURCE_OBJECT_DELETE_FAILED","Resource Object Delete Failed");
insert into stringsMap values ("RESOURCE_DELETE_FAILED","Resource delete failed");
insert into stringsMap values ("RESOURCE_DELETE_OK","selected Resources Deleted");

--
-- Policies Table
--
--  NEEDED FURTHER DISCUSSION:
--  ! regarding GUI design changes for policies add page and display entries.
--
-- SPECS:
-- 
--  1. 'PolicyType' mapping,
--      RULE:
--      For Radio Button checked(Policy for) @ List/Add pages
--      case Global : set value as "global"  
--      case Group  : set value as "group"  
--      case User   : set value as "user"
--
--  2. Port ='BeginPort-EndPort'; if EndPort is Entered in GUI
--     Port = BeginPOrt         ; if EndPort is NOT Entered in GUI
--
--  3. 'DestinationObjectType' Mapping,
--      RULE:
--      For Radio Button Checked(Apply Policy to ?) @ Add page, 
--      case Network Resource : set value as '1'
--      case Ip address       : set value as '2'
--      case IP Network       : set value as '3'
--      case All Addresses    : set value as '4'
--  
--  4. 'ServiceType' mapping:
--      RULES:
--      For DropDown Entry(Service) @ Add page,
--      case VPN Tunnel      : set value as "VIRTUAL-PASSAGE" and display as 'VPN Tunnel'
--      case Port Forwarding : set value as "VIRTUAL-TRANSPORT" and display as 'Port Forwarding'
--      case All             : set value as "all" and display as 'All'
-- 
--  5. 'Permission' mapping:
--      RULE:
--      For DropDown Entry @ Add/Edit page,
--      case PERMIT : set value as "Permit" and display as 'Permit'
--      case DENY   : set value as "Deny" and display as 'Deny'
--      To avoid confusion we can have same text at drop down and also while
--      displaying the Configured Policies.
--
--  6. Do not allow a Policy to be ADDED over a Network Resource if the NO
--      objects are configured for that resource.
--     i.e, For a policy to be added for a Network resource say r1, There should
--     atleast one row in the ResourceObjects table with 'r1' as its ResourceName.  
--
--  7. PROPOSAL :
--      'DestinationObject'  Mapping(For Displaying destination in the Policies List) 
--      RULE: 
--      For 'DestinationType'
--      case Network Resource(1) : set value same as "$ResourceName"
--      case IP Address      (2) : set value same as "$Address":"$Port"
--      case IP Network      (3) : set value same as "$Address"/"$maskLength":"$Port" 
--      case All Addresss    (4) : set value as "All"
-- 
--  8. when a policy is configured over a ResourceName, the Corresponding Resource's 
--      UseCount should be properly updated.
--  
--  9. 'PolicyOwnerName' runTime Mapping:
--      RULE :
--      When policyType is 
--      case global : set value as "global" 
--      case group  : set value as $GroupName selected from GroupList
--      case user   : set value as $UserName selected from UserList
--

CREATE TABLE SSLVPNPolicy
(
    OID                     integer         NULL,
    PolicyName              text            NOT NULL,       -- Policy Name, ID of policy for a policyOwner 
    PolicyType              text            NOT NULL,       -- Policy Type one of {"global", "group", "user"}
    PolicyOwnerName         text            NOT NULL,       -- fill based on Policy Type 
    DestinationObjectType   integer         NOT NULL,       -- Apply Policy to ? 
                                                            --  one of {NW Resoure, IP address, IP Network, All Address}
    Permission              text            NOT NULL,       -- policy is to PERMIT / DENY. Default value set is PERMIT
    DestinationObject       text            NULL,           -- The Fianl Value of Destination Object.
    ServiceType             text            NULL,           -- Service Type on which this Policy is configured.    
    Address                 text            NULL,           -- IP Address Input field.    
    MaskLength              integer         NULL,           -- Proposal to replace subnetmask
    ResourceName            text            NULL,           -- ResourceName if Destination Type is a NW resource. 
    StartPort               integer         NULL,           --  Start Port.
    EndPort                 integer         NULL,           --  End Port.
    PRIMARY KEY (PolicyType, PolicyOwnerName, PolicyName),
    FOREIGN KEY (ResourceName) REFERENCES SSLVPNResource (ResourceName)
)
;

insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM", 0, 1, "SSLVPNPolicy", 0, 1, 1, 1);
insert into tableDefaults       values ("SSLVPNPolicy", "PolicyOwnerName", "global", "", "");
insert into tableDefaults       values ("SSLVPNPolicy", "Permission", "Permit", "", ""); 
insert into stringsMap          values ("SSLVPN_POLICY_CONFIG_FAILED","SSLVPN Policy Configuration Failed");
insert into stringsMap          values ("SSLVPN_POLICY_ADD_FAILED_CONF_RESOURCE_FIRST","Configure the resource first before adding the Policy");

--
-- Client Configuration
--
--  GUI REVIEW : Proposal for addition of Option(checkBox/radioButton) to Configure DNS Attributes.
--

CREATE TABLE SSLVPNClientConf
(
    OID                 integer             NULL,
    EnableFullTunnel    boolean             NOT NULL,   --  Default is 0 (Unchecked, SPLIT TUNNEL MODE).  
    DNSSuffix           text                NULL,       --  DNS suuffix.       
    PrimaryDNSServer    text                NULL,       --  Primary DNS server.  
    SecondaryDNSServer  text                NULL,       --  secondary DNS server.
    BeginClientAddress  text                NOT NULL,   --  Begin Address, Default is : 192.168.251.1
    EndClientAddress    text                NOT NULL,   --  End Address,   Default is : 192.168.251.254
    lcpTimeout          text                NULL       --  lcpTimeout,   Default is : 60
)
;

insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM", 0, 1, "SSLVPNClientConf", 0, 1, 1, 1);
insert into tableDefaults       values ("SSLVPNClientConf", "EnableFullTunnel", "0", "", "");
insert into tableDefaults       values ("SSLVPNClientConf", "BeginClientAddress", "192.168.251.1", "", "");
insert into tableDefaults       values ("SSLVPNClientConf", "EndClientAddress", "192.168.251.254", "", "");
insert into tableDefaults       values ("SSLVPNClientConf", "lcpTimeout", "60", "0", "999999");

--
-- Client Routes
--

CREATE TABLE SSLVPNClientRoute
(
    OID                 integer             NULL,
    DestinationNetwork  text                NULL,   --  Destination Network (LAN side)
    SubnetMask          text                NULL,   --  subnet mask for above network 
    PRIMARY KEY (DestinationNetwork, SubnetMask)
)
;

insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM", 0, 1, "SSLVPNClientRoute", 0, 1, 1, 1);
insert into stringsMap values ("SSL_VPN_CLIENT_ROUTE_CONFIG_FAILED", "SSL VPN client routes configuration failed");

--
-- PortForwarding Apps Map
--

CREATE TABLE SSLVPNPortForwardingAppsMap
(
    OID                 integer             NULL,
    ServerIP            text                NOT NULL,   -- Server Address.
    Port                integer             NOT NULL,   -- Port accesses on the above server.
                                                        -- PortForwarding rule is added using above 2 values.     
    PRIMARY KEY (ServerIP, Port)
)
;

insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM", 0, 1, "SSLVPNPortForwardingAppsMap", 0, 1, 1, 1);

--
-- PortForwarding hosts Map
--

CREATE TABLE SSLVPNPortForwardingFQDNMap
(
    OID                 integer             NULL,
    ServerIP            text                NOT NULL,   -- Server Address.
--    ServerFQDN          text                NOT NULL,   -- FQDN binding for above address.
    FQDNName            text                NOT NULL,   -- FQDN binding for above address.
--    PRIMARY KEY (ServerIP, ServerFQDN)
    PRIMARY KEY (ServerIP, FQDNName)
)
;

insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM", 0, 1, "SSLVPNPortForwardingFQDNMap", 0, 1, 1, 1);

--
-- Active Users / SSL VPN connection status 
-- 
-- The inputs are not given by the user, but are generated by the system basing 
-- on the logged in User's details.
--  
-- This table's fields are not configurable. Rather The Row entries are DELETED based on the SessionID and
-- are added by the Backened When a User Logins.
--  
CREATE TABLE SSLVPNActiveUsers
(
    OID                 integer             NULL,
    SessionID           text                NOT NULL,   -- Unique session Name, hidden, Needed by Backend API    
    Cookie              text                NULL,       -- This might NOT b required if sessionName == Cookie    
    UserName            text                NOT NULL,   -- Logged in User Name.
    GroupName           text                NULL,       -- Group of Logged in User.
    LoginAddress        text                NULL,       -- IP Address from which the User had logged in to the Portal.
    LoginTime           text                NULL,       -- Time of Login.
    LastAccessTime text                     NULL,       -- last access time
    PPPInterface        text                NULL,       -- ppp inteface name
    PPPInterfaceIP      text                NULL,       -- ppp inteface's IP
    PeerPPPInterfaceIP  text                NULL,       -- peer PPP interface's IP
    TxPackets           text                NULL,
    TxPacketsDropped    text                NULL,  
    TxBytes             text                NULL,
    RxPackets           text                NULL,
    RxPacketsDropped    text                NULL, 
    RxBytes             text                NULL,
    RefererValue	text		    NULL,	-- Http referer Value
    
    PRIMARY KEY (UserName)
)
;
-- Nor really sure that the flags are properly set ?
insert into stringsMap values ("SSL_VPN_USER_DISCONNECT_SUCCESS", "SSL VPN User Disconnected Successfully");
insert into stringsMap values ("SSL_VPN_USER_DISCONNECT_FAILED", "SSL VPN User State Reset Failed");

-- Supporting Table to Track/Update ActiveUsers
--  Proposal : 
--  a. When ever a user is created an entry into 'SSLVPNUserState' should also be added.
--  b. when the ActiveUsers is Populated by a 'ActiveUsersGet' binary(TODO), the binary should also
--      update/set 'ActiveUser' entries in the 'SSLVPNUserState' table.
--  c. when 'disconnect' button is clicked, 'ActiveUser' flag for that user should be reset to 0.
--  d. when user is Deleted from DB, the corresponding user's entry in this table should also be deleted.
--  e. By default the users state is added as 0.(Inactive/NOT logged in) 
--
CREATE TABLE SSLVPNUserState
(
    OID                 integer             NULL,
    UserName            text                NOT NULL,       -- Logged in User Name.
    ActiveUser          boolean             NOT NULL,       -- flag to identify whether User is Currently 
                                                            -- logged into the box.
                                                            -- 0 : Not Logged in, 
    PRIMARY KEY (UserName)
)
;
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM", 0, 1, "SSLVPNUserState", 0, 1, 1, 1);
insert into tableDefaults values ("SSLVPNUserState", "ActiveUser", "0", "", "");
insert into stringsMap values ("VPN_SSLVPNCLIENT_CONFIG_FAILED","SSLVPN Client Config Failed");
insert into stringsMap values ("SSLVPN_PORTFORWARDING_CONFIG_FAILED","PortForwarding Config Failed");

CREATE TABLE httpsMgmt
(
    httpsMgmtPort integer NOT NULL,
    certEvent     integer NULL
)
;
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM", 0, 1, "httpsMgmt", 0, 1, 1, 1);
--insert into tableDefaults values ("httpsMgmt", "certEvent", "2", "", "");

-- Login related String Maps
insert into stringsMap values ("LOGIN_INVALID_PASSWORD","Invalid username or password");
insert into stringsMap values ("LOGIN_SESSION_EXPIRED","Session expired. Please login.");
insert into stringsMap values ("LOGIN_SESSION_TERMINATED","Your session has been terminated.");
insert into stringsMap values ("LOGIN_USER_ALREADY_LOGGED_IN","user already logged in");

CREATE TABLE Dmz
(
    OID integer,
    DmzEnable boolean NOT NULL,
    DmzIP text
);
insert into tableDefaults values ("Dmz","DmzEnable","0","0","1");
insert into stringsMap values ("DMZ_CONFIG_FAILED","DMZ Configuration Failed");
--Add this enty when dmz rule is not added from Firewall rules
--insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",1,1,"Dmz",0,1,1,1);   
-- FirmConf - table for firm check configuration
CREATE TABLE FirmConf
    (
    CheckNow Boolean NOT NULL          -- triggers the backend 
    );

-- FirmStatus - table for firmware status
CREATE TABLE FirmStatus
    (
    StatusMsg       Text,              -- status message
    StatusAvailable Integer,           -- status available
    ImageName       Text,              -- name of the image
    DownloadLink    Text,              -- firmware download link
    ReleaseNotes    Text,              -- firmware release notes link
    RegLink         Text,              -- firmware Registration Link
    UrlQuery        Text               -- Url used for Communication
    );


INSERT INTO dbUpdateRegisterTbl VALUES ("UMI_COMP_PLATFORM", 0, 1, "FirmConf", 0, 1, 1, 1);
INSERT INTO dbUpdateRegisterProgram VALUES ("/pfrm2.0/bin/firmCheckConfig", 1, 1, "FirmConf", 0, 1, 1, 0);
INSERT INTO FirmConf VALUES (0);
INSERT INTO FirmStatus VALUES ("", 0, "","","","","");
insert into stringsMap values ("FIRMWARE_INVALID_FILE","Firmware file is invalid or corrupted");
CREATE TABLE SystemStatistics
(
        OID integer ,

        CpuUsed 	integer NOT NULL,  /* CPU Total Usage */
        CpuIdle 	integer NOT NULL,  /* CPU idle */
        CpuUsedByUser 	integer NOT NULL,  /* CPU taken by user process */
        CpuUsedByKernel integer NOT NULL,  /* CPU taken by kernel process */
        CpuWaitingForIO integer NOT NULL,  /* Process waiting for IO operations*/

        MemoryFree  	integer NOT NULL,  /* Free Memory in the system */
        MemoryTotal 	integer NOT NULL,  /* Total memory of the system */
        MemoryUsed  	integer NOT NULL,  /* Used memory in the system */ 
        MemoryCached  	integer NOT NULL,  /* cached memory in the system */
        MemoryBuffers  	integer NOT NULL   /* buffered memory */
)
;


-- This file includes sslvpn component related Tables like -
--
--  1. Userdb Users
--  2. Userdb Groups
--  3. Userdb Domains
--  4. Get Users Db 

--
-- Users Table
--
--  SPECS:
--
--  1. IPSEC user should be removed from the userType as we already have 
--      Xauth users Page.  
--
--  2. Use 'AuthenticationType' as hidden key to Identify which Groups(selected
--      from GroupList) Could support selected 'userType'. 
--      RULES:
--      An 'Adminstrator' userType can belong ONLY to a group which has "local" authenticationType. 
--
--  3. a. 'AutheticationType' is only for display purpose and for RULE @ SPECS 2(above)
--     b. 'DomainName' is only for Display Purpose @ 'list of users'. 
--     c. 'AuthenticationType' and 'DomainName' are NOT configurable for User and can be obtained 
--          using DomainName reference to the DomainTable.
--      
--  4. 'DefaultUser' indicates that the User is a system generated user and marks(*) such users.
--      Default user belongs to a default group which belongs to a default domain.
--      DefaultUser takes values 0 (NOT a default user), 1 (default user)
--
--  5. 'UserType' Mapping:
--      RULES:
--      For DropDown Entry,     
--      case Adminstrator : set UserType as 3 and display in the table as "Administrator"
--      case USERDB_User  : set UserType as 0 and display in the table as "USERDB User"
--
--  6. When a User is Deleted, his/her DefinedAddresses Table and
--      DefinedBrowsers Tables should be Deleted. 
--
--  7. A Domain Should not be deleted if Users are Configured for it.
--      Error = "Domain cannot be deleted if Users are Configured for it." should be thrown.
-- 
--  8. GUI REVIEW/proposal:
--      Is it {LastName and FirstName} or {MiddleName and LastName} ?

-- ****  AdministratorPassword field is only to show in GUI.
--       No need to store this value in table. we need to call one
--       function in userLuaLib.c file written by comp dev.
--       It will validate the administrator password
--       and send ok or error, based on that value we need to update the password
--       while editing the user info. This is the password of the
--       administrator (current logged in user), the one who is trying to change the Password.
-- 

CREATE TABLE USERDBUsers
(
    OID                     integer         NULL,
    UserName                text            NOT NULL,   
    FirstName               text            NULL,      
    LastName                text            NULL,     
    GroupName               text            NOT NULL,
    UserType                integer         NULL,   --  Type of the User.
    UserTimeOut             integer         NOT NULL,   
    groupid                 integer         NOT NULL,
    AllowChangePassword     boolean         NULL,
    DenyLogin               boolean         NOT NULL,  
    DenyLoginFromWan        boolean         NOT NULL, 
    LoginFromIP             boolean         NOT NULL,  
    LoginFromBrowser        boolean         NOT NULL,  
    Password                text            NULL,    
    DefaultUser             boolean         NULL,   
    PRIMARY KEY (UserName),
    FOREIGN KEY (GroupName) REFERENCES UserGroup(GroupName)
);

insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM", 0, 1, "USERDBUsers", 0, 1, 1, 1);
insert into tableDefaults       values ("USERDBUsers", "DefaultUser", "0", "", "");
insert into tableDefaults       values ("USERDBUsers", "UserTimeOut", "10", "", "");
insert into tableDefaults       values ("USERDBUsers", "DenyLogin", "0", "", "");
insert into tableDefaults       values ("USERDBUsers", "DenyLoginFromWan", "0", "", "");
insert into tableDefaults       values ("USERDBUsers", "EditPassWord", "0", "", "");
insert into tableDefaults       values ("USERDBUsers", "LoginFromIP", "0", "", "");
insert into tableDefaults       values ("USERDBUsers", "LoginFromBrowser", "0", "", "");
insert into tableDefaults       values ("USERDBUsers", "groupid", "0", "", "");
insert into stringsMap          values ("USERDB_USER_CONFIG_FAILED","USERDB User(s) Configuration Failed");
insert into stringsMap          values ("USERDB_WRONG_USER_PASSWORD","Wrong Login User Password");
insert into stringsMap          values ("USERDB_USER_SESSION_NOT_FOUND","Failed to get login user's session to check login users password");
insert into stringsMap          values ("USERDB_USERNAME_CHANGED","User Name Changed");

insert into tableDefaults values ("USERDBUsers","loginTimeout","5", "1", "999");
insert into stringsMap values ("USERDB_CONFIG_FAILED","User configuration failed");
insert into stringsMap values ("USERDB_SUCCEEDED","User Database stored successfully");
insert into stringsMap values ("USERDB_FAILED","Storing User Database failed");


insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"XAuthUsers",0,1,1,1);

insert into tableDefaults values ("USERDBUsers", "UserType", "0", "", "");


--
-- Groups Table
--
--  SPECS:
--
--  1. In Group Edit Page,
--      RULE:
--      Ldap Attributes fields should be enabled only if AuthenticationType == 'ldap'.
--      'AuthenticationType' for a Group could be obtained from DomainTable using the 'DomainName'
--      refrence .
--   
--  2. 'DefaultGroup' is the attribute to indicate that it's created along with the
--      creation of a domain. It is used as hidden key to mark(*) in GUI as Default
--      Group.
--      DefaultGroup takes values = 0 (Not a default Group), 1 (Default group).
--      Default Groups cannot be deleted.
--
--  3. when Domain is created, its corresponding Default group(group with the same name as that of Domain)
--      should also be created in the GroupTable and the DefaultGroup flag should be set to 1 for that Group.
      
--  4. Group, if has Users Configured for it, cannot be deleted.
--      Error : "Group cannot be deleted if Users are configured for it" should be thrown.
--
--  5. GUI REVIEW/proposal:
--      a. Can the LDAP Attributes be moved to ADD Page so that the user dont miss configuring
--          LDAP attributes.(Presently LDAP Attr are configured in EDIT page of Groups)
--      b. If LDAP Attributes are Moved to ADD page, 
--          We need to use AuthenticationType of selected DomainName to enable or disable the LDAP Attributes.   
--          The above same mechanism is followed in the EDIT page too.
-- 

CREATE TABLE USERDBGroups
(
    OID                         integer     NULL,
    GroupName                   text        NOT NULL,   --  Unique Group name, PRIMARY KEY.
    DomainName                  text        NOT NULL,   --  Domain to which This Group Belongs.       
    GroupTimeOut                integer     NOT NULL,   --  Idle TimeOut of Group, Defaul is 10.
    LDAPAttribute1              text        NULL,       --  LDAP Attributes.
    LDAPAttribute2              text        NULL,       --  LDAP Attributes.
    LDAPAttribute3              text        NULL,       --  LDAP Attributes.
    LDAPAttribute4              text        NULL,       --  LDAP Attributes.    
    DefaultGroup                boolean     NULL,       --  Flag to indicate that the Group is a Domain Generated Group.
                                                        --  Default valu is set to 0.
    PRIMARY KEY (GroupName),
    FOREIGN KEY (DomainName) REFERENCES USERDBDomains (DomainName)    
)
;

insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM", 0, 1, "USERDBGroups", 0, 1, 1, 1);
insert into tableDefaults values ("USERDBGroups", "DefaultGroup", "0", "", "");
insert into tableDefaults values ("USERDBGroups", "GroupTimeOut", "5", "", "");
insert into tableDefaults values ("USERDBGroups", "LDAPAttribute1", "", "", "");
insert into tableDefaults values ("USERDBGroups", "LDAPAttribute2", "", "", "");
insert into tableDefaults values ("USERDBGroups", "LDAPAttribute3", "", "", "");
insert into tableDefaults values ("USERDBGroups", "LDAPAttribute4", "", "", "");
insert into stringsMap values ("USERDB_GROUP_CONFIG_FAILED","Groups Configuration Failed");
insert into stringsMap values ("USERDB_GROUP_HAS_USER_DEL_FAILED","Group cannot be deleted if it contains User(s)");
insert into stringsMap values ("USERDB_GROUP_IS_DEFAULT_DEL_FAILED", "Default Group(s) cannot be deleted");

--
-- Domains Table
--
-- SPECS :
--
--  1. 'AuthenticationType', decides which Fields other than DomainName and
--      PortalLayoutName, should get disabled or enabled in the Add/Edit pages
--      of Domain configuration. In other words, Only related fields of 
--      'AuthenticationType' should be Enabled. 
--     Please refer platform1.0's add domain page.
--
--  2. 'AuthenticationType' Mapping : 
--      RULES:
--           For DropDown Entry,
--                             [GUI]   : [values needed by Backend]
--           case local User Database  : set value as "local";
--           case Radius-PAP           : set value as "radius_pap";
--           case Radius-CHAP          : set value as "radius_chap";
--           case Radius-MSCHAP        : set value as "radius_mschap";
--           case Radius-MACHAPv2      : set value as "radius_mschapv2";
--           case NT Domain            : set value as "ntdomain";
--           case Active Directory     : set value as "active-directory";
--           case LDAP                 : set value as "ldap";			
--    To avoid any Confusion we can display in the DropDown List of
--    Authentication Type, the same values what we set for 'AuthenticationType'. 
--
--  3. 'DefaultDomain' is a flag to indicate that Domain is system generated.  
--      It is used as hidden key to mark(*) in GUI as Default Domain.
--      'DefaultDomain' takes values 0 (not a default), 1 (default Domain).
--      Default Domains Cannot be deleted.
--
--  4. A Domain Should not be deleted if Users are Configured for it.
--      Error = "Domain cannot be deleted if Users are Configured for it." should be thrown.
-- 
--  5. when a Domain is created, its corresponding Default group(group with the same name as that of Domain)
--      should also be created in the GroupTable and the DefaultGroup flag should be set to 1 for that Group.
-- 
--  6. Domain Should NOT be Deleted if its portal is a Default Portal(ie when DefaultPortal == 1 or 2)
--      Error: "Domain Cannot if its portal is set as Default".
--     For a Domain, 'DefaultPortal' value  of its portal can be obtained using 
--      portalLayoutName reference to PortalLayoutTable. 
--          

CREATE TABLE USERDBDomains
(
    OID                         integer     NULL,
    DomainName                  text        NOT NULL,   -- Unique DomainName, PRIMARY KEY
    PortalLayoutName            text        NOT NULL,   -- Portal for this Domain. 
    AuthenticationType          text        NOT NULL,   -- Authentication type.
    AuthenticationServer        text        NULL,       -- Authenticating server address.
    SecondAuthenticationServer  text        NULL,       -- Authenticating server address.
    ThirdAuthenticationServer   text        NULL,       -- Authenticating server address 3.
    TimeOut                     integer     NOT NULL,   -- Timeout
    Retries                     integer     NOT NULL,   -- Number of retries
    AuthenticationRadiusSecret  text        NULL,       -- Radius secret.
    SecondAuthenticationRadiusSecret  text  NULL,       -- Radius secret.
    NTDomainWorkGroup           text        NULL,       -- Work Group for NT Domain.
    SecondNTDomainWorkGroup     text        NULL,       -- Second Work Group for NT Domain.
    LDAPBaseDN                  text        NULL,       -- Base DomainName for LDAP Authentication.
    SecondLDAPBaseDN            text        NULL,       -- Second Base DomainName for LDAP Authentication.
    ActiveDirectoryDomain       text        NULL,       -- AD domain Name for AD Authentication.
    SecondActiveDirectoryDomain text        NULL,       -- Second AD domain Name for AD Authentication.
    DefaultDomain               boolean     NULL,       -- Flag to indicate that the domain is a System Generated Domain.                                                            -- Default value is set to 0.    
    PRIMARY KEY (DomainName),
    FOREIGN KEY (PortalLayoutName) REFERENCES SSLVPNPortalLayout (PortalLayoutName)

)
;

insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM", 0, 1, "USERDBDomains", 0, 1, 1, 1);
insert into tableDefaults values ("USERDBDomains", "DefaultDomain", "0", "", "");
insert into stringsMap values ("USERDB_DOMAIN_CONFIG_FAILED","Domain(s) Configuration Failed");
insert into stringsMap values ("USERDB_DOMAIN_HAS_USERS_DOM_DEL_FAILED","Can not Delete a Domain if it contains User(s)");
insert into stringsMap values ("USERDB_PORTAL_IS_DEFAULT_DOMAIN_DEL_FAILED","Cannot delete the Only domain, As its portal is set as Default");
insert into stringsMap values ("USERDB_PORTAL_IS_DEFAULT_DOMAIN_EDIT_FAILED","Cannot change the protal for the only domain, As its portal is set as Default");


--
-- GetUsersDb Table
--
-- SPECS :
--
--  1. 'GetUsersDbFile' This field contails the file name information. The file is in '.csv' format and the file 
--     is processed by the corresponding handler.
--


CREATE TABLE GETUSERSDb
(
    OID                         integer     NULL,
    GetUsersDbFile              text        NOT NULL
)
;

insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM", 0, 1, "GETUSERSDb", 0, 1, 1, 1);

-- Tables used for 3G component.
--
-- This table stores the configuration related to a 3G WAN connection.
--

-- ThreeG table
CREATE TABLE ThreeG
    (
    OID                      Integer,            -- OID
    LogicalIfName            Text     NOT NULL,  -- interface name link WAN1, WAN2
    Country                  Text,               -- country
    IspName                  Text,               -- ISP Name
    UserName                 Text,               -- user name
    Password                 Text,               -- password
    DialNumber               Text     NOT NULL,  -- dialing number
    AuthMethod               Integer  NOT NULL,  -- authentication protocol
    Apn                      text     NOT NULL,  -- Access Point Name
    GetDnsFromIsp            boolean  NOT NULL,  -- do we need DNS from ISP ?
    PrimaryDns               text,               -- if we want to specify our DNS
    SecondaryDns             text,               -- if we want to specify our DNS
    IdleTimeOutFlag          boolean  NOT NULL,  -- do we require idle timeout ?
    IdleTimeOutValue         integer  NOT NULL,  -- idle timeout value  
    -- add your new entries here

    PRIMARY KEY (LogicalIfName),                 -- primary key
    FOREIGN KEY (LogicalIfName) REFERENCES networkInterface (LogicalIfName)
    );
create table IPSConfig
    (
    OID             integer NOT NULL,
    idsEnable       boolean NOT NULL,
    ipsEnable       boolean NOT NULL,
    ipsAcclEnable   boolean,
    sigUpdateInterval   integer NOT NULL -- in hours
    );

create table IPSStatus
    (
    numRules    integer NOT NULL,
    numAlerts   integer NOT NULL,
    numDrops    integer NOT NULL,
    numRejects  integer NOT NULL
    );
insert into IPSStatus values (0, 0, 0, 0);
 
create table IPSNetworkSegments
    (
    OID             integer NOT NULL,
    FromZoneType    text NOT NULL,
    ToZoneType      text NOT NULL,
    ipsEnable       boolean NOT NULL,
    FOREIGN KEY (FromZoneType) REFERENCES ZoneTypes (ZoneType),
    FOREIGN KEY (ToZoneType) REFERENCES ZoneTypes (ZoneType)
    );

-- following table is used to build snort.conf. 1 row corresponds to 1 line
-- if option value is NULL, "optionType optionName" is added.
-- otherwise "optionType optionName: optionValue" is added. 
-- Some optionType/optionName might have a different format, in which case
-- backend can check for optionType/optionName and do special handling.
-- "managed" indicate if a particular option is exposed to user for
-- modification. This field will help, configSave() function to
-- only export "managed" entries to config file.

create table IPSAdvConfig
    (
    OID     integer NOT NULL,
    optionType  text NOT NULL, -- for example, "var", "preprocessor" etc
    optionName  text NOT NULL, -- name of the "var","preprocessor" etc
    optionValue text,          -- string containing value
    managed     boolean      
    );

-- following table generates threshold.conf
create table IPSThresholdConf
    (
    OID     integer NOT NULL,
    commandType text NOT NULL,  -- "threshold" or "suppression"
    gen_id      int NOT NULL,
    sig_id      int NOT NULL,
    limitType   text,       -- "limit", "threshold", or "both"
    trackType   text NOT NULL,  -- "by_src", or "by_dst"
    count       int,
    seconds     int,
    ipaddr      text
    );

-- following table generate classification.config
create table IPSClassConf
    (
    OID     integer NOT NULL,
    className   text NOT NULL,
    description text NOT NULL,
    priority    text NOT NULL,
    PRIMARY KEY (className)
    );

create table IPSAttackResponse
    (
    OID     integer NOT NULL,
    priority    text NOT NULL,
    response    text NOT NULL,
    log     int NOT NULL,
    PRIMARY KEY (priority)
    );

create table IPSCategories
    (
    OID         integer NOT NULL,
    categoryName        text NOT NULL,
    enable          boolean NOT NULL,
    categoryFilePrefix  text NOT NULL,
    categoryAction      text NOT NULL,
    PRIMARY KEY (categoryName)
    );
    
create table IPSRuleSource
    (
    OID     integer NOT NULL,
    ruleSource  text NOT NULL, -- roule source URL
    updateInterval  integer,
    enable      boolean NOT NULL
    );
    
-- table for user added rules (exportable to config) added to local.rules
--create table IPSUserRules 
--  (
    -- For future
--  )

-- TeamF1 defined rules (not exported to config) added to local.rules
--create table IPSTf1Rules
--  (
    -- For future
--  )

--rules from third parties (not exported to config)
create table IPSRules
    (
    rulesource text NOT NULL, -- "VRT", "VRT-community", "emerging" etc
    ruleCatogry text NOT NULL, -- "web", "p2p" etc
    ruleId      int  NOT NULL, -- rule id called "sid" in rule files
    ruleRev     int  NOT NULL, -- 
    ruleClass  text NOT NULL, -- foreign key of IPSClassConf
    ruleAction text NOT NULL, -- alert/log/pass/activate/dynamic/drop/reject/sdrop
    enable     int  NOT NULL,  -- enable/disable
    msg    text NOT NULL,  -- 
    reference  text NOT NULL  -- reference section of the rule for more info on attack
    );
---
--- USB Component schema
---

---
--- This table holds information of USB current status and USB type connected to system
---

CREATE TABLE USB 
(
	ObjId integer primary key autoincrement,
	ObjRevId integer,
	USBStatus   boolean NOT NULL,-- This field contains the status of USB,
                                 -- (1) Enabled or (0) Disabled.
	USBType  integer NOT NULL,   -- This field contains the type of USB attached
                                 -- to system for ex: NAS. Possible values 
                                 -- are Printer(0)/Network Storage(1)/Network Device(2).
        usbNum integer NOT NULL
);

CREATE TABLE USB_MNT
(
        MountPoint text  NULL,
        usbNum integer NOT NULL
);

insert into tableDefaults values ("USB", "ObjRevId", "0", "", "");
insert into stringsMap values ("USB_CONFIG_NOT_FOUND","USB configuration NOT Found");
insert into stringsMap values ("USB_CONFIG_FAILED","USB configuration Failed");

insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"USB", 0, 1,1,0);
CREATE TABLE tcpdump
(
    LogicalIfName text NOT NULL,
    tcpdumpEnabled  boolean NOT NULL,
    FOREIGN KEY (LogicalIfName) REFERENCES networkInterface(LogicalIfName)
)
;

insert into tableDefaults values ("tcpdump","tcpdumpEnabled","0","0","1");
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"tcpdump", 0, 1,1,0);
insert into stringsMap values("TCPDUMP_CONFIG_FAILED","TCPDUMP Configuration Failed");
insert into stringsMap values("TCPDUMP_IFACE_DOWN","The interface selected is down");
CREATE TABLE dhcp
(
    dhcpEnabled boolean NOT NULL,
    logicalIfName text  NOT NULL,
    domainName text,
    startAddress text NOT NULL,
    endAddress text NOT NULL,
    primaryDnsServer text,
    secondaryDnsServer text,
    winsServer text,
    leaseTime integer NOT NULL,
    enableDnsProxy boolean NOT NULL,
    primaryTftpServer text,
    secondaryTftpServer text,
    oldState	boolean 
);

CREATE TABLE DhcpLeasedClients
(
   OID           integer,
   LogicalIfName text  NOT NULL,
   IPAddr        text,
   MacAddr       text NOT NULL,
   Starts        text,
   Ends          text,
   State         text,
   PRIMARY KEY   (IPAddr)
);

CREATE TABLE DhcpfixedIpAddress
(
        OID           integer,
        LogicalIfName text  NOT NULL,
        IpAddr        text,
        MacAddr       text NOT NULL,
        PRIMARY KEY   (IpAddr,MacAddr)
)
;
-- Dhcpc table

CREATE TABLE Dhcpc
    (
    OID                      integer,           -- OID
    LogicalIfName            text     NOT NULL, -- interface names like 'lan1', 'wan2', etc.
    IspName                  integer,           -- name of the service provider
    GetDnsFromIsp            boolean  NOT NULL, -- do we need DNS from ISP ?
    PrimaryDns               text,              -- if we want to specify our DNS
    SecondaryDns             text,              -- if we want to specify our DNS
    ReleaseLease             boolean  NOT NULL, -- lease release flag
    HostName                 text,              -- host name to be sent to the server
    -- add your new entries here

    PRIMARY KEY (LogicalIfName),                -- primary key
    FOREIGN KEY (LogicalIfName) REFERENCES networkInterface (LogicalIfName)
    );

-- DhcpStatus - table for storing Dhcp related status information.
create table DhcpStatus
    (
    OID                      integer,           -- OID
    LogicalIfName            text     NOT NULL, -- interface names like 'lan1', 'wan2', etc.
    DhcpServerIp             text     NOT NULL, -- dhcp server identity.
    LeaseTime                text     NOT NULL, -- lease time of dhcp configured ip. 
    LeaseObtained            text     NOT NULL, -- time when dhcp server assgined ip to dhclient. 
    -- add newer entries here

    PRIMARY KEY (LogicalIfName),                -- primary key
    FOREIGN KEY (LogicalIfName) REFERENCES networkInterface (LogicalIfName)
    );

-- DhcpLdapServerInfo - table for storing and using Ldap Server related information
create table DhcpLdapServerInfo
    (
        OID                 integer,            -- OID
        DhcpLdapStatus       boolean NOT NULL,   -- Enable(1)/Disable(0) by default should be disabled.
        LogicalIfName       text    NOT NULL,   -- interface names like 'lan1', 'wan2', etc.
        DhcpLdapServerIP    text    NULL,       -- Ldap Server host name/IP - max text len - 21 chars.
        DhcpLdapSearchBase  text    NULL,       -- Search criterion - max text len - 255 chars.
        DhcpLdapServerPort  integer NULL,       -- Should not be > 65535 and not be less than 0.
        -- add newer entries here
        
        PRIMARY KEY (DhcpLdapServerIP),                -- primary key 
        FOREIGN KEY (LogicalIfName) REFERENCES networkInterface (LogicalIfName)
    );
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"dhcp", 0, 1,1,0);
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"DhcpfixedIpAddress", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"DhcpLdapServerInfo", 0, 1,1,1);
insert into stringsMap values("DHCP_CONFIG_FAILED","DHCP Configuration Failed");
insert into tableDefaults values ('dhcp','dhcpEnabled','1','','');
insert into tableDefaults values ('dhcp','leaseTime','24','','');
insert into tableDefaults values ('dhcp','enableDnsProxy','1','','');
insert into tableDefaults values ("dhcp", "domainName", "DLink", "", "");
insert into tableDefaults values ("dhcp", "primaryDnsServer", "", "", ""); 
insert into tableDefaults values ("dhcp", "secondaryDnsServer", "", "", ""); 
insert into tableDefaults values ("dhcp", "primaryTftpServer", "", "", ""); 
insert into tableDefaults values ("dhcp", "secondaryTftpServer", "", "", ""); 
insert into tableDefaults values ("dhcp", "winsServer", "", "", "");
insert into stringsMap values("DUPLICATE_IPADDR","IP address already assigned to other mac address. Please give another IP address.");
insert into stringsMap values("DUPLICATE_MACADDR","MAC address already configured. Please give another MAC address.");
-- SQL table for L2TP back-end component.

-- L2tp table
CREATE TABLE L2tp
    (
    OID                      integer,           -- OID
    LogicalIfName            text     NOT NULL, -- interface names like 'lan1', 'wan2', etc.
    IspName                  integer,           -- name of the service provider
    UserName                 text     NOT NULL, -- user name
    Password                 text     NOT NULL, -- password
    AccountName              text,              -- account name used in l2tp
    DomainName               text,              -- domain name used in l2tp
    Secret                   text,              -- secret to authenticate client
    GetIpFromIsp             boolean  NOT NULL, -- will ISP give IP ?
    GetDnsFromIsp            boolean  NOT NULL, -- do we need DNS from ISP ?
    MyIp                     text,              -- local IP of the wan interface
    ServerIp                 text     NOT NULL, -- l2tp server's IP
    StaticIp                 text,              -- ip which we want to assign or negotiate
    NetMask                  text,              -- net mask for the above IP
    PrimaryDns               text,              -- if we want to specify our DNS
    SecondaryDns             text,              -- if we want to specify our DNS
    IdleTimeOutFlag          boolean  NOT NULL, -- do we require idle timeout ?
    IdleTimeOutValue         integer  NOT NULL, -- idle timeout value
    SplitTunnel              boolean  NOT NULL, -- split tunnel ?
    -- add your new entries here

    PRIMARY KEY (LogicalIfName),                -- primary key
    FOREIGN KEY (LogicalIfName) REFERENCES networkInterface (LogicalIfName)
    );

insert into tableDefaults values ("L2tp","LogicalIfName","wan1", "", "");
insert into tableDefaults values ("L2tp","UseDefaultMtu","0", "0", "1");
insert into tableDefaults values ("L2tp","MtuSize","1300", "10", "1600");
insert into tableDefaults values ("L2tp","IdleTimeOutFlag","0", "0", "1");
insert into tableDefaults values ("L2tp","IdleTimeOutValue","10", "5", "999");
CREATE TABLE snmpTrap
(
	ipAddr  text NOT NULL,
	port  integer NOT NULL,
	commName text NOT NULL,
	snmpVersion text NOT NULL,

    PRIMARY KEY (ipAddr)
)
;

create table snmpAccessControl 
(
    ipAddr text NOT NULL, 
	subnetMask text NOT NULL, 
	commName text NOT NULL, 
	accessType text NOT NULL
	
-- 
--  PRIMARY KEY (ipAddr, subnetMask, commName) 
--  primary key should be :
--  ((ipAddr & subnetMask) and (commName))
--  where '&' is the bitwise/arithematic operation btwn ipAddress and subnetMask,
--  'and' is the logical operation with the above result and commName
--  
);

create table snmpv3Users 
(
    userName text NOT NULL, 
	accessType text NOT NULL, 
	securityLevel integer NOT NULL, 
	authAlgo text NOT NULL, 	 
	authPassword text, 
	privAlgo text NOT NULL, 
	privPassword text, 

	PRIMARY KEY (userName)
);

insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"snmpAccessControl",0,1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"snmpv3Users",0,1,1,1);
insert into stringsMap values ("SNMP_ACCESS_CONTROL_CONFIG_FAILED", "SNMP Access Control configuration failed");
insert into stringsMap values ("SNMPV3_USERS_CONFIG_FAILED","SNMPv3 susers configuration failed");
insert into stringsMap values ("SNMP_TRAPS_CONFIG_FAILED","SNMP traps configuration failed");

insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"snmpTrap", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"system", 0, 1,1,1);

insert into stringsMap values ("SNMP_CONFIG_FAILED","SNMP configuration failed");
insert into stringsMap values ("SYS_CONFIG_FAILED","System configuration failed");

CREATE TABLE dhcpv6c
    (
    LogicalIfName          text  NOT NULL   , -- interface on which to run
    isEnabled                     integer   , -- is enabled  
    statelessMode                 integer   , -- 1 = stateless mode 
    requestPreferredAddress       integer   , -- 
    preferredAddress              text      , -- IPv6 addres
    preferredAddressPrefixLength  integer   , -- IPv6 prefix length  
    requestDNS                    integer   , -- request DNS  
    requestDNSSearchList          integer   , -- request DNS Search List
    prefixDelegation              integer   , -- prefix delegation  
    requestPreferredPrefix        integer   , -- 1 = request Preferred Prefix 
    preferredPrefix               text      , -- IPv6 addres
    preferredPrefixPrefixLength   integer   , -- IPv6 prefix length      
    renewTime                     integer   , -- leaseTime
    sendRapidCommit               integer   , -- rapid commit  
    PRIMARY KEY (LogicalIfName)             , -- primary key
    FOREIGN KEY (LogicalIfName) REFERENCES networkInterface (LogicalIfName)
    );

CREATE TABLE dhcpv6s
    (
    LogicalIfName     text  NOT NULL        ,--  Logical Interface name   
    isEnabled           integer             ,--  DHCPv6 server enabled   
    statelessMode       integer             ,--  stateless/stateful mode   
    serverPreference    integer             ,--  server preference
    domainName          text                ,--  domain name   
    useDNSServersFrom   integer             ,--  use DNS servers from
    primaryDNSServer    text                ,--  primary DNS server
    secondaryDNSServer  text                ,--  secondary DNS server
    leaseTime           integer             ,--  lease time.   
    PRIMARY KEY (LogicalIfName)             ,-- primary key
    FOREIGN KEY (LogicalIfName) REFERENCES networkInterface (LogicalIfName)
    );

CREATE TABLE dhcpv6sLANAddrPool
    (
    startAddress        text        ,    --  Address Pool start address
    endAddress          text        ,    --  Address Pool end address   
    prefixLength        integer     ,    --  prefix length   
    delegateEnabled     integer     ,    --  prefix delegation enabled   
    delegationPrefix    text        ,    --  prefix to be delegated   
    delegationPrefixLen integer     ,    --  delegation Prefix length
    PRIMARY KEY (startAddress,endAddress)  
    );

insert into tableDefaults values ("dhcpv6s", "LogicalIfName", "LAN", "", "");
insert into stringsMap values ("IPV6_LAN_CONFIG_FAILED", "IPV6 Lan Configuration Failed");    
insert into stringsMap values ("IPV6_LAN_POOL_DELETE_SUCCESS", "IPV6 Lan Pool(s) Deleted Successfully");
insert into stringsMap values ("IPV6_LAN_POOL_DELETE_FAILED", "IPV6 Lan Pool(s) Deletion Failed");
insert into stringsMap values ("IPV6_LAN_POOL_CONFIG_FAILED", "IPV6 Lan Pool Configuration Failed");    
CREATE TABLE radvd
    (
    isEnabled            bool            , --  radvd enabled
    useDHCP6sPrefixes    bool            , --  use prefixes from upstream   
    UnicastOnly          integer         , --  unicast only !
    MaxRtrAdvInterval    integer         , --  router advertisement interval   
    AdvLinkMTU           integer         , --  Link MTU
    AdvManagedFlag       integer         , --  Use stateful protocol for 
                                           --  address assignment.
    AdvOtherConfigFlag   integer         , --  Other configuration flag
    AdvDefaultLifetime   integer         , --  default router lifetime
    AdvDefaultPreference integer         , --  router preference  
    LogicalIfName        text            , --  interface on which to run.
    FOREIGN KEY (LogicalIfName) REFERENCES networkInterface (LogicalIfName)    
    );

CREATE TABLE radvdLANPrefixPool
    (
    radvdPrefixType         integer     ,  --  6to4 or Others    
    Base6to4Interface       text        ,  --  6to4 Interface name.    
    SLAIdentifier           text        ,  --  SLA Identifier. 
    radvdAdvPrefix          text        ,  --  radvd advertisement prefix
    radvdAdvPrefixLength    integer     ,  --  radvd prefix length   
    radvdAdvPrefixLifetime  integer        --  radvd prefix lifetime   
    );
insert into stringsMap values ("RADVD_CONFIG_FAILED", "RADVD Configuration Failed");
insert into stringsMap values ("RADVD_PREFIX_CONFIG_FAILED", "RADVD Prefix Configuration Failed");
insert into stringsMap values ("RADVD_PREFIX_DELETE_FAILED", "RADVD Prefix Deletion Failed");    
--
-- This file includes Rip component related Tables like -
--
--  1. Rip

CREATE TABLE Rip
(
	Direction	    		  integer	NOT NULL, -- 0(none), 1(In only), 2(Out only),
                                                  -- 3(Both)

	Version			    	  integer	NOT NULL, -- 0(Disabled), 1(Rip1), 
                                                  -- 2(Rip2B(Broadcast)), 
                                                  -- 3(Rip2M(Multicast))

    AuthenticationType		  integer	NOT NULL, -- 0(No), 1(Yes)
	FirstKeyId			      integer	NULL,     -- MD5 first Key ID		
	FirstAuthenticationKeyId  text 		NULL,	  -- Password
	FirstKeyFrom			  text		NULL,	  -- Starting Date		
	FirstKeyTo			      text		NULL,	  -- Ending Date
	SecondKeyId			      integer	NULL,     -- MD5 second Key ID		
	SecondAuthenticationKeyId text 		NULL,     -- Password
	SecondKeyFrom			  text		NULL,	  -- Starting Date	
	SecondKeyTo			      text		NULL	  -- Ending Date
)
;


insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM", 0, 1, "Rip", 0, 1, 1, 1);
insert into stringsMap values ("RIP_CONFIG_FAILED","RIP Configuration Failed");	
insert into tableDefaults values ("RIP", "Direction", "0", "", "");
insert into tableDefaults values ("RIP", "Version", "0", "", "");
insert into tableDefaults values ("RIP", "AuthenticationType", "0", "", "");
-- dhcp relay schema

CREATE TABLE DhcpRelay
(
LogicalIfName   text     NOT NULL, -- interface names like 'LAN',
virtualIfName   text     NOT NULL,
DhcpRelayStatus boolean NOT NULL, -- Status - 1(Enable), 0 - Diable
RelayGateway    text NOT NULL,    -- Relay gateway ip address
PRIMARY KEY     (LogicalIfName),
FOREIGN KEY (LogicalIfName) REFERENCES networkInterface (LogicalIfName)
FOREIGN KEY (VirtualIfName) REFERENCES networkInterface (VirtualIfName)
);

insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM", 1, 1, "DhcpRelay", 0, 1, 1, 0);
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL", 1, 1, "DhcpRelay", 0, 1, 1, 0);

CREATE TABLE route
(
-- routeName       text NOT NULL,
routeName       text ,
dstIpAddr       text NOT NULL,
ipSNetMask      text,
gwIpAddr        text NOT NULL,
-- interfaceName   text NOT NULL,
interfaceName   text ,
metric          integer NOT NULL,
active          boolean NOT NULL,
private         boolean,

_ROWID_         integer NOT NULL,

UNIQUE      (_ROWID_)
);


CREATE TABLE route6
(
routeName text NOT NULL,
dstIpAddr text NOT NULL,
prefix text,
gwIpAddr text NOT NULL,
interfaceName text NOT NULL,
metric integer NOT NULL,
active boolean NOT NULL,
PRIMARY KEY (routeName)
);
CREATE TABLE upnp
(
        upnpEnable boolean NOT NULL,
        advPeriod integer ,
        advTimeToLive integer,
        interfaceName text NOT NULL
)
;
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"upnp", 0, 1,1,1);
insert into tableDefaults values ("upnp","upnpEnable","0","0","1");
CREATE TABLE upnpPortMap
(
        active text ,
        protocol text ,
        intPort integer ,
        extPort integer ,
        ipAddr text
)
;
-- SQL table for PPPoE back-end component.

-- Pppoe table
CREATE TABLE Pppoe
    (
    OID                      integer,           -- OID
    LogicalIfName            text     NOT NULL, -- interface names like 'lan1', 'wan2', etc.
    IspName                  integer,           -- name of the service provider
    UserName                 text     NOT NULL, -- user name
    Password                 text     NOT NULL, -- password
    AccountName              text,              -- account name used in PPPoE
    DomainName               text,              -- domain name used in PPPoE
    GetIpFromIsp             boolean  NOT NULL, -- will ISP give IP ?
    GetDnsFromIsp            boolean  NOT NULL, -- do we need DNS from ISP ?
    StaticIp                 text,              -- ip which we want to assign or negotiate
    NetMask                  text,              -- net mask for the above IP
    PrimaryDns               text,              -- if we want to specify our DNS
    SecondaryDns             text,              -- if we want to specify our DNS
    IdleTimeOutFlag          boolean  NOT NULL, -- do we require idle timeout ?
    IdleTimeOutValue         integer  NOT NULL, -- idle timeout value
    ProfileName                 text,    
    -- add your new entries here

    PRIMARY KEY (LogicalIfName),                -- primary key
    FOREIGN KEY (ProfileName) REFERENCES PppoeProfile (ProfileName),
    FOREIGN KEY (LogicalIfName) REFERENCES networkInterface (LogicalIfName)
    );


CREATE TABLE PppoeProfile
    (
    LogicalIfName            text     NOT NULL, -- interface names like 'lan1',    'wan2', etc.
    ProfileName              text     NOT NULL, -- Name of the profile
    Status                   boolean  NOT NULL, -- enable/disable the profile.
    AuthOpt                  integer  NOT NULL, -- can be NO-AUTH,PAP,CHAP,MS-CHAP,MS-CHAPV2    
    UserName                 text     NOT NULL, -- user name
    Password                 text     NOT NULL, -- password
    Service                  text,              -- service
    IdleTimeOutFlag          boolean  NOT NULL, -- do we require idle timeout ?
    IdleTimeOutValue         integer,           -- idle timeout value
    GetIpFromIsp             boolean  NOT NULL,
    StaticIp                 text,              -- ip which we want to assign or negotiate
    NetMask                  text,              -- net mask for the above IP
    GetDnsFromIsp            boolean  NOT NULL,
    PrimaryDns               text,
    SecondaryDns             text,
    -- add your new entries here

    PRIMARY KEY (ProfileName, LogicalIfName)    -- primary key
    );

-- table defaults
insert into tableDefaults values ("Pppoe", "LogicalIfName", "wan1", "", "");
insert into tableDefaults values ("Pppoe", "UseDefaultMtu", "0", "0", "1");
insert into tableDefaults values ("Pppoe", "MtuSize", "1492", "10", "1492");
insert into tableDefaults values ("Pppoe", "IdleTimeOutFlag", "0", "0", "1");
insert into tableDefaults values ("Pppoe", "IdleTimeOutValue", "5", "5", "999");
insert into tableDefaults values ("PppoeProfile", "LogicalIfName", "WAN1", "", "");
insert into tableDefaults values ("PppoeProfile", "GetIpFromIsp", "1", "0", "1");
insert into tableDefaults values ("PppoeProfile", "GetDnsFromIsp", "1", "0", "1");

insert into stringsMap values ("PPPOE_PROFILE_CONFIG_FAILED", "PPPoE Profile Configuration Failed");
insert into stringsMap values ("PPPOE_PROFILES_LIMIT_ERROR", "Can not add new profile. PPPoE Profiles max limit reached");
insert into stringsMap values ("PROFILE_NAME_ALREADY_EXIST", "Profile name already exists");
insert into stringsMap values ("SELECT_PPPOE", "Configure WAN in PPPoE Mode");
insert into stringsMap values ("ENABLE_OP_FAILED", "PPPoE Profile Enabling Failed");
insert into stringsMap values ("DISABLE_OP_FAILED", "PPPoE Profile Disabling Failed");
insert into stringsMap values ("DISABLE_SELECTED_PROFILE_BEFORE_DELETE", "Disable PPPoE profile before delete");
insert into stringsMap values ("PROF_DELETE_FAILED", "PPPoE Profile Delete Failed");

CREATE TABLE Igmp
(
OID         integer,
IgmpEnable  integer NOT NULL,
igmpLogicalIfName text NOT NULL
);
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",0,1,"Igmp", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_FIREWALL",0,1,"Igmp", 0, 1,1,1);
insert into tableDefaults values ("Igmp","IgmpEnable","0","0","1");
insert into tableDefaults values ("Igmp","igmpLogicalIfName","LAN","0","1");
insert into stringsMap values ("IGMP_CONFIG_FAILED", "IGMP Proxy Configuration Failed");

-- SQL table for PPTP back-end component.

-- Pptp table
CREATE TABLE Pptp
    (
    OID                      integer,           -- OID
    LogicalIfName            text     NOT NULL, -- interface names like 'lan1', 'wan2', etc.
    IspName                  integer,           -- name of the service provider
    UserName                 text     NOT NULL, -- user name
    Password                 text     NOT NULL, -- password
    GetIpFromIsp             boolean  NOT NULL, -- will ISP give IP ?
    GetDnsFromIsp            boolean  NOT NULL, -- do we need DNS from ISP ?
    MyIp                     text,              -- local IP of the wan interface
    ServerIp                 text     NOT NULL, -- PPTP server's IP
    StaticIp                 text,              -- ip which we want to assign or negotiate
    NetMask                  text,              -- net mask for the above IP
    PrimaryDns               text,              -- if we want to specify our DNS
    SecondaryDns             text,              -- if we want to specify our DNS
    IdleTimeOutFlag          boolean  NOT NULL, -- do we require idle timeout ?
    IdleTimeOutValue         integer  NOT NULL, -- idle timeout value
    MppeEncryptSupport       boolean  NOT NULL, -- Mppe encryption support
    SplitTunnel              boolean  NOT NULL, -- Split tunnel option
    -- add your new entries here

    PRIMARY KEY (LogicalIfName),                -- primary key
    FOREIGN KEY (LogicalIfName) REFERENCES networkInterface (LogicalIfName)
    );

-- SQL table for PPTP server back-end component.

CREATE TABLE pptpServerIPRange
    (
    pptpdEnable boolean NOT NULL, --0 for disabling pptp server 1 for enabling
    startIPAddress  text,  --The start ip address of the range
    endIPAddress text --The end ip address of the range that is
                                --allocated for remote users
    );

CREATE TABLE pptpServerUsers
    (
    userName text NOT NULL,
    remoteIP text,
    pptpIP text
    );
-- By default users added are 'enabled'. So adding a default entry.
insert into tableDefaults values ("pptpServerUsers", "enableConnection", "1", "0", "1");
CREATE TABLE sip
(
interfaceName text,
ipaddr  text ,
subnetmask  text ,
PRIMARY KEY(interfaceName),
FOREIGN KEY (interfaceName) REFERENCES networkInterface(interfaceName), 
FOREIGN KEY (ipaddr) REFERENCES networkInterface(ipaddr), 
FOREIGN KEY (subnetmask) REFERENCES networkInterface(subnetmask) 
)
;

CREATE TABLE SipStatusCtrl
(
    SipStatus boolean  NOT NULL, 
    PRIMARY KEY (SipStatus)
);

insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"SipStatusCtrl", 0, 1,1,1);
insert into stringsMap values ("SIP_CONFIG_FAILED","SIP ALG Configuration failed");
-- SQL table for L2TP server back-end component.

CREATE TABLE Xl2tpServerIPRange
    (
    Xl2tpdEnable   Boolean NOT NULL, -- 0 for disabling xl2tp server 1 for enabling
    StartIPAddress Text,             -- The start ip address of the range
    EndIPAddress   Text              -- The end ip address of the range that is allocated for remote users
    );

CREATE TABLE Xl2tpActiveUsers
    (
    userName text NOT NULL,    
    RemoteIP         Text,
    Xl2tpIP          Text 
    );

-- By default users added are 'enabled'. So adding a default entry.          
INSERT INTO tableDefaults VALUES ("Xl2tpActiveUsers", "EnableConnection", "1", "0", "1");
CREATE TABLE dot11Radio11n
    (
	radioNo integer NOT NULL,
    chanWidth integer NOT NULL,
    puren integer NOT NULL,                 -- 0 (Disable)/ 1 (Enable)
    mimoPreamble  integer NOT NULL,         -- 0 (for MM) / 1 (for GF)
	PRIMARY KEY (radioNo),
    FOREIGN KEY (radioNo) REFERENCES dot11Radio(radioNo)    
    );

insert into tableDefaults values ("dot11Radio11n","chanWidth","2040","","");

CREATE TABLE dot11nRates
    (
	vapName  text NOT NULL,
    PRIMARY KEY (vapName),
    FOREIGN KEY (vapName) REFERENCES dot11VAP(vapName)
    );

insert into saveTables (tableName) values("system");
insert into saveTables (tableName) values("FactoryStatus");
insert into saveTables (tableName) values("systemConfig");

--wireless
insert into saveTables (tableName) values("dot11GlobalConfig");
insert into saveTables (tableName) values("dot11Card");
insert into saveTables (tableName) values("dot11Radio");
insert into saveTables (tableName) values("dot11Interface");
insert into saveTables (tableName) values("dot11AuthorizedAP");
insert into saveTables (tableName) values("dot11Profile");
insert into saveTables (tableName) values("dot11VAP");
insert into saveTables (tableName) values("dot11Radio11n");
insert into saveTables (tableName) values("dot11ACL");
insert into saveTables (tableName) values("dot11IAPPRemoteAP");
insert into saveTables (tableName) values("dot11WPS");
insert into saveTables (tableName) values("bridgeTable");
insert into saveTables (tableName) values("bridgePorts");
insert into saveTables (TableName) values("VlanPortMgmt");
insert into saveTables (TableName) values("vlan");
insert into saveTables (TableName) values("VlanStatus");
insert into saveTables (tableName) values("DefaultIfGrpInfo");
insert into saveTables (tableName) values("ethernet");
insert into saveTables (tableName) values("ifStatic");
insert into saveTables (tableName) values("DhcpfixedIpAddress");
insert into saveTables (tableName) values("DhcpLdapServerInfo");
insert into saveTables (tableName) values("IPsecDhcpClientIPRange");
insert into saveTables (tableName) values("dhcp");
insert into saveTables (tableName) values("Xl2tpServerIPRange");
insert into saveTables (tableName) values("Xl2tpActiveUsers");
insert into saveTables (tableName) values("pptpServerIPRange");
insert into saveTables (tableName) values("pptpServerUsers");
insert into saveTables (TableName) values("PortMgmt");
insert into saveTables (TableName) values("PortMirroring");
insert into saveTables (tableName) values("crontab");
insert into saveTables (tableName) values("ifUsageReportConfig");
insert into saveTables (tableName) values("Dhcpc");
insert into saveTables (tableName) values("NimfMtu");
insert into saveTables (tableName) values("NimfMac");
insert into saveTables (tableName) values("WanMode");
insert into saveTables (tableName) values("WanFailure");
insert into saveTables (tableName) values("NimfConf");
insert into saveTables (tableName) values("Pppoe");
insert into saveTables (tableName) values("PppoeProfile");
insert into saveTables (tableName) values("L2tp");
insert into saveTables (tableName) values("Pptp");
insert into saveTables (tableName) values("ThreeG");
insert into saveTables (tableName) values("tcpdump");
insert into saveTables (tableName) values("ifStaticAliasAddrTbl");
insert into saveTables (tableName) values("networkInfo");
insert into saveTables (tableName) values("ntpDefServers");
insert into saveTables (tableName) values("ntp");
insert into saveTables (tableName) values("manualTime");
insert into saveTables (tableName) values("radiusClient");
insert into saveTables (tableName) values("x509DbUpdateFlag");
insert into saveTables (tableName) values("x509SelfCertReq");
insert into saveTables (tableName) values("cert");
insert into saveTables (tableName) values("snmpTrap");
insert into saveTables (tableName) values("snmpAccessControl");
insert into saveTables (tableName) values("snmpv3Users");
insert into saveTables (tableName) values("smtpServer");
insert into saveTables (tableName) values("emailLogs");
insert into saveTables (tableName) values("sysLogInfo");
insert into saveTables (tableName) values("logEventEmailConfig");
insert into saveTables (tableName) values("upnp");
insert into saveTables (tableName) values("Rip");
--insert into saveTables (tableName) values("Ripng");
insert into saveTables (tableName) values("Igmp");
insert into saveTables (tableName) values("route");
insert into saveTables (tableName) values("route6");
insert into saveTables (tableName) values("LanGroup");
--insert into saveTables (tableName) values("LanHost");
insert into saveTables (tableName) values("PortTriggering");
insert into saveTables (tableName) values("AttackChecks");
insert into saveTables (tableName) values("RemoteSnmp");
insert into saveTables (tableName) values("IcsaSettings");
insert into saveTables (tableName) values("GlobalSettings");
insert into saveTables (tableName) values("FwTrafficMeter");
insert into saveTables (tableName) values("TrafficCounters");
insert into saveTables (tableName) values("TrafficStatistics");
insert into saveTables (tableName) values("FwDosAttackSettings");
insert into saveTables (tableName) values("IPMACBinding");
insert into saveTables (tableName) values("MACFiltering");
insert into saveTables (tableName) values("SourceMACAddress");
insert into saveTables (tableName) values("ContentFiltering");
insert into saveTables (tableName) values("BlockSites");
insert into saveTables (tableName) values("TrustedDomains");
insert into saveTables (tableName) values("WebComponents");
insert into saveTables (tableName) values("NatTable");
insert into saveTables (tableName) values("ZoneTypes");
insert into saveTables (tableName) values("Zones");
insert into saveTables (tableName) values("Schedules");
insert into saveTables (tableName) values("Services");
insert into saveTables (tableName) values("FirewallRules");
insert into saveTables (tableName) values("fwPriorityMap");
insert into saveTables (tableName) values("ProtocolBinding");
insert into saveTables (tableName) values("BandWidthProfileStatus");
insert into saveTables (tableName) values("BandWidthProfile");
insert into saveTables (tableName) values("BandWidthProfileSpeed");
insert into saveTables (tableName) values("TrafficSelector");
insert into saveTables (tableName) values("accessMgmt");
insert into saveTables (tableName) values("httpsMgmt");
insert into saveTables (tableName) values("SSLVPNPortalLayout");
insert into saveTables (tableName) values("USERDBDomains");
insert into saveTables (tableName) values("USERDBGroups");
insert into saveTables (tableName) values("USERDBUsers");
insert into saveTables (tableName) values("GETUSERSDb");
insert into saveTables (tableName) values("interfaceStatsParams");
insert into saveTables (tableName) values("ddns");
insert into saveTables (tableName) values("ddnsStatus"); 
insert into saveTables (tableName) values("DhcpLeasedClients");
insert into saveTables (tableName) values("IpsecIKEPolicy");
insert into saveTables (tableName) values("IpsecVPNPolicy");
insert into saveTables (tableName) values("IpsecModeConfig");
insert into saveTables (tableName) values("XAuthUsers");
insert into saveTables (tableName) values("FirewallLogs");
insert into saveTables (tableName) values("FirewallLogs6");
insert into saveTables (tableName) values("OtherSystemLogs");
insert into saveTables (tableName) values ("IPSConfig");
insert into saveTables (tableName) values ("IPSNetworkSegments");
insert into saveTables (tableName) values("dhcpv6c");
insert into saveTables (tableName) values("dhcpv6s");
insert into saveTables (tableName) values("dhcpv6sLANAddrPool");
insert into saveTables (tableName) values("radvd");
insert into saveTables (tableName) values("radvdLANPrefixPool");
insert into saveTables (tableName) values("sixToFourTunnel");
insert into saveTables (tableName) values("isatapTunnel");
insert into saveTables (tableName) values("DscpClassList");
--insert into saveTables (tableName) values("mldproxy");
--insert into saveTables (tableName) values("FirewallRules6");
insert into saveTables (tableName) values("DhcpRelay");
--insert into saveTables (tableName) values("AttackChecks6");
--insert into saveTables (tableName) values("DOT1PState");
--insert into saveTables (tableName) values("DOT1PrioDscpMap");
--insert into saveTables (tableName) values("DOT1PrioQueueMap");
--insert into saveTables (tableName) values("interVLANRouting");

--cdp
--Entries need to be presented in both saveTables & saveTables2. 
--insert into saveTables (tableName) values("cdpPortCtrl");
--insert into saveTables (tableName) values("cdpGlobalCtrl");
--insert into saveTables2 (tableName) values("cdpPortCtrl");
--insert into saveTables2 (tableName) values("cdpGlobalCtrl");

insert into saveTables (tableName) values("oidRecord");
--insert into saveTables (tableName) values("tr069Config");

/*--verisign 2fctr authentication
insert into saveTables (tableName) values("vsign2fctrConfigTbl");
insert into saveTables (tableName) values("vsign2fctrCertificate");
insert into saveTables (tableName) values("vsign2fctrCredMgmt");
*/
-- SSLVPN

insert into saveTables (tableName) values("USB_MNT");
insert into saveTables (tableName) values("USB");
insert into saveTables (tableName) values("SSLVPNUserLoginPolicyDefinedAddress");
insert into saveTables (tableName) values("SSLVPNUserLoginPolicyDefinedBrowser");
insert into saveTables (tableName) values("SSLVPNResource");
insert into saveTables (tableName) values("SSLVPNResourceObject");
insert into saveTables (tableName) values("SSLVPNPolicy");
insert into saveTables (tableName) values("SSLVPNClientConf");
insert into saveTables (tableName) values("SSLVPNClientRoute");
insert into saveTables (tableName) values("SSLVPNPortForwardingAppsMap");
insert into saveTables (tableName) values("SSLVPNPortForwardingFQDNMap");
insert into saveTables (tableName) values("ConfigPort");

insert into saveTables (tableName) values("AlgConf");
--insert into saveTables (tableName) values("urlFilterConfig");
--insert into saveTables (tableName) values("urlFilterCategoryConfig");
--insert into saveTables (tableName) values("urlFilterBusinessHour");
--insert into saveTables (tableName) values("urlFilterUrlWhiteList");
--insert into saveTables (tableName) values("urlFilterClientWhiteList");
--insert into saveTables (tableName) values("urlFilterWebComponent");
--insert into saveTables (tableName) values("urlFilterBlockedKeyword");

--SIP status table
--insert into saveTables (tableName) values("SipStatusCtrl");



insert into saveTables (tableName) values("GettingStarted");
insert into saveTables (tableName) values("logConfig");
insert into saveTables (tableName) values("OneToOneNat");
insert into saveTables (tableName) values("OneToOneNatService");
insert into saveTables (tableName) values("Dmz");

--PowerMode Related Table
insert into saveTables (tableName) values("PowerModeConf");

--PPTP Server Related Entry
insert into environment (name, value) values("PPTP_ACTIVE_USERS_UPDATE_PROGRAM", "/pfrm2.0/bin/pptpdDisplayUserDetails");
insert into environment (name, value) values("xl2tpdDisplayUserDetails", "/pfrm2.0/bin/xl2tpdDisplayUserDetails");



insert into environment (name, value) values("NVRAM_DHCPV6C_LEASES_FILE_WAN1",
                                             "");
insert into environment (name, value) values("NVRAM_DHCPV6C_FLASH_STORE_DIR",
                                             "/flash/tmp/");
--dhcpv6 client/server related entries
insert into environment (name, value) values("NVRAM_DHCPV6C_DUID","/flash/tmp/dhcp6c_duid");
insert into environment (name, value) values("NVRAM_DHCPV6S_LEASES_FILE","/flash/tmp/server6.leases");
insert into environment (name, value) values("NVRAM_DHCPV6S_FLASH_STORE_DIR","/flash/tmp/");
insert into environment (name, value) values("NVRAM_DHCPV6S_DUID","/flash/tmp/dhcp6s_duid");                                        
insert into environment (name, value) values("TEAMF1_CFG_ASCII", "/tmp/teamf1.cfg.ascii");
insert into environment (name, value) values("TEAMF1_CERT_FILE", "/pfrm2.0/etc/https.crt");
insert into environment (name, value) values("TEAMF1_KEY_FILE", "/pfrm2.0/etc/https.key");
insert into environment (name, value) values("TEAMF1_CFG_DEFAULTS_ASCII", "/var/teamf1.cfg.defaults.ascii");
insert into environment (name, value) values("PING_FILE_NAME", "/tmp/ping.output");
insert into environment (name, value) values("CFG_READ_PROGRAM", "/pfrm2.0/bin/readcfg.sh");
insert into environment (name, value) values("CFG_WRITE_PROGRAM", "/pfrm2.0/bin/writecfg.sh");
--insert into environment (name, value) values("CFG_READ_PROGRAM", "/pfrm2.0/bin/sffs r");
--insert into environment (name, value) values("CFG_WRITE_PROGRAM", "/pfrm2.0/bin/sffs w");

insert into environment (name, value) values("UPGRADE_PROGRAM", "/mnt/tmpfs/upgrade");
insert into environment (name, value) values("FACTORY_RESET_SCRIPT", "/bin/rm -f /data/tmp/teamf1.cfg.ascii");
insert into environment (name, value) values("TFTP_COMMAND_FORMAT", "/usr/bin/tftp -g -l %s -r %s %s");
insert into environment (name, value) values("REBOOT_PROGRAM", "/sbin/reboot");

insert into environment (name, value) values("USB_STATUS_CHECK_PROGRAM", "/pfrm2.0/bin/usbStatus.sh");
insert into environment (name, value) values("USB_MOUNT_PROGRAM", "/bin/mount");
insert into environment (name, value) values("USB_UNMOUNT_PROGRAM", "/bin/umount -l");
insert into environment (name, value) values("USB_STATUS_FILE1", "/var/usb_status1.txt");
insert into environment (name, value) values("USB_FILE_LIST1", "/var/usb_filelist1.txt");
insert into environment (name, value) values("USB_STATUS_FILE2", "/var/usb_status2.txt");
insert into environment (name, value) values("USB_FILE_LIST2", "/var/usb_filelist2.txt");

insert into environment (name, value) values("PING_PROGRAM", "/bin/ping -c 5");
insert into environment (name, value) values("PING6_PROGRAM", "/bin/ping6 -c 5");
insert into environment (name, value) values("VPN_PING_PROGRAM", "/pfrm2.0/bin/doPing 5 ");
insert into environment (name, value) values("IFDEVSTATS_PROGRAM", "/pfrm2.0/bin/ifDevUpdateStats");
insert into environment (name, value) values("IFDEVUSAGE_PROGRAM", "/pfrm2.0/bin/ifDevUsageRecord");

-- DSR-1000N is 20.3.1
insert into environment (name, value) values("SNMP_SYSOBJID", "1.3.6.1.4.1.171.20.3.4");
insert into environment (name, value) values("IANA_ENTERPRISE_NUMBER","1127");
insert into environment (name, value) values("GET_BOOT_ENV", "/pfrm2.0/bin/fw_printenv");
insert into environment (name, value) values("SET_BOOT_ENV", "/pfrm2.0/bin/fw_setenv");
insert into environment (name, value) values("BOOTUP_TIME", "150");
insert into environment (name, value) values("UPDATE_FIRMWARE_VERSION_PROGRAM","/pfrm2.0/bin/ShowFirm");
insert into environment (name, value) values("RADIUS_DICTIONARY", "/pfrm2.0/etc/radius.dict");
insert into environment (name, value) values("RADIUS_DEF_PORT", "1812");
insert into environment (name, value) values("RADIUS_DEF_TIMEOUT", "10");
insert into environment (name, value) values("RADIUS_DEF_RETRIES", "3");
insert into environment (name, value) values("LOG_FILE", "/tmp/logs.temp");
insert into environment (name, value) values("SMTPCLIENT_PATH","/pfrm2.0/bin/smtpClient");
insert into environment (name, value) values("DEFAULT_USER_SHELL","/bin/sh");
insert into environment (name, value) values("EMAILLOGS_PROGRAM","/pfrm2.0/bin/emailLogs");
insert into environment (name, value) values("UPNP_REFRESH_PROGRAM",  "/pfrm2.0/bin/upnpShowConfig");
insert into environment (name, value) values("NIMFSTATUS_PROGRAM",  "/pfrm2.0/bin/nimfStatus");
insert into environment (name, value) values("LINKSTATUS_PROGRAM",  "/pfrm2.0/bin/linkStatusDetect");
insert into environment (name, value) values("LANGROUPS_CONFIG",  "/pfrm2.0/bin/langroupsConfig");
insert into environment (name, value) values("DHCP_VIEW",  "/pfrm2.0/bin/dhcpLeasesUpdate");
insert into environment (name, value) values("DDNS_VIEW",  "/pfrm2.0/bin/ddnsView"); 
insert into environment (name, value) values("PORTTRIG_STATUS_PROG", "/pfrm2.0/bin/portTriggerStatusConfig");
insert into environment (name, value) values("PFRM_BIN_PATH","/pfrm2.0/bin");
insert into environment (name, value) values("XAUTH_USER_ADD","/bin/ipsecuseradd");
insert into environment (name, value) values("XAUTH_USER_DEL","/bin/ipsecuserdel");
insert into environment (name, value) values("XAUTH_PASSWD","/bin/ipsecpasswd");
insert into environment (name, value) values("VIPSECURE_STATUS",  "/pfrm2.0/bin/vipsecureStatus"); 
insert into environment (name, value) values("TEAMF1_LUA_LIB_PATH", "/pfrm2.0/share/lua/5.1/teamf1lualib/"); 
insert into environment (name, value) values("FLASH_FIRM_PARTITION","/dev/mtd1");
insert into environment (name, value) values("KERNEL_PARTITION","/dev/mtd1");
insert into environment (name, value) values("ROOTFS_PARTITION","/dev/mtd2");
insert into environment (name, value) values("FLASH_CFG_PARTITION", "/data/tmp/teamf1.cfg.ascii");
insert into environment (name, value) values("UPGRADEPREP_PROGRAM", "/pfrm2.0/bin/upgradePrep.sh");
insert into environment (name, value) values("BOOTENVSET_BIN","/bin/fw_setenv");
insert into environment (name, value) values("BOOTLOADER_FILENAME_VAR","filename");
insert into environment (name, value) values("BOOTENVREAD_BIN","/bin/fw_printenv");
insert into environment (name, value) values("BOOTLOADER_FILEPATH_VAR","filepath");
insert into environment (name, value) values("CURRENT_FIRM_PARTITION","");
insert into environment (name, value) values("BOOTENVSET_SCRIPT","/pfrm2.0/bin/setBootEnv.sh");
--insert into environment (name, value) values("UPGRADE_SCRIPT","/pfrm2.0/bin/upgrade.sh");
insert into environment (name, value) values("BOOTENVGET_SCRIPT","/pfrm2.0/bin/getBootEnv.sh");;
insert into environment (name, value) values("CONFIG_PORT_UPDATE", "/pfrm2.0/bin/configPortUpdate /tmp/system.db 23 ConfigPort 1"); 
insert into environment (name, value) values("DOT11_MAX_ASSOC_CLIENTS","128");
insert into environment (name, value) values("WPS_PROGRAM", "/bin/wsccmd");
insert into environment (name, value) values("WPS_PIN_PROGRAM", "/pfrm2.0/bin/dot11WpsConfigPin");
insert into environment (name, value) values("WPS_PBC_PROGRAM", "/pfrm2.0/bin/dot11WpsConfigPbc");
insert into environment (name, value) values("IPV6_GW_UPDATE","/pfrm2.0/bin/iprouteIPv6GwAdd");
-- Traceroute program for diagnostics. -m defines max # hops
insert into environment (name, value) values("TRACEROUTE_PROGRAM",  "/pfrm2.0/bin/traceroute -m 10");
insert into environment (name, value) values("TRACE_ROUTE_FILE_NAME","/tmp/traceroute.output");
insert into environment (name, value) values("TIMEZONECHANGESCRIPT","/pfrm2.0/bin/timezonechangescript.sh");
insert into environment (name, value) values("NTP_CUSTOM_SERVER1","0.us.pool.ntp.org");
insert into environment (name, value) values("NTP_CUSTOM_SERVER2","1.us.pool.ntp.org");
insert into environment (name, value) values("HW_CACHE_RESET","/pfrm2.0/bin/ResetCache");
insert into environment (name, value) values("HW_OFFLOAD_PROC_ENTRY","/proc/sys/hardware_offload_control");
insert into environment (name, value) values("HW_OFFLOAD_BRCM_ENTRY","/pfrm2.0/bin/syshandler offload_brcmtag_enabled ");
insert into environment (name, value) values("HW_OFFLOAD_VLAN_ENTRY","/pfrm2.0/bin/syshandler offload_vlan_enabled ");

--To get the usage report
insert into ifUsageReportConfig values(2,4); 

-- DNS Lookup program for diagnostics
insert into environment (name, value) values("NSLOOKUP_PROGRAM",  "/usr/bin/nslookup");

-- Route program for diagnostics
insert into environment (name, value) values("ROUTE_PROGRAM",  "/sbin/route");
insert into environment (name, value) values("ROUTE6_PROGRAM",  "/sbin/route -A inet6 -n");

--switch device table
insert into SwitchDevice values (0, "/dev/bcm53980");

--SSLVPN and VPN logs
insert into environment (name, value) values("SSLVPN_CONFIG","/pfrm2.0/sslvpn/bin/sslvpnConfig");
insert into environment (name, value) values("VPN_LOG_FILE_NAME","/var/racoon.log");

--System Default user as a global variable, make sure that this field is updated accordingly
insert into environment (name, value) values("DEFAULT_SYSTEM_USER", "admin");

--cpumemusage
insert into environment (name, value) values("MEM_USAGE_SCRIPT", "/pfrm2.0/bin/mem.sh");

insert into environment (name, value) values("CPU_USAGE_SCRIPT", "/pfrm2.0/bin/cpu.sh");

insert into environment (name, value) values("SMTP_AUTH_TEST", "/pfrm2.0/bin/smtpAuthTest");

--Status tables
insert into NimfStatus( LogicalIfName, AddressFamily, Nimfstatus, Linkstatus, WanUpTime)values("WAN1", 2,"Not Yet Available", "LINK DOWN", "Not Yet Available");
insert into NimfStatus( LogicalIfName, AddressFamily, Nimfstatus, Linkstatus, WanUpTime)values("WAN1", 10,"Not Yet Available", "LINK DOWN", "Not Yet Available");
insert into NimfStatus( LogicalIfName,AddressFamily,Nimfstatus,Linkstatus,WanUpTime)values("LAN", 2,"Not Yet Available","LINK DOWN","Not Yet Available");
insert into NimfStatus( LogicalIfName, AddressFamily, Nimfstatus,Linkstatus,WanUpTime)values("LAN", 10,"Not Yet Available","LINK DOWN","Not Yet Available");
insert into DhcpStatus(LogicalIfName,DhcpServerIp,LeaseTime,LeaseObtained)values("WAN1","Not Yet Available","Not Yet Available","Not Yet Available");

insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/bridgeDBConfig",1,1,"bridgeTable", 0, 1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/vlanDBConfig",1,1,"VlanStatus", 0, 1,1,0);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/vlanDBConfig",1,1,"VlanPortMgmt", 0, 1,1,0);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/vlanDBConfig",1,1,"vlan", 0, 1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/switchVlanDBConfig",1,1,"VlanStatus", 0, 1,1,0);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/switchVlanDBConfig",1,1,"VlanPortMgmt", 0, 1,1,0);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/switchVlanDBConfig",1,1,"vlan", 0, 1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/bridgeDBConfig",1,1,"bridgePorts", 0, 1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/dhcpConfig",0,1,"networkInterface", 0, 1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/dhcpConfig",0,1,"ipAddressTable", 0, 1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/pptpdConfig",0,1,"pptpServerIPRange", 0, 1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/pptpdConfig",0,1,"networkInterface", 0, 1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/pptpdConfig",0,1,"ipAddressTable", 0, 1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/pptpdConfig",0,1,"dhcp", 0, 1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/pptpdConfig",0,1,"USERDBUsers", 0, 1,1,1);
insert into dbUpdateRegisterTbl values("UMI_COMP_PLATFORM",0,1,"pptpServerIPRange", 0, 1,1,1);
insert into dbUpdateRegisterTbl values("UMI_COMP_FIREWALL",0,1,"pptpServerIPRange", 0, 1,1,1);
insert into dbUpdateRegisterTbl values("UMI_COMP_PLATFORM",0,1,"pptpServerUsers", 0, 1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/xl2tpdConfig",0,1,"Xl2tpServerIPRange", 0, 1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/xl2tpdConfig",0,1,"USERDBUsers", 0, 1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/xl2tpdConfig",0,1,"networkInterface", 0, 1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/xl2tpdConfig",0,1,"ipAddressTable", 0, 1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/xl2tpdConfig",0,1,"dhcp", 0, 1,1,1);
insert into dbUpdateRegisterTbl values("UMI_COMP_PLATFORM",0,1,"Xl2tpServerIPRange", 0, 1,1,1);
insert into dbUpdateRegisterTbl values("UMI_COMP_FIREWALL",0,1,"Xl2tpServerIPRange", 0, 1,1,1);
insert into dbUpdateRegisterTbl values("UMI_COMP_PLATFORM",0,1,"Xl2tpActiveUsers", 0, 1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/portMgmtConfig",1,1,"PortMgmt", 0, 1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/portMgmtConfig",1,1,"PortMirroring", 0, 1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/upnpConfig",0,1,"networkInterface",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/upnpConfig",0,1,"ipAddressTable",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/x509Config",1,1,"cert", 0, 1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/x509Config",1,1,"x509SelfCertReq", 0, 1,1,1);
insert into ifDevEventTbl values("bdg1","UMI_COMP_PLATFORM","/pfrm2.0/bin/ifDevUpdateMac /tmp/system.db ethernet bdg1",0,0,0,1,0,0,0);
insert into ifDevEventTbl values("eth1","UMI_COMP_PLATFORM","/pfrm2.0/bin/ifDevUpdateMac /tmp/system.db ethernet eth1",0,0,0,1,0,0,0);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/dhcpConfig",1,1,"dhcp",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/dhcpConfig",1,1,"DhcpfixedIpAddress",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/dhcpConfig",1,1,"DhcpLdapServerInfo",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/dhcpConfig",1,1,"IPsecDhcpClientIPRange",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/upnpConfig",1,1,"upnp", 0, 1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/ripConfig",1,1,"Rip", 0, 1,1,1);
--insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/ripngConfig",1,1,"Ripng", 0, 1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/snmpConfig",1,1,"snmpTrap", 0, 1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/snmpConfig",1,1,"system", 0, 1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/snmpConfig",1,1,"snmpAccessControl", 0, 1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/snmpConfig",1,1,"snmpv3Users", 0, 1,1,1);

insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/sshdConfig",1,1,"sshd", 0, 1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/igmpConfig",1,1,"Igmp", 0, 1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/igmpConfig",1,1,"networkInterface", 0, 1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/tcpdumpConfig",1,1,"tcpdump",0,1,1,1);
insert into ifDevEventTbl values("bdg1","UMI_COMP_PLATFORM","/pfrm2.0/bin/iprouteManageRoutes /tmp/system.db route",0,0,1,0,1,0,0);
insert into ifDevEventTbl values("eth1","UMI_COMP_PLATFORM","/pfrm2.0/bin/iprouteManageRoutes /tmp/system.db route",0,0,1,0,1,0,0);
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"route", 0, 1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/iprouteConfig",1,1,"route",0,1,1,1);
--insert into sshd values (22,2,"0.0.0.0",768,1,0,0,1,0,1,1,0);

--ips related
insert into dbUpdateRegisterTbl values ('UMI_COMP_PLATFORM', 0, 0, 'IPSNetworkSegments', 0, 1, 1, 1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/ipsDBConfig", 0, 1, "IPSNetworkSegments", 0, 1, 1, 1);
insert into dbUpdateRegisterTbl values ('UMI_COMP_PLATFORM', 0, 0, 'IPSConfig', 0, 1, 1, 1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/ipsDBConfig", 0, 1, "IPSConfig", 0, 1, 1, 1);

--dscp remark table
insert into dbUpdateRegisterTbl values("UMI_COMP_PLATFORM",1,1,"DOT1PrioDscpMap", 0, 1,0,0);
insert into dbUpdateRegisterProgram values("/pfrm2.0/bin/8021pRemarkConfig",1,1,"DOT1PrioDscpMap",0,1,0,0);

insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"DOT1PState", 0,1,1,0);
insert into dbUpdateRegisterProgram values("/pfrm2.0/bin/8021pRemarkConfig",1,1,"DOT1PState",0,1,1,0);

insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"VlanStatus",0,1,0,0);
insert into dbUpdateRegisterProgram values("/pfrm2.0/bin/8021pRemarkConfig",1,1,"VlanStatus",0,1,0,0);

insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"VlanPortMgmt",0,1,0,0);
insert into dbUpdateRegisterProgram values("/pfrm2.0/bin/8021pRemarkConfig",1,1,"VlanPortMgmt",0,1,0,0);

--8021p support
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"DOT1PState", 0,1,1,0);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/8021pSupportConfig",1,1,"DOT1PState",0,1,1,0);

--QueueMap
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"DOT1PrioQueueMap", 0,1,0,0);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/8021pSupportConfig",1,1,"DOT1PrioQueueMap",0,1,0,0);

-- Resource Untilization
insert into environment (name, value) values("RESOURCE_UTILITY_PROGRAM", "/pfrm2.0/bin/cpuMemUsage");

-- dhcp relay
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/dhcprelayConfig", 0, 1, "DhcpRelay", 0, 1, 1, 1);

-- dhcp relay
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/dhcprelayConfig", 0, 1, "networkInterface", 0, 1, 1, 1);

insert into networkInterface values ("bdg1", "LAN", "bdg1", "ifStatic", "", "bridge", 1500, "192.168.10.2", "255.255.255.0", "0.0.0.0", "0.0.0.0", "0.0.0.0", 0, 1, 5, 0,1,1,1);
insert into networkInterface values ("eth0", "LAN-BDG", "eth0", "ifStatic", "","ethernet", 1500, "0.0.0.0", "0.0.0.0", "0.0.0.0", "0.0.0.0", "0.0.0.0", 0, 0, 5, 0,1,0,1);

insert into networkInterface values ("eth1", "WAN1", "eth1", "dhcpc", "","ethernet", 1500, "0.0.0.0", "0.0.0.0", "0.0.0.0", "0.0.0.0", "0.0.0.0", 0, 0, 5, 0,1,1,2);

insert into networkInterface values ("sit0", "sit0-WAN1", "sit0", "","", "tunnel", 1500, "0.0.0.0", "0.0.0.0", "0.0.0.0", "0.0.0.0", "0.0.0.0", 0, 0, 5, 0,2,1 ,2); 

insert into networkInterface values ("eth2", "DMZ", "eth2", "ifStatic", "","ethernet", 1500, "0.0.0.0", "0.0.0.0", "0.0.0.0", "0.0.0.0", "0.0.0.0", 0, 0, 5, 0,3,1,4); 
-- tr069 object record for static configurations
--insert into tr069ObjectRecord values ("InternetGatewayDevice.LANDevice.", 1, "networkInterface", "LogicalIfName", 1, 1); 
--insert into tr069ObjectRecord values ("InternetGatewayDevice.LANDevice.1.LANHostConfigManagement.IPInterface.", 1, "ifStatic", "LogicalIfName", 1, 1);

-- Registration for networkInfo
insert into dbUpdateRegisterTbl values ('UMI_COMP_PLATFORM',0,1,'networkInfo',0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/ipModeConfig",0,1,"networkInfo", 0, 1,1,1);

insert into NimfTrigger values (1, "WAN1",2, 0);
insert into NimfTrigger values (2, "WAN1",10, 0);
insert into NimfLBStatus values("",0);
insert into NimfAFStatus values("",0);
insert into MacTable values ("WAN1", 0);

insert into interfaceStats values("bdg1",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0); 
insert into interfaceStats values("eth1",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0); 
insert into interfaceStats values("ppp1",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0); 
insert into interfaceStats values("ppp2",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0); 
insert into SystemStatistics values(0, 0,0,0,0,0, 0,0,0,0,0);
insert into tableMaxRecords values ("radiusClient", 4);
-- system facility
insert into compFacilityMap values ("UMI_COMP_NTP", 3);
insert into compFacilityMap values ("UMI_COMP_IFDEV", 3);
insert into compFacilityMap values ("UMI_COMP_USERDB", 3);
-- insert into compFacilityMap values ("UMI_COMP_USB", 3);
insert into compFacilityMap values ("UMI_COMP_LOGIN", 3);
insert into compFacilityMap values ("UMI_COMP_FIREWALL", 3);
insert into compFacilityMap values ("UMI_COMP_DHCPC", 3);
insert into compFacilityMap values ("UMI_COMP_VIPSECURE", 3);
insert into compFacilityMap values ("UMI_COMP_NIMF", 3);

--cdp 
--handling cdpPortCtrl events only on the updates and not on the inserts
--handling cdpGlobalCtrl events on the updates as well as inserts. 
--handling dot11Interface events only on the updates.

--insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"cdpPortCtrl",0,1,0,0);
--insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"cdpGlobalCtrl",0, 1,1,0);
--insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"dot11Interface",0, 1,0,0);
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"ConfigPort",0, 1,1,0);

--insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/cdpConfig",1,1,"cdpPortCtrl",0,1,0,1);
--insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/cdpConfig",1,1,"cdpGlobalCtrl",0,1,1,1);
--insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/cdpConfig",1,1,"VlanStatus", 0, 1,0,0);
--insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/cdpConfig",1,1,"dot11VAP", 0, 1,1,0);
--insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/cdpConfig",1,1,"dot11Interface", 0, 1,0,0);
--insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/cdpConfig",1,1,"VlanPortMgmt", 0, 1,0,0);
--insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/cdpConfig",1,1,"ipAddressTable", 0, 1,0,0);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/cdpConfig",1,1,"ConfigPort", 0, 1,1,0);

--bandwidth profile registration
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/bwLimitConfig",1,1,"BandWidthProfile",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/bwLimitConfig",1,1,"BandWidthProfileStatus",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/bwLimitConfig",1,1,"TrafficSelector",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/bwLimitConfig",1,1,"networkInterface",0,1,1,1);

-- ddns program 
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/ddnsConfig",1,1,"ddns",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/ddnsConfig",0,1,"networkInterface",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/ddnsConfig",1,1,"NimfAFStatus",0,1,1,1);

-- vipsecure program
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/vipsecureConfig",1,1,"IpsecVPNPolicy",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/vipsecureConfig",1,1,"ipAddressTable",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/vipsecureConfig",1,1,"XAuthUsers",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/vipsecureConfig",1,1,"ConnectionStatus",0,1,0,0);
--insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/vipsecureConfig",1,1,"NimfAFStatus",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/vipsecureConfig",1,1,"radiusClient",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/vipsecureConfig",1,1,"IpsecIKEPolicy",0,0,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/quickVpnMgmtConfig",1,1,"QuickVPNUsersStatus",0,1,1,1);

-- urlfilter
--insert into urlFilterProxy values ("/pfrm2.0/bin/havp","/tmp/havp.conf","","",8080);


-- static configuration table
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",0,1,"ifStatic", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",0,1,"ifStaticAliasAddrTbl", 0, 1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/ifStaticTblManage",0,1,"ifStatic",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/ifStaticTblManage",0,1,"ifStatic",1,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/ifStaticTblManage",0,1,"ifStatic",5,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/ifStaticTblManage",0,1,"ifStatic",6,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/ifStaticTblManage",0,1,"ifStaticAliasAddrTbl",1,1,1,1);
insert into ifDevEventTbl values ('eth1',"UMI_COMP_PLATFORM","/pfrm2.0/bin/ifStaticIfDevHandler /tmp/system.db ifStatic",0,0,1,0,0,0,0);
insert into ifDevEventTbl values ('bdg1' ,"UMI_COMP_PLATFORM","/pfrm2.0/bin/ifStaticIfDevHandler /tmp/system.db ifStatic",0,0,1,0,0,0,0);
insert into ifDevEventTbl values ('eth1',"UMI_COMP_PLATFORM","/pfrm2.0/bin/ifStaticIfDevHandler /tmp/system.db ifStaticAliasAddrTbl",0,0,1,0,0,0,0);
insert into ifDevEventTbl values ('bdg1' ,"UMI_COMP_PLATFORM","/pfrm2.0/bin/ifStaticIfDevHandler /tmp/system.db ifStaticAliasAddrTbl",0,0,1,0,0,0,0);

-- IPv6 static WAN
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/ifStaticTblManage",0,1,"ifStatic",2,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/ifStaticTblManage",0,1,"ifStatic",4,1,1,1);


-- Registration for networkInfo
insert into dbUpdateRegisterTbl values ('UMI_COMP_PLATFORM',0,1,'networkInfo',0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/ipModeConfig",0,1,"networkInfo", 0, 1,1,1);
 



-- Registering to ipAddressTable
insert into dbUpdateRegisterTbl values ('UMI_COMP_FIREWALL',0,1,'ipAddressTable', 0, 1,1,1);
insert into dbUpdateRegisterTbl values ('UMI_COMP_PLATFORM',0,1,'ipAddressTable',0,1,1,1);

insert into ipAddressTable values ('LAN' ,2,1,'0.0.0.0','0.0.0.0',0,'255.255.255.0');
insert into ipAddressTable values ('WAN1',2,0,'0.0.0.0','0.0.0.0',0,'255.255.255.0');
insert into ipAddressTable values ('DMZ' ,2,0,'0.0.0.0','0.0.0.0',0,'255.255.255.0');
insert into ipAddressTable values ('WAN2',2,0,'0.0.0.0','0.0.0.0',0,'255.255.255.0');

-- IPv6 Support
--

-- IPv6 address updates
insert into ifDevEventTbl values("bdg1","UMI_COMP_PLATFORM","/pfrm2.0/bin/ifDevUpdateIp6Addr /tmp/system.db ipAddressTable",0,0,0,0,0,1,0);
insert into ifDevEventTbl values("eth1","UMI_COMP_PLATFORM","/pfrm2.0/bin/ifDevUpdateIp6Addr /tmp/system.db ipAddressTable",0,0,0,0,0,1,0);
insert into ifDevEventTbl values("eth1","UMI_COMP_PLATFORM","/pfrm2.0/bin/ifDevUpdateIp6Addr /tmp/system.db ipAddressTable",0,0,1,0,0,0,0);
insert into ifDevEventTbl values("bdg1","UMI_COMP_PLATFORM","/pfrm2.0/bin/ifDevUpdateIp6Addr /tmp/system.db ipAddressTable",0,0,1,0,0,0,0);

-- 6to4 tunnel
insert into ifDevEventTbl values("sit0","UMI_COMP_PLATFORM","/pfrm2.0/bin/ifDevUpdateIp6Addr /tmp/system.db ipAddressTable",0,0,0,0,0,1,0);
insert into ifDevEventTbl values("sit0","UMI_COMP_PLATFORM","/pfrm2.0/bin/ifDevUpdateIp6Addr /tmp/system.db ipAddressTable",0,0,1,0,0,0,0);

-- IPv6 static WAN
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/ifStaticTblManage",0,1,"ifStatic",2,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/ifStaticTblManage",0,1,"ifStatic",4,1,1,1);

-- resolver configuration table
insert into dbUpdateRegisterTbl values ('UMI_COMP_PLATFORM',0,1,'resolverConfig', 0, 1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/dproxyConfig",0,1,"resolverConfig", 0, 1,1,1);
insert into resolverConfig values ("WAN1", 10, "", "","",""); 
insert into resolverConfig values ("WAN1", 2, "0.0.0.0", "0.0.0.0", "0.0.0.0", "0.0.0.0"); 
insert into resolverConfig values ("WAN2", 10, "", "","",""); 
insert into resolverConfig values ("WAN2", 2, "0.0.0.0", "0.0.0.0", "0.0.0.0", "0.0.0.0"); 


-- default routers table
insert into defaultRouters values ("WAN1", 2, "0.0.0.0", ""); 
insert into defaultRouters values ("WAN1", 10, "", ""); 
insert into defaultRouters values ("WAN2", 2, "0.0.0.0", ""); 
insert into defaultRouters values ("WAN2", 10, "", ""); 


-- Registering for route6
insert into dbUpdateRegisterTbl values ('UMI_COMP_PLATFORM',0,1,'route6',0,1,1,1);
insert into dbUpdateRegisterProgram values ('/pfrm2.0/bin/iprouteConfig',0,1,'route6',0,1,1,1);
insert into ifDevEventTbl values("sit0","UMI_COMP_PLATFORM","/pfrm2.0/bin/iprouteManageRoutes /tmp/system.db route",0,0,1,0,1,0,0);

-- Registration for radvd 
insert into dbUpdateRegisterTbl values ('UMI_COMP_PLATFORM',0,1,'radvd',0,1,1,1);
insert into dbUpdateRegisterTbl values ('UMI_COMP_PLATFORM',0,1,'radvdLANPrefixPool',0,1,1,1);
insert into dbUpdateRegisterProgram values ('/pfrm2.0/bin/radvdConfig',0,1,'radvd',0,1,1,1);
insert into dbUpdateRegisterProgram values ('/pfrm2.0/bin/radvdLANPrefixPoolConfig',1,1,'radvdLANPrefixPool',0,1,1,1);
insert into dbUpdateRegisterProgram values ('/pfrm2.0/bin/radvd6to4PrefixRestart',0,1,'ipAddressTable',0,1,1,0);
insert into ifDevEventTbl values ('bdg1' ,"UMI_COMP_PLATFORM","/pfrm2.0/bin/radvdIfDevHandler /tmp/system.db ",0,0,1,0,0,0,0);

-- Registration for DHCPv6
insert into dbUpdateRegisterTbl values ('UMI_COMP_PLATFORM',0,1,'dhcpv6s',0,1,1,1);
insert into dbUpdateRegisterTbl values ('UMI_COMP_PLATFORM',0,1,'dhcpv6sLANAddrPool',0,1,1,1);
insert into dbUpdateRegisterProgram values ('/pfrm2.0/bin/dhcpv6sConfig',0,1,'dhcpv6s',0,1,1,1);
insert into dbUpdateRegisterProgram values ('/pfrm2.0/bin/dhcpv6sConfig',0,1,'dhcpv6sLANAddrPool',0,1,1,1);
insert into dbUpdateRegisterProgram values ('/pfrm2.0/bin/dhcpv6sConfig',0,1,'ifStatic',2,1,1,1);
insert into dbUpdateRegisterProgram values ('/pfrm2.0/bin/dhcpv6sConfig',0,1,'resolverConfig',1,1,1,1);
insert into ifDevEventTbl values("eth1","UMI_COMP_PLATFORM","/pfrm2.0/bin/dhcpv6cRestart /tmp/system.db",0,0,1,1,0,0,0);

-- Registaration for isatapTunnel
insert into dbUpdateRegisterTbl values ('UMI_COMP_PLATFORM',0,1,'isatapTunnel',0,1,1,1);
insert into dbUpdateRegisterProgram values ('/pfrm2.0/bin/isatapConfig',0,1,'isatapTunnel',0,1,1,1);
insert into ifDevEventTbl values('',"UMI_COMP_PLATFORM","/pfrm2.0/bin/isatapDevHandler /tmp/system.db",1,1,0,0,0,0,0);
insert into dbUpdateRegisterProgram values ('/pfrm2.0/bin/radvdIsatapPrefixConfig',0,1,'isatapTunnel',0,1,1,1);
    

-- Registaration for sixToFourTunnel
insert into dbUpdateRegisterTbl values ('UMI_COMP_PLATFORM',0,1,'sixToFourTunnel',0,1,1,1);
insert into dbUpdateRegisterProgram values ('/pfrm2.0/bin/6to4Config',0,1,'sixToFourTunnel',0,1,1,1);
insert into dbUpdateRegisterProgram values ('/pfrm2.0/bin/6to4Restart',0,1,'ipAddressTable',0,1,1,0);

-- Registration for SIP 
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/sipConfig",0,1,"ipAddressTable", 1, 1,1,0);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/sipConfig",0,1,"SipStatusCtrl", 0, 1,1,0);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/sipConfig",0,1,"ConfigPort", 0, 1,1,0);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/sipConfig",0,1,"AlgConf", 0, 1,1,0);
insert into dbUpdateRegisterTbl values ('UMI_COMP_PLATFORM',0,1,'AlgConf',0,1,1,1);

-- Registaration for mldproxy
--insert into dbUpdateRegisterTbl values ('UMI_COMP_PLATFORM',0,1,'mldproxy',0,1,1,1);
--insert into dbUpdateRegisterProgram values ('/pfrm2.0/bin/mldproxyConfig',0,1,'mldproxy',0,1,1,1);

-- NIMF Registration for interface UP events
insert into ifDevEventTbl values ("eth1", "UMI_COMP_NIMF", "NULL", 0, 0, 1, 0, 0, 0, 0);


--- userdbConfig program

insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/userdbConfig",1,1,"USERDBUsers",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/userdbConfig",1,1,"USERDBGroups",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/userdbConfig",1,1,"USERDBDomains",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/userdbConfig",1,1,"GETUSERSDb",0,1,1,1);

--- usbConfig program
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/usbConfig",1,1,"USB",0,1,1,1);

-- SSLVPN program
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/sslvpnConfig",1,1,"SSLVPNPortalLayout",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/sslvpnConfig",1,1,"SSLVPNUserLoginPolicyDefinedAddress",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/sslvpnConfig",1,1,"SSLVPNUserLoginPolicyDefinedBrowser",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/sslvpnConfig",1,1,"SSLVPNResource",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/sslvpnConfig",1,1,"SSLVPNResourceObject",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/sslvpnConfig",1,1,"SSLVPNPolicy",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/sslvpnConfig",1,1,"SSLVPNClientConf",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/sslvpnConfig",1,1,"SSLVPNClientRoute",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/sslvpnConfig",1,1,"SSLVPNPortForwardingAppsMap",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/sslvpnConfig",1,1,"SSLVPNPortForwardingFQDNMap",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/sslvpnConfig",1,1,"httpsMgmt",0,1,1,1);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/sslvpnConfig",1,1,"accessMgmt",0,1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"accessMgmt", 0, 1,1,1); --to make sslvpn handle thru platformd

-- WAN2 related entries

insert into ifDevEventTbl values("eth2","UMI_COMP_PLATFORM","/pfrm2.0/bin/ifDevUpdateMac /tmp/system.db ethernet eth2",0,0,0,1,0,0,0);
insert into ifDevEventTbl values("eth2","UMI_COMP_PLATFORM","/pfrm2.0/bin/iprouteManageRoutes /tmp/system.db route",0,0,1,0,1,0,0);
insert into networkInterface values ("eth2", "WAN2", "eth2", "dhcpc", "","ethernet", 1500, "0.0.0.0", "0.0.0.0", "0.0.0.0", "0.0.0.0", "0.0.0.0", 0, 0, 5, 0,3,1,2); 
insert into interfaceStats values("eth2",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0); 
insert into ifDevEventTbl values("eth2","UMI_COMP_PLATFORM","/pfrm2.0/bin/ifDevUpdateIp6Addr /tmp/system.db ipAddressTable",0,0,0,0,0,1,0);
insert into ifDevEventTbl values("eth2","UMI_COMP_PLATFORM","/pfrm2.0/bin/ifDevUpdateIp6Addr /tmp/system.db ipAddressTable",0,0,1,0,0,0,0);
insert into ifDevEventTbl values("eth2","UMI_COMP_PLATFORM","/pfrm2.0/bin/dhcpv6cRestart /tmp/system.db",0,0,1,0,0,0,0);
insert into ifDevEventTbl values("eth2","UMI_COMP_FIREWALL","",0,0,1,0,0,0,0);
insert into ifDevEventTbl values ('eth2',"UMI_COMP_PLATFORM","/pfrm2.0/bin/ifStaticIfDevHandler /tmp/system.db ifStatic",0,0,1,0,0,0,0);
insert into ifDevEventTbl values ('eth2',"UMI_COMP_PLATFORM","/pfrm2.0/bin/ifStaticIfDevHandler /tmp/system.db ifStaticAliasAddrTbl",0,0,1,0,0,0,0);
insert into NimfStatus( LogicalIfName, AddressFamily, Nimfstatus, Linkstatus, WanUpTime)values("WAN2", 2,"Not Yet Available", "LINK DOWN", "Not Yet Available");
insert into NimfStatus( LogicalIfName, AddressFamily, Nimfstatus, Linkstatus, WanUpTime)values("WAN2", 10,"Not Yet Available", "LINK DOWN", "Not Yet Available");
insert into NimfStatus( LogicalIfName, AddressFamily, Nimfstatus, Linkstatus, WanUpTime)values("DMZ", 2,"Not Yet Available", "LINK DOWN", "Not Yet Available");
insert into DhcpStatus(LogicalIfName,DhcpServerIp,LeaseTime,LeaseObtained)values("WAN2","Not Yet Available","Not Yet Available","Not Yet Available");
insert into NimfTrigger values (3, "WAN2",2, 0);
insert into NimfTrigger values (4, "WAN2",10, 0);
insert into MacTable values ("WAN2", 0);

-- table for unit information:
create table unitInfo (
modelId text NOT NULL,
hardwareVersion text NOT NULL,
regionInfo integer NOT NULL,
serialNumber text  NOT NULL,
lanMacAddress text NOT NULL
);

-- table for Factory Default State information:
create table FactoryStatus (
factStatus integer NOT NULL
);

-- active sessions table
CREATE TABLE activeSessions
     (
     protocol                text     NOT NULL,
     LocalIp                 text     NOT NULL,
     ExternalIp              text     NOT NULL,
     connState               text     NOT NULL,
     NatIps                  text     NOT NULL,
     direction               text     NOT NULL,
     priority                text     NOT NULL,
     timeout                 text     NOT NULL
     );

create table webBrandingTags
(
companyName text NOT NULL,
copyright text NOT NULL,
webAddress text NOT NULL,
supportLink text NOT NULL,
unitName text NULL,
tradeMark text NULL,
prodFamily text NULL,
appName text NULL,
Version text NULL
);


--uid specific table
create table UDIInfo
(
udi text NOT NULL,
pid text NOT NULL,
vid text NULL,
serialNumber text NOT NULL,
prodName text NULL,
prodDesc text NULL
);
insert into UDIInfo values ("udi","pid","vid","sn","prodName","prodDesc");


-- table for default mac addresses:
create table defaultMac (
logicalName text NOT NULL,
macAddress  text NULL
);

insert into ipv6Interfaces values ("eth1");
insert into ipv6Interfaces values ("bdg1");

-- table for LED Parameters:
create table LedParam (
LogicalIfName text NOT NULL,
LedPinId  text NOT NULL,
LedON  text NOT NULL,
LedOFF  text NOT NULL
);

-- table for showing getting Started page
create table GettingStarted (
dontShow integer NOT NULL
);

-- table for storing led status
create table wirelessLED (
ledStatus integer NOT NULL
);


-- vlan gui messages
--insert into stringsmap values ("VLAN_IN_USE_BY_BW_TRAFFIC_SELECTOR","Selected VLAN(s) are being using in bandwidth profiles traffic selector");
--insert into stringsmap values ("PORT_VLAN_IN_USE_BY_BW_TRAFFIC_SELECTOR","Cannot edit as this port is being used in bandwidth profiles traffic selector");
--insert into stringsmap values ("CONFIG_PORT_SET_AS_WANORDMZ","VLANID 4094 is reserved when configurable port is set as WAN/DMZ");
insert into stringsmap values ("PORT_MODE_NOT_ALLOWED","Access port can be member of only single vlan");
insert into stringsmap values ("VLAN_MAX_LIMIT_ERROR","Maximum Allowed VLANS Exceeded");
insert into stringsmap values ("VLAN_4094_EXISTS","VLANID 4094 Already Exists. Please delete VLANID 4094");
insert into stringsmap values ("PORT_VLAN_PVID_4094_EXISTS","PVID 4094 Already Exists. Please Change PVID 4094");

--wireless
insert into networkInterface values ("vap0", "LAN-BDG", "vap0","ifStatic","","dot11", 1500, "0.0.0.0", "0.0.0.0", "0.0.0.0", "0.0.0.0", "0.0.0.0", 0, 1, 5, 0,1,0,5);
insert into networkInterface values ("vap1", "LAN-BDG", "vap1","ifStatic","","dot11", 1500, "0.0.0.0", "0.0.0.0", "0.0.0.0", "0.0.0.0", "0.0.0.0", 0, 1, 5, 0,1,0,5);
insert into networkInterface values ("vap2", "LAN-BDG", "vap2","ifStatic","","dot11", 1500, "0.0.0.0", "0.0.0.0", "0.0.0.0", "0.0.0.0", "0.0.0.0", 0, 1, 5, 0,1,0,5);
insert into networkInterface values ("vap3", "LAN-BDG", "vap3","ifStatic","","dot11", 1500, "0.0.0.0", "0.0.0.0", "0.0.0.0", "0.0.0.0", "0.0.0.0", 0, 1, 5, 0,1,0,5);

insert into interfaceStats values("vap0","0","0","0","0","0","0","0","0","0","0","0","0","0","0","0","0","0","0","0","0");
insert into interfaceStats values("vap1","0","0","0","0","0","0","0","0","0","0","0","0","0","0","0","0","0","0","0","0");
insert into interfaceStats values("vap2","0","0","0","0","0","0","0","0","0","0","0","0","0","0","0","0","0","0","0","0");
insert into interfaceStats values("vap3","0","0","0","0","0","0","0","0","0","0","0","0","0","0","0","0","0","0","0","0");

insert into dbUpdateRegisterTbl values ("UMI_COMP_UDOT11",1,1,"dot11WPSSessStatus", 0, 0,0,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"dot11WPSSessStatus", 0, 1,1,1);
insert into dbUpdateRegisterTbl values ("UMI_COMP_PLATFORM",1,1,"dot11WPS", 0, 1,1,1);

-- register ledmgmt.lua with dot11 tables
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/dot11mgmt.lua",1,1,"dot11Radio", 0, 1,1,0);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/dot11mgmt.lua",1,1,"dot11VAP", 0, 1,1,0);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/dot11mgmt.lua",1,1,"dot11WPSSessStatus", 0, 1,1,0);
insert into dbUpdateRegisterProgram values ("/pfrm2.0/bin/dot11mgmt.lua",1,1,"dot11WPS", 0, 1,1,0);

-- override default configuration
update tableDefaults set defValue='64' where tableName="dot11VAP" and columnName="maxClients";
update tableDefaults set minValue='40' where tableName="dot11VAP" and columnName="beaconInterval";
update tableDefaults set maxValue='3500' where tableName="dot11VAP" and columnName="beaconInterval";
COMMIT;

BEGIN TRANSACTION;


CREATE TRIGGER "oid_upd_route"  AFTER INSERT ON route  FOR EACH ROW
BEGIN
    
    UPDATE oidRecord SET OID = OID + 1 WHERE tableName = "route";
END;


COMMIT TRANSACTION;

